/* Generated by Cython 3.2.8 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/usr/include/cpl_conv.h",
            "/usr/include/cpl_error.h",
            "/usr/include/cpl_port.h",
            "/usr/include/cpl_string.h",
            "/usr/include/cpl_vsi.h",
            "/usr/include/gdal.h",
            "/usr/include/ogr_api.h",
            "/usr/include/ogr_core.h",
            "/usr/include/ogr_srs_api.h",
            "pyogrio/arrow_bridge.h"
        ],
        "include_dirs": [
            "pyogrio",
            "/usr/include"
        ],
        "libraries": [
            "gdal"
        ],
        "library_dirs": [
            "/usr/lib"
        ],
        "name": "pyogrio._io",
        "sources": [
            "pyogrio/_io.pyx"
        ]
    },
    "module_name": "pyogrio._io"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_8"
#define CYTHON_HEX_VERSION 0x030208F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CInitCode */
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS && !CYTHON_COMPILING_IN_LIMITED_API && CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) (__Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
#else
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) __Pyx_PyList_GetItemRef(o, i)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyogrio___io
#define __PYX_HAVE_API__pyogrio___io
/* Early includes */
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "cpl_conv.h"
#include "cpl_error.h"
#include "cpl_port.h"
#include "cpl_string.h"
#include "cpl_vsi.h"
#include "ogr_core.h"
#include "ogr_srs_api.h"
#include "arrow_bridge.h"
#include "ogr_api.h"
#include "gdal.h"
#include "pythread.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "pyogrio/_io.pyx",
  "View.MemoryView",
  "pyogrio/_err.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto (used by ParseKeywordsImpl) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* IncludeStructmemberH.proto (used by FixUpExtensionType) */
#include <structmember.h>

/* BufferFormatStructs.proto */
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  const struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  const __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  const __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;

/* MemviewSliceStruct.proto */
struct __pyx_memoryview_obj;
typedef struct {
  struct __pyx_memoryview_obj *memview;
  char *data;
  Py_ssize_t shape[8];
  Py_ssize_t strides[8];
  Py_ssize_t suboffsets[8];
} __Pyx_memviewslice;
#define __Pyx_MemoryView_Len(m)  (m.shape[0])
#define __Pyx_MEMVIEW_DIRECT   1
#define __Pyx_MEMVIEW_PTR      2
#define __Pyx_MEMVIEW_FULL     4
#define __Pyx_MEMVIEW_CONTIG   8
#define __Pyx_MEMVIEW_STRIDED  16
#define __Pyx_MEMVIEW_FOLLOW   32
#define __Pyx_IS_C_CONTIG 1
#define __Pyx_IS_F_CONTIG 2
#define __Pyx_MEMSLICE_INIT  { 0, 0, { 0 }, { 0 }, { 0 } }
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview)\
             __pyx_atomic_incr_relaxed(__pyx_get_slice_count_pointer(memview))
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_atomic_decr_acq_rel(__pyx_get_slice_count_pointer(memview))
#else
    #define __pyx_add_acquisition_count(memview)\
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_7pyogrio_4_err_ErrorHandler;
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata;
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr;
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features;
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow;
struct __pyx_array_obj;
struct __pyx_MemviewEnum_obj;
struct __pyx_memoryview_obj;
struct __pyx_memoryviewslice_obj;
struct __pyx_opt_args_7pyogrio_4_ogr_get_string;

/* "pyogrio/_ogr.pxd":536
 * 
 * 
 * cdef get_string(const char *c_str, str encoding=*)             # <<<<<<<<<<<<<<
*/
struct __pyx_opt_args_7pyogrio_4_ogr_get_string {
  int __pyx_n;
  PyObject *encoding;
};
struct __pyx_opt_args_7pyogrio_3_io_execute_sql;
struct __pyx_opt_args_7pyogrio_3_io_get_fields;
struct __pyx_opt_args_7pyogrio_3_io_validate_feature_range;

/* "pyogrio/_io.pyx":380
 * 
 * 
 * cdef OGRLayerH execute_sql(             # <<<<<<<<<<<<<<
 *     GDALDatasetH ogr_dataset, str sql, str sql_dialect=None
 * ) except NULL:
*/
struct __pyx_opt_args_7pyogrio_3_io_execute_sql {
  int __pyx_n;
  PyObject *sql_dialect;
};

/* "pyogrio/_io.pyx":739
 * 
 * 
 * cdef get_fields(OGRLayerH ogr_layer, str encoding, use_arrow=False):             # <<<<<<<<<<<<<<
 *     """Get field names and types for layer.
 * 
*/
struct __pyx_opt_args_7pyogrio_3_io_get_fields {
  int __pyx_n;
  PyObject *use_arrow;
};

/* "pyogrio/_io.pyx":935
 * 
 * 
 * cdef validate_feature_range(             # <<<<<<<<<<<<<<
 *     OGRLayerH ogr_layer, int skip_features=0, int max_features=0
 * ):
*/
struct __pyx_opt_args_7pyogrio_3_io_validate_feature_range {
  int __pyx_n;
  int skip_features;
  int max_features;
};

/* "pyogrio/_err.pxd":5
 * cdef void *check_pointer(void *ptr) except NULL nogil
 * 
 * cdef class ErrorHandler:             # <<<<<<<<<<<<<<
 *     cdef object error_stack
 *     cdef int check_int(self, int retval, bint squash_errors) except -1
*/
struct __pyx_obj_7pyogrio_4_err_ErrorHandler {
  PyObject_HEAD
  struct __pyx_vtabstruct_7pyogrio_4_err_ErrorHandler *__pyx_vtab;
  PyObject *error_stack;
};


/* "pyogrio/_io.pyx":638
 *         raise RuntimeError("Could not set metadata") from None
 * 
 * cdef get_metadata(GDALMajorObjectH obj):             # <<<<<<<<<<<<<<
 *     """Get metadata for a dataset or layer.
 * 
*/
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata {
  PyObject_HEAD
  char **__pyx_v_metadata;
};


/* "pyogrio/_io.pyx":657
 *     if metadata != NULL:
 *         return dict(
 *             metadata[i].decode("UTF-8").split("=", 1)             # <<<<<<<<<<<<<<
 *             for i in range(CSLCount(metadata))
 *         )
*/
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr {
  PyObject_HEAD
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata *__pyx_outer_scope;
  int __pyx_genexpr_arg_0;
  int __pyx_v_i;
  int __pyx_t_0;
  int __pyx_t_1;
  int __pyx_t_2;
};


/* "pyogrio/_io.pyx":1225
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef get_features(
*/
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features {
  PyObject_HEAD
  int __pyx_v_datetime_as_string;
  PyObject *__pyx_v_fid_chunks;
  PyObject *__pyx_v_field_data_chunks;
  __Pyx_memviewslice __pyx_v_fields;
  PyObject *__pyx_v_geom_chunks;
  int __pyx_v_n_fields;
  uint8_t __pyx_v_read_geometry;
  uint8_t __pyx_v_return_fids;
};


/* "pyogrio/_io.pyx":1908
 * 
 * 
 * @contextlib.contextmanager             # <<<<<<<<<<<<<<
 * def ogr_open_arrow(
 *     object path_or_buffer,
*/
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow {
  PyObject_HEAD
  int __pyx_v_batch_size;
  PyObject *__pyx_v_bbox;
  PyObject *__pyx_v_capsule;
  PyObject *__pyx_v_columns;
  PyObject *__pyx_v_crs;
  PyObject *__pyx_v_dataset_kwargs;
  char **__pyx_v_dataset_options;
  PyObject *__pyx_v_datetime_as_string;
  PyObject *__pyx_v_encoding;
  PyObject *__pyx_v_ex;
  PyObject *__pyx_v_feature_count;
  PyObject *__pyx_8genexpr7__pyx_v_fid;
  PyObject *__pyx_v_fid_column;
  PyObject *__pyx_v_fid_column_where;
  PyObject *__pyx_v_fids;
  PyObject *__pyx_v_fids_str;
  PyObject *__pyx_8genexpr8__pyx_v_field;
  PyObject *__pyx_8genexpr9__pyx_v_field;
  PyObject *__pyx_v_field;
  PyObject *__pyx_v_field_b;
  char const *__pyx_v_field_c;
  PyObject *__pyx_v_fields;
  char **__pyx_v_fields_c;
  int __pyx_v_force_2d;
  PyObject *__pyx_v_geometry_name;
  PyObject *__pyx_v_geometry_type;
  PyObject *__pyx_v_idx;
  PyObject *__pyx_v_ignored_fields;
  PyObject *__pyx_v_layer;
  PyObject *__pyx_v_mask;
  int __pyx_v_max_features;
  PyObject *__pyx_v_meta;
  OGRDataSourceH __pyx_v_ogr_dataset;
  OGRLayerH __pyx_v_ogr_layer;
  PyObject *__pyx_v_ogr_subtypes;
  PyObject *__pyx_v_ogr_types;
  char **__pyx_v_options;
  int __pyx_v_override_shape_encoding;
  PyObject *__pyx_v_pa;
  PyObject *__pyx_v_path;
  PyObject *__pyx_v_path_or_buffer;
  char const *__pyx_v_prev_shape_encoding;
  int __pyx_v_read_geometry;
  PyObject *__pyx_v_reader;
  int __pyx_v_return_fids;
  int __pyx_v_reval;
  int __pyx_v_skip_features;
  PyObject *__pyx_v_sql;
  PyObject *__pyx_v_sql_dialect;
  struct ArrowArrayStream *__pyx_v_stream;
  PyObject *__pyx_v_use_pyarrow;
  int __pyx_v_use_tmp_vsimem;
  PyObject *__pyx_v_where;
};


/* "View.MemoryView":118
 * 
 * 
 * @cython.collection_type("sequence")             # <<<<<<<<<<<<<<
 * @cname("__pyx_array")
 * cdef class array:
*/
struct __pyx_array_obj {
  PyObject_HEAD
  struct __pyx_vtabstruct_array *__pyx_vtab;
  char *data;
  Py_ssize_t len;
  char *format;
  int ndim;
  Py_ssize_t *_shape;
  Py_ssize_t *_strides;
  Py_ssize_t itemsize;
  PyObject *mode;
  PyObject *_format;
  void (*callback_free_data)(void *);
  int free_data;
  int dtype_is_object;
};


/* "View.MemoryView":307
 * 
 * 
 * @cname('__pyx_MemviewEnum')             # <<<<<<<<<<<<<<
 * cdef class Enum(object):
 *     cdef object name
*/
struct __pyx_MemviewEnum_obj {
  PyObject_HEAD
  PyObject *name;
};


/* "View.MemoryView":342
 * 
 * 
 * @cname('__pyx_memoryview')             # <<<<<<<<<<<<<<
 * cdef class memoryview:
 * 
*/
struct __pyx_memoryview_obj {
  PyObject_HEAD
  struct __pyx_vtabstruct_memoryview *__pyx_vtab;
  PyObject *obj;
  PyObject *_size;
  void *_unused;
  PyThread_type_lock lock;
  __pyx_atomic_int_type acquisition_count;
  Py_buffer view;
  int flags;
  int dtype_is_object;
  __Pyx_TypeInfo const *typeinfo;
};


/* "View.MemoryView":856
 * 
 * 
 * @cython.collection_type("sequence")             # <<<<<<<<<<<<<<
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):
*/
struct __pyx_memoryviewslice_obj {
  struct __pyx_memoryview_obj __pyx_base;
  __Pyx_memviewslice from_slice;
  PyObject *from_object;
  PyObject *(*to_object_func)(char *);
  int (*to_dtype_func)(char *, PyObject *);
};



/* "pyogrio/_err.pxd":5
 * cdef void *check_pointer(void *ptr) except NULL nogil
 * 
 * cdef class ErrorHandler:             # <<<<<<<<<<<<<<
 *     cdef object error_stack
 *     cdef int check_int(self, int retval, bint squash_errors) except -1
*/

struct __pyx_vtabstruct_7pyogrio_4_err_ErrorHandler {
  int (*check_int)(struct __pyx_obj_7pyogrio_4_err_ErrorHandler *, int, int);
  void *(*check_pointer)(struct __pyx_obj_7pyogrio_4_err_ErrorHandler *, void *, int);
  void (*_handle_error_stack)(struct __pyx_obj_7pyogrio_4_err_ErrorHandler *, int);
};
static struct __pyx_vtabstruct_7pyogrio_4_err_ErrorHandler *__pyx_vtabptr_7pyogrio_4_err_ErrorHandler;


/* "View.MemoryView":118
 * 
 * 
 * @cython.collection_type("sequence")             # <<<<<<<<<<<<<<
 * @cname("__pyx_array")
 * cdef class array:
*/

struct __pyx_vtabstruct_array {
  PyObject *(*get_memview)(struct __pyx_array_obj *);
};
static struct __pyx_vtabstruct_array *__pyx_vtabptr_array;


/* "View.MemoryView":342
 * 
 * 
 * @cname('__pyx_memoryview')             # <<<<<<<<<<<<<<
 * cdef class memoryview:
 * 
*/

struct __pyx_vtabstruct_memoryview {
  char *(*get_item_pointer)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*is_slice)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_slice_assignment)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*setitem_slice_assign_scalar)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_indexed)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*convert_item_to_object)(struct __pyx_memoryview_obj *, char *);
  PyObject *(*assign_item_from_object)(struct __pyx_memoryview_obj *, char *, PyObject *);
  PyObject *(*_get_base)(struct __pyx_memoryview_obj *);
};
static struct __pyx_vtabstruct_memoryview *__pyx_vtabptr_memoryview;


/* "View.MemoryView":856
 * 
 * 
 * @cython.collection_type("sequence")             # <<<<<<<<<<<<<<
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):
*/

struct __pyx_vtabstruct__memoryviewslice {
  struct __pyx_vtabstruct_memoryview __pyx_base;
};
static struct __pyx_vtabstruct__memoryviewslice *__pyx_vtabptr__memoryviewslice;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by GetBuiltinName) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto (used by BytesEquals) */
#include <string.h>

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto (used by PyObjectCallOneArg) */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* PyObjectCallOneArg.proto (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTestFunc.export */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))

/* PyValueError_Check.proto */
#define __Pyx_PyExc_ValueError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ValueError)

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyMemoryError_Check.proto */
#define __Pyx_PyExc_MemoryError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_MemoryError)

/* BuildPyUnicode.proto (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* COrdinalToPyUnicode.proto (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value);
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t width, char padding_char);

/* GCCDiagnostics.proto (used by CIntToPyUnicode) */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* IncludeStdlibH.proto (used by CIntToPyUnicode) */
#include <stdlib.h>

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_int(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_int(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_int(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_Py_ssize_t(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_Py_ssize_t(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_Py_ssize_t(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char);

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

CYTHON_UNUSED static int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *); /*proto*/
/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* RejectKeywords.export */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* PyTypeError_Check.proto */
#define __Pyx_PyExc_TypeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TypeError)

/* DivInt[Py_ssize_t].proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t, int b_is_constant);

/* UnaryNegOverflows.proto */
#define __Pyx_UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_LIMITED_API  ||  PY_VERSION_HEX >= 0x030C0000
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  #if __clang__ || __GNUC__
  __attribute__((no_sanitize("thread")))
  #endif
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* PyAssertionError_Check.proto */
#define __Pyx_PyExc_AssertionError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AssertionError)

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetTopmostException.proto (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* HasAttr.proto (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

CYTHON_UNUSED static int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PySequenceMultiply.proto */
#define __Pyx_PySequence_Multiply_Left(mul, seq)  __Pyx_PySequence_Multiply(seq, mul)
#if !CYTHON_USE_TYPE_SLOTS
#define  __Pyx_PySequence_Multiply PySequence_Repeat
#else
static CYTHON_INLINE PyObject* __Pyx_PySequence_Multiply(PyObject *seq, Py_ssize_t mul);
#endif

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* PyObjectFormat.proto */
#if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* s, PyObject* f);
#else
#define __Pyx_PyObject_Format(s, f) PyObject_Format(s, f)
#endif

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* RaiseUnboundLocalError.proto */
static void __Pyx_RaiseUnboundLocalError(const char *varname);

/* PyIndexError_Check.proto */
#define __Pyx_PyExc_IndexError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_IndexError)

/* DivInt[long].proto */
static CYTHON_INLINE long __Pyx_div_long(long, long, int b_is_constant);

/* IterFinish.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallNoArg.proto (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);
#endif

/* PyObjectCallMethod0.proto (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* UnpackItemEndCheck.proto (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* UnpackTupleError.proto (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
    PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple);
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* decode_c_string_utf16.proto (used by decode_c_string) */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* PyObject_Unicode.proto */
#define __Pyx_PyObject_Unicode(obj)\
    (likely(PyUnicode_CheckExact(obj)) ? __Pyx_NewRef(obj) : PyObject_Str(obj))

/* PyUnicodeContains.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_ContainsTF(PyObject* substring, PyObject* text, int eq) {
    int result = PyUnicode_Contains(text, substring);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared)
    #endif
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_OwnStrongReference)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_SharedReference)
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right))

/* PyUnicode_Unicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj);

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_MultiplyObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_MultiplyObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceMultiply(op1, op2) : PyNumber_Multiply(op1, op2))
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* RaiseClosureNameError.proto */
static void __Pyx_RaiseClosureNameError(const char *varname);

/* dict_setdefault.proto (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* LimitedApiGetTypeDict.proto (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto (used by FetchCommonType) */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* AddModuleRef.proto (used by FetchSharedCythonModule) */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* FetchSharedCythonModule.proto (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto (used by CommonTypesMetaclass) */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto (used by CythonFunctionShared) */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* CallTypeTraverse.proto (used by CythonFunctionShared) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* PyMethodNew.proto (used by CythonFunctionShared) */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto (used by CythonFunction) */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_dict;
#endif
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyRange_Check.proto */
#if CYTHON_COMPILING_IN_PYPY && !defined(PyRange_Check)
  #define PyRange_Check(obj)  __Pyx_TypeCheck((obj), &PyRange_Type)
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyObjectDelAttr.proto (used by PyObjectSetAttrStr) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
#define __Pyx_PyObject_DelAttr(o, n) PyObject_SetAttr(o, n, NULL)
#else
#define __Pyx_PyObject_DelAttr(o, n) PyObject_DelAttr(o, n)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   __Pyx_PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyNotImplementedError_Check.proto */
#define __Pyx_PyExc_NotImplementedError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_NotImplementedError)

/* unicode_tailmatch.proto */
static int __Pyx_PyUnicode_Tailmatch(
    PyObject* s, PyObject* substr, Py_ssize_t start, Py_ssize_t end, int direction);

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_FloorDivideObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceFloorDivide(op1, op2) : PyNumber_FloorDivide(op1, op2))
#endif

/* DictGetItem.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_unsigned_int(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_unsigned_int(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_unsigned_int(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char, char format_char);

/* PyAttributeError_Check.proto */
#define __Pyx_PyExc_AttributeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AttributeError)

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_TrueDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_TrueDivideObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_int64_t(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_int64_t(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_int64_t(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int64_t(int64_t value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int64_t(int64_t value, Py_ssize_t width, char padding_char, char format_char);

/* CheckTypeForFreelists.proto */
#if CYTHON_USE_FREELISTS
#if CYTHON_USE_TYPE_SPECS
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t) == (expected_tp)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  Py_TPFLAGS_IS_ABSTRACT
#else
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t)->tp_basicsize == (expected_size)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)
#endif
#define __PYX_CHECK_TYPE_FOR_FREELISTS(t, expected_tp, expected_size)\
    (__PYX_CHECK_FINAL_TYPE_FOR_FREELISTS((t), (expected_tp), (expected_size)) &\
     (int) (!__Pyx_PyType_HasFeature((t), __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS)))
#endif

/* AllocateExtensionType.proto */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final);

/* DeallocKeepAlive.proto */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_DeallocKeepAliveBegin(o) do {\
        _Py_atomic_store_uintptr_relaxed(&(o)->ob_tid, _Py_ThreadId());\
        _Py_atomic_store_uint32_relaxed(&(o)->ob_ref_local, 1);\
        _Py_atomic_store_ssize_relaxed(&(o)->ob_ref_shared, 0);\
    } while (0)
#define __Pyx_DeallocKeepAliveEnd(o)\
        _Py_atomic_store_uint32_relaxed(&(o)->ob_ref_local, 0)
#else
#define __Pyx_DeallocKeepAliveBegin(o) __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1)
#define __Pyx_DeallocKeepAliveEnd(o)   __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1)
#endif

/* ValidateBasesTuple.proto (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* DelItemOnTypeDict.proto (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_8
#define __PYX_HAVE_RT_ImportType_proto_3_2_8
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_8(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_8(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_8 {
   __Pyx_ImportType_CheckSize_Error_3_2_8 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_8 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_8 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_8(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_8 check_size);
#endif

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_2_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* ListPack.proto */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* CalculateMetaclass.proto (used by Py3ClassCreate) */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* PyObjectCall2Args.proto (used by Py3ClassCreate) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* BufferStructDeclare.proto */
typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

/* MemviewRefcount.proto */
static CYTHON_INLINE int __pyx_add_acquisition_count_locked(
    __pyx_atomic_int_type *acquisition_count, PyThread_type_lock lock);
static CYTHON_INLINE int __pyx_sub_acquisition_count_locked(
    __pyx_atomic_int_type *acquisition_count, PyThread_type_lock lock);
#define __pyx_get_slice_count_pointer(memview) (&memview->acquisition_count)
#define __PYX_INC_MEMVIEW(slice, have_gil) __Pyx_INC_MEMVIEW(slice, have_gil, __LINE__)
#define __PYX_XCLEAR_MEMVIEW(slice, have_gil) __Pyx_XCLEAR_MEMVIEW(slice, have_gil, __LINE__)
static CYTHON_INLINE void __Pyx_INC_MEMVIEW(__Pyx_memviewslice *, int, int);
static CYTHON_INLINE void __Pyx_XCLEAR_MEMVIEW(__Pyx_memviewslice *, int, int);

/* MemviewSliceIsContig.proto */
static int __pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim);

/* OverlappingSlices.proto */
static int __pyx_slices_overlap(__Pyx_memviewslice *slice1,
                                __Pyx_memviewslice *slice2,
                                int ndim, size_t itemsize);

/* MemviewSliceInit.proto */
static int __Pyx_init_memviewslice(
                struct __pyx_memoryview_obj *memview,
                int ndim,
                __Pyx_memviewslice *memviewslice,
                int memview_is_new_reference);

/* SliceMemoryviewSlice.proto */
static CYTHON_INLINE int __pyx_memoryview_slice_memviewslice(
        __Pyx_memviewslice *dst,
        Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
        int dim, int new_ndim, int *suboffset_dim,
        Py_ssize_t start, Py_ssize_t stop, Py_ssize_t step,
        int have_start, int have_stop, int have_step,
        int is_slice);

/* MemviewObjectToObject.proto */
static PyObject *__pyx_memview_get_object(const char *itemp);
static int __pyx_memview_set_object(const char *itemp, PyObject *obj);

/* MemviewDtypeToObject.proto */
static CYTHON_INLINE PyObject *__pyx_memview_get_int(const char *itemp);
static CYTHON_INLINE int __pyx_memview_set_int(char *itemp, PyObject *obj);

/* IsLittleEndian.proto (used by BufferFormatCheck) */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void);

/* BufferFormatCheck.proto (used by MemviewSliceValidateAndInit) */
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts);
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              const __Pyx_TypeInfo* type);

/* TypeInfoCompare.proto (used by MemviewSliceValidateAndInit) */
static int __pyx_typeinfo_cmp(const __Pyx_TypeInfo *a, const __Pyx_TypeInfo *b);

/* MemviewSliceValidateAndInit.export */
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                const __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_int(PyObject *, int writable_flag);

/* ObjectToMemviewSlice.proto */
static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_object(PyObject *, int writable_flag);

/* MemviewSliceCopy.proto */
static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_OGRFieldType(OGRFieldType value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_OGRFieldSubType(OGRFieldSubType value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int64_t(int64_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_GIntBig(GIntBig value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE OGRFieldType __Pyx_PyLong_As_OGRFieldType(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE OGRFieldSubType __Pyx_PyLong_As_OGRFieldSubType(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int64_t __Pyx_PyLong_As_int64_t(PyObject *);

/* PyObjectCallMethod1.proto (used by UpdateUnpickledDict) */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* UpdateUnpickledDict.proto */
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index);

/* CheckUnpickleChecksum.proto */
static CYTHON_INLINE int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE char __Pyx_PyLong_As_char(PyObject *);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* IterNextPlain.proto (used by CoroutineBase) */
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator);
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void);
#endif

/* ReturnWithStopIteration.proto (used by CoroutineBase) */
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext);

/* CoroutineBase.proto (used by Generator) */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *gi_weakreflist;
#endif
    PyObject *classobj;
    PyObject *yieldfrom;
    __Pyx_pyiter_sendfunc yieldfrom_am_send;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
#if CYTHON_USE_SYS_MONITORING && (CYTHON_PROFILE || CYTHON_TRACE)
    PyMonitoringState __pyx_pymonitoring_state[__Pyx_MonitoringEventTypes_CyGen_count];
    uint64_t __pyx_pymonitoring_version;
#endif
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static __Pyx_PySendResult __Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static __Pyx_PySendResult __Pyx_Coroutine_Close(PyObject *self, PyObject **retval);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen);
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen);
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen);
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure);
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send);
#endif
static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg);

/* Generator.proto */
#define __Pyx_Generator_USED
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_mstate_global->__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);
static CYTHON_INLINE PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self); /* proto*/
static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto*/
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview__get_base(struct __pyx_memoryview_obj *__pyx_v_self); /* proto*/
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryviewslice__get_base(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto*/

/* Module declarations from "libc.stdint" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdlib" */

/* Module declarations from "libc.math" */

/* Module declarations from "cpython.pycapsule" */

/* Module declarations from "cython.view" */

/* Module declarations from "cython.dataclasses" */

/* Module declarations from "cython" */

/* Module declarations from "pyogrio._err" */
static PyObject *(*__pyx_f_7pyogrio_4_err_check_last_error)(void); /*proto*/
static int (*__pyx_f_7pyogrio_4_err_check_int)(int); /*proto*/
static void *(*__pyx_f_7pyogrio_4_err_check_pointer)(void *); /*proto*/

/* Module declarations from "libc.stdio" */

/* Module declarations from "pyogrio._ogr" */
static PyObject *(*__pyx_f_7pyogrio_4_ogr_get_string)(char const *, struct __pyx_opt_args_7pyogrio_4_ogr_get_string *__pyx_optional_args); /*proto*/

/* Module declarations from "pyogrio._geometry" */
static PyObject *(*__pyx_f_7pyogrio_9_geometry_get_geometry_type)(void *); /*proto*/
static OGRwkbGeometryType (*__pyx_f_7pyogrio_9_geometry_get_geometry_type_code)(PyObject *); /*proto*/

/* Module declarations from "pyogrio._vsi" */
static PyObject *(*__pyx_f_7pyogrio_4_vsi_get_ogr_vsimem_write_path)(PyObject *, PyObject *); /*proto*/
static PyObject *(*__pyx_f_7pyogrio_4_vsi_read_buffer_to_vsimem)(PyObject *); /*proto*/
static PyObject *(*__pyx_f_7pyogrio_4_vsi_read_vsimem_to_buffer)(PyObject *, PyObject *); /*proto*/
static PyObject *(*__pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel)(PyObject *, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from "pyogrio._io" */
static PyObject *__pyx_collections_abc_Sequence = 0;
static PyObject *generic = 0;
static PyObject *strided = 0;
static PyObject *indirect = 0;
static PyObject *contiguous = 0;
static PyObject *indirect_contiguous = 0;
static int __pyx_memoryview_thread_locks_used;
static PyThread_type_lock __pyx_memoryview_thread_locks[8];
static int __pyx_f_7pyogrio_3_io_start_transaction(OGRDataSourceH, int); /*proto*/
static int __pyx_f_7pyogrio_3_io_commit_transaction(OGRDataSourceH); /*proto*/
static char **__pyx_f_7pyogrio_3_io_dict_to_options(PyObject *); /*proto*/
static char const *__pyx_f_7pyogrio_3_io_override_threadlocal_config_option(PyObject *, PyObject *); /*proto*/
static void *__pyx_f_7pyogrio_3_io_ogr_open(char const *, int, char **); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_ogr_close(GDALDatasetH); /*proto*/
static OGRLayerH __pyx_f_7pyogrio_3_io_get_ogr_layer(GDALDatasetH, PyObject *); /*proto*/
static OGRLayerH __pyx_f_7pyogrio_3_io_execute_sql(GDALDatasetH, PyObject *, struct __pyx_opt_args_7pyogrio_3_io_execute_sql *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_crs(OGRLayerH); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_driver(OGRDataSourceH); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_feature_count(OGRLayerH, int); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_total_bounds(OGRLayerH, int); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_set_metadata(GDALMajorObjectH, PyObject *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_metadata(GDALMajorObjectH); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_detect_encoding(OGRDataSourceH, OGRLayerH); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_fields(OGRLayerH, PyObject *, struct __pyx_opt_args_7pyogrio_3_io_get_fields *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_apply_where_filter(OGRLayerH, PyObject *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_apply_bbox_filter(OGRLayerH, PyObject *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_apply_geometry_filter(OGRLayerH, PyObject *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_apply_skip_features(OGRLayerH, int); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_validate_feature_range(OGRLayerH, struct __pyx_opt_args_7pyogrio_3_io_validate_feature_range *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_process_geometry(OGRFeatureH, int, PyObject *, uint8_t); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_process_fields(OGRFeatureH, int, int, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, int); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_features(OGRLayerH, __Pyx_memviewslice, PyObject *, uint8_t, uint8_t, int, int, uint8_t, int); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_features_by_fid(OGRLayerH, __Pyx_memviewslice, __Pyx_memviewslice, PyObject *, uint8_t, uint8_t, int); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_bounds(OGRLayerH, int, int); /*proto*/
static void __pyx_f_7pyogrio_3_io_pycapsule_array_stream_deleter(PyObject *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_alloc_c_stream(struct ArrowArrayStream **); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_default_layer(OGRDataSourceH); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_layer_names(OGRDataSourceH); /*proto*/
static void *__pyx_f_7pyogrio_3_io_ogr_create(char const *, char const *, char **); /*proto*/
static void *__pyx_f_7pyogrio_3_io_create_crs(PyObject *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_infer_field_types(PyObject *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_create_ogr_dataset_layer(PyObject *, int, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, int, PyObject *, PyObject *, OGRDataSourceH *, OGRLayerH *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_get_arrow_extension_metadata(struct ArrowSchema const *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_is_arrow_geometry_field(struct ArrowSchema const *); /*proto*/
static PyObject *__pyx_f_7pyogrio_3_io_create_fields_from_arrow_schema(OGRLayerH, struct ArrowSchema const *, char **, PyObject *); /*proto*/
static int __pyx_array_allocate_buffer(struct __pyx_array_obj *); /*proto*/
static struct __pyx_array_obj *__pyx_array_new(PyObject *, Py_ssize_t, char *, char const *, char *); /*proto*/
static PyObject *__pyx_memoryview_new(PyObject *, int, int, __Pyx_TypeInfo const *); /*proto*/
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *); /*proto*/
static PyObject *_unellipsify(PyObject *, int); /*proto*/
static int assert_direct_dimensions(Py_ssize_t *, int); /*proto*/
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *, PyObject *); /*proto*/
static char *__pyx_pybuffer_index(Py_buffer *, char *, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memslice_transpose(__Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice, int, PyObject *(*)(char *), int (*)(char *, PyObject *), int); /*proto*/
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *); /*proto*/
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static Py_ssize_t abs_py_ssize_t(Py_ssize_t); /*proto*/
static char __pyx_get_best_slice_order(__Pyx_memviewslice *, int); /*proto*/
static void _copy_strided_to_strided(char *, Py_ssize_t *, char *, Py_ssize_t *, Py_ssize_t *, Py_ssize_t *, int, size_t); /*proto*/
static void copy_strided_to_strided(__Pyx_memviewslice *, __Pyx_memviewslice *, int, size_t); /*proto*/
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *, int); /*proto*/
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *, Py_ssize_t *, Py_ssize_t, int, char); /*proto*/
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *, __Pyx_memviewslice *, char, int); /*proto*/
static int __pyx_memoryview_err_extents(int, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memoryview_err_dim(PyObject *, PyObject *, int); /*proto*/
static int __pyx_memoryview_err(PyObject *, PyObject *); /*proto*/
static int __pyx_memoryview_err_no_memory(void); /*proto*/
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice, __Pyx_memviewslice, int, int, int); /*proto*/
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *, int, int); /*proto*/
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *, int, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *, int, size_t, void *, int); /*proto*/
static void __pyx_memoryview__slice_assign_scalar(char *, Py_ssize_t *, Py_ssize_t *, int, size_t, void *); /*proto*/
static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *, PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
static const __Pyx_TypeInfo __Pyx_TypeInfo_int = { "int", NULL, sizeof(int), { 0 }, 0, __PYX_IS_UNSIGNED(int) ? 'U' : 'I', __PYX_IS_UNSIGNED(int), 0 };
static const __Pyx_TypeInfo __Pyx_TypeInfo_object = { "Python object", NULL, sizeof(PyObject *), { 0 }, 0, 'O', 0, 0 };
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "pyogrio._io"
extern int __pyx_module_is_main_pyogrio___io;
int __pyx_module_is_main_pyogrio___io = 0;

/* Implementation of "pyogrio._io" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_round;
static PyObject *__pyx_builtin___import__;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_Ellipsis;
static PyObject *__pyx_builtin_id;
/* #### Code section: string_decls ### */
static const char __pyx_k_c[] = "c";
static const char __pyx_k_NO[] = "NO";
static const char __pyx_k_YES[] = "YES";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_fortran[] = "fortran";
static const char __pyx_k_ENCODING[] = "ENCODING";
static const char __pyx_k_INCLUDE_FID[] = "INCLUDE_FID";
static const char __pyx_k_SHAPE_ENCODING[] = "SHAPE_ENCODING";
static const char __pyx_k_DATETIME_AS_STRING[] = "DATETIME_AS_STRING";
static const char __pyx_k_arrow_array_stream[] = "arrow_array_stream";
static const char __pyx_k_MAX_FEATURES_IN_BATCH[] = "MAX_FEATURES_IN_BATCH";
static const char __pyx_k_GEOMETRY_METADATA_ENCODING[] = "GEOMETRY_METADATA_ENCODING";
static const char __pyx_k_IO_support_for_OGR_vector_data_s[] = "IO support for OGR vector data sources.";
/* #### Code section: decls ### */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object); /* proto */
static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_12get_metadata_genexpr(PyObject *__pyx_self, int __pyx_genexpr_arg_0); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_12get_features_add_data_chunk(PyObject *__pyx_self, PyObject *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_ogr_read(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer, PyObject *__pyx_v_encoding, int __pyx_v_read_geometry, int __pyx_v_force_2d, PyObject *__pyx_v_columns, int __pyx_v_skip_features, int __pyx_v_max_features, PyObject *__pyx_v_where, PyObject *__pyx_v_bbox, PyObject *__pyx_v_mask, PyObject *__pyx_v_fids, PyObject *__pyx_v_sql, PyObject *__pyx_v_sql_dialect, int __pyx_v_return_fids, int __pyx_v_datetime_as_string); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_12_ArrowStream___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_capsule); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_12_ArrowStream_2__arrow_c_stream__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_requested_schema); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_2ogr_open_arrow(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer, PyObject *__pyx_v_encoding, int __pyx_v_read_geometry, int __pyx_v_force_2d, PyObject *__pyx_v_columns, int __pyx_v_skip_features, int __pyx_v_max_features, PyObject *__pyx_v_where, PyObject *__pyx_v_bbox, PyObject *__pyx_v_mask, PyObject *__pyx_v_fids, PyObject *__pyx_v_sql, PyObject *__pyx_v_sql_dialect, int __pyx_v_return_fids, int __pyx_v_batch_size, PyObject *__pyx_v_use_pyarrow, PyObject *__pyx_v_datetime_as_string); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_5ogr_read_bounds(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_layer, CYTHON_UNUSED PyObject *__pyx_v_encoding, CYTHON_UNUSED int __pyx_v_read_geometry, CYTHON_UNUSED int __pyx_v_force_2d, CYTHON_UNUSED PyObject *__pyx_v_columns, int __pyx_v_skip_features, int __pyx_v_max_features, PyObject *__pyx_v_where, PyObject *__pyx_v_bbox, PyObject *__pyx_v_mask); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_7ogr_read_info(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer, PyObject *__pyx_v_encoding, int __pyx_v_force_feature_count, int __pyx_v_force_total_bounds); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_9ogr_list_layers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_11ogr_write(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_fp, PyObject *__pyx_v_layer, PyObject *__pyx_v_driver, PyObject *__pyx_v_geometry, PyObject *__pyx_v_fields, PyObject *__pyx_v_field_data, PyObject *__pyx_v_field_mask, PyObject *__pyx_v_crs, PyObject *__pyx_v_geometry_type, PyObject *__pyx_v_encoding, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer_kwargs, int __pyx_v_promote_to_multi, int __pyx_v_nan_as_null, int __pyx_v_append, PyObject *__pyx_v_dataset_metadata, PyObject *__pyx_v_layer_metadata, PyObject *__pyx_v_gdal_tz_offsets); /* proto */
static PyObject *__pyx_pf_7pyogrio_3_io_13ogr_write_arrow(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_fp, PyObject *__pyx_v_layer, PyObject *__pyx_v_driver, PyObject *__pyx_v_arrow_obj, PyObject *__pyx_v_crs, PyObject *__pyx_v_geometry_type, PyObject *__pyx_v_geometry_name, PyObject *__pyx_v_encoding, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer_kwargs, int __pyx_v_append, PyObject *__pyx_v_dataset_metadata, PyObject *__pyx_v_layer_metadata); /* proto */
static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_7pyogrio_4_err_ErrorHandler;
  PyObject *__pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata;
  PyObject *__pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr;
  PyObject *__pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features;
  PyObject *__pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow;
  PyObject *__pyx_type___pyx_array;
  PyObject *__pyx_type___pyx_MemviewEnum;
  PyObject *__pyx_type___pyx_memoryview;
  PyObject *__pyx_type___pyx_memoryviewslice;
  PyTypeObject *__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata;
  PyTypeObject *__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr;
  PyTypeObject *__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features;
  PyTypeObject *__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow;
  PyTypeObject *__pyx_array_type;
  PyTypeObject *__pyx_MemviewEnum_type;
  PyTypeObject *__pyx_memoryview_type;
  PyTypeObject *__pyx_memoryviewslice_type;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  __Pyx_CachedCFunction __pyx_umethod_PyUnicode_Type__upper;
  PyObject *__pyx_slice[2];
  PyObject *__pyx_tuple[10];
  PyObject *__pyx_codeobj_tab[11];
  PyObject *__pyx_string_tab[536];
  PyObject *__pyx_number_tab[11];
/* #### Code section: module_state_contents ### */
/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CythonFunctionShared.module_state_decls */
PyTypeObject *__pyx_CyFunctionType;


#if CYTHON_USE_FREELISTS
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata *__pyx_freelist_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata[8];
int __pyx_freecount_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_1_genexpr[8];
int __pyx_freecount_7pyogrio_3_io___pyx_scope_struct_1_genexpr;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features[8];
int __pyx_freecount_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow[8];
int __pyx_freecount_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow;
#endif
/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* IterNextPlain.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
PyObject *__Pyx_GetBuiltinNext_LimitedAPI_cache;
#endif

/* Generator.module_state_decls */
PyTypeObject *__pyx_GeneratorType;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_Applying_skip_features __pyx_string_tab[1]
#define __pyx_kp_u_Arrow_array_stream_was_already_r __pyx_string_tab[2]
#define __pyx_kp_u_Buffer_view_does_not_expose_stri __pyx_string_tab[3]
#define __pyx_kp_u_CRS_could_not_be_extracted_as_WK __pyx_string_tab[4]
#define __pyx_kp_u_Can_only_create_a_buffer_that_is __pyx_string_tab[5]
#define __pyx_kp_u_Cannot_assign_to_read_only_memor __pyx_string_tab[6]
#define __pyx_kp_u_Cannot_create_writable_memory_vi __pyx_string_tab[7]
#define __pyx_kp_u_Cannot_index_with_type __pyx_string_tab[8]
#define __pyx_kp_u_Cannot_transpose_memoryview_with __pyx_string_tab[9]
#define __pyx_kp_u_Could_not_add_feature_to_layer_a __pyx_string_tab[10]
#define __pyx_kp_u_Could_not_create_feature_at_inde __pyx_string_tab[11]
#define __pyx_kp_u_Could_not_create_geometry_at_ind __pyx_string_tab[12]
#define __pyx_kp_u_Could_not_create_geometry_from_W __pyx_string_tab[13]
#define __pyx_kp_u_Could_not_create_mask_geometry __pyx_string_tab[14]
#define __pyx_kp_u_Could_not_detect_driver_of_datas __pyx_string_tab[15]
#define __pyx_kp_u_Could_not_encode_value __pyx_string_tab[16]
#define __pyx_kp_u_Could_not_extract_valid_Arrow_ar __pyx_string_tab[17]
#define __pyx_kp_u_Could_not_find_field_index_for_f __pyx_string_tab[18]
#define __pyx_kp_u_Could_not_get_Arrow_schema_from __pyx_string_tab[19]
#define __pyx_kp_u_Could_not_get_field_definition_f __pyx_string_tab[20]
#define __pyx_kp_u_Could_not_get_layer_definition __pyx_string_tab[21]
#define __pyx_kp_u_Could_not_iterate_over_features __pyx_string_tab[22]
#define __pyx_kp_u_Could_not_obtain_driver __pyx_string_tab[23]
#define __pyx_kp_u_Could_not_read_feature_with_fid __pyx_string_tab[24]
#define __pyx_kp_u_Could_not_set_CRS __pyx_string_tab[25]
#define __pyx_kp_u_Could_not_set_geometry_for_featu __pyx_string_tab[26]
#define __pyx_kp_u_Could_not_set_metadata __pyx_string_tab[27]
#define __pyx_kp_u_Created __pyx_string_tab[28]
#define __pyx_kp_u_Dimension_d_is_not_direct __pyx_string_tab[29]
#define __pyx_kp_u_Driver __pyx_string_tab[30]
#define __pyx_kp_u_EPSG_2 __pyx_string_tab[31]
#define __pyx_kp_u_ESRI_Shapefile __pyx_string_tab[32]
#define __pyx_kp_u_Empty_shape_tuple_for_cython_arr __pyx_string_tab[33]
#define __pyx_kp_u_Error_adding_field __pyx_string_tab[34]
#define __pyx_kp_u_Error_executing_sql __pyx_string_tab[35]
#define __pyx_kp_u_Error_while_accessing_batch_from __pyx_string_tab[36]
#define __pyx_kp_u_Error_while_creating_field_from __pyx_string_tab[37]
#define __pyx_kp_u_Error_while_writing_batch_to_OGR __pyx_string_tab[38]
#define __pyx_kp_u_Failed_to_commit_transaction __pyx_string_tab[39]
#define __pyx_kp_u_Failed_to_create_dataset_with_dr __pyx_string_tab[40]
#define __pyx_kp_u_Failed_to_open_ArrowArrayStream __pyx_string_tab[41]
#define __pyx_kp_u_Failed_to_open_dataset_mode __pyx_string_tab[42]
#define __pyx_kp_u_Failed_to_start_transaction __pyx_string_tab[43]
#define __pyx_kp_u_Failed_to_write_features_to_data __pyx_string_tab[44]
#define __pyx_kp_u_GDAL_returned_more_records_than __pyx_string_tab[45]
#define __pyx_kp_u_IN __pyx_string_tab[46]
#define __pyx_kp_u_ISO_8859_1 __pyx_string_tab[47]
#define __pyx_kp_u_Indirect_dimensions_not_supporte __pyx_string_tab[48]
#define __pyx_kp_u_Invalid_SQL_query __pyx_string_tab[49]
#define __pyx_kp_u_Invalid_SQL_query_2 __pyx_string_tab[50]
#define __pyx_kp_u_Invalid_SQL_query_for_layer __pyx_string_tab[51]
#define __pyx_kp_u_Invalid_bbox __pyx_string_tab[52]
#define __pyx_kp_u_Invalid_mode_expected_c_or_fortr __pyx_string_tab[53]
#define __pyx_kp_u_Invalid_shape_in_axis __pyx_string_tab[54]
#define __pyx_kp_u_It_might_help_to_specify_the_co __pyx_string_tab[55]
#define __pyx_kp_u_Layer __pyx_string_tab[56]
#define __pyx_kp_u_MemoryView_of __pyx_string_tab[57]
#define __pyx_kp_u_More_than_one_layer_found_in __pyx_string_tab[58]
#define __pyx_kp_u_None __pyx_string_tab[59]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[60]
#define __pyx_kp_u_OGR_field_type_is_not_supported __pyx_string_tab[61]
#define __pyx_kp_u_Out_of_bounds_on_buffer_access_a __pyx_string_tab[62]
#define __pyx_kp_u_Reading_more_features_than_indic __pyx_string_tab[63]
#define __pyx_kp_u_Received_invalid_Arrow_schema_nu __pyx_string_tab[64]
#define __pyx_kp_u_SET_interest_layers __pyx_string_tab[65]
#define __pyx_kp_u_Skipping_field __pyx_string_tab[66]
#define __pyx_kp_u_Specify_layer_parameter_to_avoi __pyx_string_tab[67]
#define __pyx_kp_u_UTF_8 __pyx_string_tab[68]
#define __pyx_kp_u_Unable_to_convert_item_to_object __pyx_string_tab[69]
#define __pyx_kp_u_Writing_a_geometry_column_of_typ __pyx_string_tab[70]
#define __pyx_kp_u_You_must_provide_at_least_a_geom __pyx_string_tab[71]
#define __pyx_kp_u__10 __pyx_string_tab[72]
#define __pyx_kp_u__11 __pyx_string_tab[73]
#define __pyx_kp_u__12 __pyx_string_tab[74]
#define __pyx_kp_u__13 __pyx_string_tab[75]
#define __pyx_kp_u__14 __pyx_string_tab[76]
#define __pyx_kp_u__15 __pyx_string_tab[77]
#define __pyx_kp_u__16 __pyx_string_tab[78]
#define __pyx_kp_u__2 __pyx_string_tab[79]
#define __pyx_kp_u__3 __pyx_string_tab[80]
#define __pyx_kp_u__4 __pyx_string_tab[81]
#define __pyx_kp_u__5 __pyx_string_tab[82]
#define __pyx_kp_u__6 __pyx_string_tab[83]
#define __pyx_kp_u__7 __pyx_string_tab[84]
#define __pyx_kp_u__8 __pyx_string_tab[85]
#define __pyx_kp_u__9 __pyx_string_tab[86]
#define __pyx_kp_u_a_supported_file_format __pyx_string_tab[87]
#define __pyx_kp_u_add_note __pyx_string_tab[88]
#define __pyx_kp_u_and __pyx_string_tab[89]
#define __pyx_kp_u_and_type __pyx_string_tab[90]
#define __pyx_kp_u_at_0x __pyx_string_tab[91]
#define __pyx_kp_u_at_least_one_of_read_geometry_or __pyx_string_tab[92]
#define __pyx_kp_u_cannot_provide_both_encoding_par __pyx_string_tab[93]
#define __pyx_kp_u_cannot_provide_both_encoding_par_2 __pyx_string_tab[94]
#define __pyx_kp_u_cannot_set_both_bbox_and_mask __pyx_string_tab[95]
#define __pyx_kp_u_cannot_set_both_fids_and_any_of __pyx_string_tab[96]
#define __pyx_kp_u_cannot_set_both_fids_and_any_of_2 __pyx_string_tab[97]
#define __pyx_kp_u_check_that_it_was_installed_cor __pyx_string_tab[98]
#define __pyx_kp_u_collections_abc __pyx_string_tab[99]
#define __pyx_kp_u_contiguous_and_direct __pyx_string_tab[100]
#define __pyx_kp_u_contiguous_and_indirect __pyx_string_tab[101]
#define __pyx_kp_u_could_not_be_opened __pyx_string_tab[102]
#define __pyx_kp_u_datetime64_D __pyx_string_tab[103]
#define __pyx_kp_u_datetime64_ms __pyx_string_tab[104]
#define __pyx_kp_u_default __pyx_string_tab[105]
#define __pyx_kp_u_disable __pyx_string_tab[106]
#define __pyx_kp_u_does_not_support_write_function __pyx_string_tab[107]
#define __pyx_kp_u_enable __pyx_string_tab[108]
#define __pyx_kp_u_error_applying_filter_for __pyx_string_tab[109]
#define __pyx_kp_u_failed_to_prepare_SQL __pyx_string_tab[110]
#define __pyx_kp_u_fids_max_number_for_drivers_wit __pyx_string_tab[111]
#define __pyx_kp_u_field_data_and_field_mask_must_b __pyx_string_tab[112]
#define __pyx_kp_u_field_data_array_needs_to_be_sam __pyx_string_tab[113]
#define __pyx_kp_u_field_data_arrays_must_be_same_l __pyx_string_tab[114]
#define __pyx_kp_u_field_data_arrays_must_be_same_l_2 __pyx_string_tab[115]
#define __pyx_kp_u_field_index __pyx_string_tab[116]
#define __pyx_kp_u_field_mask_arrays_must_be_same_l __pyx_string_tab[117]
#define __pyx_kp_u_field_type_is_not_supported __pyx_string_tab[118]
#define __pyx_kp_u_for_WKB_type __pyx_string_tab[119]
#define __pyx_kp_u_forcing_2D_is_not_supported_for __pyx_string_tab[120]
#define __pyx_kp_u_gc __pyx_string_tab[121]
#define __pyx_kp_u_gdb __pyx_string_tab[122]
#define __pyx_kp_u_got __pyx_string_tab[123]
#define __pyx_kp_u_got_differing_extents_in_dimensi __pyx_string_tab[124]
#define __pyx_kp_u_in_field __pyx_string_tab[125]
#define __pyx_kp_u_is_not_yet_supported_Only_WKB_i __pyx_string_tab[126]
#define __pyx_kp_u_isenabled __pyx_string_tab[127]
#define __pyx_kp_u_itemsize_0_for_cython_array __pyx_string_tab[128]
#define __pyx_kp_u_layer_parameter_must_be_a_str_o __pyx_string_tab[129]
#define __pyx_kp_u_list_float64 __pyx_string_tab[130]
#define __pyx_kp_u_list_int32 __pyx_string_tab[131]
#define __pyx_kp_u_list_int64 __pyx_string_tab[132]
#define __pyx_kp_u_list_str __pyx_string_tab[133]
#define __pyx_kp_u_max_features_must_be_0 __pyx_string_tab[134]
#define __pyx_kp_u_no_default___reduce___due_to_non __pyx_string_tab[135]
#define __pyx_kp_u_non_UTF_8_encoding_is_not_suppor __pyx_string_tab[136]
#define __pyx_kp_u_object_2 __pyx_string_tab[137]
#define __pyx_kp_u_pyogrio__io_pyx __pyx_string_tab[138]
#define __pyx_kp_u_raised_an_error __pyx_string_tab[139]
#define __pyx_kp_u_records __pyx_string_tab[140]
#define __pyx_kp_u_requested_schema_is_not_supporte __pyx_string_tab[141]
#define __pyx_kp_u_skip_features_must_be_0 __pyx_string_tab[142]
#define __pyx_kp_u_specifying_max_features_is_not_s __pyx_string_tab[143]
#define __pyx_kp_u_sql_parameter_cannot_be_combine __pyx_string_tab[144]
#define __pyx_kp_u_strided_and_direct __pyx_string_tab[145]
#define __pyx_kp_u_strided_and_direct_or_indirect __pyx_string_tab[146]
#define __pyx_kp_u_strided_and_indirect __pyx_string_tab[147]
#define __pyx_kp_u_to_layer __pyx_string_tab[148]
#define __pyx_kp_u_to_string __pyx_string_tab[149]
#define __pyx_kp_u_unable_to_allocate_array_data __pyx_string_tab[150]
#define __pyx_kp_u_unable_to_allocate_shape_and_str __pyx_string_tab[151]
#define __pyx_kp_u_unsupported_OGR_type __pyx_string_tab[152]
#define __pyx_kp_u_utf_8 __pyx_string_tab[153]
#define __pyx_kp_u_with_name __pyx_string_tab[154]
#define __pyx_n_u_ASCII __pyx_string_tab[155]
#define __pyx_n_u_ArrowStream __pyx_string_tab[156]
#define __pyx_n_u_ArrowStream___arrow_c_stream __pyx_string_tab[157]
#define __pyx_n_u_ArrowStream___init __pyx_string_tab[158]
#define __pyx_n_u_CPLE_BaseError __pyx_string_tab[159]
#define __pyx_n_u_CPLE_NotSupportedError __pyx_string_tab[160]
#define __pyx_n_u_CRSError __pyx_string_tab[161]
#define __pyx_n_u_DTYPE_OGR_FIELD_TYPES __pyx_string_tab[162]
#define __pyx_n_u_DataLayerError __pyx_string_tab[163]
#define __pyx_n_u_DataSourceError __pyx_string_tab[164]
#define __pyx_n_u_ENCODING __pyx_string_tab[165]
#define __pyx_n_u_EPSG __pyx_string_tab[166]
#define __pyx_n_u_Ellipsis __pyx_string_tab[167]
#define __pyx_n_u_FID __pyx_string_tab[168]
#define __pyx_n_u_FIELD_SUBTYPES __pyx_string_tab[169]
#define __pyx_n_u_FIELD_SUBTYPE_NAMES __pyx_string_tab[170]
#define __pyx_n_u_FIELD_TYPES __pyx_string_tab[171]
#define __pyx_n_u_FIELD_TYPE_NAMES __pyx_string_tab[172]
#define __pyx_n_u_FeatureError __pyx_string_tab[173]
#define __pyx_n_u_FieldError __pyx_string_tab[174]
#define __pyx_n_u_FlatGeobuf __pyx_string_tab[175]
#define __pyx_n_u_GEOMETRY_NAME __pyx_string_tab[176]
#define __pyx_n_u_GeoJSON __pyx_string_tab[177]
#define __pyx_n_u_GeoJSONSeq __pyx_string_tab[178]
#define __pyx_n_u_GeometryError __pyx_string_tab[179]
#define __pyx_n_u_MULTI_EXTENSIONS __pyx_string_tab[180]
#define __pyx_n_u_NaT __pyx_string_tab[181]
#define __pyx_n_u_NullPointerError __pyx_string_tab[182]
#define __pyx_n_u_O __pyx_string_tab[183]
#define __pyx_n_u_ODS __pyx_string_tab[184]
#define __pyx_n_u_OFSTBoolean __pyx_string_tab[185]
#define __pyx_n_u_OFSTFloat32 __pyx_string_tab[186]
#define __pyx_n_u_OFSTInt16 __pyx_string_tab[187]
#define __pyx_n_u_OFSTJSON __pyx_string_tab[188]
#define __pyx_n_u_OFSTMaxSubType __pyx_string_tab[189]
#define __pyx_n_u_OFSTNone __pyx_string_tab[190]
#define __pyx_n_u_OFSTUUID __pyx_string_tab[191]
#define __pyx_n_u_OFTBinary __pyx_string_tab[192]
#define __pyx_n_u_OFTDate __pyx_string_tab[193]
#define __pyx_n_u_OFTDateTime __pyx_string_tab[194]
#define __pyx_n_u_OFTInteger __pyx_string_tab[195]
#define __pyx_n_u_OFTInteger64 __pyx_string_tab[196]
#define __pyx_n_u_OFTInteger64List __pyx_string_tab[197]
#define __pyx_n_u_OFTIntegerList __pyx_string_tab[198]
#define __pyx_n_u_OFTReal __pyx_string_tab[199]
#define __pyx_n_u_OFTRealList __pyx_string_tab[200]
#define __pyx_n_u_OFTString __pyx_string_tab[201]
#define __pyx_n_u_OFTStringList __pyx_string_tab[202]
#define __pyx_n_u_OFTTime __pyx_string_tab[203]
#define __pyx_n_u_OFTWideString __pyx_string_tab[204]
#define __pyx_n_u_OFTWideStringList __pyx_string_tab[205]
#define __pyx_n_u_OGC_FID __pyx_string_tab[206]
#define __pyx_n_u_OGR_GEOMETRY __pyx_string_tab[207]
#define __pyx_n_u_OSM __pyx_string_tab[208]
#define __pyx_n_u_OpenFileGDB __pyx_string_tab[209]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[210]
#define __pyx_n_u_RecordBatchStreamReader __pyx_string_tab[211]
#define __pyx_n_u_SHAPE_ENCODING __pyx_string_tab[212]
#define __pyx_n_u_SQLite __pyx_string_tab[213]
#define __pyx_n_u_Sequence __pyx_string_tab[214]
#define __pyx_n_u_UTF8 __pyx_string_tab[215]
#define __pyx_n_u_Unknown __pyx_string_tab[216]
#define __pyx_n_u_View_MemoryView __pyx_string_tab[217]
#define __pyx_n_u_XLSX __pyx_string_tab[218]
#define __pyx_n_u_abc __pyx_string_tab[219]
#define __pyx_n_u_add_data_chunk __pyx_string_tab[220]
#define __pyx_n_u_allocate_buffer __pyx_string_tab[221]
#define __pyx_n_u_annotate __pyx_string_tab[222]
#define __pyx_n_u_append __pyx_string_tab[223]
#define __pyx_n_u_array __pyx_string_tab[224]
#define __pyx_n_u_arrow_c_stream __pyx_string_tab[225]
#define __pyx_n_u_arrow_obj __pyx_string_tab[226]
#define __pyx_n_u_asarray __pyx_string_tab[227]
#define __pyx_n_u_astype __pyx_string_tab[228]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[229]
#define __pyx_n_u_b __pyx_string_tab[230]
#define __pyx_n_u_base __pyx_string_tab[231]
#define __pyx_n_u_basename __pyx_string_tab[232]
#define __pyx_n_u_batch_size __pyx_string_tab[233]
#define __pyx_n_u_bbox __pyx_string_tab[234]
#define __pyx_n_u_bool __pyx_string_tab[235]
#define __pyx_n_u_bounds __pyx_string_tab[236]
#define __pyx_n_u_byteorder __pyx_string_tab[237]
#define __pyx_n_u_bytes __pyx_string_tab[238]
#define __pyx_n_u_c __pyx_string_tab[239]
#define __pyx_n_u_capabilities __pyx_string_tab[240]
#define __pyx_n_u_capsule __pyx_string_tab[241]
#define __pyx_n_u_capsule_2 __pyx_string_tab[242]
#define __pyx_n_u_capture_errors __pyx_string_tab[243]
#define __pyx_n_u_class __pyx_string_tab[244]
#define __pyx_n_u_class_getitem __pyx_string_tab[245]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[246]
#define __pyx_n_u_close __pyx_string_tab[247]
#define __pyx_n_u_columns __pyx_string_tab[248]
#define __pyx_n_u_concatenate __pyx_string_tab[249]
#define __pyx_n_u_contextlib __pyx_string_tab[250]
#define __pyx_n_u_contextmanager __pyx_string_tab[251]
#define __pyx_n_u_count __pyx_string_tab[252]
#define __pyx_n_u_crs __pyx_string_tab[253]
#define __pyx_n_u_dataset_kwargs __pyx_string_tab[254]
#define __pyx_n_u_dataset_metadata __pyx_string_tab[255]
#define __pyx_n_u_dataset_options __pyx_string_tab[256]
#define __pyx_n_u_date __pyx_string_tab[257]
#define __pyx_n_u_datetime __pyx_string_tab[258]
#define __pyx_n_u_datetime64 __pyx_string_tab[259]
#define __pyx_n_u_datetime_as_string __pyx_string_tab[260]
#define __pyx_n_u_datetime_value __pyx_string_tab[261]
#define __pyx_n_u_day __pyx_string_tab[262]
#define __pyx_n_u_decode __pyx_string_tab[263]
#define __pyx_n_u_dict __pyx_string_tab[264]
#define __pyx_n_u_doc __pyx_string_tab[265]
#define __pyx_n_u_driver __pyx_string_tab[266]
#define __pyx_n_u_dtype __pyx_string_tab[267]
#define __pyx_n_u_dtype_is_object __pyx_string_tab[268]
#define __pyx_n_u_dtypes __pyx_string_tab[269]
#define __pyx_n_u_empty __pyx_string_tab[270]
#define __pyx_n_u_encode __pyx_string_tab[271]
#define __pyx_n_u_encoding __pyx_string_tab[272]
#define __pyx_n_u_enter __pyx_string_tab[273]
#define __pyx_n_u_enumerate __pyx_string_tab[274]
#define __pyx_n_u_err __pyx_string_tab[275]
#define __pyx_n_u_error __pyx_string_tab[276]
#define __pyx_n_u_ex __pyx_string_tab[277]
#define __pyx_n_u_exc __pyx_string_tab[278]
#define __pyx_n_u_exists __pyx_string_tab[279]
#define __pyx_n_u_exit __pyx_string_tab[280]
#define __pyx_n_u_fast_feature_count __pyx_string_tab[281]
#define __pyx_n_u_fast_set_next_by_index __pyx_string_tab[282]
#define __pyx_n_u_fast_spatial_filter __pyx_string_tab[283]
#define __pyx_n_u_fast_total_bounds __pyx_string_tab[284]
#define __pyx_n_u_feature_count __pyx_string_tab[285]
#define __pyx_n_u_features __pyx_string_tab[286]
#define __pyx_n_u_fid __pyx_string_tab[287]
#define __pyx_n_u_fid_column __pyx_string_tab[288]
#define __pyx_n_u_fid_column_where __pyx_string_tab[289]
#define __pyx_n_u_fid_data __pyx_string_tab[290]
#define __pyx_n_u_fids __pyx_string_tab[291]
#define __pyx_n_u_fids_str __pyx_string_tab[292]
#define __pyx_n_u_field __pyx_string_tab[293]
#define __pyx_n_u_field_b __pyx_string_tab[294]
#define __pyx_n_u_field_c __pyx_string_tab[295]
#define __pyx_n_u_field_data __pyx_string_tab[296]
#define __pyx_n_u_field_data_chunk __pyx_string_tab[297]
#define __pyx_n_u_field_idx __pyx_string_tab[298]
#define __pyx_n_u_field_index_2 __pyx_string_tab[299]
#define __pyx_n_u_field_indexes __pyx_string_tab[300]
#define __pyx_n_u_field_mask __pyx_string_tab[301]
#define __pyx_n_u_field_ogr_type __pyx_string_tab[302]
#define __pyx_n_u_field_ogr_types __pyx_string_tab[303]
#define __pyx_n_u_field_subtype __pyx_string_tab[304]
#define __pyx_n_u_field_type __pyx_string_tab[305]
#define __pyx_n_u_field_types __pyx_string_tab[306]
#define __pyx_n_u_field_value __pyx_string_tab[307]
#define __pyx_n_u_fields __pyx_string_tab[308]
#define __pyx_n_u_fields_c __pyx_string_tab[309]
#define __pyx_n_u_flags __pyx_string_tab[310]
#define __pyx_n_u_float __pyx_string_tab[311]
#define __pyx_n_u_float32 __pyx_string_tab[312]
#define __pyx_n_u_float64 __pyx_string_tab[313]
#define __pyx_n_u_force_2d __pyx_string_tab[314]
#define __pyx_n_u_force_feature_count __pyx_string_tab[315]
#define __pyx_n_u_force_total_bounds __pyx_string_tab[316]
#define __pyx_n_u_format __pyx_string_tab[317]
#define __pyx_n_u_fortran __pyx_string_tab[318]
#define __pyx_n_u_from_bytes __pyx_string_tab[319]
#define __pyx_n_u_func __pyx_string_tab[320]
#define __pyx_n_u_gdal_msg __pyx_string_tab[321]
#define __pyx_n_u_gdal_tz __pyx_string_tab[322]
#define __pyx_n_u_gdal_tz_offsets __pyx_string_tab[323]
#define __pyx_n_u_genexpr __pyx_string_tab[324]
#define __pyx_n_u_geometries __pyx_string_tab[325]
#define __pyx_n_u_geometry __pyx_string_tab[326]
#define __pyx_n_u_geometry_name __pyx_string_tab[327]
#define __pyx_n_u_geometry_type __pyx_string_tab[328]
#define __pyx_n_u_get __pyx_string_tab[329]
#define __pyx_n_u_getLogger __pyx_string_tab[330]
#define __pyx_n_u_get_features_locals_add_data_chu __pyx_string_tab[331]
#define __pyx_n_u_get_metadata_locals_genexpr __pyx_string_tab[332]
#define __pyx_n_u_getpreferredencoding __pyx_string_tab[333]
#define __pyx_n_u_getstate __pyx_string_tab[334]
#define __pyx_n_u_hour __pyx_string_tab[335]
#define __pyx_n_u_i __pyx_string_tab[336]
#define __pyx_n_u_id __pyx_string_tab[337]
#define __pyx_n_u_idx __pyx_string_tab[338]
#define __pyx_n_u_ignored_fields __pyx_string_tab[339]
#define __pyx_n_u_import __pyx_string_tab[340]
#define __pyx_n_u_import_from_c __pyx_string_tab[341]
#define __pyx_n_u_index __pyx_string_tab[342]
#define __pyx_n_u_info __pyx_string_tab[343]
#define __pyx_n_u_init __pyx_string_tab[344]
#define __pyx_n_u_int __pyx_string_tab[345]
#define __pyx_n_u_int16 __pyx_string_tab[346]
#define __pyx_n_u_int32 __pyx_string_tab[347]
#define __pyx_n_u_int64 __pyx_string_tab[348]
#define __pyx_n_u_int8 __pyx_string_tab[349]
#define __pyx_n_u_intc __pyx_string_tab[350]
#define __pyx_n_u_intersect1d __pyx_string_tab[351]
#define __pyx_n_u_is_coroutine __pyx_string_tab[352]
#define __pyx_n_u_isdir __pyx_string_tab[353]
#define __pyx_n_u_isnan __pyx_string_tab[354]
#define __pyx_n_u_isnat __pyx_string_tab[355]
#define __pyx_n_u_isoformat __pyx_string_tab[356]
#define __pyx_n_u_item __pyx_string_tab[357]
#define __pyx_n_u_items __pyx_string_tab[358]
#define __pyx_n_u_itemsize __pyx_string_tab[359]
#define __pyx_n_u_kind __pyx_string_tab[360]
#define __pyx_n_u_layer __pyx_string_tab[361]
#define __pyx_n_u_layer_created __pyx_string_tab[362]
#define __pyx_n_u_layer_kwargs __pyx_string_tab[363]
#define __pyx_n_u_layer_metadata __pyx_string_tab[364]
#define __pyx_n_u_layer_name __pyx_string_tab[365]
#define __pyx_n_u_layers __pyx_string_tab[366]
#define __pyx_n_u_list __pyx_string_tab[367]
#define __pyx_n_u_locale __pyx_string_tab[368]
#define __pyx_n_u_log __pyx_string_tab[369]
#define __pyx_n_u_logging __pyx_string_tab[370]
#define __pyx_n_u_main __pyx_string_tab[371]
#define __pyx_n_u_mask __pyx_string_tab[372]
#define __pyx_n_u_max_features __pyx_string_tab[373]
#define __pyx_n_u_memview __pyx_string_tab[374]
#define __pyx_n_u_meta __pyx_string_tab[375]
#define __pyx_n_u_metaclass __pyx_string_tab[376]
#define __pyx_n_u_microsecond __pyx_string_tab[377]
#define __pyx_n_u_minute __pyx_string_tab[378]
#define __pyx_n_u_mode __pyx_string_tab[379]
#define __pyx_n_u_module __pyx_string_tab[380]
#define __pyx_n_u_month __pyx_string_tab[381]
#define __pyx_n_u_name __pyx_string_tab[382]
#define __pyx_n_u_name_2 __pyx_string_tab[383]
#define __pyx_n_u_name_b __pyx_string_tab[384]
#define __pyx_n_u_nan __pyx_string_tab[385]
#define __pyx_n_u_nan_as_null __pyx_string_tab[386]
#define __pyx_n_u_ndarray __pyx_string_tab[387]
#define __pyx_n_u_ndim __pyx_string_tab[388]
#define __pyx_n_u_new __pyx_string_tab[389]
#define __pyx_n_u_next __pyx_string_tab[390]
#define __pyx_n_u_np __pyx_string_tab[391]
#define __pyx_n_u_num_features __pyx_string_tab[392]
#define __pyx_n_u_num_field_data __pyx_string_tab[393]
#define __pyx_n_u_num_fields __pyx_string_tab[394]
#define __pyx_n_u_num_records __pyx_string_tab[395]
#define __pyx_n_u_numpy __pyx_string_tab[396]
#define __pyx_n_u_obj __pyx_string_tab[397]
#define __pyx_n_u_object __pyx_string_tab[398]
#define __pyx_n_u_ogr_dataset __pyx_string_tab[399]
#define __pyx_n_u_ogr_feature __pyx_string_tab[400]
#define __pyx_n_u_ogr_featuredef __pyx_string_tab[401]
#define __pyx_n_u_ogr_fielddef __pyx_string_tab[402]
#define __pyx_n_u_ogr_geometry __pyx_string_tab[403]
#define __pyx_n_u_ogr_layer __pyx_string_tab[404]
#define __pyx_n_u_ogr_list_layers __pyx_string_tab[405]
#define __pyx_n_u_ogr_open_arrow __pyx_string_tab[406]
#define __pyx_n_u_ogr_read __pyx_string_tab[407]
#define __pyx_n_u_ogr_read_bounds __pyx_string_tab[408]
#define __pyx_n_u_ogr_read_info __pyx_string_tab[409]
#define __pyx_n_u_ogr_subtypes __pyx_string_tab[410]
#define __pyx_n_u_ogr_types __pyx_string_tab[411]
#define __pyx_n_u_ogr_write __pyx_string_tab[412]
#define __pyx_n_u_ogr_write_arrow __pyx_string_tab[413]
#define __pyx_n_u_options __pyx_string_tab[414]
#define __pyx_n_u_opts __pyx_string_tab[415]
#define __pyx_n_u_os __pyx_string_tab[416]
#define __pyx_n_u_override_shape_encoding __pyx_string_tab[417]
#define __pyx_n_u_pa __pyx_string_tab[418]
#define __pyx_n_u_pack __pyx_string_tab[419]
#define __pyx_n_u_path __pyx_string_tab[420]
#define __pyx_n_u_path_or_buffer __pyx_string_tab[421]
#define __pyx_n_u_path_or_fp __pyx_string_tab[422]
#define __pyx_n_u_pop __pyx_string_tab[423]
#define __pyx_n_u_precision __pyx_string_tab[424]
#define __pyx_n_u_prepare __pyx_string_tab[425]
#define __pyx_n_u_prev_shape_encoding __pyx_string_tab[426]
#define __pyx_n_u_promote_to_multi __pyx_string_tab[427]
#define __pyx_n_u_pyarrow __pyx_string_tab[428]
#define __pyx_n_u_pyogrio__err __pyx_string_tab[429]
#define __pyx_n_u_pyogrio__io __pyx_string_tab[430]
#define __pyx_n_u_pyogrio__ogr __pyx_string_tab[431]
#define __pyx_n_u_pyogrio_errors __pyx_string_tab[432]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[433]
#define __pyx_n_u_pyx_state __pyx_string_tab[434]
#define __pyx_n_u_pyx_type __pyx_string_tab[435]
#define __pyx_n_u_pyx_unpickle_Enum __pyx_string_tab[436]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[437]
#define __pyx_n_u_qualname __pyx_string_tab[438]
#define __pyx_n_u_random_read __pyx_string_tab[439]
#define __pyx_n_u_read_geometry __pyx_string_tab[440]
#define __pyx_n_u_reader __pyx_string_tab[441]
#define __pyx_n_u_reduce __pyx_string_tab[442]
#define __pyx_n_u_reduce_cython __pyx_string_tab[443]
#define __pyx_n_u_reduce_ex __pyx_string_tab[444]
#define __pyx_n_u_register __pyx_string_tab[445]
#define __pyx_n_u_replace __pyx_string_tab[446]
#define __pyx_n_u_requested_schema __pyx_string_tab[447]
#define __pyx_n_u_return_fids __pyx_string_tab[448]
#define __pyx_n_u_return_indices __pyx_string_tab[449]
#define __pyx_n_u_reval __pyx_string_tab[450]
#define __pyx_n_u_rmtree __pyx_string_tab[451]
#define __pyx_n_u_round __pyx_string_tab[452]
#define __pyx_n_u_schema __pyx_string_tab[453]
#define __pyx_n_u_second __pyx_string_tab[454]
#define __pyx_n_u_self __pyx_string_tab[455]
#define __pyx_n_u_send __pyx_string_tab[456]
#define __pyx_n_u_set_name __pyx_string_tab[457]
#define __pyx_n_u_setdefault __pyx_string_tab[458]
#define __pyx_n_u_setstate __pyx_string_tab[459]
#define __pyx_n_u_setstate_cython __pyx_string_tab[460]
#define __pyx_n_u_shape __pyx_string_tab[461]
#define __pyx_n_u_shutil __pyx_string_tab[462]
#define __pyx_n_u_signed __pyx_string_tab[463]
#define __pyx_n_u_size __pyx_string_tab[464]
#define __pyx_n_u_skip_features __pyx_string_tab[465]
#define __pyx_n_u_sort __pyx_string_tab[466]
#define __pyx_n_u_split __pyx_string_tab[467]
#define __pyx_n_u_splitext __pyx_string_tab[468]
#define __pyx_n_u_sql __pyx_string_tab[469]
#define __pyx_n_u_sql_dialect __pyx_string_tab[470]
#define __pyx_n_u_stacklevel __pyx_string_tab[471]
#define __pyx_n_u_start __pyx_string_tab[472]
#define __pyx_n_u_startswith __pyx_string_tab[473]
#define __pyx_n_u_step __pyx_string_tab[474]
#define __pyx_n_u_stop __pyx_string_tab[475]
#define __pyx_n_u_str __pyx_string_tab[476]
#define __pyx_n_u_stream __pyx_string_tab[477]
#define __pyx_n_u_stream_capsule __pyx_string_tab[478]
#define __pyx_n_u_struct __pyx_string_tab[479]
#define __pyx_n_u_supports_transactions __pyx_string_tab[480]
#define __pyx_n_u_sys __pyx_string_tab[481]
#define __pyx_n_u_test __pyx_string_tab[482]
#define __pyx_n_u_throw __pyx_string_tab[483]
#define __pyx_n_u_time __pyx_string_tab[484]
#define __pyx_n_u_total_bounds __pyx_string_tab[485]
#define __pyx_n_u_type __pyx_string_tab[486]
#define __pyx_n_u_tz_array __pyx_string_tab[487]
#define __pyx_n_u_u __pyx_string_tab[488]
#define __pyx_n_u_uint __pyx_string_tab[489]
#define __pyx_n_u_uint16 __pyx_string_tab[490]
#define __pyx_n_u_uint32 __pyx_string_tab[491]
#define __pyx_n_u_uint64 __pyx_string_tab[492]
#define __pyx_n_u_uint8 __pyx_string_tab[493]
#define __pyx_n_u_unlink __pyx_string_tab[494]
#define __pyx_n_u_unpack __pyx_string_tab[495]
#define __pyx_n_u_update __pyx_string_tab[496]
#define __pyx_n_u_upper __pyx_string_tab[497]
#define __pyx_n_u_use_pyarrow __pyx_string_tab[498]
#define __pyx_n_u_use_tmp_vsimem __pyx_string_tab[499]
#define __pyx_n_u_value __pyx_string_tab[500]
#define __pyx_n_u_value_b __pyx_string_tab[501]
#define __pyx_n_u_values __pyx_string_tab[502]
#define __pyx_n_u_warn __pyx_string_tab[503]
#define __pyx_n_u_warning __pyx_string_tab[504]
#define __pyx_n_u_warnings __pyx_string_tab[505]
#define __pyx_n_u_where __pyx_string_tab[506]
#define __pyx_n_u_width __pyx_string_tab[507]
#define __pyx_n_u_wkb __pyx_string_tab[508]
#define __pyx_n_u_wkb_buffer __pyx_string_tab[509]
#define __pyx_n_u_wkbtype __pyx_string_tab[510]
#define __pyx_n_u_x __pyx_string_tab[511]
#define __pyx_n_u_year __pyx_string_tab[512]
#define __pyx_n_u_zeros __pyx_string_tab[513]
#define __pyx_kp_b_ARROW_extension_metadata __pyx_string_tab[514]
#define __pyx_kp_b_ARROW_extension_name __pyx_string_tab[515]
#define __pyx_kp_b_OGRwkbGeometryType_PyObject_PyOb __pyx_string_tab[516]
#define __pyx_kp_b_PyObject_PyObject_PyObject_PyObj __pyx_string_tab[517]
#define __pyx_kp_b_PyObject_char_const_struct___pyx __pyx_string_tab[518]
#define __pyx_kp_b_PyObject_void_int_int_void_void __pyx_string_tab[519]
#define __pyx_kp_b_geoarrow __pyx_string_tab[520]
#define __pyx_kp_b_geoarrow_wkb __pyx_string_tab[521]
#define __pyx_kp_b_iso88591_A_1_gQb_awivRq_1_war_q_y_a_Q_O5 __pyx_string_tab[522]
#define __pyx_kp_b_iso88591_A_L __pyx_string_tab[523]
#define __pyx_kp_b_iso88591_G1_Qa_t1 __pyx_string_tab[524]
#define __pyx_kp_b_iso88591_Q __pyx_string_tab[525]
#define __pyx_kp_b_iso88591__17 __pyx_string_tab[526]
#define __pyx_kp_b_iso88591_a_q_1_Q_a_1_Q_1_A_c_G_Q_Qk_z_a __pyx_string_tab[527]
#define __pyx_kp_b_iso88591_a_q_q_A_Q_A_8_Q_0_oQ_N_a_1_T_s __pyx_string_tab[528]
#define __pyx_kp_b_iso88591_z_a_55I_hat7_S_s_Qa_1_1_1 __pyx_string_tab[529]
#define __pyx_kp_b_iso88591_z_a_q_1_A_1_a_DA_hat7_S_6_A_Qa __pyx_string_tab[530]
#define __pyx_kp_b_iso88591_z_a_q_uD_j_Rq_j_Ba_j_A_hat7_S_6 __pyx_string_tab[531]
#define __pyx_kp_b_iso88591_z_q_1_a_q_1_uG1_uG1_uG1_t7_q_q __pyx_string_tab[532]
#define __pyx_kp_b_ogc_wkb __pyx_string_tab[533]
#define __pyx_n_b_GEOARROW __pyx_string_tab[534]
#define __pyx_n_b_O __pyx_string_tab[535]
#define __pyx_int_0 __pyx_number_tab[0]
#define __pyx_int_neg_1 __pyx_number_tab[1]
#define __pyx_int_1 __pyx_number_tab[2]
#define __pyx_int_2 __pyx_number_tab[3]
#define __pyx_int_3 __pyx_number_tab[4]
#define __pyx_int_4 __pyx_number_tab[5]
#define __pyx_int_5 __pyx_number_tab[6]
#define __pyx_int_500 __pyx_number_tab[7]
#define __pyx_int_1000 __pyx_number_tab[8]
#define __pyx_int_1000000 __pyx_number_tab[9]
#define __pyx_int_136983863 __pyx_number_tab[10]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7pyogrio_4_err_ErrorHandler);
  Py_CLEAR(clear_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata);
  Py_CLEAR(clear_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata);
  Py_CLEAR(clear_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr);
  Py_CLEAR(clear_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr);
  Py_CLEAR(clear_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features);
  Py_CLEAR(clear_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features);
  Py_CLEAR(clear_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow);
  Py_CLEAR(clear_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow);
  Py_CLEAR(clear_module_state->__pyx_array_type);
  Py_CLEAR(clear_module_state->__pyx_type___pyx_array);
  Py_CLEAR(clear_module_state->__pyx_MemviewEnum_type);
  Py_CLEAR(clear_module_state->__pyx_type___pyx_MemviewEnum);
  Py_CLEAR(clear_module_state->__pyx_memoryview_type);
  Py_CLEAR(clear_module_state->__pyx_type___pyx_memoryview);
  Py_CLEAR(clear_module_state->__pyx_memoryviewslice_type);
  Py_CLEAR(clear_module_state->__pyx_type___pyx_memoryviewslice);
  for (int i=0; i<2; ++i) { Py_CLEAR(clear_module_state->__pyx_slice[i]); }
  for (int i=0; i<10; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<11; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<536; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<11; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* CommonTypesMetaclass.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CyFunctionType);

/* Generator.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_GeneratorType);

/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_7pyogrio_4_err_ErrorHandler);
  Py_VISIT(traverse_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata);
  Py_VISIT(traverse_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata);
  Py_VISIT(traverse_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr);
  Py_VISIT(traverse_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr);
  Py_VISIT(traverse_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features);
  Py_VISIT(traverse_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features);
  Py_VISIT(traverse_module_state->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow);
  Py_VISIT(traverse_module_state->__pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow);
  Py_VISIT(traverse_module_state->__pyx_array_type);
  Py_VISIT(traverse_module_state->__pyx_type___pyx_array);
  Py_VISIT(traverse_module_state->__pyx_MemviewEnum_type);
  Py_VISIT(traverse_module_state->__pyx_type___pyx_MemviewEnum);
  Py_VISIT(traverse_module_state->__pyx_memoryview_type);
  Py_VISIT(traverse_module_state->__pyx_type___pyx_memoryview);
  Py_VISIT(traverse_module_state->__pyx_memoryviewslice_type);
  Py_VISIT(traverse_module_state->__pyx_type___pyx_memoryviewslice);
  for (int i=0; i<2; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_slice[i]); }
  for (int i=0; i<10; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<11; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<536; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<11; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* CommonTypesMetaclass.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CyFunctionType);

/* Generator.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_GeneratorType);

/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */

/* "View.MemoryView":137
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
*/

/* Python wrapper */
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shape = 0;
  Py_ssize_t __pyx_v_itemsize;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_mode = 0;
  int __pyx_v_allocate_buffer;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_shape,&__pyx_mstate_global->__pyx_n_u_itemsize,&__pyx_mstate_global->__pyx_n_u_format,&__pyx_mstate_global->__pyx_n_u_mode,&__pyx_mstate_global->__pyx_n_u_allocate_buffer,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 137, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(1, 137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(1, 137, __pyx_L3_error)
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_c));
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, i); __PYX_ERR(1, 137, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(1, 137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 137, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 137, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 137, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 137, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_c));
    }
    __pyx_v_shape = ((PyObject*)values[0]);
    __pyx_v_itemsize = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_itemsize == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 137, __pyx_L3_error)
    __pyx_v_format = values[2];
    __pyx_v_mode = values[3];
    if (values[4]) {
      __pyx_v_allocate_buffer = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_allocate_buffer == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 138, __pyx_L3_error)
    } else {

      /* "View.MemoryView":138
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,
 *                   mode="c", bint allocate_buffer=True):             # <<<<<<<<<<<<<<
 * 
 *         cdef int idx
*/
      __pyx_v_allocate_buffer = ((int)1);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, __pyx_nargs); __PYX_ERR(1, 137, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shape), (&PyTuple_Type), 1, "shape", 1))) __PYX_ERR(1, 137, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_format) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "format"); __PYX_ERR(1, 137, __pyx_L1_error)
  }
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(((struct __pyx_array_obj *)__pyx_v_self), __pyx_v_shape, __pyx_v_itemsize, __pyx_v_format, __pyx_v_mode, __pyx_v_allocate_buffer);

  /* "View.MemoryView":137
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_v_dim;
  char __pyx_v_order;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  char *__pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11[5];
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __Pyx_INCREF(__pyx_v_format);

  /* "View.MemoryView":143
 *         cdef Py_ssize_t dim
 * 
 *         self.ndim = <int> len(shape)             # <<<<<<<<<<<<<<
 *         self.itemsize = itemsize
 * 
*/
  if (unlikely(__pyx_v_shape == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 143, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyTuple_GET_SIZE(__pyx_v_shape); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(1, 143, __pyx_L1_error)
  __pyx_v_self->ndim = ((int)__pyx_t_1);

  /* "View.MemoryView":144
 * 
 *         self.ndim = <int> len(shape)
 *         self.itemsize = itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not self.ndim:
*/
  __pyx_v_self->itemsize = __pyx_v_itemsize;

  /* "View.MemoryView":146
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError, "Empty shape tuple for cython.array"
 * 
*/
  __pyx_t_2 = (!(__pyx_v_self->ndim != 0));
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":147
 * 
 *         if not self.ndim:
 *             raise ValueError, "Empty shape tuple for cython.array"             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_mstate_global->__pyx_kp_u_Empty_shape_tuple_for_cython_arr, 0, 0);
    __PYX_ERR(1, 147, __pyx_L1_error)

    /* "View.MemoryView":146
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError, "Empty shape tuple for cython.array"
 * 
*/
  }

  /* "View.MemoryView":149
 *             raise ValueError, "Empty shape tuple for cython.array"
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError, "itemsize <= 0 for cython.array"
 * 
*/
  __pyx_t_2 = (__pyx_v_itemsize <= 0);
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":150
 * 
 *         if itemsize <= 0:
 *             raise ValueError, "itemsize <= 0 for cython.array"             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_mstate_global->__pyx_kp_u_itemsize_0_for_cython_array, 0, 0);
    __PYX_ERR(1, 150, __pyx_L1_error)

    /* "View.MemoryView":149
 *             raise ValueError, "Empty shape tuple for cython.array"
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError, "itemsize <= 0 for cython.array"
 * 
*/
  }

  /* "View.MemoryView":152
 *             raise ValueError, "itemsize <= 0 for cython.array"
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
*/
  __pyx_t_2 = PyBytes_Check(__pyx_v_format); 
  __pyx_t_3 = (!__pyx_t_2);
  if (__pyx_t_3) {

    /* "View.MemoryView":153
 * 
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')             # <<<<<<<<<<<<<<
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format
*/
    __pyx_t_5 = __pyx_v_format;
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_n_u_ASCII};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF_SET(__pyx_v_format, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "View.MemoryView":152
 *             raise ValueError, "itemsize <= 0 for cython.array"
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
*/
  }

  /* "View.MemoryView":154
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string             # <<<<<<<<<<<<<<
 *         self.format = self._format
 * 
*/
  __pyx_t_4 = __pyx_v_format;
  __Pyx_INCREF(__pyx_t_4);
  if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_4))) __PYX_ERR(1, 154, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->_format);
  __Pyx_DECREF(__pyx_v_self->_format);
  __pyx_v_self->_format = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "View.MemoryView":155
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format             # <<<<<<<<<<<<<<
 * 
 * 
*/
  if (unlikely(__pyx_v_self->_format == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(1, 155, __pyx_L1_error)
  }
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_self->_format); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(1, 155, __pyx_L1_error)
  __pyx_v_self->format = __pyx_t_7;

  /* "View.MemoryView":158
 * 
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)             # <<<<<<<<<<<<<<
 *         self._strides = self._shape + self.ndim
 * 
*/
  __pyx_v_self->_shape = ((Py_ssize_t *)PyObject_Malloc((((sizeof(Py_ssize_t)) * __pyx_v_self->ndim) * 2)));

  /* "View.MemoryView":159
 * 
 *         self._shape = <Py_ssize_t *> PyObject_Malloc(sizeof(Py_ssize_t)*self.ndim*2)
 *         self._strides = self._shape + self.ndim             # <<<<<<<<<<<<<<
 * 
 *         if not self._shape:
*/
  __pyx_v_self->_strides = (__pyx_v_self->_shape + __pyx_v_self->ndim);

  /* "View.MemoryView":161
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError, "unable to allocate shape and strides."
 * 
*/
  __pyx_t_3 = (!(__pyx_v_self->_shape != 0));
  if (unlikely(__pyx_t_3)) {

    /* "View.MemoryView":162
 * 
 *         if not self._shape:
 *             raise MemoryError, "unable to allocate shape and strides."             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_MemoryError))), __pyx_mstate_global->__pyx_kp_u_unable_to_allocate_shape_and_str, 0, 0);
    __PYX_ERR(1, 162, __pyx_L1_error)

    /* "View.MemoryView":161
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError, "unable to allocate shape and strides."
 * 
*/
  }

  /* "View.MemoryView":165
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError, f"Invalid shape in axis {idx}: {dim}."
*/
  __pyx_t_8 = 0;
  __pyx_t_4 = __pyx_v_shape; __Pyx_INCREF(__pyx_t_4);
  __pyx_t_1 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 165, __pyx_L1_error)
      #endif
      if (__pyx_t_1 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_5 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_1));
    #else
    __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_1);
    #endif
    ++__pyx_t_1;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 165, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_dim = __pyx_t_9;
    __pyx_v_idx = __pyx_t_8;
    __pyx_t_8 = (__pyx_t_8 + 1);

    /* "View.MemoryView":166
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError, f"Invalid shape in axis {idx}: {dim}."
 *             self._shape[idx] = dim
*/
    __pyx_t_3 = (__pyx_v_dim <= 0);
    if (unlikely(__pyx_t_3)) {

      /* "View.MemoryView":167
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:
 *                 raise ValueError, f"Invalid shape in axis {idx}: {dim}."             # <<<<<<<<<<<<<<
 *             self._shape[idx] = dim
 * 
*/
      __pyx_t_5 = __Pyx_PyUnicode_From_int(__pyx_v_idx, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_dim, 0, ' ', 'd'); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11[0] = __pyx_mstate_global->__pyx_kp_u_Invalid_shape_in_axis;
      __pyx_t_11[1] = __pyx_t_5;
      __pyx_t_11[2] = __pyx_mstate_global->__pyx_kp_u_;
      __pyx_t_11[3] = __pyx_t_10;
      __pyx_t_11[4] = __pyx_mstate_global->__pyx_kp_u__2;
      __pyx_t_12 = __Pyx_PyUnicode_Join(__pyx_t_11, 5, 22 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5) + 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10) + 1, 127);
      if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 167, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_t_12, 0, 0);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __PYX_ERR(1, 167, __pyx_L1_error)

      /* "View.MemoryView":166
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError, f"Invalid shape in axis {idx}: {dim}."
 *             self._shape[idx] = dim
*/
    }

    /* "View.MemoryView":168
 *             if dim <= 0:
 *                 raise ValueError, f"Invalid shape in axis {idx}: {dim}."
 *             self._shape[idx] = dim             # <<<<<<<<<<<<<<
 * 
 *         cdef char order
*/
    (__pyx_v_self->_shape[__pyx_v_idx]) = __pyx_v_dim;

    /* "View.MemoryView":165
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError, f"Invalid shape in axis {idx}: {dim}."
*/
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "View.MemoryView":171
 * 
 *         cdef char order
 *         if mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_mstate_global->__pyx_n_u_c, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 171, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "View.MemoryView":172
 *         cdef char order
 *         if mode == 'c':
 *             order = b'C'             # <<<<<<<<<<<<<<
 *             self.mode = u'c'
 *         elif mode == 'fortran':
*/
    __pyx_v_order = 'C';

    /* "View.MemoryView":173
 *         if mode == 'c':
 *             order = b'C'
 *             self.mode = u'c'             # <<<<<<<<<<<<<<
 *         elif mode == 'fortran':
 *             order = b'F'
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_c);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_mstate_global->__pyx_n_u_c;

    /* "View.MemoryView":171
 * 
 *         cdef char order
 *         if mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
*/
    goto __pyx_L11;
  }

  /* "View.MemoryView":174
 *             order = b'C'
 *             self.mode = u'c'
 *         elif mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_mode, __pyx_mstate_global->__pyx_n_u_fortran, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 174, __pyx_L1_error)
  if (likely(__pyx_t_3)) {

    /* "View.MemoryView":175
 *             self.mode = u'c'
 *         elif mode == 'fortran':
 *             order = b'F'             # <<<<<<<<<<<<<<
 *             self.mode = u'fortran'
 *         else:
*/
    __pyx_v_order = 'F';

    /* "View.MemoryView":176
 *         elif mode == 'fortran':
 *             order = b'F'
 *             self.mode = u'fortran'             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError, f"Invalid mode, expected 'c' or 'fortran', got {mode}"
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_fortran);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_fortran);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_mstate_global->__pyx_n_u_fortran;

    /* "View.MemoryView":174
 *             order = b'C'
 *             self.mode = u'c'
 *         elif mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
*/
    goto __pyx_L11;
  }

  /* "View.MemoryView":178
 *             self.mode = u'fortran'
 *         else:
 *             raise ValueError, f"Invalid mode, expected 'c' or 'fortran', got {mode}"             # <<<<<<<<<<<<<<
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides, itemsize, self.ndim, order)
*/
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_mode, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_12 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Invalid_mode_expected_c_or_fortr, __pyx_t_4); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_t_12, 0, 0);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __PYX_ERR(1, 178, __pyx_L1_error)
  }
  __pyx_L11:;

  /* "View.MemoryView":180
 *             raise ValueError, f"Invalid mode, expected 'c' or 'fortran', got {mode}"
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides, itemsize, self.ndim, order)             # <<<<<<<<<<<<<<
 * 
 *         self.free_data = allocate_buffer
*/
  __pyx_v_self->len = __pyx_fill_contig_strides_array(__pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_itemsize, __pyx_v_self->ndim, __pyx_v_order);

  /* "View.MemoryView":182
 *         self.len = fill_contig_strides_array(self._shape, self._strides, itemsize, self.ndim, order)
 * 
 *         self.free_data = allocate_buffer             # <<<<<<<<<<<<<<
 *         self.dtype_is_object = format == b'O'
 * 
*/
  __pyx_v_self->free_data = __pyx_v_allocate_buffer;

  /* "View.MemoryView":183
 * 
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'             # <<<<<<<<<<<<<<
 * 
 *         if allocate_buffer:
*/
  __pyx_t_12 = PyObject_RichCompare(__pyx_v_format, __pyx_mstate_global->__pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_12); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 183, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 183, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_self->dtype_is_object = __pyx_t_3;

  /* "View.MemoryView":185
 *         self.dtype_is_object = format == b'O'
 * 
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 *             _allocate_buffer(self)
 * 
*/
  if (__pyx_v_allocate_buffer) {

    /* "View.MemoryView":186
 * 
 *         if allocate_buffer:
 *             _allocate_buffer(self)             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
*/
    __pyx_t_8 = __pyx_array_allocate_buffer(__pyx_v_self); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(1, 186, __pyx_L1_error)

    /* "View.MemoryView":185
 *         self.dtype_is_object = format == b'O'
 * 
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 *             _allocate_buffer(self)
 * 
*/
  }

  /* "View.MemoryView":137
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_format);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":188
 *             _allocate_buffer(self)
 * 
 *     @cname('getbuffer')             # <<<<<<<<<<<<<<
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
*/

/* Python wrapper */
CYTHON_UNUSED static int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
CYTHON_UNUSED static int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(((struct __pyx_array_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_bufmode;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  char *__pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  if (unlikely(__pyx_v_info == NULL)) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":190
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1             # <<<<<<<<<<<<<<
 *         if flags & (PyBUF_C_CONTIGUOUS | PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS):
 *             if self.mode == u"c":
*/
  __pyx_v_bufmode = -1;

  /* "View.MemoryView":191
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if flags & (PyBUF_C_CONTIGUOUS | PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *             if self.mode == u"c":
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
*/
  __pyx_t_1 = ((__pyx_v_flags & ((PyBUF_C_CONTIGUOUS | PyBUF_F_CONTIGUOUS) | PyBUF_ANY_CONTIGUOUS)) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":192
 *         cdef int bufmode = -1
 *         if flags & (PyBUF_C_CONTIGUOUS | PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS):
 *             if self.mode == u"c":             # <<<<<<<<<<<<<<
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             elif self.mode == u"fortran":
*/
    __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_mstate_global->__pyx_n_u_c, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(1, 192, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "View.MemoryView":193
 *         if flags & (PyBUF_C_CONTIGUOUS | PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS):
 *             if self.mode == u"c":
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *             elif self.mode == u"fortran":
 *                 bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
*/
      __pyx_v_bufmode = (PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

      /* "View.MemoryView":192
 *         cdef int bufmode = -1
 *         if flags & (PyBUF_C_CONTIGUOUS | PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS):
 *             if self.mode == u"c":             # <<<<<<<<<<<<<<
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             elif self.mode == u"fortran":
*/
      goto __pyx_L4;
    }

    /* "View.MemoryView":194
 *             if self.mode == u"c":
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *                 bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             if not (flags & bufmode):
*/
    __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_mstate_global->__pyx_n_u_fortran, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(1, 194, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "View.MemoryView":195
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             elif self.mode == u"fortran":
 *                 bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *             if not (flags & bufmode):
 *                 raise ValueError, "Can only create a buffer that is contiguous in memory."
*/
      __pyx_v_bufmode = (PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

      /* "View.MemoryView":194
 *             if self.mode == u"c":
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *                 bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             if not (flags & bufmode):
*/
    }
    __pyx_L4:;

    /* "View.MemoryView":196
 *             elif self.mode == u"fortran":
 *                 bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *                 raise ValueError, "Can only create a buffer that is contiguous in memory."
 *         info.buf = self.data
*/
    __pyx_t_1 = (!((__pyx_v_flags & __pyx_v_bufmode) != 0));
    if (unlikely(__pyx_t_1)) {

      /* "View.MemoryView":197
 *                 bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             if not (flags & bufmode):
 *                 raise ValueError, "Can only create a buffer that is contiguous in memory."             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_mstate_global->__pyx_kp_u_Can_only_create_a_buffer_that_is, 0, 0);
      __PYX_ERR(1, 197, __pyx_L1_error)

      /* "View.MemoryView":196
 *             elif self.mode == u"fortran":
 *                 bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *             if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *                 raise ValueError, "Can only create a buffer that is contiguous in memory."
 *         info.buf = self.data
*/
    }

    /* "View.MemoryView":191
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if flags & (PyBUF_C_CONTIGUOUS | PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS):             # <<<<<<<<<<<<<<
 *             if self.mode == u"c":
 *                 bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
*/
  }

  /* "View.MemoryView":198
 *             if not (flags & bufmode):
 *                 raise ValueError, "Can only create a buffer that is contiguous in memory."
 *         info.buf = self.data             # <<<<<<<<<<<<<<
 *         info.len = self.len
 * 
*/
  __pyx_t_2 = __pyx_v_self->data;
  __pyx_v_info->buf = __pyx_t_2;

  /* "View.MemoryView":199
 *                 raise ValueError, "Can only create a buffer that is contiguous in memory."
 *         info.buf = self.data
 *         info.len = self.len             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_STRIDES:
*/
  __pyx_t_3 = __pyx_v_self->len;
  __pyx_v_info->len = __pyx_t_3;

  /* "View.MemoryView":201
 *         info.len = self.len
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.ndim = self.ndim
 *             info.shape = self._shape
*/
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":202
 * 
 *         if flags & PyBUF_STRIDES:
 *             info.ndim = self.ndim             # <<<<<<<<<<<<<<
 *             info.shape = self._shape
 *             info.strides = self._strides
*/
    __pyx_t_4 = __pyx_v_self->ndim;
    __pyx_v_info->ndim = __pyx_t_4;

    /* "View.MemoryView":203
 *         if flags & PyBUF_STRIDES:
 *             info.ndim = self.ndim
 *             info.shape = self._shape             # <<<<<<<<<<<<<<
 *             info.strides = self._strides
 *         else:
*/
    __pyx_t_5 = __pyx_v_self->_shape;
    __pyx_v_info->shape = __pyx_t_5;

    /* "View.MemoryView":204
 *             info.ndim = self.ndim
 *             info.shape = self._shape
 *             info.strides = self._strides             # <<<<<<<<<<<<<<
 *         else:
 *             info.ndim = 1
*/
    __pyx_t_5 = __pyx_v_self->_strides;
    __pyx_v_info->strides = __pyx_t_5;

    /* "View.MemoryView":201
 *         info.len = self.len
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.ndim = self.ndim
 *             info.shape = self._shape
*/
    goto __pyx_L6;
  }

  /* "View.MemoryView":206
 *             info.strides = self._strides
 *         else:
 *             info.ndim = 1             # <<<<<<<<<<<<<<
 *             info.shape = &self.len if flags & PyBUF_ND else NULL
 *             info.strides = NULL
*/
  /*else*/ {
    __pyx_v_info->ndim = 1;

    /* "View.MemoryView":207
 *         else:
 *             info.ndim = 1
 *             info.shape = &self.len if flags & PyBUF_ND else NULL             # <<<<<<<<<<<<<<
 *             info.strides = NULL
 * 
*/
    __pyx_t_1 = ((__pyx_v_flags & PyBUF_ND) != 0);
    if (__pyx_t_1) {
      __pyx_t_5 = (&__pyx_v_self->len);
    } else {
      __pyx_t_5 = NULL;
    }
    __pyx_v_info->shape = __pyx_t_5;

    /* "View.MemoryView":208
 *             info.ndim = 1
 *             info.shape = &self.len if flags & PyBUF_ND else NULL
 *             info.strides = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.suboffsets = NULL
*/
    __pyx_v_info->strides = NULL;
  }
  __pyx_L6:;

  /* "View.MemoryView":210
 *             info.strides = NULL
 * 
 *         info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *         info.itemsize = self.itemsize
 *         info.readonly = 0
*/
  __pyx_v_info->suboffsets = NULL;

  /* "View.MemoryView":211
 * 
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 *         info.format = self.format if flags & PyBUF_FORMAT else NULL
*/
  __pyx_t_3 = __pyx_v_self->itemsize;
  __pyx_v_info->itemsize = __pyx_t_3;

  /* "View.MemoryView":212
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 *         info.format = self.format if flags & PyBUF_FORMAT else NULL
 *         info.obj = self
*/
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":213
 *         info.itemsize = self.itemsize
 *         info.readonly = 0
 *         info.format = self.format if flags & PyBUF_FORMAT else NULL             # <<<<<<<<<<<<<<
 *         info.obj = self
 * 
*/
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {
    __pyx_t_2 = __pyx_v_self->format;
  } else {
    __pyx_t_2 = NULL;
  }
  __pyx_v_info->format = __pyx_t_2;

  /* "View.MemoryView":214
 *         info.readonly = 0
 *         info.format = self.format if flags & PyBUF_FORMAT else NULL
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(array self):
*/
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self);
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":188
 *             _allocate_buffer(self)
 * 
 *     @cname('getbuffer')             # <<<<<<<<<<<<<<
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":216
 *         info.obj = self
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
*/

/* Python wrapper */
static void __pyx_array___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_array___dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self) {
  int __pyx_t_1;
  int __pyx_t_2;

  /* "View.MemoryView":217
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data and self.data is not NULL:
*/
  __pyx_t_1 = (__pyx_v_self->callback_free_data != NULL);
  if (__pyx_t_1) {

    /* "View.MemoryView":218
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)             # <<<<<<<<<<<<<<
 *         elif self.free_data and self.data is not NULL:
 *             if self.dtype_is_object:
*/
    __pyx_v_self->callback_free_data(__pyx_v_self->data);

    /* "View.MemoryView":217
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data and self.data is not NULL:
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":219
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data and self.data is not NULL:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape, self._strides, self.ndim, inc=False)
*/
  if (__pyx_v_self->free_data) {
  } else {
    __pyx_t_1 = __pyx_v_self->free_data;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->data != NULL);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "View.MemoryView":220
 *             self.callback_free_data(self.data)
 *         elif self.free_data and self.data is not NULL:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape, self._strides, self.ndim, inc=False)
 *             free(self.data)
*/
    if (__pyx_v_self->dtype_is_object) {

      /* "View.MemoryView":221
 *         elif self.free_data and self.data is not NULL:
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape, self._strides, self.ndim, inc=False)             # <<<<<<<<<<<<<<
 *             free(self.data)
 *         PyObject_Free(self._shape)
*/
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_self->data, __pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_self->ndim, 0);

      /* "View.MemoryView":220
 *             self.callback_free_data(self.data)
 *         elif self.free_data and self.data is not NULL:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape, self._strides, self.ndim, inc=False)
 *             free(self.data)
*/
    }

    /* "View.MemoryView":222
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape, self._strides, self.ndim, inc=False)
 *             free(self.data)             # <<<<<<<<<<<<<<
 *         PyObject_Free(self._shape)
 * 
*/
    free(__pyx_v_self->data);

    /* "View.MemoryView":219
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data and self.data is not NULL:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape, self._strides, self.ndim, inc=False)
*/
  }
  __pyx_L3:;

  /* "View.MemoryView":223
 *                 refcount_objects_in_slice(self.data, self._shape, self._strides, self.ndim, inc=False)
 *             free(self.data)
 *         PyObject_Free(self._shape)             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  PyObject_Free(__pyx_v_self->_shape);

  /* "View.MemoryView":216
 *         info.obj = self
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
*/

  /* function exit code */
}

/* "View.MemoryView":225
 *         PyObject_Free(self._shape)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def memview(self):
 *         return self.get_memview()
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_5array_7memview___get__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":227
 *     @property
 *     def memview(self):
 *         return self.get_memview()             # <<<<<<<<<<<<<<
 * 
 *     @cname('get_memview')
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_array *)__pyx_v_self->__pyx_vtab)->get_memview(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":225
 *         PyObject_Free(self._shape)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def memview(self):
 *         return self.get_memview()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.memview.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":229
 *         return self.get_memview()
 * 
 *     @cname('get_memview')             # <<<<<<<<<<<<<<
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
*/

static PyObject *__pyx_array_get_memview(struct __pyx_array_obj *__pyx_v_self) {
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_memview", 0);

  /* "View.MemoryView":231
 *     @cname('get_memview')
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE             # <<<<<<<<<<<<<<
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
*/
  __pyx_v_flags = ((PyBUF_ANY_CONTIGUOUS | PyBUF_FORMAT) | PyBUF_WRITABLE);

  /* "View.MemoryView":232
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *         return  memoryview(self, flags, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_2, ((PyObject *)__pyx_v_self), __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_memoryview_type, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 232, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":229
 *         return self.get_memview()
 * 
 *     @cname('get_memview')             # <<<<<<<<<<<<<<
 *     cdef get_memview(self):
 *         flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.array.get_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":234
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
*/

/* Python wrapper */
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_array___len__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__len__(struct __pyx_array_obj *__pyx_v_self) {
  Py_ssize_t __pyx_r;

  /* "View.MemoryView":235
 * 
 *     def __len__(self):
 *         return self._shape[0]             # <<<<<<<<<<<<<<
 * 
 *     def __getattr__(self, attr):
*/
  __pyx_r = (__pyx_v_self->_shape[0]);
  goto __pyx_L0;

  /* "View.MemoryView":234
 *         return  memoryview(self, flags, self.dtype_is_object)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return self._shape[0]
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":237
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr); /*proto*/
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getattr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_attr));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getattr__", 0);

  /* "View.MemoryView":238
 * 
 *     def __getattr__(self, attr):
 *         return getattr(self.memview, attr)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetAttr(__pyx_t_1, __pyx_v_attr); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":237
 *         return self._shape[0]
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getattr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":240
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
*/

/* Python wrapper */
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":241
 * 
 *     def __getitem__(self, item):
 *         return self.memview[item]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, item, value):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_item); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":240
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":243
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
*/

/* Python wrapper */
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /*proto*/
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_12__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "View.MemoryView":244
 * 
 *     def __setitem__(self, item, value):
 *         self.memview[item] = value             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_memview); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely((PyObject_SetItem(__pyx_t_1, __pyx_v_item, __pyx_v_value) < 0))) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":243
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_array_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf___pyx_array___reduce_cython__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_array___reduce_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.array.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_array_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView.array.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf___pyx_array_2__setstate_cython__(((struct __pyx_array_obj *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_array_2__setstate_cython__(CYTHON_UNUSED struct __pyx_array_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.array.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":253
 *         pass
 * 
 * @cname("__pyx_array_allocate_buffer")             # <<<<<<<<<<<<<<
 * cdef int _allocate_buffer(array self) except -1:
 * 
*/

static int __pyx_array_allocate_buffer(struct __pyx_array_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_i;
  PyObject **__pyx_v_p;
  int __pyx_r;
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":260
 *     cdef PyObject **p
 * 
 *     self.free_data = True             # <<<<<<<<<<<<<<
 *     self.data = <char *>malloc(self.len)
 *     if not self.data:
*/
  __pyx_v_self->free_data = 1;

  /* "View.MemoryView":261
 * 
 *     self.free_data = True
 *     self.data = <char *>malloc(self.len)             # <<<<<<<<<<<<<<
 *     if not self.data:
 *         raise MemoryError, "unable to allocate array data."
*/
  __pyx_v_self->data = ((char *)malloc(__pyx_v_self->len));

  /* "View.MemoryView":262
 *     self.free_data = True
 *     self.data = <char *>malloc(self.len)
 *     if not self.data:             # <<<<<<<<<<<<<<
 *         raise MemoryError, "unable to allocate array data."
 * 
*/
  __pyx_t_1 = (!(__pyx_v_self->data != 0));
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":263
 *     self.data = <char *>malloc(self.len)
 *     if not self.data:
 *         raise MemoryError, "unable to allocate array data."             # <<<<<<<<<<<<<<
 * 
 *     if self.dtype_is_object:
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_MemoryError))), __pyx_mstate_global->__pyx_kp_u_unable_to_allocate_array_data, 0, 0);
    __PYX_ERR(1, 263, __pyx_L1_error)

    /* "View.MemoryView":262
 *     self.free_data = True
 *     self.data = <char *>malloc(self.len)
 *     if not self.data:             # <<<<<<<<<<<<<<
 *         raise MemoryError, "unable to allocate array data."
 * 
*/
  }

  /* "View.MemoryView":265
 *         raise MemoryError, "unable to allocate array data."
 * 
 *     if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *         p = <PyObject **> self.data
 *         for i in range(self.len // self.itemsize):
*/
  if (__pyx_v_self->dtype_is_object) {

    /* "View.MemoryView":266
 * 
 *     if self.dtype_is_object:
 *         p = <PyObject **> self.data             # <<<<<<<<<<<<<<
 *         for i in range(self.len // self.itemsize):
 *             p[i] = Py_None
*/
    __pyx_v_p = ((PyObject **)__pyx_v_self->data);

    /* "View.MemoryView":267
 *     if self.dtype_is_object:
 *         p = <PyObject **> self.data
 *         for i in range(self.len // self.itemsize):             # <<<<<<<<<<<<<<
 *             p[i] = Py_None
 *             Py_INCREF(Py_None)
*/
    if (unlikely(__pyx_v_self->itemsize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(1, 267, __pyx_L1_error)
    }
    else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_self->itemsize == (Py_ssize_t)-1)  && unlikely(__Pyx_UNARY_NEG_WOULD_OVERFLOW(__pyx_v_self->len))) {
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      __PYX_ERR(1, 267, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_div_Py_ssize_t(__pyx_v_self->len, __pyx_v_self->itemsize, 0);
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":268
 *         p = <PyObject **> self.data
 *         for i in range(self.len // self.itemsize):
 *             p[i] = Py_None             # <<<<<<<<<<<<<<
 *             Py_INCREF(Py_None)
 *     return 0
*/
      (__pyx_v_p[__pyx_v_i]) = Py_None;

      /* "View.MemoryView":269
 *         for i in range(self.len // self.itemsize):
 *             p[i] = Py_None
 *             Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 *     return 0
 * 
*/
      Py_INCREF(Py_None);
    }

    /* "View.MemoryView":265
 *         raise MemoryError, "unable to allocate array data."
 * 
 *     if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *         p = <PyObject **> self.data
 *         for i in range(self.len // self.itemsize):
*/
  }

  /* "View.MemoryView":270
 *             p[i] = Py_None
 *             Py_INCREF(Py_None)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":253
 *         pass
 * 
 * @cname("__pyx_array_allocate_buffer")             # <<<<<<<<<<<<<<
 * cdef int _allocate_buffer(array self) except -1:
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView._allocate_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":273
 * 
 * 
 * @cname("__pyx_array_new")             # <<<<<<<<<<<<<<
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format, const char *c_mode, char *buf):
 *     cdef array result
*/

static struct __pyx_array_obj *__pyx_array_new(PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, char *__pyx_v_format, char const *__pyx_v_c_mode, char *__pyx_v_buf) {
  struct __pyx_array_obj *__pyx_v_result = 0;
  PyObject *__pyx_v_mode = 0;
  struct __pyx_array_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("array_cwrapper", 0);

  /* "View.MemoryView":276
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format, const char *c_mode, char *buf):
 *     cdef array result
 *     cdef str mode = "fortran" if c_mode[0] == b'f' else "c"  # this often comes from a constant C string.             # <<<<<<<<<<<<<<
 * 
 *     if buf is NULL:
*/
  __pyx_t_2 = ((__pyx_v_c_mode[0]) == 'f');
  if (__pyx_t_2) {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_fortran);
    __pyx_t_1 = __pyx_mstate_global->__pyx_n_u_fortran;
  } else {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_c);
    __pyx_t_1 = __pyx_mstate_global->__pyx_n_u_c;
  }
  __pyx_v_mode = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":278
 *     cdef str mode = "fortran" if c_mode[0] == b'f' else "c"  # this often comes from a constant C string.
 * 
 *     if buf is NULL:             # <<<<<<<<<<<<<<
 *         result = array.__new__(array, shape, itemsize, format, mode)
 *     else:
*/
  __pyx_t_2 = (__pyx_v_buf == NULL);
  if (__pyx_t_2) {

    /* "View.MemoryView":279
 * 
 *     if buf is NULL:
 *         result = array.__new__(array, shape, itemsize, format, mode)             # <<<<<<<<<<<<<<
 *     else:
 *         result = array.__new__(array, shape, itemsize, format, mode, allocate_buffer=False)
*/
    __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_shape) != (0)) __PYX_ERR(1, 279, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 279, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3) != (0)) __PYX_ERR(1, 279, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_mode);
    __Pyx_GIVEREF(__pyx_v_mode);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_v_mode) != (0)) __PYX_ERR(1, 279, __pyx_L1_error);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)__pyx_tp_new_array(((PyTypeObject *)__pyx_mstate_global->__pyx_array_type), __pyx_t_4, NULL)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 279, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":278
 *     cdef str mode = "fortran" if c_mode[0] == b'f' else "c"  # this often comes from a constant C string.
 * 
 *     if buf is NULL:             # <<<<<<<<<<<<<<
 *         result = array.__new__(array, shape, itemsize, format, mode)
 *     else:
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":281
 *         result = array.__new__(array, shape, itemsize, format, mode)
 *     else:
 *         result = array.__new__(array, shape, itemsize, format, mode, allocate_buffer=False)             # <<<<<<<<<<<<<<
 *         result.data = buf
 * 
*/
  /*else*/ {
    __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_shape) != (0)) __PYX_ERR(1, 281, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3) != (0)) __PYX_ERR(1, 281, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_t_4) != (0)) __PYX_ERR(1, 281, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_mode);
    __Pyx_GIVEREF(__pyx_v_mode);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_mode) != (0)) __PYX_ERR(1, 281, __pyx_L1_error);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_allocate_buffer, Py_False) < (0)) __PYX_ERR(1, 281, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)__pyx_tp_new_array(((PyTypeObject *)__pyx_mstate_global->__pyx_array_type), __pyx_t_1, __pyx_t_4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 281, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":282
 *     else:
 *         result = array.__new__(array, shape, itemsize, format, mode, allocate_buffer=False)
 *         result.data = buf             # <<<<<<<<<<<<<<
 * 
 *     return result
*/
    __pyx_v_result->data = __pyx_v_buf;
  }
  __pyx_L3:;

  /* "View.MemoryView":284
 *         result.data = buf
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":273
 * 
 * 
 * @cname("__pyx_array_new")             # <<<<<<<<<<<<<<
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format, const char *c_mode, char *buf):
 *     cdef array result
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.array_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_mode);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":310
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
*/

/* Python wrapper */
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 310, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 310, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(1, 310, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(1, 310, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 310, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 310, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView.Enum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "View.MemoryView":311
 *     cdef object name
 *     def __init__(self, name):
 *         self.name = name             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return self.name
*/
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "View.MemoryView":310
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":312
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
*/

/* Python wrapper */
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":313
 *         self.name = name
 *     def __repr__(self):
 *         return self.name             # <<<<<<<<<<<<<<
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* "View.MemoryView":312
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_MemviewEnum_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf___pyx_MemviewEnum___reduce_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_MemviewEnum___reduce_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.name,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->name) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.name is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.name,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, None), state
*/
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->name != Py_None);
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_136983863);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_136983863);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_136983863) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, None), state
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Enum); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_136983863);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_136983863);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_136983863) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.Enum.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_MemviewEnum_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView.Enum.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf___pyx_MemviewEnum_2__setstate_cython__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_MemviewEnum_2__setstate_cython__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(1, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(1, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_unpickle_Enum__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_Enum, (type(self), 0x82a3537, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Enum__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.Enum.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":356
 *     cdef const __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
*/

/* Python wrapper */
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_obj = 0;
  int __pyx_v_flags;
  int __pyx_v_dtype_is_object;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_obj,&__pyx_mstate_global->__pyx_n_u_flags,&__pyx_mstate_global->__pyx_n_u_dtype_is_object,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 356, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 356, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 356, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 356, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(1, 356, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, i); __PYX_ERR(1, 356, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 356, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 356, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 356, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_obj = values[0];
    __pyx_v_flags = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 356, __pyx_L3_error)
    if (values[2]) {
      __pyx_v_dtype_is_object = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_dtype_is_object == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 356, __pyx_L3_error)
    } else {
      __pyx_v_dtype_is_object = ((int)0);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, __pyx_nargs); __PYX_ERR(1, 356, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_obj, __pyx_v_flags, __pyx_v_dtype_is_object);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "View.MemoryView":357
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
*/
  __Pyx_INCREF(__pyx_v_obj);
  __Pyx_GIVEREF(__pyx_v_obj);
  __Pyx_GOTREF(__pyx_v_self->obj);
  __Pyx_DECREF(__pyx_v_self->obj);
  __pyx_v_self->obj = __pyx_v_obj;

  /* "View.MemoryView":358
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj
 *         self.flags = flags             # <<<<<<<<<<<<<<
 *         if type(self) is memoryview or obj is not None:
 *             PyObject_GetBuffer(obj, &self.view, flags)
*/
  __pyx_v_self->flags = __pyx_v_flags;

  /* "View.MemoryView":359
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             PyObject_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
*/
  __pyx_t_2 = (((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))) == ((PyObject *)__pyx_mstate_global->__pyx_memoryview_type));
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_obj != Py_None);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "View.MemoryView":360
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 *             PyObject_GetBuffer(obj, &self.view, flags)             # <<<<<<<<<<<<<<
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
*/
    __pyx_t_3 = PyObject_GetBuffer(__pyx_v_obj, (&__pyx_v_self->view), __pyx_v_flags); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 360, __pyx_L1_error)

    /* "View.MemoryView":361
 *         if type(self) is memoryview or obj is not None:
 *             PyObject_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
*/
    __pyx_t_1 = (((PyObject *)__pyx_v_self->view.obj) == NULL);
    if (__pyx_t_1) {

      /* "View.MemoryView":362
 *             PyObject_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None             # <<<<<<<<<<<<<<
 *                 Py_INCREF(Py_None)
 * 
*/
      ((Py_buffer *)(&__pyx_v_self->view))->obj = Py_None;

      /* "View.MemoryView":363
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         if not __PYX_CYTHON_ATOMICS_ENABLED():
*/
      Py_INCREF(Py_None);

      /* "View.MemoryView":361
 *         if type(self) is memoryview or obj is not None:
 *             PyObject_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
*/
    }

    /* "View.MemoryView":359
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             PyObject_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
*/
  }

  /* "View.MemoryView":365
 *                 Py_INCREF(Py_None)
 * 
 *         if not __PYX_CYTHON_ATOMICS_ENABLED():             # <<<<<<<<<<<<<<
 *             global __pyx_memoryview_thread_locks_used
 *             if (__pyx_memoryview_thread_locks_used < 8 and
*/
  __pyx_t_1 = (!__PYX_CYTHON_ATOMICS_ENABLED());
  if (__pyx_t_1) {

    /* "View.MemoryView":367
 *         if not __PYX_CYTHON_ATOMICS_ENABLED():
 *             global __pyx_memoryview_thread_locks_used
 *             if (__pyx_memoryview_thread_locks_used < 8 and             # <<<<<<<<<<<<<<
 * 
 *                     not __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING()):
*/
    __pyx_t_2 = (__pyx_memoryview_thread_locks_used < 8);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L9_bool_binop_done;
    }

    /* "View.MemoryView":369
 *             if (__pyx_memoryview_thread_locks_used < 8 and
 * 
 *                     not __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING()):             # <<<<<<<<<<<<<<
 *                 self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *                 __pyx_memoryview_thread_locks_used += 1
*/
    __pyx_t_2 = (!__PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING());
    __pyx_t_1 = __pyx_t_2;
    __pyx_L9_bool_binop_done:;

    /* "View.MemoryView":367
 *         if not __PYX_CYTHON_ATOMICS_ENABLED():
 *             global __pyx_memoryview_thread_locks_used
 *             if (__pyx_memoryview_thread_locks_used < 8 and             # <<<<<<<<<<<<<<
 * 
 *                     not __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING()):
*/
    if (__pyx_t_1) {

      /* "View.MemoryView":370
 * 
 *                     not __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING()):
 *                 self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]             # <<<<<<<<<<<<<<
 *                 __pyx_memoryview_thread_locks_used += 1
 *             if self.lock is NULL:
*/
      __pyx_v_self->lock = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);

      /* "View.MemoryView":371
 *                     not __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING()):
 *                 self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *                 __pyx_memoryview_thread_locks_used += 1             # <<<<<<<<<<<<<<
 *             if self.lock is NULL:
 *                 self.lock = PyThread_allocate_lock()
*/
      __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used + 1);

      /* "View.MemoryView":367
 *         if not __PYX_CYTHON_ATOMICS_ENABLED():
 *             global __pyx_memoryview_thread_locks_used
 *             if (__pyx_memoryview_thread_locks_used < 8 and             # <<<<<<<<<<<<<<
 * 
 *                     not __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING()):
*/
    }

    /* "View.MemoryView":372
 *                 self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *                 __pyx_memoryview_thread_locks_used += 1
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 self.lock = PyThread_allocate_lock()
 *                 if self.lock is NULL:
*/
    __pyx_t_1 = (__pyx_v_self->lock == NULL);
    if (__pyx_t_1) {

      /* "View.MemoryView":373
 *                 __pyx_memoryview_thread_locks_used += 1
 *             if self.lock is NULL:
 *                 self.lock = PyThread_allocate_lock()             # <<<<<<<<<<<<<<
 *                 if self.lock is NULL:
 *                     raise MemoryError
*/
      __pyx_v_self->lock = PyThread_allocate_lock();

      /* "View.MemoryView":374
 *             if self.lock is NULL:
 *                 self.lock = PyThread_allocate_lock()
 *                 if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError
 * 
*/
      __pyx_t_1 = (__pyx_v_self->lock == NULL);
      if (unlikely(__pyx_t_1)) {

        /* "View.MemoryView":375
 *                 self.lock = PyThread_allocate_lock()
 *                 if self.lock is NULL:
 *                     raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
*/
        PyErr_NoMemory(); __PYX_ERR(1, 375, __pyx_L1_error)

        /* "View.MemoryView":374
 *             if self.lock is NULL:
 *                 self.lock = PyThread_allocate_lock()
 *                 if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                     raise MemoryError
 * 
*/
      }

      /* "View.MemoryView":372
 *                 self.lock = __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]
 *                 __pyx_memoryview_thread_locks_used += 1
 *             if self.lock is NULL:             # <<<<<<<<<<<<<<
 *                 self.lock = PyThread_allocate_lock()
 *                 if self.lock is NULL:
*/
    }

    /* "View.MemoryView":365
 *                 Py_INCREF(Py_None)
 * 
 *         if not __PYX_CYTHON_ATOMICS_ENABLED():             # <<<<<<<<<<<<<<
 *             global __pyx_memoryview_thread_locks_used
 *             if (__pyx_memoryview_thread_locks_used < 8 and
*/
  }

  /* "View.MemoryView":377
 *                     raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
*/
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":378
 * 
 *         if flags & PyBUF_FORMAT:
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')             # <<<<<<<<<<<<<<
 *         else:
 *             self.dtype_is_object = dtype_is_object
*/
    __pyx_t_2 = ((__pyx_v_self->view.format[0]) == 'O');
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L14_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_self->view.format[1]) == '\x00');
    __pyx_t_1 = __pyx_t_2;
    __pyx_L14_bool_binop_done:;
    __pyx_v_self->dtype_is_object = __pyx_t_1;

    /* "View.MemoryView":377
 *                     raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
*/
    goto __pyx_L13;
  }

  /* "View.MemoryView":380
 *             self.dtype_is_object = (self.view.format[0] == b'O' and self.view.format[1] == b'\0')
 *         else:
 *             self.dtype_is_object = dtype_is_object             # <<<<<<<<<<<<<<
 * 
 *         with cython.cdivision(True):
*/
  /*else*/ {
    __pyx_v_self->dtype_is_object = __pyx_v_dtype_is_object;
  }
  __pyx_L13:;

  /* "View.MemoryView":383
 * 
 *         with cython.cdivision(True):
 *             assert <Py_intptr_t>(&self.acquisition_count) % sizeof(__pyx_atomic_int_type) == 0             # <<<<<<<<<<<<<<
 *         self.typeinfo = NULL
 * 
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_1 = ((((Py_intptr_t)(&__pyx_v_self->acquisition_count)) % (sizeof(__pyx_atomic_int_type))) == 0);
    if (unlikely(!__pyx_t_1)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(1, 383, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(1, 383, __pyx_L1_error)
  #endif

  /* "View.MemoryView":384
 *         with cython.cdivision(True):
 *             assert <Py_intptr_t>(&self.acquisition_count) % sizeof(__pyx_atomic_int_type) == 0
 *         self.typeinfo = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(memoryview self):
*/
  __pyx_v_self->typeinfo = NULL;

  /* "View.MemoryView":356
 *     cdef const __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":386
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             PyBuffer_Release(&self.view)
*/

/* Python wrapper */
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self) {
  int __pyx_v_i;
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyThread_type_lock __pyx_t_5;
  PyThread_type_lock __pyx_t_6;

  /* "View.MemoryView":387
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             PyBuffer_Release(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
*/
  __pyx_t_1 = (__pyx_v_self->obj != Py_None);
  if (__pyx_t_1) {

    /* "View.MemoryView":388
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:
 *             PyBuffer_Release(&self.view)             # <<<<<<<<<<<<<<
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
*/
    PyBuffer_Release((&__pyx_v_self->view));

    /* "View.MemoryView":387
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             PyBuffer_Release(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":389
 *         if self.obj is not None:
 *             PyBuffer_Release(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
*/
  __pyx_t_1 = (((Py_buffer *)(&__pyx_v_self->view))->obj == Py_None);
  if (__pyx_t_1) {

    /* "View.MemoryView":391
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL             # <<<<<<<<<<<<<<
 *             Py_DECREF(Py_None)
 * 
*/
    ((Py_buffer *)(&__pyx_v_self->view))->obj = NULL;

    /* "View.MemoryView":392
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
 *             Py_DECREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         cdef int i
*/
    Py_DECREF(Py_None);

    /* "View.MemoryView":389
 *         if self.obj is not None:
 *             PyBuffer_Release(&self.view)
 *         elif (<__pyx_buffer *> &self.view).obj == Py_None:             # <<<<<<<<<<<<<<
 * 
 *             (<__pyx_buffer *> &self.view).obj = NULL
*/
  }
  __pyx_L3:;

  /* "View.MemoryView":396
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(0 if __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() else __pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
*/
  __pyx_t_1 = (__pyx_v_self->lock != NULL);
  if (__pyx_t_1) {

    /* "View.MemoryView":397
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:
 *             for i in range(0 if __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() else __pyx_memoryview_thread_locks_used):             # <<<<<<<<<<<<<<
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
*/
    __pyx_t_1 = __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING();
    if (__pyx_t_1) {
      __pyx_t_2 = 0;
    } else {
      __pyx_t_2 = __pyx_memoryview_thread_locks_used;
    }
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":398
 *         if self.lock != NULL:
 *             for i in range(0 if __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() else __pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
*/
      __pyx_t_1 = ((__pyx_memoryview_thread_locks[__pyx_v_i]) == __pyx_v_self->lock);
      if (__pyx_t_1) {

        /* "View.MemoryView":399
 *             for i in range(0 if __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() else __pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1             # <<<<<<<<<<<<<<
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
*/
        __pyx_memoryview_thread_locks_used = (__pyx_memoryview_thread_locks_used - 1);

        /* "View.MemoryView":400
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
*/
        __pyx_t_1 = (__pyx_v_i != __pyx_memoryview_thread_locks_used);
        if (__pyx_t_1) {

          /* "View.MemoryView":402
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])             # <<<<<<<<<<<<<<
 *                     break
 *             else:
*/
          __pyx_t_5 = (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]);
          __pyx_t_6 = (__pyx_memoryview_thread_locks[__pyx_v_i]);

          /* "View.MemoryView":401
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (             # <<<<<<<<<<<<<<
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break
*/
          (__pyx_memoryview_thread_locks[__pyx_v_i]) = __pyx_t_5;
          (__pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used]) = __pyx_t_6;

          /* "View.MemoryView":400
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:             # <<<<<<<<<<<<<<
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
*/
        }

        /* "View.MemoryView":403
 *                         __pyx_memoryview_thread_locks[i], __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used] = (
 *                             __pyx_memoryview_thread_locks[__pyx_memoryview_thread_locks_used], __pyx_memoryview_thread_locks[i])
 *                     break             # <<<<<<<<<<<<<<
 *             else:
 *                 PyThread_free_lock(self.lock)
*/
        goto __pyx_L6_break;

        /* "View.MemoryView":398
 *         if self.lock != NULL:
 *             for i in range(0 if __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() else __pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:             # <<<<<<<<<<<<<<
 *                     __pyx_memoryview_thread_locks_used -= 1
 *                     if i != __pyx_memoryview_thread_locks_used:
*/
      }
    }
    /*else*/ {

      /* "View.MemoryView":405
 *                     break
 *             else:
 *                 PyThread_free_lock(self.lock)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
*/
      PyThread_free_lock(__pyx_v_self->lock);
    }
    __pyx_L6_break:;

    /* "View.MemoryView":396
 *         cdef int i
 *         global __pyx_memoryview_thread_locks_used
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             for i in range(0 if __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() else __pyx_memoryview_thread_locks_used):
 *                 if __pyx_memoryview_thread_locks[i] is self.lock:
*/
  }

  /* "View.MemoryView":386
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             PyBuffer_Release(&self.view)
*/

  /* function exit code */
}

/* "View.MemoryView":407
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
*/

static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  Py_ssize_t __pyx_v_dim;
  char *__pyx_v_itemp;
  PyObject *__pyx_v_idx = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_item_pointer", 0);

  /* "View.MemoryView":409
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf             # <<<<<<<<<<<<<<
 * 
 *         for dim, idx in enumerate(index):
*/
  __pyx_v_itemp = ((char *)__pyx_v_self->view.buf);

  /* "View.MemoryView":411
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
*/
  __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_v_index)) || PyTuple_CheckExact(__pyx_v_index)) {
    __pyx_t_2 = __pyx_v_index; __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 411, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 411, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 411, __pyx_L1_error)
          #endif
          if (__pyx_t_3 >= __pyx_temp) break;
        }
        __pyx_t_5 = __Pyx_PyList_GetItemRefFast(__pyx_t_2, __pyx_t_3, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_3;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 411, __pyx_L1_error)
          #endif
          if (__pyx_t_3 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3));
        #else
        __pyx_t_5 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_3);
        #endif
        ++__pyx_t_3;
      }
      if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 411, __pyx_L1_error)
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(1, 411, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_v_dim = __pyx_t_1;
    __pyx_t_1 = (__pyx_t_1 + 1);

    /* "View.MemoryView":412
 * 
 *         for dim, idx in enumerate(index):
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)             # <<<<<<<<<<<<<<
 * 
 *         return itemp
*/
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 412, __pyx_L1_error)
    __pyx_t_7 = __pyx_pybuffer_index((&__pyx_v_self->view), __pyx_v_itemp, __pyx_t_6, __pyx_v_dim); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(1, 412, __pyx_L1_error)
    __pyx_v_itemp = __pyx_t_7;

    /* "View.MemoryView":411
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
*/
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":414
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 *         return itemp             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_itemp;
  goto __pyx_L0;

  /* "View.MemoryView":407
 *                 PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.get_item_pointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":417
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
*/

/* Python wrapper */
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_indices = NULL;
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  char *__pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":418
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
*/
  __pyx_t_1 = (__pyx_v_index == __pyx_builtin_Ellipsis);
  if (__pyx_t_1) {

    /* "View.MemoryView":419
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF((PyObject *)__pyx_v_self);
    __pyx_r = ((PyObject *)__pyx_v_self);
    goto __pyx_L0;

    /* "View.MemoryView":418
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
*/
  }

  /* "View.MemoryView":421
 *             return self
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         cdef char *itemp
*/
  __pyx_t_2 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(__pyx_t_2 != Py_None)) {
    PyObject* sequence = __pyx_t_2;
    Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(1, 421, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
    __Pyx_INCREF(__pyx_t_4);
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 421, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(1, 421, __pyx_L1_error)
  }
  __pyx_v_have_slices = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_indices = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "View.MemoryView":424
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(1, 424, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "View.MemoryView":425
 *         cdef char *itemp
 *         if have_slices:
 *             return memview_slice(self, indices)             # <<<<<<<<<<<<<<
 *         else:
 *             itemp = self.get_item_pointer(indices)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((PyObject *)__pyx_memview_slice(__pyx_v_self, __pyx_v_indices)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":424
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
*/
  }

  /* "View.MemoryView":427
 *             return memview_slice(self, indices)
 *         else:
 *             itemp = self.get_item_pointer(indices)             # <<<<<<<<<<<<<<
 *             return self.convert_item_to_object(itemp)
 * 
*/
  /*else*/ {
    __pyx_t_5 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_indices); if (unlikely(__pyx_t_5 == ((void *)NULL))) __PYX_ERR(1, 427, __pyx_L1_error)
    __pyx_v_itemp = __pyx_t_5;

    /* "View.MemoryView":428
 *         else:
 *             itemp = self.get_item_pointer(indices)
 *             return self.convert_item_to_object(itemp)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(memoryview self, object index, object value):
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->convert_item_to_object(__pyx_v_self, __pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 428, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":417
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":430
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError, "Cannot assign to read-only memoryview"
*/

/* Python wrapper */
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /*proto*/
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_obj = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);
  __Pyx_INCREF(__pyx_v_index);

  /* "View.MemoryView":431
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError, "Cannot assign to read-only memoryview"
 * 
*/
  if (unlikely(__pyx_v_self->view.readonly)) {

    /* "View.MemoryView":432
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:
 *             raise TypeError, "Cannot assign to read-only memoryview"             # <<<<<<<<<<<<<<
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_Cannot_assign_to_read_only_memor, 0, 0);
    __PYX_ERR(1, 432, __pyx_L1_error)

    /* "View.MemoryView":431
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         if self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise TypeError, "Cannot assign to read-only memoryview"
 * 
*/
  }

  /* "View.MemoryView":434
 *             raise TypeError, "Cannot assign to read-only memoryview"
 * 
 *         have_slices, index = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         if have_slices:
*/
  __pyx_t_1 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(__pyx_t_1 != Py_None)) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(1, 434, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(1, 434, __pyx_L1_error)
  }
  __pyx_v_have_slices = __pyx_t_2;
  __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":436
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj is not None:
*/
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(1, 436, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "View.MemoryView":437
 * 
 *         if have_slices:
 *             obj = self.is_slice(value)             # <<<<<<<<<<<<<<
 *             if obj is not None:
 *                 self.setitem_slice_assignment(self[index], obj)
*/
    __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->is_slice(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_obj = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "View.MemoryView":438
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj is not None:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
*/
    __pyx_t_4 = (__pyx_v_obj != Py_None);
    if (__pyx_t_4) {

      /* "View.MemoryView":439
 *             obj = self.is_slice(value)
 *             if obj is not None:
 *                 self.setitem_slice_assignment(self[index], obj)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)
*/
      __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assignment(__pyx_v_self, __pyx_t_1, __pyx_v_obj); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "View.MemoryView":438
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj is not None:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
*/
      goto __pyx_L5;
    }

    /* "View.MemoryView":441
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)             # <<<<<<<<<<<<<<
 *         else:
 *             self.setitem_indexed(index, value)
*/
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 441, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_memoryview_type))))) __PYX_ERR(1, 441, __pyx_L1_error)
      __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assign_scalar(__pyx_v_self, ((struct __pyx_memoryview_obj *)__pyx_t_3), __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 441, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L5:;

    /* "View.MemoryView":436
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj is not None:
*/
    goto __pyx_L4;
  }

  /* "View.MemoryView":443
 *                 self.setitem_slice_assign_scalar(self[index], value)
 *         else:
 *             self.setitem_indexed(index, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef is_slice(self, obj):
*/
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_indexed(__pyx_v_self, __pyx_v_index, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 443, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L4:;

  /* "View.MemoryView":430
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         if self.view.readonly:
 *             raise TypeError, "Cannot assign to read-only memoryview"
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":445
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
*/

static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_slice", 0);
  __Pyx_INCREF(__pyx_v_obj);

  /* "View.MemoryView":446
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_mstate_global->__pyx_memoryview_type); 
  __pyx_t_2 = (!__pyx_t_1);
  if (__pyx_t_2) {

    /* "View.MemoryView":447
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "View.MemoryView":448
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
*/
        __pyx_t_7 = NULL;
        __pyx_t_8 = __Pyx_PyLong_From_int(((__pyx_v_self->flags & (~PyBUF_WRITABLE)) | PyBUF_ANY_CONTIGUOUS)); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 448, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_8);

        /* "View.MemoryView":449
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 return None
*/
        __pyx_t_9 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 449, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = 1;
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_7, __pyx_v_obj, __pyx_t_8, __pyx_t_9};
          __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_memoryview_type, __pyx_callargs+__pyx_t_10, (4-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 448, __pyx_L4_error)
          __Pyx_GOTREF((PyObject *)__pyx_t_6);
        }
        __Pyx_DECREF_SET(__pyx_v_obj, ((PyObject *)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "View.MemoryView":447
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
*/
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L9_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "View.MemoryView":450
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 return None
 * 
*/
      __pyx_t_11 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_TypeError))));
      if (__pyx_t_11) {
        __Pyx_ErrRestore(0,0,0);

        /* "View.MemoryView":451
 *                                  self.dtype_is_object)
 *             except TypeError:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         return obj
*/
        __Pyx_XDECREF(__pyx_r);
        __pyx_r = Py_None; __Pyx_INCREF(Py_None);
        goto __pyx_L7_except_return;
      }
      goto __pyx_L6_except_error;

      /* "View.MemoryView":447
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
*/
      __pyx_L6_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L7_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L9_try_end:;
    }

    /* "View.MemoryView":446
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags & ~PyBUF_WRITABLE | PyBUF_ANY_CONTIGUOUS,
*/
  }

  /* "View.MemoryView":453
 *                 return None
 * 
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assignment(self, dst, src):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "View.MemoryView":445
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":455
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
*/

static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src) {
  __Pyx_memviewslice __pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_src_slice;
  __Pyx_memviewslice __pyx_v_msrc;
  __Pyx_memviewslice __pyx_v_mdst;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assignment", 0);

  /* "View.MemoryView":458
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 *         cdef __Pyx_memviewslice msrc = get_slice_from_memview(src, &src_slice)[0]             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mdst = get_slice_from_memview(dst, &dst_slice)[0]
 * 
*/
  if (!(likely(((__pyx_v_src) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_src, __pyx_mstate_global->__pyx_memoryview_type))))) __PYX_ERR(1, 458, __pyx_L1_error)
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_src), (&__pyx_v_src_slice)); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(1, 458, __pyx_L1_error)
  __pyx_v_msrc = (__pyx_t_1[0]);

  /* "View.MemoryView":459
 *         cdef __Pyx_memviewslice src_slice
 *         cdef __Pyx_memviewslice msrc = get_slice_from_memview(src, &src_slice)[0]
 *         cdef __Pyx_memviewslice mdst = get_slice_from_memview(dst, &dst_slice)[0]             # <<<<<<<<<<<<<<
 * 
 *         memoryview_copy_contents(msrc, mdst, src.ndim, dst.ndim, self.dtype_is_object)
*/
  if (!(likely(((__pyx_v_dst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dst, __pyx_mstate_global->__pyx_memoryview_type))))) __PYX_ERR(1, 459, __pyx_L1_error)
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_dst), (&__pyx_v_dst_slice)); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(1, 459, __pyx_L1_error)
  __pyx_v_mdst = (__pyx_t_1[0]);

  /* "View.MemoryView":461
 *         cdef __Pyx_memviewslice mdst = get_slice_from_memview(dst, &dst_slice)[0]
 * 
 *         memoryview_copy_contents(msrc, mdst, src.ndim, dst.ndim, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_src, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 461, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_dst, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyLong_As_int(__pyx_t_2); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 461, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __pyx_memoryview_copy_contents(__pyx_v_msrc, __pyx_v_mdst, __pyx_t_3, __pyx_t_4, __pyx_v_self->dtype_is_object); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 461, __pyx_L1_error)

  /* "View.MemoryView":455
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":463
 *         memoryview_copy_contents(msrc, mdst, src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
*/

static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value) {
  int __pyx_v_array[128];
  void *__pyx_v_tmp;
  void *__pyx_v_item;
  __Pyx_memviewslice *__pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_tmp_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  char const *__pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assign_scalar", 0);

  /* "View.MemoryView":465
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 *         cdef int array[128]
 *         cdef void *tmp = NULL             # <<<<<<<<<<<<<<
 *         cdef void *item
 * 
*/
  __pyx_v_tmp = NULL;

  /* "View.MemoryView":470
 *         cdef __Pyx_memviewslice *dst_slice
 *         cdef __Pyx_memviewslice tmp_slice
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)             # <<<<<<<<<<<<<<
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
*/
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_dst, (&__pyx_v_tmp_slice)); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(1, 470, __pyx_L1_error)
  __pyx_v_dst_slice = __pyx_t_1;

  /* "View.MemoryView":472
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
*/
  __pyx_t_2 = (((size_t)__pyx_v_self->view.itemsize) > (sizeof(__pyx_v_array)));
  if (__pyx_t_2) {

    /* "View.MemoryView":473
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)             # <<<<<<<<<<<<<<
 *             if tmp == NULL:
 *                 raise MemoryError
*/
    __pyx_v_tmp = PyMem_Malloc(__pyx_v_self->view.itemsize);

    /* "View.MemoryView":474
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
*/
    __pyx_t_2 = (__pyx_v_tmp == NULL);
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":475
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             item = tmp
 *         else:
*/
      PyErr_NoMemory(); __PYX_ERR(1, 475, __pyx_L1_error)

      /* "View.MemoryView":474
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
*/
    }

    /* "View.MemoryView":476
 *             if tmp == NULL:
 *                 raise MemoryError
 *             item = tmp             # <<<<<<<<<<<<<<
 *         else:
 *             item = <void *> array
*/
    __pyx_v_item = __pyx_v_tmp;

    /* "View.MemoryView":472
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":478
 *             item = tmp
 *         else:
 *             item = <void *> array             # <<<<<<<<<<<<<<
 * 
 *         try:
*/
  /*else*/ {
    __pyx_v_item = ((void *)__pyx_v_array);
  }
  __pyx_L3:;

  /* "View.MemoryView":480
 *             item = <void *> array
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value
*/
  /*try:*/ {

    /* "View.MemoryView":481
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
*/
    if (__pyx_v_self->dtype_is_object) {

      /* "View.MemoryView":482
 *         try:
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value             # <<<<<<<<<<<<<<
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)
*/
      (((PyObject **)__pyx_v_item)[0]) = ((PyObject *)__pyx_v_value);

      /* "View.MemoryView":481
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
*/
      goto __pyx_L8;
    }

    /* "View.MemoryView":484
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    /*else*/ {
      __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, ((char *)__pyx_v_item), __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 484, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __pyx_L8:;

    /* "View.MemoryView":488
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
*/
    __pyx_t_2 = (__pyx_v_self->view.suboffsets != NULL);
    if (__pyx_t_2) {

      /* "View.MemoryView":489
 * 
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)             # <<<<<<<<<<<<<<
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 *                                 item, self.dtype_is_object)
*/
      __pyx_t_4 = assert_direct_dimensions(__pyx_v_self->view.suboffsets, __pyx_v_self->view.ndim); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 489, __pyx_L6_error)

      /* "View.MemoryView":488
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
*/
    }

    /* "View.MemoryView":490
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,             # <<<<<<<<<<<<<<
 *                                 item, self.dtype_is_object)
 *         finally:
*/
    __pyx_memoryview_slice_assign_scalar(__pyx_v_dst_slice, __pyx_v_dst->view.ndim, __pyx_v_self->view.itemsize, __pyx_v_item, __pyx_v_self->dtype_is_object);
  }

  /* "View.MemoryView":493
 *                                 item, self.dtype_is_object)
 *         finally:
 *             PyMem_Free(tmp)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_indexed(self, index, value):
*/
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_tmp);
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
      if ( unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __pyx_t_4 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_6 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_tmp);
      }
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_9);
      __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
      __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_6;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "View.MemoryView":463
 *         memoryview_copy_contents(msrc, mdst, src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assign_scalar", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":495
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
*/

static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_indexed", 0);

  /* "View.MemoryView":496
 * 
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)             # <<<<<<<<<<<<<<
 *         self.assign_item_from_object(itemp, value)
 * 
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_index); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(1, 496, __pyx_L1_error)
  __pyx_v_itemp = __pyx_t_1;

  /* "View.MemoryView":497
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
*/
  __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":495
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_indexed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":499
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
*/

static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_bytesitem = 0;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":502
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef bytes bytesitem
 * 
*/
  __pyx_t_2 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_struct, 0, 0, NULL, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 502, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_2;
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":505
 *         cdef bytes bytesitem
 * 
 *         bytesitem = itemp[:self.view.itemsize]             # <<<<<<<<<<<<<<
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
*/
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_itemp + 0, __pyx_v_self->view.itemsize - 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bytesitem = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":506
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "View.MemoryView":507
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)             # <<<<<<<<<<<<<<
 *         except struct.error:
 *             raise ValueError, "Unable to convert item to object"
*/
      __pyx_t_5 = __pyx_v_struct;
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 507, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = 0;
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_t_6, __pyx_v_bytesitem};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_unpack, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 507, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __pyx_v_result = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "View.MemoryView":506
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
*/
    }

    /* "View.MemoryView":511
 *             raise ValueError, "Unable to convert item to object"
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
*/
    /*else:*/ {
      __pyx_t_8 = __Pyx_ssize_strlen(__pyx_v_self->view.format); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(1, 511, __pyx_L5_except_error)
      __pyx_t_9 = (__pyx_t_8 == 1);
      if (__pyx_t_9) {

        /* "View.MemoryView":512
 *         else:
 *             if len(self.view.format) == 1:
 *                 return result[0]             # <<<<<<<<<<<<<<
 *             return result
 * 
*/
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_result, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 512, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_r = __pyx_t_1;
        __pyx_t_1 = 0;
        goto __pyx_L6_except_return;

        /* "View.MemoryView":511
 *             raise ValueError, "Unable to convert item to object"
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
*/
      }

      /* "View.MemoryView":513
 *             if len(self.view.format) == 1:
 *                 return result[0]
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_result);
      __pyx_r = __pyx_v_result;
      goto __pyx_L6_except_return;
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "View.MemoryView":508
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:             # <<<<<<<<<<<<<<
 *             raise ValueError, "Unable to convert item to object"
 *         else:
*/
    __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_6, &__pyx_t_5);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_mstate_global->__pyx_n_u_error); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 508, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_10);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_ErrRestore(__pyx_t_1, __pyx_t_6, __pyx_t_5);
    __pyx_t_1 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0;
    if (__pyx_t_11) {
      __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_1) < 0) __PYX_ERR(1, 508, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_1);

      /* "View.MemoryView":509
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError, "Unable to convert item to object"             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_mstate_global->__pyx_kp_u_Unable_to_convert_item_to_object, 0, 0);
      __PYX_ERR(1, 509, __pyx_L5_except_error)
    }
    goto __pyx_L5_except_error;

    /* "View.MemoryView":506
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
  }

  /* "View.MemoryView":499
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesitem);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":515
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
*/

static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_struct = NULL;
  char __pyx_v_c;
  PyObject *__pyx_v_bytesvalue = 0;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  char *__pyx_t_10;
  char *__pyx_t_11;
  Py_ssize_t __pyx_t_12;
  char *__pyx_t_13;
  char *__pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":518
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef char c
 *         cdef bytes bytesvalue
*/
  __pyx_t_2 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_struct, 0, 0, NULL, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 518, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_2;
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":523
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
*/
  __pyx_t_3 = PyTuple_Check(__pyx_v_value); 
  if (__pyx_t_3) {

    /* "View.MemoryView":524
 * 
 *         if isinstance(value, tuple):
 *             bytesvalue = struct.pack(self.view.format, *value)             # <<<<<<<<<<<<<<
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_mstate_global->__pyx_n_u_pack); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(1, 524, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_v_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyNumber_Add(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_4))) __PYX_ERR(1, 524, __pyx_L1_error)
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "View.MemoryView":523
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":526
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)             # <<<<<<<<<<<<<<
 * 
 *         for i, c in enumerate(bytesvalue):
*/
  /*else*/ {
    __pyx_t_6 = __pyx_v_struct;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 526, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_t_1, __pyx_v_value};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_pack, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 526, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_4))) __PYX_ERR(1, 526, __pyx_L1_error)
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":528
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
*/
  __pyx_t_8 = 0;
  if (unlikely(__pyx_v_bytesvalue == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' is not iterable");
    __PYX_ERR(1, 528, __pyx_L1_error)
  }
  __Pyx_INCREF(__pyx_v_bytesvalue);
  __pyx_t_9 = __pyx_v_bytesvalue;
  __pyx_t_11 = __Pyx_PyBytes_AsWritableString(__pyx_t_9); if (unlikely(__pyx_t_11 == ((char *)NULL))) __PYX_ERR(1, 528, __pyx_L1_error)
  __pyx_t_12 = __Pyx_PyBytes_GET_SIZE(__pyx_t_9); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(1, 528, __pyx_L1_error)
  __pyx_t_13 = (__pyx_t_11 + __pyx_t_12);
  for (__pyx_t_14 = __pyx_t_11; __pyx_t_14 < __pyx_t_13; __pyx_t_14++) {
    __pyx_t_10 = __pyx_t_14;
    __pyx_v_c = (__pyx_t_10[0]);

    /* "View.MemoryView":529
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
*/
    __pyx_v_i = __pyx_t_8;

    /* "View.MemoryView":528
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
*/
    __pyx_t_8 = (__pyx_t_8 + 1);

    /* "View.MemoryView":529
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
*/
    (__pyx_v_itemp[__pyx_v_i]) = __pyx_v_c;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "View.MemoryView":515
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memoryview.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":531
 *             itemp[i] = c
 * 
 *     @cname('getbuffer')             # <<<<<<<<<<<<<<
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
*/

/* Python wrapper */
CYTHON_UNUSED static int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
CYTHON_UNUSED static int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  char *__pyx_t_4;
  void *__pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  if (unlikely(__pyx_v_info == NULL)) {
    PyErr_SetString(PyExc_BufferError, "PyObject_GetBuffer: view==NULL argument is obsolete");
    return -1;
  }
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(__pyx_v_info->obj);

  /* "View.MemoryView":533
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError, "Cannot create writable memory view from read-only memoryview"
 * 
*/
  __pyx_t_2 = ((__pyx_v_flags & PyBUF_WRITABLE) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = __pyx_v_self->view.readonly;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":534
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
 *             raise ValueError, "Cannot create writable memory view from read-only memoryview"             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_ND:
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_mstate_global->__pyx_kp_u_Cannot_create_writable_memory_vi, 0, 0);
    __PYX_ERR(1, 534, __pyx_L1_error)

    /* "View.MemoryView":533
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:             # <<<<<<<<<<<<<<
 *             raise ValueError, "Cannot create writable memory view from read-only memoryview"
 * 
*/
  }

  /* "View.MemoryView":536
 *             raise ValueError, "Cannot create writable memory view from read-only memoryview"
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
*/
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_ND) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":537
 * 
 *         if flags & PyBUF_ND:
 *             info.shape = self.view.shape             # <<<<<<<<<<<<<<
 *         else:
 *             info.shape = NULL
*/
    __pyx_t_3 = __pyx_v_self->view.shape;
    __pyx_v_info->shape = __pyx_t_3;

    /* "View.MemoryView":536
 *             raise ValueError, "Cannot create writable memory view from read-only memoryview"
 * 
 *         if flags & PyBUF_ND:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
*/
    goto __pyx_L6;
  }

  /* "View.MemoryView":539
 *             info.shape = self.view.shape
 *         else:
 *             info.shape = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_STRIDES:
*/
  /*else*/ {
    __pyx_v_info->shape = NULL;
  }
  __pyx_L6:;

  /* "View.MemoryView":541
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
*/
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":542
 * 
 *         if flags & PyBUF_STRIDES:
 *             info.strides = self.view.strides             # <<<<<<<<<<<<<<
 *         else:
 *             info.strides = NULL
*/
    __pyx_t_3 = __pyx_v_self->view.strides;
    __pyx_v_info->strides = __pyx_t_3;

    /* "View.MemoryView":541
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
*/
    goto __pyx_L7;
  }

  /* "View.MemoryView":544
 *             info.strides = self.view.strides
 *         else:
 *             info.strides = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_INDIRECT:
*/
  /*else*/ {
    __pyx_v_info->strides = NULL;
  }
  __pyx_L7:;

  /* "View.MemoryView":546
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
*/
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_INDIRECT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":547
 * 
 *         if flags & PyBUF_INDIRECT:
 *             info.suboffsets = self.view.suboffsets             # <<<<<<<<<<<<<<
 *         else:
 *             info.suboffsets = NULL
*/
    __pyx_t_3 = __pyx_v_self->view.suboffsets;
    __pyx_v_info->suboffsets = __pyx_t_3;

    /* "View.MemoryView":546
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
*/
    goto __pyx_L8;
  }

  /* "View.MemoryView":549
 *             info.suboffsets = self.view.suboffsets
 *         else:
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
*/
  /*else*/ {
    __pyx_v_info->suboffsets = NULL;
  }
  __pyx_L8:;

  /* "View.MemoryView":551
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
*/
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":552
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.view.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
*/
    __pyx_t_4 = __pyx_v_self->view.format;
    __pyx_v_info->format = __pyx_t_4;

    /* "View.MemoryView":551
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
*/
    goto __pyx_L9;
  }

  /* "View.MemoryView":554
 *             info.format = self.view.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.buf = self.view.buf
*/
  /*else*/ {
    __pyx_v_info->format = NULL;
  }
  __pyx_L9:;

  /* "View.MemoryView":556
 *             info.format = NULL
 * 
 *         info.buf = self.view.buf             # <<<<<<<<<<<<<<
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
*/
  __pyx_t_5 = __pyx_v_self->view.buf;
  __pyx_v_info->buf = __pyx_t_5;

  /* "View.MemoryView":557
 * 
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim             # <<<<<<<<<<<<<<
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
*/
  __pyx_t_6 = __pyx_v_self->view.ndim;
  __pyx_v_info->ndim = __pyx_t_6;

  /* "View.MemoryView":558
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize             # <<<<<<<<<<<<<<
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
*/
  __pyx_t_7 = __pyx_v_self->view.itemsize;
  __pyx_v_info->itemsize = __pyx_t_7;

  /* "View.MemoryView":559
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len             # <<<<<<<<<<<<<<
 *         info.readonly = self.view.readonly
 *         info.obj = self
*/
  __pyx_t_7 = __pyx_v_self->view.len;
  __pyx_v_info->len = __pyx_t_7;

  /* "View.MemoryView":560
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly             # <<<<<<<<<<<<<<
 *         info.obj = self
 * 
*/
  __pyx_t_1 = __pyx_v_self->view.readonly;
  __pyx_v_info->readonly = __pyx_t_1;

  /* "View.MemoryView":561
 *         info.len = self.view.len
 *         info.readonly = self.view.readonly
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self);
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":531
 *             itemp[i] = c
 * 
 *     @cname('getbuffer')             # <<<<<<<<<<<<<<
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_WRITABLE and self.view.readonly:
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = 0;
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":564
 * 
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":566
 *     @property
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&result.from_slice)
 *         return result
*/
  __pyx_t_1 = __pyx_memoryview_copy_object(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_memoryviewslice_type))))) __PYX_ERR(1, 566, __pyx_L1_error)
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":567
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
  __pyx_t_2 = __pyx_memslice_transpose((&__pyx_v_result->from_slice)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 567, __pyx_L1_error)

  /* "View.MemoryView":568
 *         cdef _memoryviewslice result = memoryview_copy(self)
 *         transpose_memslice(&result.from_slice)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_result);
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":564
 * 
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def T(self):
 *         cdef _memoryviewslice result = memoryview_copy(self)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.T.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":570
 *         return result
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def base(self):
 *         return self._get_base()
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":572
 *     @property
 *     def base(self):
 *         return self._get_base()             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_base(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->_get_base(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":570
 *         return result
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def base(self):
 *         return self._get_base()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.base.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":574
 *         return self._get_base()
 * 
 *     cdef _get_base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
*/

static PyObject *__pyx_memoryview__get_base(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_base", 0);

  /* "View.MemoryView":575
 * 
 *     cdef _get_base(self):
 *         return self.obj             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->obj);
  __pyx_r = __pyx_v_self->obj;
  goto __pyx_L0;

  /* "View.MemoryView":574
 *         return self._get_base()
 * 
 *     cdef _get_base(self):             # <<<<<<<<<<<<<<
 *         return self.obj
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":577
 *         return self.obj
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def shape(self):
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_7genexpr__pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":579
 *     @property
 *     def shape(self):
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);
    for (__pyx_t_4 = __pyx_v_self->view.shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {
      __pyx_t_2 = __pyx_t_4;
      __pyx_7genexpr__pyx_v_length = (__pyx_t_2[0]);
      __pyx_t_5 = PyLong_FromSsize_t(__pyx_7genexpr__pyx_v_length); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 579, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(1, 579, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  } /* exit inner scope */
  __pyx_t_5 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":577
 *         return self.obj
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def shape(self):
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.shape.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":581
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def strides(self):
 *         if self.view.strides == NULL:
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_8genexpr1__pyx_v_stride;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":583
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError, "Buffer view does not expose strides"
*/
  __pyx_t_1 = (__pyx_v_self->view.strides == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "View.MemoryView":585
 *         if self.view.strides == NULL:
 * 
 *             raise ValueError, "Buffer view does not expose strides"             # <<<<<<<<<<<<<<
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_mstate_global->__pyx_kp_u_Buffer_view_does_not_expose_stri, 0, 0);
    __PYX_ERR(1, 585, __pyx_L1_error)

    /* "View.MemoryView":583
 *     @property
 *     def strides(self):
 *         if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *             raise ValueError, "Buffer view does not expose strides"
*/
  }

  /* "View.MemoryView":587
 *             raise ValueError, "Buffer view does not expose strides"
 * 
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 587, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = (__pyx_v_self->view.strides + __pyx_v_self->view.ndim);
    for (__pyx_t_5 = __pyx_v_self->view.strides; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {
      __pyx_t_3 = __pyx_t_5;
      __pyx_8genexpr1__pyx_v_stride = (__pyx_t_3[0]);
      __pyx_t_6 = PyLong_FromSsize_t(__pyx_8genexpr1__pyx_v_stride); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 587, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_6))) __PYX_ERR(1, 587, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  } /* exit inner scope */
  __pyx_t_6 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":581
 *         return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def strides(self):
 *         if self.view.strides == NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.strides.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":589
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_8genexpr2__pyx_v_suboffset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":591
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
*/
  __pyx_t_1 = (__pyx_v_self->view.suboffsets == NULL);
  if (__pyx_t_1) {

    /* "View.MemoryView":592
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PySequence_Multiply(__pyx_mstate_global->__pyx_tuple[0], __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 592, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":591
 *     @property
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *             return (-1,) * self.view.ndim
 * 
*/
  }

  /* "View.MemoryView":594
 *             return (-1,) * self.view.ndim
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 594, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = (__pyx_v_self->view.suboffsets + __pyx_v_self->view.ndim);
    for (__pyx_t_5 = __pyx_v_self->view.suboffsets; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {
      __pyx_t_3 = __pyx_t_5;
      __pyx_8genexpr2__pyx_v_suboffset = (__pyx_t_3[0]);
      __pyx_t_6 = PyLong_FromSsize_t(__pyx_8genexpr2__pyx_v_suboffset); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 594, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_6))) __PYX_ERR(1, 594, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  } /* exit inner scope */
  __pyx_t_6 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":589
 *         return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.suboffsets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":596
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def ndim(self):
 *         return self.view.ndim
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":598
 *     @property
 *     def ndim(self):
 *         return self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":596
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def ndim(self):
 *         return self.view.ndim
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.ndim.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":600
 *         return self.view.ndim
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def itemsize(self):
 *         return self.view.itemsize
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":602
 *     @property
 *     def itemsize(self):
 *         return self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":600
 *         return self.view.ndim
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def itemsize(self):
 *         return self.view.itemsize
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":604
 *         return self.view.itemsize
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def nbytes(self):
 *         return self.size * self.view.itemsize
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":606
 *     @property
 *     def nbytes(self):
 *         return self.size * self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":604
 *         return self.view.itemsize
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def nbytes(self):
 *         return self.size * self.view.itemsize
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.nbytes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":608
 *         return self.size * self.view.itemsize
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def size(self):
 *         if self._size is None:
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":610
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_size == Py_None);
  if (__pyx_t_1) {

    /* "View.MemoryView":611
 *     def size(self):
 *         if self._size is None:
 *             result = 1             # <<<<<<<<<<<<<<
 * 
 *             for length in self.view.shape[:self.view.ndim]:
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
    __pyx_v_result = __pyx_mstate_global->__pyx_int_1;

    /* "View.MemoryView":613
 *             result = 1
 * 
 *             for length in self.view.shape[:self.view.ndim]:             # <<<<<<<<<<<<<<
 *                 result *= length
 * 
*/
    __pyx_t_3 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);
    for (__pyx_t_4 = __pyx_v_self->view.shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {
      __pyx_t_2 = __pyx_t_4;
      __pyx_t_5 = PyLong_FromSsize_t((__pyx_t_2[0])); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 613, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "View.MemoryView":614
 * 
 *             for length in self.view.shape[:self.view.ndim]:
 *                 result *= length             # <<<<<<<<<<<<<<
 * 
 *             self._size = result
*/
      __pyx_t_5 = PyNumber_InPlaceMultiply(__pyx_v_result, __pyx_v_length); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 614, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_5);
      __pyx_t_5 = 0;
    }

    /* "View.MemoryView":616
 *                 result *= length
 * 
 *             self._size = result             # <<<<<<<<<<<<<<
 * 
 *         return self._size
*/
    __Pyx_INCREF(__pyx_v_result);
    __Pyx_GIVEREF(__pyx_v_result);
    __Pyx_GOTREF(__pyx_v_self->_size);
    __Pyx_DECREF(__pyx_v_self->_size);
    __pyx_v_self->_size = __pyx_v_result;

    /* "View.MemoryView":610
 *     @property
 *     def size(self):
 *         if self._size is None:             # <<<<<<<<<<<<<<
 *             result = 1
 * 
*/
  }

  /* "View.MemoryView":618
 *             self._size = result
 * 
 *         return self._size             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_size);
  __pyx_r = __pyx_v_self->_size;
  goto __pyx_L0;

  /* "View.MemoryView":608
 *         return self.size * self.view.itemsize
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def size(self):
 *         if self._size is None:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":620
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
*/

/* Python wrapper */
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":621
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
*/
  __pyx_t_1 = (__pyx_v_self->view.ndim >= 1);
  if (__pyx_t_1) {

    /* "View.MemoryView":622
 *     def __len__(self):
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         return 0
*/
    __pyx_r = (__pyx_v_self->view.shape[0]);
    goto __pyx_L0;

    /* "View.MemoryView":621
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
*/
  }

  /* "View.MemoryView":624
 *             return self.view.shape[0]
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":620
 *         return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":626
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
*/

/* Python wrapper */
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":627
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_t_1), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":628
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
*/
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Format(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4[0] = __pyx_mstate_global->__pyx_kp_u_MemoryView_of;
  __pyx_t_4[1] = __pyx_t_2;
  __pyx_t_4[2] = __pyx_mstate_global->__pyx_kp_u_at_0x;
  __pyx_t_4[3] = __pyx_t_3;
  __pyx_t_4[4] = __pyx_mstate_global->__pyx_kp_u__3;

  /* "View.MemoryView":627
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
*/
  __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_4, 5, 15 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3));
  if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":626
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":630
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "View.MemoryView":631
 * 
 *     def __str__(self):
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_base); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_t_1), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3[0] = __pyx_mstate_global->__pyx_kp_u_MemoryView_of;
  __pyx_t_3[1] = __pyx_t_2;
  __pyx_t_3[2] = __pyx_mstate_global->__pyx_kp_u_object_2;
  __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_3, 3, 15 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 8, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
  if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":630
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":634
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
*/

/* Python wrapper */
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_c_contig (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("is_c_contig", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("is_c_contig", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_c_contig", 0);

  /* "View.MemoryView":637
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
*/
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(1, 637, __pyx_L1_error)
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":638
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def is_f_contig(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'C', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":634
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_c_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":640
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
*/

/* Python wrapper */
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_f_contig (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("is_f_contig", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("is_f_contig", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_f_contig", 0);

  /* "View.MemoryView":643
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
*/
  __pyx_t_1 = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp)); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(1, 643, __pyx_L1_error)
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":644
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def copy(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig((__pyx_v_mslice[0]), 'F', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":640
 *         return slice_is_contig(mslice[0], 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_f_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":646
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
*/

/* Python wrapper */
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("copy", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("copy", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_mslice;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "View.MemoryView":648
 *     def copy(self):
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &mslice)
*/
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_F_CONTIGUOUS));

  /* "View.MemoryView":650
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 * 
 *         slice_copy(self, &mslice)             # <<<<<<<<<<<<<<
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,
 *                                    self.view.itemsize,
*/
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_mslice));

  /* "View.MemoryView":651
 * 
 *         slice_copy(self, &mslice)
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                    self.view.itemsize,
 *                                    flags|PyBUF_C_CONTIGUOUS,
*/
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_mslice), __pyx_k_c, __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_C_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 651, __pyx_L1_error)
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":656
 *                                    self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &mslice)             # <<<<<<<<<<<<<<
 * 
 *     def copy_fortran(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_mslice)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":646
 *         return slice_is_contig(mslice[0], 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":658
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
*/

/* Python wrapper */
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_fortran (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("copy_fortran", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("copy_fortran", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_fortran", 0);

  /* "View.MemoryView":660
 *     def copy_fortran(self):
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &src)
*/
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_C_CONTIGUOUS));

  /* "View.MemoryView":662
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 * 
 *         slice_copy(self, &src)             # <<<<<<<<<<<<<<
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,
 *                                 self.view.itemsize,
*/
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_src));

  /* "View.MemoryView":663
 * 
 *         slice_copy(self, &src)
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                 self.view.itemsize,
 *                                 flags|PyBUF_F_CONTIGUOUS,
*/
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_src), __pyx_k_fortran, __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_F_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) __PYX_ERR(1, 663, __pyx_L1_error)
  __pyx_v_dst = __pyx_t_1;

  /* "View.MemoryView":668
 *                                 self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &dst)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_dst)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":658
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy_fortran", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_memoryview_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf___pyx_memoryview___reduce_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryview___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_memoryview_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf___pyx_memoryview_2__setstate_cython__(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryview_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryview_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":671
 * 
 * 
 * @cname('__pyx_memoryview_new')             # <<<<<<<<<<<<<<
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, const __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
*/

static PyObject *__pyx_memoryview_new(PyObject *__pyx_v_o, int __pyx_v_flags, int __pyx_v_dtype_is_object, __Pyx_TypeInfo const *__pyx_v_typeinfo) {
  struct __pyx_memoryview_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_cwrapper", 0);

  /* "View.MemoryView":673
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, const __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)             # <<<<<<<<<<<<<<
 *     result.typeinfo = typeinfo
 *     return result
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_2, __pyx_v_o, __pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_memoryview_type, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 673, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_result = ((struct __pyx_memoryview_obj *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":674
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, const __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo             # <<<<<<<<<<<<<<
 *     return result
 * 
*/
  __pyx_v_result->typeinfo = __pyx_v_typeinfo;

  /* "View.MemoryView":675
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_check')
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_result);
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":671
 * 
 * 
 * @cname('__pyx_memoryview_new')             # <<<<<<<<<<<<<<
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, const __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.memoryview_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":677
 *     return result
 * 
 * @cname('__pyx_memoryview_check')             # <<<<<<<<<<<<<<
 * cdef inline bint memoryview_check(object o) noexcept:
 *     return isinstance(o, memoryview)
*/

static CYTHON_INLINE int __pyx_memoryview_check(PyObject *__pyx_v_o) {
  int __pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":679
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o) noexcept:
 *     return isinstance(o, memoryview)             # <<<<<<<<<<<<<<
 * 
 * cdef tuple _unellipsify(object index, int ndim):
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_o, __pyx_mstate_global->__pyx_memoryview_type); 
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "View.MemoryView":677
 *     return result
 * 
 * @cname('__pyx_memoryview_check')             # <<<<<<<<<<<<<<
 * cdef inline bint memoryview_check(object o) noexcept:
 *     return isinstance(o, memoryview)
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":681
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
*/

static PyObject *_unellipsify(PyObject *__pyx_v_index, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_idx;
  PyObject *__pyx_v_tup = NULL;
  PyObject *__pyx_v_result = NULL;
  int __pyx_v_have_slices;
  int __pyx_v_seen_ellipsis;
  PyObject *__pyx_v_item = NULL;
  Py_ssize_t __pyx_v_nslices;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6[3];
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unellipsify", 0);

  /* "View.MemoryView":687
 *     """
 *     cdef Py_ssize_t idx
 *     tup = <tuple>index if isinstance(index, tuple) else (index,)             # <<<<<<<<<<<<<<
 * 
 *     result = [slice(None)] * ndim
*/
  __pyx_t_2 = PyTuple_Check(__pyx_v_index); 
  if (__pyx_t_2) {
    __Pyx_INCREF(((PyObject*)__pyx_v_index));
    __pyx_t_1 = __pyx_v_index;
  } else {
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_index);
    __Pyx_GIVEREF(__pyx_v_index);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_index) != (0)) __PYX_ERR(1, 687, __pyx_L1_error);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_v_tup = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":689
 *     tup = <tuple>index if isinstance(index, tuple) else (index,)
 * 
 *     result = [slice(None)] * ndim             # <<<<<<<<<<<<<<
 *     have_slices = False
 *     seen_ellipsis = False
*/
  __pyx_t_1 = PyList_New(1 * ((__pyx_v_ndim<0) ? 0:__pyx_v_ndim)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  { Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < __pyx_v_ndim; __pyx_temp++) {
      __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[0]);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);
      if (__Pyx_PyList_SET_ITEM(__pyx_t_1, __pyx_temp, __pyx_mstate_global->__pyx_slice[0]) != (0)) __PYX_ERR(1, 689, __pyx_L1_error);
    }
  }
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":690
 * 
 *     result = [slice(None)] * ndim
 *     have_slices = False             # <<<<<<<<<<<<<<
 *     seen_ellipsis = False
 *     idx = 0
*/
  __pyx_v_have_slices = 0;

  /* "View.MemoryView":691
 *     result = [slice(None)] * ndim
 *     have_slices = False
 *     seen_ellipsis = False             # <<<<<<<<<<<<<<
 *     idx = 0
 *     for item in tup:
*/
  __pyx_v_seen_ellipsis = 0;

  /* "View.MemoryView":692
 *     have_slices = False
 *     seen_ellipsis = False
 *     idx = 0             # <<<<<<<<<<<<<<
 *     for item in tup:
 *         if item is Ellipsis:
*/
  __pyx_v_idx = 0;

  /* "View.MemoryView":693
 *     seen_ellipsis = False
 *     idx = 0
 *     for item in tup:             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
*/
  if (unlikely(__pyx_v_tup == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 693, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_tup; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_4 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 693, __pyx_L1_error)
      #endif
      if (__pyx_t_4 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4));
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_4);
    #endif
    ++__pyx_t_4;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 693, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":694
 *     idx = 0
 *     for item in tup:
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 idx += ndim - len(tup)
*/
    __pyx_t_2 = (__pyx_v_item == __pyx_builtin_Ellipsis);
    if (__pyx_t_2) {

      /* "View.MemoryView":695
 *     for item in tup:
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 idx += ndim - len(tup)
 *                 seen_ellipsis = True
*/
      __pyx_t_2 = (!__pyx_v_seen_ellipsis);
      if (__pyx_t_2) {

        /* "View.MemoryView":696
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 idx += ndim - len(tup)             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             have_slices = True
*/
        if (unlikely(__pyx_v_tup == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
          __PYX_ERR(1, 696, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(__pyx_v_tup); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 696, __pyx_L1_error)
        __pyx_v_idx = (__pyx_v_idx + (__pyx_v_ndim - __pyx_t_5));

        /* "View.MemoryView":697
 *             if not seen_ellipsis:
 *                 idx += ndim - len(tup)
 *                 seen_ellipsis = True             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
*/
        __pyx_v_seen_ellipsis = 1;

        /* "View.MemoryView":695
 *     for item in tup:
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 idx += ndim - len(tup)
 *                 seen_ellipsis = True
*/
      }

      /* "View.MemoryView":698
 *                 idx += ndim - len(tup)
 *                 seen_ellipsis = True
 *             have_slices = True             # <<<<<<<<<<<<<<
 *         else:
 *             if isinstance(item, slice):
*/
      __pyx_v_have_slices = 1;

      /* "View.MemoryView":694
 *     idx = 0
 *     for item in tup:
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 idx += ndim - len(tup)
*/
      goto __pyx_L5;
    }

    /* "View.MemoryView":700
 *             have_slices = True
 *         else:
 *             if isinstance(item, slice):             # <<<<<<<<<<<<<<
 *                 have_slices = True
 *             elif not PyIndex_Check(item):
*/
    /*else*/ {
      __pyx_t_2 = PySlice_Check(__pyx_v_item); 
      if (__pyx_t_2) {

        /* "View.MemoryView":701
 *         else:
 *             if isinstance(item, slice):
 *                 have_slices = True             # <<<<<<<<<<<<<<
 *             elif not PyIndex_Check(item):
 *                 raise TypeError, f"Cannot index with type '{type(item)}'"
*/
        __pyx_v_have_slices = 1;

        /* "View.MemoryView":700
 *             have_slices = True
 *         else:
 *             if isinstance(item, slice):             # <<<<<<<<<<<<<<
 *                 have_slices = True
 *             elif not PyIndex_Check(item):
*/
        goto __pyx_L7;
      }

      /* "View.MemoryView":702
 *             if isinstance(item, slice):
 *                 have_slices = True
 *             elif not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError, f"Cannot index with type '{type(item)}'"
 *             result[idx] = item
*/
      __pyx_t_2 = (!(PyIndex_Check(__pyx_v_item) != 0));
      if (unlikely(__pyx_t_2)) {

        /* "View.MemoryView":703
 *                 have_slices = True
 *             elif not PyIndex_Check(item):
 *                 raise TypeError, f"Cannot index with type '{type(item)}'"             # <<<<<<<<<<<<<<
 *             result[idx] = item
 *         idx += 1
*/
        __pyx_t_3 = __Pyx_PyObject_FormatSimple(((PyObject *)Py_TYPE(__pyx_v_item)), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 703, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_6[0] = __pyx_mstate_global->__pyx_kp_u_Cannot_index_with_type;
        __pyx_t_6[1] = __pyx_t_3;
        __pyx_t_6[2] = __pyx_mstate_global->__pyx_kp_u__4;
        __pyx_t_7 = __Pyx_PyUnicode_Join(__pyx_t_6, 3, 24 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3));
        if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 703, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_t_7, 0, 0);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __PYX_ERR(1, 703, __pyx_L1_error)

        /* "View.MemoryView":702
 *             if isinstance(item, slice):
 *                 have_slices = True
 *             elif not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError, f"Cannot index with type '{type(item)}'"
 *             result[idx] = item
*/
      }
      __pyx_L7:;

      /* "View.MemoryView":704
 *             elif not PyIndex_Check(item):
 *                 raise TypeError, f"Cannot index with type '{type(item)}'"
 *             result[idx] = item             # <<<<<<<<<<<<<<
 *         idx += 1
 * 
*/
      if (unlikely((__Pyx_SetItemInt(__pyx_v_result, __pyx_v_idx, __pyx_v_item, Py_ssize_t, 1, PyLong_FromSsize_t, 1, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(1, 704, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "View.MemoryView":705
 *                 raise TypeError, f"Cannot index with type '{type(item)}'"
 *             result[idx] = item
 *         idx += 1             # <<<<<<<<<<<<<<
 * 
 *     nslices = ndim - idx
*/
    __pyx_v_idx = (__pyx_v_idx + 1);

    /* "View.MemoryView":693
 *     seen_ellipsis = False
 *     idx = 0
 *     for item in tup:             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":707
 *         idx += 1
 * 
 *     nslices = ndim - idx             # <<<<<<<<<<<<<<
 *     return have_slices or nslices, tuple(result)
 * 
*/
  __pyx_v_nslices = (__pyx_v_ndim - __pyx_v_idx);

  /* "View.MemoryView":708
 * 
 *     nslices = ndim - idx
 *     return have_slices or nslices, tuple(result)             # <<<<<<<<<<<<<<
 * 
 * cdef int assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim) except -1:
*/
  __Pyx_XDECREF(__pyx_r);
  if (!__pyx_v_have_slices) {
  } else {
    __pyx_t_7 = __Pyx_PyBool_FromLong(__pyx_v_have_slices); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_7 = PyLong_FromSsize_t(__pyx_v_nslices); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 708, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __pyx_t_7;
  __pyx_t_7 = 0;
  __pyx_L9_bool_binop_done:;
  __pyx_t_7 = PyList_AsTuple(__pyx_v_result); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 708, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 708, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1) != (0)) __PYX_ERR(1, 708, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_7) != (0)) __PYX_ERR(1, 708, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_7 = 0;
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":681
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("View.MemoryView._unellipsify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tup);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":710
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef int assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim) except -1:             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
*/

static int assert_direct_dimensions(Py_ssize_t *__pyx_v_suboffsets, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_suboffset;
  int __pyx_r;
  Py_ssize_t *__pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":711
 * 
 * cdef int assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim) except -1:
 *     for suboffset in suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             raise ValueError, "Indirect dimensions not supported"
*/
  __pyx_t_2 = (__pyx_v_suboffsets + __pyx_v_ndim);
  for (__pyx_t_3 = __pyx_v_suboffsets; __pyx_t_3 < __pyx_t_2; __pyx_t_3++) {
    __pyx_t_1 = __pyx_t_3;
    __pyx_v_suboffset = (__pyx_t_1[0]);

    /* "View.MemoryView":712
 * cdef int assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim) except -1:
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError, "Indirect dimensions not supported"
 *     return 0  # return type just used as an error flag
*/
    __pyx_t_4 = (__pyx_v_suboffset >= 0);
    if (unlikely(__pyx_t_4)) {

      /* "View.MemoryView":713
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError, "Indirect dimensions not supported"             # <<<<<<<<<<<<<<
 *     return 0  # return type just used as an error flag
 * 
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_mstate_global->__pyx_kp_u_Indirect_dimensions_not_supporte, 0, 0);
      __PYX_ERR(1, 713, __pyx_L1_error)

      /* "View.MemoryView":712
 * cdef int assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim) except -1:
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError, "Indirect dimensions not supported"
 *     return 0  # return type just used as an error flag
*/
    }
  }

  /* "View.MemoryView":714
 *         if suboffset >= 0:
 *             raise ValueError, "Indirect dimensions not supported"
 *     return 0  # return type just used as an error flag             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":710
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef int assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim) except -1:             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView.assert_direct_dimensions", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":720
 * 
 * 
 * @cname('__pyx_memview_slice')             # <<<<<<<<<<<<<<
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
*/

static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *__pyx_v_memview, PyObject *__pyx_v_indices) {
  int __pyx_v_new_ndim;
  int __pyx_v_suboffset_dim;
  int __pyx_v_dim;
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  __Pyx_memviewslice *__pyx_v_p_src;
  struct __pyx_memoryviewslice_obj *__pyx_v_memviewsliceobj = 0;
  __Pyx_memviewslice *__pyx_v_p_dst;
  int *__pyx_v_p_suboffset_dim;
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_stop;
  Py_ssize_t __pyx_v_step;
  Py_ssize_t __pyx_v_cindex;
  int __pyx_v_have_start;
  int __pyx_v_have_stop;
  int __pyx_v_have_step;
  PyObject *__pyx_v_index = NULL;
  struct __pyx_memoryview_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  struct __pyx_memoryview_obj *__pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memview_slice", 0);

  /* "View.MemoryView":722
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim             # <<<<<<<<<<<<<<
 *     cdef bint negative_step
 *     cdef __Pyx_memviewslice src, dst
*/
  __pyx_v_new_ndim = 0;
  __pyx_v_suboffset_dim = -1;

  /* "View.MemoryView":729
 * 
 * 
 *     memset(&dst, 0, sizeof(dst))             # <<<<<<<<<<<<<<
 * 
 *     cdef _memoryviewslice memviewsliceobj
*/
  (void)(memset((&__pyx_v_dst), 0, (sizeof(__pyx_v_dst))));

  /* "View.MemoryView":733
 *     cdef _memoryviewslice memviewsliceobj
 * 
 *     assert memview.view.ndim > 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
*/
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(__pyx_assertions_enabled())) {
    __pyx_t_1 = (__pyx_v_memview->view.ndim > 0);
    if (unlikely(!__pyx_t_1)) {
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
      __PYX_ERR(1, 733, __pyx_L1_error)
    }
  }
  #else
  if ((1)); else __PYX_ERR(1, 733, __pyx_L1_error)
  #endif

  /* "View.MemoryView":735
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_mstate_global->__pyx_memoryviewslice_type); 
  if (__pyx_t_1) {

    /* "View.MemoryView":736
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview             # <<<<<<<<<<<<<<
 *         p_src = &memviewsliceobj.from_slice
 *     else:
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_memoryviewslice_type))))) __PYX_ERR(1, 736, __pyx_L1_error)
    __pyx_v_memviewsliceobj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "View.MemoryView":737
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, &src)
*/
    __pyx_v_p_src = (&__pyx_v_memviewsliceobj->from_slice);

    /* "View.MemoryView":735
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":739
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 *         slice_copy(memview, &src)             # <<<<<<<<<<<<<<
 *         p_src = &src
 * 
*/
  /*else*/ {
    __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_src));

    /* "View.MemoryView":740
 *     else:
 *         slice_copy(memview, &src)
 *         p_src = &src             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_v_p_src = (&__pyx_v_src);
  }
  __pyx_L3:;

  /* "View.MemoryView":746
 * 
 * 
 *     dst.memview = p_src.memview             # <<<<<<<<<<<<<<
 *     dst.data = p_src.data
 * 
*/
  __pyx_t_3 = __pyx_v_p_src->memview;
  __pyx_v_dst.memview = __pyx_t_3;

  /* "View.MemoryView":747
 * 
 *     dst.memview = p_src.memview
 *     dst.data = p_src.data             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = __pyx_v_p_src->data;
  __pyx_v_dst.data = __pyx_t_4;

  /* "View.MemoryView":752
 * 
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst             # <<<<<<<<<<<<<<
 *     cdef int *p_suboffset_dim = &suboffset_dim
 *     cdef Py_ssize_t start, stop, step, cindex
*/
  __pyx_v_p_dst = (&__pyx_v_dst);

  /* "View.MemoryView":753
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst
 *     cdef int *p_suboffset_dim = &suboffset_dim             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t start, stop, step, cindex
 *     cdef bint have_start, have_stop, have_step
*/
  __pyx_v_p_suboffset_dim = (&__pyx_v_suboffset_dim);

  /* "View.MemoryView":757
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             cindex = index
*/
  __pyx_t_5 = 0;
  if (likely(PyList_CheckExact(__pyx_v_indices)) || PyTuple_CheckExact(__pyx_v_indices)) {
    __pyx_t_2 = __pyx_v_indices; __Pyx_INCREF(__pyx_t_2);
    __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_indices); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 757, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 757, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 757, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        __pyx_t_8 = __Pyx_PyList_GetItemRefFast(__pyx_t_2, __pyx_t_6, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_6;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(1, 757, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_8 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6));
        #else
        __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_6);
        #endif
        ++__pyx_t_6;
      }
      if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 757, __pyx_L1_error)
    } else {
      __pyx_t_8 = __pyx_t_7(__pyx_t_2);
      if (unlikely(!__pyx_t_8)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(1, 757, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_v_dim = __pyx_t_5;
    __pyx_t_5 = (__pyx_t_5 + 1);

    /* "View.MemoryView":758
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             cindex = index
 *             slice_memviewslice(
*/
    __pyx_t_1 = (PyIndex_Check(__pyx_v_index) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":759
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):
 *             cindex = index             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
*/
      __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 759, __pyx_L1_error)
      __pyx_v_cindex = __pyx_t_9;

      /* "View.MemoryView":760
 *         if PyIndex_Check(index):
 *             cindex = index
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
*/
      __pyx_t_10 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_v_cindex, 0, 0, 0, 0, 0, 0); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(1, 760, __pyx_L1_error)

      /* "View.MemoryView":758
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             cindex = index
 *             slice_memviewslice(
*/
      goto __pyx_L6;
    }

    /* "View.MemoryView":766
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
*/
    __pyx_t_1 = (__pyx_v_index == Py_None);
    if (__pyx_t_1) {

      /* "View.MemoryView":767
 *                 False)
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1             # <<<<<<<<<<<<<<
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
*/
      (__pyx_v_p_dst->shape[__pyx_v_new_ndim]) = 1;

      /* "View.MemoryView":768
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0             # <<<<<<<<<<<<<<
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1
*/
      (__pyx_v_p_dst->strides[__pyx_v_new_ndim]) = 0;

      /* "View.MemoryView":769
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1             # <<<<<<<<<<<<<<
 *             new_ndim += 1
 *         else:
*/
      (__pyx_v_p_dst->suboffsets[__pyx_v_new_ndim]) = -1L;

      /* "View.MemoryView":770
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 *         else:
 *             start = index.start or 0
*/
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);

      /* "View.MemoryView":766
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
*/
      goto __pyx_L6;
    }

    /* "View.MemoryView":772
 *             new_ndim += 1
 *         else:
 *             start = index.start or 0             # <<<<<<<<<<<<<<
 *             stop = index.stop or 0
 *             step = index.step or 0
*/
    /*else*/ {
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_mstate_global->__pyx_n_u_start); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 772, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(1, 772, __pyx_L1_error)
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __pyx_t_11 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_11 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 772, __pyx_L1_error)
        __pyx_t_9 = __pyx_t_11;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_9 = 0;
      __pyx_L7_bool_binop_done:;
      __pyx_v_start = __pyx_t_9;

      /* "View.MemoryView":773
 *         else:
 *             start = index.start or 0
 *             stop = index.stop or 0             # <<<<<<<<<<<<<<
 *             step = index.step or 0
 * 
*/
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_mstate_global->__pyx_n_u_stop); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 773, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(1, 773, __pyx_L1_error)
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __pyx_t_11 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_11 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 773, __pyx_L1_error)
        __pyx_t_9 = __pyx_t_11;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_9 = 0;
      __pyx_L9_bool_binop_done:;
      __pyx_v_stop = __pyx_t_9;

      /* "View.MemoryView":774
 *             start = index.start or 0
 *             stop = index.stop or 0
 *             step = index.step or 0             # <<<<<<<<<<<<<<
 * 
 *             have_start = index.start is not None
*/
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_mstate_global->__pyx_n_u_step); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 774, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(1, 774, __pyx_L1_error)
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __pyx_t_11 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_11 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 774, __pyx_L1_error)
        __pyx_t_9 = __pyx_t_11;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_9 = 0;
      __pyx_L11_bool_binop_done:;
      __pyx_v_step = __pyx_t_9;

      /* "View.MemoryView":776
 *             step = index.step or 0
 * 
 *             have_start = index.start is not None             # <<<<<<<<<<<<<<
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None
*/
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_mstate_global->__pyx_n_u_start); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 776, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = (__pyx_t_8 != Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_v_have_start = __pyx_t_1;

      /* "View.MemoryView":777
 * 
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None             # <<<<<<<<<<<<<<
 *             have_step = index.step is not None
 * 
*/
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_mstate_global->__pyx_n_u_stop); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 777, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = (__pyx_t_8 != Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_v_have_stop = __pyx_t_1;

      /* "View.MemoryView":778
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None             # <<<<<<<<<<<<<<
 * 
 *             slice_memviewslice(
*/
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_mstate_global->__pyx_n_u_step); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 778, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = (__pyx_t_8 != Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_v_have_step = __pyx_t_1;

      /* "View.MemoryView":780
 *             have_step = index.step is not None
 * 
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
*/
      __pyx_t_10 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_have_start, __pyx_v_have_stop, __pyx_v_have_step, 1); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(1, 780, __pyx_L1_error)

      /* "View.MemoryView":786
 *                 have_start, have_stop, have_step,
 *                 True)
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
*/
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);
    }
    __pyx_L6:;

    /* "View.MemoryView":757
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             cindex = index
*/
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":788
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_mstate_global->__pyx_memoryviewslice_type); 
  if (__pyx_t_1) {

    /* "View.MemoryView":789
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
*/
    __Pyx_XDECREF((PyObject *)__pyx_r);

    /* "View.MemoryView":790
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_dtype_func,
 *                                     memview.dtype_is_object)
*/
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(1, 790, __pyx_L1_error) }

    /* "View.MemoryView":791
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 *     else:
*/
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); __PYX_ERR(1, 791, __pyx_L1_error) }

    /* "View.MemoryView":789
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
*/
    __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, __pyx_v_memviewsliceobj->to_object_func, __pyx_v_memviewsliceobj->to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 789, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_memoryview_type))))) __PYX_ERR(1, 789, __pyx_L1_error)
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":788
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
*/
  }

  /* "View.MemoryView":794
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
*/
  /*else*/ {
    __Pyx_XDECREF((PyObject *)__pyx_r);

    /* "View.MemoryView":795
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,
 *                                     memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, NULL, NULL, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 794, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "View.MemoryView":794
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
*/
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_memoryview_type))))) __PYX_ERR(1, 794, __pyx_L1_error)
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":720
 * 
 * 
 * @cname('__pyx_memview_slice')             # <<<<<<<<<<<<<<
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("View.MemoryView.memview_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_memviewsliceobj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":801
 * 
 * 
 * @cname('__pyx_pybuffer_index')             # <<<<<<<<<<<<<<
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
*/

static char *__pyx_pybuffer_index(Py_buffer *__pyx_v_view, char *__pyx_v_bufp, Py_ssize_t __pyx_v_index, Py_ssize_t __pyx_v_dim) {
  Py_ssize_t __pyx_v_shape;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_suboffset;
  Py_ssize_t __pyx_v_itemsize;
  char *__pyx_v_resultp;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4[3];
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pybuffer_index", 0);

  /* "View.MemoryView":804
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t itemsize = view.itemsize
 *     cdef char *resultp
*/
  __pyx_v_suboffset = -1L;

  /* "View.MemoryView":805
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 *     cdef Py_ssize_t itemsize = view.itemsize             # <<<<<<<<<<<<<<
 *     cdef char *resultp
 * 
*/
  __pyx_t_1 = __pyx_v_view->itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":808
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len // itemsize
 *         stride = itemsize
*/
  __pyx_t_2 = (__pyx_v_view->ndim == 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":809
 * 
 *     if view.ndim == 0:
 *         shape = view.len // itemsize             # <<<<<<<<<<<<<<
 *         stride = itemsize
 *     else:
*/
    if (unlikely(__pyx_v_itemsize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(1, 809, __pyx_L1_error)
    }
    else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(__Pyx_UNARY_NEG_WOULD_OVERFLOW(__pyx_v_view->len))) {
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      __PYX_ERR(1, 809, __pyx_L1_error)
    }
    __pyx_v_shape = __Pyx_div_Py_ssize_t(__pyx_v_view->len, __pyx_v_itemsize, 0);

    /* "View.MemoryView":810
 *     if view.ndim == 0:
 *         shape = view.len // itemsize
 *         stride = itemsize             # <<<<<<<<<<<<<<
 *     else:
 *         shape = view.shape[dim]
*/
    __pyx_v_stride = __pyx_v_itemsize;

    /* "View.MemoryView":808
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len // itemsize
 *         stride = itemsize
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":812
 *         stride = itemsize
 *     else:
 *         shape = view.shape[dim]             # <<<<<<<<<<<<<<
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
*/
  /*else*/ {
    __pyx_v_shape = (__pyx_v_view->shape[__pyx_v_dim]);

    /* "View.MemoryView":813
 *     else:
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]             # <<<<<<<<<<<<<<
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]
*/
    __pyx_v_stride = (__pyx_v_view->strides[__pyx_v_dim]);

    /* "View.MemoryView":814
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
*/
    __pyx_t_2 = (__pyx_v_view->suboffsets != NULL);
    if (__pyx_t_2) {

      /* "View.MemoryView":815
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]             # <<<<<<<<<<<<<<
 * 
 *     if index < 0:
*/
      __pyx_v_suboffset = (__pyx_v_view->suboffsets[__pyx_v_dim]);

      /* "View.MemoryView":814
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
*/
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":817
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
*/
  __pyx_t_2 = (__pyx_v_index < 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":818
 * 
 *     if index < 0:
 *         index += view.shape[dim]             # <<<<<<<<<<<<<<
 *         if index < 0:
 *             raise IndexError, f"Out of bounds on buffer access (axis {dim})"
*/
    __pyx_v_index = (__pyx_v_index + (__pyx_v_view->shape[__pyx_v_dim]));

    /* "View.MemoryView":819
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError, f"Out of bounds on buffer access (axis {dim})"
 * 
*/
    __pyx_t_2 = (__pyx_v_index < 0);
    if (unlikely(__pyx_t_2)) {

      /* "View.MemoryView":820
 *         index += view.shape[dim]
 *         if index < 0:
 *             raise IndexError, f"Out of bounds on buffer access (axis {dim})"             # <<<<<<<<<<<<<<
 * 
 *     if index >= shape:
*/
      __pyx_t_3 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_dim, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 820, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4[0] = __pyx_mstate_global->__pyx_kp_u_Out_of_bounds_on_buffer_access_a;
      __pyx_t_4[1] = __pyx_t_3;
      __pyx_t_4[2] = __pyx_mstate_global->__pyx_kp_u__5;
      __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_4, 3, 37 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127);
      if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 820, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_IndexError))), __pyx_t_5, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(1, 820, __pyx_L1_error)

      /* "View.MemoryView":819
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError, f"Out of bounds on buffer access (axis {dim})"
 * 
*/
    }

    /* "View.MemoryView":817
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
*/
  }

  /* "View.MemoryView":822
 *             raise IndexError, f"Out of bounds on buffer access (axis {dim})"
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError, f"Out of bounds on buffer access (axis {dim})"
 * 
*/
  __pyx_t_2 = (__pyx_v_index >= __pyx_v_shape);
  if (unlikely(__pyx_t_2)) {

    /* "View.MemoryView":823
 * 
 *     if index >= shape:
 *         raise IndexError, f"Out of bounds on buffer access (axis {dim})"             # <<<<<<<<<<<<<<
 * 
 *     resultp = bufp + index * stride
*/
    __pyx_t_5 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_dim, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 823, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4[0] = __pyx_mstate_global->__pyx_kp_u_Out_of_bounds_on_buffer_access_a;
    __pyx_t_4[1] = __pyx_t_5;
    __pyx_t_4[2] = __pyx_mstate_global->__pyx_kp_u__5;
    __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_4, 3, 37 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5) + 1, 127);
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 823, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_IndexError))), __pyx_t_3, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 823, __pyx_L1_error)

    /* "View.MemoryView":822
 *             raise IndexError, f"Out of bounds on buffer access (axis {dim})"
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError, f"Out of bounds on buffer access (axis {dim})"
 * 
*/
  }

  /* "View.MemoryView":825
 *         raise IndexError, f"Out of bounds on buffer access (axis {dim})"
 * 
 *     resultp = bufp + index * stride             # <<<<<<<<<<<<<<
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset
*/
  __pyx_v_resultp = (__pyx_v_bufp + (__pyx_v_index * __pyx_v_stride));

  /* "View.MemoryView":826
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
*/
  __pyx_t_2 = (__pyx_v_suboffset >= 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":827
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset             # <<<<<<<<<<<<<<
 * 
 *     return resultp
*/
    __pyx_v_resultp = ((((char **)__pyx_v_resultp)[0]) + __pyx_v_suboffset);

    /* "View.MemoryView":826
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
*/
  }

  /* "View.MemoryView":829
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 *     return resultp             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_resultp;
  goto __pyx_L0;

  /* "View.MemoryView":801
 * 
 * 
 * @cname('__pyx_pybuffer_index')             # <<<<<<<<<<<<<<
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.pybuffer_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":834
 * 
 * 
 * @cname('__pyx_memslice_transpose')             # <<<<<<<<<<<<<<
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) except -1 nogil:
 *     cdef int ndim = memslice.memview.view.ndim
*/

static int __pyx_memslice_transpose(__Pyx_memviewslice *__pyx_v_memslice) {
  int __pyx_v_ndim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_r;
  int __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  long __pyx_t_3;
  long __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyGILState_STATE __pyx_gilstate_save;

  /* "View.MemoryView":836
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) except -1 nogil:
 *     cdef int ndim = memslice.memview.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
*/
  __pyx_t_1 = __pyx_v_memslice->memview->view.ndim;
  __pyx_v_ndim = __pyx_t_1;

  /* "View.MemoryView":838
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 *     cdef Py_ssize_t *shape = memslice.shape             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t *strides = memslice.strides
 * 
*/
  __pyx_t_2 = __pyx_v_memslice->shape;
  __pyx_v_shape = __pyx_t_2;

  /* "View.MemoryView":839
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 *     cdef Py_ssize_t *strides = memslice.strides             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = __pyx_v_memslice->strides;
  __pyx_v_strides = __pyx_t_2;

  /* "View.MemoryView":843
 * 
 *     cdef int i, j
 *     for i in range(ndim // 2):             # <<<<<<<<<<<<<<
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
*/
  __pyx_t_3 = __Pyx_div_long(__pyx_v_ndim, 2, 1);
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_4; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":844
 *     cdef int i, j
 *     for i in range(ndim // 2):
 *         j = ndim - 1 - i             # <<<<<<<<<<<<<<
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]
*/
    __pyx_v_j = ((__pyx_v_ndim - 1) - __pyx_v_i);

    /* "View.MemoryView":845
 *     for i in range(ndim // 2):
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]             # <<<<<<<<<<<<<<
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
*/
    __pyx_t_5 = (__pyx_v_strides[__pyx_v_j]);
    __pyx_t_6 = (__pyx_v_strides[__pyx_v_i]);
    (__pyx_v_strides[__pyx_v_i]) = __pyx_t_5;
    (__pyx_v_strides[__pyx_v_j]) = __pyx_t_6;

    /* "View.MemoryView":846
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]             # <<<<<<<<<<<<<<
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
*/
    __pyx_t_6 = (__pyx_v_shape[__pyx_v_j]);
    __pyx_t_5 = (__pyx_v_shape[__pyx_v_i]);
    (__pyx_v_shape[__pyx_v_i]) = __pyx_t_6;
    (__pyx_v_shape[__pyx_v_j]) = __pyx_t_5;

    /* "View.MemoryView":848
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(PyExc_ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
*/
    __pyx_t_8 = ((__pyx_v_memslice->suboffsets[__pyx_v_i]) >= 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_8 = ((__pyx_v_memslice->suboffsets[__pyx_v_j]) >= 0);
    __pyx_t_7 = __pyx_t_8;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_7) {

      /* "View.MemoryView":849
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 *             _err(PyExc_ValueError, "Cannot transpose memoryview with indirect dimensions")             # <<<<<<<<<<<<<<
 * 
 *     return 0
*/
      __pyx_t_9 = __pyx_memoryview_err(PyExc_ValueError, __pyx_mstate_global->__pyx_kp_u_Cannot_transpose_memoryview_with); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(1, 849, __pyx_L1_error)

      /* "View.MemoryView":848
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(PyExc_ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
*/
    }
  }

  /* "View.MemoryView":851
 *             _err(PyExc_ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":834
 * 
 * 
 * @cname('__pyx_memslice_transpose')             # <<<<<<<<<<<<<<
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) except -1 nogil:
 *     cdef int ndim = memslice.memview.view.ndim
*/

  /* function exit code */
  __pyx_L1_error:;
  __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  __Pyx_AddTraceback("View.MemoryView.transpose_memslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":869
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XCLEAR_MEMVIEW(&self.from_slice, 1)
 * 
*/

/* Python wrapper */
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {

  /* "View.MemoryView":870
 * 
 *     def __dealloc__(self):
 *         __PYX_XCLEAR_MEMVIEW(&self.from_slice, 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
*/
  __PYX_XCLEAR_MEMVIEW((&__pyx_v_self->from_slice), 1);

  /* "View.MemoryView":869
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XCLEAR_MEMVIEW(&self.from_slice, 1)
 * 
*/

  /* function exit code */
}

/* "View.MemoryView":872
 *         __PYX_XCLEAR_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
*/

static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":873
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
*/
  __pyx_t_1 = (__pyx_v_self->to_object_func != NULL);
  if (__pyx_t_1) {

    /* "View.MemoryView":874
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)             # <<<<<<<<<<<<<<
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_v_self->to_object_func(__pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 874, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":873
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
*/
  }

  /* "View.MemoryView":876
 *             return self.to_object_func(itemp)
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_memoryview_convert_item_to_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 876, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":872
 *         __PYX_XCLEAR_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":878
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
*/

static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":879
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
*/
  __pyx_t_1 = (__pyx_v_self->to_dtype_func != NULL);
  if (__pyx_t_1) {

    /* "View.MemoryView":880
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)             # <<<<<<<<<<<<<<
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)
*/
    __pyx_t_2 = __pyx_v_self->to_dtype_func(__pyx_v_itemp, __pyx_v_value); if (unlikely(__pyx_t_2 == ((int)0))) __PYX_ERR(1, 880, __pyx_L1_error)

    /* "View.MemoryView":879
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":882
 *             self.to_dtype_func(itemp, value)
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_base(self):
*/
  /*else*/ {
    __pyx_t_3 = __pyx_memoryview_assign_item_from_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 882, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":878
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":884
 *             memoryview.assign_item_from_object(self, itemp, value)
 * 
 *     cdef _get_base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
*/

static PyObject *__pyx_memoryviewslice__get_base(struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_base", 0);

  /* "View.MemoryView":885
 * 
 *     cdef _get_base(self):
 *         return self.from_object             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->from_object);
  __pyx_r = __pyx_v_self->from_object;
  goto __pyx_L0;

  /* "View.MemoryView":884
 *             memoryview.assign_item_from_object(self, itemp, value)
 * 
 *     cdef _get_base(self):             # <<<<<<<<<<<<<<
 *         return self.from_object
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_memoryviewslice_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf___pyx_memoryviewslice___reduce_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryviewslice___reduce_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_pw___pyx_memoryviewslice_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf___pyx_memoryviewslice_2__setstate_cython__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf___pyx_memoryviewslice_2__setstate_cython__(CYTHON_UNUSED struct __pyx_memoryviewslice_obj *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":904
 *     pass  # ignore failure, it's a minor issue
 * 
 * @cname('__pyx_memoryview_fromslice')             # <<<<<<<<<<<<<<
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,
 *                           int ndim,
*/

static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice __pyx_v_memviewslice, int __pyx_v_ndim, PyObject *(*__pyx_v_to_object_func)(char *), int (*__pyx_v_to_dtype_func)(char *, PyObject *), int __pyx_v_dtype_is_object) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  Py_ssize_t __pyx_v_suboffset;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_TypeInfo const *__pyx_t_4;
  Py_buffer __pyx_t_5;
  Py_ssize_t *__pyx_t_6;
  Py_ssize_t *__pyx_t_7;
  Py_ssize_t *__pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_fromslice", 0);

  /* "View.MemoryView":913
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_memviewslice.memview) == Py_None);
  if (__pyx_t_1) {

    /* "View.MemoryView":914
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:
 *         return None             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "View.MemoryView":913
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
  }

  /* "View.MemoryView":919
 * 
 * 
 *     result = _memoryviewslice.__new__(_memoryviewslice, None, 0, dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     result.from_slice = memviewslice
*/
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, Py_None) != (0)) __PYX_ERR(1, 919, __pyx_L1_error);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(1, 919, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2) != (0)) __PYX_ERR(1, 919, __pyx_L1_error);
  __pyx_t_2 = 0;
  __pyx_t_2 = ((PyObject *)__pyx_tp_new__memoryviewslice(((PyTypeObject *)__pyx_mstate_global->__pyx_memoryviewslice_type), __pyx_t_3, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 919, __pyx_L1_error)
  __Pyx_GOTREF((PyObject *)__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":921
 *     result = _memoryviewslice.__new__(_memoryviewslice, None, 0, dtype_is_object)
 * 
 *     result.from_slice = memviewslice             # <<<<<<<<<<<<<<
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
*/
  __pyx_v_result->from_slice = __pyx_v_memviewslice;

  /* "View.MemoryView":922
 * 
 *     result.from_slice = memviewslice
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)             # <<<<<<<<<<<<<<
 * 
 *     result.from_object = (<memoryview> memviewslice.memview)._get_base()
*/
  __PYX_INC_MEMVIEW((&__pyx_v_memviewslice), 1);

  /* "View.MemoryView":924
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 *     result.from_object = (<memoryview> memviewslice.memview)._get_base()             # <<<<<<<<<<<<<<
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
*/
  __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)((struct __pyx_memoryview_obj *)__pyx_v_memviewslice.memview)->__pyx_vtab)->_get_base(((struct __pyx_memoryview_obj *)__pyx_v_memviewslice.memview)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 924, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_result->from_object);
  __Pyx_DECREF(__pyx_v_result->from_object);
  __pyx_v_result->from_object = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "View.MemoryView":925
 * 
 *     result.from_object = (<memoryview> memviewslice.memview)._get_base()
 *     result.typeinfo = memviewslice.memview.typeinfo             # <<<<<<<<<<<<<<
 * 
 *     result.view = memviewslice.memview.view
*/
  __pyx_t_4 = __pyx_v_memviewslice.memview->typeinfo;
  __pyx_v_result->__pyx_base.typeinfo = __pyx_t_4;

  /* "View.MemoryView":927
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 *     result.view = memviewslice.memview.view             # <<<<<<<<<<<<<<
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
*/
  __pyx_t_5 = __pyx_v_memviewslice.memview->view;
  __pyx_v_result->__pyx_base.view = __pyx_t_5;

  /* "View.MemoryView":928
 * 
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data             # <<<<<<<<<<<<<<
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
*/
  __pyx_v_result->__pyx_base.view.buf = ((void *)__pyx_v_memviewslice.data);

  /* "View.MemoryView":929
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim             # <<<<<<<<<<<<<<
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)
*/
  __pyx_v_result->__pyx_base.view.ndim = __pyx_v_ndim;

  /* "View.MemoryView":930
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None             # <<<<<<<<<<<<<<
 *     Py_INCREF(Py_None)
 * 
*/
  ((Py_buffer *)(&__pyx_v_result->__pyx_base.view))->obj = Py_None;

  /* "View.MemoryView":931
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
*/
  Py_INCREF(Py_None);

  /* "View.MemoryView":933
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
*/
  __pyx_t_1 = ((((struct __pyx_memoryview_obj *)__pyx_v_memviewslice.memview)->flags & PyBUF_WRITABLE) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":934
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:
 *         result.flags = PyBUF_RECORDS             # <<<<<<<<<<<<<<
 *     else:
 *         result.flags = PyBUF_RECORDS_RO
*/
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS;

    /* "View.MemoryView":933
 *     Py_INCREF(Py_None)
 * 
 *     if (<memoryview>memviewslice.memview).flags & PyBUF_WRITABLE:             # <<<<<<<<<<<<<<
 *         result.flags = PyBUF_RECORDS
 *     else:
*/
    goto __pyx_L4;
  }

  /* "View.MemoryView":936
 *         result.flags = PyBUF_RECORDS
 *     else:
 *         result.flags = PyBUF_RECORDS_RO             # <<<<<<<<<<<<<<
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
*/
  /*else*/ {
    __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS_RO;
  }
  __pyx_L4:;

  /* "View.MemoryView":938
 *         result.flags = PyBUF_RECORDS_RO
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape             # <<<<<<<<<<<<<<
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides
 * 
*/
  __pyx_v_result->__pyx_base.view.shape = ((Py_ssize_t *)__pyx_v_result->from_slice.shape);

  /* "View.MemoryView":939
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_v_result->__pyx_base.view.strides = ((Py_ssize_t *)__pyx_v_result->from_slice.strides);

  /* "View.MemoryView":942
 * 
 * 
 *     result.view.suboffsets = NULL             # <<<<<<<<<<<<<<
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
*/
  __pyx_v_result->__pyx_base.view.suboffsets = NULL;

  /* "View.MemoryView":943
 * 
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
*/
  __pyx_t_7 = (__pyx_v_result->from_slice.suboffsets + __pyx_v_ndim);
  for (__pyx_t_8 = __pyx_v_result->from_slice.suboffsets; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {
    __pyx_t_6 = __pyx_t_8;
    __pyx_v_suboffset = (__pyx_t_6[0]);

    /* "View.MemoryView":944
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
*/
    __pyx_t_1 = (__pyx_v_suboffset >= 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":945
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets             # <<<<<<<<<<<<<<
 *             break
 * 
*/
      __pyx_v_result->__pyx_base.view.suboffsets = ((Py_ssize_t *)__pyx_v_result->from_slice.suboffsets);

      /* "View.MemoryView":946
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break             # <<<<<<<<<<<<<<
 * 
 *     result.view.len = result.view.itemsize
*/
      goto __pyx_L6_break;

      /* "View.MemoryView":944
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
*/
    }
  }
  __pyx_L6_break:;

  /* "View.MemoryView":948
 *             break
 * 
 *     result.view.len = result.view.itemsize             # <<<<<<<<<<<<<<
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length
*/
  __pyx_t_9 = __pyx_v_result->__pyx_base.view.itemsize;
  __pyx_v_result->__pyx_base.view.len = __pyx_t_9;

  /* "View.MemoryView":949
 * 
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         result.view.len *= length
 * 
*/
  __pyx_t_7 = (__pyx_v_result->__pyx_base.view.shape + __pyx_v_ndim);
  for (__pyx_t_8 = __pyx_v_result->__pyx_base.view.shape; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {
    __pyx_t_6 = __pyx_t_8;
    __pyx_t_2 = PyLong_FromSsize_t((__pyx_t_6[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "View.MemoryView":950
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length             # <<<<<<<<<<<<<<
 * 
 *     result.to_object_func = to_object_func
*/
    __pyx_t_2 = PyLong_FromSsize_t(__pyx_v_result->__pyx_base.view.len); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 950, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_InPlaceMultiply(__pyx_t_2, __pyx_v_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 950, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 950, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_result->__pyx_base.view.len = __pyx_t_9;
  }

  /* "View.MemoryView":952
 *         result.view.len *= length
 * 
 *     result.to_object_func = to_object_func             # <<<<<<<<<<<<<<
 *     result.to_dtype_func = to_dtype_func
 * 
*/
  __pyx_v_result->to_object_func = __pyx_v_to_object_func;

  /* "View.MemoryView":953
 * 
 *     result.to_object_func = to_object_func
 *     result.to_dtype_func = to_dtype_func             # <<<<<<<<<<<<<<
 * 
 *     return result
*/
  __pyx_v_result->to_dtype_func = __pyx_v_to_dtype_func;

  /* "View.MemoryView":955
 *     result.to_dtype_func = to_dtype_func
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_result);
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":904
 *     pass  # ignore failure, it's a minor issue
 * 
 * @cname('__pyx_memoryview_fromslice')             # <<<<<<<<<<<<<<
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,
 *                           int ndim,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview_fromslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":957
 *     return result
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')             # <<<<<<<<<<<<<<
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,
 *                                                    __Pyx_memviewslice *mslice) except NULL:
*/

static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_mslice) {
  struct __pyx_memoryviewslice_obj *__pyx_v_obj = 0;
  __Pyx_memviewslice *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_slice_from_memview", 0);

  /* "View.MemoryView":961
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_mstate_global->__pyx_memoryviewslice_type); 
  if (__pyx_t_1) {

    /* "View.MemoryView":962
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview             # <<<<<<<<<<<<<<
 *         return &obj.from_slice
 *     else:
*/
    __pyx_t_2 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_memoryviewslice_type))))) __PYX_ERR(1, 962, __pyx_L1_error)
    __pyx_v_obj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "View.MemoryView":963
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview
 *         return &obj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, mslice)
*/
    __pyx_r = (&__pyx_v_obj->from_slice);
    goto __pyx_L0;

    /* "View.MemoryView":961
 *                                                    __Pyx_memviewslice *mslice) except NULL:
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
*/
  }

  /* "View.MemoryView":965
 *         return &obj.from_slice
 *     else:
 *         slice_copy(memview, mslice)             # <<<<<<<<<<<<<<
 *         return mslice
 * 
*/
  /*else*/ {
    __pyx_memoryview_slice_copy(__pyx_v_memview, __pyx_v_mslice);

    /* "View.MemoryView":966
 *     else:
 *         slice_copy(memview, mslice)
 *         return mslice             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_slice_copy')
*/
    __pyx_r = __pyx_v_mslice;
    goto __pyx_L0;
  }

  /* "View.MemoryView":957
 *     return result
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')             # <<<<<<<<<<<<<<
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,
 *                                                    __Pyx_memviewslice *mslice) except NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.get_slice_from_memview", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":968
 *         return mslice
 * 
 * @cname('__pyx_memoryview_slice_copy')             # <<<<<<<<<<<<<<
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst) noexcept:
 *     cdef int dim
*/

static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_dst) {
  int __pyx_v_dim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  Py_ssize_t *__pyx_v_suboffsets;
  Py_ssize_t *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;

  /* "View.MemoryView":973
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 * 
 *     shape = memview.view.shape             # <<<<<<<<<<<<<<
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets
*/
  __pyx_t_1 = __pyx_v_memview->view.shape;
  __pyx_v_shape = __pyx_t_1;

  /* "View.MemoryView":974
 * 
 *     shape = memview.view.shape
 *     strides = memview.view.strides             # <<<<<<<<<<<<<<
 *     suboffsets = memview.view.suboffsets
 * 
*/
  __pyx_t_1 = __pyx_v_memview->view.strides;
  __pyx_v_strides = __pyx_t_1;

  /* "View.MemoryView":975
 *     shape = memview.view.shape
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets             # <<<<<<<<<<<<<<
 * 
 *     dst.memview = <__pyx_memoryview *> memview
*/
  __pyx_t_1 = __pyx_v_memview->view.suboffsets;
  __pyx_v_suboffsets = __pyx_t_1;

  /* "View.MemoryView":977
 *     suboffsets = memview.view.suboffsets
 * 
 *     dst.memview = <__pyx_memoryview *> memview             # <<<<<<<<<<<<<<
 *     dst.data = <char *> memview.view.buf
 * 
*/
  __pyx_v_dst->memview = ((struct __pyx_memoryview_obj *)__pyx_v_memview);

  /* "View.MemoryView":978
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 *     dst.data = <char *> memview.view.buf             # <<<<<<<<<<<<<<
 * 
 *     for dim in range(memview.view.ndim):
*/
  __pyx_v_dst->data = ((char *)__pyx_v_memview->view.buf);

  /* "View.MemoryView":980
 *     dst.data = <char *> memview.view.buf
 * 
 *     for dim in range(memview.view.ndim):             # <<<<<<<<<<<<<<
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
*/
  __pyx_t_2 = __pyx_v_memview->view.ndim;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_dim = __pyx_t_4;

    /* "View.MemoryView":981
 * 
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]             # <<<<<<<<<<<<<<
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
*/
    (__pyx_v_dst->shape[__pyx_v_dim]) = (__pyx_v_shape[__pyx_v_dim]);

    /* "View.MemoryView":982
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]             # <<<<<<<<<<<<<<
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 * 
*/
    (__pyx_v_dst->strides[__pyx_v_dim]) = (__pyx_v_strides[__pyx_v_dim]);

    /* "View.MemoryView":983
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object')
*/
    __pyx_t_6 = (__pyx_v_suboffsets != 0);
    if (__pyx_t_6) {
      __pyx_t_5 = (__pyx_v_suboffsets[__pyx_v_dim]);
    } else {
      __pyx_t_5 = -1L;
    }
    (__pyx_v_dst->suboffsets[__pyx_v_dim]) = __pyx_t_5;
  }

  /* "View.MemoryView":968
 *         return mslice
 * 
 * @cname('__pyx_memoryview_slice_copy')             # <<<<<<<<<<<<<<
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst) noexcept:
 *     cdef int dim
*/

  /* function exit code */
}

/* "View.MemoryView":985
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 * 
 * @cname('__pyx_memoryview_copy_object')             # <<<<<<<<<<<<<<
 * cdef memoryview_copy(memoryview memview):
 *     "Create a new memoryview object"
*/

static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *__pyx_v_memview) {
  __Pyx_memviewslice __pyx_v_memviewslice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy", 0);

  /* "View.MemoryView":989
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)             # <<<<<<<<<<<<<<
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
*/
  __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_memviewslice));

  /* "View.MemoryView":990
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)
 *     return memoryview_copy_from_slice(memview, &memviewslice)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_copy_object_from_slice(__pyx_v_memview, (&__pyx_v_memviewslice)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 990, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":985
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 * 
 * @cname('__pyx_memoryview_copy_object')             # <<<<<<<<<<<<<<
 * cdef memoryview_copy(memoryview memview):
 *     "Create a new memoryview object"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":992
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')             # <<<<<<<<<<<<<<
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):
 *     """
*/

static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_memviewslice) {
  PyObject *(*__pyx_v_to_object_func)(char *);
  int (*__pyx_v_to_dtype_func)(char *, PyObject *);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *(*__pyx_t_2)(char *);
  int (*__pyx_t_3)(char *, PyObject *);
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy_from_slice", 0);

  /* "View.MemoryView":1000
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
*/
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_mstate_global->__pyx_memoryviewslice_type); 
  if (__pyx_t_1) {

    /* "View.MemoryView":1001
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func             # <<<<<<<<<<<<<<
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
*/
    __pyx_t_2 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_object_func;
    __pyx_v_to_object_func = __pyx_t_2;

    /* "View.MemoryView":1002
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func             # <<<<<<<<<<<<<<
 *     else:
 *         to_object_func = NULL
*/
    __pyx_t_3 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_dtype_func;
    __pyx_v_to_dtype_func = __pyx_t_3;

    /* "View.MemoryView":1000
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":1004
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 *         to_object_func = NULL             # <<<<<<<<<<<<<<
 *         to_dtype_func = NULL
 * 
*/
  /*else*/ {
    __pyx_v_to_object_func = NULL;

    /* "View.MemoryView":1005
 *     else:
 *         to_object_func = NULL
 *         to_dtype_func = NULL             # <<<<<<<<<<<<<<
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
*/
    __pyx_v_to_dtype_func = NULL;
  }
  __pyx_L3:;

  /* "View.MemoryView":1007
 *         to_dtype_func = NULL
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,             # <<<<<<<<<<<<<<
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)
*/
  __Pyx_XDECREF(__pyx_r);

  /* "View.MemoryView":1009
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = __pyx_memoryview_fromslice((__pyx_v_memviewslice[0]), __pyx_v_memview->view.ndim, __pyx_v_to_object_func, __pyx_v_to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1007, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":992
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')             # <<<<<<<<<<<<<<
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):
 *     """
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy_from_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1015
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) noexcept nogil:             # <<<<<<<<<<<<<<
 *     return -arg if arg < 0 else arg
 * 
*/

static Py_ssize_t abs_py_ssize_t(Py_ssize_t __pyx_v_arg) {
  Py_ssize_t __pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;

  /* "View.MemoryView":1016
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) noexcept nogil:
 *     return -arg if arg < 0 else arg             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_get_best_slice_order')
*/
  __pyx_t_2 = (__pyx_v_arg < 0);
  if (__pyx_t_2) {
    __pyx_t_1 = (-__pyx_v_arg);
  } else {
    __pyx_t_1 = __pyx_v_arg;
  }
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "View.MemoryView":1015
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) noexcept nogil:             # <<<<<<<<<<<<<<
 *     return -arg if arg < 0 else arg
 * 
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1018
 *     return -arg if arg < 0 else arg
 * 
 * @cname('__pyx_get_best_slice_order')             # <<<<<<<<<<<<<<
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) noexcept nogil:
 *     """
*/

static char __pyx_get_best_slice_order(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim) {
  int __pyx_v_i;
  Py_ssize_t __pyx_v_c_stride;
  Py_ssize_t __pyx_v_f_stride;
  char __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "View.MemoryView":1024
 *     """
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t f_stride = 0
 * 
*/
  __pyx_v_c_stride = 0;

  /* "View.MemoryView":1025
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0
 *     cdef Py_ssize_t f_stride = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
*/
  __pyx_v_f_stride = 0;

  /* "View.MemoryView":1027
 *     cdef Py_ssize_t f_stride = 0
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
*/
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1028
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
*/
    __pyx_t_2 = ((__pyx_v_mslice->shape[__pyx_v_i]) > 1);
    if (__pyx_t_2) {

      /* "View.MemoryView":1029
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
*/
      __pyx_v_c_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1030
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
*/
      goto __pyx_L4_break;

      /* "View.MemoryView":1028
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
*/
    }
  }
  __pyx_L4_break:;

  /* "View.MemoryView":1032
 *             break
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
*/
  __pyx_t_1 = __pyx_v_ndim;
  __pyx_t_3 = __pyx_t_1;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1033
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
*/
    __pyx_t_2 = ((__pyx_v_mslice->shape[__pyx_v_i]) > 1);
    if (__pyx_t_2) {

      /* "View.MemoryView":1034
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
*/
      __pyx_v_f_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1035
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
*/
      goto __pyx_L7_break;

      /* "View.MemoryView":1033
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
*/
    }
  }
  __pyx_L7_break:;

  /* "View.MemoryView":1037
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
*/
  __pyx_t_2 = (abs_py_ssize_t(__pyx_v_c_stride) <= abs_py_ssize_t(__pyx_v_f_stride));
  if (__pyx_t_2) {

    /* "View.MemoryView":1038
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 *         return 'C'             # <<<<<<<<<<<<<<
 *     else:
 *         return 'F'
*/
    __pyx_r = 'C';
    goto __pyx_L0;

    /* "View.MemoryView":1037
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
*/
  }

  /* "View.MemoryView":1040
 *         return 'C'
 *     else:
 *         return 'F'             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
*/
  /*else*/ {
    __pyx_r = 'F';
    goto __pyx_L0;
  }

  /* "View.MemoryView":1018
 *     return -arg if arg < 0 else arg
 * 
 * @cname('__pyx_get_best_slice_order')             # <<<<<<<<<<<<<<
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) noexcept nogil:
 *     """
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1042
 *         return 'F'
 * 
 * @cython.cdivision(True)             # <<<<<<<<<<<<<<
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,
 *                                    char *dst_data, Py_ssize_t *dst_strides,
*/

static void _copy_strided_to_strided(char *__pyx_v_src_data, Py_ssize_t *__pyx_v_src_strides, char *__pyx_v_dst_data, Py_ssize_t *__pyx_v_dst_strides, Py_ssize_t *__pyx_v_src_shape, Py_ssize_t *__pyx_v_dst_shape, int __pyx_v_ndim, size_t __pyx_v_itemsize) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  CYTHON_UNUSED Py_ssize_t __pyx_v_src_extent;
  Py_ssize_t __pyx_v_dst_extent;
  Py_ssize_t __pyx_v_src_stride;
  Py_ssize_t __pyx_v_dst_stride;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;

  /* "View.MemoryView":1050
 * 
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
*/
  __pyx_v_src_extent = (__pyx_v_src_shape[0]);

  /* "View.MemoryView":1051
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
*/
  __pyx_v_dst_extent = (__pyx_v_dst_shape[0]);

  /* "View.MemoryView":1052
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
*/
  __pyx_v_src_stride = (__pyx_v_src_strides[0]);

  /* "View.MemoryView":1053
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
*/
  __pyx_v_dst_stride = (__pyx_v_dst_strides[0]);

  /* "View.MemoryView":1055
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         if (src_stride > 0 and dst_stride > 0 and
 *                 <size_t> src_stride == itemsize == <size_t> dst_stride):
*/
  __pyx_t_1 = (__pyx_v_ndim == 1);
  if (__pyx_t_1) {

    /* "View.MemoryView":1056
 * 
 *     if ndim == 1:
 *         if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *                 <size_t> src_stride == itemsize == <size_t> dst_stride):
 *             memcpy(dst_data, src_data, itemsize * dst_extent)
*/
    __pyx_t_2 = (__pyx_v_src_stride > 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_dst_stride > 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "View.MemoryView":1057
 *     if ndim == 1:
 *         if (src_stride > 0 and dst_stride > 0 and
 *                 <size_t> src_stride == itemsize == <size_t> dst_stride):             # <<<<<<<<<<<<<<
 *             memcpy(dst_data, src_data, itemsize * dst_extent)
 *         else:
*/
    __pyx_t_2 = (((size_t)__pyx_v_src_stride) == __pyx_v_itemsize);
    if (__pyx_t_2) {
      __pyx_t_2 = (__pyx_v_itemsize == ((size_t)__pyx_v_dst_stride));
    }
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;

    /* "View.MemoryView":1056
 * 
 *     if ndim == 1:
 *         if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *                 <size_t> src_stride == itemsize == <size_t> dst_stride):
 *             memcpy(dst_data, src_data, itemsize * dst_extent)
*/
    if (__pyx_t_1) {

      /* "View.MemoryView":1058
 *         if (src_stride > 0 and dst_stride > 0 and
 *                 <size_t> src_stride == itemsize == <size_t> dst_stride):
 *             memcpy(dst_data, src_data, itemsize * dst_extent)             # <<<<<<<<<<<<<<
 *         else:
 *             for i in range(dst_extent):
*/
      (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, (__pyx_v_itemsize * __pyx_v_dst_extent)));

      /* "View.MemoryView":1056
 * 
 *     if ndim == 1:
 *         if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *                 <size_t> src_stride == itemsize == <size_t> dst_stride):
 *             memcpy(dst_data, src_data, itemsize * dst_extent)
*/
      goto __pyx_L4;
    }

    /* "View.MemoryView":1060
 *             memcpy(dst_data, src_data, itemsize * dst_extent)
 *         else:
 *             for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *                 memcpy(dst_data, src_data, itemsize)
 *                 src_data += src_stride
*/
    /*else*/ {
      __pyx_t_3 = __pyx_v_dst_extent;
      __pyx_t_4 = __pyx_t_3;
      for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
        __pyx_v_i = __pyx_t_5;

        /* "View.MemoryView":1061
 *         else:
 *             for i in range(dst_extent):
 *                 memcpy(dst_data, src_data, itemsize)             # <<<<<<<<<<<<<<
 *                 src_data += src_stride
 *                 dst_data += dst_stride
*/
        (void)(memcpy(__pyx_v_dst_data, __pyx_v_src_data, __pyx_v_itemsize));

        /* "View.MemoryView":1062
 *             for i in range(dst_extent):
 *                 memcpy(dst_data, src_data, itemsize)
 *                 src_data += src_stride             # <<<<<<<<<<<<<<
 *                 dst_data += dst_stride
 *     else:
*/
        __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

        /* "View.MemoryView":1063
 *                 memcpy(dst_data, src_data, itemsize)
 *                 src_data += src_stride
 *                 dst_data += dst_stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(dst_extent):
*/
        __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
      }
    }
    __pyx_L4:;

    /* "View.MemoryView":1055
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         if (src_stride > 0 and dst_stride > 0 and
 *                 <size_t> src_stride == itemsize == <size_t> dst_stride):
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":1065
 *                 dst_data += dst_stride
 *     else:
 *         for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *             _copy_strided_to_strided(src_data, src_strides + 1,
 *                                      dst_data, dst_strides + 1,
*/
  /*else*/ {
    __pyx_t_3 = __pyx_v_dst_extent;
    __pyx_t_4 = __pyx_t_3;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "View.MemoryView":1066
 *     else:
 *         for i in range(dst_extent):
 *             _copy_strided_to_strided(src_data, src_strides + 1,             # <<<<<<<<<<<<<<
 *                                      dst_data, dst_strides + 1,
 *                                      src_shape + 1, dst_shape + 1,
*/
      _copy_strided_to_strided(__pyx_v_src_data, (__pyx_v_src_strides + 1), __pyx_v_dst_data, (__pyx_v_dst_strides + 1), (__pyx_v_src_shape + 1), (__pyx_v_dst_shape + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize);

      /* "View.MemoryView":1070
 *                                      src_shape + 1, dst_shape + 1,
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride             # <<<<<<<<<<<<<<
 *             dst_data += dst_stride
 * 
*/
      __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

      /* "View.MemoryView":1071
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride
 *             dst_data += dst_stride             # <<<<<<<<<<<<<<
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,
*/
      __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1042
 *         return 'F'
 * 
 * @cython.cdivision(True)             # <<<<<<<<<<<<<<
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,
 *                                    char *dst_data, Py_ssize_t *dst_strides,
*/

  /* function exit code */
}

/* "View.MemoryView":1073
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) noexcept nogil:
*/

static void copy_strided_to_strided(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize) {

  /* "View.MemoryView":1076
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) noexcept nogil:
 *     _copy_strided_to_strided(src.data, src.strides, dst.data, dst.strides,             # <<<<<<<<<<<<<<
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
*/
  _copy_strided_to_strided(__pyx_v_src->data, __pyx_v_src->strides, __pyx_v_dst->data, __pyx_v_dst->strides, __pyx_v_src->shape, __pyx_v_dst->shape, __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1073
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) noexcept nogil:
*/

  /* function exit code */
}

/* "View.MemoryView":1079
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
 * @cname('__pyx_memoryview_slice_get_size')             # <<<<<<<<<<<<<<
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) noexcept nogil:
 *     "Return the size of the memory occupied by the slice in number of bytes"
*/

static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *__pyx_v_src, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_shape;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;

  /* "View.MemoryView":1082
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) noexcept nogil:
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     for shape in src.shape[:ndim]:
*/
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_size = __pyx_t_1;

  /* "View.MemoryView":1084
 *     cdef Py_ssize_t shape, size = src.memview.view.itemsize
 * 
 *     for shape in src.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         size *= shape
 * 
*/
  __pyx_t_3 = (__pyx_v_src->shape + __pyx_v_ndim);
  for (__pyx_t_4 = __pyx_v_src->shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {
    __pyx_t_2 = __pyx_t_4;
    __pyx_v_shape = (__pyx_t_2[0]);

    /* "View.MemoryView":1085
 * 
 *     for shape in src.shape[:ndim]:
 *         size *= shape             # <<<<<<<<<<<<<<
 * 
 *     return size
*/
    __pyx_v_size = (__pyx_v_size * __pyx_v_shape);
  }

  /* "View.MemoryView":1087
 *         size *= shape
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_fill_contig_strides_array')
*/
  __pyx_r = __pyx_v_size;
  goto __pyx_L0;

  /* "View.MemoryView":1079
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
 * @cname('__pyx_memoryview_slice_get_size')             # <<<<<<<<<<<<<<
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) noexcept nogil:
 *     "Return the size of the memory occupied by the slice in number of bytes"
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1089
 *     return size
 * 
 * @cname('__pyx_fill_contig_strides_array')             # <<<<<<<<<<<<<<
 * cdef Py_ssize_t fill_contig_strides_array(
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
*/

static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, Py_ssize_t __pyx_v_stride, int __pyx_v_ndim, char __pyx_v_order) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;

  /* "View.MemoryView":1099
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
*/
  __pyx_t_1 = (__pyx_v_order == 'F');
  if (__pyx_t_1) {

    /* "View.MemoryView":1100
 * 
 *     if order == 'F':
 *         for idx in range(ndim):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
*/
    __pyx_t_2 = __pyx_v_ndim;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_idx = __pyx_t_4;

      /* "View.MemoryView":1101
 *     if order == 'F':
 *         for idx in range(ndim):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 *     else:
*/
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1102
 *         for idx in range(ndim):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
*/
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }

    /* "View.MemoryView":1099
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":1104
 *             stride *= shape[idx]
 *     else:
 *         for idx in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride *= shape[idx]
*/
  /*else*/ {
    for (__pyx_t_2 = (__pyx_v_ndim - 1); __pyx_t_2 > -1; __pyx_t_2-=1) {
      __pyx_v_idx = __pyx_t_2;

      /* "View.MemoryView":1105
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride *= shape[idx]
 * 
*/
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1106
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride
 *             stride *= shape[idx]             # <<<<<<<<<<<<<<
 * 
 *     return stride
*/
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1108
 *             stride *= shape[idx]
 * 
 *     return stride             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
*/
  __pyx_r = __pyx_v_stride;
  goto __pyx_L0;

  /* "View.MemoryView":1089
 *     return size
 * 
 * @cname('__pyx_fill_contig_strides_array')             # <<<<<<<<<<<<<<
 * cdef Py_ssize_t fill_contig_strides_array(
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1110
 *     return stride
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')             # <<<<<<<<<<<<<<
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,
 *                              __Pyx_memviewslice *tmpslice,
*/

static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_tmpslice, char __pyx_v_order, int __pyx_v_ndim) {
  int __pyx_v_i;
  void *__pyx_v_result;
  size_t __pyx_v_itemsize;
  size_t __pyx_v_size;
  void *__pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  struct __pyx_memoryview_obj *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyGILState_STATE __pyx_gilstate_save;

  /* "View.MemoryView":1122
 *     cdef void *result
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
*/
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1123
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef size_t size = slice_get_size(src, ndim)             # <<<<<<<<<<<<<<
 * 
 *     result = malloc(size)
*/
  __pyx_v_size = __pyx_memoryview_slice_get_size(__pyx_v_src, __pyx_v_ndim);

  /* "View.MemoryView":1125
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 *     result = malloc(size)             # <<<<<<<<<<<<<<
 *     if not result:
 *         _err_no_memory()
*/
  __pyx_v_result = malloc(__pyx_v_size);

  /* "View.MemoryView":1126
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err_no_memory()
 * 
*/
  __pyx_t_2 = (!(__pyx_v_result != 0));
  if (__pyx_t_2) {

    /* "View.MemoryView":1127
 *     result = malloc(size)
 *     if not result:
 *         _err_no_memory()             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_3 = __pyx_memoryview_err_no_memory(); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(1, 1127, __pyx_L1_error)

    /* "View.MemoryView":1126
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err_no_memory()
 * 
*/
  }

  /* "View.MemoryView":1130
 * 
 * 
 *     tmpslice.data = <char *> result             # <<<<<<<<<<<<<<
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
*/
  __pyx_v_tmpslice->data = ((char *)__pyx_v_result);

  /* "View.MemoryView":1131
 * 
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
*/
  __pyx_t_4 = __pyx_v_src->memview;
  __pyx_v_tmpslice->memview = __pyx_t_4;

  /* "View.MemoryView":1132
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1
*/
  __pyx_t_3 = __pyx_v_ndim;
  __pyx_t_5 = __pyx_t_3;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1133
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]             # <<<<<<<<<<<<<<
 *         tmpslice.suboffsets[i] = -1
 * 
*/
    (__pyx_v_tmpslice->shape[__pyx_v_i]) = (__pyx_v_src->shape[__pyx_v_i]);

    /* "View.MemoryView":1134
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize, ndim, order)
*/
    (__pyx_v_tmpslice->suboffsets[__pyx_v_i]) = -1L;
  }

  /* "View.MemoryView":1136
 *         tmpslice.suboffsets[i] = -1
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize, ndim, order)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  (void)(__pyx_fill_contig_strides_array((&(__pyx_v_tmpslice->shape[0])), (&(__pyx_v_tmpslice->strides[0])), __pyx_v_itemsize, __pyx_v_ndim, __pyx_v_order));

  /* "View.MemoryView":1139
 * 
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0
*/
  __pyx_t_3 = __pyx_v_ndim;
  __pyx_t_5 = __pyx_t_3;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "View.MemoryView":1140
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
*/
    __pyx_t_2 = ((__pyx_v_tmpslice->shape[__pyx_v_i]) == 1);
    if (__pyx_t_2) {

      /* "View.MemoryView":1141
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0             # <<<<<<<<<<<<<<
 * 
 *     if slice_is_contig(src[0], order, ndim):
*/
      (__pyx_v_tmpslice->strides[__pyx_v_i]) = 0;

      /* "View.MemoryView":1140
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
*/
    }
  }

  /* "View.MemoryView":1143
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
*/
  __pyx_t_2 = __pyx_memviewslice_is_contig((__pyx_v_src[0]), __pyx_v_order, __pyx_v_ndim);
  if (__pyx_t_2) {

    /* "View.MemoryView":1144
 * 
 *     if slice_is_contig(src[0], order, ndim):
 *         memcpy(result, src.data, size)             # <<<<<<<<<<<<<<
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
*/
    (void)(memcpy(__pyx_v_result, __pyx_v_src->data, __pyx_v_size));

    /* "View.MemoryView":1143
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src[0], order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
*/
    goto __pyx_L9;
  }

  /* "View.MemoryView":1146
 *         memcpy(result, src.data, size)
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)             # <<<<<<<<<<<<<<
 * 
 *     return result
*/
  /*else*/ {
    copy_strided_to_strided(__pyx_v_src, __pyx_v_tmpslice, __pyx_v_ndim, __pyx_v_itemsize);
  }
  __pyx_L9:;

  /* "View.MemoryView":1148
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":1110
 *     return stride
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')             # <<<<<<<<<<<<<<
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,
 *                              __Pyx_memviewslice *tmpslice,
*/

  /* function exit code */
  __pyx_L1_error:;
  __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  __Pyx_AddTraceback("View.MemoryView.copy_data_to_temp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1152
 * 
 * 
 * @cname('__pyx_memoryview_err_extents')             # <<<<<<<<<<<<<<
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
*/

static int __pyx_memoryview_err_extents(int __pyx_v_i, Py_ssize_t __pyx_v_extent1, Py_ssize_t __pyx_v_extent2) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4[7];
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  __Pyx_RefNannySetupContext("_err_extents", 0);

  /* "View.MemoryView":1155
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError, f"got differing extents in dimension {i} (got {extent1} and {extent2})"             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err_dim')
*/
  __pyx_t_1 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_extent1, 0, ' ', 'd'); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_extent2, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4[0] = __pyx_mstate_global->__pyx_kp_u_got_differing_extents_in_dimensi;
  __pyx_t_4[1] = __pyx_t_1;
  __pyx_t_4[2] = __pyx_mstate_global->__pyx_kp_u_got;
  __pyx_t_4[3] = __pyx_t_2;
  __pyx_t_4[4] = __pyx_mstate_global->__pyx_kp_u_and;
  __pyx_t_4[5] = __pyx_t_3;
  __pyx_t_4[6] = __pyx_mstate_global->__pyx_kp_u__5;
  __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_4, 7, 35 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 5 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127);
  if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 1155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_ValueError))), __pyx_t_5, 0, 0);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __PYX_ERR(1, 1155, __pyx_L1_error)

  /* "View.MemoryView":1152
 * 
 * 
 * @cname('__pyx_memoryview_err_extents')             # <<<<<<<<<<<<<<
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView._err_extents", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  return __pyx_r;
}

/* "View.MemoryView":1157
 *     raise ValueError, f"got differing extents in dimension {i} (got {extent1} and {extent2})"
 * 
 * @cname('__pyx_memoryview_err_dim')             # <<<<<<<<<<<<<<
 * cdef int _err_dim(PyObject *error, str msg, int dim) except -1 with gil:
 *     raise <object>error, msg % dim
*/

static int __pyx_memoryview_err_dim(PyObject *__pyx_v_error, PyObject *__pyx_v_msg, int __pyx_v_dim) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  __Pyx_RefNannySetupContext("_err_dim", 0);
  __Pyx_INCREF(__pyx_v_msg);

  /* "View.MemoryView":1159
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(PyObject *error, str msg, int dim) except -1 with gil:
 *     raise <object>error, msg % dim             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err')
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_dim); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyUnicode_FormatSafe(__pyx_v_msg, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_Raise(((PyObject *)__pyx_v_error), __pyx_t_2, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(1, 1159, __pyx_L1_error)

  /* "View.MemoryView":1157
 *     raise ValueError, f"got differing extents in dimension {i} (got {extent1} and {extent2})"
 * 
 * @cname('__pyx_memoryview_err_dim')             # <<<<<<<<<<<<<<
 * cdef int _err_dim(PyObject *error, str msg, int dim) except -1 with gil:
 *     raise <object>error, msg % dim
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView._err_dim", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_RefNannyFinishContext();
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  return __pyx_r;
}

/* "View.MemoryView":1161
 *     raise <object>error, msg % dim
 * 
 * @cname('__pyx_memoryview_err')             # <<<<<<<<<<<<<<
 * cdef int _err(PyObject *error, str msg) except -1 with gil:
 *     raise <object>error, msg
*/

static int __pyx_memoryview_err(PyObject *__pyx_v_error, PyObject *__pyx_v_msg) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  __Pyx_RefNannySetupContext("_err", 0);
  __Pyx_INCREF(__pyx_v_msg);

  /* "View.MemoryView":1163
 * @cname('__pyx_memoryview_err')
 * cdef int _err(PyObject *error, str msg) except -1 with gil:
 *     raise <object>error, msg             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err_no_memory')
*/
  __Pyx_Raise(((PyObject *)__pyx_v_error), __pyx_v_msg, 0, 0);
  __PYX_ERR(1, 1163, __pyx_L1_error)

  /* "View.MemoryView":1161
 *     raise <object>error, msg % dim
 * 
 * @cname('__pyx_memoryview_err')             # <<<<<<<<<<<<<<
 * cdef int _err(PyObject *error, str msg) except -1 with gil:
 *     raise <object>error, msg
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView._err", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_RefNannyFinishContext();
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  return __pyx_r;
}

/* "View.MemoryView":1165
 *     raise <object>error, msg
 * 
 * @cname('__pyx_memoryview_err_no_memory')             # <<<<<<<<<<<<<<
 * cdef int _err_no_memory() except -1 with gil:
 *     raise MemoryError
*/

static int __pyx_memoryview_err_no_memory(void) {
  int __pyx_r;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();

  /* "View.MemoryView":1167
 * @cname('__pyx_memoryview_err_no_memory')
 * cdef int _err_no_memory() except -1 with gil:
 *     raise MemoryError             # <<<<<<<<<<<<<<
 * 
 * 
*/
  PyErr_NoMemory(); __PYX_ERR(1, 1167, __pyx_L1_error)

  /* "View.MemoryView":1165
 *     raise <object>error, msg
 * 
 * @cname('__pyx_memoryview_err_no_memory')             # <<<<<<<<<<<<<<
 * cdef int _err_no_memory() except -1 with gil:
 *     raise MemoryError
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("View.MemoryView._err_no_memory", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  return __pyx_r;
}

/* "View.MemoryView":1170
 * 
 * 
 * @cname('__pyx_memoryview_copy_contents')             # <<<<<<<<<<<<<<
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,
 *                                   __Pyx_memviewslice dst,
*/

static int __pyx_memoryview_copy_contents(__Pyx_memviewslice __pyx_v_src, __Pyx_memviewslice __pyx_v_dst, int __pyx_v_src_ndim, int __pyx_v_dst_ndim, int __pyx_v_dtype_is_object) {
  void *__pyx_v_tmpdata;
  size_t __pyx_v_itemsize;
  int __pyx_v_i;
  char __pyx_v_order;
  int __pyx_v_broadcasting;
  int __pyx_v_direct_copy;
  __Pyx_memviewslice __pyx_v_tmp;
  int __pyx_v_ndim;
  int __pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  void *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyGILState_STATE __pyx_gilstate_save;

  /* "View.MemoryView":1179
 *     Check for overlapping memory and verify the shapes.
 *     """
 *     cdef void *tmpdata = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
*/
  __pyx_v_tmpdata = NULL;

  /* "View.MemoryView":1180
 *     """
 *     cdef void *tmpdata = NULL
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
*/
  __pyx_t_1 = __pyx_v_src.memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1182
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)             # <<<<<<<<<<<<<<
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False
*/
  __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_src), __pyx_v_src_ndim);

  /* "View.MemoryView":1183
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False             # <<<<<<<<<<<<<<
 *     cdef bint direct_copy = False
 *     cdef __Pyx_memviewslice tmp
*/
  __pyx_v_broadcasting = 0;

  /* "View.MemoryView":1184
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False             # <<<<<<<<<<<<<<
 *     cdef __Pyx_memviewslice tmp
 * 
*/
  __pyx_v_direct_copy = 0;

  /* "View.MemoryView":1187
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
*/
  __pyx_t_2 = (__pyx_v_src_ndim < __pyx_v_dst_ndim);
  if (__pyx_t_2) {

    /* "View.MemoryView":1188
 * 
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
*/
    __pyx_memoryview_broadcast_leading((&__pyx_v_src), __pyx_v_src_ndim, __pyx_v_dst_ndim);

    /* "View.MemoryView":1187
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":1189
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
*/
  __pyx_t_2 = (__pyx_v_dst_ndim < __pyx_v_src_ndim);
  if (__pyx_t_2) {

    /* "View.MemoryView":1190
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)             # <<<<<<<<<<<<<<
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)
*/
    __pyx_memoryview_broadcast_leading((&__pyx_v_dst), __pyx_v_dst_ndim, __pyx_v_src_ndim);

    /* "View.MemoryView":1189
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
*/
  }
  __pyx_L3:;

  /* "View.MemoryView":1192
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
*/
  __pyx_t_3 = __pyx_v_dst_ndim;
  __pyx_t_4 = __pyx_v_src_ndim;
  __pyx_t_2 = (__pyx_t_3 > __pyx_t_4);
  if (__pyx_t_2) {
    __pyx_t_5 = __pyx_t_3;
  } else {
    __pyx_t_5 = __pyx_t_4;
  }
  __pyx_v_ndim = __pyx_t_5;

  /* "View.MemoryView":1194
 *     cdef int ndim = max(src_ndim, dst_ndim)
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
*/
  __pyx_t_5 = __pyx_v_ndim;
  __pyx_t_3 = __pyx_t_5;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "View.MemoryView":1195
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
*/
    __pyx_t_2 = ((__pyx_v_src.shape[__pyx_v_i]) != (__pyx_v_dst.shape[__pyx_v_i]));
    if (__pyx_t_2) {

      /* "View.MemoryView":1196
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
*/
      __pyx_t_2 = ((__pyx_v_src.shape[__pyx_v_i]) == 1);
      if (__pyx_t_2) {

        /* "View.MemoryView":1197
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 *                 broadcasting = True             # <<<<<<<<<<<<<<
 *                 src.strides[i] = 0
 *             else:
*/
        __pyx_v_broadcasting = 1;

        /* "View.MemoryView":1198
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 *                 src.strides[i] = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])
*/
        (__pyx_v_src.strides[__pyx_v_i]) = 0;

        /* "View.MemoryView":1196
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
*/
        goto __pyx_L7;
      }

      /* "View.MemoryView":1200
 *                 src.strides[i] = 0
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])             # <<<<<<<<<<<<<<
 * 
 *         if src.suboffsets[i] >= 0:
*/
      /*else*/ {
        __pyx_t_6 = __pyx_memoryview_err_extents(__pyx_v_i, (__pyx_v_dst.shape[__pyx_v_i]), (__pyx_v_src.shape[__pyx_v_i])); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 1200, __pyx_L1_error)
      }
      __pyx_L7:;

      /* "View.MemoryView":1195
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
*/
    }

    /* "View.MemoryView":1202
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(PyExc_ValueError, "Dimension %d is not direct", i)
 * 
*/
    __pyx_t_2 = ((__pyx_v_src.suboffsets[__pyx_v_i]) >= 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1203
 * 
 *         if src.suboffsets[i] >= 0:
 *             _err_dim(PyExc_ValueError, "Dimension %d is not direct", i)             # <<<<<<<<<<<<<<
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):
*/
      __pyx_t_6 = __pyx_memoryview_err_dim(PyExc_ValueError, __pyx_mstate_global->__pyx_kp_u_Dimension_d_is_not_direct, __pyx_v_i); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 1203, __pyx_L1_error)

      /* "View.MemoryView":1202
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(PyExc_ValueError, "Dimension %d is not direct", i)
 * 
*/
    }
  }

  /* "View.MemoryView":1205
 *             _err_dim(PyExc_ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
*/
  __pyx_t_2 = __pyx_slices_overlap((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize);
  if (__pyx_t_2) {

    /* "View.MemoryView":1207
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
*/
    __pyx_t_2 = (!__pyx_memviewslice_is_contig(__pyx_v_src, __pyx_v_order, __pyx_v_ndim));
    if (__pyx_t_2) {

      /* "View.MemoryView":1208
 * 
 *         if not slice_is_contig(src, order, ndim):
 *             order = get_best_order(&dst, ndim)             # <<<<<<<<<<<<<<
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
*/
      __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim);

      /* "View.MemoryView":1207
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
*/
    }

    /* "View.MemoryView":1210
 *             order = get_best_order(&dst, ndim)
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)             # <<<<<<<<<<<<<<
 *         src = tmp
 * 
*/
    __pyx_t_7 = __pyx_memoryview_copy_data_to_temp((&__pyx_v_src), (&__pyx_v_tmp), __pyx_v_order, __pyx_v_ndim); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(1, 1210, __pyx_L1_error)
    __pyx_v_tmpdata = __pyx_t_7;

    /* "View.MemoryView":1211
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 *         src = tmp             # <<<<<<<<<<<<<<
 * 
 *     if not broadcasting:
*/
    __pyx_v_src = __pyx_v_tmp;

    /* "View.MemoryView":1205
 *             _err_dim(PyExc_ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(src, order, ndim):
*/
  }

  /* "View.MemoryView":1213
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = (!__pyx_v_broadcasting);
  if (__pyx_t_2) {

    /* "View.MemoryView":1216
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
*/
    __pyx_t_2 = __pyx_memviewslice_is_contig(__pyx_v_src, 'C', __pyx_v_ndim);
    if (__pyx_t_2) {

      /* "View.MemoryView":1217
 * 
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)             # <<<<<<<<<<<<<<
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
*/
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'C', __pyx_v_ndim);

      /* "View.MemoryView":1216
 * 
 * 
 *         if slice_is_contig(src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
*/
      goto __pyx_L12;
    }

    /* "View.MemoryView":1218
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
*/
    __pyx_t_2 = __pyx_memviewslice_is_contig(__pyx_v_src, 'F', __pyx_v_ndim);
    if (__pyx_t_2) {

      /* "View.MemoryView":1219
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):
 *             direct_copy = slice_is_contig(dst, 'F', ndim)             # <<<<<<<<<<<<<<
 * 
 *         if direct_copy:
*/
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig(__pyx_v_dst, 'F', __pyx_v_ndim);

      /* "View.MemoryView":1218
 *         if slice_is_contig(src, 'C', ndim):
 *             direct_copy = slice_is_contig(dst, 'C', ndim)
 *         elif slice_is_contig(src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
*/
    }
    __pyx_L12:;

    /* "View.MemoryView":1221
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=False)
*/
    if (__pyx_v_direct_copy) {

      /* "View.MemoryView":1223
 *         if direct_copy:
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=False)             # <<<<<<<<<<<<<<
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=True)
*/
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

      /* "View.MemoryView":1224
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))             # <<<<<<<<<<<<<<
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=True)
 *             free(tmpdata)
*/
      (void)(memcpy(__pyx_v_dst.data, __pyx_v_src.data, __pyx_memoryview_slice_get_size((&__pyx_v_src), __pyx_v_ndim)));

      /* "View.MemoryView":1225
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=True)             # <<<<<<<<<<<<<<
 *             free(tmpdata)
 *             return 0
*/
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

      /* "View.MemoryView":1226
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=True)
 *             free(tmpdata)             # <<<<<<<<<<<<<<
 *             return 0
 * 
*/
      free(__pyx_v_tmpdata);

      /* "View.MemoryView":1227
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=True)
 *             free(tmpdata)
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):
*/
      __pyx_r = 0;
      goto __pyx_L0;

      /* "View.MemoryView":1221
 *             direct_copy = slice_is_contig(dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, inc=False)
*/
    }

    /* "View.MemoryView":1213
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
*/
  }

  /* "View.MemoryView":1229
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = (__pyx_v_order == 'F');
  if (__pyx_t_2) {
    __pyx_t_2 = ('F' == __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim));
  }
  if (__pyx_t_2) {

    /* "View.MemoryView":1232
 * 
 * 
 *         transpose_memslice(&src)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&dst)
 * 
*/
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_src)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 1232, __pyx_L1_error)

    /* "View.MemoryView":1233
 * 
 *         transpose_memslice(&src)
 *         transpose_memslice(&dst)             # <<<<<<<<<<<<<<
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=False)
*/
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_dst)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 1233, __pyx_L1_error)

    /* "View.MemoryView":1229
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
*/
  }

  /* "View.MemoryView":1235
 *         transpose_memslice(&dst)
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=False)             # <<<<<<<<<<<<<<
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=True)
*/
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1236
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)             # <<<<<<<<<<<<<<
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=True)
 * 
*/
  copy_strided_to_strided((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1237
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=True)             # <<<<<<<<<<<<<<
 * 
 *     free(tmpdata)
*/
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1239
 *     refcount_copying(&dst, dtype_is_object, ndim, inc=True)
 * 
 *     free(tmpdata)             # <<<<<<<<<<<<<<
 *     return 0
 * 
*/
  free(__pyx_v_tmpdata);

  /* "View.MemoryView":1240
 * 
 *     free(tmpdata)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1170
 * 
 * 
 * @cname('__pyx_memoryview_copy_contents')             # <<<<<<<<<<<<<<
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,
 *                                   __Pyx_memviewslice dst,
*/

  /* function exit code */
  __pyx_L1_error:;
  __pyx_gilstate_save = __Pyx_PyGILState_Ensure();
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy_contents", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1242
 *     return 0
 * 
 * @cname('__pyx_memoryview_broadcast_leading')             # <<<<<<<<<<<<<<
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,
 *                             int ndim,
*/

static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim, int __pyx_v_ndim_other) {
  int __pyx_v_i;
  int __pyx_v_offset;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1247
 *                             int ndim_other) noexcept nogil:
 *     cdef int i
 *     cdef int offset = ndim_other - ndim             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
*/
  __pyx_v_offset = (__pyx_v_ndim_other - __pyx_v_ndim);

  /* "View.MemoryView":1249
 *     cdef int offset = ndim_other - ndim
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
*/
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1250
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]             # <<<<<<<<<<<<<<
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
*/
    (__pyx_v_mslice->shape[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->shape[__pyx_v_i]);

    /* "View.MemoryView":1251
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
*/
    (__pyx_v_mslice->strides[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->strides[__pyx_v_i]);

    /* "View.MemoryView":1252
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(offset):
*/
    (__pyx_v_mslice->suboffsets[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->suboffsets[__pyx_v_i]);
  }

  /* "View.MemoryView":1254
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 *     for i in range(offset):             # <<<<<<<<<<<<<<
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
*/
  __pyx_t_1 = __pyx_v_offset;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1255
 * 
 *     for i in range(offset):
 *         mslice.shape[i] = 1             # <<<<<<<<<<<<<<
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1
*/
    (__pyx_v_mslice->shape[__pyx_v_i]) = 1;

    /* "View.MemoryView":1256
 *     for i in range(offset):
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i] = -1
 * 
*/
    (__pyx_v_mslice->strides[__pyx_v_i]) = (__pyx_v_mslice->strides[0]);

    /* "View.MemoryView":1257
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 * 
*/
    (__pyx_v_mslice->suboffsets[__pyx_v_i]) = -1L;
  }

  /* "View.MemoryView":1242
 *     return 0
 * 
 * @cname('__pyx_memoryview_broadcast_leading')             # <<<<<<<<<<<<<<
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,
 *                             int ndim,
*/

  /* function exit code */
}

/* "View.MemoryView":1264
 * 
 * 
 * @cname('__pyx_memoryview_refcount_copying')             # <<<<<<<<<<<<<<
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object, int ndim, bint inc) noexcept nogil:
 * 
*/

static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_dtype_is_object, int __pyx_v_ndim, int __pyx_v_inc) {

  /* "View.MemoryView":1267
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object, int ndim, bint inc) noexcept nogil:
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape, dst.strides, ndim, inc)
 * 
*/
  if (__pyx_v_dtype_is_object) {

    /* "View.MemoryView":1268
 * 
 *     if dtype_is_object:
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape, dst.strides, ndim, inc)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
*/
    __pyx_memoryview_refcount_objects_in_slice_with_gil(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_inc);

    /* "View.MemoryView":1267
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object, int ndim, bint inc) noexcept nogil:
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape, dst.strides, ndim, inc)
 * 
*/
  }

  /* "View.MemoryView":1264
 * 
 * 
 * @cname('__pyx_memoryview_refcount_copying')             # <<<<<<<<<<<<<<
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object, int ndim, bint inc) noexcept nogil:
 * 
*/

  /* function exit code */
}

/* "View.MemoryView":1270
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape, dst.strides, ndim, inc)
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')             # <<<<<<<<<<<<<<
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,
 *                                              Py_ssize_t *strides, int ndim,
*/

static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  PyGILState_STATE __pyx_gilstate_save = __Pyx_PyGILState_Ensure();

  /* "View.MemoryView":1274
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) noexcept with gil:
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
*/
  __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, __pyx_v_shape, __pyx_v_strides, __pyx_v_ndim, __pyx_v_inc);

  /* "View.MemoryView":1270
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape, dst.strides, ndim, inc)
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')             # <<<<<<<<<<<<<<
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,
 *                                              Py_ssize_t *strides, int ndim,
*/

  /* function exit code */
  __Pyx_PyGILState_Release(__pyx_gilstate_save);
}

/* "View.MemoryView":1276
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')             # <<<<<<<<<<<<<<
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,
 *                                     Py_ssize_t *strides, int ndim, bint inc) noexcept:
*/

static void __pyx_memoryview_refcount_objects_in_slice(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;

  /* "View.MemoryView":1280
 *                                     Py_ssize_t *strides, int ndim, bint inc) noexcept:
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(shape[0]):
*/
  __pyx_v_stride = (__pyx_v_strides[0]);

  /* "View.MemoryView":1282
 *     cdef Py_ssize_t stride = strides[0]
 * 
 *     for i in range(shape[0]):             # <<<<<<<<<<<<<<
 *         if ndim == 1:
 *             if inc:
*/
  __pyx_t_1 = (__pyx_v_shape[0]);
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1283
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
*/
    __pyx_t_4 = (__pyx_v_ndim == 1);
    if (__pyx_t_4) {

      /* "View.MemoryView":1284
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
*/
      if (__pyx_v_inc) {

        /* "View.MemoryView":1285
 *         if ndim == 1:
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])
*/
        Py_INCREF((((PyObject **)__pyx_v_data)[0]));

        /* "View.MemoryView":1284
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
*/
        goto __pyx_L6;
      }

      /* "View.MemoryView":1287
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1, ndim - 1, inc)
*/
      /*else*/ {
        Py_DECREF((((PyObject **)__pyx_v_data)[0]));
      }
      __pyx_L6:;

      /* "View.MemoryView":1283
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
*/
      goto __pyx_L5;
    }

    /* "View.MemoryView":1289
 *                 Py_DECREF((<PyObject **> data)[0])
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1, ndim - 1, inc)             # <<<<<<<<<<<<<<
 * 
 *         data += stride
*/
    /*else*/ {
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_inc);
    }
    __pyx_L5:;

    /* "View.MemoryView":1291
 *             refcount_objects_in_slice(data, shape + 1, strides + 1, ndim - 1, inc)
 * 
 *         data += stride             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
  }

  /* "View.MemoryView":1276
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')             # <<<<<<<<<<<<<<
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,
 *                                     Py_ssize_t *strides, int ndim, bint inc) noexcept:
*/

  /* function exit code */
}

/* "View.MemoryView":1296
 * 
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')             # <<<<<<<<<<<<<<
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,
 *                               size_t itemsize, void *item,
*/

static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item, int __pyx_v_dtype_is_object) {

  /* "View.MemoryView":1300
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) noexcept nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, inc=False)             # <<<<<<<<<<<<<<
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim, itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, inc=True)
*/
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1301
 *                               bint dtype_is_object) noexcept nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, inc=False)
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim, itemsize, item)             # <<<<<<<<<<<<<<
 *     refcount_copying(dst, dtype_is_object, ndim, inc=True)
 * 
*/
  __pyx_memoryview__slice_assign_scalar(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_itemsize, __pyx_v_item);

  /* "View.MemoryView":1302
 *     refcount_copying(dst, dtype_is_object, ndim, inc=False)
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim, itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, inc=True)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1296
 * 
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')             # <<<<<<<<<<<<<<
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,
 *                               size_t itemsize, void *item,
*/

  /* function exit code */
}

/* "View.MemoryView":1305
 * 
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')             # <<<<<<<<<<<<<<
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,
 *                               Py_ssize_t *strides, int ndim,
*/

static void __pyx_memoryview__slice_assign_scalar(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_extent;
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "View.MemoryView":1310
 *                               size_t itemsize, void *item) noexcept nogil:
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t extent = shape[0]
 * 
*/
  __pyx_v_stride = (__pyx_v_strides[0]);

  /* "View.MemoryView":1311
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]
 *     cdef Py_ssize_t extent = shape[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
*/
  __pyx_v_extent = (__pyx_v_shape[0]);

  /* "View.MemoryView":1313
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
*/
  __pyx_t_1 = (__pyx_v_ndim == 1);
  if (__pyx_t_1) {

    /* "View.MemoryView":1314
 * 
 *     if ndim == 1:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             memcpy(data, item, itemsize)
 *             data += stride
*/
    __pyx_t_2 = __pyx_v_extent;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1315
 *     if ndim == 1:
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)             # <<<<<<<<<<<<<<
 *             data += stride
 *     else:
*/
      (void)(memcpy(__pyx_v_data, __pyx_v_item, __pyx_v_itemsize));

      /* "View.MemoryView":1316
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 *             data += stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(extent):
*/
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }

    /* "View.MemoryView":1313
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
*/
    goto __pyx_L3;
  }

  /* "View.MemoryView":1318
 *             data += stride
 *     else:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             _slice_assign_scalar(data, shape + 1, strides + 1, ndim - 1, itemsize, item)
 *             data += stride
*/
  /*else*/ {
    __pyx_t_2 = __pyx_v_extent;
    __pyx_t_3 = __pyx_t_2;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "View.MemoryView":1319
 *     else:
 *         for i in range(extent):
 *             _slice_assign_scalar(data, shape + 1, strides + 1, ndim - 1, itemsize, item)             # <<<<<<<<<<<<<<
 *             data += stride
 * 
*/
      __pyx_memoryview__slice_assign_scalar(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize, __pyx_v_item);

      /* "View.MemoryView":1320
 *         for i in range(extent):
 *             _slice_assign_scalar(data, shape + 1, strides + 1, ndim - 1, itemsize, item)
 *             data += stride             # <<<<<<<<<<<<<<
 * 
*/
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1305
 * 
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')             # <<<<<<<<<<<<<<
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,
 *                               Py_ssize_t *strides, int ndim,
*/

  /* function exit code */
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
*/

/* Python wrapper */
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum = {"__pyx_unpickle_Enum", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_15View_dot_MemoryView_1__pyx_unpickle_Enum(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_Enum", 0) < (0)) __PYX_ERR(1, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, i); __PYX_ERR(1, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Enum", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(1, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView___pyx_unpickle_Enum(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')             # <<<<<<<<<<<<<<
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, __pyx_k_name); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
 *     __pyx_result = Enum.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_MemviewEnum_type);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(1, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_unpickle_Enum__set_state(((struct __pyx_MemviewEnum_obj *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
 *     __pyx_result = Enum.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)
*/

static PyObject *__pyx_unpickle_Enum__set_state(struct __pyx_MemviewEnum_obj *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Enum__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name);
  __Pyx_DECREF(__pyx_v___pyx_result->name);
  __pyx_v___pyx_result->name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, __pyx_state: tuple):
 *     __pyx_result.name = __pyx_state[0]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_Enum__set_state(<Enum> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Enum__set_state(Enum __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.name = __pyx_state[0]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 1)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.__pyx_unpickle_Enum__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":129
 * 
 * 
 * cdef int start_transaction(OGRDataSourceH ogr_dataset, int force) except 1:             # <<<<<<<<<<<<<<
 *     """Start a transaction on the dataset.
 * 
*/

static int __pyx_f_7pyogrio_3_io_start_transaction(OGRDataSourceH __pyx_v_ogr_dataset, int __pyx_v_force) {
  int __pyx_v_err;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("start_transaction", 0);

  /* "pyogrio/_io.pyx":140
 * 
 *     """
 *     cdef int err = GDALDatasetStartTransaction(ogr_dataset, force)             # <<<<<<<<<<<<<<
 *     if err == OGRERR_FAILURE:
 *         raise DataSourceError("Failed to start transaction")
*/
  __pyx_v_err = GDALDatasetStartTransaction(__pyx_v_ogr_dataset, __pyx_v_force);

  /* "pyogrio/_io.pyx":141
 *     """
 *     cdef int err = GDALDatasetStartTransaction(ogr_dataset, force)
 *     if err == OGRERR_FAILURE:             # <<<<<<<<<<<<<<
 *         raise DataSourceError("Failed to start transaction")
 * 
*/
  __pyx_t_1 = (__pyx_v_err == OGRERR_FAILURE);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":142
 *     cdef int err = GDALDatasetStartTransaction(ogr_dataset, force)
 *     if err == OGRERR_FAILURE:
 *         raise DataSourceError("Failed to start transaction")             # <<<<<<<<<<<<<<
 * 
 *     return 0
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Failed_to_start_transaction};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 142, __pyx_L1_error)

    /* "pyogrio/_io.pyx":141
 *     """
 *     cdef int err = GDALDatasetStartTransaction(ogr_dataset, force)
 *     if err == OGRERR_FAILURE:             # <<<<<<<<<<<<<<
 *         raise DataSourceError("Failed to start transaction")
 * 
*/
  }

  /* "pyogrio/_io.pyx":144
 *         raise DataSourceError("Failed to start transaction")
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":129
 * 
 * 
 * cdef int start_transaction(OGRDataSourceH ogr_dataset, int force) except 1:             # <<<<<<<<<<<<<<
 *     """Start a transaction on the dataset.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyogrio._io.start_transaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":147
 * 
 * 
 * cdef int commit_transaction(OGRDataSourceH ogr_dataset) except 1:             # <<<<<<<<<<<<<<
 *     """Commit a transaction on the dataset.
 * 
*/

static int __pyx_f_7pyogrio_3_io_commit_transaction(OGRDataSourceH __pyx_v_ogr_dataset) {
  int __pyx_v_err;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("commit_transaction", 0);

  /* "pyogrio/_io.pyx":156
 * 
 *     """
 *     cdef int err = GDALDatasetCommitTransaction(ogr_dataset)             # <<<<<<<<<<<<<<
 *     if err == OGRERR_FAILURE:
 *         raise DataSourceError("Failed to commit transaction")
*/
  __pyx_v_err = GDALDatasetCommitTransaction(__pyx_v_ogr_dataset);

  /* "pyogrio/_io.pyx":157
 *     """
 *     cdef int err = GDALDatasetCommitTransaction(ogr_dataset)
 *     if err == OGRERR_FAILURE:             # <<<<<<<<<<<<<<
 *         raise DataSourceError("Failed to commit transaction")
 * 
*/
  __pyx_t_1 = (__pyx_v_err == OGRERR_FAILURE);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":158
 *     cdef int err = GDALDatasetCommitTransaction(ogr_dataset)
 *     if err == OGRERR_FAILURE:
 *         raise DataSourceError("Failed to commit transaction")             # <<<<<<<<<<<<<<
 * 
 *     return 0
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Failed_to_commit_transaction};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 158, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 158, __pyx_L1_error)

    /* "pyogrio/_io.pyx":157
 *     """
 *     cdef int err = GDALDatasetCommitTransaction(ogr_dataset)
 *     if err == OGRERR_FAILURE:             # <<<<<<<<<<<<<<
 *         raise DataSourceError("Failed to commit transaction")
 * 
*/
  }

  /* "pyogrio/_io.pyx":160
 *         raise DataSourceError("Failed to commit transaction")
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":147
 * 
 * 
 * cdef int commit_transaction(OGRDataSourceH ogr_dataset) except 1:             # <<<<<<<<<<<<<<
 *     """Commit a transaction on the dataset.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyogrio._io.commit_transaction", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":172
 * 
 * 
 * cdef char** dict_to_options(object values):             # <<<<<<<<<<<<<<
 *     """Convert a python dictionary into name / value pairs (stored in a char**).
 * 
*/

static char **__pyx_f_7pyogrio_3_io_dict_to_options(PyObject *__pyx_v_values) {
  char **__pyx_v_options;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  char **__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  size_t __pyx_t_9;
  char const *__pyx_t_10;
  char const *__pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("dict_to_options", 0);

  /* "pyogrio/_io.pyx":186
 * 
 *     """
 *     cdef char **options = NULL             # <<<<<<<<<<<<<<
 * 
 *     if values is None:
*/
  __pyx_v_options = NULL;

  /* "pyogrio/_io.pyx":188
 *     cdef char **options = NULL
 * 
 *     if values is None:             # <<<<<<<<<<<<<<
 *         return NULL
 * 
*/
  __pyx_t_1 = (__pyx_v_values == Py_None);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":189
 * 
 *     if values is None:
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 *     for k, v in values.items():
*/
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":188
 *     cdef char **options = NULL
 * 
 *     if values is None:             # <<<<<<<<<<<<<<
 *         return NULL
 * 
*/
  }

  /* "pyogrio/_io.pyx":191
 *         return NULL
 * 
 *     for k, v in values.items():             # <<<<<<<<<<<<<<
 *         k = k.encode("UTF-8")
 *         v = v.encode("UTF-8")
*/
  __pyx_t_3 = 0;
  if (unlikely(__pyx_v_values == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
    __PYX_ERR(0, 191, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_dict_iterator(__pyx_v_values, 0, __pyx_mstate_global->__pyx_n_u_items, (&__pyx_t_4), (&__pyx_t_5)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_2);
  __pyx_t_2 = __pyx_t_6;
  __pyx_t_6 = 0;
  while (1) {
    __pyx_t_8 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_4, &__pyx_t_3, &__pyx_t_6, &__pyx_t_7, NULL, __pyx_t_5);
    if (unlikely(__pyx_t_8 == 0)) break;
    if (unlikely(__pyx_t_8 == -1)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":192
 * 
 *     for k, v in values.items():
 *         k = k.encode("UTF-8")             # <<<<<<<<<<<<<<
 *         v = v.encode("UTF-8")
 *         options = CSLAddNameValue(options, <const char *>k, <const char *>v)
*/
    __pyx_t_6 = __pyx_v_k;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_9 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 192, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_DECREF_SET(__pyx_v_k, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":193
 *     for k, v in values.items():
 *         k = k.encode("UTF-8")
 *         v = v.encode("UTF-8")             # <<<<<<<<<<<<<<
 *         options = CSLAddNameValue(options, <const char *>k, <const char *>v)
 * 
*/
    __pyx_t_6 = __pyx_v_v;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_9 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":194
 *         k = k.encode("UTF-8")
 *         v = v.encode("UTF-8")
 *         options = CSLAddNameValue(options, <const char *>k, <const char *>v)             # <<<<<<<<<<<<<<
 * 
 *     return options
*/
    __pyx_t_10 = __Pyx_PyObject_AsString(__pyx_v_k); if (unlikely((!__pyx_t_10) && PyErr_Occurred())) __PYX_ERR(0, 194, __pyx_L1_error)
    __pyx_t_11 = __Pyx_PyObject_AsString(__pyx_v_v); if (unlikely((!__pyx_t_11) && PyErr_Occurred())) __PYX_ERR(0, 194, __pyx_L1_error)
    __pyx_v_options = CSLAddNameValue(__pyx_v_options, ((char const *)__pyx_t_10), ((char const *)__pyx_t_11));
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":196
 *         options = CSLAddNameValue(options, <const char *>k, <const char *>v)
 * 
 *     return options             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_options;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":172
 * 
 * 
 * cdef char** dict_to_options(object values):             # <<<<<<<<<<<<<<
 *     """Convert a python dictionary into name / value pairs (stored in a char**).
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyogrio._io.dict_to_options", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":199
 * 
 * 
 * cdef const char* override_threadlocal_config_option(str key, str value):             # <<<<<<<<<<<<<<
 *     """Set the CPLSetThreadLocalConfigOption for key=value.
 * 
*/

static char const *__pyx_f_7pyogrio_3_io_override_threadlocal_config_option(PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_key_b = NULL;
  char const *__pyx_v_key_c;
  PyObject *__pyx_v_value_b = NULL;
  char const *__pyx_v_value_c;
  char const *__pyx_v_prev_value;
  char const *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char const *__pyx_t_2;
  char const *__pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("override_threadlocal_config_option", 0);

  /* "pyogrio/_io.pyx":217
 *     """
 * 
 *     key_b = key.encode("UTF-8")             # <<<<<<<<<<<<<<
 *     cdef const char* key_c = key_b
 * 
*/
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 217, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_key_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":218
 * 
 *     key_b = key.encode("UTF-8")
 *     cdef const char* key_c = key_b             # <<<<<<<<<<<<<<
 * 
 *     value_b = value.encode("UTF-8")
*/
  __pyx_t_2 = __Pyx_PyBytes_AsString(__pyx_v_key_b); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 218, __pyx_L1_error)
  __pyx_v_key_c = __pyx_t_2;

  /* "pyogrio/_io.pyx":220
 *     cdef const char* key_c = key_b
 * 
 *     value_b = value.encode("UTF-8")             # <<<<<<<<<<<<<<
 *     cdef const char* value_c = value_b
 * 
*/
  if (unlikely(__pyx_v_value == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 220, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":221
 * 
 *     value_b = value.encode("UTF-8")
 *     cdef const char* value_c = value_b             # <<<<<<<<<<<<<<
 * 
 *     cdef const char *prev_value = CPLGetThreadLocalConfigOption(key_c, NULL)
*/
  __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_value_b); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 221, __pyx_L1_error)
  __pyx_v_value_c = __pyx_t_3;

  /* "pyogrio/_io.pyx":223
 *     cdef const char* value_c = value_b
 * 
 *     cdef const char *prev_value = CPLGetThreadLocalConfigOption(key_c, NULL)             # <<<<<<<<<<<<<<
 *     if prev_value != NULL:
 *         # strings returned from config options may be replaced via
*/
  __pyx_v_prev_value = CPLGetThreadLocalConfigOption(__pyx_v_key_c, NULL);

  /* "pyogrio/_io.pyx":224
 * 
 *     cdef const char *prev_value = CPLGetThreadLocalConfigOption(key_c, NULL)
 *     if prev_value != NULL:             # <<<<<<<<<<<<<<
 *         # strings returned from config options may be replaced via
 *         # CPLSetConfigOption() below; GDAL instructs us to save a copy
*/
  __pyx_t_4 = (__pyx_v_prev_value != NULL);
  if (__pyx_t_4) {

    /* "pyogrio/_io.pyx":228
 *         # CPLSetConfigOption() below; GDAL instructs us to save a copy
 *         # in a new string
 *         prev_value = CPLStrdup(prev_value)             # <<<<<<<<<<<<<<
 * 
 *     CPLSetThreadLocalConfigOption(key_c, value_c)
*/
    __pyx_v_prev_value = CPLStrdup(__pyx_v_prev_value);

    /* "pyogrio/_io.pyx":224
 * 
 *     cdef const char *prev_value = CPLGetThreadLocalConfigOption(key_c, NULL)
 *     if prev_value != NULL:             # <<<<<<<<<<<<<<
 *         # strings returned from config options may be replaced via
 *         # CPLSetConfigOption() below; GDAL instructs us to save a copy
*/
  }

  /* "pyogrio/_io.pyx":230
 *         prev_value = CPLStrdup(prev_value)
 * 
 *     CPLSetThreadLocalConfigOption(key_c, value_c)             # <<<<<<<<<<<<<<
 * 
 *     return prev_value
*/
  CPLSetThreadLocalConfigOption(__pyx_v_key_c, __pyx_v_value_c);

  /* "pyogrio/_io.pyx":232
 *     CPLSetThreadLocalConfigOption(key_c, value_c)
 * 
 *     return prev_value             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_prev_value;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":199
 * 
 * 
 * cdef const char* override_threadlocal_config_option(str key, str value):             # <<<<<<<<<<<<<<
 *     """Set the CPLSetThreadLocalConfigOption for key=value.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyogrio._io.override_threadlocal_config_option", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key_b);
  __Pyx_XDECREF(__pyx_v_value_b);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":235
 * 
 * 
 * cdef void* ogr_open(const char* path_c, int mode, char** options) except NULL:             # <<<<<<<<<<<<<<
 *     """Open an existing OGR data source.
 * 
*/

static void *__pyx_f_7pyogrio_3_io_ogr_open(char const *__pyx_v_path_c, int __pyx_v_mode, char **__pyx_v_options) {
  void *__pyx_v_ogr_dataset;
  struct __pyx_obj_7pyogrio_4_err_ErrorHandler *__pyx_v_errors = 0;
  int __pyx_v_flags;
  PyObject *__pyx_v_exc = NULL;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  void *__pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  Py_ssize_t __pyx_t_21;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23[4];
  PyObject *__pyx_t_24 = NULL;
  int __pyx_t_25;
  char const *__pyx_t_26;
  PyObject *__pyx_t_27 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_open", 0);

  /* "pyogrio/_io.pyx":253
 * 
 *     """
 *     cdef void *ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef ErrorHandler errors
 *     cdef int flags
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":258
 * 
 *     # Force linear approximations in all cases
 *     OGRSetNonLinearGeometriesEnabledFlag(0)             # <<<<<<<<<<<<<<
 * 
 *     flags = GDAL_OF_VECTOR | GDAL_OF_VERBOSE_ERROR
*/
  OGRSetNonLinearGeometriesEnabledFlag(0);

  /* "pyogrio/_io.pyx":260
 *     OGRSetNonLinearGeometriesEnabledFlag(0)
 * 
 *     flags = GDAL_OF_VECTOR | GDAL_OF_VERBOSE_ERROR             # <<<<<<<<<<<<<<
 *     if mode == 1:
 *         flags |= GDAL_OF_UPDATE
*/
  __pyx_v_flags = (GDAL_OF_VECTOR | GDAL_OF_VERBOSE_ERROR);

  /* "pyogrio/_io.pyx":261
 * 
 *     flags = GDAL_OF_VECTOR | GDAL_OF_VERBOSE_ERROR
 *     if mode == 1:             # <<<<<<<<<<<<<<
 *         flags |= GDAL_OF_UPDATE
 *     else:
*/
  __pyx_t_1 = (__pyx_v_mode == 1);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":262
 *     flags = GDAL_OF_VECTOR | GDAL_OF_VERBOSE_ERROR
 *     if mode == 1:
 *         flags |= GDAL_OF_UPDATE             # <<<<<<<<<<<<<<
 *     else:
 *         flags |= GDAL_OF_READONLY
*/
    __pyx_v_flags = (__pyx_v_flags | GDAL_OF_UPDATE);

    /* "pyogrio/_io.pyx":261
 * 
 *     flags = GDAL_OF_VECTOR | GDAL_OF_VERBOSE_ERROR
 *     if mode == 1:             # <<<<<<<<<<<<<<
 *         flags |= GDAL_OF_UPDATE
 *     else:
*/
    goto __pyx_L3;
  }

  /* "pyogrio/_io.pyx":264
 *         flags |= GDAL_OF_UPDATE
 *     else:
 *         flags |= GDAL_OF_READONLY             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
  /*else*/ {
    __pyx_v_flags = (__pyx_v_flags | GDAL_OF_READONLY);
  }
  __pyx_L3:;

  /* "pyogrio/_io.pyx":266
 *         flags |= GDAL_OF_READONLY
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         # WARNING: GDAL logs warnings about invalid open options to stderr
 *         # instead of raising an error
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "pyogrio/_io.pyx":269
 *         # WARNING: GDAL logs warnings about invalid open options to stderr
 *         # instead of raising an error
 *         with capture_errors() as errors:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 ogr_dataset = GDALOpenEx(
*/
      /*with:*/ {
        __pyx_t_6 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_capture_errors); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 269, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
          assert(__pyx_t_6);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
          __pyx_t_8 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
          __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 269, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __pyx_t_9 = __Pyx_PyObject_LookupSpecial(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 269, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_6 = NULL;
        __pyx_t_10 = __Pyx_PyObject_LookupSpecial(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 269, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_8 = 1;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_10);
          assert(__pyx_t_6);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
          __pyx_t_8 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
          __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 269, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_7);
        }
        __pyx_t_10 = __pyx_t_7;
        __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        /*try:*/ {
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_11);
            __Pyx_XGOTREF(__pyx_t_12);
            __Pyx_XGOTREF(__pyx_t_13);
            /*try:*/ {
              if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_mstate_global->__pyx_ptype_7pyogrio_4_err_ErrorHandler))))) __PYX_ERR(0, 269, __pyx_L14_error)
              __pyx_v_errors = ((struct __pyx_obj_7pyogrio_4_err_ErrorHandler *)__pyx_t_10);
              __pyx_t_10 = 0;

              /* "pyogrio/_io.pyx":270
 *         # instead of raising an error
 *         with capture_errors() as errors:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 ogr_dataset = GDALOpenEx(
 *                     path_c, flags, NULL, <const char *const *>options, NULL
*/
              {
                  PyThreadState * _save;
                  _save = PyEval_SaveThread();
                  __Pyx_FastGIL_Remember();
                  /*try:*/ {

                    /* "pyogrio/_io.pyx":271
 *         with capture_errors() as errors:
 *             with nogil:
 *                 ogr_dataset = GDALOpenEx(             # <<<<<<<<<<<<<<
 *                     path_c, flags, NULL, <const char *const *>options, NULL
 *                 )
*/
                    __pyx_v_ogr_dataset = GDALOpenEx(__pyx_v_path_c, __pyx_v_flags, NULL, ((char const *const *)__pyx_v_options), NULL);
                  }

                  /* "pyogrio/_io.pyx":270
 *         # instead of raising an error
 *         with capture_errors() as errors:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 ogr_dataset = GDALOpenEx(
 *                     path_c, flags, NULL, <const char *const *>options, NULL
*/
                  /*finally:*/ {
                    /*normal exit:*/{
                      __Pyx_FastGIL_Forget();
                      PyEval_RestoreThread(_save);
                      goto __pyx_L22;
                    }
                    __pyx_L22:;
                  }
              }

              /* "pyogrio/_io.pyx":274
 *                     path_c, flags, NULL, <const char *const *>options, NULL
 *                 )
 *             return errors.check_pointer(ogr_dataset, True)             # <<<<<<<<<<<<<<
 * 
 *     except NullPointerError:
*/
              __pyx_t_14 = ((struct __pyx_vtabstruct_7pyogrio_4_err_ErrorHandler *)__pyx_v_errors->__pyx_vtab)->check_pointer(__pyx_v_errors, __pyx_v_ogr_dataset, 1); if (unlikely(__pyx_t_14 == ((void *)NULL))) __PYX_ERR(0, 274, __pyx_L14_error)
              __pyx_r = __pyx_t_14;
              goto __pyx_L18_try_return;

              /* "pyogrio/_io.pyx":269
 *         # WARNING: GDAL logs warnings about invalid open options to stderr
 *         # instead of raising an error
 *         with capture_errors() as errors:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 ogr_dataset = GDALOpenEx(
*/
            }
            __pyx_L14_error:;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            /*except:*/ {
              __Pyx_AddTraceback("pyogrio._io.ogr_open", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_5, &__pyx_t_7) < 0) __PYX_ERR(0, 269, __pyx_L16_except_error)
              __Pyx_XGOTREF(__pyx_t_10);
              __Pyx_XGOTREF(__pyx_t_5);
              __Pyx_XGOTREF(__pyx_t_7);
              __pyx_t_6 = PyTuple_Pack(3, __pyx_t_10, __pyx_t_5, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 269, __pyx_L16_except_error)
              __Pyx_GOTREF(__pyx_t_6);
              __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_6, NULL);
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 269, __pyx_L16_except_error)
              __Pyx_GOTREF(__pyx_t_15);
              __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_15);
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              if (__pyx_t_1 < (0)) __PYX_ERR(0, 269, __pyx_L16_except_error)
              __pyx_t_16 = (!__pyx_t_1);
              if (unlikely(__pyx_t_16)) {
                __Pyx_GIVEREF(__pyx_t_10);
                __Pyx_GIVEREF(__pyx_t_5);
                __Pyx_XGIVEREF(__pyx_t_7);
                __Pyx_ErrRestoreWithState(__pyx_t_10, __pyx_t_5, __pyx_t_7);
                __pyx_t_10 = 0;  __pyx_t_5 = 0;  __pyx_t_7 = 0; 
                __PYX_ERR(0, 269, __pyx_L16_except_error)
              }
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              goto __pyx_L15_exception_handled;
            }
            __pyx_L16_except_error:;
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
            goto __pyx_L4_error;
            __pyx_L18_try_return:;
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
            goto __pyx_L11_return;
            __pyx_L15_exception_handled:;
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
          }
        }
        /*finally:*/ {
          /*normal exit:*/{
            if (__pyx_t_9) {
              __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_mstate_global->__pyx_tuple[1], NULL);
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
              if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 269, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_13);
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            }
            goto __pyx_L13;
          }
          __pyx_L11_return: {
            __pyx_t_14 = __pyx_r;
            if (__pyx_t_9) {
              __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_mstate_global->__pyx_tuple[1], NULL);
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
              if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 269, __pyx_L4_error)
              __Pyx_GOTREF(__pyx_t_13);
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            }
            __pyx_r = __pyx_t_14;
            goto __pyx_L8_try_return;
          }
          __pyx_L13:;
        }
        goto __pyx_L26;
        __pyx_L10_error:;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L4_error;
        __pyx_L26:;
      }

      /* "pyogrio/_io.pyx":266
 *         flags |= GDAL_OF_READONLY
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         # WARNING: GDAL logs warnings about invalid open options to stderr
 *         # instead of raising an error
*/
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L9_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":276
 *             return errors.check_pointer(ogr_dataset, True)
 * 
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(
 *             f"Failed to open dataset ({mode=}): {path_c.decode('utf-8')}"
*/
    __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_5, &__pyx_t_10);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 276, __pyx_L6_except_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_17 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_7, __pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_ErrRestore(__pyx_t_7, __pyx_t_5, __pyx_t_10);
    __pyx_t_7 = 0; __pyx_t_5 = 0; __pyx_t_10 = 0;
    if (__pyx_t_17) {
      __Pyx_AddTraceback("pyogrio._io.ogr_open", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_5, &__pyx_t_7) < 0) __PYX_ERR(0, 276, __pyx_L6_except_error)
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_7);

      /* "pyogrio/_io.pyx":277
 * 
 *     except NullPointerError:
 *         raise DataSourceError(             # <<<<<<<<<<<<<<
 *             f"Failed to open dataset ({mode=}): {path_c.decode('utf-8')}"
 *         ) from None
*/
      __pyx_t_18 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 277, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_19);

      /* "pyogrio/_io.pyx":278
 *     except NullPointerError:
 *         raise DataSourceError(
 *             f"Failed to open dataset ({mode=}): {path_c.decode('utf-8')}"             # <<<<<<<<<<<<<<
 *         ) from None
 * 
*/
      __pyx_t_20 = __Pyx_PyUnicode_From_int(__pyx_v_mode, 0, ' ', 'd'); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 278, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_20);
      __pyx_t_21 = __Pyx_ssize_strlen(__pyx_v_path_c); if (unlikely(__pyx_t_21 == ((Py_ssize_t)-1))) __PYX_ERR(0, 278, __pyx_L6_except_error)
      __pyx_t_22 = __Pyx_decode_c_string(__pyx_v_path_c, 0, __pyx_t_21, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 278, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_22);
      __pyx_t_23[0] = __pyx_mstate_global->__pyx_kp_u_Failed_to_open_dataset_mode;
      __pyx_t_23[1] = __pyx_t_20;
      __pyx_t_23[2] = __pyx_mstate_global->__pyx_kp_u__6;
      __pyx_t_23[3] = __pyx_t_22;
      __pyx_t_24 = __Pyx_PyUnicode_Join(__pyx_t_23, 4, 29 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_20) + 3 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_22), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_22));
      if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 278, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_24);
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
      __pyx_t_8 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_19))) {
        __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_19);
        assert(__pyx_t_18);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_19);
        __Pyx_INCREF(__pyx_t_18);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_19, __pyx__function);
        __pyx_t_8 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_18, __pyx_t_24};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_19, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
        __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 277, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_6);
      }

      /* "pyogrio/_io.pyx":279
 *         raise DataSourceError(
 *             f"Failed to open dataset ({mode=}): {path_c.decode('utf-8')}"
 *         ) from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
      __Pyx_Raise(__pyx_t_6, 0, 0, Py_None);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 277, __pyx_L6_except_error)
    }

    /* "pyogrio/_io.pyx":281
 *         ) from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         if " a supported file format." in str(exc):
 *             # In gdal 3.9, this error message was slightly changed, so we can only check
*/
    __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_5, &__pyx_t_10);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 281, __pyx_L6_except_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_17 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_7, __pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_ErrRestore(__pyx_t_7, __pyx_t_5, __pyx_t_10);
    __pyx_t_7 = 0; __pyx_t_5 = 0; __pyx_t_10 = 0;
    if (__pyx_t_17) {
      __Pyx_AddTraceback("pyogrio._io.ogr_open", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_5, &__pyx_t_7) < 0) __PYX_ERR(0, 281, __pyx_L6_except_error)
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_v_exc = __pyx_t_5;
      /*try:*/ {

        /* "pyogrio/_io.pyx":282
 * 
 *     except CPLE_BaseError as exc:
 *         if " a supported file format." in str(exc):             # <<<<<<<<<<<<<<
 *             # In gdal 3.9, this error message was slightly changed, so we can only check
 *             # on this part of the error message.
*/
        __pyx_t_6 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 282, __pyx_L34_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_16 = (__Pyx_PyUnicode_ContainsTF(__pyx_mstate_global->__pyx_kp_u_a_supported_file_format, __pyx_t_6, Py_EQ)); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 282, __pyx_L34_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(__pyx_t_16)) {

          /* "pyogrio/_io.pyx":285
 *             # In gdal 3.9, this error message was slightly changed, so we can only check
 *             # on this part of the error message.
 *             raise DataSourceError(             # <<<<<<<<<<<<<<
 *                 f"{str(exc)}; It might help to specify the correct driver explicitly "
 *                 "by prefixing the file path with '<DRIVER>:', e.g. 'CSV:path'."
*/
          __pyx_t_19 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_24, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 285, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_24);

          /* "pyogrio/_io.pyx":286
 *             # on this part of the error message.
 *             raise DataSourceError(
 *                 f"{str(exc)}; It might help to specify the correct driver explicitly "             # <<<<<<<<<<<<<<
 *                 "by prefixing the file path with '<DRIVER>:', e.g. 'CSV:path'."
 *             ) from None
*/
          __pyx_t_18 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 286, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_18);
          __pyx_t_22 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(__pyx_t_18, __pyx_mstate_global->__pyx_kp_u_It_might_help_to_specify_the_co); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 286, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_22);
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          __pyx_t_8 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_24))) {
            __pyx_t_19 = PyMethod_GET_SELF(__pyx_t_24);
            assert(__pyx_t_19);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_24);
            __Pyx_INCREF(__pyx_t_19);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_24, __pyx__function);
            __pyx_t_8 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_19, __pyx_t_22};
            __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_24, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
            __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 285, __pyx_L34_error)
            __Pyx_GOTREF(__pyx_t_6);
          }

          /* "pyogrio/_io.pyx":288
 *                 f"{str(exc)}; It might help to specify the correct driver explicitly "
 *                 "by prefixing the file path with '<DRIVER>:', e.g. 'CSV:path'."
 *             ) from None             # <<<<<<<<<<<<<<
 * 
 *         raise DataSourceError(str(exc)) from None
*/
          __Pyx_Raise(__pyx_t_6, 0, 0, Py_None);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __PYX_ERR(0, 285, __pyx_L34_error)

          /* "pyogrio/_io.pyx":282
 * 
 *     except CPLE_BaseError as exc:
 *         if " a supported file format." in str(exc):             # <<<<<<<<<<<<<<
 *             # In gdal 3.9, this error message was slightly changed, so we can only check
 *             # on this part of the error message.
*/
        }

        /* "pyogrio/_io.pyx":290
 *             ) from None
 * 
 *         raise DataSourceError(str(exc)) from None             # <<<<<<<<<<<<<<
 * 
 * 
*/
        __pyx_t_24 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_22, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 290, __pyx_L34_error)
        __Pyx_GOTREF(__pyx_t_22);
        __pyx_t_19 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 290, __pyx_L34_error)
        __Pyx_GOTREF(__pyx_t_19);
        __pyx_t_8 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_22))) {
          __pyx_t_24 = PyMethod_GET_SELF(__pyx_t_22);
          assert(__pyx_t_24);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_22);
          __Pyx_INCREF(__pyx_t_24);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_22, __pyx__function);
          __pyx_t_8 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_24, __pyx_t_19};
          __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_22, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
          __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
          __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 290, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_Raise(__pyx_t_6, 0, 0, Py_None);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __PYX_ERR(0, 290, __pyx_L34_error)
      }

      /* "pyogrio/_io.pyx":281
 *         ) from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         if " a supported file format." in str(exc):
 *             # In gdal 3.9, this error message was slightly changed, so we can only check
*/
      /*finally:*/ {
        __pyx_L34_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_9 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_15 = 0; __pyx_t_27 = 0;
          __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
          __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
          __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
          __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_11, &__pyx_t_15, &__pyx_t_27);
          if ( unlikely(__Pyx_GetException(&__pyx_t_9, &__pyx_t_13, &__pyx_t_12) < 0)) __Pyx_ErrFetch(&__pyx_t_9, &__pyx_t_13, &__pyx_t_12);
          __Pyx_XGOTREF(__pyx_t_9);
          __Pyx_XGOTREF(__pyx_t_13);
          __Pyx_XGOTREF(__pyx_t_12);
          __Pyx_XGOTREF(__pyx_t_11);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_27);
          __pyx_t_17 = __pyx_lineno; __pyx_t_25 = __pyx_clineno; __pyx_t_26 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_11);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_27);
          __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_15, __pyx_t_27);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_XGIVEREF(__pyx_t_13);
          __Pyx_XGIVEREF(__pyx_t_12);
          __Pyx_ErrRestore(__pyx_t_9, __pyx_t_13, __pyx_t_12);
          __pyx_t_9 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_15 = 0; __pyx_t_27 = 0;
          __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_25; __pyx_filename = __pyx_t_26;
          goto __pyx_L6_except_error;
        }
      }
    }
    goto __pyx_L6_except_error;

    /* "pyogrio/_io.pyx":266
 *         flags |= GDAL_OF_READONLY
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         # WARNING: GDAL logs warnings about invalid open options to stderr
 *         # instead of raising an error
*/
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
    __pyx_L9_try_end:;
  }

  /* "pyogrio/_io.pyx":235
 * 
 * 
 * cdef void* ogr_open(const char* path_c, int mode, char** options) except NULL:             # <<<<<<<<<<<<<<
 *     """Open an existing OGR data source.
 * 
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_24);
  __Pyx_AddTraceback("pyogrio._io.ogr_open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_errors);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":293
 * 
 * 
 * cdef ogr_close(GDALDatasetH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Close the dataset and raise exception if that fails.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_ogr_close(GDALDatasetH __pyx_v_ogr_dataset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_close", 0);

  /* "pyogrio/_io.pyx":304
 * 
 *     """
 *     if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *         IF CTE_GDAL_VERSION >= (3, 7, 0):
 *             if GDALClose(ogr_dataset) != CE_None:
*/
  __pyx_t_1 = (__pyx_v_ogr_dataset != NULL);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":306
 *     if ogr_dataset != NULL:
 *         IF CTE_GDAL_VERSION >= (3, 7, 0):
 *             if GDALClose(ogr_dataset) != CE_None:             # <<<<<<<<<<<<<<
 *                 return check_last_error()
 * 
*/
    __pyx_t_1 = (GDALClose(__pyx_v_ogr_dataset) != CE_None);
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":307
 *         IF CTE_GDAL_VERSION >= (3, 7, 0):
 *             if GDALClose(ogr_dataset) != CE_None:
 *                 return check_last_error()             # <<<<<<<<<<<<<<
 * 
 *             return
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_2 = __pyx_f_7pyogrio_4_err_check_last_error(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 307, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      goto __pyx_L0;

      /* "pyogrio/_io.pyx":306
 *     if ogr_dataset != NULL:
 *         IF CTE_GDAL_VERSION >= (3, 7, 0):
 *             if GDALClose(ogr_dataset) != CE_None:             # <<<<<<<<<<<<<<
 *                 return check_last_error()
 * 
*/
    }

    /* "pyogrio/_io.pyx":309
 *                 return check_last_error()
 * 
 *             return             # <<<<<<<<<<<<<<
 * 
 *         ELSE:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":304
 * 
 *     """
 *     if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *         IF CTE_GDAL_VERSION >= (3, 7, 0):
 *             if GDALClose(ogr_dataset) != CE_None:
*/
  }

  /* "pyogrio/_io.pyx":293
 * 
 * 
 * cdef ogr_close(GDALDatasetH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Close the dataset and raise exception if that fails.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyogrio._io.ogr_close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":319
 * 
 * 
 * cdef OGRLayerH get_ogr_layer(GDALDatasetH ogr_dataset, layer) except NULL:             # <<<<<<<<<<<<<<
 *     """Open OGR layer by index or name.
 * 
*/

static OGRLayerH __pyx_f_7pyogrio_3_io_get_ogr_layer(GDALDatasetH __pyx_v_ogr_dataset, PyObject *__pyx_v_layer) {
  OGRLayerH __pyx_v_ogr_layer;
  char *__pyx_v_layer_c;
  int __pyx_v_layer_int;
  char *__pyx_v_sql_c;
  PyObject *__pyx_v_layer_b = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_layer_name = NULL;
  PyObject *__pyx_v_sql_b = NULL;
  OGRLayerH __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  char *__pyx_t_8;
  void *__pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16[3];
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_ogr_layer", 0);

  /* "pyogrio/_io.pyx":335
 * 
 *     """
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 *     cdef char *layer_c
 *     cdef int layer_int
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":340
 *     cdef char *sql_c
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         if isinstance(layer, str):
 *             layer_b = layer.encode("utf-8")
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "pyogrio/_io.pyx":341
 * 
 *     try:
 *         if isinstance(layer, str):             # <<<<<<<<<<<<<<
 *             layer_b = layer.encode("utf-8")
 *             layer_c = layer_b
*/
      __pyx_t_4 = PyUnicode_Check(__pyx_v_layer); 
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":342
 *     try:
 *         if isinstance(layer, str):
 *             layer_b = layer.encode("utf-8")             # <<<<<<<<<<<<<<
 *             layer_c = layer_b
 *             with nogil:
*/
        __pyx_t_6 = __pyx_v_layer;
        __Pyx_INCREF(__pyx_t_6);
        __pyx_t_7 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_utf_8};
          __pyx_t_5 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 342, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __pyx_v_layer_b = __pyx_t_5;
        __pyx_t_5 = 0;

        /* "pyogrio/_io.pyx":343
 *         if isinstance(layer, str):
 *             layer_b = layer.encode("utf-8")
 *             layer_c = layer_b             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 ogr_layer = GDALDatasetGetLayerByName(ogr_dataset, layer_c)
*/
        __pyx_t_8 = __Pyx_PyObject_AsWritableString(__pyx_v_layer_b); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 343, __pyx_L3_error)
        __pyx_v_layer_c = __pyx_t_8;

        /* "pyogrio/_io.pyx":344
 *             layer_b = layer.encode("utf-8")
 *             layer_c = layer_b
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 ogr_layer = GDALDatasetGetLayerByName(ogr_dataset, layer_c)
 *             ogr_layer = check_pointer(ogr_layer)
*/
        {
            PyThreadState * _save;
            _save = PyEval_SaveThread();
            __Pyx_FastGIL_Remember();
            /*try:*/ {

              /* "pyogrio/_io.pyx":345
 *             layer_c = layer_b
 *             with nogil:
 *                 ogr_layer = GDALDatasetGetLayerByName(ogr_dataset, layer_c)             # <<<<<<<<<<<<<<
 *             ogr_layer = check_pointer(ogr_layer)
 * 
*/
              __pyx_v_ogr_layer = GDALDatasetGetLayerByName(__pyx_v_ogr_dataset, __pyx_v_layer_c);
            }

            /* "pyogrio/_io.pyx":344
 *             layer_b = layer.encode("utf-8")
 *             layer_c = layer_b
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 ogr_layer = GDALDatasetGetLayerByName(ogr_dataset, layer_c)
 *             ogr_layer = check_pointer(ogr_layer)
*/
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_FastGIL_Forget();
                PyEval_RestoreThread(_save);
                goto __pyx_L12;
              }
              __pyx_L12:;
            }
        }

        /* "pyogrio/_io.pyx":346
 *             with nogil:
 *                 ogr_layer = GDALDatasetGetLayerByName(ogr_dataset, layer_c)
 *             ogr_layer = check_pointer(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         elif isinstance(layer, int):
*/
        __pyx_t_9 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_layer); if (unlikely(__pyx_t_9 == ((void *)NULL))) __PYX_ERR(0, 346, __pyx_L3_error)
        __pyx_v_ogr_layer = __pyx_t_9;

        /* "pyogrio/_io.pyx":341
 * 
 *     try:
 *         if isinstance(layer, str):             # <<<<<<<<<<<<<<
 *             layer_b = layer.encode("utf-8")
 *             layer_c = layer_b
*/
        goto __pyx_L9;
      }

      /* "pyogrio/_io.pyx":348
 *             ogr_layer = check_pointer(ogr_layer)
 * 
 *         elif isinstance(layer, int):             # <<<<<<<<<<<<<<
 *             layer_int = layer
 *             with nogil:
*/
      __pyx_t_4 = PyLong_Check(__pyx_v_layer); 
      if (likely(__pyx_t_4)) {

        /* "pyogrio/_io.pyx":349
 * 
 *         elif isinstance(layer, int):
 *             layer_int = layer             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 ogr_layer = GDALDatasetGetLayer(ogr_dataset, layer_int)
*/
        __pyx_t_10 = __Pyx_PyLong_As_int(__pyx_v_layer); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 349, __pyx_L3_error)
        __pyx_v_layer_int = __pyx_t_10;

        /* "pyogrio/_io.pyx":350
 *         elif isinstance(layer, int):
 *             layer_int = layer
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 ogr_layer = GDALDatasetGetLayer(ogr_dataset, layer_int)
 *             ogr_layer = check_pointer(ogr_layer)
*/
        {
            PyThreadState * _save;
            _save = PyEval_SaveThread();
            __Pyx_FastGIL_Remember();
            /*try:*/ {

              /* "pyogrio/_io.pyx":351
 *             layer_int = layer
 *             with nogil:
 *                 ogr_layer = GDALDatasetGetLayer(ogr_dataset, layer_int)             # <<<<<<<<<<<<<<
 *             ogr_layer = check_pointer(ogr_layer)
 *         else:
*/
              __pyx_v_ogr_layer = GDALDatasetGetLayer(__pyx_v_ogr_dataset, __pyx_v_layer_int);
            }

            /* "pyogrio/_io.pyx":350
 *         elif isinstance(layer, int):
 *             layer_int = layer
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 ogr_layer = GDALDatasetGetLayer(ogr_dataset, layer_int)
 *             ogr_layer = check_pointer(ogr_layer)
*/
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_FastGIL_Forget();
                PyEval_RestoreThread(_save);
                goto __pyx_L15;
              }
              __pyx_L15:;
            }
        }

        /* "pyogrio/_io.pyx":352
 *             with nogil:
 *                 ogr_layer = GDALDatasetGetLayer(ogr_dataset, layer_int)
 *             ogr_layer = check_pointer(ogr_layer)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError(
*/
        __pyx_t_9 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_layer); if (unlikely(__pyx_t_9 == ((void *)NULL))) __PYX_ERR(0, 352, __pyx_L3_error)
        __pyx_v_ogr_layer = __pyx_t_9;

        /* "pyogrio/_io.pyx":348
 *             ogr_layer = check_pointer(ogr_layer)
 * 
 *         elif isinstance(layer, int):             # <<<<<<<<<<<<<<
 *             layer_int = layer
 *             with nogil:
*/
        goto __pyx_L9;
      }

      /* "pyogrio/_io.pyx":354
 *             ogr_layer = check_pointer(ogr_layer)
 *         else:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 f"'layer' parameter must be a str or int, got {type(layer)}"
 *             )
*/
      /*else*/ {
        __pyx_t_6 = NULL;

        /* "pyogrio/_io.pyx":355
 *         else:
 *             raise ValueError(
 *                 f"'layer' parameter must be a str or int, got {type(layer)}"             # <<<<<<<<<<<<<<
 *             )
 * 
*/
        __pyx_t_11 = __Pyx_PyObject_FormatSimple(((PyObject *)Py_TYPE(__pyx_v_layer)), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 355, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_layer_parameter_must_be_a_str_o, __pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 355, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_7 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_12};
          __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 354, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(0, 354, __pyx_L3_error)
      }
      __pyx_L9:;

      /* "pyogrio/_io.pyx":340
 *     cdef char *sql_c
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         if isinstance(layer, str):
 *             layer_b = layer.encode("utf-8")
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "pyogrio/_io.pyx":359
 * 
 *     # GDAL does not always raise exception messages in this case
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(f"Layer '{layer}' could not be opened") from None
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_12, &__pyx_t_6);
    __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 359, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_10 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_11);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_12, __pyx_t_6);
    __pyx_t_5 = 0; __pyx_t_12 = 0; __pyx_t_6 = 0;
    if (__pyx_t_10) {
      __Pyx_AddTraceback("pyogrio._io.get_ogr_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_12, &__pyx_t_5) < 0) __PYX_ERR(0, 359, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_5);

      /* "pyogrio/_io.pyx":360
 *     # GDAL does not always raise exception messages in this case
 *     except NullPointerError:
 *         raise DataLayerError(f"Layer '{layer}' could not be opened") from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
      __pyx_t_13 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 360, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_layer, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 360, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_16[0] = __pyx_mstate_global->__pyx_kp_u_Layer;
      __pyx_t_16[1] = __pyx_t_15;
      __pyx_t_16[2] = __pyx_mstate_global->__pyx_kp_u_could_not_be_opened;
      __pyx_t_17 = __Pyx_PyUnicode_Join(__pyx_t_16, 3, 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 21, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15));
      if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 360, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_17);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_7 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_14);
        assert(__pyx_t_13);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
        __pyx_t_7 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_17};
        __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 360, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_11);
      }
      __Pyx_Raise(__pyx_t_11, 0, 0, Py_None);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __PYX_ERR(0, 360, __pyx_L5_except_error)
    }

    /* "pyogrio/_io.pyx":362
 *         raise DataLayerError(f"Layer '{layer}' could not be opened") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_12, &__pyx_t_6);
    __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 362, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_10 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_11);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_12, __pyx_t_6);
    __pyx_t_5 = 0; __pyx_t_12 = 0; __pyx_t_6 = 0;
    if (__pyx_t_10) {
      __Pyx_AddTraceback("pyogrio._io.get_ogr_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_12, &__pyx_t_5) < 0) __PYX_ERR(0, 362, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_12);
      __pyx_v_exc = __pyx_t_12;
      /*try:*/ {

        /* "pyogrio/_io.pyx":363
 * 
 *     except CPLE_BaseError as exc:
 *         raise DataLayerError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *     # if the driver is OSM, we need to execute SQL to set the layer to read in
*/
        __pyx_t_14 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 363, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_13 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 363, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_17))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_17);
          assert(__pyx_t_14);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_17);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_17, __pyx__function);
          __pyx_t_7 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_13};
          __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_17, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 363, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_11);
        }
        __Pyx_Raise(__pyx_t_11, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __PYX_ERR(0, 363, __pyx_L23_error)
      }

      /* "pyogrio/_io.pyx":362
 *         raise DataLayerError(f"Layer '{layer}' could not be opened") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
      /*finally:*/ {
        __pyx_L23_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
          if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __Pyx_XGOTREF(__pyx_t_25);
          __pyx_t_10 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_25);
          __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "pyogrio/_io.pyx":340
 *     cdef char *sql_c
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         if isinstance(layer, str):
 *             layer_b = layer.encode("utf-8")
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "pyogrio/_io.pyx":367
 *     # if the driver is OSM, we need to execute SQL to set the layer to read in
 *     # order to read it properly
 *     if get_driver(ogr_dataset) == "OSM":             # <<<<<<<<<<<<<<
 *         # Note: this returns NULL and does not need to be freed via
 *         # GDALDatasetReleaseResultSet()
*/
  __pyx_t_5 = __pyx_f_7pyogrio_3_io_get_driver(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_OSM, Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_4) {

    /* "pyogrio/_io.pyx":370
 *         # Note: this returns NULL and does not need to be freed via
 *         # GDALDatasetReleaseResultSet()
 *         layer_name = get_string(OGR_L_GetName(ogr_layer))             # <<<<<<<<<<<<<<
 *         sql_b = f"SET interest_layers = {layer_name}".encode("utf-8")
 *         sql_c = sql_b
*/
    __pyx_t_5 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetName(__pyx_v_ogr_layer), NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 370, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_layer_name = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":371
 *         # GDALDatasetReleaseResultSet()
 *         layer_name = get_string(OGR_L_GetName(ogr_layer))
 *         sql_b = f"SET interest_layers = {layer_name}".encode("utf-8")             # <<<<<<<<<<<<<<
 *         sql_c = sql_b
 * 
*/
    __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_v_layer_name, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_12 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_SET_interest_layers, __pyx_t_5); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyUnicode_AsUTF8String(((PyObject*)__pyx_t_12)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_v_sql_b = ((PyObject*)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":372
 *         layer_name = get_string(OGR_L_GetName(ogr_layer))
 *         sql_b = f"SET interest_layers = {layer_name}".encode("utf-8")
 *         sql_c = sql_b             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_8 = __Pyx_PyBytes_AsWritableString(__pyx_v_sql_b); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 372, __pyx_L1_error)
    __pyx_v_sql_c = __pyx_t_8;

    /* "pyogrio/_io.pyx":374
 *         sql_c = sql_b
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
 * 
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "pyogrio/_io.pyx":375
 * 
 *         with nogil:
 *             GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)             # <<<<<<<<<<<<<<
 * 
 *     return ogr_layer
*/
          (void)(GDALDatasetExecuteSQL(__pyx_v_ogr_dataset, __pyx_v_sql_c, NULL, NULL));
        }

        /* "pyogrio/_io.pyx":374
 *         sql_c = sql_b
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
 * 
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L32;
          }
          __pyx_L32:;
        }
    }

    /* "pyogrio/_io.pyx":367
 *     # if the driver is OSM, we need to execute SQL to set the layer to read in
 *     # order to read it properly
 *     if get_driver(ogr_dataset) == "OSM":             # <<<<<<<<<<<<<<
 *         # Note: this returns NULL and does not need to be freed via
 *         # GDALDatasetReleaseResultSet()
*/
  }

  /* "pyogrio/_io.pyx":377
 *             GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
 * 
 *     return ogr_layer             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_ogr_layer;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":319
 * 
 * 
 * cdef OGRLayerH get_ogr_layer(GDALDatasetH ogr_dataset, layer) except NULL:             # <<<<<<<<<<<<<<
 *     """Open OGR layer by index or name.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("pyogrio._io.get_ogr_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_layer_b);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_layer_name);
  __Pyx_XDECREF(__pyx_v_sql_b);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":380
 * 
 * 
 * cdef OGRLayerH execute_sql(             # <<<<<<<<<<<<<<
 *     GDALDatasetH ogr_dataset, str sql, str sql_dialect=None
 * ) except NULL:
*/

static OGRLayerH __pyx_f_7pyogrio_3_io_execute_sql(GDALDatasetH __pyx_v_ogr_dataset, PyObject *__pyx_v_sql, struct __pyx_opt_args_7pyogrio_3_io_execute_sql *__pyx_optional_args) {

  /* "pyogrio/_io.pyx":381
 * 
 * cdef OGRLayerH execute_sql(
 *     GDALDatasetH ogr_dataset, str sql, str sql_dialect=None             # <<<<<<<<<<<<<<
 * ) except NULL:
 *     """Execute an SQL statement on a dataset.
*/
  PyObject *__pyx_v_sql_dialect = ((PyObject*)Py_None);
  char *__pyx_v_sql_c;
  char *__pyx_v_sql_dialect_c;
  OGRLayerH __pyx_v_retval;
  PyObject *__pyx_v_sql_b = NULL;
  PyObject *__pyx_v_sql_dialect_b = NULL;
  PyObject *__pyx_v_exc = NULL;
  OGRLayerH __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  char *__pyx_t_5;
  int __pyx_t_6;
  void *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15[3];
  PyObject *__pyx_t_16 = NULL;
  size_t __pyx_t_17;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("execute_sql", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_sql_dialect = __pyx_optional_args->sql_dialect;
    }
  }

  /* "pyogrio/_io.pyx":402
 *     cdef char * sql_c
 *     cdef char * sql_dialect_c
 *     cdef OGRLayerH retval = NULL             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
  __pyx_v_retval = NULL;

  /* "pyogrio/_io.pyx":404
 *     cdef OGRLayerH retval = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         sql_b = sql.encode("utf-8")
 *         sql_c = sql_b
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "pyogrio/_io.pyx":405
 * 
 *     try:
 *         sql_b = sql.encode("utf-8")             # <<<<<<<<<<<<<<
 *         sql_c = sql_b
 *         if sql_dialect is None:
*/
      if (unlikely(__pyx_v_sql == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
        __PYX_ERR(0, 405, __pyx_L3_error)
      }
      __pyx_t_4 = PyUnicode_AsUTF8String(__pyx_v_sql); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 405, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_sql_b = ((PyObject*)__pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyogrio/_io.pyx":406
 *     try:
 *         sql_b = sql.encode("utf-8")
 *         sql_c = sql_b             # <<<<<<<<<<<<<<
 *         if sql_dialect is None:
 *             with nogil:
*/
      __pyx_t_5 = __Pyx_PyBytes_AsWritableString(__pyx_v_sql_b); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 406, __pyx_L3_error)
      __pyx_v_sql_c = __pyx_t_5;

      /* "pyogrio/_io.pyx":407
 *         sql_b = sql.encode("utf-8")
 *         sql_c = sql_b
 *         if sql_dialect is None:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
*/
      __pyx_t_6 = (__pyx_v_sql_dialect == ((PyObject*)Py_None));
      if (__pyx_t_6) {

        /* "pyogrio/_io.pyx":408
 *         sql_c = sql_b
 *         if sql_dialect is None:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
 *             return check_pointer(retval)
*/
        {
            PyThreadState * _save;
            _save = PyEval_SaveThread();
            __Pyx_FastGIL_Remember();
            /*try:*/ {

              /* "pyogrio/_io.pyx":409
 *         if sql_dialect is None:
 *             with nogil:
 *                 retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)             # <<<<<<<<<<<<<<
 *             return check_pointer(retval)
 * 
*/
              __pyx_v_retval = GDALDatasetExecuteSQL(__pyx_v_ogr_dataset, __pyx_v_sql_c, NULL, NULL);
            }

            /* "pyogrio/_io.pyx":408
 *         sql_c = sql_b
 *         if sql_dialect is None:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
 *             return check_pointer(retval)
*/
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_FastGIL_Forget();
                PyEval_RestoreThread(_save);
                goto __pyx_L12;
              }
              __pyx_L12:;
            }
        }

        /* "pyogrio/_io.pyx":410
 *             with nogil:
 *                 retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
 *             return check_pointer(retval)             # <<<<<<<<<<<<<<
 * 
 *         sql_dialect_b = sql_dialect.encode("utf-8")
*/
        __pyx_t_7 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_retval); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 410, __pyx_L3_error)
        __pyx_r = __pyx_t_7;
        goto __pyx_L7_try_return;

        /* "pyogrio/_io.pyx":407
 *         sql_b = sql.encode("utf-8")
 *         sql_c = sql_b
 *         if sql_dialect is None:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, NULL)
*/
      }

      /* "pyogrio/_io.pyx":412
 *             return check_pointer(retval)
 * 
 *         sql_dialect_b = sql_dialect.encode("utf-8")             # <<<<<<<<<<<<<<
 *         sql_dialect_c = sql_dialect_b
 *         with nogil:
*/
      if (unlikely(__pyx_v_sql_dialect == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
        __PYX_ERR(0, 412, __pyx_L3_error)
      }
      __pyx_t_4 = PyUnicode_AsUTF8String(__pyx_v_sql_dialect); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 412, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_sql_dialect_b = ((PyObject*)__pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyogrio/_io.pyx":413
 * 
 *         sql_dialect_b = sql_dialect.encode("utf-8")
 *         sql_dialect_c = sql_dialect_b             # <<<<<<<<<<<<<<
 *         with nogil:
 *             retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, sql_dialect_c)
*/
      __pyx_t_5 = __Pyx_PyBytes_AsWritableString(__pyx_v_sql_dialect_b); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 413, __pyx_L3_error)
      __pyx_v_sql_dialect_c = __pyx_t_5;

      /* "pyogrio/_io.pyx":414
 *         sql_dialect_b = sql_dialect.encode("utf-8")
 *         sql_dialect_c = sql_dialect_b
 *         with nogil:             # <<<<<<<<<<<<<<
 *             retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, sql_dialect_c)
 *         return check_pointer(retval)
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "pyogrio/_io.pyx":415
 *         sql_dialect_c = sql_dialect_b
 *         with nogil:
 *             retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, sql_dialect_c)             # <<<<<<<<<<<<<<
 *         return check_pointer(retval)
 * 
*/
            __pyx_v_retval = GDALDatasetExecuteSQL(__pyx_v_ogr_dataset, __pyx_v_sql_c, NULL, __pyx_v_sql_dialect_c);
          }

          /* "pyogrio/_io.pyx":414
 *         sql_dialect_b = sql_dialect.encode("utf-8")
 *         sql_dialect_c = sql_dialect_b
 *         with nogil:             # <<<<<<<<<<<<<<
 *             retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, sql_dialect_c)
 *         return check_pointer(retval)
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L15;
            }
            __pyx_L15:;
          }
      }

      /* "pyogrio/_io.pyx":416
 *         with nogil:
 *             retval = GDALDatasetExecuteSQL(ogr_dataset, sql_c, NULL, sql_dialect_c)
 *         return check_pointer(retval)             # <<<<<<<<<<<<<<
 * 
 *     # GDAL does not always raise exception messages in this case
*/
      __pyx_t_7 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_retval); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 416, __pyx_L3_error)
      __pyx_r = __pyx_t_7;
      goto __pyx_L7_try_return;

      /* "pyogrio/_io.pyx":404
 *     cdef OGRLayerH retval = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         sql_b = sql.encode("utf-8")
 *         sql_c = sql_b
*/
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pyogrio/_io.pyx":419
 * 
 *     # GDAL does not always raise exception messages in this case
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(f"Error executing sql '{sql}'") from None
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_4, &__pyx_t_8, &__pyx_t_9);
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 419, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_4, __pyx_t_10);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_ErrRestore(__pyx_t_4, __pyx_t_8, __pyx_t_9);
    __pyx_t_4 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0;
    if (__pyx_t_11) {
      __Pyx_AddTraceback("pyogrio._io.execute_sql", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_8, &__pyx_t_4) < 0) __PYX_ERR(0, 419, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_4);

      /* "pyogrio/_io.pyx":420
 *     # GDAL does not always raise exception messages in this case
 *     except NullPointerError:
 *         raise DataLayerError(f"Error executing sql '{sql}'") from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
      __pyx_t_12 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 420, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_14 = __Pyx_PyUnicode_Unicode(__pyx_v_sql); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 420, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_15[0] = __pyx_mstate_global->__pyx_kp_u_Error_executing_sql;
      __pyx_t_15[1] = __pyx_t_14;
      __pyx_t_15[2] = __pyx_mstate_global->__pyx_kp_u__4;
      __pyx_t_16 = __Pyx_PyUnicode_Join(__pyx_t_15, 3, 21 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14));
      if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 420, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_17 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);
        assert(__pyx_t_12);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
        __pyx_t_17 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_t_16};
        __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_17, (2-__pyx_t_17) | (__pyx_t_17*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 420, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_Raise(__pyx_t_10, 0, 0, Py_None);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __PYX_ERR(0, 420, __pyx_L5_except_error)
    }

    /* "pyogrio/_io.pyx":422
 *         raise DataLayerError(f"Error executing sql '{sql}'") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_4, &__pyx_t_8, &__pyx_t_9);
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 422, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_4, __pyx_t_10);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_ErrRestore(__pyx_t_4, __pyx_t_8, __pyx_t_9);
    __pyx_t_4 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0;
    if (__pyx_t_11) {
      __Pyx_AddTraceback("pyogrio._io.execute_sql", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_8, &__pyx_t_4) < 0) __PYX_ERR(0, 422, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_8);
      __pyx_v_exc = __pyx_t_8;
      /*try:*/ {

        /* "pyogrio/_io.pyx":423
 * 
 *     except CPLE_BaseError as exc:
 *         raise DataLayerError(str(exc))             # <<<<<<<<<<<<<<
 * 
 * 
*/
        __pyx_t_13 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 423, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_12 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 423, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_17 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_16))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_16);
          assert(__pyx_t_13);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
          __pyx_t_17 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_12};
          __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_17, (2-__pyx_t_17) | (__pyx_t_17*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 423, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_10);
        }
        __Pyx_Raise(__pyx_t_10, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __PYX_ERR(0, 423, __pyx_L23_error)
      }

      /* "pyogrio/_io.pyx":422
 *         raise DataLayerError(f"Error executing sql '{sql}'") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
      /*finally:*/ {
        __pyx_L23_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
          if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __Pyx_XGOTREF(__pyx_t_25);
          __pyx_t_11 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_25);
          __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "pyogrio/_io.pyx":404
 *     cdef OGRLayerH retval = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         sql_b = sql.encode("utf-8")
 *         sql_c = sql_b
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L7_try_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
  }

  /* "pyogrio/_io.pyx":380
 * 
 * 
 * cdef OGRLayerH execute_sql(             # <<<<<<<<<<<<<<
 *     GDALDatasetH ogr_dataset, str sql, str sql_dialect=None
 * ) except NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("pyogrio._io.execute_sql", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sql_b);
  __Pyx_XDECREF(__pyx_v_sql_dialect_b);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":426
 * 
 * 
 * cdef str get_crs(OGRLayerH ogr_layer):             # <<<<<<<<<<<<<<
 *     """Read CRS from layer as EPSG:<code> if available or WKT.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_crs(OGRLayerH __pyx_v_ogr_layer) {
  void *__pyx_v_ogr_crs;
  char const *__pyx_v_authority_key;
  char const *__pyx_v_authority_val;
  char *__pyx_v_ogr_wkt;
  PyObject *__pyx_v_exc = NULL;
  CYTHON_UNUSED int __pyx_v__retval;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_wkt = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  void *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  size_t __pyx_t_13;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_t_22;
  int __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_crs", 0);

  /* "pyogrio/_io.pyx":439
 *         EPSG:<code> or WKT
 *     """
 *     cdef void *ogr_crs = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *authority_key = NULL
 *     cdef const char *authority_val = NULL
*/
  __pyx_v_ogr_crs = NULL;

  /* "pyogrio/_io.pyx":440
 *     """
 *     cdef void *ogr_crs = NULL
 *     cdef const char *authority_key = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *authority_val = NULL
 *     cdef char *ogr_wkt = NULL
*/
  __pyx_v_authority_key = NULL;

  /* "pyogrio/_io.pyx":441
 *     cdef void *ogr_crs = NULL
 *     cdef const char *authority_key = NULL
 *     cdef const char *authority_val = NULL             # <<<<<<<<<<<<<<
 *     cdef char *ogr_wkt = NULL
 * 
*/
  __pyx_v_authority_val = NULL;

  /* "pyogrio/_io.pyx":442
 *     cdef const char *authority_key = NULL
 *     cdef const char *authority_val = NULL
 *     cdef char *ogr_wkt = NULL             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
  __pyx_v_ogr_wkt = NULL;

  /* "pyogrio/_io.pyx":444
 *     cdef char *ogr_wkt = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_crs = OGR_L_GetSpatialRef(ogr_layer)
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "pyogrio/_io.pyx":445
 * 
 *     try:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_crs = OGR_L_GetSpatialRef(ogr_layer)
 *         ogr_crs = check_pointer(ogr_crs)
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "pyogrio/_io.pyx":446
 *     try:
 *         with nogil:
 *             ogr_crs = OGR_L_GetSpatialRef(ogr_layer)             # <<<<<<<<<<<<<<
 *         ogr_crs = check_pointer(ogr_crs)
 * 
*/
            __pyx_v_ogr_crs = OGR_L_GetSpatialRef(__pyx_v_ogr_layer);
          }

          /* "pyogrio/_io.pyx":445
 * 
 *     try:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_crs = OGR_L_GetSpatialRef(ogr_layer)
 *         ogr_crs = check_pointer(ogr_crs)
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "pyogrio/_io.pyx":447
 *         with nogil:
 *             ogr_crs = OGR_L_GetSpatialRef(ogr_layer)
 *         ogr_crs = check_pointer(ogr_crs)             # <<<<<<<<<<<<<<
 * 
 *     except NullPointerError:
*/
      __pyx_t_4 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_crs); if (unlikely(__pyx_t_4 == ((void *)NULL))) __PYX_ERR(0, 447, __pyx_L3_error)
      __pyx_v_ogr_crs = __pyx_t_4;

      /* "pyogrio/_io.pyx":444
 *     cdef char *ogr_wkt = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_crs = OGR_L_GetSpatialRef(ogr_layer)
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "pyogrio/_io.pyx":449
 *         ogr_crs = check_pointer(ogr_crs)
 * 
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         # No coordinate system defined.
 *         # This is expected and valid for nonspatial tables.
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 449, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_ErrRestore(0,0,0);

      /* "pyogrio/_io.pyx":452
 *         # No coordinate system defined.
 *         # This is expected and valid for nonspatial tables.
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = ((PyObject*)Py_None); __Pyx_INCREF(Py_None);
      goto __pyx_L6_except_return;
    }

    /* "pyogrio/_io.pyx":454
 *         return None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise CRSError(str(exc))
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 454, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_7, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_7, __pyx_t_6, __pyx_t_5);
    __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.get_crs", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7) < 0) __PYX_ERR(0, 454, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_exc = __pyx_t_6;
      /*try:*/ {

        /* "pyogrio/_io.pyx":455
 * 
 *     except CPLE_BaseError as exc:
 *         raise CRSError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *     # If CRS can be decoded to an EPSG code, use that.
*/
        __pyx_t_10 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_CRSError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 455, __pyx_L19_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 455, __pyx_L19_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
          assert(__pyx_t_10);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
          __pyx_t_13 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_12};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_13, (2-__pyx_t_13) | (__pyx_t_13*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 455, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 455, __pyx_L19_error)
      }

      /* "pyogrio/_io.pyx":454
 *         return None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise CRSError(str(exc))
 * 
*/
      /*finally:*/ {
        __pyx_L19_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __pyx_t_9 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "pyogrio/_io.pyx":444
 *     cdef char *ogr_wkt = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_crs = OGR_L_GetSpatialRef(ogr_layer)
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L0;
    __pyx_L8_try_end:;
  }

  /* "pyogrio/_io.pyx":459
 *     # If CRS can be decoded to an EPSG code, use that.
 *     # The following pointers will be NULL if it cannot be decoded.
 *     _retval = OSRAutoIdentifyEPSG(ogr_crs)             # <<<<<<<<<<<<<<
 *     authority_key = <const char *>OSRGetAuthorityName(ogr_crs, NULL)
 *     authority_val = <const char *>OSRGetAuthorityCode(ogr_crs, NULL)
*/
  __pyx_v__retval = OSRAutoIdentifyEPSG(__pyx_v_ogr_crs);

  /* "pyogrio/_io.pyx":460
 *     # The following pointers will be NULL if it cannot be decoded.
 *     _retval = OSRAutoIdentifyEPSG(ogr_crs)
 *     authority_key = <const char *>OSRGetAuthorityName(ogr_crs, NULL)             # <<<<<<<<<<<<<<
 *     authority_val = <const char *>OSRGetAuthorityCode(ogr_crs, NULL)
 * 
*/
  __pyx_v_authority_key = ((char const *)OSRGetAuthorityName(__pyx_v_ogr_crs, NULL));

  /* "pyogrio/_io.pyx":461
 *     _retval = OSRAutoIdentifyEPSG(ogr_crs)
 *     authority_key = <const char *>OSRGetAuthorityName(ogr_crs, NULL)
 *     authority_val = <const char *>OSRGetAuthorityCode(ogr_crs, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if authority_key != NULL and authority_val != NULL:
*/
  __pyx_v_authority_val = ((char const *)OSRGetAuthorityCode(__pyx_v_ogr_crs, NULL));

  /* "pyogrio/_io.pyx":463
 *     authority_val = <const char *>OSRGetAuthorityCode(ogr_crs, NULL)
 * 
 *     if authority_key != NULL and authority_val != NULL:             # <<<<<<<<<<<<<<
 *         key = get_string(authority_key)
 *         if key == "EPSG":
*/
  __pyx_t_23 = (__pyx_v_authority_key != NULL);
  if (__pyx_t_23) {
  } else {
    __pyx_t_22 = __pyx_t_23;
    goto __pyx_L26_bool_binop_done;
  }
  __pyx_t_23 = (__pyx_v_authority_val != NULL);
  __pyx_t_22 = __pyx_t_23;
  __pyx_L26_bool_binop_done:;
  if (__pyx_t_22) {

    /* "pyogrio/_io.pyx":464
 * 
 *     if authority_key != NULL and authority_val != NULL:
 *         key = get_string(authority_key)             # <<<<<<<<<<<<<<
 *         if key == "EPSG":
 *             value = get_string(authority_val)
*/
    __pyx_t_7 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_v_authority_key, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 464, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_v_key = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":465
 *     if authority_key != NULL and authority_val != NULL:
 *         key = get_string(authority_key)
 *         if key == "EPSG":             # <<<<<<<<<<<<<<
 *             value = get_string(authority_val)
 *             return f"EPSG:{value}"
*/
    __pyx_t_22 = (__Pyx_PyUnicode_Equals(__pyx_v_key, __pyx_mstate_global->__pyx_n_u_EPSG, Py_EQ)); if (unlikely((__pyx_t_22 < 0))) __PYX_ERR(0, 465, __pyx_L1_error)
    if (__pyx_t_22) {

      /* "pyogrio/_io.pyx":466
 *         key = get_string(authority_key)
 *         if key == "EPSG":
 *             value = get_string(authority_val)             # <<<<<<<<<<<<<<
 *             return f"EPSG:{value}"
 * 
*/
      __pyx_t_7 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_v_authority_val, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 466, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_v_value = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "pyogrio/_io.pyx":467
 *         if key == "EPSG":
 *             value = get_string(authority_val)
 *             return f"EPSG:{value}"             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_value, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 467, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_EPSG_2, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 467, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_r = ((PyObject*)__pyx_t_6);
      __pyx_t_6 = 0;
      goto __pyx_L0;

      /* "pyogrio/_io.pyx":465
 *     if authority_key != NULL and authority_val != NULL:
 *         key = get_string(authority_key)
 *         if key == "EPSG":             # <<<<<<<<<<<<<<
 *             value = get_string(authority_val)
 *             return f"EPSG:{value}"
*/
    }

    /* "pyogrio/_io.pyx":463
 *     authority_val = <const char *>OSRGetAuthorityCode(ogr_crs, NULL)
 * 
 *     if authority_key != NULL and authority_val != NULL:             # <<<<<<<<<<<<<<
 *         key = get_string(authority_key)
 *         if key == "EPSG":
*/
  }

  /* "pyogrio/_io.pyx":469
 *             return f"EPSG:{value}"
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         OSRExportToWkt(ogr_crs, &ogr_wkt)
 *         if ogr_wkt == NULL:
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":470
 * 
 *     try:
 *         OSRExportToWkt(ogr_crs, &ogr_wkt)             # <<<<<<<<<<<<<<
 *         if ogr_wkt == NULL:
 *             raise CRSError("CRS could not be extracted as WKT") from None
*/
    (void)(OSRExportToWkt(__pyx_v_ogr_crs, (&__pyx_v_ogr_wkt)));

    /* "pyogrio/_io.pyx":471
 *     try:
 *         OSRExportToWkt(ogr_crs, &ogr_wkt)
 *         if ogr_wkt == NULL:             # <<<<<<<<<<<<<<
 *             raise CRSError("CRS could not be extracted as WKT") from None
 * 
*/
    __pyx_t_22 = (__pyx_v_ogr_wkt == NULL);
    if (unlikely(__pyx_t_22)) {

      /* "pyogrio/_io.pyx":472
 *         OSRExportToWkt(ogr_crs, &ogr_wkt)
 *         if ogr_wkt == NULL:
 *             raise CRSError("CRS could not be extracted as WKT") from None             # <<<<<<<<<<<<<<
 * 
 *         wkt = get_string(ogr_wkt)
*/
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_CRSError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 472, __pyx_L30_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_13 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_13 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_CRS_could_not_be_extracted_as_WK};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_13, (2-__pyx_t_13) | (__pyx_t_13*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 472, __pyx_L30_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_Raise(__pyx_t_6, 0, 0, Py_None);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 472, __pyx_L30_error)

      /* "pyogrio/_io.pyx":471
 *     try:
 *         OSRExportToWkt(ogr_crs, &ogr_wkt)
 *         if ogr_wkt == NULL:             # <<<<<<<<<<<<<<
 *             raise CRSError("CRS could not be extracted as WKT") from None
 * 
*/
    }

    /* "pyogrio/_io.pyx":474
 *             raise CRSError("CRS could not be extracted as WKT") from None
 * 
 *         wkt = get_string(ogr_wkt)             # <<<<<<<<<<<<<<
 * 
 *     finally:
*/
    __pyx_t_6 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_v_ogr_wkt, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 474, __pyx_L30_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_wkt = __pyx_t_6;
    __pyx_t_6 = 0;
  }

  /* "pyogrio/_io.pyx":477
 * 
 *     finally:
 *         CPLFree(ogr_wkt)             # <<<<<<<<<<<<<<
 *         return wkt
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      CPLFree(__pyx_v_ogr_wkt);

      /* "pyogrio/_io.pyx":478
 *     finally:
 *         CPLFree(ogr_wkt)
 *         return wkt             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_6 = __pyx_v_wkt;
      __Pyx_INCREF(__pyx_t_6);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_6))) __PYX_ERR(0, 478, __pyx_L1_error)
      __pyx_r = ((PyObject*)__pyx_t_6);
      __pyx_t_6 = 0;
      goto __pyx_L0;
    }
    __pyx_L30_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_1 = 0; __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_21, &__pyx_t_20, &__pyx_t_19);
      if ( unlikely(__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1) < 0)) __Pyx_ErrFetch(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_19);
      {

        /* "pyogrio/_io.pyx":477
 * 
 *     finally:
 *         CPLFree(ogr_wkt)             # <<<<<<<<<<<<<<
 *         return wkt
 * 
*/
        CPLFree(__pyx_v_ogr_wkt);

        /* "pyogrio/_io.pyx":478
 *     finally:
 *         CPLFree(ogr_wkt)
 *         return wkt             # <<<<<<<<<<<<<<
 * 
 * 
*/
        __Pyx_XDECREF(__pyx_r);
        if (unlikely(!__pyx_v_wkt)) { __Pyx_RaiseUnboundLocalError("wkt"); __PYX_ERR(0, 478, __pyx_L34_error) }
        __pyx_t_6 = __pyx_v_wkt;
        __Pyx_INCREF(__pyx_t_6);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_6))) __PYX_ERR(0, 478, __pyx_L34_error)
        __pyx_r = ((PyObject*)__pyx_t_6);
        __pyx_t_6 = 0;
        goto __pyx_L33_return;
      }
      __pyx_L33_return:;
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_20, __pyx_t_19);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0;
      goto __pyx_L0;
      __pyx_L34_error:;
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_20, __pyx_t_19);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0;
      goto __pyx_L1_error;
    }
  }

  /* "pyogrio/_io.pyx":426
 * 
 * 
 * cdef str get_crs(OGRLayerH ogr_layer):             # <<<<<<<<<<<<<<
 *     """Read CRS from layer as EPSG:<code> if available or WKT.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("pyogrio._io.get_crs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_wkt);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":481
 * 
 * 
 * cdef get_driver(OGRDataSourceH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Get the driver for a dataset.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_driver(OGRDataSourceH __pyx_v_ogr_dataset) {
  void *__pyx_v_ogr_driver;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_driver = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  void *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  char const *__pyx_t_22;
  Py_ssize_t __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_driver", 0);

  /* "pyogrio/_io.pyx":501
 *     cdef void *ogr_driver
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_driver = GDALGetDatasetDriver(ogr_dataset)
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "pyogrio/_io.pyx":502
 * 
 *     try:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_driver = GDALGetDatasetDriver(ogr_dataset)
 *         ogr_driver = check_pointer(ogr_driver)
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "pyogrio/_io.pyx":503
 *     try:
 *         with nogil:
 *             ogr_driver = GDALGetDatasetDriver(ogr_dataset)             # <<<<<<<<<<<<<<
 *         ogr_driver = check_pointer(ogr_driver)
 * 
*/
            __pyx_v_ogr_driver = GDALGetDatasetDriver(__pyx_v_ogr_dataset);
          }

          /* "pyogrio/_io.pyx":502
 * 
 *     try:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_driver = GDALGetDatasetDriver(ogr_dataset)
 *         ogr_driver = check_pointer(ogr_driver)
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "pyogrio/_io.pyx":504
 *         with nogil:
 *             ogr_driver = GDALGetDatasetDriver(ogr_dataset)
 *         ogr_driver = check_pointer(ogr_driver)             # <<<<<<<<<<<<<<
 * 
 *     except NullPointerError:
*/
      __pyx_t_4 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_driver); if (unlikely(__pyx_t_4 == ((void *)NULL))) __PYX_ERR(0, 504, __pyx_L3_error)
      __pyx_v_ogr_driver = __pyx_t_4;

      /* "pyogrio/_io.pyx":501
 *     cdef void *ogr_driver
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_driver = GDALGetDatasetDriver(ogr_dataset)
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "pyogrio/_io.pyx":506
 *         ogr_driver = check_pointer(ogr_driver)
 * 
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         raise DataLayerError("Could not detect driver of dataset") from None
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 506, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.get_driver", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 506, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);

      /* "pyogrio/_io.pyx":507
 * 
 *     except NullPointerError:
 *         raise DataLayerError("Could not detect driver of dataset") from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
      __pyx_t_10 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 507, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
        assert(__pyx_t_10);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
        __pyx_t_12 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_mstate_global->__pyx_kp_u_Could_not_detect_driver_of_datas};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 507, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 507, __pyx_L5_except_error)
    }

    /* "pyogrio/_io.pyx":509
 *         raise DataLayerError("Could not detect driver of dataset") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 509, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.get_driver", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 509, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_exc = __pyx_t_6;
      /*try:*/ {

        /* "pyogrio/_io.pyx":510
 * 
 *     except CPLE_BaseError as exc:
 *         raise DataLayerError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *     driver = OGR_Dr_GetName(ogr_driver).decode("UTF-8")
*/
        __pyx_t_11 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 510, __pyx_L19_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_13 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 510, __pyx_L19_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_12 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
          assert(__pyx_t_11);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
          __pyx_t_12 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_13};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 510, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 510, __pyx_L19_error)
      }

      /* "pyogrio/_io.pyx":509
 *         raise DataLayerError("Could not detect driver of dataset") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
      /*finally:*/ {
        __pyx_L19_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __pyx_t_9 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "pyogrio/_io.pyx":501
 *     cdef void *ogr_driver
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_driver = GDALGetDatasetDriver(ogr_dataset)
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "pyogrio/_io.pyx":512
 *         raise DataLayerError(str(exc))
 * 
 *     driver = OGR_Dr_GetName(ogr_driver).decode("UTF-8")             # <<<<<<<<<<<<<<
 *     return driver
 * 
*/
  __pyx_t_22 = OGR_Dr_GetName(__pyx_v_ogr_driver);
  __pyx_t_23 = __Pyx_ssize_strlen(__pyx_t_22); if (unlikely(__pyx_t_23 == ((Py_ssize_t)-1))) __PYX_ERR(0, 512, __pyx_L1_error)
  __pyx_t_5 = __Pyx_decode_c_string(__pyx_t_22, 0, __pyx_t_23, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_t_5);
  __pyx_v_driver = __pyx_t_5;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":513
 * 
 *     driver = OGR_Dr_GetName(ogr_driver).decode("UTF-8")
 *     return driver             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_driver);
  __pyx_r = __pyx_v_driver;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":481
 * 
 * 
 * cdef get_driver(OGRDataSourceH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Get the driver for a dataset.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyogrio._io.get_driver", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_driver);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":516
 * 
 * 
 * cdef get_feature_count(OGRLayerH ogr_layer, int force):             # <<<<<<<<<<<<<<
 *     """Get the feature count of a layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_feature_count(OGRLayerH __pyx_v_ogr_layer, int __pyx_v_force) {
  OGRFeatureH __pyx_v_ogr_feature;
  int __pyx_v_feature_count;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  void *__pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  size_t __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  char const *__pyx_t_18;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  char const *__pyx_t_25;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_feature_count", 0);

  /* "pyogrio/_io.pyx":535
 * 
 *     """
 *     cdef OGRFeatureH ogr_feature = NULL             # <<<<<<<<<<<<<<
 *     cdef int feature_count
 * 
*/
  __pyx_v_ogr_feature = NULL;

  /* "pyogrio/_io.pyx":538
 *     cdef int feature_count
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         feature_count = OGR_L_GetFeatureCount(ogr_layer, force)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":539
 * 
 *     with nogil:
 *         feature_count = OGR_L_GetFeatureCount(ogr_layer, force)             # <<<<<<<<<<<<<<
 * 
 *     # if GDAL refuses to give us the feature count, we have to loop over all
*/
        __pyx_v_feature_count = OGR_L_GetFeatureCount(__pyx_v_ogr_layer, __pyx_v_force);
      }

      /* "pyogrio/_io.pyx":538
 *     cdef int feature_count
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         feature_count = OGR_L_GetFeatureCount(ogr_layer, force)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":544
 *     # features ourselves and get the count.  This can happen for some drivers
 *     # (e.g., OSM) or if a where clause is invalid but not rejected as error
 *     if force and feature_count == -1:             # <<<<<<<<<<<<<<
 *         # make sure layer is read from beginning
 *         with nogil:
*/
  __pyx_t_2 = (__pyx_v_force != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_feature_count == -1L);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":546
 *     if force and feature_count == -1:
 *         # make sure layer is read from beginning
 *         with nogil:             # <<<<<<<<<<<<<<
 *             OGR_L_ResetReading(ogr_layer)
 * 
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "pyogrio/_io.pyx":547
 *         # make sure layer is read from beginning
 *         with nogil:
 *             OGR_L_ResetReading(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         feature_count = 0
*/
          OGR_L_ResetReading(__pyx_v_ogr_layer);
        }

        /* "pyogrio/_io.pyx":546
 *     if force and feature_count == -1:
 *         # make sure layer is read from beginning
 *         with nogil:             # <<<<<<<<<<<<<<
 *             OGR_L_ResetReading(ogr_layer)
 * 
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L11;
          }
          __pyx_L11:;
        }
    }

    /* "pyogrio/_io.pyx":549
 *             OGR_L_ResetReading(ogr_layer)
 * 
 *         feature_count = 0             # <<<<<<<<<<<<<<
 * 
 *         try:
*/
    __pyx_v_feature_count = 0;

    /* "pyogrio/_io.pyx":551
 *         feature_count = 0
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 while True:
*/
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_5);
        /*try:*/ {

          /* "pyogrio/_io.pyx":552
 * 
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 while True:
 *                     ogr_feature = OGR_L_GetNextFeature(ogr_layer)
*/
          {
              PyThreadState * _save;
              _save = PyEval_SaveThread();
              __Pyx_FastGIL_Remember();
              /*try:*/ {

                /* "pyogrio/_io.pyx":553
 *         try:
 *             with nogil:
 *                 while True:             # <<<<<<<<<<<<<<
 *                     ogr_feature = OGR_L_GetNextFeature(ogr_layer)
 *                     ogr_feature = check_pointer(ogr_feature)
*/
                while (1) {

                  /* "pyogrio/_io.pyx":554
 *             with nogil:
 *                 while True:
 *                     ogr_feature = OGR_L_GetNextFeature(ogr_layer)             # <<<<<<<<<<<<<<
 *                     ogr_feature = check_pointer(ogr_feature)
 *                     feature_count +=1
*/
                  __pyx_v_ogr_feature = OGR_L_GetNextFeature(__pyx_v_ogr_layer);

                  /* "pyogrio/_io.pyx":555
 *                 while True:
 *                     ogr_feature = OGR_L_GetNextFeature(ogr_layer)
 *                     ogr_feature = check_pointer(ogr_feature)             # <<<<<<<<<<<<<<
 *                     feature_count +=1
 * 
*/
                  __pyx_t_6 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_feature); if (unlikely(__pyx_t_6 == ((void *)NULL))) __PYX_ERR(0, 555, __pyx_L22_error)
                  __pyx_v_ogr_feature = __pyx_t_6;

                  /* "pyogrio/_io.pyx":556
 *                     ogr_feature = OGR_L_GetNextFeature(ogr_layer)
 *                     ogr_feature = check_pointer(ogr_feature)
 *                     feature_count +=1             # <<<<<<<<<<<<<<
 * 
 *         except NullPointerError:
*/
                  __pyx_v_feature_count = (__pyx_v_feature_count + 1);
                }
              }

              /* "pyogrio/_io.pyx":552
 * 
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 while True:
 *                     ogr_feature = OGR_L_GetNextFeature(ogr_layer)
*/
              /*finally:*/ {
                /*normal exit:*/{
                  __Pyx_FastGIL_Forget();
                  PyEval_RestoreThread(_save);
                  goto __pyx_L23;
                }
                __pyx_L22_error: {
                  __Pyx_FastGIL_Forget();
                  PyEval_RestoreThread(_save);
                  goto __pyx_L15_error;
                }
                __pyx_L23:;
              }
          }

          /* "pyogrio/_io.pyx":551
 *         feature_count = 0
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 while True:
*/
        }
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        goto __pyx_L20_try_end;
        __pyx_L15_error:;

        /* "pyogrio/_io.pyx":558
 *                     feature_count +=1
 * 
 *         except NullPointerError:             # <<<<<<<<<<<<<<
 *             # No more rows available, so stop reading
 *             pass
*/
        __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 558, __pyx_L17_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_7, __pyx_t_10);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0;
        if (__pyx_t_11) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L16_exception_handled;
        }

        /* "pyogrio/_io.pyx":565
 *         # increasing, the default config option OSM_USE_CUSTOM_INDEXING=YES
 *         # causes errors iterating over features
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             # if an invalid where clause is used for a GPKG file, it is not
 *             # caught as an error until attempting to iterate over features;
*/
        __Pyx_ErrFetch(&__pyx_t_9, &__pyx_t_8, &__pyx_t_7);
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 565, __pyx_L17_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_9, __pyx_t_10);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_ErrRestore(__pyx_t_9, __pyx_t_8, __pyx_t_7);
        __pyx_t_9 = 0; __pyx_t_8 = 0; __pyx_t_7 = 0;
        if (__pyx_t_11) {
          __Pyx_AddTraceback("pyogrio._io.get_feature_count", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9) < 0) __PYX_ERR(0, 565, __pyx_L17_except_error)
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_8);
          __pyx_v_exc = __pyx_t_8;
          /*try:*/ {

            /* "pyogrio/_io.pyx":569
 *             # caught as an error until attempting to iterate over features;
 *             # catch it here
 *             if "failed to prepare SQL" in str(exc):             # <<<<<<<<<<<<<<
 *                 raise ValueError(f"Invalid SQL query: {str(exc)}") from None
 * 
*/
            __pyx_t_10 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 569, __pyx_L33_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_1 = (__Pyx_PyUnicode_ContainsTF(__pyx_mstate_global->__pyx_kp_u_failed_to_prepare_SQL, __pyx_t_10, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 569, __pyx_L33_error)
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(__pyx_t_1)) {

              /* "pyogrio/_io.pyx":570
 *             # catch it here
 *             if "failed to prepare SQL" in str(exc):
 *                 raise ValueError(f"Invalid SQL query: {str(exc)}") from None             # <<<<<<<<<<<<<<
 * 
 *             raise DataLayerError(
*/
              __pyx_t_12 = NULL;
              __pyx_t_13 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 570, __pyx_L33_error)
              __Pyx_GOTREF(__pyx_t_13);
              __pyx_t_14 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Invalid_SQL_query, __pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 570, __pyx_L33_error)
              __Pyx_GOTREF(__pyx_t_14);
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              __pyx_t_15 = 1;
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_t_14};
                __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_15, (2-__pyx_t_15) | (__pyx_t_15*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 570, __pyx_L33_error)
                __Pyx_GOTREF(__pyx_t_10);
              }
              __Pyx_Raise(__pyx_t_10, 0, 0, Py_None);
              __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
              __PYX_ERR(0, 570, __pyx_L33_error)

              /* "pyogrio/_io.pyx":569
 *             # caught as an error until attempting to iterate over features;
 *             # catch it here
 *             if "failed to prepare SQL" in str(exc):             # <<<<<<<<<<<<<<
 *                 raise ValueError(f"Invalid SQL query: {str(exc)}") from None
 * 
*/
            }

            /* "pyogrio/_io.pyx":572
 *                 raise ValueError(f"Invalid SQL query: {str(exc)}") from None
 * 
 *             raise DataLayerError(             # <<<<<<<<<<<<<<
 *                 f"Could not iterate over features: {str(exc)}"
 *             ) from None
*/
            __pyx_t_14 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 572, __pyx_L33_error)
            __Pyx_GOTREF(__pyx_t_12);

            /* "pyogrio/_io.pyx":573
 * 
 *             raise DataLayerError(
 *                 f"Could not iterate over features: {str(exc)}"             # <<<<<<<<<<<<<<
 *             ) from None
 * 
*/
            __pyx_t_13 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 573, __pyx_L33_error)
            __Pyx_GOTREF(__pyx_t_13);
            __pyx_t_16 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Could_not_iterate_over_features, __pyx_t_13); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 573, __pyx_L33_error)
            __Pyx_GOTREF(__pyx_t_16);
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            __pyx_t_15 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_12))) {
              __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_12);
              assert(__pyx_t_14);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
              __Pyx_INCREF(__pyx_t_14);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
              __pyx_t_15 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_16};
              __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_15, (2-__pyx_t_15) | (__pyx_t_15*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
              if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 572, __pyx_L33_error)
              __Pyx_GOTREF(__pyx_t_10);
            }

            /* "pyogrio/_io.pyx":574
 *             raise DataLayerError(
 *                 f"Could not iterate over features: {str(exc)}"
 *             ) from None             # <<<<<<<<<<<<<<
 * 
 *         finally:
*/
            __Pyx_Raise(__pyx_t_10, 0, 0, Py_None);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __PYX_ERR(0, 572, __pyx_L33_error)
          }

          /* "pyogrio/_io.pyx":565
 *         # increasing, the default config option OSM_USE_CUSTOM_INDEXING=YES
 *         # causes errors iterating over features
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             # if an invalid where clause is used for a GPKG file, it is not
 *             # caught as an error until attempting to iterate over features;
*/
          /*finally:*/ {
            __pyx_L33_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
               __Pyx_ExceptionSwap(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
              if ( unlikely(__Pyx_GetException(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21) < 0)) __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
              __Pyx_XGOTREF(__pyx_t_19);
              __Pyx_XGOTREF(__pyx_t_20);
              __Pyx_XGOTREF(__pyx_t_21);
              __Pyx_XGOTREF(__pyx_t_22);
              __Pyx_XGOTREF(__pyx_t_23);
              __Pyx_XGOTREF(__pyx_t_24);
              __pyx_t_11 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_18 = __pyx_filename;
              {
                __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_22);
              __Pyx_XGIVEREF(__pyx_t_23);
              __Pyx_XGIVEREF(__pyx_t_24);
              __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_23, __pyx_t_24);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_XGIVEREF(__pyx_t_20);
              __Pyx_XGIVEREF(__pyx_t_21);
              __Pyx_ErrRestore(__pyx_t_19, __pyx_t_20, __pyx_t_21);
              __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
              __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_18;
              goto __pyx_L17_except_error;
            }
          }
        }
        goto __pyx_L17_except_error;

        /* "pyogrio/_io.pyx":551
 *         feature_count = 0
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 while True:
*/
        __pyx_L17_except_error:;
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
        goto __pyx_L13_error;
        __pyx_L16_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
        __pyx_L20_try_end:;
      }
    }

    /* "pyogrio/_io.pyx":577
 * 
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
    /*finally:*/ {
      /*normal exit:*/{
        __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":578
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
          OGR_F_Destroy(__pyx_v_ogr_feature);

          /* "pyogrio/_io.pyx":579
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     return feature_count
*/
          __pyx_v_ogr_feature = NULL;

          /* "pyogrio/_io.pyx":577
 * 
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
        }
        goto __pyx_L14;
      }
      __pyx_L13_error:;
      /*exception exit:*/{
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __pyx_t_5 = 0; __pyx_t_4 = 0; __pyx_t_3 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
         __Pyx_ExceptionSwap(&__pyx_t_24, &__pyx_t_23, &__pyx_t_22);
        if ( unlikely(__Pyx_GetException(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3) < 0)) __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_24);
        __Pyx_XGOTREF(__pyx_t_23);
        __Pyx_XGOTREF(__pyx_t_22);
        __pyx_t_17 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_25 = __pyx_filename;
        {
          __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
          if (__pyx_t_1) {

            /* "pyogrio/_io.pyx":578
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
            OGR_F_Destroy(__pyx_v_ogr_feature);

            /* "pyogrio/_io.pyx":579
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     return feature_count
*/
            __pyx_v_ogr_feature = NULL;

            /* "pyogrio/_io.pyx":577
 * 
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
          }
        }
        __Pyx_XGIVEREF(__pyx_t_24);
        __Pyx_XGIVEREF(__pyx_t_23);
        __Pyx_XGIVEREF(__pyx_t_22);
        __Pyx_ExceptionReset(__pyx_t_24, __pyx_t_23, __pyx_t_22);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_ErrRestore(__pyx_t_5, __pyx_t_4, __pyx_t_3);
        __pyx_t_5 = 0; __pyx_t_4 = 0; __pyx_t_3 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0;
        __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_25;
        goto __pyx_L1_error;
      }
      __pyx_L14:;
    }

    /* "pyogrio/_io.pyx":544
 *     # features ourselves and get the count.  This can happen for some drivers
 *     # (e.g., OSM) or if a where clause is invalid but not rejected as error
 *     if force and feature_count == -1:             # <<<<<<<<<<<<<<
 *         # make sure layer is read from beginning
 *         with nogil:
*/
  }

  /* "pyogrio/_io.pyx":581
 *                 ogr_feature = NULL
 * 
 *     return feature_count             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __Pyx_PyLong_From_int(__pyx_v_feature_count); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":516
 * 
 * 
 * cdef get_feature_count(OGRLayerH ogr_layer, int force):             # <<<<<<<<<<<<<<
 *     """Get the feature count of a layer.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("pyogrio._io.get_feature_count", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":584
 * 
 * 
 * cdef get_total_bounds(OGRLayerH ogr_layer, int force):             # <<<<<<<<<<<<<<
 *     """Get the total bounds of a layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_total_bounds(OGRLayerH __pyx_v_ogr_layer, int __pyx_v_force) {
  OGREnvelope __pyx_v_ogr_envelope;
  OGRErr __pyx_v_err;
  PyObject *__pyx_v_bounds = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_total_bounds", 0);

  /* "pyogrio/_io.pyx":601
 *     cdef OGREnvelope ogr_envelope
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         err = OGR_L_GetExtent(ogr_layer, &ogr_envelope, force)
 *     if err == OGRERR_NONE:
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":602
 * 
 *     with nogil:
 *         err = OGR_L_GetExtent(ogr_layer, &ogr_envelope, force)             # <<<<<<<<<<<<<<
 *     if err == OGRERR_NONE:
 *         bounds = (
*/
        __pyx_v_err = OGR_L_GetExtent(__pyx_v_ogr_layer, (&__pyx_v_ogr_envelope), __pyx_v_force);
      }

      /* "pyogrio/_io.pyx":601
 *     cdef OGREnvelope ogr_envelope
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         err = OGR_L_GetExtent(ogr_layer, &ogr_envelope, force)
 *     if err == OGRERR_NONE:
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":603
 *     with nogil:
 *         err = OGR_L_GetExtent(ogr_layer, &ogr_envelope, force)
 *     if err == OGRERR_NONE:             # <<<<<<<<<<<<<<
 *         bounds = (
 *            ogr_envelope.MinX, ogr_envelope.MinY, ogr_envelope.MaxX, ogr_envelope.MaxY
*/
  __pyx_t_1 = (__pyx_v_err == OGRERR_NONE);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":605
 *     if err == OGRERR_NONE:
 *         bounds = (
 *            ogr_envelope.MinX, ogr_envelope.MinY, ogr_envelope.MaxX, ogr_envelope.MaxY             # <<<<<<<<<<<<<<
 *         )
 *     else:
*/
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MinX); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MinY); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MaxX); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MaxY); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2) != (0)) __PYX_ERR(0, 605, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3) != (0)) __PYX_ERR(0, 605, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_4) != (0)) __PYX_ERR(0, 605, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_5) != (0)) __PYX_ERR(0, 605, __pyx_L1_error);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_v_bounds = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "pyogrio/_io.pyx":603
 *     with nogil:
 *         err = OGR_L_GetExtent(ogr_layer, &ogr_envelope, force)
 *     if err == OGRERR_NONE:             # <<<<<<<<<<<<<<
 *         bounds = (
 *            ogr_envelope.MinX, ogr_envelope.MinY, ogr_envelope.MaxX, ogr_envelope.MaxY
*/
    goto __pyx_L6;
  }

  /* "pyogrio/_io.pyx":608
 *         )
 *     else:
 *         bounds = None             # <<<<<<<<<<<<<<
 * 
 *     return bounds
*/
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __pyx_v_bounds = Py_None;
  }
  __pyx_L6:;

  /* "pyogrio/_io.pyx":610
 *         bounds = None
 * 
 *     return bounds             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_bounds);
  __pyx_r = __pyx_v_bounds;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":584
 * 
 * 
 * cdef get_total_bounds(OGRLayerH ogr_layer, int force):             # <<<<<<<<<<<<<<
 *     """Get the total bounds of a layer.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyogrio._io.get_total_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_bounds);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":613
 * 
 * 
 * cdef set_metadata(GDALMajorObjectH obj, object metadata):             # <<<<<<<<<<<<<<
 *     """Set metadata on a dataset or layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_set_metadata(GDALMajorObjectH __pyx_v_obj, PyObject *__pyx_v_metadata) {
  char **__pyx_v_metadata_items;
  int __pyx_v_err;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char **__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_metadata", 0);

  /* "pyogrio/_io.pyx":624
 * 
 *     """
 *     cdef char **metadata_items = NULL             # <<<<<<<<<<<<<<
 *     cdef int err = 0
 * 
*/
  __pyx_v_metadata_items = NULL;

  /* "pyogrio/_io.pyx":625
 *     """
 *     cdef char **metadata_items = NULL
 *     cdef int err = 0             # <<<<<<<<<<<<<<
 * 
 *     metadata_items = dict_to_options(metadata)
*/
  __pyx_v_err = 0;

  /* "pyogrio/_io.pyx":627
 *     cdef int err = 0
 * 
 *     metadata_items = dict_to_options(metadata)             # <<<<<<<<<<<<<<
 *     if metadata_items != NULL:
 *         # only default namepace is currently supported
*/
  __pyx_t_1 = __pyx_f_7pyogrio_3_io_dict_to_options(__pyx_v_metadata); if (unlikely(__pyx_t_1 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 627, __pyx_L1_error)
  __pyx_v_metadata_items = __pyx_t_1;

  /* "pyogrio/_io.pyx":628
 * 
 *     metadata_items = dict_to_options(metadata)
 *     if metadata_items != NULL:             # <<<<<<<<<<<<<<
 *         # only default namepace is currently supported
 *         err = GDALSetMetadata(obj, metadata_items, NULL)
*/
  __pyx_t_2 = (__pyx_v_metadata_items != NULL);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":630
 *     if metadata_items != NULL:
 *         # only default namepace is currently supported
 *         err = GDALSetMetadata(obj, metadata_items, NULL)             # <<<<<<<<<<<<<<
 * 
 *         CSLDestroy(metadata_items)
*/
    __pyx_v_err = GDALSetMetadata(__pyx_v_obj, __pyx_v_metadata_items, NULL);

    /* "pyogrio/_io.pyx":632
 *         err = GDALSetMetadata(obj, metadata_items, NULL)
 * 
 *         CSLDestroy(metadata_items)             # <<<<<<<<<<<<<<
 *         metadata_items = NULL
 * 
*/
    CSLDestroy(__pyx_v_metadata_items);

    /* "pyogrio/_io.pyx":633
 * 
 *         CSLDestroy(metadata_items)
 *         metadata_items = NULL             # <<<<<<<<<<<<<<
 * 
 *     if err:
*/
    __pyx_v_metadata_items = NULL;

    /* "pyogrio/_io.pyx":628
 * 
 *     metadata_items = dict_to_options(metadata)
 *     if metadata_items != NULL:             # <<<<<<<<<<<<<<
 *         # only default namepace is currently supported
 *         err = GDALSetMetadata(obj, metadata_items, NULL)
*/
  }

  /* "pyogrio/_io.pyx":635
 *         metadata_items = NULL
 * 
 *     if err:             # <<<<<<<<<<<<<<
 *         raise RuntimeError("Could not set metadata") from None
 * 
*/
  __pyx_t_2 = (__pyx_v_err != 0);
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":636
 * 
 *     if err:
 *         raise RuntimeError("Could not set metadata") from None             # <<<<<<<<<<<<<<
 * 
 * cdef get_metadata(GDALMajorObjectH obj):
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Could_not_set_metadata};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 636, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, Py_None);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 636, __pyx_L1_error)

    /* "pyogrio/_io.pyx":635
 *         metadata_items = NULL
 * 
 *     if err:             # <<<<<<<<<<<<<<
 *         raise RuntimeError("Could not set metadata") from None
 * 
*/
  }

  /* "pyogrio/_io.pyx":613
 * 
 * 
 * cdef set_metadata(GDALMajorObjectH obj, object metadata):             # <<<<<<<<<<<<<<
 *     """Set metadata on a dataset or layer.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyogrio._io.set_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyogrio_3_io_12get_metadata_2generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyogrio/_io.pyx":657
 *     if metadata != NULL:
 *         return dict(
 *             metadata[i].decode("UTF-8").split("=", 1)             # <<<<<<<<<<<<<<
 *             for i in range(CSLCount(metadata))
 *         )
*/

static PyObject *__pyx_pf_7pyogrio_3_io_12get_metadata_genexpr(PyObject *__pyx_self, int __pyx_genexpr_arg_0) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_1_genexpr(__pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 657, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata *) __pyx_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_outer_scope);
  __pyx_cur_scope->__pyx_genexpr_arg_0 = __pyx_genexpr_arg_0;
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyogrio_3_io_12get_metadata_2generator1, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_genexpr, __pyx_mstate_global->__pyx_n_u_get_metadata_locals_genexpr, __pyx_mstate_global->__pyx_n_u_pyogrio__io); if (unlikely(!gen)) __PYX_ERR(0, 657, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyogrio._io.get_metadata.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyogrio_3_io_12get_metadata_2generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(0, 657, __pyx_L1_error)
  }

  /* "pyogrio/_io.pyx":658
 *         return dict(
 *             metadata[i].decode("UTF-8").split("=", 1)
 *             for i in range(CSLCount(metadata))             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_1 = __pyx_cur_scope->__pyx_genexpr_arg_0;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_3;

    /* "pyogrio/_io.pyx":657
 *     if metadata != NULL:
 *         return dict(
 *             metadata[i].decode("UTF-8").split("=", 1)             # <<<<<<<<<<<<<<
 *             for i in range(CSLCount(metadata))
 *         )
*/
    __pyx_t_4 = (__pyx_cur_scope->__pyx_outer_scope->__pyx_v_metadata[__pyx_cur_scope->__pyx_v_i]);
    __pyx_t_5 = __Pyx_ssize_strlen(__pyx_t_4); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 657, __pyx_L1_error)
    __pyx_t_6 = __Pyx_decode_c_string(__pyx_t_4, 0, __pyx_t_5, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyUnicode_Split(__pyx_t_6, __pyx_mstate_global->__pyx_kp_u__7, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 657, __pyx_L1_error)
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":638
 *         raise RuntimeError("Could not set metadata") from None
 * 
 * cdef get_metadata(GDALMajorObjectH obj):             # <<<<<<<<<<<<<<
 *     """Get metadata for a dataset or layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_metadata(GDALMajorObjectH __pyx_v_obj) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata *__pyx_cur_scope;
  PyObject *__pyx_gb_7pyogrio_3_io_12get_metadata_2generator1 = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_metadata", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata(__pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 638, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }

  /* "pyogrio/_io.pyx":653
 *     """
 *     # only default namespace is currently supported
 *     cdef char **metadata = GDALGetMetadata(obj, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if metadata != NULL:
*/
  __pyx_cur_scope->__pyx_v_metadata = GDALGetMetadata(__pyx_v_obj, NULL);

  /* "pyogrio/_io.pyx":655
 *     cdef char **metadata = GDALGetMetadata(obj, NULL)
 * 
 *     if metadata != NULL:             # <<<<<<<<<<<<<<
 *         return dict(
 *             metadata[i].decode("UTF-8").split("=", 1)
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_metadata != NULL);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":656
 * 
 *     if metadata != NULL:
 *         return dict(             # <<<<<<<<<<<<<<
 *             metadata[i].decode("UTF-8").split("=", 1)
 *             for i in range(CSLCount(metadata))
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = NULL;

    /* "pyogrio/_io.pyx":657
 *     if metadata != NULL:
 *         return dict(
 *             metadata[i].decode("UTF-8").split("=", 1)             # <<<<<<<<<<<<<<
 *             for i in range(CSLCount(metadata))
 *         )
*/
    __pyx_t_4 = __pyx_pf_7pyogrio_3_io_12get_metadata_genexpr(((PyObject*)__pyx_cur_scope), CSLCount(__pyx_cur_scope->__pyx_v_metadata)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(&PyDict_Type), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":655
 *     cdef char **metadata = GDALGetMetadata(obj, NULL)
 * 
 *     if metadata != NULL:             # <<<<<<<<<<<<<<
 *         return dict(
 *             metadata[i].decode("UTF-8").split("=", 1)
*/
  }

  /* "pyogrio/_io.pyx":661
 *         )
 * 
 *     return None             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":638
 *         raise RuntimeError("Could not set metadata") from None
 * 
 * cdef get_metadata(GDALMajorObjectH obj):             # <<<<<<<<<<<<<<
 *     """Get metadata for a dataset or layer.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyogrio._io.get_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_gb_7pyogrio_3_io_12get_metadata_2generator1);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":664
 * 
 * 
 * cdef detect_encoding(OGRDataSourceH ogr_dataset, OGRLayerH ogr_layer):             # <<<<<<<<<<<<<<
 *     """Attempt to detect the encoding to use to read/write string values.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_detect_encoding(OGRDataSourceH __pyx_v_ogr_dataset, OGRLayerH __pyx_v_ogr_layer) {
  PyObject *__pyx_v_driver = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("detect_encoding", 0);

  /* "pyogrio/_io.pyx":684
 * 
 *     """
 *     if OGR_L_TestCapability(ogr_layer, OLCStringsAsUTF8):             # <<<<<<<<<<<<<<
 *         # OGR_L_TestCapability returns True for OLCStringsAsUTF8 if GDAL hides encoding
 *         # complexities for this layer/driver type. In this case all string attribute
*/
  __pyx_t_1 = (OGR_L_TestCapability(__pyx_v_ogr_layer, OLCStringsAsUTF8) != 0);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":696
 *         # when executing SQL, even when it supports UTF-8 (patched below);
 *         # this may be fixed by https://github.com/OSGeo/gdal/pull/9649 (GDAL >=3.9.0?)
 *         return "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *     driver = get_driver(ogr_dataset)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_UTF_8;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":684
 * 
 *     """
 *     if OGR_L_TestCapability(ogr_layer, OLCStringsAsUTF8):             # <<<<<<<<<<<<<<
 *         # OGR_L_TestCapability returns True for OLCStringsAsUTF8 if GDAL hides encoding
 *         # complexities for this layer/driver type. In this case all string attribute
*/
  }

  /* "pyogrio/_io.pyx":698
 *         return "UTF-8"
 * 
 *     driver = get_driver(ogr_dataset)             # <<<<<<<<<<<<<<
 *     if driver == "ESRI Shapefile":
 *         # OGR_L_TestCapability returns True for OLCStringsAsUTF8 (above) for
*/
  __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_driver(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_driver = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":699
 * 
 *     driver = get_driver(ogr_dataset)
 *     if driver == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *         # OGR_L_TestCapability returns True for OLCStringsAsUTF8 (above) for
 *         # shapefiles when a .cpg file is present with a valid encoding, or GDAL
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 699, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":711
 *         # SQL results from shapefiles are fixed (above)
 *         # This block should only be used for unfixed versions of GDAL (<3.9.0?)
 *         if OGR_L_TestCapability(GDALDatasetGetLayer(ogr_dataset, 0), OLCStringsAsUTF8):             # <<<<<<<<<<<<<<
 *             return "UTF-8"
 * 
*/
    __pyx_t_1 = (OGR_L_TestCapability(GDALDatasetGetLayer(__pyx_v_ogr_dataset, 0), OLCStringsAsUTF8) != 0);
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":712
 *         # This block should only be used for unfixed versions of GDAL (<3.9.0?)
 *         if OGR_L_TestCapability(GDALDatasetGetLayer(ogr_dataset, 0), OLCStringsAsUTF8):
 *             return "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *         return "ISO-8859-1"
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
      __pyx_r = __pyx_mstate_global->__pyx_kp_u_UTF_8;
      goto __pyx_L0;

      /* "pyogrio/_io.pyx":711
 *         # SQL results from shapefiles are fixed (above)
 *         # This block should only be used for unfixed versions of GDAL (<3.9.0?)
 *         if OGR_L_TestCapability(GDALDatasetGetLayer(ogr_dataset, 0), OLCStringsAsUTF8):             # <<<<<<<<<<<<<<
 *             return "UTF-8"
 * 
*/
    }

    /* "pyogrio/_io.pyx":714
 *             return "UTF-8"
 * 
 *         return "ISO-8859-1"             # <<<<<<<<<<<<<<
 * 
 *     if driver == "OSM":
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_ISO_8859_1);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_ISO_8859_1;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":699
 * 
 *     driver = get_driver(ogr_dataset)
 *     if driver == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *         # OGR_L_TestCapability returns True for OLCStringsAsUTF8 (above) for
 *         # shapefiles when a .cpg file is present with a valid encoding, or GDAL
*/
  }

  /* "pyogrio/_io.pyx":716
 *         return "ISO-8859-1"
 * 
 *     if driver == "OSM":             # <<<<<<<<<<<<<<
 *         # always set OSM data to UTF-8
 *         # per https://help.openstreetmap.org/questions/2172/what-encoding-does-openstreetmap-use  # noqa: E501
*/
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_OSM, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 716, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":719
 *         # always set OSM data to UTF-8
 *         # per https://help.openstreetmap.org/questions/2172/what-encoding-does-openstreetmap-use  # noqa: E501
 *         return "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *     if driver in ("XLSX", "ODS"):
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_UTF_8;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":716
 *         return "ISO-8859-1"
 * 
 *     if driver == "OSM":             # <<<<<<<<<<<<<<
 *         # always set OSM data to UTF-8
 *         # per https://help.openstreetmap.org/questions/2172/what-encoding-does-openstreetmap-use  # noqa: E501
*/
  }

  /* "pyogrio/_io.pyx":721
 *         return "UTF-8"
 * 
 *     if driver in ("XLSX", "ODS"):             # <<<<<<<<<<<<<<
 *         # TestCapability for OLCStringsAsUTF8 for XLSX and ODS was False for new files
 *         # being created for GDAL < 3.8.5. Once these versions of GDAL are no longer
*/
  __Pyx_INCREF(__pyx_v_driver);
  __pyx_t_2 = __pyx_v_driver;
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_XLSX, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 721, __pyx_L1_error)
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ODS, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 721, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_3;
  __pyx_L8_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_t_1;
  if (__pyx_t_3) {

    /* "pyogrio/_io.pyx":725
 *         # being created for GDAL < 3.8.5. Once these versions of GDAL are no longer
 *         # supported, this can be removed.
 *         return "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *     if driver == "GeoJSONSeq":
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_UTF_8;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":721
 *         return "UTF-8"
 * 
 *     if driver in ("XLSX", "ODS"):             # <<<<<<<<<<<<<<
 *         # TestCapability for OLCStringsAsUTF8 for XLSX and ODS was False for new files
 *         # being created for GDAL < 3.8.5. Once these versions of GDAL are no longer
*/
  }

  /* "pyogrio/_io.pyx":727
 *         return "UTF-8"
 * 
 *     if driver == "GeoJSONSeq":             # <<<<<<<<<<<<<<
 *         # In old gdal versions, OLCStringsAsUTF8 wasn't advertised yet.
 *         return "UTF-8"
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_GeoJSONSeq, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 727, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "pyogrio/_io.pyx":729
 *     if driver == "GeoJSONSeq":
 *         # In old gdal versions, OLCStringsAsUTF8 wasn't advertised yet.
 *         return "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *     if driver == "SQLite":
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_UTF_8;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":727
 *         return "UTF-8"
 * 
 *     if driver == "GeoJSONSeq":             # <<<<<<<<<<<<<<
 *         # In old gdal versions, OLCStringsAsUTF8 wasn't advertised yet.
 *         return "UTF-8"
*/
  }

  /* "pyogrio/_io.pyx":731
 *         return "UTF-8"
 * 
 *     if driver == "SQLite":             # <<<<<<<<<<<<<<
 *         # TestCapability for OLCStringsAsUTF8 returns False for SQLite in GDAL 3.11.3.
 *         # Issue opened: https://github.com/OSGeo/gdal/issues/12962
*/
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_SQLite, Py_EQ)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 731, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "pyogrio/_io.pyx":734
 *         # TestCapability for OLCStringsAsUTF8 returns False for SQLite in GDAL 3.11.3.
 *         # Issue opened: https://github.com/OSGeo/gdal/issues/12962
 *         return "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *     return locale.getpreferredencoding()
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_UTF_8;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":731
 *         return "UTF-8"
 * 
 *     if driver == "SQLite":             # <<<<<<<<<<<<<<
 *         # TestCapability for OLCStringsAsUTF8 returns False for SQLite in GDAL 3.11.3.
 *         # Issue opened: https://github.com/OSGeo/gdal/issues/12962
*/
  }

  /* "pyogrio/_io.pyx":736
 *         return "UTF-8"
 * 
 *     return locale.getpreferredencoding()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_locale); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_getpreferredencoding); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_7 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_7 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (1-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":664
 * 
 * 
 * cdef detect_encoding(OGRDataSourceH ogr_dataset, OGRLayerH ogr_layer):             # <<<<<<<<<<<<<<
 *     """Attempt to detect the encoding to use to read/write string values.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyogrio._io.detect_encoding", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_driver);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":739
 * 
 * 
 * cdef get_fields(OGRLayerH ogr_layer, str encoding, use_arrow=False):             # <<<<<<<<<<<<<<
 *     """Get field names and types for layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_fields(OGRLayerH __pyx_v_ogr_layer, PyObject *__pyx_v_encoding, struct __pyx_opt_args_7pyogrio_3_io_get_fields *__pyx_optional_args) {
  PyObject *__pyx_v_use_arrow = ((PyObject *)Py_False);
  int __pyx_v_i;
  int __pyx_v_field_count;
  OGRFeatureDefnH __pyx_v_ogr_featuredef;
  OGRFieldDefnH __pyx_v_ogr_fielddef;
  int __pyx_v_field_subtype;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_fields = NULL;
  PyObject *__pyx_v_fields_view = NULL;
  int __pyx_v_skipped_fields;
  PyObject *__pyx_v_field_name = NULL;
  int __pyx_v_field_type;
  PyObject *__pyx_v_np_type = NULL;
  PyObject *__pyx_v_subtype = NULL;
  PyObject *__pyx_v_mask = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_idx = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  void *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_t_22;
  int __pyx_t_23;
  PyObject *__pyx_t_24 = NULL;
  int __pyx_t_25;
  char const *__pyx_t_26;
  struct __pyx_opt_args_7pyogrio_4_ogr_get_string __pyx_t_27;
  int __pyx_t_28;
  int __pyx_t_29;
  int __pyx_t_30;
  PyObject *__pyx_t_31[4];
  Py_ssize_t __pyx_t_32;
  PyObject *(*__pyx_t_33)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_fields", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_use_arrow = __pyx_optional_args->use_arrow;
    }
  }

  /* "pyogrio/_io.pyx":759
 *     cdef int i
 *     cdef int field_count
 *     cdef OGRFeatureDefnH ogr_featuredef = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRFieldDefnH ogr_fielddef = NULL
 *     cdef int field_subtype
*/
  __pyx_v_ogr_featuredef = NULL;

  /* "pyogrio/_io.pyx":760
 *     cdef int field_count
 *     cdef OGRFeatureDefnH ogr_featuredef = NULL
 *     cdef OGRFieldDefnH ogr_fielddef = NULL             # <<<<<<<<<<<<<<
 *     cdef int field_subtype
 * 
*/
  __pyx_v_ogr_fielddef = NULL;

  /* "pyogrio/_io.pyx":763
 *     cdef int field_subtype
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "pyogrio/_io.pyx":764
 * 
 *     try:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
 *         ogr_featuredef = check_pointer(ogr_featuredef)
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "pyogrio/_io.pyx":765
 *     try:
 *         with nogil:
 *             ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)             # <<<<<<<<<<<<<<
 *         ogr_featuredef = check_pointer(ogr_featuredef)
 * 
*/
            __pyx_v_ogr_featuredef = OGR_L_GetLayerDefn(__pyx_v_ogr_layer);
          }

          /* "pyogrio/_io.pyx":764
 * 
 *     try:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
 *         ogr_featuredef = check_pointer(ogr_featuredef)
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "pyogrio/_io.pyx":766
 *         with nogil:
 *             ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
 *         ogr_featuredef = check_pointer(ogr_featuredef)             # <<<<<<<<<<<<<<
 * 
 *     except NullPointerError:
*/
      __pyx_t_4 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_featuredef); if (unlikely(__pyx_t_4 == ((void *)NULL))) __PYX_ERR(0, 766, __pyx_L3_error)
      __pyx_v_ogr_featuredef = __pyx_t_4;

      /* "pyogrio/_io.pyx":763
 *     cdef int field_subtype
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "pyogrio/_io.pyx":768
 *         ogr_featuredef = check_pointer(ogr_featuredef)
 * 
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         raise DataLayerError("Could not get layer definition") from None
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 768, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.get_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 768, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);

      /* "pyogrio/_io.pyx":769
 * 
 *     except NullPointerError:
 *         raise DataLayerError("Could not get layer definition") from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
      __pyx_t_10 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 769, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
        assert(__pyx_t_10);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
        __pyx_t_12 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_mstate_global->__pyx_kp_u_Could_not_get_layer_definition};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 769, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 769, __pyx_L5_except_error)
    }

    /* "pyogrio/_io.pyx":771
 *         raise DataLayerError("Could not get layer definition") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 771, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.get_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 771, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_exc = __pyx_t_6;
      /*try:*/ {

        /* "pyogrio/_io.pyx":772
 * 
 *     except CPLE_BaseError as exc:
 *         raise DataLayerError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *     field_count = OGR_FD_GetFieldCount(ogr_featuredef)
*/
        __pyx_t_11 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 772, __pyx_L19_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_13 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 772, __pyx_L19_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_12 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
          assert(__pyx_t_11);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
          __pyx_t_12 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_13};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 772, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 772, __pyx_L19_error)
      }

      /* "pyogrio/_io.pyx":771
 *         raise DataLayerError("Could not get layer definition") from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
      /*finally:*/ {
        __pyx_L19_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __pyx_t_9 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "pyogrio/_io.pyx":763
 *     cdef int field_subtype
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "pyogrio/_io.pyx":774
 *         raise DataLayerError(str(exc))
 * 
 *     field_count = OGR_FD_GetFieldCount(ogr_featuredef)             # <<<<<<<<<<<<<<
 * 
 *     fields = np.empty(shape=(field_count, 5), dtype=object)
*/
  __pyx_v_field_count = OGR_FD_GetFieldCount(__pyx_v_ogr_featuredef);

  /* "pyogrio/_io.pyx":776
 *     field_count = OGR_FD_GetFieldCount(ogr_featuredef)
 * 
 *     fields = np.empty(shape=(field_count, 5), dtype=object)             # <<<<<<<<<<<<<<
 *     fields_view = fields[:, :]
 * 
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_field_count); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 776, __pyx_L1_error);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_5);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_5);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_mstate_global->__pyx_int_5) != (0)) __PYX_ERR(0, 776, __pyx_L1_error);
  __pyx_t_7 = 0;
  __pyx_t_12 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_6);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_12 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_6, NULL};
    __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_10, __pyx_t_7, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 776, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_builtin_object, __pyx_t_7, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 776, __pyx_L1_error)
    __pyx_t_5 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
  }
  __pyx_v_fields = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":777
 * 
 *     fields = np.empty(shape=(field_count, 5), dtype=object)
 *     fields_view = fields[:, :]             # <<<<<<<<<<<<<<
 * 
 *     skipped_fields = False
*/
  __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[2]); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v_fields_view = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":779
 *     fields_view = fields[:, :]
 * 
 *     skipped_fields = False             # <<<<<<<<<<<<<<
 * 
 *     for i in range(field_count):
*/
  __pyx_v_skipped_fields = 0;

  /* "pyogrio/_io.pyx":781
 *     skipped_fields = False
 * 
 *     for i in range(field_count):             # <<<<<<<<<<<<<<
 *         try:
 *             ogr_fielddef = check_pointer(OGR_FD_GetFieldDefn(ogr_featuredef, i))
*/
  __pyx_t_14 = __pyx_v_field_count;
  __pyx_t_9 = __pyx_t_14;
  for (__pyx_t_22 = 0; __pyx_t_22 < __pyx_t_9; __pyx_t_22+=1) {
    __pyx_v_i = __pyx_t_22;

    /* "pyogrio/_io.pyx":782
 * 
 *     for i in range(field_count):
 *         try:             # <<<<<<<<<<<<<<
 *             ogr_fielddef = check_pointer(OGR_FD_GetFieldDefn(ogr_featuredef, i))
 * 
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_1);
      /*try:*/ {

        /* "pyogrio/_io.pyx":783
 *     for i in range(field_count):
 *         try:
 *             ogr_fielddef = check_pointer(OGR_FD_GetFieldDefn(ogr_featuredef, i))             # <<<<<<<<<<<<<<
 * 
 *         except NullPointerError:
*/
        __pyx_t_4 = __pyx_f_7pyogrio_4_err_check_pointer(OGR_FD_GetFieldDefn(__pyx_v_ogr_featuredef, __pyx_v_i)); if (unlikely(__pyx_t_4 == ((void *)NULL))) __PYX_ERR(0, 783, __pyx_L27_error)
        __pyx_v_ogr_fielddef = __pyx_t_4;

        /* "pyogrio/_io.pyx":782
 * 
 *     for i in range(field_count):
 *         try:             # <<<<<<<<<<<<<<
 *             ogr_fielddef = check_pointer(OGR_FD_GetFieldDefn(ogr_featuredef, i))
 * 
*/
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L34_try_end;
      __pyx_L27_error:;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":785
 *             ogr_fielddef = check_pointer(OGR_FD_GetFieldDefn(ogr_featuredef, i))
 * 
 *         except NullPointerError:             # <<<<<<<<<<<<<<
 *             raise FieldError(
 *                 f"Could not get field definition for field at index {i}"
*/
      __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_8, &__pyx_t_7);
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 785, __pyx_L29_except_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_23 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_10);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_ErrRestore(__pyx_t_5, __pyx_t_8, __pyx_t_7);
      __pyx_t_5 = 0; __pyx_t_8 = 0; __pyx_t_7 = 0;
      if (__pyx_t_23) {
        __Pyx_AddTraceback("pyogrio._io.get_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_5) < 0) __PYX_ERR(0, 785, __pyx_L29_except_error)
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_5);

        /* "pyogrio/_io.pyx":786
 * 
 *         except NullPointerError:
 *             raise FieldError(             # <<<<<<<<<<<<<<
 *                 f"Could not get field definition for field at index {i}"
 *             ) from None
*/
        __pyx_t_6 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_FieldError); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 786, __pyx_L29_except_error)
        __Pyx_GOTREF(__pyx_t_13);

        /* "pyogrio/_io.pyx":787
 *         except NullPointerError:
 *             raise FieldError(
 *                 f"Could not get field definition for field at index {i}"             # <<<<<<<<<<<<<<
 *             ) from None
 * 
*/
        __pyx_t_11 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 787, __pyx_L29_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_24 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Could_not_get_field_definition_f, __pyx_t_11); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 787, __pyx_L29_except_error)
        __Pyx_GOTREF(__pyx_t_24);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_12 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_13))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
          assert(__pyx_t_6);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
          __pyx_t_12 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_24};
          __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 786, __pyx_L29_except_error)
          __Pyx_GOTREF(__pyx_t_10);
        }

        /* "pyogrio/_io.pyx":788
 *             raise FieldError(
 *                 f"Could not get field definition for field at index {i}"
 *             ) from None             # <<<<<<<<<<<<<<
 * 
 *         except CPLE_BaseError as exc:
*/
        __Pyx_Raise(__pyx_t_10, 0, 0, Py_None);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __PYX_ERR(0, 786, __pyx_L29_except_error)
      }

      /* "pyogrio/_io.pyx":790
 *             ) from None
 * 
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             raise FieldError(str(exc))
 * 
*/
      __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_8, &__pyx_t_7);
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 790, __pyx_L29_except_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_23 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_10);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_ErrRestore(__pyx_t_5, __pyx_t_8, __pyx_t_7);
      __pyx_t_5 = 0; __pyx_t_8 = 0; __pyx_t_7 = 0;
      if (__pyx_t_23) {
        __Pyx_AddTraceback("pyogrio._io.get_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_5) < 0) __PYX_ERR(0, 790, __pyx_L29_except_error)
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __pyx_v_exc = __pyx_t_8;
        /*try:*/ {

          /* "pyogrio/_io.pyx":791
 * 
 *         except CPLE_BaseError as exc:
 *             raise FieldError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *         field_name = get_string(OGR_Fld_GetNameRef(ogr_fielddef), encoding=encoding)
*/
          __pyx_t_13 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_24, __pyx_mstate_global->__pyx_n_u_FieldError); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 791, __pyx_L42_error)
          __Pyx_GOTREF(__pyx_t_24);
          __pyx_t_6 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 791, __pyx_L42_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_12 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_24))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_24);
            assert(__pyx_t_13);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_24);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_24, __pyx__function);
            __pyx_t_12 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_6};
            __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_24, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 791, __pyx_L42_error)
            __Pyx_GOTREF(__pyx_t_10);
          }
          __Pyx_Raise(__pyx_t_10, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __PYX_ERR(0, 791, __pyx_L42_error)
        }

        /* "pyogrio/_io.pyx":790
 *             ) from None
 * 
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             raise FieldError(str(exc))
 * 
*/
        /*finally:*/ {
          __pyx_L42_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_16 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_17, &__pyx_t_16);
            if ( unlikely(__Pyx_GetException(&__pyx_t_21, &__pyx_t_20, &__pyx_t_19) < 0)) __Pyx_ErrFetch(&__pyx_t_21, &__pyx_t_20, &__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_21);
            __Pyx_XGOTREF(__pyx_t_20);
            __Pyx_XGOTREF(__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_16);
            __pyx_t_23 = __pyx_lineno; __pyx_t_25 = __pyx_clineno; __pyx_t_26 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_17, __pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_ErrRestore(__pyx_t_21, __pyx_t_20, __pyx_t_19);
            __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_16 = 0;
            __pyx_lineno = __pyx_t_23; __pyx_clineno = __pyx_t_25; __pyx_filename = __pyx_t_26;
            goto __pyx_L29_except_error;
          }
        }
      }
      goto __pyx_L29_except_error;

      /* "pyogrio/_io.pyx":782
 * 
 *     for i in range(field_count):
 *         try:             # <<<<<<<<<<<<<<
 *             ogr_fielddef = check_pointer(OGR_FD_GetFieldDefn(ogr_featuredef, i))
 * 
*/
      __pyx_L29_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_2);
      __Pyx_XGIVEREF(__pyx_t_1);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_2, __pyx_t_1);
      goto __pyx_L1_error;
      __pyx_L34_try_end:;
    }

    /* "pyogrio/_io.pyx":793
 *             raise FieldError(str(exc))
 * 
 *         field_name = get_string(OGR_Fld_GetNameRef(ogr_fielddef), encoding=encoding)             # <<<<<<<<<<<<<<
 * 
 *         field_type = OGR_Fld_GetType(ogr_fielddef)
*/
    __pyx_t_27.__pyx_n = 1;
    __pyx_t_27.encoding = __pyx_v_encoding;
    __pyx_t_5 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_Fld_GetNameRef(__pyx_v_ogr_fielddef), &__pyx_t_27); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 793, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_field_name, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":795
 *         field_name = get_string(OGR_Fld_GetNameRef(ogr_fielddef), encoding=encoding)
 * 
 *         field_type = OGR_Fld_GetType(ogr_fielddef)             # <<<<<<<<<<<<<<
 *         np_type = FIELD_TYPES[field_type]
 *         if not np_type and not use_arrow:
*/
    __pyx_v_field_type = OGR_Fld_GetType(__pyx_v_ogr_fielddef);

    /* "pyogrio/_io.pyx":796
 * 
 *         field_type = OGR_Fld_GetType(ogr_fielddef)
 *         np_type = FIELD_TYPES[field_type]             # <<<<<<<<<<<<<<
 *         if not np_type and not use_arrow:
 *             skipped_fields = True
*/
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_FIELD_TYPES); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_GetItemInt(__pyx_t_5, __pyx_v_field_type, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_np_type, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":797
 *         field_type = OGR_Fld_GetType(ogr_fielddef)
 *         np_type = FIELD_TYPES[field_type]
 *         if not np_type and not use_arrow:             # <<<<<<<<<<<<<<
 *             skipped_fields = True
 *             log.warning(
*/
    __pyx_t_29 = __Pyx_PyObject_IsTrue(__pyx_v_np_type); if (unlikely((__pyx_t_29 < 0))) __PYX_ERR(0, 797, __pyx_L1_error)
    __pyx_t_30 = (!__pyx_t_29);
    if (__pyx_t_30) {
    } else {
      __pyx_t_28 = __pyx_t_30;
      goto __pyx_L49_bool_binop_done;
    }
    __pyx_t_30 = __Pyx_PyObject_IsTrue(__pyx_v_use_arrow); if (unlikely((__pyx_t_30 < 0))) __PYX_ERR(0, 797, __pyx_L1_error)
    __pyx_t_29 = (!__pyx_t_30);
    __pyx_t_28 = __pyx_t_29;
    __pyx_L49_bool_binop_done:;
    if (__pyx_t_28) {

      /* "pyogrio/_io.pyx":798
 *         np_type = FIELD_TYPES[field_type]
 *         if not np_type and not use_arrow:
 *             skipped_fields = True             # <<<<<<<<<<<<<<
 *             log.warning(
 *                 f"Skipping field {field_name}: unsupported OGR type: {field_type}")
*/
      __pyx_v_skipped_fields = 1;

      /* "pyogrio/_io.pyx":799
 *         if not np_type and not use_arrow:
 *             skipped_fields = True
 *             log.warning(             # <<<<<<<<<<<<<<
 *                 f"Skipping field {field_name}: unsupported OGR type: {field_type}")
 *             continue
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_log); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 799, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_warning); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 799, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyogrio/_io.pyx":800
 *             skipped_fields = True
 *             log.warning(
 *                 f"Skipping field {field_name}: unsupported OGR type: {field_type}")             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
      __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_field_name, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 800, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_24 = __Pyx_PyUnicode_From_int(__pyx_v_field_type, 0, ' ', 'd'); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 800, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_24);
      __pyx_t_31[0] = __pyx_mstate_global->__pyx_kp_u_Skipping_field;
      __pyx_t_31[1] = __pyx_t_7;
      __pyx_t_31[2] = __pyx_mstate_global->__pyx_kp_u_unsupported_OGR_type;
      __pyx_t_31[3] = __pyx_t_24;
      __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_31, 4, 15 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 24 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_24), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7));
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 800, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
      __pyx_t_12 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_10);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
        __pyx_t_12 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_6};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 799, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":801
 *             log.warning(
 *                 f"Skipping field {field_name}: unsupported OGR type: {field_type}")
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         field_subtype = OGR_Fld_GetSubType(ogr_fielddef)
*/
      goto __pyx_L25_continue;

      /* "pyogrio/_io.pyx":797
 *         field_type = OGR_Fld_GetType(ogr_fielddef)
 *         np_type = FIELD_TYPES[field_type]
 *         if not np_type and not use_arrow:             # <<<<<<<<<<<<<<
 *             skipped_fields = True
 *             log.warning(
*/
    }

    /* "pyogrio/_io.pyx":803
 *             continue
 * 
 *         field_subtype = OGR_Fld_GetSubType(ogr_fielddef)             # <<<<<<<<<<<<<<
 *         subtype = FIELD_SUBTYPES.get(field_subtype)
 *         if subtype is not None:
*/
    __pyx_v_field_subtype = OGR_Fld_GetSubType(__pyx_v_ogr_fielddef);

    /* "pyogrio/_io.pyx":804
 * 
 *         field_subtype = OGR_Fld_GetSubType(ogr_fielddef)
 *         subtype = FIELD_SUBTYPES.get(field_subtype)             # <<<<<<<<<<<<<<
 *         if subtype is not None:
 *             # bool, int16, float32 dtypes
*/
    __pyx_t_10 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_FIELD_SUBTYPES); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_field_subtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_12 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_10);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_12 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_6};
      __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 804, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }
    __Pyx_XDECREF_SET(__pyx_v_subtype, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":805
 *         field_subtype = OGR_Fld_GetSubType(ogr_fielddef)
 *         subtype = FIELD_SUBTYPES.get(field_subtype)
 *         if subtype is not None:             # <<<<<<<<<<<<<<
 *             # bool, int16, float32 dtypes
 *             np_type = subtype
*/
    __pyx_t_28 = (__pyx_v_subtype != Py_None);
    if (__pyx_t_28) {

      /* "pyogrio/_io.pyx":807
 *         if subtype is not None:
 *             # bool, int16, float32 dtypes
 *             np_type = subtype             # <<<<<<<<<<<<<<
 * 
 *         fields_view[i, 0] = i
*/
      __Pyx_INCREF(__pyx_v_subtype);
      __Pyx_DECREF_SET(__pyx_v_np_type, __pyx_v_subtype);

      /* "pyogrio/_io.pyx":805
 *         field_subtype = OGR_Fld_GetSubType(ogr_fielddef)
 *         subtype = FIELD_SUBTYPES.get(field_subtype)
 *         if subtype is not None:             # <<<<<<<<<<<<<<
 *             # bool, int16, float32 dtypes
 *             np_type = subtype
*/
    }

    /* "pyogrio/_io.pyx":809
 *             np_type = subtype
 * 
 *         fields_view[i, 0] = i             # <<<<<<<<<<<<<<
 *         fields_view[i, 1] = field_type
 *         fields_view[i, 2] = field_name
*/
    __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5) != (0)) __PYX_ERR(0, 809, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(0, 809, __pyx_L1_error);
    __pyx_t_5 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_fields_view, __pyx_t_6, __pyx_t_8) < 0))) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":810
 * 
 *         fields_view[i, 0] = i
 *         fields_view[i, 1] = field_type             # <<<<<<<<<<<<<<
 *         fields_view[i, 2] = field_name
 *         fields_view[i, 3] = np_type
*/
    __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_field_type); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 810, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 810, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 810, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6) != (0)) __PYX_ERR(0, 810, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_mstate_global->__pyx_int_1) != (0)) __PYX_ERR(0, 810, __pyx_L1_error);
    __pyx_t_6 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_fields_view, __pyx_t_5, __pyx_t_8) < 0))) __PYX_ERR(0, 810, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":811
 *         fields_view[i, 0] = i
 *         fields_view[i, 1] = field_type
 *         fields_view[i, 2] = field_name             # <<<<<<<<<<<<<<
 *         fields_view[i, 3] = np_type
 *         fields_view[i, 4] = field_subtype
*/
    __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 811, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 811, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8) != (0)) __PYX_ERR(0, 811, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_2);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_2);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_mstate_global->__pyx_int_2) != (0)) __PYX_ERR(0, 811, __pyx_L1_error);
    __pyx_t_8 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_fields_view, __pyx_t_5, __pyx_v_field_name) < 0))) __PYX_ERR(0, 811, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":812
 *         fields_view[i, 1] = field_type
 *         fields_view[i, 2] = field_name
 *         fields_view[i, 3] = np_type             # <<<<<<<<<<<<<<
 *         fields_view[i, 4] = field_subtype
 * 
*/
    __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 812, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 812, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5) != (0)) __PYX_ERR(0, 812, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_3);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_mstate_global->__pyx_int_3) != (0)) __PYX_ERR(0, 812, __pyx_L1_error);
    __pyx_t_5 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_fields_view, __pyx_t_8, __pyx_v_np_type) < 0))) __PYX_ERR(0, 812, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":813
 *         fields_view[i, 2] = field_name
 *         fields_view[i, 3] = np_type
 *         fields_view[i, 4] = field_subtype             # <<<<<<<<<<<<<<
 * 
 *     if skipped_fields:
*/
    __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_field_subtype); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 813, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 813, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 813, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5) != (0)) __PYX_ERR(0, 813, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_4);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_mstate_global->__pyx_int_4) != (0)) __PYX_ERR(0, 813, __pyx_L1_error);
    __pyx_t_5 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_fields_view, __pyx_t_6, __pyx_t_8) < 0))) __PYX_ERR(0, 813, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_L25_continue:;
  }

  /* "pyogrio/_io.pyx":815
 *         fields_view[i, 4] = field_subtype
 * 
 *     if skipped_fields:             # <<<<<<<<<<<<<<
 *         # filter out skipped fields
 *         mask = np.array([idx is not None for idx in fields[:, 0]])
*/
  if (__pyx_v_skipped_fields) {

    /* "pyogrio/_io.pyx":817
 *     if skipped_fields:
 *         # filter out skipped fields
 *         mask = np.array([idx is not None for idx in fields[:, 0]])             # <<<<<<<<<<<<<<
 *         fields = fields[mask]
 * 
*/
    __pyx_t_6 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 817, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 817, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    { /* enter inner scope */
      __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 817, __pyx_L55_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_24 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[3]); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 817, __pyx_L55_error)
      __Pyx_GOTREF(__pyx_t_24);
      if (likely(PyList_CheckExact(__pyx_t_24)) || PyTuple_CheckExact(__pyx_t_24)) {
        __pyx_t_7 = __pyx_t_24; __Pyx_INCREF(__pyx_t_7);
        __pyx_t_32 = 0;
        __pyx_t_33 = NULL;
      } else {
        __pyx_t_32 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_t_24); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 817, __pyx_L55_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_33 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7); if (unlikely(!__pyx_t_33)) __PYX_ERR(0, 817, __pyx_L55_error)
      }
      __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
      for (;;) {
        if (likely(!__pyx_t_33)) {
          if (likely(PyList_CheckExact(__pyx_t_7))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_7);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 817, __pyx_L55_error)
              #endif
              if (__pyx_t_32 >= __pyx_temp) break;
            }
            __pyx_t_24 = __Pyx_PyList_GetItemRefFast(__pyx_t_7, __pyx_t_32, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_32;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_7);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 817, __pyx_L55_error)
              #endif
              if (__pyx_t_32 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_24 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_32));
            #else
            __pyx_t_24 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_32);
            #endif
            ++__pyx_t_32;
          }
          if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 817, __pyx_L55_error)
        } else {
          __pyx_t_24 = __pyx_t_33(__pyx_t_7);
          if (unlikely(!__pyx_t_24)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 817, __pyx_L55_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_24);
        __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_idx, __pyx_t_24);
        __pyx_t_24 = 0;
        __pyx_t_28 = (__pyx_8genexpr1__pyx_v_idx != Py_None);
        __pyx_t_24 = __Pyx_PyBool_FromLong(__pyx_t_28); if (unlikely(!__pyx_t_24)) __PYX_ERR(0, 817, __pyx_L55_error)
        __Pyx_GOTREF(__pyx_t_24);
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_5, (PyObject*)__pyx_t_24))) __PYX_ERR(0, 817, __pyx_L55_error)
        __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_idx); __pyx_8genexpr1__pyx_v_idx = 0;
      goto __pyx_L59_exit_scope;
      __pyx_L55_error:;
      __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_idx); __pyx_8genexpr1__pyx_v_idx = 0;
      goto __pyx_L1_error;
      __pyx_L59_exit_scope:;
    } /* exit inner scope */
    __pyx_t_12 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_10);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
      __pyx_t_12 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_5};
      __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 817, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }
    __pyx_v_mask = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":818
 *         # filter out skipped fields
 *         mask = np.array([idx is not None for idx in fields[:, 0]])
 *         fields = fields[mask]             # <<<<<<<<<<<<<<
 * 
 *     return fields
*/
    __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_v_mask); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF_SET(__pyx_v_fields, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":815
 *         fields_view[i, 4] = field_subtype
 * 
 *     if skipped_fields:             # <<<<<<<<<<<<<<
 *         # filter out skipped fields
 *         mask = np.array([idx is not None for idx in fields[:, 0]])
*/
  }

  /* "pyogrio/_io.pyx":820
 *         fields = fields[mask]
 * 
 *     return fields             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_fields);
  __pyx_r = __pyx_v_fields;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":739
 * 
 * 
 * cdef get_fields(OGRLayerH ogr_layer, str encoding, use_arrow=False):             # <<<<<<<<<<<<<<
 *     """Get field names and types for layer.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_24);
  __Pyx_AddTraceback("pyogrio._io.get_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_fields_view);
  __Pyx_XDECREF(__pyx_v_field_name);
  __Pyx_XDECREF(__pyx_v_np_type);
  __Pyx_XDECREF(__pyx_v_subtype);
  __Pyx_XDECREF(__pyx_v_mask);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_idx);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":823
 * 
 * 
 * cdef apply_where_filter(OGRLayerH ogr_layer, str where):             # <<<<<<<<<<<<<<
 *     """Applies where filter to layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_apply_where_filter(OGRLayerH __pyx_v_ogr_layer, PyObject *__pyx_v_where) {
  char const *__pyx_v_where_c;
  int __pyx_v_err;
  PyObject *__pyx_v_where_b = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char const *__pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  size_t __pyx_t_13;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_where_filter", 0);

  /* "pyogrio/_io.pyx":844
 *     cdef int err
 * 
 *     where_b = where.encode("utf-8")             # <<<<<<<<<<<<<<
 *     where_c = where_b
 *     with nogil:
*/
  if (unlikely(__pyx_v_where == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 844, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_where); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_where_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":845
 * 
 *     where_b = where.encode("utf-8")
 *     where_c = where_b             # <<<<<<<<<<<<<<
 *     with nogil:
 *         err = OGR_L_SetAttributeFilter(ogr_layer, where_c)
*/
  __pyx_t_2 = __Pyx_PyBytes_AsString(__pyx_v_where_b); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 845, __pyx_L1_error)
  __pyx_v_where_c = __pyx_t_2;

  /* "pyogrio/_io.pyx":846
 *     where_b = where.encode("utf-8")
 *     where_c = where_b
 *     with nogil:             # <<<<<<<<<<<<<<
 *         err = OGR_L_SetAttributeFilter(ogr_layer, where_c)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":847
 *     where_c = where_b
 *     with nogil:
 *         err = OGR_L_SetAttributeFilter(ogr_layer, where_c)             # <<<<<<<<<<<<<<
 * 
 *     # WARNING: GDAL does not raise this error for GPKG but instead only
*/
        __pyx_v_err = OGR_L_SetAttributeFilter(__pyx_v_ogr_layer, __pyx_v_where_c);
      }

      /* "pyogrio/_io.pyx":846
 *     where_b = where.encode("utf-8")
 *     where_c = where_b
 *     with nogil:             # <<<<<<<<<<<<<<
 *         err = OGR_L_SetAttributeFilter(ogr_layer, where_c)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":851
 *     # WARNING: GDAL does not raise this error for GPKG but instead only
 *     # logs to stderr
 *     if err != OGRERR_NONE:             # <<<<<<<<<<<<<<
 *         try:
 *             check_last_error()
*/
  __pyx_t_3 = (__pyx_v_err != OGRERR_NONE);
  if (__pyx_t_3) {

    /* "pyogrio/_io.pyx":852
 *     # logs to stderr
 *     if err != OGRERR_NONE:
 *         try:             # <<<<<<<<<<<<<<
 *             check_last_error()
 *         except CPLE_BaseError as exc:
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      /*try:*/ {

        /* "pyogrio/_io.pyx":853
 *     if err != OGRERR_NONE:
 *         try:
 *             check_last_error()             # <<<<<<<<<<<<<<
 *         except CPLE_BaseError as exc:
 *             raise ValueError(str(exc))
*/
        __pyx_t_1 = __pyx_f_7pyogrio_4_err_check_last_error(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pyogrio/_io.pyx":852
 *     # logs to stderr
 *     if err != OGRERR_NONE:
 *         try:             # <<<<<<<<<<<<<<
 *             check_last_error()
 *         except CPLE_BaseError as exc:
*/
      }
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L12_try_end;
      __pyx_L7_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pyogrio/_io.pyx":854
 *         try:
 *             check_last_error()
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             raise ValueError(str(exc))
 * 
*/
      __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_7, &__pyx_t_8);
      __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 854, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_9);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_ErrRestore(__pyx_t_1, __pyx_t_7, __pyx_t_8);
      __pyx_t_1 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0;
      if (__pyx_t_10) {
        __Pyx_AddTraceback("pyogrio._io.apply_where_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_7, &__pyx_t_1) < 0) __PYX_ERR(0, 854, __pyx_L9_except_error)
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_7);
        __pyx_v_exc = __pyx_t_7;
        /*try:*/ {

          /* "pyogrio/_io.pyx":855
 *             check_last_error()
 *         except CPLE_BaseError as exc:
 *             raise ValueError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *         raise ValueError(
*/
          __pyx_t_11 = NULL;
          __pyx_t_12 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 855, __pyx_L18_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_13 = 1;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_12};
            __pyx_t_9 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_13, (2-__pyx_t_13) | (__pyx_t_13*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 855, __pyx_L18_error)
            __Pyx_GOTREF(__pyx_t_9);
          }
          __Pyx_Raise(__pyx_t_9, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __PYX_ERR(0, 855, __pyx_L18_error)
        }

        /* "pyogrio/_io.pyx":854
 *         try:
 *             check_last_error()
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             raise ValueError(str(exc))
 * 
*/
        /*finally:*/ {
          __pyx_L18_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
            if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_20);
            __Pyx_XGOTREF(__pyx_t_21);
            __pyx_t_10 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
            __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
            __pyx_lineno = __pyx_t_10; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
            goto __pyx_L9_except_error;
          }
        }
      }
      goto __pyx_L9_except_error;

      /* "pyogrio/_io.pyx":852
 *     # logs to stderr
 *     if err != OGRERR_NONE:
 *         try:             # <<<<<<<<<<<<<<
 *             check_last_error()
 *         except CPLE_BaseError as exc:
*/
      __pyx_L9_except_error:;
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
      goto __pyx_L1_error;
      __pyx_L12_try_end:;
    }

    /* "pyogrio/_io.pyx":857
 *             raise ValueError(str(exc))
 * 
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             f"Invalid SQL query for layer '{OGR_L_GetName(ogr_layer)}': '{where}'"
 *         )
*/
    __pyx_t_7 = NULL;

    /* "pyogrio/_io.pyx":858
 * 
 *         raise ValueError(
 *             f"Invalid SQL query for layer '{OGR_L_GetName(ogr_layer)}': '{where}'"             # <<<<<<<<<<<<<<
 *         )
 * 
*/
    __pyx_t_8 = __Pyx_PyBytes_FromString(OGR_L_GetName(__pyx_v_ogr_layer)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_FormatSimple(__pyx_t_8, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyUnicode_Unicode(__pyx_v_where); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_22[0] = __pyx_mstate_global->__pyx_kp_u_Invalid_SQL_query_for_layer;
    __pyx_t_22[1] = __pyx_t_9;
    __pyx_t_22[2] = __pyx_mstate_global->__pyx_kp_u__8;
    __pyx_t_22[3] = __pyx_t_8;
    __pyx_t_22[4] = __pyx_mstate_global->__pyx_kp_u__4;
    __pyx_t_12 = __Pyx_PyUnicode_Join(__pyx_t_22, 5, 29 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 4 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 858, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_13 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_12};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_13, (2-__pyx_t_13) | (__pyx_t_13*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 857, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 857, __pyx_L1_error)

    /* "pyogrio/_io.pyx":851
 *     # WARNING: GDAL does not raise this error for GPKG but instead only
 *     # logs to stderr
 *     if err != OGRERR_NONE:             # <<<<<<<<<<<<<<
 *         try:
 *             check_last_error()
*/
  }

  /* "pyogrio/_io.pyx":823
 * 
 * 
 * cdef apply_where_filter(OGRLayerH ogr_layer, str where):             # <<<<<<<<<<<<<<
 *     """Applies where filter to layer.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("pyogrio._io.apply_where_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_where_b);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":862
 * 
 * 
 * cdef apply_bbox_filter(OGRLayerH ogr_layer, bbox):             # <<<<<<<<<<<<<<
 *     """Applies bounding box spatial filter to layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_apply_bbox_filter(OGRLayerH __pyx_v_ogr_layer, PyObject *__pyx_v_bbox) {
  double __pyx_v_xmin;
  double __pyx_v_ymin;
  double __pyx_v_xmax;
  double __pyx_v_ymax;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *(*__pyx_t_11)(PyObject *);
  double __pyx_t_12;
  double __pyx_t_13;
  double __pyx_t_14;
  double __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_bbox_filter", 0);

  /* "pyogrio/_io.pyx":879
 *     cdef double xmin, ymin, xmax, ymax
 * 
 *     if not (isinstance(bbox, (tuple, list)) and len(bbox) == 4):             # <<<<<<<<<<<<<<
 *         raise ValueError(f"Invalid bbox: {bbox}")
 * 
*/
  __pyx_t_3 = PyTuple_Check(__pyx_v_bbox); 
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_3 = PyList_Check(__pyx_v_bbox); 
  __pyx_t_2 = __pyx_t_3;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = PyObject_Length(__pyx_v_bbox); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 879, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_4 == 4);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":880
 * 
 *     if not (isinstance(bbox, (tuple, list)) and len(bbox) == 4):
 *         raise ValueError(f"Invalid bbox: {bbox}")             # <<<<<<<<<<<<<<
 * 
 *     xmin, ymin, xmax, ymax = bbox
*/
    __pyx_t_6 = NULL;
    __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_bbox, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Invalid_bbox, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 880, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_9 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_8};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 880, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 880, __pyx_L1_error)

    /* "pyogrio/_io.pyx":879
 *     cdef double xmin, ymin, xmax, ymax
 * 
 *     if not (isinstance(bbox, (tuple, list)) and len(bbox) == 4):             # <<<<<<<<<<<<<<
 *         raise ValueError(f"Invalid bbox: {bbox}")
 * 
*/
  }

  /* "pyogrio/_io.pyx":882
 *         raise ValueError(f"Invalid bbox: {bbox}")
 * 
 *     xmin, ymin, xmax, ymax = bbox             # <<<<<<<<<<<<<<
 *     with nogil:
 *         OGR_L_SetSpatialFilterRect(ogr_layer, xmin, ymin, xmax, ymax)
*/
  if ((likely(PyTuple_CheckExact(__pyx_v_bbox))) || (PyList_CheckExact(__pyx_v_bbox))) {
    PyObject* sequence = __pyx_v_bbox;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 882, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_8);
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 3);
      __Pyx_INCREF(__pyx_t_7);
    } else {
      __pyx_t_5 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 882, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 882, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_8);
      __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 2, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 882, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyList_GetItemRefFast(sequence, 3, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 882, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_7);
    }
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_5,&__pyx_t_8,&__pyx_t_6,&__pyx_t_7};
      for (i=0; i < 4; i++) {
        PyObject* item = __Pyx_PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[4] = {&__pyx_t_5,&__pyx_t_8,&__pyx_t_6,&__pyx_t_7};
    __pyx_t_10 = PyObject_GetIter(__pyx_v_bbox); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 882, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_10);
    for (index=0; index < 4; index++) {
      PyObject* item = __pyx_t_11(__pyx_t_10); if (unlikely(!item)) goto __pyx_L8_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_10), 4) < (0)) __PYX_ERR(0, 882, __pyx_L1_error)
    __pyx_t_11 = NULL;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    goto __pyx_L9_unpacking_done;
    __pyx_L8_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_11 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 882, __pyx_L1_error)
    __pyx_L9_unpacking_done:;
  }
  __pyx_t_12 = __Pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_12 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 882, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_13 = __Pyx_PyFloat_AsDouble(__pyx_t_8); if (unlikely((__pyx_t_13 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 882, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyFloat_AsDouble(__pyx_t_6); if (unlikely((__pyx_t_14 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 882, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_15 = __Pyx_PyFloat_AsDouble(__pyx_t_7); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 882, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_xmin = __pyx_t_12;
  __pyx_v_ymin = __pyx_t_13;
  __pyx_v_xmax = __pyx_t_14;
  __pyx_v_ymax = __pyx_t_15;

  /* "pyogrio/_io.pyx":883
 * 
 *     xmin, ymin, xmax, ymax = bbox
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_SetSpatialFilterRect(ogr_layer, xmin, ymin, xmax, ymax)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":884
 *     xmin, ymin, xmax, ymax = bbox
 *     with nogil:
 *         OGR_L_SetSpatialFilterRect(ogr_layer, xmin, ymin, xmax, ymax)             # <<<<<<<<<<<<<<
 * 
 * 
*/
        OGR_L_SetSpatialFilterRect(__pyx_v_ogr_layer, __pyx_v_xmin, __pyx_v_ymin, __pyx_v_xmax, __pyx_v_ymax);
      }

      /* "pyogrio/_io.pyx":883
 * 
 *     xmin, ymin, xmax, ymax = bbox
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_SetSpatialFilterRect(ogr_layer, xmin, ymin, xmax, ymax)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L12;
        }
        __pyx_L12:;
      }
  }

  /* "pyogrio/_io.pyx":862
 * 
 * 
 * cdef apply_bbox_filter(OGRLayerH ogr_layer, bbox):             # <<<<<<<<<<<<<<
 *     """Applies bounding box spatial filter to layer.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pyogrio._io.apply_bbox_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":887
 * 
 * 
 * cdef apply_geometry_filter(OGRLayerH ogr_layer, wkb):             # <<<<<<<<<<<<<<
 *     """Applies geometry spatial filter to the layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_apply_geometry_filter(OGRLayerH __pyx_v_ogr_layer, PyObject *__pyx_v_wkb) {
  OGRGeometryH __pyx_v_ogr_geometry;
  unsigned char *__pyx_v_wkb_buffer;
  OGRErr __pyx_v_err;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  unsigned char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_geometry_filter", 0);

  /* "pyogrio/_io.pyx":897
 *         Geometry encoded as WKB
 *     """
 *     cdef OGRGeometryH ogr_geometry = NULL             # <<<<<<<<<<<<<<
 *     cdef unsigned char *wkb_buffer = wkb
 * 
*/
  __pyx_v_ogr_geometry = NULL;

  /* "pyogrio/_io.pyx":898
 *     """
 *     cdef OGRGeometryH ogr_geometry = NULL
 *     cdef unsigned char *wkb_buffer = wkb             # <<<<<<<<<<<<<<
 * 
 *     err = OGR_G_CreateFromWkb(wkb_buffer, NULL, &ogr_geometry, len(wkb))
*/
  __pyx_t_1 = __Pyx_PyObject_AsWritableUString(__pyx_v_wkb); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 898, __pyx_L1_error)
  __pyx_v_wkb_buffer = __pyx_t_1;

  /* "pyogrio/_io.pyx":900
 *     cdef unsigned char *wkb_buffer = wkb
 * 
 *     err = OGR_G_CreateFromWkb(wkb_buffer, NULL, &ogr_geometry, len(wkb))             # <<<<<<<<<<<<<<
 *     if err:
 *         if ogr_geometry != NULL:
*/
  __pyx_t_2 = PyObject_Length(__pyx_v_wkb); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 900, __pyx_L1_error)
  __pyx_v_err = OGR_G_CreateFromWkb(__pyx_v_wkb_buffer, NULL, (&__pyx_v_ogr_geometry), __pyx_t_2);

  /* "pyogrio/_io.pyx":901
 * 
 *     err = OGR_G_CreateFromWkb(wkb_buffer, NULL, &ogr_geometry, len(wkb))
 *     if err:             # <<<<<<<<<<<<<<
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)
*/
  if (__pyx_v_err) {

    /* "pyogrio/_io.pyx":902
 *     err = OGR_G_CreateFromWkb(wkb_buffer, NULL, &ogr_geometry, len(wkb))
 *     if err:
 *         if ogr_geometry != NULL:             # <<<<<<<<<<<<<<
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *         raise GeometryError("Could not create mask geometry") from None
*/
    __pyx_t_3 = (__pyx_v_ogr_geometry != NULL);
    if (__pyx_t_3) {

      /* "pyogrio/_io.pyx":903
 *     if err:
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)             # <<<<<<<<<<<<<<
 *         raise GeometryError("Could not create mask geometry") from None
 * 
*/
      OGR_G_DestroyGeometry(__pyx_v_ogr_geometry);

      /* "pyogrio/_io.pyx":902
 *     err = OGR_G_CreateFromWkb(wkb_buffer, NULL, &ogr_geometry, len(wkb))
 *     if err:
 *         if ogr_geometry != NULL:             # <<<<<<<<<<<<<<
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *         raise GeometryError("Could not create mask geometry") from None
*/
    }

    /* "pyogrio/_io.pyx":904
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *         raise GeometryError("Could not create mask geometry") from None             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_GeometryError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_Could_not_create_mask_geometry};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 904, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_Raise(__pyx_t_4, 0, 0, Py_None);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 904, __pyx_L1_error)

    /* "pyogrio/_io.pyx":901
 * 
 *     err = OGR_G_CreateFromWkb(wkb_buffer, NULL, &ogr_geometry, len(wkb))
 *     if err:             # <<<<<<<<<<<<<<
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)
*/
  }

  /* "pyogrio/_io.pyx":906
 *         raise GeometryError("Could not create mask geometry") from None
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_SetSpatialFilter(ogr_layer, ogr_geometry)
 *     OGR_G_DestroyGeometry(ogr_geometry)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":907
 * 
 *     with nogil:
 *         OGR_L_SetSpatialFilter(ogr_layer, ogr_geometry)             # <<<<<<<<<<<<<<
 *     OGR_G_DestroyGeometry(ogr_geometry)
 * 
*/
        OGR_L_SetSpatialFilter(__pyx_v_ogr_layer, __pyx_v_ogr_geometry);
      }

      /* "pyogrio/_io.pyx":906
 *         raise GeometryError("Could not create mask geometry") from None
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_SetSpatialFilter(ogr_layer, ogr_geometry)
 *     OGR_G_DestroyGeometry(ogr_geometry)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "pyogrio/_io.pyx":908
 *     with nogil:
 *         OGR_L_SetSpatialFilter(ogr_layer, ogr_geometry)
 *     OGR_G_DestroyGeometry(ogr_geometry)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  OGR_G_DestroyGeometry(__pyx_v_ogr_geometry);

  /* "pyogrio/_io.pyx":887
 * 
 * 
 * cdef apply_geometry_filter(OGRLayerH ogr_layer, wkb):             # <<<<<<<<<<<<<<
 *     """Applies geometry spatial filter to the layer.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyogrio._io.apply_geometry_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":911
 * 
 * 
 * cdef apply_skip_features(OGRLayerH ogr_layer, int skip_features):             # <<<<<<<<<<<<<<
 *     """Applies skip_features to layer.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_apply_skip_features(OGRLayerH __pyx_v_ogr_layer, int __pyx_v_skip_features) {
  OGRErr __pyx_v_err;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  int __pyx_t_13;
  char const *__pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("apply_skip_features", 0);

  /* "pyogrio/_io.pyx":921
 *         Number of features to skip from the beginning of the layer
 *     """
 *     with nogil:             # <<<<<<<<<<<<<<
 *         err = OGR_L_SetNextByIndex(ogr_layer, skip_features)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":922
 *     """
 *     with nogil:
 *         err = OGR_L_SetNextByIndex(ogr_layer, skip_features)             # <<<<<<<<<<<<<<
 * 
 *     # GDAL can raise an error (depending on the format) for out-of-bound index,
*/
        __pyx_v_err = OGR_L_SetNextByIndex(__pyx_v_ogr_layer, __pyx_v_skip_features);
      }

      /* "pyogrio/_io.pyx":921
 *         Number of features to skip from the beginning of the layer
 *     """
 *     with nogil:             # <<<<<<<<<<<<<<
 *         err = OGR_L_SetNextByIndex(ogr_layer, skip_features)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":926
 *     # GDAL can raise an error (depending on the format) for out-of-bound index,
 *     # but `validate_feature_range()` should ensure we only pass a valid number
 *     if err != OGRERR_NONE:             # <<<<<<<<<<<<<<
 *         try:
 *             check_last_error()
*/
  __pyx_t_1 = (__pyx_v_err != OGRERR_NONE);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":927
 *     # but `validate_feature_range()` should ensure we only pass a valid number
 *     if err != OGRERR_NONE:
 *         try:             # <<<<<<<<<<<<<<
 *             check_last_error()
 *         except CPLE_BaseError as exc:
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      /*try:*/ {

        /* "pyogrio/_io.pyx":928
 *     if err != OGRERR_NONE:
 *         try:
 *             check_last_error()             # <<<<<<<<<<<<<<
 *         except CPLE_BaseError as exc:
 *             raise ValueError(str(exc))
*/
        __pyx_t_5 = __pyx_f_7pyogrio_4_err_check_last_error(); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 928, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "pyogrio/_io.pyx":927
 *     # but `validate_feature_range()` should ensure we only pass a valid number
 *     if err != OGRERR_NONE:
 *         try:             # <<<<<<<<<<<<<<
 *             check_last_error()
 *         except CPLE_BaseError as exc:
*/
      }
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L12_try_end;
      __pyx_L7_error:;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "pyogrio/_io.pyx":929
 *         try:
 *             check_last_error()
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             raise ValueError(str(exc))
 * 
*/
      __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 929, __pyx_L9_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
      if (__pyx_t_9) {
        __Pyx_AddTraceback("pyogrio._io.apply_skip_features", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 929, __pyx_L9_except_error)
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __pyx_v_exc = __pyx_t_6;
        /*try:*/ {

          /* "pyogrio/_io.pyx":930
 *             check_last_error()
 *         except CPLE_BaseError as exc:
 *             raise ValueError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *         raise ValueError(f"Applying {skip_features=} raised an error")
*/
          __pyx_t_10 = NULL;
          __pyx_t_11 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 930, __pyx_L18_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = 1;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_11};
            __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 930, __pyx_L18_error)
            __Pyx_GOTREF(__pyx_t_8);
          }
          __Pyx_Raise(__pyx_t_8, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __PYX_ERR(0, 930, __pyx_L18_error)
        }

        /* "pyogrio/_io.pyx":929
 *         try:
 *             check_last_error()
 *         except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *             raise ValueError(str(exc))
 * 
*/
        /*finally:*/ {
          __pyx_L18_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
            if ( unlikely(__Pyx_GetException(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17) < 0)) __Pyx_ErrFetch(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_20);
            __pyx_t_9 = __pyx_lineno; __pyx_t_13 = __pyx_clineno; __pyx_t_14 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_ErrRestore(__pyx_t_15, __pyx_t_16, __pyx_t_17);
            __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
            __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_13; __pyx_filename = __pyx_t_14;
            goto __pyx_L9_except_error;
          }
        }
      }
      goto __pyx_L9_except_error;

      /* "pyogrio/_io.pyx":927
 *     # but `validate_feature_range()` should ensure we only pass a valid number
 *     if err != OGRERR_NONE:
 *         try:             # <<<<<<<<<<<<<<
 *             check_last_error()
 *         except CPLE_BaseError as exc:
*/
      __pyx_L9_except_error:;
      __Pyx_XGIVEREF(__pyx_t_2);
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
      goto __pyx_L1_error;
      __pyx_L12_try_end:;
    }

    /* "pyogrio/_io.pyx":932
 *             raise ValueError(str(exc))
 * 
 *         raise ValueError(f"Applying {skip_features=} raised an error")             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_6 = NULL;
    __pyx_t_7 = __Pyx_PyUnicode_From_int(__pyx_v_skip_features, 0, ' ', 'd'); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 932, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_21[0] = __pyx_mstate_global->__pyx_kp_u_Applying_skip_features;
    __pyx_t_21[1] = __pyx_t_7;
    __pyx_t_21[2] = __pyx_mstate_global->__pyx_kp_u_raised_an_error;
    __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_21, 3, 23 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 16, 127);
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 932, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_8};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 932, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 932, __pyx_L1_error)

    /* "pyogrio/_io.pyx":926
 *     # GDAL can raise an error (depending on the format) for out-of-bound index,
 *     # but `validate_feature_range()` should ensure we only pass a valid number
 *     if err != OGRERR_NONE:             # <<<<<<<<<<<<<<
 *         try:
 *             check_last_error()
*/
  }

  /* "pyogrio/_io.pyx":911
 * 
 * 
 * cdef apply_skip_features(OGRLayerH ogr_layer, int skip_features):             # <<<<<<<<<<<<<<
 *     """Applies skip_features to layer.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pyogrio._io.apply_skip_features", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":935
 * 
 * 
 * cdef validate_feature_range(             # <<<<<<<<<<<<<<
 *     OGRLayerH ogr_layer, int skip_features=0, int max_features=0
 * ):
*/

static PyObject *__pyx_f_7pyogrio_3_io_validate_feature_range(OGRLayerH __pyx_v_ogr_layer, struct __pyx_opt_args_7pyogrio_3_io_validate_feature_range *__pyx_optional_args) {
  int __pyx_v_skip_features = ((int)0);
  int __pyx_v_max_features = ((int)0);
  PyObject *__pyx_v_feature_count = NULL;
  PyObject *__pyx_v_num_features = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("validate_feature_range", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_skip_features = __pyx_optional_args->skip_features;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_max_features = __pyx_optional_args->max_features;
      }
    }
  }

  /* "pyogrio/_io.pyx":957
 *         A tuple containing the adjusted skip_features and max_features values
 *     """
 *     feature_count = get_feature_count(ogr_layer, 1)             # <<<<<<<<<<<<<<
 *     num_features = max_features
 * 
*/
  __pyx_t_1 = __pyx_f_7pyogrio_3_io_get_feature_count(__pyx_v_ogr_layer, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 957, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_feature_count = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":958
 *     """
 *     feature_count = get_feature_count(ogr_layer, 1)
 *     num_features = max_features             # <<<<<<<<<<<<<<
 * 
 *     if feature_count == 0:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_max_features); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_num_features = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":960
 *     num_features = max_features
 * 
 *     if feature_count == 0:             # <<<<<<<<<<<<<<
 *         return 0, 0
 * 
*/
  __pyx_t_2 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_feature_count, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 960, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":961
 * 
 *     if feature_count == 0:
 *         return 0, 0             # <<<<<<<<<<<<<<
 * 
 *     if skip_features >= feature_count:
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_tuple[4]);
    __pyx_r = __pyx_mstate_global->__pyx_tuple[4];
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":960
 *     num_features = max_features
 * 
 *     if feature_count == 0:             # <<<<<<<<<<<<<<
 *         return 0, 0
 * 
*/
  }

  /* "pyogrio/_io.pyx":963
 *         return 0, 0
 * 
 *     if skip_features >= feature_count:             # <<<<<<<<<<<<<<
 *         return 0, 0
 * 
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_skip_features); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_v_feature_count, Py_GE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 963, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":964
 * 
 *     if skip_features >= feature_count:
 *         return 0, 0             # <<<<<<<<<<<<<<
 * 
 *     if max_features == 0:
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_tuple[4]);
    __pyx_r = __pyx_mstate_global->__pyx_tuple[4];
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":963
 *         return 0, 0
 * 
 *     if skip_features >= feature_count:             # <<<<<<<<<<<<<<
 *         return 0, 0
 * 
*/
  }

  /* "pyogrio/_io.pyx":966
 *         return 0, 0
 * 
 *     if max_features == 0:             # <<<<<<<<<<<<<<
 *         num_features = feature_count - skip_features
 * 
*/
  __pyx_t_2 = (__pyx_v_max_features == 0);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":967
 * 
 *     if max_features == 0:
 *         num_features = feature_count - skip_features             # <<<<<<<<<<<<<<
 * 
 *     elif max_features > feature_count:
*/
    __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_skip_features); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyNumber_Subtract(__pyx_v_feature_count, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 967, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_num_features, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyogrio/_io.pyx":966
 *         return 0, 0
 * 
 *     if max_features == 0:             # <<<<<<<<<<<<<<
 *         num_features = feature_count - skip_features
 * 
*/
    goto __pyx_L5;
  }

  /* "pyogrio/_io.pyx":969
 *         num_features = feature_count - skip_features
 * 
 *     elif max_features > feature_count:             # <<<<<<<<<<<<<<
 *         num_features = feature_count
 * 
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_max_features); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 969, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_v_feature_count, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 969, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 969, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":970
 * 
 *     elif max_features > feature_count:
 *         num_features = feature_count             # <<<<<<<<<<<<<<
 * 
 *     return skip_features, num_features
*/
    __Pyx_INCREF(__pyx_v_feature_count);
    __Pyx_DECREF_SET(__pyx_v_num_features, __pyx_v_feature_count);

    /* "pyogrio/_io.pyx":969
 *         num_features = feature_count - skip_features
 * 
 *     elif max_features > feature_count:             # <<<<<<<<<<<<<<
 *         num_features = feature_count
 * 
*/
  }
  __pyx_L5:;

  /* "pyogrio/_io.pyx":972
 *         num_features = feature_count
 * 
 *     return skip_features, num_features             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_skip_features); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 972, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_num_features);
  __Pyx_GIVEREF(__pyx_v_num_features);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_num_features) != (0)) __PYX_ERR(0, 972, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":935
 * 
 * 
 * cdef validate_feature_range(             # <<<<<<<<<<<<<<
 *     OGRLayerH ogr_layer, int skip_features=0, int max_features=0
 * ):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyogrio._io.validate_feature_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_feature_count);
  __Pyx_XDECREF(__pyx_v_num_features);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":975
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef process_geometry(OGRFeatureH ogr_feature, int i, geom_view, uint8_t force_2d):
*/

static PyObject *__pyx_f_7pyogrio_3_io_process_geometry(OGRFeatureH __pyx_v_ogr_feature, int __pyx_v_i, PyObject *__pyx_v_geom_view, uint8_t __pyx_v_force_2d) {
  OGRGeometryH __pyx_v_ogr_geometry;
  OGRwkbGeometryType __pyx_v_ogr_geometry_type;
  unsigned char *__pyx_v_wkb;
  int __pyx_v_ret_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  char const *__pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_geometry", 0);

  /* "pyogrio/_io.pyx":994
 * 
 *     """
 *     cdef OGRGeometryH ogr_geometry = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRwkbGeometryType ogr_geometry_type
 * 
*/
  __pyx_v_ogr_geometry = NULL;

  /* "pyogrio/_io.pyx":997
 *     cdef OGRwkbGeometryType ogr_geometry_type
 * 
 *     cdef unsigned char *wkb = NULL             # <<<<<<<<<<<<<<
 *     cdef int ret_length
 * 
*/
  __pyx_v_wkb = NULL;

  /* "pyogrio/_io.pyx":1000
 *     cdef int ret_length
 * 
 *     ogr_geometry = OGR_F_GetGeometryRef(ogr_feature)             # <<<<<<<<<<<<<<
 * 
 *     if ogr_geometry == NULL:
*/
  __pyx_v_ogr_geometry = OGR_F_GetGeometryRef(__pyx_v_ogr_feature);

  /* "pyogrio/_io.pyx":1002
 *     ogr_geometry = OGR_F_GetGeometryRef(ogr_feature)
 * 
 *     if ogr_geometry == NULL:             # <<<<<<<<<<<<<<
 *         geom_view[i] = None
 *     else:
*/
  __pyx_t_1 = (__pyx_v_ogr_geometry == NULL);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1003
 * 
 *     if ogr_geometry == NULL:
 *         geom_view[i] = None             # <<<<<<<<<<<<<<
 *     else:
 *         try:
*/
    if (unlikely((__Pyx_SetItemInt(__pyx_v_geom_view, __pyx_v_i, Py_None, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_FunctionArgument) < 0))) __PYX_ERR(0, 1003, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1002
 *     ogr_geometry = OGR_F_GetGeometryRef(ogr_feature)
 * 
 *     if ogr_geometry == NULL:             # <<<<<<<<<<<<<<
 *         geom_view[i] = None
 *     else:
*/
    goto __pyx_L3;
  }

  /* "pyogrio/_io.pyx":1005
 *         geom_view[i] = None
 *     else:
 *         try:             # <<<<<<<<<<<<<<
 *             ogr_geometry_type = OGR_G_GetGeometryType(ogr_geometry)
 * 
*/
  /*else*/ {
    /*try:*/ {

      /* "pyogrio/_io.pyx":1006
 *     else:
 *         try:
 *             ogr_geometry_type = OGR_G_GetGeometryType(ogr_geometry)             # <<<<<<<<<<<<<<
 * 
 *             # if geometry has M values, these need to be removed first
*/
      __pyx_v_ogr_geometry_type = OGR_G_GetGeometryType(__pyx_v_ogr_geometry);

      /* "pyogrio/_io.pyx":1009
 * 
 *             # if geometry has M values, these need to be removed first
 *             if (OGR_G_IsMeasured(ogr_geometry)):             # <<<<<<<<<<<<<<
 *                 OGR_G_SetMeasured(ogr_geometry, 0)
 * 
*/
      __pyx_t_1 = (OGR_G_IsMeasured(__pyx_v_ogr_geometry) != 0);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1010
 *             # if geometry has M values, these need to be removed first
 *             if (OGR_G_IsMeasured(ogr_geometry)):
 *                 OGR_G_SetMeasured(ogr_geometry, 0)             # <<<<<<<<<<<<<<
 * 
 *             if force_2d and OGR_G_Is3D(ogr_geometry):
*/
        OGR_G_SetMeasured(__pyx_v_ogr_geometry, 0);

        /* "pyogrio/_io.pyx":1009
 * 
 *             # if geometry has M values, these need to be removed first
 *             if (OGR_G_IsMeasured(ogr_geometry)):             # <<<<<<<<<<<<<<
 *                 OGR_G_SetMeasured(ogr_geometry, 0)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1012
 *                 OGR_G_SetMeasured(ogr_geometry, 0)
 * 
 *             if force_2d and OGR_G_Is3D(ogr_geometry):             # <<<<<<<<<<<<<<
 *                 OGR_G_Set3D(ogr_geometry, 0)
 * 
*/
      __pyx_t_2 = (__pyx_v_force_2d != 0);
      if (__pyx_t_2) {
      } else {
        __pyx_t_1 = __pyx_t_2;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_2 = (OGR_G_Is3D(__pyx_v_ogr_geometry) != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1013
 * 
 *             if force_2d and OGR_G_Is3D(ogr_geometry):
 *                 OGR_G_Set3D(ogr_geometry, 0)             # <<<<<<<<<<<<<<
 * 
 *             # if non-linear (e.g., curve), force to linear type
*/
        OGR_G_Set3D(__pyx_v_ogr_geometry, 0);

        /* "pyogrio/_io.pyx":1012
 *                 OGR_G_SetMeasured(ogr_geometry, 0)
 * 
 *             if force_2d and OGR_G_Is3D(ogr_geometry):             # <<<<<<<<<<<<<<
 *                 OGR_G_Set3D(ogr_geometry, 0)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1016
 * 
 *             # if non-linear (e.g., curve), force to linear type
 *             if OGR_GT_IsNonLinear(ogr_geometry_type):             # <<<<<<<<<<<<<<
 *                 ogr_geometry = OGR_G_GetLinearGeometry(ogr_geometry, 0, NULL)
 * 
*/
      __pyx_t_1 = (OGR_GT_IsNonLinear(__pyx_v_ogr_geometry_type) != 0);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1017
 *             # if non-linear (e.g., curve), force to linear type
 *             if OGR_GT_IsNonLinear(ogr_geometry_type):
 *                 ogr_geometry = OGR_G_GetLinearGeometry(ogr_geometry, 0, NULL)             # <<<<<<<<<<<<<<
 * 
 *             ret_length = OGR_G_WkbSize(ogr_geometry)
*/
        __pyx_v_ogr_geometry = OGR_G_GetLinearGeometry(__pyx_v_ogr_geometry, 0.0, NULL);

        /* "pyogrio/_io.pyx":1016
 * 
 *             # if non-linear (e.g., curve), force to linear type
 *             if OGR_GT_IsNonLinear(ogr_geometry_type):             # <<<<<<<<<<<<<<
 *                 ogr_geometry = OGR_G_GetLinearGeometry(ogr_geometry, 0, NULL)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1019
 *                 ogr_geometry = OGR_G_GetLinearGeometry(ogr_geometry, 0, NULL)
 * 
 *             ret_length = OGR_G_WkbSize(ogr_geometry)             # <<<<<<<<<<<<<<
 *             wkb = <unsigned char*>malloc(sizeof(unsigned char)*ret_length)
 *             OGR_G_ExportToWkb(ogr_geometry, 1, wkb)
*/
      __pyx_v_ret_length = OGR_G_WkbSize(__pyx_v_ogr_geometry);

      /* "pyogrio/_io.pyx":1020
 * 
 *             ret_length = OGR_G_WkbSize(ogr_geometry)
 *             wkb = <unsigned char*>malloc(sizeof(unsigned char)*ret_length)             # <<<<<<<<<<<<<<
 *             OGR_G_ExportToWkb(ogr_geometry, 1, wkb)
 *             geom_view[i] = wkb[:ret_length]
*/
      __pyx_v_wkb = ((unsigned char *)malloc(((sizeof(unsigned char)) * __pyx_v_ret_length)));

      /* "pyogrio/_io.pyx":1021
 *             ret_length = OGR_G_WkbSize(ogr_geometry)
 *             wkb = <unsigned char*>malloc(sizeof(unsigned char)*ret_length)
 *             OGR_G_ExportToWkb(ogr_geometry, 1, wkb)             # <<<<<<<<<<<<<<
 *             geom_view[i] = wkb[:ret_length]
 * 
*/
      OGR_G_ExportToWkb(__pyx_v_ogr_geometry, 1, __pyx_v_wkb);

      /* "pyogrio/_io.pyx":1022
 *             wkb = <unsigned char*>malloc(sizeof(unsigned char)*ret_length)
 *             OGR_G_ExportToWkb(ogr_geometry, 1, wkb)
 *             geom_view[i] = wkb[:ret_length]             # <<<<<<<<<<<<<<
 * 
 *         finally:
*/
      __pyx_t_3 = __Pyx_PyBytes_FromStringAndSize(((char const *)__pyx_v_wkb) + 0, __pyx_v_ret_length - 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1022, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely((__Pyx_SetItemInt(__pyx_v_geom_view, __pyx_v_i, __pyx_t_3, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_FunctionArgument) < 0))) __PYX_ERR(0, 1022, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }

    /* "pyogrio/_io.pyx":1025
 * 
 *         finally:
 *             free(wkb)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    /*finally:*/ {
      /*normal exit:*/{
        free(__pyx_v_wkb);
        goto __pyx_L6;
      }
      __pyx_L5_error:;
      /*exception exit:*/{
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
         __Pyx_ExceptionSwap(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
        if ( unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_12);
        __pyx_t_4 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_6 = __pyx_filename;
        {
          free(__pyx_v_wkb);
        }
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ErrRestore(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0;
        __pyx_lineno = __pyx_t_4; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_6;
        goto __pyx_L1_error;
      }
      __pyx_L6:;
    }
  }
  __pyx_L3:;

  /* "pyogrio/_io.pyx":975
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef process_geometry(OGRFeatureH ogr_feature, int i, geom_view, uint8_t force_2d):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyogrio._io.process_geometry", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1028
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef process_fields(
*/

static PyObject *__pyx_f_7pyogrio_3_io_process_fields(OGRFeatureH __pyx_v_ogr_feature, int __pyx_v_i, int __pyx_v_n_fields, PyObject *__pyx_v_field_data, PyObject *__pyx_v_field_data_view, PyObject *__pyx_v_field_indexes, PyObject *__pyx_v_field_ogr_types, PyObject *__pyx_v_encoding, int __pyx_v_datetime_as_string) {
  int __pyx_v_j;
  int __pyx_v_success;
  int __pyx_v_field_index;
  int __pyx_v_field_type;
  int __pyx_v_ret_length;
  int const *__pyx_v_ints_c;
  GIntBig const *__pyx_v_int64s_c;
  double const *__pyx_v_doubles_c;
  char **__pyx_v_strings_c;
  GByte *__pyx_v_bin_value;
  int __pyx_v_year;
  int __pyx_v_month;
  int __pyx_v_day;
  int __pyx_v_hour;
  int __pyx_v_minute;
  float __pyx_v_fsecond;
  float __pyx_v_ss;
  int __pyx_v_timezone;
  PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_isnull = NULL;
  PyObject *__pyx_v_value = NULL;
  float __pyx_v_ms;
  PyObject *__pyx_v_second = NULL;
  PyObject *__pyx_v_microsecond = NULL;
  PyObject *__pyx_v_int_arr = NULL;
  PyObject *__pyx_v_double_arr = NULL;
  PyObject *__pyx_v_string_list_index = NULL;
  PyObject *__pyx_v_vals = NULL;
  PyObject *__pyx_v_val = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  size_t __pyx_t_11;
  int __pyx_t_12;
  struct __pyx_opt_args_7pyogrio_4_ogr_get_string __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_t_20;
  int __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  char const *__pyx_t_23;
  int __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_fields", 0);

  /* "pyogrio/_io.pyx":1077
 *     cdef char **strings_c
 *     cdef GByte *bin_value
 *     cdef int year = 0             # <<<<<<<<<<<<<<
 *     cdef int month = 0
 *     cdef int day = 0
*/
  __pyx_v_year = 0;

  /* "pyogrio/_io.pyx":1078
 *     cdef GByte *bin_value
 *     cdef int year = 0
 *     cdef int month = 0             # <<<<<<<<<<<<<<
 *     cdef int day = 0
 *     cdef int hour = 0
*/
  __pyx_v_month = 0;

  /* "pyogrio/_io.pyx":1079
 *     cdef int year = 0
 *     cdef int month = 0
 *     cdef int day = 0             # <<<<<<<<<<<<<<
 *     cdef int hour = 0
 *     cdef int minute = 0
*/
  __pyx_v_day = 0;

  /* "pyogrio/_io.pyx":1080
 *     cdef int month = 0
 *     cdef int day = 0
 *     cdef int hour = 0             # <<<<<<<<<<<<<<
 *     cdef int minute = 0
 *     cdef float fsecond = 0.0
*/
  __pyx_v_hour = 0;

  /* "pyogrio/_io.pyx":1081
 *     cdef int day = 0
 *     cdef int hour = 0
 *     cdef int minute = 0             # <<<<<<<<<<<<<<
 *     cdef float fsecond = 0.0
 *     cdef float ss = 0.0
*/
  __pyx_v_minute = 0;

  /* "pyogrio/_io.pyx":1082
 *     cdef int hour = 0
 *     cdef int minute = 0
 *     cdef float fsecond = 0.0             # <<<<<<<<<<<<<<
 *     cdef float ss = 0.0
 *     cdef int timezone = 0
*/
  __pyx_v_fsecond = 0.0;

  /* "pyogrio/_io.pyx":1083
 *     cdef int minute = 0
 *     cdef float fsecond = 0.0
 *     cdef float ss = 0.0             # <<<<<<<<<<<<<<
 *     cdef int timezone = 0
 * 
*/
  __pyx_v_ss = 0.0;

  /* "pyogrio/_io.pyx":1084
 *     cdef float fsecond = 0.0
 *     cdef float ss = 0.0
 *     cdef int timezone = 0             # <<<<<<<<<<<<<<
 * 
 *     for j in range(n_fields):
*/
  __pyx_v_timezone = 0;

  /* "pyogrio/_io.pyx":1086
 *     cdef int timezone = 0
 * 
 *     for j in range(n_fields):             # <<<<<<<<<<<<<<
 *         field_index = field_indexes[j]
 *         field_type = field_ogr_types[j]
*/
  __pyx_t_1 = __pyx_v_n_fields;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_j = __pyx_t_3;

    /* "pyogrio/_io.pyx":1087
 * 
 *     for j in range(n_fields):
 *         field_index = field_indexes[j]             # <<<<<<<<<<<<<<
 *         field_type = field_ogr_types[j]
 *         data = field_data_view[j]
*/
    if (unlikely(__pyx_v_field_indexes == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1087, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyLong_As_int(__Pyx_PyList_GET_ITEM(__pyx_v_field_indexes, __pyx_v_j)); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1087, __pyx_L1_error)
    __pyx_v_field_index = __pyx_t_4;

    /* "pyogrio/_io.pyx":1088
 *     for j in range(n_fields):
 *         field_index = field_indexes[j]
 *         field_type = field_ogr_types[j]             # <<<<<<<<<<<<<<
 *         data = field_data_view[j]
 * 
*/
    if (unlikely(__pyx_v_field_ogr_types == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1088, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyLong_As_int(__Pyx_PyList_GET_ITEM(__pyx_v_field_ogr_types, __pyx_v_j)); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1088, __pyx_L1_error)
    __pyx_v_field_type = __pyx_t_4;

    /* "pyogrio/_io.pyx":1089
 *         field_index = field_indexes[j]
 *         field_type = field_ogr_types[j]
 *         data = field_data_view[j]             # <<<<<<<<<<<<<<
 * 
 *         isnull = OGR_F_IsFieldSetAndNotNull(ogr_feature, field_index) == 0
*/
    if (unlikely(__pyx_v_field_data_view == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 1089, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyList_GET_ITEM(__pyx_v_field_data_view, __pyx_v_j);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_data, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":1091
 *         data = field_data_view[j]
 * 
 *         isnull = OGR_F_IsFieldSetAndNotNull(ogr_feature, field_index) == 0             # <<<<<<<<<<<<<<
 *         if isnull:
 *             if field_type in (OFTInteger, OFTInteger64, OFTReal):
*/
    __pyx_t_5 = __Pyx_PyBool_FromLong((OGR_F_IsFieldSetAndNotNull(__pyx_v_ogr_feature, __pyx_v_field_index) == 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1091, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_isnull, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":1092
 * 
 *         isnull = OGR_F_IsFieldSetAndNotNull(ogr_feature, field_index) == 0
 *         if isnull:             # <<<<<<<<<<<<<<
 *             if field_type in (OFTInteger, OFTInteger64, OFTReal):
 *                 # if a boolean or integer type, have to cast to float to hold
*/
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_isnull); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1092, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "pyogrio/_io.pyx":1093
 *         isnull = OGR_F_IsFieldSetAndNotNull(ogr_feature, field_index) == 0
 *         if isnull:
 *             if field_type in (OFTInteger, OFTInteger64, OFTReal):             # <<<<<<<<<<<<<<
 *                 # if a boolean or integer type, have to cast to float to hold
 *                 # NaN values
*/
      switch (__pyx_v_field_type) {
        case OFTInteger:
        case OFTInteger64:
        case OFTReal:
        __pyx_t_6 = 1;
        break;
        default:
        __pyx_t_6 = 0;
        break;
      }
      __pyx_t_7 = __pyx_t_6;
      if (__pyx_t_7) {

        /* "pyogrio/_io.pyx":1096
 *                 # if a boolean or integer type, have to cast to float to hold
 *                 # NaN values
 *                 if data.dtype.kind in ("b", "i", "u"):             # <<<<<<<<<<<<<<
 *                     field_data[j] = field_data[j].astype(np.float64)
 *                     field_data_view[j] = field_data[j][:]
*/
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1096, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1096, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_b, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1096, __pyx_L1_error)
        if (!__pyx_t_6) {
        } else {
          __pyx_t_7 = __pyx_t_6;
          goto __pyx_L8_bool_binop_done;
        }
        __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_i, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1096, __pyx_L1_error)
        if (!__pyx_t_6) {
        } else {
          __pyx_t_7 = __pyx_t_6;
          goto __pyx_L8_bool_binop_done;
        }
        __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_u, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1096, __pyx_L1_error)
        __pyx_t_7 = __pyx_t_6;
        __pyx_L8_bool_binop_done:;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_6 = __pyx_t_7;
        if (__pyx_t_6) {

          /* "pyogrio/_io.pyx":1097
 *                 # NaN values
 *                 if data.dtype.kind in ("b", "i", "u"):
 *                     field_data[j] = field_data[j].astype(np.float64)             # <<<<<<<<<<<<<<
 *                     field_data_view[j] = field_data[j][:]
 *                     field_data_view[j][i] = np.nan
*/
          if (unlikely(__pyx_v_field_data == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 1097, __pyx_L1_error)
          }
          __pyx_t_5 = __Pyx_PyList_GET_ITEM(__pyx_v_field_data, __pyx_v_j);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1097, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_float64); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1097, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_t_11 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_10};
            __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1097, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
          }
          if (unlikely(__pyx_v_field_data == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 1097, __pyx_L1_error)
          }
          if (unlikely((__Pyx_SetItemInt(__pyx_v_field_data, __pyx_v_j, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 1, 0, 0, 1, __Pyx_ReferenceSharing_FunctionArgument) < 0))) __PYX_ERR(0, 1097, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "pyogrio/_io.pyx":1098
 *                 if data.dtype.kind in ("b", "i", "u"):
 *                     field_data[j] = field_data[j].astype(np.float64)
 *                     field_data_view[j] = field_data[j][:]             # <<<<<<<<<<<<<<
 *                     field_data_view[j][i] = np.nan
 *                 else:
*/
          if (unlikely(__pyx_v_field_data == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 1098, __pyx_L1_error)
          }
          __pyx_t_8 = __Pyx_PyObject_GetSlice(__Pyx_PyList_GET_ITEM(__pyx_v_field_data, __pyx_v_j), 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1098, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (unlikely(__pyx_v_field_data_view == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 1098, __pyx_L1_error)
          }
          if (unlikely((__Pyx_SetItemInt(__pyx_v_field_data_view, __pyx_v_j, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 1, 0, 0, 1, __Pyx_ReferenceSharing_FunctionArgument) < 0))) __PYX_ERR(0, 1098, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "pyogrio/_io.pyx":1099
 *                     field_data[j] = field_data[j].astype(np.float64)
 *                     field_data_view[j] = field_data[j][:]
 *                     field_data_view[j][i] = np.nan             # <<<<<<<<<<<<<<
 *                 else:
 *                     data[i] = np.nan
*/
          __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1099, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_nan); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1099, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(__pyx_v_field_data_view == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 1099, __pyx_L1_error)
          }
          if (unlikely((__Pyx_SetItemInt(__Pyx_PyList_GET_ITEM(__pyx_v_field_data_view, __pyx_v_j), __pyx_v_i, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_SharedReference) < 0))) __PYX_ERR(0, 1099, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":1096
 *                 # if a boolean or integer type, have to cast to float to hold
 *                 # NaN values
 *                 if data.dtype.kind in ("b", "i", "u"):             # <<<<<<<<<<<<<<
 *                     field_data[j] = field_data[j].astype(np.float64)
 *                     field_data_view[j] = field_data[j][:]
*/
          goto __pyx_L7;
        }

        /* "pyogrio/_io.pyx":1101
 *                     field_data_view[j][i] = np.nan
 *                 else:
 *                     data[i] = np.nan             # <<<<<<<<<<<<<<
 * 
 *             elif field_type in (OFTDate, OFTDateTime) and not datetime_as_string:
*/
        /*else*/ {
          __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1101, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_nan); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1101, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1101, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __pyx_L7:;

        /* "pyogrio/_io.pyx":1093
 *         isnull = OGR_F_IsFieldSetAndNotNull(ogr_feature, field_index) == 0
 *         if isnull:
 *             if field_type in (OFTInteger, OFTInteger64, OFTReal):             # <<<<<<<<<<<<<<
 *                 # if a boolean or integer type, have to cast to float to hold
 *                 # NaN values
*/
        goto __pyx_L6;
      }

      /* "pyogrio/_io.pyx":1103
 *                     data[i] = np.nan
 * 
 *             elif field_type in (OFTDate, OFTDateTime) and not datetime_as_string:             # <<<<<<<<<<<<<<
 *                 data[i] = np.datetime64("NaT")
 * 
*/
      switch (__pyx_v_field_type) {
        case OFTDate:
        case OFTDateTime:
        __pyx_t_7 = 1;
        break;
        default:
        __pyx_t_7 = 0;
        break;
      }
      __pyx_t_12 = __pyx_t_7;
      if (__pyx_t_12) {
      } else {
        __pyx_t_6 = __pyx_t_12;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_12 = (!__pyx_v_datetime_as_string);
      __pyx_t_6 = __pyx_t_12;
      __pyx_L11_bool_binop_done:;
      if (__pyx_t_6) {

        /* "pyogrio/_io.pyx":1104
 * 
 *             elif field_type in (OFTDate, OFTDateTime) and not datetime_as_string:
 *                 data[i] = np.datetime64("NaT")             # <<<<<<<<<<<<<<
 * 
 *             else:
*/
        __pyx_t_10 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1104, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_datetime64); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1104, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_11 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
          assert(__pyx_t_10);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
          __pyx_t_11 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_mstate_global->__pyx_n_u_NaT};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1104, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1104, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyogrio/_io.pyx":1103
 *                     data[i] = np.nan
 * 
 *             elif field_type in (OFTDate, OFTDateTime) and not datetime_as_string:             # <<<<<<<<<<<<<<
 *                 data[i] = np.datetime64("NaT")
 * 
*/
        goto __pyx_L6;
      }

      /* "pyogrio/_io.pyx":1107
 * 
 *             else:
 *                 data[i] = None             # <<<<<<<<<<<<<<
 * 
 *             continue
*/
      /*else*/ {
        if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, Py_None, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1107, __pyx_L1_error)
      }
      __pyx_L6:;

      /* "pyogrio/_io.pyx":1109
 *                 data[i] = None
 * 
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         if field_type == OFTInteger:
*/
      goto __pyx_L3_continue;

      /* "pyogrio/_io.pyx":1092
 * 
 *         isnull = OGR_F_IsFieldSetAndNotNull(ogr_feature, field_index) == 0
 *         if isnull:             # <<<<<<<<<<<<<<
 *             if field_type in (OFTInteger, OFTInteger64, OFTReal):
 *                 # if a boolean or integer type, have to cast to float to hold
*/
    }

    /* "pyogrio/_io.pyx":1111
 *             continue
 * 
 *         if field_type == OFTInteger:             # <<<<<<<<<<<<<<
 *             data[i] = OGR_F_GetFieldAsInteger(ogr_feature, field_index)
 * 
*/
    switch (__pyx_v_field_type) {
      case OFTInteger:

      /* "pyogrio/_io.pyx":1112
 * 
 *         if field_type == OFTInteger:
 *             data[i] = OGR_F_GetFieldAsInteger(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTInteger64:
*/
      __pyx_t_8 = __Pyx_PyLong_From_int(OGR_F_GetFieldAsInteger(__pyx_v_ogr_feature, __pyx_v_field_index)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1112, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":1111
 *             continue
 * 
 *         if field_type == OFTInteger:             # <<<<<<<<<<<<<<
 *             data[i] = OGR_F_GetFieldAsInteger(ogr_feature, field_index)
 * 
*/
      break;
      case OFTInteger64:

      /* "pyogrio/_io.pyx":1115
 * 
 *         elif field_type == OFTInteger64:
 *             data[i] = OGR_F_GetFieldAsInteger64(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTReal:
*/
      __pyx_t_8 = __Pyx_PyLong_From_int64_t(OGR_F_GetFieldAsInteger64(__pyx_v_ogr_feature, __pyx_v_field_index)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1115, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":1114
 *             data[i] = OGR_F_GetFieldAsInteger(ogr_feature, field_index)
 * 
 *         elif field_type == OFTInteger64:             # <<<<<<<<<<<<<<
 *             data[i] = OGR_F_GetFieldAsInteger64(ogr_feature, field_index)
 * 
*/
      break;
      case OFTReal:

      /* "pyogrio/_io.pyx":1118
 * 
 *         elif field_type == OFTReal:
 *             data[i] = OGR_F_GetFieldAsDouble(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTString:
*/
      __pyx_t_8 = PyFloat_FromDouble(OGR_F_GetFieldAsDouble(__pyx_v_ogr_feature, __pyx_v_field_index)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1118, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1118, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":1117
 *             data[i] = OGR_F_GetFieldAsInteger64(ogr_feature, field_index)
 * 
 *         elif field_type == OFTReal:             # <<<<<<<<<<<<<<
 *             data[i] = OGR_F_GetFieldAsDouble(ogr_feature, field_index)
 * 
*/
      break;
      case OFTString:

      /* "pyogrio/_io.pyx":1122
 *         elif field_type == OFTString:
 *             value = get_string(OGR_F_GetFieldAsString(
 *                 ogr_feature, field_index), encoding=encoding             # <<<<<<<<<<<<<<
 *             )
 *             data[i] = value
*/
      __pyx_t_8 = __pyx_v_encoding;
      __Pyx_INCREF(__pyx_t_8);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_8))) __PYX_ERR(0, 1122, __pyx_L1_error)

      /* "pyogrio/_io.pyx":1121
 * 
 *         elif field_type == OFTString:
 *             value = get_string(OGR_F_GetFieldAsString(             # <<<<<<<<<<<<<<
 *                 ogr_feature, field_index), encoding=encoding
 *             )
*/
      __pyx_t_13.__pyx_n = 1;
      __pyx_t_13.encoding = ((PyObject*)__pyx_t_8);
      __pyx_t_9 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_F_GetFieldAsString(__pyx_v_ogr_feature, __pyx_v_field_index), &__pyx_t_13); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_9);
      __pyx_t_9 = 0;

      /* "pyogrio/_io.pyx":1124
 *                 ogr_feature, field_index), encoding=encoding
 *             )
 *             data[i] = value             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTBinary:
*/
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_v_value, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1124, __pyx_L1_error)

      /* "pyogrio/_io.pyx":1120
 *             data[i] = OGR_F_GetFieldAsDouble(ogr_feature, field_index)
 * 
 *         elif field_type == OFTString:             # <<<<<<<<<<<<<<
 *             value = get_string(OGR_F_GetFieldAsString(
 *                 ogr_feature, field_index), encoding=encoding
*/
      break;
      case OFTBinary:

      /* "pyogrio/_io.pyx":1127
 * 
 *         elif field_type == OFTBinary:
 *             bin_value = OGR_F_GetFieldAsBinary(ogr_feature, field_index, &ret_length)             # <<<<<<<<<<<<<<
 *             data[i] = bin_value[:ret_length]
 * 
*/
      __pyx_v_bin_value = OGR_F_GetFieldAsBinary(__pyx_v_ogr_feature, __pyx_v_field_index, (&__pyx_v_ret_length));

      /* "pyogrio/_io.pyx":1128
 *         elif field_type == OFTBinary:
 *             bin_value = OGR_F_GetFieldAsBinary(ogr_feature, field_index, &ret_length)
 *             data[i] = bin_value[:ret_length]             # <<<<<<<<<<<<<<
 * 
 *         elif field_type in (OFTDateTime, OFTDate, OFTTime):
*/
      __pyx_t_9 = __Pyx_PyBytes_FromStringAndSize(((char const *)__pyx_v_bin_value) + 0, __pyx_v_ret_length - 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1128, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_9, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1128, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "pyogrio/_io.pyx":1126
 *             data[i] = value
 * 
 *         elif field_type == OFTBinary:             # <<<<<<<<<<<<<<
 *             bin_value = OGR_F_GetFieldAsBinary(ogr_feature, field_index, &ret_length)
 *             data[i] = bin_value[:ret_length]
*/
      break;
      case OFTDateTime:

      /* "pyogrio/_io.pyx":1130
 *             data[i] = bin_value[:ret_length]
 * 
 *         elif field_type in (OFTDateTime, OFTDate, OFTTime):             # <<<<<<<<<<<<<<
 *             if field_type == OFTDateTime and datetime_as_string:
 *                 # defer datetime parsing to user/ pandas layer
*/
      case OFTDate:
      case OFTTime:

      /* "pyogrio/_io.pyx":1131
 * 
 *         elif field_type in (OFTDateTime, OFTDate, OFTTime):
 *             if field_type == OFTDateTime and datetime_as_string:             # <<<<<<<<<<<<<<
 *                 # defer datetime parsing to user/ pandas layer
 *                 IF CTE_GDAL_VERSION >= (3, 7, 0):
*/
      __pyx_t_12 = (__pyx_v_field_type == OFTDateTime);
      if (__pyx_t_12) {
      } else {
        __pyx_t_6 = __pyx_t_12;
        goto __pyx_L14_bool_binop_done;
      }
      __pyx_t_6 = __pyx_v_datetime_as_string;
      __pyx_L14_bool_binop_done:;
      if (__pyx_t_6) {

        /* "pyogrio/_io.pyx":1136
 *                     data[i] = get_string(
 *                         OGR_F_GetFieldAsISO8601DateTime(ogr_feature, field_index, NULL),
 *                         encoding=encoding,             # <<<<<<<<<<<<<<
 *                     )
 *                 ELSE:
*/
        __pyx_t_9 = __pyx_v_encoding;
        __Pyx_INCREF(__pyx_t_9);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_9))||((__pyx_t_9) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_9))) __PYX_ERR(0, 1136, __pyx_L1_error)

        /* "pyogrio/_io.pyx":1134
 *                 # defer datetime parsing to user/ pandas layer
 *                 IF CTE_GDAL_VERSION >= (3, 7, 0):
 *                     data[i] = get_string(             # <<<<<<<<<<<<<<
 *                         OGR_F_GetFieldAsISO8601DateTime(ogr_feature, field_index, NULL),
 *                         encoding=encoding,
*/
        __pyx_t_13.__pyx_n = 1;
        __pyx_t_13.encoding = ((PyObject*)__pyx_t_9);
        __pyx_t_8 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_F_GetFieldAsISO8601DateTime(__pyx_v_ogr_feature, __pyx_v_field_index, NULL), &__pyx_t_13); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1134, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_8, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1134, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyogrio/_io.pyx":1131
 * 
 *         elif field_type in (OFTDateTime, OFTDate, OFTTime):
 *             if field_type == OFTDateTime and datetime_as_string:             # <<<<<<<<<<<<<<
 *                 # defer datetime parsing to user/ pandas layer
 *                 IF CTE_GDAL_VERSION >= (3, 7, 0):
*/
        goto __pyx_L13;
      }

      /* "pyogrio/_io.pyx":1144
 *                     )
 *             else:
 *                 success = OGR_F_GetFieldAsDateTimeEx(             # <<<<<<<<<<<<<<
 *                     ogr_feature,
 *                     field_index,
*/
      /*else*/ {

        /* "pyogrio/_io.pyx":1153
 *                     &minute,
 *                     &fsecond,
 *                     &timezone,             # <<<<<<<<<<<<<<
 *                 )
 * 
*/
        __pyx_v_success = OGR_F_GetFieldAsDateTimeEx(__pyx_v_ogr_feature, __pyx_v_field_index, (&__pyx_v_year), (&__pyx_v_month), (&__pyx_v_day), (&__pyx_v_hour), (&__pyx_v_minute), (&__pyx_v_fsecond), (&__pyx_v_timezone));

        /* "pyogrio/_io.pyx":1156
 *                 )
 * 
 *                 ms = modff(fsecond, &ss)             # <<<<<<<<<<<<<<
 *                 second = int(ss)
 *                 # fsecond has millisecond accuracy
*/
        __pyx_v_ms = modff(__pyx_v_fsecond, (&__pyx_v_ss));

        /* "pyogrio/_io.pyx":1157
 * 
 *                 ms = modff(fsecond, &ss)
 *                 second = int(ss)             # <<<<<<<<<<<<<<
 *                 # fsecond has millisecond accuracy
 *                 microsecond = round(ms * 1000) * 1000
*/
        __pyx_t_8 = PyLong_FromDouble(__pyx_v_ss); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1157, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_XDECREF_SET(__pyx_v_second, ((PyObject*)__pyx_t_8));
        __pyx_t_8 = 0;

        /* "pyogrio/_io.pyx":1159
 *                 second = int(ss)
 *                 # fsecond has millisecond accuracy
 *                 microsecond = round(ms * 1000) * 1000             # <<<<<<<<<<<<<<
 * 
 *                 if not success:
*/
        __pyx_t_9 = NULL;
        __pyx_t_10 = PyFloat_FromDouble((__pyx_v_ms * 1000.0)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1159, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_10};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_round, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1159, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __pyx_t_10 = __Pyx_PyLong_MultiplyObjC(__pyx_t_8, __pyx_mstate_global->__pyx_int_1000, 0x3E8, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1159, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF_SET(__pyx_v_microsecond, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "pyogrio/_io.pyx":1161
 *                 microsecond = round(ms * 1000) * 1000
 * 
 *                 if not success:             # <<<<<<<<<<<<<<
 *                     data[i] = np.datetime64("NaT")
 * 
*/
        __pyx_t_6 = (!(__pyx_v_success != 0));
        if (__pyx_t_6) {

          /* "pyogrio/_io.pyx":1162
 * 
 *                 if not success:
 *                     data[i] = np.datetime64("NaT")             # <<<<<<<<<<<<<<
 * 
 *                 elif field_type == OFTDate:
*/
          __pyx_t_8 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1162, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_datetime64); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1162, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_t_11 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
            assert(__pyx_t_8);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
            __pyx_t_11 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_mstate_global->__pyx_n_u_NaT};
            __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1162, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
          }
          if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1162, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":1161
 *                 microsecond = round(ms * 1000) * 1000
 * 
 *                 if not success:             # <<<<<<<<<<<<<<
 *                     data[i] = np.datetime64("NaT")
 * 
*/
          goto __pyx_L16;
        }

        /* "pyogrio/_io.pyx":1164
 *                     data[i] = np.datetime64("NaT")
 * 
 *                 elif field_type == OFTDate:             # <<<<<<<<<<<<<<
 *                     data[i] = datetime.date(year, month, day).isoformat()
 * 
*/
        __pyx_t_6 = (__pyx_v_field_type == OFTDate);
        if (__pyx_t_6) {

          /* "pyogrio/_io.pyx":1165
 * 
 *                 elif field_type == OFTDate:
 *                     data[i] = datetime.date(year, month, day).isoformat()             # <<<<<<<<<<<<<<
 * 
 *                 elif field_type == OFTDateTime:
*/
          __pyx_t_9 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_datetime); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_date); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_14 = __Pyx_PyLong_From_int(__pyx_v_year); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_16 = __Pyx_PyLong_From_int(__pyx_v_month); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __pyx_t_17 = __Pyx_PyLong_From_int(__pyx_v_day); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_11 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_15))) {
            __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_15);
            assert(__pyx_t_9);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
            __pyx_t_11 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[4] = {__pyx_t_9, __pyx_t_14, __pyx_t_16, __pyx_t_17};
            __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_11, (4-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1165, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
          }
          __pyx_t_5 = __pyx_t_8;
          __Pyx_INCREF(__pyx_t_5);
          __pyx_t_11 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_5, NULL};
            __pyx_t_10 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_isoformat, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1165, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
          }
          if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1165, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":1164
 *                     data[i] = np.datetime64("NaT")
 * 
 *                 elif field_type == OFTDate:             # <<<<<<<<<<<<<<
 *                     data[i] = datetime.date(year, month, day).isoformat()
 * 
*/
          goto __pyx_L16;
        }

        /* "pyogrio/_io.pyx":1167
 *                     data[i] = datetime.date(year, month, day).isoformat()
 * 
 *                 elif field_type == OFTDateTime:             # <<<<<<<<<<<<<<
 *                     data[i] = datetime.datetime(
 *                         year, month, day, hour, minute, second, microsecond
*/
        __pyx_t_6 = (__pyx_v_field_type == OFTDateTime);
        if (__pyx_t_6) {

          /* "pyogrio/_io.pyx":1168
 * 
 *                 elif field_type == OFTDateTime:
 *                     data[i] = datetime.datetime(             # <<<<<<<<<<<<<<
 *                         year, month, day, hour, minute, second, microsecond
 *                     ).isoformat()
*/
          __pyx_t_15 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_datetime); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1168, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_datetime); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1168, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

          /* "pyogrio/_io.pyx":1169
 *                 elif field_type == OFTDateTime:
 *                     data[i] = datetime.datetime(
 *                         year, month, day, hour, minute, second, microsecond             # <<<<<<<<<<<<<<
 *                     ).isoformat()
 * 
*/
          __pyx_t_17 = __Pyx_PyLong_From_int(__pyx_v_year); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1169, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_14 = __Pyx_PyLong_From_int(__pyx_v_month); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1169, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_9 = __Pyx_PyLong_From_int(__pyx_v_day); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1169, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_18 = __Pyx_PyLong_From_int(__pyx_v_hour); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1169, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_18);
          __pyx_t_19 = __Pyx_PyLong_From_int(__pyx_v_minute); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1169, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_19);
          __pyx_t_11 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_16))) {
            __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_16);
            assert(__pyx_t_15);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
            __pyx_t_11 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[8] = {__pyx_t_15, __pyx_t_17, __pyx_t_14, __pyx_t_9, __pyx_t_18, __pyx_t_19, __pyx_v_second, __pyx_v_microsecond};
            __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_11, (8-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
            __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
            if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1168, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
          }
          __pyx_t_8 = __pyx_t_5;
          __Pyx_INCREF(__pyx_t_8);
          __pyx_t_11 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
            __pyx_t_10 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_isoformat, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1170, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
          }

          /* "pyogrio/_io.pyx":1168
 * 
 *                 elif field_type == OFTDateTime:
 *                     data[i] = datetime.datetime(             # <<<<<<<<<<<<<<
 *                         year, month, day, hour, minute, second, microsecond
 *                     ).isoformat()
*/
          if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1168, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":1167
 *                     data[i] = datetime.date(year, month, day).isoformat()
 * 
 *                 elif field_type == OFTDateTime:             # <<<<<<<<<<<<<<
 *                     data[i] = datetime.datetime(
 *                         year, month, day, hour, minute, second, microsecond
*/
          goto __pyx_L16;
        }

        /* "pyogrio/_io.pyx":1172
 *                     ).isoformat()
 * 
 *                 elif field_type == OFTTime:             # <<<<<<<<<<<<<<
 *                     data[i] = datetime.time(hour, minute, second, microsecond)
 * 
*/
        __pyx_t_6 = (__pyx_v_field_type == OFTTime);
        if (__pyx_t_6) {

          /* "pyogrio/_io.pyx":1173
 * 
 *                 elif field_type == OFTTime:
 *                     data[i] = datetime.time(hour, minute, second, microsecond)             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTIntegerList:
*/
          __pyx_t_5 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_datetime); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1173, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_time); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1173, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_8 = __Pyx_PyLong_From_int(__pyx_v_hour); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1173, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_19 = __Pyx_PyLong_From_int(__pyx_v_minute); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1173, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_19);
          __pyx_t_11 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_16))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_16);
            assert(__pyx_t_5);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
            __pyx_t_11 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[5] = {__pyx_t_5, __pyx_t_8, __pyx_t_19, __pyx_v_second, __pyx_v_microsecond};
            __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_11, (5-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
            __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1173, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_10);
          }
          if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1173, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":1172
 *                     ).isoformat()
 * 
 *                 elif field_type == OFTTime:             # <<<<<<<<<<<<<<
 *                     data[i] = datetime.time(hour, minute, second, microsecond)
 * 
*/
        }
        __pyx_L16:;
      }
      __pyx_L13:;

      /* "pyogrio/_io.pyx":1130
 *             data[i] = bin_value[:ret_length]
 * 
 *         elif field_type in (OFTDateTime, OFTDate, OFTTime):             # <<<<<<<<<<<<<<
 *             if field_type == OFTDateTime and datetime_as_string:
 *                 # defer datetime parsing to user/ pandas layer
*/
      break;
      case OFTIntegerList:

      /* "pyogrio/_io.pyx":1178
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
 *             ints_c = OGR_F_GetFieldAsIntegerList(ogr_feature, field_index, &ret_length)             # <<<<<<<<<<<<<<
 *             int_arr = np.ndarray(shape=(ret_length,), dtype=np.int32)
 *             for j in range(ret_length):
*/
      __pyx_v_ints_c = OGR_F_GetFieldAsIntegerList(__pyx_v_ogr_feature, __pyx_v_field_index, (&__pyx_v_ret_length));

      /* "pyogrio/_io.pyx":1179
 *             # valid result. So don't use check_pointer as it would throw an exception.
 *             ints_c = OGR_F_GetFieldAsIntegerList(ogr_feature, field_index, &ret_length)
 *             int_arr = np.ndarray(shape=(ret_length,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *             for j in range(ret_length):
 *                 int_arr[j] = ints_c[j]
*/
      __pyx_t_16 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __pyx_t_19 = __Pyx_PyLong_From_int(__pyx_v_ret_length); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_19);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_19) != (0)) __PYX_ERR(0, 1179, __pyx_L1_error);
      __pyx_t_19 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __pyx_t_11 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_16);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_11 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_16, NULL};
        __pyx_t_19 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1179, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_19);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_5, __pyx_t_19, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1179, __pyx_L1_error)
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_18, __pyx_t_19, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1179, __pyx_L1_error)
        __pyx_t_10 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_19);
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1179, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_int_arr, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":1180
 *             ints_c = OGR_F_GetFieldAsIntegerList(ogr_feature, field_index, &ret_length)
 *             int_arr = np.ndarray(shape=(ret_length,), dtype=np.int32)
 *             for j in range(ret_length):             # <<<<<<<<<<<<<<
 *                 int_arr[j] = ints_c[j]
 *             data[i] = int_arr
*/
      __pyx_t_4 = __pyx_v_ret_length;
      __pyx_t_20 = __pyx_t_4;
      for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
        __pyx_v_j = __pyx_t_21;

        /* "pyogrio/_io.pyx":1181
 *             int_arr = np.ndarray(shape=(ret_length,), dtype=np.int32)
 *             for j in range(ret_length):
 *                 int_arr[j] = ints_c[j]             # <<<<<<<<<<<<<<
 *             data[i] = int_arr
 * 
*/
        __pyx_t_10 = __Pyx_PyLong_From_int((__pyx_v_ints_c[__pyx_v_j])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1181, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (unlikely((__Pyx_SetItemInt(__pyx_v_int_arr, __pyx_v_j, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1181, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }

      /* "pyogrio/_io.pyx":1182
 *             for j in range(ret_length):
 *                 int_arr[j] = ints_c[j]
 *             data[i] = int_arr             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTInteger64List:
*/
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_v_int_arr, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1182, __pyx_L1_error)

      /* "pyogrio/_io.pyx":1175
 *                     data[i] = datetime.time(hour, minute, second, microsecond)
 * 
 *         elif field_type == OFTIntegerList:             # <<<<<<<<<<<<<<
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
*/
      break;
      case OFTInteger64List:

      /* "pyogrio/_io.pyx":1187
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
 *             int64s_c = OGR_F_GetFieldAsInteger64List(             # <<<<<<<<<<<<<<
 *                 ogr_feature, field_index, &ret_length
 *             )
*/
      __pyx_v_int64s_c = OGR_F_GetFieldAsInteger64List(__pyx_v_ogr_feature, __pyx_v_field_index, (&__pyx_v_ret_length));

      /* "pyogrio/_io.pyx":1191
 *             )
 * 
 *             int_arr = np.ndarray(shape=(ret_length,), dtype=np.int64)             # <<<<<<<<<<<<<<
 *             for j in range(ret_length):
 *                 int_arr[j] = int64s_c[j]
*/
      __pyx_t_8 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __pyx_t_19 = __Pyx_PyLong_From_int(__pyx_v_ret_length); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_19);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_19) != (0)) __PYX_ERR(0, 1191, __pyx_L1_error);
      __pyx_t_19 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __pyx_t_11 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_18))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_18);
        assert(__pyx_t_8);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_18);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_18, __pyx__function);
        __pyx_t_11 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_8, NULL};
        __pyx_t_19 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1191, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_19);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_5, __pyx_t_19, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1191, __pyx_L1_error)
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_16, __pyx_t_19, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1191, __pyx_L1_error)
        __pyx_t_10 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_18, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_19);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1191, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_int_arr, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":1192
 * 
 *             int_arr = np.ndarray(shape=(ret_length,), dtype=np.int64)
 *             for j in range(ret_length):             # <<<<<<<<<<<<<<
 *                 int_arr[j] = int64s_c[j]
 *             data[i] = int_arr
*/
      __pyx_t_4 = __pyx_v_ret_length;
      __pyx_t_20 = __pyx_t_4;
      for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
        __pyx_v_j = __pyx_t_21;

        /* "pyogrio/_io.pyx":1193
 *             int_arr = np.ndarray(shape=(ret_length,), dtype=np.int64)
 *             for j in range(ret_length):
 *                 int_arr[j] = int64s_c[j]             # <<<<<<<<<<<<<<
 *             data[i] = int_arr
 * 
*/
        __pyx_t_10 = __Pyx_PyLong_From_GIntBig((__pyx_v_int64s_c[__pyx_v_j])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1193, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (unlikely((__Pyx_SetItemInt(__pyx_v_int_arr, __pyx_v_j, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1193, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }

      /* "pyogrio/_io.pyx":1194
 *             for j in range(ret_length):
 *                 int_arr[j] = int64s_c[j]
 *             data[i] = int_arr             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTRealList:
*/
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_v_int_arr, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1194, __pyx_L1_error)

      /* "pyogrio/_io.pyx":1184
 *             data[i] = int_arr
 * 
 *         elif field_type == OFTInteger64List:             # <<<<<<<<<<<<<<
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
*/
      break;
      case OFTRealList:

      /* "pyogrio/_io.pyx":1199
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
 *             doubles_c = OGR_F_GetFieldAsDoubleList(             # <<<<<<<<<<<<<<
 *                 ogr_feature, field_index, &ret_length
 *             )
*/
      __pyx_v_doubles_c = OGR_F_GetFieldAsDoubleList(__pyx_v_ogr_feature, __pyx_v_field_index, (&__pyx_v_ret_length));

      /* "pyogrio/_io.pyx":1203
 *             )
 * 
 *             double_arr = np.ndarray(shape=(ret_length,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *             for j in range(ret_length):
 *                 double_arr[j] = doubles_c[j]
*/
      __pyx_t_18 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __pyx_t_19 = __Pyx_PyLong_From_int(__pyx_v_ret_length); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_19);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_19) != (0)) __PYX_ERR(0, 1203, __pyx_L1_error);
      __pyx_t_19 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_float64); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1203, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __pyx_t_11 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_16))) {
        __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_16);
        assert(__pyx_t_18);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_18);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
        __pyx_t_11 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_18, NULL};
        __pyx_t_19 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1203, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_19);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_5, __pyx_t_19, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1203, __pyx_L1_error)
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_8, __pyx_t_19, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1203, __pyx_L1_error)
        __pyx_t_10 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_19);
        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1203, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_double_arr, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":1204
 * 
 *             double_arr = np.ndarray(shape=(ret_length,), dtype=np.float64)
 *             for j in range(ret_length):             # <<<<<<<<<<<<<<
 *                 double_arr[j] = doubles_c[j]
 *             data[i] = double_arr
*/
      __pyx_t_4 = __pyx_v_ret_length;
      __pyx_t_20 = __pyx_t_4;
      for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
        __pyx_v_j = __pyx_t_21;

        /* "pyogrio/_io.pyx":1205
 *             double_arr = np.ndarray(shape=(ret_length,), dtype=np.float64)
 *             for j in range(ret_length):
 *                 double_arr[j] = doubles_c[j]             # <<<<<<<<<<<<<<
 *             data[i] = double_arr
 * 
*/
        __pyx_t_10 = PyFloat_FromDouble((__pyx_v_doubles_c[__pyx_v_j])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1205, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (unlikely((__Pyx_SetItemInt(__pyx_v_double_arr, __pyx_v_j, __pyx_t_10, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1205, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }

      /* "pyogrio/_io.pyx":1206
 *             for j in range(ret_length):
 *                 double_arr[j] = doubles_c[j]
 *             data[i] = double_arr             # <<<<<<<<<<<<<<
 * 
 *         elif field_type == OFTStringList:
*/
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_v_double_arr, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1206, __pyx_L1_error)

      /* "pyogrio/_io.pyx":1196
 *             data[i] = int_arr
 * 
 *         elif field_type == OFTRealList:             # <<<<<<<<<<<<<<
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
*/
      break;
      case OFTStringList:

      /* "pyogrio/_io.pyx":1211
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
 *             strings_c = OGR_F_GetFieldAsStringList(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 * 
 *             string_list_index = 0
*/
      __pyx_v_strings_c = OGR_F_GetFieldAsStringList(__pyx_v_ogr_feature, __pyx_v_field_index);

      /* "pyogrio/_io.pyx":1213
 *             strings_c = OGR_F_GetFieldAsStringList(ogr_feature, field_index)
 * 
 *             string_list_index = 0             # <<<<<<<<<<<<<<
 *             vals = []
 *             if strings_c != NULL:
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_XDECREF_SET(__pyx_v_string_list_index, __pyx_mstate_global->__pyx_int_0);

      /* "pyogrio/_io.pyx":1214
 * 
 *             string_list_index = 0
 *             vals = []             # <<<<<<<<<<<<<<
 *             if strings_c != NULL:
 *                 # According to GDAL doc, the list is terminated by a NULL pointer.
*/
      __pyx_t_10 = PyList_New(0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1214, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_XDECREF_SET(__pyx_v_vals, ((PyObject*)__pyx_t_10));
      __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":1215
 *             string_list_index = 0
 *             vals = []
 *             if strings_c != NULL:             # <<<<<<<<<<<<<<
 *                 # According to GDAL doc, the list is terminated by a NULL pointer.
 *                 while strings_c[string_list_index] != NULL:
*/
      __pyx_t_6 = (__pyx_v_strings_c != NULL);
      if (__pyx_t_6) {

        /* "pyogrio/_io.pyx":1217
 *             if strings_c != NULL:
 *                 # According to GDAL doc, the list is terminated by a NULL pointer.
 *                 while strings_c[string_list_index] != NULL:             # <<<<<<<<<<<<<<
 *                     val = strings_c[string_list_index]
 *                     vals.append(get_string(val, encoding=encoding))
*/
        while (1) {
          __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_string_list_index); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1217, __pyx_L1_error)
          __pyx_t_6 = ((__pyx_v_strings_c[__pyx_t_22]) != NULL);
          if (!__pyx_t_6) break;

          /* "pyogrio/_io.pyx":1218
 *                 # According to GDAL doc, the list is terminated by a NULL pointer.
 *                 while strings_c[string_list_index] != NULL:
 *                     val = strings_c[string_list_index]             # <<<<<<<<<<<<<<
 *                     vals.append(get_string(val, encoding=encoding))
 *                     string_list_index += 1
*/
          __pyx_t_22 = __Pyx_PyIndex_AsSsize_t(__pyx_v_string_list_index); if (unlikely((__pyx_t_22 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1218, __pyx_L1_error)
          __pyx_t_10 = __Pyx_PyBytes_FromString((__pyx_v_strings_c[__pyx_t_22])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1218, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_XDECREF_SET(__pyx_v_val, __pyx_t_10);
          __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":1219
 *                 while strings_c[string_list_index] != NULL:
 *                     val = strings_c[string_list_index]
 *                     vals.append(get_string(val, encoding=encoding))             # <<<<<<<<<<<<<<
 *                     string_list_index += 1
 * 
*/
          __pyx_t_23 = __Pyx_PyObject_AsString(__pyx_v_val); if (unlikely((!__pyx_t_23) && PyErr_Occurred())) __PYX_ERR(0, 1219, __pyx_L1_error)
          __pyx_t_10 = __pyx_v_encoding;
          __Pyx_INCREF(__pyx_t_10);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_10))||((__pyx_t_10) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_10))) __PYX_ERR(0, 1219, __pyx_L1_error)
          __pyx_t_13.__pyx_n = 1;
          __pyx_t_13.encoding = ((PyObject*)__pyx_t_10);
          __pyx_t_16 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_t_23, &__pyx_t_13); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1219, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_24 = __Pyx_PyList_Append(__pyx_v_vals, __pyx_t_16); if (unlikely(__pyx_t_24 == ((int)-1))) __PYX_ERR(0, 1219, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

          /* "pyogrio/_io.pyx":1220
 *                     val = strings_c[string_list_index]
 *                     vals.append(get_string(val, encoding=encoding))
 *                     string_list_index += 1             # <<<<<<<<<<<<<<
 * 
 *             data[i] = np.array(vals)
*/
          __pyx_t_16 = __Pyx_PyLong_AddObjC(__pyx_v_string_list_index, __pyx_mstate_global->__pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1220, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_DECREF_SET(__pyx_v_string_list_index, __pyx_t_16);
          __pyx_t_16 = 0;
        }

        /* "pyogrio/_io.pyx":1215
 *             string_list_index = 0
 *             vals = []
 *             if strings_c != NULL:             # <<<<<<<<<<<<<<
 *                 # According to GDAL doc, the list is terminated by a NULL pointer.
 *                 while strings_c[string_list_index] != NULL:
*/
      }

      /* "pyogrio/_io.pyx":1222
 *                     string_list_index += 1
 * 
 *             data[i] = np.array(vals)             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __pyx_t_10 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 1222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __pyx_t_11 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_10);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_11 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_vals};
        __pyx_t_16 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_11, (2-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1222, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
      }
      if (unlikely((__Pyx_SetItemInt(__pyx_v_data, __pyx_v_i, __pyx_t_16, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1222, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

      /* "pyogrio/_io.pyx":1208
 *             data[i] = double_arr
 * 
 *         elif field_type == OFTStringList:             # <<<<<<<<<<<<<<
 *             # According to GDAL doc, this can return NULL for an empty list, which is a
 *             # valid result. So don't use check_pointer as it would throw an exception.
*/
      break;
      default: break;
    }
    __pyx_L3_continue:;
  }

  /* "pyogrio/_io.pyx":1028
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef process_fields(
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_AddTraceback("pyogrio._io.process_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_isnull);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_second);
  __Pyx_XDECREF(__pyx_v_microsecond);
  __Pyx_XDECREF(__pyx_v_int_arr);
  __Pyx_XDECREF(__pyx_v_double_arr);
  __Pyx_XDECREF(__pyx_v_string_list_index);
  __Pyx_XDECREF(__pyx_v_vals);
  __Pyx_XDECREF(__pyx_v_val);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1293
 *     field_data_chunks: list[list[np.ndarray]] = []
 * 
 *     def add_data_chunk(size):             # <<<<<<<<<<<<<<
 *         """Add a new data chunk to the output array lists."""
 *         if return_fids:
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_12get_features_1add_data_chunk(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_12get_features_add_data_chunk, "Add a new data chunk to the output array lists.");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_12get_features_1add_data_chunk = {"add_data_chunk", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_12get_features_1add_data_chunk, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_12get_features_add_data_chunk};
static PyObject *__pyx_pw_7pyogrio_3_io_12get_features_1add_data_chunk(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_size = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_data_chunk (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1293, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1293, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "add_data_chunk", 0) < (0)) __PYX_ERR(0, 1293, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("add_data_chunk", 1, 1, 1, i); __PYX_ERR(0, 1293, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1293, __pyx_L3_error)
    }
    __pyx_v_size = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_data_chunk", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1293, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.get_features.add_data_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyogrio_3_io_12get_features_add_data_chunk(__pyx_self, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_12get_features_add_data_chunk(PyObject *__pyx_self, PyObject *__pyx_v_size) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *__pyx_cur_scope;
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *__pyx_outer_scope;
  PyObject *__pyx_v_field_data_chunk = 0;
  int __pyx_v_field_index;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_data_chunk", 0);
  __pyx_outer_scope = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "pyogrio/_io.pyx":1295
 *     def add_data_chunk(size):
 *         """Add a new data chunk to the output array lists."""
 *         if return_fids:             # <<<<<<<<<<<<<<
 *             fid_chunks.append(np.empty(shape=(size, ), dtype=np.int64))
 *         if read_geometry:
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_return_fids != 0);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1296
 *         """Add a new data chunk to the output array lists."""
 *         if return_fids:
 *             fid_chunks.append(np.empty(shape=(size, ), dtype=np.int64))             # <<<<<<<<<<<<<<
 *         if read_geometry:
 *             geom_chunks.append(np.empty(shape=(size, ), dtype="object"))
*/
    if (unlikely(!__pyx_cur_scope->__pyx_v_fid_chunks)) { __Pyx_RaiseClosureNameError("fid_chunks"); __PYX_ERR(0, 1296, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_fid_chunks == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 1296, __pyx_L1_error)
    }
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_size) != (0)) __PYX_ERR(0, 1296, __pyx_L1_error);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_3, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1296, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_4, __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1296, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_6, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1296, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1296, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_fid_chunks, __pyx_t_2); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1296, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1295
 *     def add_data_chunk(size):
 *         """Add a new data chunk to the output array lists."""
 *         if return_fids:             # <<<<<<<<<<<<<<
 *             fid_chunks.append(np.empty(shape=(size, ), dtype=np.int64))
 *         if read_geometry:
*/
  }

  /* "pyogrio/_io.pyx":1297
 *         if return_fids:
 *             fid_chunks.append(np.empty(shape=(size, ), dtype=np.int64))
 *         if read_geometry:             # <<<<<<<<<<<<<<
 *             geom_chunks.append(np.empty(shape=(size, ), dtype="object"))
 * 
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_read_geometry != 0);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1298
 *             fid_chunks.append(np.empty(shape=(size, ), dtype=np.int64))
 *         if read_geometry:
 *             geom_chunks.append(np.empty(shape=(size, ), dtype="object"))             # <<<<<<<<<<<<<<
 * 
 *         field_data_chunk: list[np.ndarray] = []
*/
    if (unlikely(!__pyx_cur_scope->__pyx_v_geom_chunks)) { __Pyx_RaiseClosureNameError("geom_chunks"); __PYX_ERR(0, 1298, __pyx_L1_error) }
    if (unlikely(__pyx_cur_scope->__pyx_v_geom_chunks == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 1298, __pyx_L1_error)
    }
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1298, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1298, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1298, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_size) != (0)) __PYX_ERR(0, 1298, __pyx_L1_error);
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_5, NULL};
      __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_6, __pyx_t_4, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1298, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_object, __pyx_t_4, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1298, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1298, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_geom_chunks, __pyx_t_2); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1298, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1297
 *         if return_fids:
 *             fid_chunks.append(np.empty(shape=(size, ), dtype=np.int64))
 *         if read_geometry:             # <<<<<<<<<<<<<<
 *             geom_chunks.append(np.empty(shape=(size, ), dtype="object"))
 * 
*/
  }

  /* "pyogrio/_io.pyx":1300
 *             geom_chunks.append(np.empty(shape=(size, ), dtype="object"))
 * 
 *         field_data_chunk: list[np.ndarray] = []             # <<<<<<<<<<<<<<
 *         for field_index in range(n_fields):
 *             if datetime_as_string and fields[field_index, 3].startswith("datetime"):
*/
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_field_data_chunk = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1301
 * 
 *         field_data_chunk: list[np.ndarray] = []
 *         for field_index in range(n_fields):             # <<<<<<<<<<<<<<
 *             if datetime_as_string and fields[field_index, 3].startswith("datetime"):
 *                 dtype = "object"
*/
  __pyx_t_10 = __pyx_cur_scope->__pyx_v_n_fields;
  __pyx_t_11 = __pyx_t_10;
  for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
    __pyx_v_field_index = __pyx_t_12;

    /* "pyogrio/_io.pyx":1302
 *         field_data_chunk: list[np.ndarray] = []
 *         for field_index in range(n_fields):
 *             if datetime_as_string and fields[field_index, 3].startswith("datetime"):             # <<<<<<<<<<<<<<
 *                 dtype = "object"
 *             elif fields[field_index, 3].startswith("list"):
*/
    if (__pyx_cur_scope->__pyx_v_datetime_as_string) {
    } else {
      __pyx_t_1 = __pyx_cur_scope->__pyx_v_datetime_as_string;
      goto __pyx_L8_bool_binop_done;
    }
    if (unlikely(!__pyx_cur_scope->__pyx_v_fields.memview)) { __Pyx_RaiseClosureNameError("fields"); __PYX_ERR(0, 1302, __pyx_L1_error) }
    __pyx_t_13 = __pyx_v_field_index;
    __pyx_t_14 = 3;
    __pyx_t_4 = (PyObject *) *((PyObject * *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_cur_scope->__pyx_v_fields.data + __pyx_t_13 * __pyx_cur_scope->__pyx_v_fields.strides[0]) ) + __pyx_t_14 * __pyx_cur_scope->__pyx_v_fields.strides[1]) ));
    if (unlikely(__pyx_t_4 == NULL)) __pyx_t_4 = Py_None;
    __Pyx_INCREF((PyObject*)__pyx_t_4);
    __pyx_t_7 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_n_u_datetime};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_startswith, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_15 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_15 < 0))) __PYX_ERR(0, 1302, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __pyx_t_15;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":1303
 *         for field_index in range(n_fields):
 *             if datetime_as_string and fields[field_index, 3].startswith("datetime"):
 *                 dtype = "object"             # <<<<<<<<<<<<<<
 *             elif fields[field_index, 3].startswith("list"):
 *                 dtype = "object"
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_object);
      __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_object);

      /* "pyogrio/_io.pyx":1302
 *         field_data_chunk: list[np.ndarray] = []
 *         for field_index in range(n_fields):
 *             if datetime_as_string and fields[field_index, 3].startswith("datetime"):             # <<<<<<<<<<<<<<
 *                 dtype = "object"
 *             elif fields[field_index, 3].startswith("list"):
*/
      goto __pyx_L7;
    }

    /* "pyogrio/_io.pyx":1304
 *             if datetime_as_string and fields[field_index, 3].startswith("datetime"):
 *                 dtype = "object"
 *             elif fields[field_index, 3].startswith("list"):             # <<<<<<<<<<<<<<
 *                 dtype = "object"
 *             else:
*/
    if (unlikely(!__pyx_cur_scope->__pyx_v_fields.memview)) { __Pyx_RaiseClosureNameError("fields"); __PYX_ERR(0, 1304, __pyx_L1_error) }
    __pyx_t_14 = __pyx_v_field_index;
    __pyx_t_13 = 3;
    __pyx_t_7 = (PyObject *) *((PyObject * *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_cur_scope->__pyx_v_fields.data + __pyx_t_14 * __pyx_cur_scope->__pyx_v_fields.strides[0]) ) + __pyx_t_13 * __pyx_cur_scope->__pyx_v_fields.strides[1]) ));
    if (unlikely(__pyx_t_7 == NULL)) __pyx_t_7 = Py_None;
    __Pyx_INCREF((PyObject*)__pyx_t_7);
    __pyx_t_4 = __pyx_t_7;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_n_u_list};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_startswith, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1304, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":1305
 *                 dtype = "object"
 *             elif fields[field_index, 3].startswith("list"):
 *                 dtype = "object"             # <<<<<<<<<<<<<<
 *             else:
 *                 dtype = fields[field_index, 3]
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_object);
      __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_object);

      /* "pyogrio/_io.pyx":1304
 *             if datetime_as_string and fields[field_index, 3].startswith("datetime"):
 *                 dtype = "object"
 *             elif fields[field_index, 3].startswith("list"):             # <<<<<<<<<<<<<<
 *                 dtype = "object"
 *             else:
*/
      goto __pyx_L7;
    }

    /* "pyogrio/_io.pyx":1307
 *                 dtype = "object"
 *             else:
 *                 dtype = fields[field_index, 3]             # <<<<<<<<<<<<<<
 * 
 *             field_data_chunk.append(np.empty(shape=(size, ), dtype=dtype))
*/
    /*else*/ {
      if (unlikely(!__pyx_cur_scope->__pyx_v_fields.memview)) { __Pyx_RaiseClosureNameError("fields"); __PYX_ERR(0, 1307, __pyx_L1_error) }
      __pyx_t_13 = __pyx_v_field_index;
      __pyx_t_14 = 3;
      __pyx_t_2 = (PyObject *) *((PyObject * *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_cur_scope->__pyx_v_fields.data + __pyx_t_13 * __pyx_cur_scope->__pyx_v_fields.strides[0]) ) + __pyx_t_14 * __pyx_cur_scope->__pyx_v_fields.strides[1]) ));
      if (unlikely(__pyx_t_2 == NULL)) __pyx_t_2 = Py_None;
      __Pyx_INCREF((PyObject*)__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L7:;

    /* "pyogrio/_io.pyx":1309
 *                 dtype = fields[field_index, 3]
 * 
 *             field_data_chunk.append(np.empty(shape=(size, ), dtype=dtype))             # <<<<<<<<<<<<<<
 * 
 *         field_data_chunks.append(field_data_chunk)
*/
    __pyx_t_7 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_size) != (0)) __PYX_ERR(0, 1309, __pyx_L1_error);
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_7);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_7, NULL};
      __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_4, __pyx_t_5, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1309, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dtype, __pyx_t_5, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1309, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_field_data_chunk, __pyx_t_2); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1309, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "pyogrio/_io.pyx":1311
 *             field_data_chunk.append(np.empty(shape=(size, ), dtype=dtype))
 * 
 *         field_data_chunks.append(field_data_chunk)             # <<<<<<<<<<<<<<
 * 
 *     i = 0
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_field_data_chunks)) { __Pyx_RaiseClosureNameError("field_data_chunks"); __PYX_ERR(0, 1311, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_v_field_data_chunks == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 1311, __pyx_L1_error)
  }
  __pyx_t_9 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_field_data_chunks, __pyx_v_field_data_chunk); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1311, __pyx_L1_error)

  /* "pyogrio/_io.pyx":1293
 *     field_data_chunks: list[list[np.ndarray]] = []
 * 
 *     def add_data_chunk(size):             # <<<<<<<<<<<<<<
 *         """Add a new data chunk to the output array lists."""
 *         if return_fids:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyogrio._io.get_features.add_data_chunk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_field_data_chunk);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1225
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef get_features(
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_features(OGRLayerH __pyx_v_ogr_layer, __Pyx_memviewslice __pyx_v_fields, PyObject *__pyx_v_encoding, uint8_t __pyx_v_read_geometry, uint8_t __pyx_v_force_2d, int __pyx_v_skip_features, int __pyx_v_num_features, uint8_t __pyx_v_return_fids, int __pyx_v_datetime_as_string) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *__pyx_cur_scope;
  OGRFeatureH __pyx_v_ogr_feature;
  int __pyx_v_i;
  int __pyx_v_field_index;
  PyObject *__pyx_v_last_chunk_size = NULL;
  long __pyx_v_max_chunk_size;
  PyObject *__pyx_v_add_data_chunk = 0;
  PyObject *__pyx_v_i_total = NULL;
  PyObject *__pyx_v_last_chunk_index = NULL;
  PyObject *__pyx_v_field_indexes = NULL;
  PyObject *__pyx_v_field_ogr_types = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_fid_view = NULL;
  PyObject *__pyx_v_geom_view = NULL;
  PyObject *__pyx_v_field_data_view = NULL;
  PyObject *__pyx_v_fid_data = NULL;
  PyObject *__pyx_v_geom_data = NULL;
  PyObject *__pyx_v_field_data = NULL;
  PyObject *__pyx_v_data_to_concat = NULL;
  PyObject *__pyx_v_chunk = NULL;
  int __pyx_8genexpr2__pyx_v_field_index;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_memviewslice __pyx_t_3 = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  void *__pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  size_t __pyx_t_16;
  int __pyx_t_17;
  char const *__pyx_t_18;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  long __pyx_t_25;
  int __pyx_t_26;
  char const *__pyx_t_27;
  PyObject *(*__pyx_t_28)(PyObject *);
  int __pyx_t_29;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_features", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features(__pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1225, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_fields = __pyx_v_fields;
  __PYX_INC_MEMVIEW(&__pyx_cur_scope->__pyx_v_fields, 1);
  __pyx_cur_scope->__pyx_v_read_geometry = __pyx_v_read_geometry;
  __pyx_cur_scope->__pyx_v_return_fids = __pyx_v_return_fids;
  __pyx_cur_scope->__pyx_v_datetime_as_string = __pyx_v_datetime_as_string;

  /* "pyogrio/_io.pyx":1268
 * 
 *     """
 *     cdef OGRFeatureH ogr_feature = NULL             # <<<<<<<<<<<<<<
 *     cdef int n_fields
 *     cdef int i
*/
  __pyx_v_ogr_feature = NULL;

  /* "pyogrio/_io.pyx":1274
 * 
 *     # Make sure the layer is read from the beginning
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_ResetReading(ogr_layer)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":1275
 *     # Make sure the layer is read from the beginning
 *     with nogil:
 *         OGR_L_ResetReading(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *     if skip_features > 0:
*/
        OGR_L_ResetReading(__pyx_v_ogr_layer);
      }

      /* "pyogrio/_io.pyx":1274
 * 
 *     # Make sure the layer is read from the beginning
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_ResetReading(ogr_layer)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":1277
 *         OGR_L_ResetReading(ogr_layer)
 * 
 *     if skip_features > 0:             # <<<<<<<<<<<<<<
 *         apply_skip_features(ogr_layer, skip_features)
 * 
*/
  __pyx_t_1 = (__pyx_v_skip_features > 0);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1278
 * 
 *     if skip_features > 0:
 *         apply_skip_features(ogr_layer, skip_features)             # <<<<<<<<<<<<<<
 * 
 *     # We don't (always) know beforehand how many features there will be read, so
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_apply_skip_features(__pyx_v_ogr_layer, __pyx_v_skip_features); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1277
 *         OGR_L_ResetReading(ogr_layer)
 * 
 *     if skip_features > 0:             # <<<<<<<<<<<<<<
 *         apply_skip_features(ogr_layer, skip_features)
 * 
*/
  }

  /* "pyogrio/_io.pyx":1282
 *     # We don't (always) know beforehand how many features there will be read, so
 *     # allocate output arrays as needed (in chunks), and return the concatenated results.
 *     last_chunk_size = 500    # the first chunk size will be 2 * last_chunk_size             # <<<<<<<<<<<<<<
 *     max_chunk_size = 65_536  # this is the default chunk size for arrow arrays
 * 
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_500);
  __pyx_v_last_chunk_size = __pyx_mstate_global->__pyx_int_500;

  /* "pyogrio/_io.pyx":1283
 *     # allocate output arrays as needed (in chunks), and return the concatenated results.
 *     last_chunk_size = 500    # the first chunk size will be 2 * last_chunk_size
 *     max_chunk_size = 65_536  # this is the default chunk size for arrow arrays             # <<<<<<<<<<<<<<
 * 
 *     if num_features >= 0:
*/
  __pyx_v_max_chunk_size = 0x10000;

  /* "pyogrio/_io.pyx":1285
 *     max_chunk_size = 65_536  # this is the default chunk size for arrow arrays
 * 
 *     if num_features >= 0:             # <<<<<<<<<<<<<<
 *         last_chunk_size = num_features
 * 
*/
  __pyx_t_1 = (__pyx_v_num_features >= 0);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1286
 * 
 *     if num_features >= 0:
 *         last_chunk_size = num_features             # <<<<<<<<<<<<<<
 * 
 *     fid_chunks = []
*/
    __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_num_features); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1286, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_last_chunk_size, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1285
 *     max_chunk_size = 65_536  # this is the default chunk size for arrow arrays
 * 
 *     if num_features >= 0:             # <<<<<<<<<<<<<<
 *         last_chunk_size = num_features
 * 
*/
  }

  /* "pyogrio/_io.pyx":1288
 *         last_chunk_size = num_features
 * 
 *     fid_chunks = []             # <<<<<<<<<<<<<<
 *     geom_chunks = []
 *     n_fields = fields.shape[0]
*/
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_fid_chunks = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1289
 * 
 *     fid_chunks = []
 *     geom_chunks = []             # <<<<<<<<<<<<<<
 *     n_fields = fields.shape[0]
 *     field_data_chunks: list[list[np.ndarray]] = []
*/
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_geom_chunks = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1290
 *     fid_chunks = []
 *     geom_chunks = []
 *     n_fields = fields.shape[0]             # <<<<<<<<<<<<<<
 *     field_data_chunks: list[list[np.ndarray]] = []
 * 
*/
  __pyx_cur_scope->__pyx_v_n_fields = (__pyx_cur_scope->__pyx_v_fields.shape[0]);

  /* "pyogrio/_io.pyx":1291
 *     geom_chunks = []
 *     n_fields = fields.shape[0]
 *     field_data_chunks: list[list[np.ndarray]] = []             # <<<<<<<<<<<<<<
 * 
 *     def add_data_chunk(size):
*/
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_field_data_chunks = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1293
 *     field_data_chunks: list[list[np.ndarray]] = []
 * 
 *     def add_data_chunk(size):             # <<<<<<<<<<<<<<
 *         """Add a new data chunk to the output array lists."""
 *         if return_fids:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_12get_features_1add_data_chunk, 0, __pyx_mstate_global->__pyx_n_u_get_features_locals_add_data_chu, ((PyObject*)__pyx_cur_scope), __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_add_data_chunk = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1313
 *         field_data_chunks.append(field_data_chunk)
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     i_total = 0
 *     last_chunk_index = -1
*/
  __pyx_v_i = 0;

  /* "pyogrio/_io.pyx":1314
 * 
 *     i = 0
 *     i_total = 0             # <<<<<<<<<<<<<<
 *     last_chunk_index = -1
 *     field_indexes = list(fields[:, 0])
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
  __pyx_v_i_total = __pyx_mstate_global->__pyx_int_0;

  /* "pyogrio/_io.pyx":1315
 *     i = 0
 *     i_total = 0
 *     last_chunk_index = -1             # <<<<<<<<<<<<<<
 *     field_indexes = list(fields[:, 0])
 *     field_ogr_types = list(fields[:, 1])
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_neg_1);
  __pyx_v_last_chunk_index = __pyx_mstate_global->__pyx_int_neg_1;

  /* "pyogrio/_io.pyx":1316
 *     i_total = 0
 *     last_chunk_index = -1
 *     field_indexes = list(fields[:, 0])             # <<<<<<<<<<<<<<
 *     field_ogr_types = list(fields[:, 1])
 * 
*/
  __pyx_t_3.data = __pyx_cur_scope->__pyx_v_fields.data;
  __pyx_t_3.memview = __pyx_cur_scope->__pyx_v_fields.memview;
  __PYX_INC_MEMVIEW(&__pyx_t_3, 1);
  __pyx_t_3.shape[0] = __pyx_cur_scope->__pyx_v_fields.shape[0];
__pyx_t_3.strides[0] = __pyx_cur_scope->__pyx_v_fields.strides[0];
    __pyx_t_3.suboffsets[0] = -1;

{
    Py_ssize_t __pyx_tmp_idx = 0;
    Py_ssize_t __pyx_tmp_stride = __pyx_cur_scope->__pyx_v_fields.strides[1];
        __pyx_t_3.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_2 = __pyx_memoryview_fromslice(__pyx_t_3, 1, (PyObject *(*)(char *)) __pyx_memview_get_object, (int (*)(char *, PyObject *)) __pyx_memview_set_object, 1);; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_3, 1);
  __pyx_t_3.memview = NULL; __pyx_t_3.data = NULL;
  __pyx_t_4 = __Pyx_PySequence_ListKeepNew(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_field_indexes = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":1317
 *     last_chunk_index = -1
 *     field_indexes = list(fields[:, 0])
 *     field_ogr_types = list(fields[:, 1])             # <<<<<<<<<<<<<<
 * 
 *     while True:
*/
  __pyx_t_3.data = __pyx_cur_scope->__pyx_v_fields.data;
  __pyx_t_3.memview = __pyx_cur_scope->__pyx_v_fields.memview;
  __PYX_INC_MEMVIEW(&__pyx_t_3, 1);
  __pyx_t_3.shape[0] = __pyx_cur_scope->__pyx_v_fields.shape[0];
__pyx_t_3.strides[0] = __pyx_cur_scope->__pyx_v_fields.strides[0];
    __pyx_t_3.suboffsets[0] = -1;

{
    Py_ssize_t __pyx_tmp_idx = 1;
    Py_ssize_t __pyx_tmp_stride = __pyx_cur_scope->__pyx_v_fields.strides[1];
        __pyx_t_3.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_4 = __pyx_memoryview_fromslice(__pyx_t_3, 1, (PyObject *(*)(char *)) __pyx_memview_get_object, (int (*)(char *, PyObject *)) __pyx_memview_set_object, 1);; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_3, 1);
  __pyx_t_3.memview = NULL; __pyx_t_3.data = NULL;
  __pyx_t_2 = __Pyx_PySequence_ListKeepNew(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_field_ogr_types = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1319
 *     field_ogr_types = list(fields[:, 1])
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         try:
 *             if num_features >= 0 and i_total == num_features:
*/
  while (1) {

    /* "pyogrio/_io.pyx":1320
 * 
 *     while True:
 *         try:             # <<<<<<<<<<<<<<
 *             if num_features >= 0 and i_total == num_features:
 *                 break
*/
    /*try:*/ {

      /* "pyogrio/_io.pyx":1321
 *     while True:
 *         try:
 *             if num_features >= 0 and i_total == num_features:             # <<<<<<<<<<<<<<
 *                 break
 * 
*/
      __pyx_t_5 = (__pyx_v_num_features >= 0);
      if (__pyx_t_5) {
      } else {
        __pyx_t_1 = __pyx_t_5;
        goto __pyx_L16_bool_binop_done;
      }
      __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_num_features); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1321, __pyx_L13_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_i_total, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1321, __pyx_L13_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1321, __pyx_L13_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_1 = __pyx_t_5;
      __pyx_L16_bool_binop_done:;
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1322
 *         try:
 *             if num_features >= 0 and i_total == num_features:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             try:
*/
        goto __pyx_L11_break;

        /* "pyogrio/_io.pyx":1321
 *     while True:
 *         try:
 *             if num_features >= 0 and i_total == num_features:             # <<<<<<<<<<<<<<
 *                 break
 * 
*/
      }

      /* "pyogrio/_io.pyx":1324
 *                 break
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 # Don't use `with nogil` inside this loop, as if there are many
 *                 # features, the overhead of acquiring and releasing the GIL becomes
*/
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "pyogrio/_io.pyx":1328
 *                 # features, the overhead of acquiring and releasing the GIL becomes
 *                 # significant on linux.
 *                 ogr_feature = OGR_L_GetNextFeature(ogr_layer)             # <<<<<<<<<<<<<<
 *                 ogr_feature = check_pointer(ogr_feature)
 * 
*/
          __pyx_v_ogr_feature = OGR_L_GetNextFeature(__pyx_v_ogr_layer);

          /* "pyogrio/_io.pyx":1329
 *                 # significant on linux.
 *                 ogr_feature = OGR_L_GetNextFeature(ogr_layer)
 *                 ogr_feature = check_pointer(ogr_feature)             # <<<<<<<<<<<<<<
 * 
 *             except NullPointerError:
*/
          __pyx_t_9 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_feature); if (unlikely(__pyx_t_9 == ((void *)NULL))) __PYX_ERR(0, 1329, __pyx_L18_error)
          __pyx_v_ogr_feature = __pyx_t_9;

          /* "pyogrio/_io.pyx":1324
 *                 break
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 # Don't use `with nogil` inside this loop, as if there are many
 *                 # features, the overhead of acquiring and releasing the GIL becomes
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L25_try_end;
        __pyx_L18_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_XCLEAR_MEMVIEW(&__pyx_t_3, 1);
        __pyx_t_3.memview = NULL; __pyx_t_3.data = NULL;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pyogrio/_io.pyx":1331
 *                 ogr_feature = check_pointer(ogr_feature)
 * 
 *             except NullPointerError:             # <<<<<<<<<<<<<<
 *                 # No more rows available, so stop reading
 *                 break
*/
        __Pyx_ErrFetch(&__pyx_t_4, &__pyx_t_2, &__pyx_t_10);
        __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1331, __pyx_L20_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_4, __pyx_t_11);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_ErrRestore(__pyx_t_4, __pyx_t_2, __pyx_t_10);
        __pyx_t_4 = 0; __pyx_t_2 = 0; __pyx_t_10 = 0;
        if (__pyx_t_12) {
          __Pyx_AddTraceback("pyogrio._io.get_features", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_2, &__pyx_t_4) < 0) __PYX_ERR(0, 1331, __pyx_L20_except_error)
          __Pyx_XGOTREF(__pyx_t_10);
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);

          /* "pyogrio/_io.pyx":1333
 *             except NullPointerError:
 *                 # No more rows available, so stop reading
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             except CPLE_BaseError as exc:
*/
          goto __pyx_L27_except_break;
          __pyx_L27_except_break:;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L23_try_break;
        }

        /* "pyogrio/_io.pyx":1335
 *                 break
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 if "failed to prepare SQL" in str(exc):
 *                     raise ValueError(f"Invalid SQL query: {str(exc)}") from None
*/
        __Pyx_ErrFetch(&__pyx_t_4, &__pyx_t_2, &__pyx_t_10);
        __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1335, __pyx_L20_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_4, __pyx_t_11);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_ErrRestore(__pyx_t_4, __pyx_t_2, __pyx_t_10);
        __pyx_t_4 = 0; __pyx_t_2 = 0; __pyx_t_10 = 0;
        if (__pyx_t_12) {
          __Pyx_AddTraceback("pyogrio._io.get_features", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_2, &__pyx_t_4) < 0) __PYX_ERR(0, 1335, __pyx_L20_except_error)
          __Pyx_XGOTREF(__pyx_t_10);
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __pyx_v_exc = __pyx_t_2;
          /*try:*/ {

            /* "pyogrio/_io.pyx":1336
 * 
 *             except CPLE_BaseError as exc:
 *                 if "failed to prepare SQL" in str(exc):             # <<<<<<<<<<<<<<
 *                     raise ValueError(f"Invalid SQL query: {str(exc)}") from None
 * 
*/
            __pyx_t_11 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1336, __pyx_L33_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_1 = (__Pyx_PyUnicode_ContainsTF(__pyx_mstate_global->__pyx_kp_u_failed_to_prepare_SQL, __pyx_t_11, Py_EQ)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1336, __pyx_L33_error)
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(__pyx_t_1)) {

              /* "pyogrio/_io.pyx":1337
 *             except CPLE_BaseError as exc:
 *                 if "failed to prepare SQL" in str(exc):
 *                     raise ValueError(f"Invalid SQL query: {str(exc)}") from None             # <<<<<<<<<<<<<<
 * 
 *                 raise FeatureError(str(exc))
*/
              __pyx_t_13 = NULL;
              __pyx_t_14 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1337, __pyx_L33_error)
              __Pyx_GOTREF(__pyx_t_14);
              __pyx_t_15 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Invalid_SQL_query, __pyx_t_14); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1337, __pyx_L33_error)
              __Pyx_GOTREF(__pyx_t_15);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_16 = 1;
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_15};
                __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1337, __pyx_L33_error)
                __Pyx_GOTREF(__pyx_t_11);
              }
              __Pyx_Raise(__pyx_t_11, 0, 0, Py_None);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
              __PYX_ERR(0, 1337, __pyx_L33_error)

              /* "pyogrio/_io.pyx":1336
 * 
 *             except CPLE_BaseError as exc:
 *                 if "failed to prepare SQL" in str(exc):             # <<<<<<<<<<<<<<
 *                     raise ValueError(f"Invalid SQL query: {str(exc)}") from None
 * 
*/
            }

            /* "pyogrio/_io.pyx":1339
 *                     raise ValueError(f"Invalid SQL query: {str(exc)}") from None
 * 
 *                 raise FeatureError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *             if num_features > 0 and i_total >= num_features:
*/
            __pyx_t_15 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1339, __pyx_L33_error)
            __Pyx_GOTREF(__pyx_t_13);
            __pyx_t_14 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1339, __pyx_L33_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_16 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_13))) {
              __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_13);
              assert(__pyx_t_15);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
              __Pyx_INCREF(__pyx_t_15);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
              __pyx_t_16 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_t_14};
              __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1339, __pyx_L33_error)
              __Pyx_GOTREF(__pyx_t_11);
            }
            __Pyx_Raise(__pyx_t_11, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __PYX_ERR(0, 1339, __pyx_L33_error)
          }

          /* "pyogrio/_io.pyx":1335
 *                 break
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 if "failed to prepare SQL" in str(exc):
 *                     raise ValueError(f"Invalid SQL query: {str(exc)}") from None
*/
          /*finally:*/ {
            __pyx_L33_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __PYX_XCLEAR_MEMVIEW(&__pyx_t_3, 1);
              __pyx_t_3.memview = NULL; __pyx_t_3.data = NULL;
               __Pyx_ExceptionSwap(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
              if ( unlikely(__Pyx_GetException(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21) < 0)) __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
              __Pyx_XGOTREF(__pyx_t_19);
              __Pyx_XGOTREF(__pyx_t_20);
              __Pyx_XGOTREF(__pyx_t_21);
              __Pyx_XGOTREF(__pyx_t_22);
              __Pyx_XGOTREF(__pyx_t_23);
              __Pyx_XGOTREF(__pyx_t_24);
              __pyx_t_12 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_18 = __pyx_filename;
              {
                __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_22);
              __Pyx_XGIVEREF(__pyx_t_23);
              __Pyx_XGIVEREF(__pyx_t_24);
              __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_23, __pyx_t_24);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_XGIVEREF(__pyx_t_20);
              __Pyx_XGIVEREF(__pyx_t_21);
              __Pyx_ErrRestore(__pyx_t_19, __pyx_t_20, __pyx_t_21);
              __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
              __pyx_lineno = __pyx_t_12; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_18;
              goto __pyx_L20_except_error;
            }
          }
        }
        goto __pyx_L20_except_error;

        /* "pyogrio/_io.pyx":1324
 *                 break
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 # Don't use `with nogil` inside this loop, as if there are many
 *                 # features, the overhead of acquiring and releasing the GIL becomes
*/
        __pyx_L20_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L13_error;
        __pyx_L23_try_break:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L11_break;
        __pyx_L25_try_end:;
      }

      /* "pyogrio/_io.pyx":1341
 *                 raise FeatureError(str(exc))
 * 
 *             if num_features > 0 and i_total >= num_features:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(
 *                     "GDAL returned more records than expected based on the count of "
*/
      __pyx_t_5 = (__pyx_v_num_features > 0);
      if (__pyx_t_5) {
      } else {
        __pyx_t_1 = __pyx_t_5;
        goto __pyx_L41_bool_binop_done;
      }
      __pyx_t_4 = __Pyx_PyLong_From_int(__pyx_v_num_features); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1341, __pyx_L13_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = PyObject_RichCompare(__pyx_v_i_total, __pyx_t_4, Py_GE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1341, __pyx_L13_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1341, __pyx_L13_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_1 = __pyx_t_5;
      __pyx_L41_bool_binop_done:;
      if (unlikely(__pyx_t_1)) {

        /* "pyogrio/_io.pyx":1342
 * 
 *             if num_features > 0 and i_total >= num_features:
 *                 raise FeatureError(             # <<<<<<<<<<<<<<
 *                     "GDAL returned more records than expected based on the count of "
 *                     "records that may meet your combination of filters against this "
*/
        __pyx_t_4 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1342, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_16 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
          assert(__pyx_t_4);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
          __pyx_t_16 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_GDAL_returned_more_records_than};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1342, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_2);
        }

        /* "pyogrio/_io.pyx":1348
 *                     "(https://github.com/geopandas/pyogrio/issues) to report "
 *                     "encountering this error."
 *                 ) from None             # <<<<<<<<<<<<<<
 * 
 *             if i_total == 0 or i == last_chunk_size:
*/
        __Pyx_Raise(__pyx_t_2, 0, 0, Py_None);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 1342, __pyx_L13_error)

        /* "pyogrio/_io.pyx":1341
 *                 raise FeatureError(str(exc))
 * 
 *             if num_features > 0 and i_total >= num_features:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(
 *                     "GDAL returned more records than expected based on the count of "
*/
      }

      /* "pyogrio/_io.pyx":1350
 *                 ) from None
 * 
 *             if i_total == 0 or i == last_chunk_size:             # <<<<<<<<<<<<<<
 *                 # No chunk yet or last chunk filled up: add a new chunk.
 *                 i = 0
*/
      __pyx_t_5 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_i_total, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1350, __pyx_L13_error)
      if (!__pyx_t_5) {
      } else {
        __pyx_t_1 = __pyx_t_5;
        goto __pyx_L44_bool_binop_done;
      }
      __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1350, __pyx_L13_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10 = PyObject_RichCompare(__pyx_t_2, __pyx_v_last_chunk_size, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1350, __pyx_L13_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1350, __pyx_L13_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_1 = __pyx_t_5;
      __pyx_L44_bool_binop_done:;
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1352
 *             if i_total == 0 or i == last_chunk_size:
 *                 # No chunk yet or last chunk filled up: add a new chunk.
 *                 i = 0             # <<<<<<<<<<<<<<
 *                 last_chunk_index += 1
 *                 if num_features < 0 and last_chunk_size < max_chunk_size:
*/
        __pyx_v_i = 0;

        /* "pyogrio/_io.pyx":1353
 *                 # No chunk yet or last chunk filled up: add a new chunk.
 *                 i = 0
 *                 last_chunk_index += 1             # <<<<<<<<<<<<<<
 *                 if num_features < 0 and last_chunk_size < max_chunk_size:
 *                     # Double chunk size for next chunk, up to max_chunk_size
*/
        __pyx_t_10 = __Pyx_PyLong_AddObjC(__pyx_v_last_chunk_index, __pyx_mstate_global->__pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1353, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF_SET(__pyx_v_last_chunk_index, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "pyogrio/_io.pyx":1354
 *                 i = 0
 *                 last_chunk_index += 1
 *                 if num_features < 0 and last_chunk_size < max_chunk_size:             # <<<<<<<<<<<<<<
 *                     # Double chunk size for next chunk, up to max_chunk_size
 *                     last_chunk_size = min(last_chunk_size * 2, max_chunk_size)
*/
        __pyx_t_5 = (__pyx_v_num_features < 0);
        if (__pyx_t_5) {
        } else {
          __pyx_t_1 = __pyx_t_5;
          goto __pyx_L47_bool_binop_done;
        }
        __pyx_t_10 = __Pyx_PyLong_From_long(__pyx_v_max_chunk_size); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1354, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_2 = PyObject_RichCompare(__pyx_v_last_chunk_size, __pyx_t_10, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1354, __pyx_L13_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1354, __pyx_L13_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_1 = __pyx_t_5;
        __pyx_L47_bool_binop_done:;
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":1356
 *                 if num_features < 0 and last_chunk_size < max_chunk_size:
 *                     # Double chunk size for next chunk, up to max_chunk_size
 *                     last_chunk_size = min(last_chunk_size * 2, max_chunk_size)             # <<<<<<<<<<<<<<
 * 
 *                 add_data_chunk(last_chunk_size)
*/
          __pyx_t_25 = __pyx_v_max_chunk_size;
          __pyx_t_2 = __Pyx_PyLong_MultiplyObjC(__pyx_v_last_chunk_size, __pyx_mstate_global->__pyx_int_2, 2, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1356, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_4 = __Pyx_PyLong_From_long(__pyx_t_25); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1356, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_11 = PyObject_RichCompare(__pyx_t_4, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_11); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1356, __pyx_L13_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_11); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1356, __pyx_L13_error)
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_1) {
            __pyx_t_11 = __Pyx_PyLong_From_long(__pyx_t_25); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1356, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_10 = __pyx_t_11;
            __pyx_t_11 = 0;
          } else {
            __Pyx_INCREF(__pyx_t_2);
            __pyx_t_10 = __pyx_t_2;
          }
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __pyx_t_10;
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF_SET(__pyx_v_last_chunk_size, __pyx_t_2);
          __pyx_t_2 = 0;

          /* "pyogrio/_io.pyx":1354
 *                 i = 0
 *                 last_chunk_index += 1
 *                 if num_features < 0 and last_chunk_size < max_chunk_size:             # <<<<<<<<<<<<<<
 *                     # Double chunk size for next chunk, up to max_chunk_size
 *                     last_chunk_size = min(last_chunk_size * 2, max_chunk_size)
*/
        }

        /* "pyogrio/_io.pyx":1358
 *                     last_chunk_size = min(last_chunk_size * 2, max_chunk_size)
 * 
 *                 add_data_chunk(last_chunk_size)             # <<<<<<<<<<<<<<
 *                 if return_fids:
 *                     fid_view = fid_chunks[last_chunk_index][:]
*/
        __pyx_t_2 = __pyx_pf_7pyogrio_3_io_12get_features_add_data_chunk(__pyx_v_add_data_chunk, __pyx_v_last_chunk_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1358, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyogrio/_io.pyx":1359
 * 
 *                 add_data_chunk(last_chunk_size)
 *                 if return_fids:             # <<<<<<<<<<<<<<
 *                     fid_view = fid_chunks[last_chunk_index][:]
 *                 if read_geometry:
*/
        __pyx_t_1 = (__pyx_cur_scope->__pyx_v_return_fids != 0);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":1360
 *                 add_data_chunk(last_chunk_size)
 *                 if return_fids:
 *                     fid_view = fid_chunks[last_chunk_index][:]             # <<<<<<<<<<<<<<
 *                 if read_geometry:
 *                     geom_view = geom_chunks[last_chunk_index][:]
*/
          __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_fid_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1360, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_10 = __Pyx_PyObject_GetSlice(__pyx_t_2, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1360, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF_SET(__pyx_v_fid_view, __pyx_t_10);
          __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":1359
 * 
 *                 add_data_chunk(last_chunk_size)
 *                 if return_fids:             # <<<<<<<<<<<<<<
 *                     fid_view = fid_chunks[last_chunk_index][:]
 *                 if read_geometry:
*/
        }

        /* "pyogrio/_io.pyx":1361
 *                 if return_fids:
 *                     fid_view = fid_chunks[last_chunk_index][:]
 *                 if read_geometry:             # <<<<<<<<<<<<<<
 *                     geom_view = geom_chunks[last_chunk_index][:]
 *                 field_data_view = [
*/
        __pyx_t_1 = (__pyx_cur_scope->__pyx_v_read_geometry != 0);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":1362
 *                     fid_view = fid_chunks[last_chunk_index][:]
 *                 if read_geometry:
 *                     geom_view = geom_chunks[last_chunk_index][:]             # <<<<<<<<<<<<<<
 *                 field_data_view = [
 *                     field_data_chunks[last_chunk_index][field_index][:]
*/
          __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_geom_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1362, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_10, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1362, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF_SET(__pyx_v_geom_view, __pyx_t_2);
          __pyx_t_2 = 0;

          /* "pyogrio/_io.pyx":1361
 *                 if return_fids:
 *                     fid_view = fid_chunks[last_chunk_index][:]
 *                 if read_geometry:             # <<<<<<<<<<<<<<
 *                     geom_view = geom_chunks[last_chunk_index][:]
 *                 field_data_view = [
*/
        }

        /* "pyogrio/_io.pyx":1363
 *                 if read_geometry:
 *                     geom_view = geom_chunks[last_chunk_index][:]
 *                 field_data_view = [             # <<<<<<<<<<<<<<
 *                     field_data_chunks[last_chunk_index][field_index][:]
 *                     for field_index in range(n_fields)
*/
        { /* enter inner scope */
          __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1363, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_2);

          /* "pyogrio/_io.pyx":1365
 *                 field_data_view = [
 *                     field_data_chunks[last_chunk_index][field_index][:]
 *                     for field_index in range(n_fields)             # <<<<<<<<<<<<<<
 *                 ]
 * 
*/
          __pyx_t_17 = __pyx_cur_scope->__pyx_v_n_fields;
          __pyx_t_12 = __pyx_t_17;
          for (__pyx_t_26 = 0; __pyx_t_26 < __pyx_t_12; __pyx_t_26+=1) {
            __pyx_8genexpr2__pyx_v_field_index = __pyx_t_26;

            /* "pyogrio/_io.pyx":1364
 *                     geom_view = geom_chunks[last_chunk_index][:]
 *                 field_data_view = [
 *                     field_data_chunks[last_chunk_index][field_index][:]             # <<<<<<<<<<<<<<
 *                     for field_index in range(n_fields)
 *                 ]
*/
            __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_field_data_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1364, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_11 = __Pyx_GetItemInt(__pyx_t_10, __pyx_8genexpr2__pyx_v_field_index, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1364, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __pyx_t_10 = __Pyx_PyObject_GetSlice(__pyx_t_11, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1364, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_10))) __PYX_ERR(0, 1363, __pyx_L13_error)
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          }
        } /* exit inner scope */
        __Pyx_XDECREF_SET(__pyx_v_field_data_view, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyogrio/_io.pyx":1350
 *                 ) from None
 * 
 *             if i_total == 0 or i == last_chunk_size:             # <<<<<<<<<<<<<<
 *                 # No chunk yet or last chunk filled up: add a new chunk.
 *                 i = 0
*/
      }

      /* "pyogrio/_io.pyx":1368
 *                 ]
 * 
 *             if return_fids:             # <<<<<<<<<<<<<<
 *                 fid_view[i] = OGR_F_GetFID(ogr_feature)
 * 
*/
      __pyx_t_1 = (__pyx_cur_scope->__pyx_v_return_fids != 0);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1369
 * 
 *             if return_fids:
 *                 fid_view[i] = OGR_F_GetFID(ogr_feature)             # <<<<<<<<<<<<<<
 * 
 *             if read_geometry:
*/
        __pyx_t_2 = __Pyx_PyLong_From_int64_t(OGR_F_GetFID(__pyx_v_ogr_feature)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1369, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(!__pyx_v_fid_view)) { __Pyx_RaiseUnboundLocalError("fid_view"); __PYX_ERR(0, 1369, __pyx_L13_error) }
        if (unlikely((__Pyx_SetItemInt(__pyx_v_fid_view, __pyx_v_i, __pyx_t_2, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1369, __pyx_L13_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyogrio/_io.pyx":1368
 *                 ]
 * 
 *             if return_fids:             # <<<<<<<<<<<<<<
 *                 fid_view[i] = OGR_F_GetFID(ogr_feature)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1371
 *                 fid_view[i] = OGR_F_GetFID(ogr_feature)
 * 
 *             if read_geometry:             # <<<<<<<<<<<<<<
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)
 * 
*/
      __pyx_t_1 = (__pyx_cur_scope->__pyx_v_read_geometry != 0);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1372
 * 
 *             if read_geometry:
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)             # <<<<<<<<<<<<<<
 * 
 *             process_fields(
*/
        if (unlikely(!__pyx_v_geom_view)) { __Pyx_RaiseUnboundLocalError("geom_view"); __PYX_ERR(0, 1372, __pyx_L13_error) }
        __pyx_t_2 = __pyx_f_7pyogrio_3_io_process_geometry(__pyx_v_ogr_feature, __pyx_v_i, __pyx_v_geom_view, __pyx_v_force_2d); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1372, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyogrio/_io.pyx":1371
 *                 fid_view[i] = OGR_F_GetFID(ogr_feature)
 * 
 *             if read_geometry:             # <<<<<<<<<<<<<<
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1375
 * 
 *             process_fields(
 *                 ogr_feature, i, n_fields, field_data_chunks[last_chunk_index],             # <<<<<<<<<<<<<<
 *                 field_data_view, field_indexes, field_ogr_types, encoding,
 *                 datetime_as_string
*/
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_field_data_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1375, __pyx_L13_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_2))) __PYX_ERR(0, 1375, __pyx_L13_error)

      /* "pyogrio/_io.pyx":1376
 *             process_fields(
 *                 ogr_feature, i, n_fields, field_data_chunks[last_chunk_index],
 *                 field_data_view, field_indexes, field_ogr_types, encoding,             # <<<<<<<<<<<<<<
 *                 datetime_as_string
 *             )
*/
      if (unlikely(!__pyx_v_field_data_view)) { __Pyx_RaiseUnboundLocalError("field_data_view"); __PYX_ERR(0, 1376, __pyx_L13_error) }

      /* "pyogrio/_io.pyx":1374
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)
 * 
 *             process_fields(             # <<<<<<<<<<<<<<
 *                 ogr_feature, i, n_fields, field_data_chunks[last_chunk_index],
 *                 field_data_view, field_indexes, field_ogr_types, encoding,
*/
      __pyx_t_10 = __pyx_f_7pyogrio_3_io_process_fields(__pyx_v_ogr_feature, __pyx_v_i, __pyx_cur_scope->__pyx_v_n_fields, ((PyObject*)__pyx_t_2), __pyx_v_field_data_view, __pyx_v_field_indexes, __pyx_v_field_ogr_types, __pyx_v_encoding, __pyx_cur_scope->__pyx_v_datetime_as_string); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1374, __pyx_L13_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":1380
 *             )
 * 
 *             i += 1             # <<<<<<<<<<<<<<
 *             i_total += 1
 * 
*/
      __pyx_v_i = (__pyx_v_i + 1);

      /* "pyogrio/_io.pyx":1381
 * 
 *             i += 1
 *             i_total += 1             # <<<<<<<<<<<<<<
 * 
 *         finally:
*/
      __pyx_t_10 = __Pyx_PyLong_AddObjC(__pyx_v_i_total, __pyx_mstate_global->__pyx_int_1, 1, 1, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1381, __pyx_L13_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF_SET(__pyx_v_i_total, __pyx_t_10);
      __pyx_t_10 = 0;
    }

    /* "pyogrio/_io.pyx":1384
 * 
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
    /*finally:*/ {
      /*normal exit:*/{
        __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":1385
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
          OGR_F_Destroy(__pyx_v_ogr_feature);

          /* "pyogrio/_io.pyx":1386
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     # If no features were read, add an empty chunk
*/
          __pyx_v_ogr_feature = NULL;

          /* "pyogrio/_io.pyx":1384
 * 
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
        }
        goto __pyx_L14;
      }
      __pyx_L13_error:;
      /*exception exit:*/{
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __pyx_t_8 = 0; __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_XCLEAR_MEMVIEW(&__pyx_t_3, 1);
        __pyx_t_3.memview = NULL; __pyx_t_3.data = NULL;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
         __Pyx_ExceptionSwap(&__pyx_t_24, &__pyx_t_23, &__pyx_t_22);
        if ( unlikely(__Pyx_GetException(&__pyx_t_8, &__pyx_t_7, &__pyx_t_6) < 0)) __Pyx_ErrFetch(&__pyx_t_8, &__pyx_t_7, &__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_24);
        __Pyx_XGOTREF(__pyx_t_23);
        __Pyx_XGOTREF(__pyx_t_22);
        __pyx_t_17 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_27 = __pyx_filename;
        {
          __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
          if (__pyx_t_1) {

            /* "pyogrio/_io.pyx":1385
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
            OGR_F_Destroy(__pyx_v_ogr_feature);

            /* "pyogrio/_io.pyx":1386
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     # If no features were read, add an empty chunk
*/
            __pyx_v_ogr_feature = NULL;

            /* "pyogrio/_io.pyx":1384
 * 
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
          }
        }
        __Pyx_XGIVEREF(__pyx_t_24);
        __Pyx_XGIVEREF(__pyx_t_23);
        __Pyx_XGIVEREF(__pyx_t_22);
        __Pyx_ExceptionReset(__pyx_t_24, __pyx_t_23, __pyx_t_22);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_ErrRestore(__pyx_t_8, __pyx_t_7, __pyx_t_6);
        __pyx_t_8 = 0; __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0;
        __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_27;
        goto __pyx_L1_error;
      }
      __pyx_L11_break: {
        __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":1385
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
          OGR_F_Destroy(__pyx_v_ogr_feature);

          /* "pyogrio/_io.pyx":1386
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     # If no features were read, add an empty chunk
*/
          __pyx_v_ogr_feature = NULL;

          /* "pyogrio/_io.pyx":1384
 * 
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
        }
        goto __pyx_L9_break;
      }
      __pyx_L14:;
    }
  }
  __pyx_L9_break:;

  /* "pyogrio/_io.pyx":1389
 * 
 *     # If no features were read, add an empty chunk
 *     if i_total == 0:             # <<<<<<<<<<<<<<
 *         add_data_chunk(0)
 *         last_chunk_size = 0
*/
  __pyx_t_1 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_i_total, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1389, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1390
 *     # If no features were read, add an empty chunk
 *     if i_total == 0:
 *         add_data_chunk(0)             # <<<<<<<<<<<<<<
 *         last_chunk_size = 0
 *         last_chunk_index = 0
*/
    __pyx_t_10 = __pyx_pf_7pyogrio_3_io_12get_features_add_data_chunk(__pyx_v_add_data_chunk, __pyx_mstate_global->__pyx_int_0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "pyogrio/_io.pyx":1391
 *     if i_total == 0:
 *         add_data_chunk(0)
 *         last_chunk_size = 0             # <<<<<<<<<<<<<<
 *         last_chunk_index = 0
 * 
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __Pyx_DECREF_SET(__pyx_v_last_chunk_size, __pyx_mstate_global->__pyx_int_0);

    /* "pyogrio/_io.pyx":1392
 *         add_data_chunk(0)
 *         last_chunk_size = 0
 *         last_chunk_index = 0             # <<<<<<<<<<<<<<
 * 
 *     # If the last chunk wasn't completely full, drop empty rows
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __Pyx_DECREF_SET(__pyx_v_last_chunk_index, __pyx_mstate_global->__pyx_int_0);

    /* "pyogrio/_io.pyx":1389
 * 
 *     # If no features were read, add an empty chunk
 *     if i_total == 0:             # <<<<<<<<<<<<<<
 *         add_data_chunk(0)
 *         last_chunk_size = 0
*/
  }

  /* "pyogrio/_io.pyx":1395
 * 
 *     # If the last chunk wasn't completely full, drop empty rows
 *     if i < last_chunk_size:             # <<<<<<<<<<<<<<
 *         if return_fids:
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_10, __pyx_v_last_chunk_size, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1395, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1396
 *     # If the last chunk wasn't completely full, drop empty rows
 *     if i < last_chunk_size:
 *         if return_fids:             # <<<<<<<<<<<<<<
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]
 *         if read_geometry:
*/
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_return_fids != 0);
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":1397
 *     if i < last_chunk_size:
 *         if return_fids:
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]             # <<<<<<<<<<<<<<
 *         if read_geometry:
 *             geom_chunks[last_chunk_index] = geom_chunks[last_chunk_index][:i]
*/
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_fid_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10 = __Pyx_PyObject_GetSlice(__pyx_t_2, 0, __pyx_v_i, NULL, NULL, NULL, 0, 1, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely((PyObject_SetItem(__pyx_cur_scope->__pyx_v_fid_chunks, __pyx_v_last_chunk_index, __pyx_t_10) < 0))) __PYX_ERR(0, 1397, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":1396
 *     # If the last chunk wasn't completely full, drop empty rows
 *     if i < last_chunk_size:
 *         if return_fids:             # <<<<<<<<<<<<<<
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]
 *         if read_geometry:
*/
    }

    /* "pyogrio/_io.pyx":1398
 *         if return_fids:
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]
 *         if read_geometry:             # <<<<<<<<<<<<<<
 *             geom_chunks[last_chunk_index] = geom_chunks[last_chunk_index][:i]
 *         for field_index in range(n_fields):
*/
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_read_geometry != 0);
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":1399
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]
 *         if read_geometry:
 *             geom_chunks[last_chunk_index] = geom_chunks[last_chunk_index][:i]             # <<<<<<<<<<<<<<
 *         for field_index in range(n_fields):
 *             field_data_chunks[last_chunk_index][field_index] = (
*/
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_geom_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_10, 0, __pyx_v_i, NULL, NULL, NULL, 0, 1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely((PyObject_SetItem(__pyx_cur_scope->__pyx_v_geom_chunks, __pyx_v_last_chunk_index, __pyx_t_2) < 0))) __PYX_ERR(0, 1399, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyogrio/_io.pyx":1398
 *         if return_fids:
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]
 *         if read_geometry:             # <<<<<<<<<<<<<<
 *             geom_chunks[last_chunk_index] = geom_chunks[last_chunk_index][:i]
 *         for field_index in range(n_fields):
*/
    }

    /* "pyogrio/_io.pyx":1400
 *         if read_geometry:
 *             geom_chunks[last_chunk_index] = geom_chunks[last_chunk_index][:i]
 *         for field_index in range(n_fields):             # <<<<<<<<<<<<<<
 *             field_data_chunks[last_chunk_index][field_index] = (
 *                 field_data_chunks[last_chunk_index][field_index][:i]
*/
    __pyx_t_12 = __pyx_cur_scope->__pyx_v_n_fields;
    __pyx_t_17 = __pyx_t_12;
    for (__pyx_t_26 = 0; __pyx_t_26 < __pyx_t_17; __pyx_t_26+=1) {
      __pyx_v_field_index = __pyx_t_26;

      /* "pyogrio/_io.pyx":1402
 *         for field_index in range(n_fields):
 *             field_data_chunks[last_chunk_index][field_index] = (
 *                 field_data_chunks[last_chunk_index][field_index][:i]             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_field_data_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_10 = __Pyx_GetItemInt(__pyx_t_2, __pyx_v_field_index, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_10, 0, __pyx_v_i, NULL, NULL, NULL, 0, 1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1402, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":1401
 *             geom_chunks[last_chunk_index] = geom_chunks[last_chunk_index][:i]
 *         for field_index in range(n_fields):
 *             field_data_chunks[last_chunk_index][field_index] = (             # <<<<<<<<<<<<<<
 *                 field_data_chunks[last_chunk_index][field_index][:i]
 *             )
*/
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_field_data_chunks, __pyx_v_last_chunk_index); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (unlikely((__Pyx_SetItemInt(__pyx_t_10, __pyx_v_field_index, __pyx_t_2, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1401, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }

    /* "pyogrio/_io.pyx":1395
 * 
 *     # If the last chunk wasn't completely full, drop empty rows
 *     if i < last_chunk_size:             # <<<<<<<<<<<<<<
 *         if return_fids:
 *             fid_chunks[last_chunk_index] = fid_chunks[last_chunk_index][:i]
*/
  }

  /* "pyogrio/_io.pyx":1405
 *             )
 * 
 *     if last_chunk_index == 0:             # <<<<<<<<<<<<<<
 *         # Only one chunk, so avoid concatenation
 *         fid_data = fid_chunks[0] if return_fids else None
*/
  __pyx_t_1 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_last_chunk_index, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1405, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1407
 *     if last_chunk_index == 0:
 *         # Only one chunk, so avoid concatenation
 *         fid_data = fid_chunks[0] if return_fids else None             # <<<<<<<<<<<<<<
 *         geom_data = geom_chunks[0] if read_geometry else None
 * 
*/
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_return_fids != 0);
    if (__pyx_t_1) {
      __Pyx_INCREF(__Pyx_PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_fid_chunks, 0));
      __pyx_t_2 = __Pyx_PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_fid_chunks, 0);
    } else {
      __Pyx_INCREF(Py_None);
      __pyx_t_2 = Py_None;
    }
    __pyx_v_fid_data = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1408
 *         # Only one chunk, so avoid concatenation
 *         fid_data = fid_chunks[0] if return_fids else None
 *         geom_data = geom_chunks[0] if read_geometry else None             # <<<<<<<<<<<<<<
 * 
 *         return fid_data, geom_data, field_data_chunks[0]
*/
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_read_geometry != 0);
    if (__pyx_t_1) {
      __Pyx_INCREF(__Pyx_PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_geom_chunks, 0));
      __pyx_t_2 = __Pyx_PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_geom_chunks, 0);
    } else {
      __Pyx_INCREF(Py_None);
      __pyx_t_2 = Py_None;
    }
    __pyx_v_geom_data = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1410
 *         geom_data = geom_chunks[0] if read_geometry else None
 * 
 *         return fid_data, geom_data, field_data_chunks[0]             # <<<<<<<<<<<<<<
 * 
 *     else:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1410, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_fid_data);
    __Pyx_GIVEREF(__pyx_v_fid_data);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_fid_data) != (0)) __PYX_ERR(0, 1410, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_geom_data);
    __Pyx_GIVEREF(__pyx_v_geom_data);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_geom_data) != (0)) __PYX_ERR(0, 1410, __pyx_L1_error);
    __Pyx_INCREF(__Pyx_PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_field_data_chunks, 0));
    __Pyx_GIVEREF(__Pyx_PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_field_data_chunks, 0));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 2, __Pyx_PyList_GET_ITEM(__pyx_cur_scope->__pyx_v_field_data_chunks, 0)) != (0)) __PYX_ERR(0, 1410, __pyx_L1_error);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":1405
 *             )
 * 
 *     if last_chunk_index == 0:             # <<<<<<<<<<<<<<
 *         # Only one chunk, so avoid concatenation
 *         fid_data = fid_chunks[0] if return_fids else None
*/
  }

  /* "pyogrio/_io.pyx":1414
 *     else:
 *         # Concatenate all chunks
 *         fid_data = np.concatenate(fid_chunks) if return_fids else None             # <<<<<<<<<<<<<<
 *         geom_data = np.concatenate(geom_chunks) if read_geometry else None
 * 
*/
  /*else*/ {
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_return_fids != 0);
    if (__pyx_t_1) {
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1414, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_concatenate); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1414, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_16 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_13);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
        __pyx_t_16 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_cur_scope->__pyx_v_fid_chunks};
        __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1414, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __pyx_t_2 = __pyx_t_10;
      __pyx_t_10 = 0;
    } else {
      __Pyx_INCREF(Py_None);
      __pyx_t_2 = Py_None;
    }
    __pyx_v_fid_data = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1415
 *         # Concatenate all chunks
 *         fid_data = np.concatenate(fid_chunks) if return_fids else None
 *         geom_data = np.concatenate(geom_chunks) if read_geometry else None             # <<<<<<<<<<<<<<
 * 
 *         field_data = []
*/
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_read_geometry != 0);
    if (__pyx_t_1) {
      __pyx_t_13 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1415, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_concatenate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1415, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_16 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_13);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_16 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_cur_scope->__pyx_v_geom_chunks};
        __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1415, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __pyx_t_2 = __pyx_t_10;
      __pyx_t_10 = 0;
    } else {
      __Pyx_INCREF(Py_None);
      __pyx_t_2 = Py_None;
    }
    __pyx_v_geom_data = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1417
 *         geom_data = np.concatenate(geom_chunks) if read_geometry else None
 * 
 *         field_data = []             # <<<<<<<<<<<<<<
 *         for field_index in range(n_fields):
 *             data_to_concat = []
*/
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_field_data = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1418
 * 
 *         field_data = []
 *         for field_index in range(n_fields):             # <<<<<<<<<<<<<<
 *             data_to_concat = []
 *             for chunk in range(last_chunk_index + 1):
*/
    __pyx_t_12 = __pyx_cur_scope->__pyx_v_n_fields;
    __pyx_t_17 = __pyx_t_12;
    for (__pyx_t_26 = 0; __pyx_t_26 < __pyx_t_17; __pyx_t_26+=1) {
      __pyx_v_field_index = __pyx_t_26;

      /* "pyogrio/_io.pyx":1419
 *         field_data = []
 *         for field_index in range(n_fields):
 *             data_to_concat = []             # <<<<<<<<<<<<<<
 *             for chunk in range(last_chunk_index + 1):
 *                 data_to_concat.append(field_data_chunks[chunk][field_index])
*/
      __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1419, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_data_to_concat, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyogrio/_io.pyx":1420
 *         for field_index in range(n_fields):
 *             data_to_concat = []
 *             for chunk in range(last_chunk_index + 1):             # <<<<<<<<<<<<<<
 *                 data_to_concat.append(field_data_chunks[chunk][field_index])
 *             field_data.append(np.concatenate(data_to_concat))
*/
      __pyx_t_10 = NULL;
      __pyx_t_4 = __Pyx_PyLong_AddObjC(__pyx_v_last_chunk_index, __pyx_mstate_global->__pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1420, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_16 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(&PyRange_Type), __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1420, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1420, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_28 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_28)) __PYX_ERR(0, 1420, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      for (;;) {
        {
          __pyx_t_2 = __pyx_t_28(__pyx_t_4);
          if (unlikely(!__pyx_t_2)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1420, __pyx_L1_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_chunk, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "pyogrio/_io.pyx":1421
 *             data_to_concat = []
 *             for chunk in range(last_chunk_index + 1):
 *                 data_to_concat.append(field_data_chunks[chunk][field_index])             # <<<<<<<<<<<<<<
 *             field_data.append(np.concatenate(data_to_concat))
 * 
*/
        __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_field_data_chunks, __pyx_v_chunk); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1421, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_10 = __Pyx_GetItemInt(__pyx_t_2, __pyx_v_field_index, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1421, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_29 = __Pyx_PyList_Append(__pyx_v_data_to_concat, __pyx_t_10); if (unlikely(__pyx_t_29 == ((int)-1))) __PYX_ERR(0, 1421, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "pyogrio/_io.pyx":1420
 *         for field_index in range(n_fields):
 *             data_to_concat = []
 *             for chunk in range(last_chunk_index + 1):             # <<<<<<<<<<<<<<
 *                 data_to_concat.append(field_data_chunks[chunk][field_index])
 *             field_data.append(np.concatenate(data_to_concat))
*/
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyogrio/_io.pyx":1422
 *             for chunk in range(last_chunk_index + 1):
 *                 data_to_concat.append(field_data_chunks[chunk][field_index])
 *             field_data.append(np.concatenate(data_to_concat))             # <<<<<<<<<<<<<<
 * 
 *         return fid_data, geom_data, field_data
*/
      __pyx_t_10 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1422, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_concatenate); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1422, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_16 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_13);
        assert(__pyx_t_10);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
        __pyx_t_16 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_data_to_concat};
        __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1422, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __pyx_t_29 = __Pyx_PyList_Append(__pyx_v_field_data, __pyx_t_4); if (unlikely(__pyx_t_29 == ((int)-1))) __PYX_ERR(0, 1422, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }

    /* "pyogrio/_io.pyx":1424
 *             field_data.append(np.concatenate(data_to_concat))
 * 
 *         return fid_data, geom_data, field_data             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1424, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v_fid_data);
    __Pyx_GIVEREF(__pyx_v_fid_data);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_fid_data) != (0)) __PYX_ERR(0, 1424, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_geom_data);
    __Pyx_GIVEREF(__pyx_v_geom_data);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_geom_data) != (0)) __PYX_ERR(0, 1424, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_field_data);
    __Pyx_GIVEREF(__pyx_v_field_data);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_field_data) != (0)) __PYX_ERR(0, 1424, __pyx_L1_error);
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "pyogrio/_io.pyx":1225
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef get_features(
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_3, 1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("pyogrio._io.get_features", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_last_chunk_size);
  __Pyx_XDECREF(__pyx_v_add_data_chunk);
  __Pyx_XDECREF(__pyx_v_i_total);
  __Pyx_XDECREF(__pyx_v_last_chunk_index);
  __Pyx_XDECREF(__pyx_v_field_indexes);
  __Pyx_XDECREF(__pyx_v_field_ogr_types);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_fid_view);
  __Pyx_XDECREF(__pyx_v_geom_view);
  __Pyx_XDECREF(__pyx_v_field_data_view);
  __Pyx_XDECREF(__pyx_v_fid_data);
  __Pyx_XDECREF(__pyx_v_geom_data);
  __Pyx_XDECREF(__pyx_v_field_data);
  __Pyx_XDECREF(__pyx_v_data_to_concat);
  __Pyx_XDECREF(__pyx_v_chunk);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1427
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef get_features_by_fid(
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_features_by_fid(OGRLayerH __pyx_v_ogr_layer, __Pyx_memviewslice __pyx_v_fids, __Pyx_memviewslice __pyx_v_fields, PyObject *__pyx_v_encoding, uint8_t __pyx_v_read_geometry, uint8_t __pyx_v_force_2d, int __pyx_v_datetime_as_string) {
  OGRFeatureH __pyx_v_ogr_feature;
  int __pyx_v_n_fields;
  int __pyx_v_i;
  int __pyx_v_fid;
  int __pyx_v_count;
  PyObject *__pyx_v_geometries = NULL;
  PyObject *__pyx_v_geom_view = NULL;
  PyObject *__pyx_v_field_indexes = NULL;
  PyObject *__pyx_v_field_ogr_types = NULL;
  PyObject *__pyx_v_field_data = NULL;
  PyObject *__pyx_v_field_data_view = NULL;
  PyObject *__pyx_v_exc = NULL;
  int __pyx_8genexpr3__pyx_v_field_index;
  int __pyx_8genexpr4__pyx_v_field_index;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  int __pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  void *__pyx_t_23;
  int __pyx_t_24;
  int __pyx_t_25;
  char const *__pyx_t_26;
  PyObject *__pyx_t_27 = NULL;
  PyObject *__pyx_t_28 = NULL;
  PyObject *__pyx_t_29 = NULL;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  char const *__pyx_t_33;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_features_by_fid", 0);

  /* "pyogrio/_io.pyx":1464
 * 
 *     """
 *     cdef OGRFeatureH ogr_feature = NULL             # <<<<<<<<<<<<<<
 *     cdef int n_fields
 *     cdef int i
*/
  __pyx_v_ogr_feature = NULL;

  /* "pyogrio/_io.pyx":1469
 *     cdef int fid
 *     cdef int field_index
 *     cdef int count = len(fids)             # <<<<<<<<<<<<<<
 * 
 *     # make sure layer is read from beginning
*/
  __pyx_t_1 = __Pyx_MemoryView_Len(__pyx_v_fids); 
  __pyx_v_count = __pyx_t_1;

  /* "pyogrio/_io.pyx":1472
 * 
 *     # make sure layer is read from beginning
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_ResetReading(ogr_layer)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":1473
 *     # make sure layer is read from beginning
 *     with nogil:
 *         OGR_L_ResetReading(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *     if read_geometry:
*/
        OGR_L_ResetReading(__pyx_v_ogr_layer);
      }

      /* "pyogrio/_io.pyx":1472
 * 
 *     # make sure layer is read from beginning
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_ResetReading(ogr_layer)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":1475
 *         OGR_L_ResetReading(ogr_layer)
 * 
 *     if read_geometry:             # <<<<<<<<<<<<<<
 *         geometries = np.empty(shape=(count, ), dtype="object")
 *         geom_view = geometries[:]
*/
  __pyx_t_2 = (__pyx_v_read_geometry != 0);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":1476
 * 
 *     if read_geometry:
 *         geometries = np.empty(shape=(count, ), dtype="object")             # <<<<<<<<<<<<<<
 *         geom_view = geometries[:]
 * 
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyLong_From_int(__pyx_v_count); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5) != (0)) __PYX_ERR(0, 1476, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_4, NULL};
      __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_7, __pyx_t_5, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1476, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_object, __pyx_t_5, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1476, __pyx_L1_error)
      __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1476, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_v_geometries = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":1477
 *     if read_geometry:
 *         geometries = np.empty(shape=(count, ), dtype="object")
 *         geom_view = geometries[:]             # <<<<<<<<<<<<<<
 * 
 *     else:
*/
    __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_geometries, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1477, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_geom_view = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":1475
 *         OGR_L_ResetReading(ogr_layer)
 * 
 *     if read_geometry:             # <<<<<<<<<<<<<<
 *         geometries = np.empty(shape=(count, ), dtype="object")
 *         geom_view = geometries[:]
*/
    goto __pyx_L6;
  }

  /* "pyogrio/_io.pyx":1480
 * 
 *     else:
 *         geometries = None             # <<<<<<<<<<<<<<
 * 
 *     n_fields = fields.shape[0]
*/
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __pyx_v_geometries = Py_None;
  }
  __pyx_L6:;

  /* "pyogrio/_io.pyx":1482
 *         geometries = None
 * 
 *     n_fields = fields.shape[0]             # <<<<<<<<<<<<<<
 *     field_indexes = list(fields[:, 0])
 *     field_ogr_types = list(fields[:, 1])
*/
  __pyx_v_n_fields = (__pyx_v_fields.shape[0]);

  /* "pyogrio/_io.pyx":1483
 * 
 *     n_fields = fields.shape[0]
 *     field_indexes = list(fields[:, 0])             # <<<<<<<<<<<<<<
 *     field_ogr_types = list(fields[:, 1])
 *     field_data = [
*/
  __pyx_t_9.data = __pyx_v_fields.data;
  __pyx_t_9.memview = __pyx_v_fields.memview;
  __PYX_INC_MEMVIEW(&__pyx_t_9, 1);
  __pyx_t_9.shape[0] = __pyx_v_fields.shape[0];
__pyx_t_9.strides[0] = __pyx_v_fields.strides[0];
    __pyx_t_9.suboffsets[0] = -1;

{
    Py_ssize_t __pyx_tmp_idx = 0;
    Py_ssize_t __pyx_tmp_stride = __pyx_v_fields.strides[1];
        __pyx_t_9.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_3 = __pyx_memoryview_fromslice(__pyx_t_9, 1, (PyObject *(*)(char *)) __pyx_memview_get_object, (int (*)(char *, PyObject *)) __pyx_memview_set_object, 1);; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_9, 1);
  __pyx_t_9.memview = NULL; __pyx_t_9.data = NULL;
  __pyx_t_6 = __Pyx_PySequence_ListKeepNew(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_field_indexes = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "pyogrio/_io.pyx":1484
 *     n_fields = fields.shape[0]
 *     field_indexes = list(fields[:, 0])
 *     field_ogr_types = list(fields[:, 1])             # <<<<<<<<<<<<<<
 *     field_data = [
 *         np.empty(
*/
  __pyx_t_9.data = __pyx_v_fields.data;
  __pyx_t_9.memview = __pyx_v_fields.memview;
  __PYX_INC_MEMVIEW(&__pyx_t_9, 1);
  __pyx_t_9.shape[0] = __pyx_v_fields.shape[0];
__pyx_t_9.strides[0] = __pyx_v_fields.strides[0];
    __pyx_t_9.suboffsets[0] = -1;

{
    Py_ssize_t __pyx_tmp_idx = 1;
    Py_ssize_t __pyx_tmp_stride = __pyx_v_fields.strides[1];
        __pyx_t_9.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_6 = __pyx_memoryview_fromslice(__pyx_t_9, 1, (PyObject *(*)(char *)) __pyx_memview_get_object, (int (*)(char *, PyObject *)) __pyx_memview_set_object, 1);; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_9, 1);
  __pyx_t_9.memview = NULL; __pyx_t_9.data = NULL;
  __pyx_t_3 = __Pyx_PySequence_ListKeepNew(__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_field_ogr_types = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyogrio/_io.pyx":1485
 *     field_indexes = list(fields[:, 0])
 *     field_ogr_types = list(fields[:, 1])
 *     field_data = [             # <<<<<<<<<<<<<<
 *         np.empty(
 *             shape=(count, ),
*/
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1485, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "pyogrio/_io.pyx":1493
 *                 else fields[field_index, 3]
 *             )
 *         ) for field_index in range(n_fields)             # <<<<<<<<<<<<<<
 *     ]
 * 
*/
    __pyx_t_10 = __pyx_v_n_fields;
    __pyx_t_11 = __pyx_t_10;
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_8genexpr3__pyx_v_field_index = __pyx_t_12;

      /* "pyogrio/_io.pyx":1486
 *     field_ogr_types = list(fields[:, 1])
 *     field_data = [
 *         np.empty(             # <<<<<<<<<<<<<<
 *             shape=(count, ),
 *             dtype=(
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1486, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1486, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "pyogrio/_io.pyx":1487
 *     field_data = [
 *         np.empty(
 *             shape=(count, ),             # <<<<<<<<<<<<<<
 *             dtype=(
 *                 "object"
*/
      __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_count); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1487, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_13 = PyTuple_New(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1487, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_GIVEREF(__pyx_t_7);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_7) != (0)) __PYX_ERR(0, 1487, __pyx_L1_error);
      __pyx_t_7 = 0;

      /* "pyogrio/_io.pyx":1490
 *             dtype=(
 *                 "object"
 *                 if datetime_as_string and fields[field_index, 3].startswith("datetime")             # <<<<<<<<<<<<<<
 *                 else fields[field_index, 3]
 *             )
*/
      if (__pyx_v_datetime_as_string) {
      } else {
        __pyx_t_2 = __pyx_v_datetime_as_string;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_17 = __pyx_8genexpr3__pyx_v_field_index;
      __pyx_t_18 = 3;
      __pyx_t_16 = (PyObject *) *((PyObject * *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_fields.data + __pyx_t_17 * __pyx_v_fields.strides[0]) ) + __pyx_t_18 * __pyx_v_fields.strides[1]) ));
      if (unlikely(__pyx_t_16 == NULL)) __pyx_t_16 = Py_None;
      __Pyx_INCREF((PyObject*)__pyx_t_16);
      __pyx_t_15 = __pyx_t_16;
      __Pyx_INCREF(__pyx_t_15);
      __pyx_t_8 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_mstate_global->__pyx_n_u_datetime};
        __pyx_t_14 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_startswith, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1490, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
      }
      __pyx_t_19 = __Pyx_PyObject_IsTrue(__pyx_t_14); if (unlikely((__pyx_t_19 < 0))) __PYX_ERR(0, 1490, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_2 = __pyx_t_19;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_2) {
        __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_object);
        __pyx_t_7 = __pyx_mstate_global->__pyx_n_u_object;
      } else {

        /* "pyogrio/_io.pyx":1491
 *                 "object"
 *                 if datetime_as_string and fields[field_index, 3].startswith("datetime")
 *                 else fields[field_index, 3]             # <<<<<<<<<<<<<<
 *             )
 *         ) for field_index in range(n_fields)
*/
        __pyx_t_18 = __pyx_8genexpr3__pyx_v_field_index;
        __pyx_t_17 = 3;
        __pyx_t_14 = (PyObject *) *((PyObject * *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_fields.data + __pyx_t_18 * __pyx_v_fields.strides[0]) ) + __pyx_t_17 * __pyx_v_fields.strides[1]) ));
        if (unlikely(__pyx_t_14 == NULL)) __pyx_t_14 = Py_None;
        __Pyx_INCREF((PyObject*)__pyx_t_14);
        __pyx_t_7 = __pyx_t_14;
        __pyx_t_14 = 0;
      }
      __pyx_t_8 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_8 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_5, NULL};
        __pyx_t_14 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1486, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_13, __pyx_t_14, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1486, __pyx_L1_error)
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_14, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1486, __pyx_L1_error)
        __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_14);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1486, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 1485, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  } /* exit inner scope */
  __pyx_v_field_data = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyogrio/_io.pyx":1496
 *     ]
 * 
 *     field_data_view = [field_data[field_index][:] for field_index in range(n_fields)]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(count):
*/
  { /* enter inner scope */
    __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1496, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = __pyx_v_n_fields;
    __pyx_t_11 = __pyx_t_10;
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_8genexpr4__pyx_v_field_index = __pyx_t_12;
      __pyx_t_6 = __Pyx_PyObject_GetSlice(__Pyx_PyList_GET_ITEM(__pyx_v_field_data, __pyx_8genexpr4__pyx_v_field_index), 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1496, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 1496, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  } /* exit inner scope */
  __pyx_v_field_data_view = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pyogrio/_io.pyx":1498
 *     field_data_view = [field_data[field_index][:] for field_index in range(n_fields)]
 * 
 *     for i in range(count):             # <<<<<<<<<<<<<<
 *         try:
 *             fid = fids[i]
*/
  __pyx_t_10 = __pyx_v_count;
  __pyx_t_11 = __pyx_t_10;
  for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
    __pyx_v_i = __pyx_t_12;

    /* "pyogrio/_io.pyx":1499
 * 
 *     for i in range(count):
 *         try:             # <<<<<<<<<<<<<<
 *             fid = fids[i]
 * 
*/
    /*try:*/ {

      /* "pyogrio/_io.pyx":1500
 *     for i in range(count):
 *         try:
 *             fid = fids[i]             # <<<<<<<<<<<<<<
 * 
 *             try:
*/
      __pyx_t_17 = __pyx_v_i;
      __pyx_v_fid = (*((int *) ( /* dim=0 */ (__pyx_v_fids.data + __pyx_t_17 * __pyx_v_fids.strides[0]) )));

      /* "pyogrio/_io.pyx":1502
 *             fid = fids[i]
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 with nogil:
 *                     ogr_feature = OGR_L_GetFeature(ogr_layer, fid)
*/
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
        __Pyx_XGOTREF(__pyx_t_20);
        __Pyx_XGOTREF(__pyx_t_21);
        __Pyx_XGOTREF(__pyx_t_22);
        /*try:*/ {

          /* "pyogrio/_io.pyx":1503
 * 
 *             try:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     ogr_feature = OGR_L_GetFeature(ogr_layer, fid)
 *                 ogr_feature = check_pointer(ogr_feature)
*/
          {
              PyThreadState * _save;
              _save = PyEval_SaveThread();
              __Pyx_FastGIL_Remember();
              /*try:*/ {

                /* "pyogrio/_io.pyx":1504
 *             try:
 *                 with nogil:
 *                     ogr_feature = OGR_L_GetFeature(ogr_layer, fid)             # <<<<<<<<<<<<<<
 *                 ogr_feature = check_pointer(ogr_feature)
 * 
*/
                __pyx_v_ogr_feature = OGR_L_GetFeature(__pyx_v_ogr_layer, __pyx_v_fid);
              }

              /* "pyogrio/_io.pyx":1503
 * 
 *             try:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     ogr_feature = OGR_L_GetFeature(ogr_layer, fid)
 *                 ogr_feature = check_pointer(ogr_feature)
*/
              /*finally:*/ {
                /*normal exit:*/{
                  __Pyx_FastGIL_Forget();
                  PyEval_RestoreThread(_save);
                  goto __pyx_L32;
                }
                __pyx_L32:;
              }
          }

          /* "pyogrio/_io.pyx":1505
 *                 with nogil:
 *                     ogr_feature = OGR_L_GetFeature(ogr_layer, fid)
 *                 ogr_feature = check_pointer(ogr_feature)             # <<<<<<<<<<<<<<
 * 
 *             except NullPointerError:
*/
          __pyx_t_23 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_feature); if (unlikely(__pyx_t_23 == ((void *)NULL))) __PYX_ERR(0, 1505, __pyx_L20_error)
          __pyx_v_ogr_feature = __pyx_t_23;

          /* "pyogrio/_io.pyx":1502
 *             fid = fids[i]
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 with nogil:
 *                     ogr_feature = OGR_L_GetFeature(ogr_layer, fid)
*/
        }
        __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
        __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
        __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
        goto __pyx_L27_try_end;
        __pyx_L20_error:;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __PYX_XCLEAR_MEMVIEW(&__pyx_t_9, 1);
        __pyx_t_9.memview = NULL; __pyx_t_9.data = NULL;

        /* "pyogrio/_io.pyx":1507
 *                 ogr_feature = check_pointer(ogr_feature)
 * 
 *             except NullPointerError:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(f"Could not read feature with fid {fid}") from None
 * 
*/
        __Pyx_ErrFetch(&__pyx_t_3, &__pyx_t_6, &__pyx_t_4);
        __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1507, __pyx_L22_except_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_24 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_3, __pyx_t_14);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_ErrRestore(__pyx_t_3, __pyx_t_6, __pyx_t_4);
        __pyx_t_3 = 0; __pyx_t_6 = 0; __pyx_t_4 = 0;
        if (__pyx_t_24) {
          __Pyx_AddTraceback("pyogrio._io.get_features_by_fid", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_3) < 0) __PYX_ERR(0, 1507, __pyx_L22_except_error)
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_t_3);

          /* "pyogrio/_io.pyx":1508
 * 
 *             except NullPointerError:
 *                 raise FeatureError(f"Could not read feature with fid {fid}") from None             # <<<<<<<<<<<<<<
 * 
 *             except CPLE_BaseError as exc:
*/
          __pyx_t_7 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1508, __pyx_L22_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_5 = __Pyx_PyUnicode_From_int(__pyx_v_fid, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1508, __pyx_L22_except_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_16 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Could_not_read_feature_with_fid, __pyx_t_5); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1508, __pyx_L22_except_error)
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_8 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_13))) {
            __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_13);
            assert(__pyx_t_7);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
            __pyx_t_8 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_16};
            __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1508, __pyx_L22_except_error)
            __Pyx_GOTREF(__pyx_t_14);
          }
          __Pyx_Raise(__pyx_t_14, 0, 0, Py_None);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __PYX_ERR(0, 1508, __pyx_L22_except_error)
        }

        /* "pyogrio/_io.pyx":1510
 *                 raise FeatureError(f"Could not read feature with fid {fid}") from None
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(str(exc))
 * 
*/
        __Pyx_ErrFetch(&__pyx_t_3, &__pyx_t_6, &__pyx_t_4);
        __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1510, __pyx_L22_except_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_24 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_3, __pyx_t_14);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_ErrRestore(__pyx_t_3, __pyx_t_6, __pyx_t_4);
        __pyx_t_3 = 0; __pyx_t_6 = 0; __pyx_t_4 = 0;
        if (__pyx_t_24) {
          __Pyx_AddTraceback("pyogrio._io.get_features_by_fid", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_6, &__pyx_t_3) < 0) __PYX_ERR(0, 1510, __pyx_L22_except_error)
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_6);
          __pyx_v_exc = __pyx_t_6;
          /*try:*/ {

            /* "pyogrio/_io.pyx":1511
 * 
 *             except CPLE_BaseError as exc:
 *                 raise FeatureError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *             if read_geometry:
*/
            __pyx_t_13 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1511, __pyx_L40_error)
            __Pyx_GOTREF(__pyx_t_16);
            __pyx_t_7 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1511, __pyx_L40_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_8 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_16))) {
              __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_16);
              assert(__pyx_t_13);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
              __pyx_t_8 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_7};
              __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
              if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1511, __pyx_L40_error)
              __Pyx_GOTREF(__pyx_t_14);
            }
            __Pyx_Raise(__pyx_t_14, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __PYX_ERR(0, 1511, __pyx_L40_error)
          }

          /* "pyogrio/_io.pyx":1510
 *                 raise FeatureError(f"Could not read feature with fid {fid}") from None
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(str(exc))
 * 
*/
          /*finally:*/ {
            __pyx_L40_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_27 = 0; __pyx_t_28 = 0; __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              __PYX_XCLEAR_MEMVIEW(&__pyx_t_9, 1);
              __pyx_t_9.memview = NULL; __pyx_t_9.data = NULL;
               __Pyx_ExceptionSwap(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32);
              if ( unlikely(__Pyx_GetException(&__pyx_t_27, &__pyx_t_28, &__pyx_t_29) < 0)) __Pyx_ErrFetch(&__pyx_t_27, &__pyx_t_28, &__pyx_t_29);
              __Pyx_XGOTREF(__pyx_t_27);
              __Pyx_XGOTREF(__pyx_t_28);
              __Pyx_XGOTREF(__pyx_t_29);
              __Pyx_XGOTREF(__pyx_t_30);
              __Pyx_XGOTREF(__pyx_t_31);
              __Pyx_XGOTREF(__pyx_t_32);
              __pyx_t_24 = __pyx_lineno; __pyx_t_25 = __pyx_clineno; __pyx_t_26 = __pyx_filename;
              {
                __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_30);
              __Pyx_XGIVEREF(__pyx_t_31);
              __Pyx_XGIVEREF(__pyx_t_32);
              __Pyx_ExceptionReset(__pyx_t_30, __pyx_t_31, __pyx_t_32);
              __Pyx_XGIVEREF(__pyx_t_27);
              __Pyx_XGIVEREF(__pyx_t_28);
              __Pyx_XGIVEREF(__pyx_t_29);
              __Pyx_ErrRestore(__pyx_t_27, __pyx_t_28, __pyx_t_29);
              __pyx_t_27 = 0; __pyx_t_28 = 0; __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0;
              __pyx_lineno = __pyx_t_24; __pyx_clineno = __pyx_t_25; __pyx_filename = __pyx_t_26;
              goto __pyx_L22_except_error;
            }
          }
        }
        goto __pyx_L22_except_error;

        /* "pyogrio/_io.pyx":1502
 *             fid = fids[i]
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 with nogil:
 *                     ogr_feature = OGR_L_GetFeature(ogr_layer, fid)
*/
        __pyx_L22_except_error:;
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_XGIVEREF(__pyx_t_21);
        __Pyx_XGIVEREF(__pyx_t_22);
        __Pyx_ExceptionReset(__pyx_t_20, __pyx_t_21, __pyx_t_22);
        goto __pyx_L18_error;
        __pyx_L27_try_end:;
      }

      /* "pyogrio/_io.pyx":1513
 *                 raise FeatureError(str(exc))
 * 
 *             if read_geometry:             # <<<<<<<<<<<<<<
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)
 * 
*/
      __pyx_t_2 = (__pyx_v_read_geometry != 0);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1514
 * 
 *             if read_geometry:
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)             # <<<<<<<<<<<<<<
 * 
 *             process_fields(
*/
        if (unlikely(!__pyx_v_geom_view)) { __Pyx_RaiseUnboundLocalError("geom_view"); __PYX_ERR(0, 1514, __pyx_L18_error) }
        __pyx_t_3 = __pyx_f_7pyogrio_3_io_process_geometry(__pyx_v_ogr_feature, __pyx_v_i, __pyx_v_geom_view, __pyx_v_force_2d); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1514, __pyx_L18_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyogrio/_io.pyx":1513
 *                 raise FeatureError(str(exc))
 * 
 *             if read_geometry:             # <<<<<<<<<<<<<<
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1516
 *                 process_geometry(ogr_feature, i, geom_view, force_2d)
 * 
 *             process_fields(             # <<<<<<<<<<<<<<
 *                 ogr_feature, i, n_fields, field_data, field_data_view,
 *                 field_indexes, field_ogr_types, encoding, datetime_as_string
*/
      __pyx_t_3 = __pyx_f_7pyogrio_3_io_process_fields(__pyx_v_ogr_feature, __pyx_v_i, __pyx_v_n_fields, __pyx_v_field_data, __pyx_v_field_data_view, __pyx_v_field_indexes, __pyx_v_field_ogr_types, __pyx_v_encoding, __pyx_v_datetime_as_string); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1516, __pyx_L18_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }

    /* "pyogrio/_io.pyx":1521
 *             )
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
    /*finally:*/ {
      /*normal exit:*/{
        __pyx_t_2 = (__pyx_v_ogr_feature != NULL);
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":1522
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
          OGR_F_Destroy(__pyx_v_ogr_feature);

          /* "pyogrio/_io.pyx":1523
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     return (geometries, field_data)
*/
          __pyx_v_ogr_feature = NULL;

          /* "pyogrio/_io.pyx":1521
 *             )
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
        }
        goto __pyx_L19;
      }
      __pyx_L18_error:;
      /*exception exit:*/{
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_32 = 0; __pyx_t_31 = 0; __pyx_t_30 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __PYX_XCLEAR_MEMVIEW(&__pyx_t_9, 1);
        __pyx_t_9.memview = NULL; __pyx_t_9.data = NULL;
         __Pyx_ExceptionSwap(&__pyx_t_32, &__pyx_t_31, &__pyx_t_30);
        if ( unlikely(__Pyx_GetException(&__pyx_t_22, &__pyx_t_21, &__pyx_t_20) < 0)) __Pyx_ErrFetch(&__pyx_t_22, &__pyx_t_21, &__pyx_t_20);
        __Pyx_XGOTREF(__pyx_t_22);
        __Pyx_XGOTREF(__pyx_t_21);
        __Pyx_XGOTREF(__pyx_t_20);
        __Pyx_XGOTREF(__pyx_t_32);
        __Pyx_XGOTREF(__pyx_t_31);
        __Pyx_XGOTREF(__pyx_t_30);
        __pyx_t_25 = __pyx_lineno; __pyx_t_24 = __pyx_clineno; __pyx_t_33 = __pyx_filename;
        {
          __pyx_t_2 = (__pyx_v_ogr_feature != NULL);
          if (__pyx_t_2) {

            /* "pyogrio/_io.pyx":1522
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
            OGR_F_Destroy(__pyx_v_ogr_feature);

            /* "pyogrio/_io.pyx":1523
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     return (geometries, field_data)
*/
            __pyx_v_ogr_feature = NULL;

            /* "pyogrio/_io.pyx":1521
 *             )
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
          }
        }
        __Pyx_XGIVEREF(__pyx_t_32);
        __Pyx_XGIVEREF(__pyx_t_31);
        __Pyx_XGIVEREF(__pyx_t_30);
        __Pyx_ExceptionReset(__pyx_t_32, __pyx_t_31, __pyx_t_30);
        __Pyx_XGIVEREF(__pyx_t_22);
        __Pyx_XGIVEREF(__pyx_t_21);
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_ErrRestore(__pyx_t_22, __pyx_t_21, __pyx_t_20);
        __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_32 = 0; __pyx_t_31 = 0; __pyx_t_30 = 0;
        __pyx_lineno = __pyx_t_25; __pyx_clineno = __pyx_t_24; __pyx_filename = __pyx_t_33;
        goto __pyx_L1_error;
      }
      __pyx_L19:;
    }
  }

  /* "pyogrio/_io.pyx":1525
 *                 ogr_feature = NULL
 * 
 *     return (geometries, field_data)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_geometries);
  __Pyx_GIVEREF(__pyx_v_geometries);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_geometries) != (0)) __PYX_ERR(0, 1525, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_field_data);
  __Pyx_GIVEREF(__pyx_v_field_data);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_field_data) != (0)) __PYX_ERR(0, 1525, __pyx_L1_error);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":1427
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef get_features_by_fid(
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_9, 1);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("pyogrio._io.get_features_by_fid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_geometries);
  __Pyx_XDECREF(__pyx_v_geom_view);
  __Pyx_XDECREF(__pyx_v_field_indexes);
  __Pyx_XDECREF(__pyx_v_field_ogr_types);
  __Pyx_XDECREF(__pyx_v_field_data);
  __Pyx_XDECREF(__pyx_v_field_data_view);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1528
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef get_bounds(OGRLayerH ogr_layer, int skip_features, int num_features):
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_bounds(OGRLayerH __pyx_v_ogr_layer, int __pyx_v_skip_features, int __pyx_v_num_features) {
  OGRFeatureH __pyx_v_ogr_feature;
  OGRGeometryH __pyx_v_ogr_geometry;
  OGREnvelope __pyx_v_ogr_envelope;
  int __pyx_v_i;
  PyObject *__pyx_v_fid_data = NULL;
  PyObject *__pyx_v_fid_view = NULL;
  PyObject *__pyx_v_bounds_data = NULL;
  PyObject *__pyx_v_bounds_view = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  void *__pyx_t_13;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  char const *__pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  char const *__pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_bounds", 0);

  /* "pyogrio/_io.pyx":1549
 * 
 *     """
 *     cdef OGRFeatureH ogr_feature = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRGeometryH ogr_geometry = NULL
 *     cdef OGREnvelope ogr_envelope  # = NULL
*/
  __pyx_v_ogr_feature = NULL;

  /* "pyogrio/_io.pyx":1550
 *     """
 *     cdef OGRFeatureH ogr_feature = NULL
 *     cdef OGRGeometryH ogr_geometry = NULL             # <<<<<<<<<<<<<<
 *     cdef OGREnvelope ogr_envelope  # = NULL
 *     cdef int i
*/
  __pyx_v_ogr_geometry = NULL;

  /* "pyogrio/_io.pyx":1555
 * 
 *     # make sure layer is read from beginning
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_ResetReading(ogr_layer)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":1556
 *     # make sure layer is read from beginning
 *     with nogil:
 *         OGR_L_ResetReading(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *     if skip_features > 0:
*/
        OGR_L_ResetReading(__pyx_v_ogr_layer);
      }

      /* "pyogrio/_io.pyx":1555
 * 
 *     # make sure layer is read from beginning
 *     with nogil:             # <<<<<<<<<<<<<<
 *         OGR_L_ResetReading(ogr_layer)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":1558
 *         OGR_L_ResetReading(ogr_layer)
 * 
 *     if skip_features > 0:             # <<<<<<<<<<<<<<
 *         apply_skip_features(ogr_layer, skip_features)
 * 
*/
  __pyx_t_1 = (__pyx_v_skip_features > 0);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1559
 * 
 *     if skip_features > 0:
 *         apply_skip_features(ogr_layer, skip_features)             # <<<<<<<<<<<<<<
 * 
 *     fid_data = np.empty(shape=(num_features), dtype=np.int64)
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_apply_skip_features(__pyx_v_ogr_layer, __pyx_v_skip_features); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1558
 *         OGR_L_ResetReading(ogr_layer)
 * 
 *     if skip_features > 0:             # <<<<<<<<<<<<<<
 *         apply_skip_features(ogr_layer, skip_features)
 * 
*/
  }

  /* "pyogrio/_io.pyx":1561
 *         apply_skip_features(ogr_layer, skip_features)
 * 
 *     fid_data = np.empty(shape=(num_features), dtype=np.int64)             # <<<<<<<<<<<<<<
 *     fid_view = fid_data[:]
 * 
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyLong_From_int(__pyx_v_num_features); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_8 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_8 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_3, NULL};
    __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_4, __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1561, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_7, __pyx_t_6, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_fid_data = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1562
 * 
 *     fid_data = np.empty(shape=(num_features), dtype=np.int64)
 *     fid_view = fid_data[:]             # <<<<<<<<<<<<<<
 * 
 *     bounds_data = np.empty(shape=(4, num_features), dtype="float64")
*/
  __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_fid_data, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_fid_view = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1564
 *     fid_view = fid_data[:]
 * 
 *     bounds_data = np.empty(shape=(4, num_features), dtype="float64")             # <<<<<<<<<<<<<<
 *     bounds_view = bounds_data[:]
 * 
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_num_features); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_4);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_mstate_global->__pyx_int_4) != (0)) __PYX_ERR(0, 1564, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_6);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6) != (0)) __PYX_ERR(0, 1564, __pyx_L1_error);
  __pyx_t_6 = 0;
  __pyx_t_8 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_8 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_5, NULL};
    __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_4, __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1564, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_mstate_global->__pyx_n_u_float64, __pyx_t_6, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1564, __pyx_L1_error)
    __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_8, (1-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_bounds_data = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1565
 * 
 *     bounds_data = np.empty(shape=(4, num_features), dtype="float64")
 *     bounds_view = bounds_data[:]             # <<<<<<<<<<<<<<
 * 
 *     i = 0
*/
  __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_v_bounds_data, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_bounds_view = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":1567
 *     bounds_view = bounds_data[:]
 * 
 *     i = 0             # <<<<<<<<<<<<<<
 *     while True:
 *         try:
*/
  __pyx_v_i = 0;

  /* "pyogrio/_io.pyx":1568
 * 
 *     i = 0
 *     while True:             # <<<<<<<<<<<<<<
 *         try:
 *             if num_features > 0 and i == num_features:
*/
  while (1) {

    /* "pyogrio/_io.pyx":1569
 *     i = 0
 *     while True:
 *         try:             # <<<<<<<<<<<<<<
 *             if num_features > 0 and i == num_features:
 *                 break
*/
    /*try:*/ {

      /* "pyogrio/_io.pyx":1570
 *     while True:
 *         try:
 *             if num_features > 0 and i == num_features:             # <<<<<<<<<<<<<<
 *                 break
 * 
*/
      __pyx_t_9 = (__pyx_v_num_features > 0);
      if (__pyx_t_9) {
      } else {
        __pyx_t_1 = __pyx_t_9;
        goto __pyx_L15_bool_binop_done;
      }
      __pyx_t_9 = (__pyx_v_i == __pyx_v_num_features);
      __pyx_t_1 = __pyx_t_9;
      __pyx_L15_bool_binop_done:;
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1571
 *         try:
 *             if num_features > 0 and i == num_features:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             try:
*/
        goto __pyx_L10_break;

        /* "pyogrio/_io.pyx":1570
 *     while True:
 *         try:
 *             if num_features > 0 and i == num_features:             # <<<<<<<<<<<<<<
 *                 break
 * 
*/
      }

      /* "pyogrio/_io.pyx":1573
 *                 break
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 # Don't use `with nogil` inside this loop, as if there are many
 *                 # features, the overhead of acquiring and releasing the GIL becomes
*/
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_12);
        /*try:*/ {

          /* "pyogrio/_io.pyx":1577
 *                 # features, the overhead of acquiring and releasing the GIL becomes
 *                 # significant on linux.
 *                 ogr_feature = OGR_L_GetNextFeature(ogr_layer)             # <<<<<<<<<<<<<<
 *                 ogr_feature = check_pointer(ogr_feature)
 * 
*/
          __pyx_v_ogr_feature = OGR_L_GetNextFeature(__pyx_v_ogr_layer);

          /* "pyogrio/_io.pyx":1578
 *                 # significant on linux.
 *                 ogr_feature = OGR_L_GetNextFeature(ogr_layer)
 *                 ogr_feature = check_pointer(ogr_feature)             # <<<<<<<<<<<<<<
 * 
 *             except NullPointerError:
*/
          __pyx_t_13 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_v_ogr_feature); if (unlikely(__pyx_t_13 == ((void *)NULL))) __PYX_ERR(0, 1578, __pyx_L17_error)
          __pyx_v_ogr_feature = __pyx_t_13;

          /* "pyogrio/_io.pyx":1573
 *                 break
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 # Don't use `with nogil` inside this loop, as if there are many
 *                 # features, the overhead of acquiring and releasing the GIL becomes
*/
        }
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        goto __pyx_L24_try_end;
        __pyx_L17_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":1580
 *                 ogr_feature = check_pointer(ogr_feature)
 * 
 *             except NullPointerError:             # <<<<<<<<<<<<<<
 *                 # No more rows available, so stop reading
 *                 break
*/
        __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_7, &__pyx_t_6);
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1580, __pyx_L19_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_14 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_4);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_ErrRestore(__pyx_t_2, __pyx_t_7, __pyx_t_6);
        __pyx_t_2 = 0; __pyx_t_7 = 0; __pyx_t_6 = 0;
        if (__pyx_t_14) {
          __Pyx_AddTraceback("pyogrio._io.get_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_2) < 0) __PYX_ERR(0, 1580, __pyx_L19_except_error)
          __Pyx_XGOTREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_2);

          /* "pyogrio/_io.pyx":1582
 *             except NullPointerError:
 *                 # No more rows available, so stop reading
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             except CPLE_BaseError as exc:
*/
          goto __pyx_L26_except_break;
          __pyx_L26_except_break:;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L22_try_break;
        }

        /* "pyogrio/_io.pyx":1584
 *                 break
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(str(exc))
 * 
*/
        __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_7, &__pyx_t_6);
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1584, __pyx_L19_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_14 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_4);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_ErrRestore(__pyx_t_2, __pyx_t_7, __pyx_t_6);
        __pyx_t_2 = 0; __pyx_t_7 = 0; __pyx_t_6 = 0;
        if (__pyx_t_14) {
          __Pyx_AddTraceback("pyogrio._io.get_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_2) < 0) __PYX_ERR(0, 1584, __pyx_L19_except_error)
          __Pyx_XGOTREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_7);
          __pyx_v_exc = __pyx_t_7;
          /*try:*/ {

            /* "pyogrio/_io.pyx":1585
 * 
 *             except CPLE_BaseError as exc:
 *                 raise FeatureError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *             if i >= num_features:
*/
            __pyx_t_5 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1585, __pyx_L32_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_15 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1585, __pyx_L32_error)
            __Pyx_GOTREF(__pyx_t_15);
            __pyx_t_8 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_3))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
              assert(__pyx_t_5);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
              __pyx_t_8 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_15};
              __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1585, __pyx_L32_error)
              __Pyx_GOTREF(__pyx_t_4);
            }
            __Pyx_Raise(__pyx_t_4, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __PYX_ERR(0, 1585, __pyx_L32_error)
          }

          /* "pyogrio/_io.pyx":1584
 *                 break
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(str(exc))
 * 
*/
          /*finally:*/ {
            __pyx_L32_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0;
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
               __Pyx_ExceptionSwap(&__pyx_t_21, &__pyx_t_22, &__pyx_t_23);
              if ( unlikely(__Pyx_GetException(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20) < 0)) __Pyx_ErrFetch(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
              __Pyx_XGOTREF(__pyx_t_18);
              __Pyx_XGOTREF(__pyx_t_19);
              __Pyx_XGOTREF(__pyx_t_20);
              __Pyx_XGOTREF(__pyx_t_21);
              __Pyx_XGOTREF(__pyx_t_22);
              __Pyx_XGOTREF(__pyx_t_23);
              __pyx_t_14 = __pyx_lineno; __pyx_t_16 = __pyx_clineno; __pyx_t_17 = __pyx_filename;
              {
                __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_21);
              __Pyx_XGIVEREF(__pyx_t_22);
              __Pyx_XGIVEREF(__pyx_t_23);
              __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_22, __pyx_t_23);
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_XGIVEREF(__pyx_t_20);
              __Pyx_ErrRestore(__pyx_t_18, __pyx_t_19, __pyx_t_20);
              __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0;
              __pyx_lineno = __pyx_t_14; __pyx_clineno = __pyx_t_16; __pyx_filename = __pyx_t_17;
              goto __pyx_L19_except_error;
            }
          }
        }
        goto __pyx_L19_except_error;

        /* "pyogrio/_io.pyx":1573
 *                 break
 * 
 *             try:             # <<<<<<<<<<<<<<
 *                 # Don't use `with nogil` inside this loop, as if there are many
 *                 # features, the overhead of acquiring and releasing the GIL becomes
*/
        __pyx_L19_except_error:;
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        goto __pyx_L12_error;
        __pyx_L22_try_break:;
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        goto __pyx_L10_break;
        __pyx_L24_try_end:;
      }

      /* "pyogrio/_io.pyx":1587
 *                 raise FeatureError(str(exc))
 * 
 *             if i >= num_features:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(
 *                     "Reading more features than indicated by OGR_L_GetFeatureCount is "
*/
      __pyx_t_1 = (__pyx_v_i >= __pyx_v_num_features);
      if (unlikely(__pyx_t_1)) {

        /* "pyogrio/_io.pyx":1588
 * 
 *             if i >= num_features:
 *                 raise FeatureError(             # <<<<<<<<<<<<<<
 *                     "Reading more features than indicated by OGR_L_GetFeatureCount is "
 *                     "not supported"
*/
        __pyx_t_7 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1588, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_8 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          assert(__pyx_t_7);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
          __pyx_t_8 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_Reading_more_features_than_indic};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1588, __pyx_L12_error)
          __Pyx_GOTREF(__pyx_t_2);
        }

        /* "pyogrio/_io.pyx":1591
 *                     "Reading more features than indicated by OGR_L_GetFeatureCount is "
 *                     "not supported"
 *                 ) from None             # <<<<<<<<<<<<<<
 * 
 *             fid_view[i] = OGR_F_GetFID(ogr_feature)
*/
        __Pyx_Raise(__pyx_t_2, 0, 0, Py_None);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 1588, __pyx_L12_error)

        /* "pyogrio/_io.pyx":1587
 *                 raise FeatureError(str(exc))
 * 
 *             if i >= num_features:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(
 *                     "Reading more features than indicated by OGR_L_GetFeatureCount is "
*/
      }

      /* "pyogrio/_io.pyx":1593
 *                 ) from None
 * 
 *             fid_view[i] = OGR_F_GetFID(ogr_feature)             # <<<<<<<<<<<<<<
 * 
 *             ogr_geometry = OGR_F_GetGeometryRef(ogr_feature)
*/
      __pyx_t_2 = __Pyx_PyLong_From_int64_t(OGR_F_GetFID(__pyx_v_ogr_feature)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1593, __pyx_L12_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely((__Pyx_SetItemInt(__pyx_v_fid_view, __pyx_v_i, __pyx_t_2, int, 1, __Pyx_PyLong_From_int, 0, 0, 0, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 1593, __pyx_L12_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyogrio/_io.pyx":1595
 *             fid_view[i] = OGR_F_GetFID(ogr_feature)
 * 
 *             ogr_geometry = OGR_F_GetGeometryRef(ogr_feature)             # <<<<<<<<<<<<<<
 * 
 *             if ogr_geometry == NULL:
*/
      __pyx_v_ogr_geometry = OGR_F_GetGeometryRef(__pyx_v_ogr_feature);

      /* "pyogrio/_io.pyx":1597
 *             ogr_geometry = OGR_F_GetGeometryRef(ogr_feature)
 * 
 *             if ogr_geometry == NULL:             # <<<<<<<<<<<<<<
 *                 bounds_view[:, i] = np.nan
 * 
*/
      __pyx_t_1 = (__pyx_v_ogr_geometry == NULL);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":1598
 * 
 *             if ogr_geometry == NULL:
 *                 bounds_view[:, i] = np.nan             # <<<<<<<<<<<<<<
 * 
 *             else:
*/
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1598, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_nan); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1598, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1598, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1598, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[0]);
        __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_mstate_global->__pyx_slice[0]) != (0)) __PYX_ERR(0, 1598, __pyx_L12_error);
        __Pyx_GIVEREF(__pyx_t_2);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 1598, __pyx_L12_error);
        __pyx_t_2 = 0;
        if (unlikely((PyObject_SetItem(__pyx_v_bounds_view, __pyx_t_7, __pyx_t_6) < 0))) __PYX_ERR(0, 1598, __pyx_L12_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyogrio/_io.pyx":1597
 *             ogr_geometry = OGR_F_GetGeometryRef(ogr_feature)
 * 
 *             if ogr_geometry == NULL:             # <<<<<<<<<<<<<<
 *                 bounds_view[:, i] = np.nan
 * 
*/
        goto __pyx_L39;
      }

      /* "pyogrio/_io.pyx":1601
 * 
 *             else:
 *                 OGR_G_GetEnvelope(ogr_geometry, &ogr_envelope)             # <<<<<<<<<<<<<<
 *                 bounds_view[0, i] = ogr_envelope.MinX
 *                 bounds_view[1, i] = ogr_envelope.MinY
*/
      /*else*/ {
        OGR_G_GetEnvelope(__pyx_v_ogr_geometry, (&__pyx_v_ogr_envelope));

        /* "pyogrio/_io.pyx":1602
 *             else:
 *                 OGR_G_GetEnvelope(ogr_geometry, &ogr_envelope)
 *                 bounds_view[0, i] = ogr_envelope.MinX             # <<<<<<<<<<<<<<
 *                 bounds_view[1, i] = ogr_envelope.MinY
 *                 bounds_view[2, i] = ogr_envelope.MaxX
*/
        __pyx_t_6 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MinX); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1602, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1602, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1602, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
        __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(0, 1602, __pyx_L12_error);
        __Pyx_GIVEREF(__pyx_t_7);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 1602, __pyx_L12_error);
        __pyx_t_7 = 0;
        if (unlikely((PyObject_SetItem(__pyx_v_bounds_view, __pyx_t_2, __pyx_t_6) < 0))) __PYX_ERR(0, 1602, __pyx_L12_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyogrio/_io.pyx":1603
 *                 OGR_G_GetEnvelope(ogr_geometry, &ogr_envelope)
 *                 bounds_view[0, i] = ogr_envelope.MinX
 *                 bounds_view[1, i] = ogr_envelope.MinY             # <<<<<<<<<<<<<<
 *                 bounds_view[2, i] = ogr_envelope.MaxX
 *                 bounds_view[3, i] = ogr_envelope.MaxY
*/
        __pyx_t_6 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MinY); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1603, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1603, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1603, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
        __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_1);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_mstate_global->__pyx_int_1) != (0)) __PYX_ERR(0, 1603, __pyx_L12_error);
        __Pyx_GIVEREF(__pyx_t_2);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 1603, __pyx_L12_error);
        __pyx_t_2 = 0;
        if (unlikely((PyObject_SetItem(__pyx_v_bounds_view, __pyx_t_7, __pyx_t_6) < 0))) __PYX_ERR(0, 1603, __pyx_L12_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyogrio/_io.pyx":1604
 *                 bounds_view[0, i] = ogr_envelope.MinX
 *                 bounds_view[1, i] = ogr_envelope.MinY
 *                 bounds_view[2, i] = ogr_envelope.MaxX             # <<<<<<<<<<<<<<
 *                 bounds_view[3, i] = ogr_envelope.MaxY
 * 
*/
        __pyx_t_6 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MaxX); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1604, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1604, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1604, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_mstate_global->__pyx_int_2);
        __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_2);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_mstate_global->__pyx_int_2) != (0)) __PYX_ERR(0, 1604, __pyx_L12_error);
        __Pyx_GIVEREF(__pyx_t_7);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 1604, __pyx_L12_error);
        __pyx_t_7 = 0;
        if (unlikely((PyObject_SetItem(__pyx_v_bounds_view, __pyx_t_2, __pyx_t_6) < 0))) __PYX_ERR(0, 1604, __pyx_L12_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyogrio/_io.pyx":1605
 *                 bounds_view[1, i] = ogr_envelope.MinY
 *                 bounds_view[2, i] = ogr_envelope.MaxX
 *                 bounds_view[3, i] = ogr_envelope.MaxY             # <<<<<<<<<<<<<<
 * 
 *             i += 1
*/
        __pyx_t_6 = PyFloat_FromDouble(__pyx_v_ogr_envelope.MaxY); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1605, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_2 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1605, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1605, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_mstate_global->__pyx_int_3);
        __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_3);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_mstate_global->__pyx_int_3) != (0)) __PYX_ERR(0, 1605, __pyx_L12_error);
        __Pyx_GIVEREF(__pyx_t_2);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 1605, __pyx_L12_error);
        __pyx_t_2 = 0;
        if (unlikely((PyObject_SetItem(__pyx_v_bounds_view, __pyx_t_7, __pyx_t_6) < 0))) __PYX_ERR(0, 1605, __pyx_L12_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __pyx_L39:;

      /* "pyogrio/_io.pyx":1607
 *                 bounds_view[3, i] = ogr_envelope.MaxY
 * 
 *             i += 1             # <<<<<<<<<<<<<<
 *         finally:
 *             if ogr_feature != NULL:
*/
      __pyx_v_i = (__pyx_v_i + 1);
    }

    /* "pyogrio/_io.pyx":1609
 *             i += 1
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
    /*finally:*/ {
      /*normal exit:*/{
        __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":1610
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
          OGR_F_Destroy(__pyx_v_ogr_feature);

          /* "pyogrio/_io.pyx":1611
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Less rows read than anticipated, so drop empty rows
*/
          __pyx_v_ogr_feature = NULL;

          /* "pyogrio/_io.pyx":1609
 *             i += 1
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
        }
        goto __pyx_L13;
      }
      __pyx_L12_error:;
      /*exception exit:*/{
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_10 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
         __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_22, &__pyx_t_21);
        if ( unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_23);
        __Pyx_XGOTREF(__pyx_t_22);
        __Pyx_XGOTREF(__pyx_t_21);
        __pyx_t_16 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_24 = __pyx_filename;
        {
          __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
          if (__pyx_t_1) {

            /* "pyogrio/_io.pyx":1610
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
            OGR_F_Destroy(__pyx_v_ogr_feature);

            /* "pyogrio/_io.pyx":1611
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Less rows read than anticipated, so drop empty rows
*/
            __pyx_v_ogr_feature = NULL;

            /* "pyogrio/_io.pyx":1609
 *             i += 1
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
          }
        }
        __Pyx_XGIVEREF(__pyx_t_23);
        __Pyx_XGIVEREF(__pyx_t_22);
        __Pyx_XGIVEREF(__pyx_t_21);
        __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_22, __pyx_t_21);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ErrRestore(__pyx_t_12, __pyx_t_11, __pyx_t_10);
        __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_10 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0;
        __pyx_lineno = __pyx_t_16; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_24;
        goto __pyx_L1_error;
      }
      __pyx_L10_break: {
        __pyx_t_1 = (__pyx_v_ogr_feature != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":1610
 *         finally:
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *                 ogr_feature = NULL
 * 
*/
          OGR_F_Destroy(__pyx_v_ogr_feature);

          /* "pyogrio/_io.pyx":1611
 *             if ogr_feature != NULL:
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Less rows read than anticipated, so drop empty rows
*/
          __pyx_v_ogr_feature = NULL;

          /* "pyogrio/_io.pyx":1609
 *             i += 1
 *         finally:
 *             if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *                 OGR_F_Destroy(ogr_feature)
 *                 ogr_feature = NULL
*/
        }
        goto __pyx_L8_break;
      }
      __pyx_L13:;
    }
  }
  __pyx_L8_break:;

  /* "pyogrio/_io.pyx":1614
 * 
 *     # Less rows read than anticipated, so drop empty rows
 *     if i < num_features:             # <<<<<<<<<<<<<<
 *         fid_data = fid_data[:i]
 *         bounds_data = bounds_data[:, :i]
*/
  __pyx_t_1 = (__pyx_v_i < __pyx_v_num_features);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1615
 *     # Less rows read than anticipated, so drop empty rows
 *     if i < num_features:
 *         fid_data = fid_data[:i]             # <<<<<<<<<<<<<<
 *         bounds_data = bounds_data[:, :i]
 * 
*/
    __pyx_t_6 = __Pyx_PyObject_GetSlice(__pyx_v_fid_data, 0, __pyx_v_i, NULL, NULL, NULL, 0, 1, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1615, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF_SET(__pyx_v_fid_data, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "pyogrio/_io.pyx":1616
 *     if i < num_features:
 *         fid_data = fid_data[:i]
 *         bounds_data = bounds_data[:, :i]             # <<<<<<<<<<<<<<
 * 
 *     return fid_data, bounds_data
*/
    __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1616, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PySlice_New(Py_None, __pyx_t_6, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1616, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1616, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[0]);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_mstate_global->__pyx_slice[0]) != (0)) __PYX_ERR(0, 1616, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 1616, __pyx_L1_error);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_v_bounds_data, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1616, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_bounds_data, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":1614
 * 
 *     # Less rows read than anticipated, so drop empty rows
 *     if i < num_features:             # <<<<<<<<<<<<<<
 *         fid_data = fid_data[:i]
 *         bounds_data = bounds_data[:, :i]
*/
  }

  /* "pyogrio/_io.pyx":1618
 *         bounds_data = bounds_data[:, :i]
 * 
 *     return fid_data, bounds_data             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_fid_data);
  __Pyx_GIVEREF(__pyx_v_fid_data);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_fid_data) != (0)) __PYX_ERR(0, 1618, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_bounds_data);
  __Pyx_GIVEREF(__pyx_v_bounds_data);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_bounds_data) != (0)) __PYX_ERR(0, 1618, __pyx_L1_error);
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":1528
 * 
 * 
 * @cython.boundscheck(False)  # Deactivate bounds checking             # <<<<<<<<<<<<<<
 * @cython.wraparound(False)   # Deactivate negative indexing.
 * cdef get_bounds(OGRLayerH ogr_layer, int skip_features, int num_features):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("pyogrio._io.get_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fid_data);
  __Pyx_XDECREF(__pyx_v_fid_view);
  __Pyx_XDECREF(__pyx_v_bounds_data);
  __Pyx_XDECREF(__pyx_v_bounds_view);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1621
 * 
 * 
 * def ogr_read(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object dataset_kwargs,
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_1ogr_read(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_ogr_read, "Read features from a dataset.\n\n    Returns\n    -------\n    tuple(dict, ndarray, ndarray, ndarray)\n        A tuple containing a dictionary with metadata, an array with feature IDs,\n        an array with geometries and an array with field data read from the dataset.\n\n    ");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_1ogr_read = {"ogr_read", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_1ogr_read, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_ogr_read};
static PyObject *__pyx_pw_7pyogrio_3_io_1ogr_read(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path_or_buffer = 0;
  PyObject *__pyx_v_dataset_kwargs = 0;
  PyObject *__pyx_v_layer = 0;
  PyObject *__pyx_v_encoding = 0;
  int __pyx_v_read_geometry;
  int __pyx_v_force_2d;
  PyObject *__pyx_v_columns = 0;
  int __pyx_v_skip_features;
  int __pyx_v_max_features;
  PyObject *__pyx_v_where = 0;
  PyObject *__pyx_v_bbox = 0;
  PyObject *__pyx_v_mask = 0;
  PyObject *__pyx_v_fids = 0;
  PyObject *__pyx_v_sql = 0;
  PyObject *__pyx_v_sql_dialect = 0;
  int __pyx_v_return_fids;
  int __pyx_v_datetime_as_string;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[17] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_read (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path_or_buffer,&__pyx_mstate_global->__pyx_n_u_dataset_kwargs,&__pyx_mstate_global->__pyx_n_u_layer,&__pyx_mstate_global->__pyx_n_u_encoding,&__pyx_mstate_global->__pyx_n_u_read_geometry,&__pyx_mstate_global->__pyx_n_u_force_2d,&__pyx_mstate_global->__pyx_n_u_columns,&__pyx_mstate_global->__pyx_n_u_skip_features,&__pyx_mstate_global->__pyx_n_u_max_features,&__pyx_mstate_global->__pyx_n_u_where,&__pyx_mstate_global->__pyx_n_u_bbox,&__pyx_mstate_global->__pyx_n_u_mask,&__pyx_mstate_global->__pyx_n_u_fids,&__pyx_mstate_global->__pyx_n_u_sql,&__pyx_mstate_global->__pyx_n_u_sql_dialect,&__pyx_mstate_global->__pyx_n_u_return_fids,&__pyx_mstate_global->__pyx_n_u_datetime_as_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1621, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case 17:
        values[16] = __Pyx_ArgRef_FASTCALL(__pyx_args, 16);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[16])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 16:
        values[15] = __Pyx_ArgRef_FASTCALL(__pyx_args, 15);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[15])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 15:
        values[14] = __Pyx_ArgRef_FASTCALL(__pyx_args, 14);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[14])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 14:
        values[13] = __Pyx_ArgRef_FASTCALL(__pyx_args, 13);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[13])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "ogr_read", 0) < (0)) __PYX_ERR(0, 1621, __pyx_L3_error)

      /* "pyogrio/_io.pyx":1624
 *     object path_or_buffer,
 *     object dataset_kwargs,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int read_geometry=True,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1625
 *     object dataset_kwargs,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int read_geometry=True,
 *     int force_2d=False,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1628
 *     int read_geometry=True,
 *     int force_2d=False,
 *     object columns=None,             # <<<<<<<<<<<<<<
 *     int skip_features=0,
 *     int max_features=0,
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1631
 *     int skip_features=0,
 *     int max_features=0,
 *     object where=None,             # <<<<<<<<<<<<<<
 *     tuple bbox=None,
 *     object mask=None,
*/
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1632
 *     int max_features=0,
 *     object where=None,
 *     tuple bbox=None,             # <<<<<<<<<<<<<<
 *     object mask=None,
 *     object fids=None,
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1633
 *     object where=None,
 *     tuple bbox=None,
 *     object mask=None,             # <<<<<<<<<<<<<<
 *     object fids=None,
 *     str sql=None,
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1634
 *     tuple bbox=None,
 *     object mask=None,
 *     object fids=None,             # <<<<<<<<<<<<<<
 *     str sql=None,
 *     str sql_dialect=None,
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1635
 *     object mask=None,
 *     object fids=None,
 *     str sql=None,             # <<<<<<<<<<<<<<
 *     str sql_dialect=None,
 *     int return_fids=False,
*/
      if (!values[13]) values[13] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1636
 *     object fids=None,
 *     str sql=None,
 *     str sql_dialect=None,             # <<<<<<<<<<<<<<
 *     int return_fids=False,
 *     bint datetime_as_string=False,
*/
      if (!values[14]) values[14] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("ogr_read", 0, 2, 17, i); __PYX_ERR(0, 1621, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case 17:
        values[16] = __Pyx_ArgRef_FASTCALL(__pyx_args, 16);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[16])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 16:
        values[15] = __Pyx_ArgRef_FASTCALL(__pyx_args, 15);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[15])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 15:
        values[14] = __Pyx_ArgRef_FASTCALL(__pyx_args, 14);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[14])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 14:
        values[13] = __Pyx_ArgRef_FASTCALL(__pyx_args, 13);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[13])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1621, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1621, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1621, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "pyogrio/_io.pyx":1624
 *     object path_or_buffer,
 *     object dataset_kwargs,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int read_geometry=True,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1625
 *     object dataset_kwargs,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int read_geometry=True,
 *     int force_2d=False,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1628
 *     int read_geometry=True,
 *     int force_2d=False,
 *     object columns=None,             # <<<<<<<<<<<<<<
 *     int skip_features=0,
 *     int max_features=0,
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1631
 *     int skip_features=0,
 *     int max_features=0,
 *     object where=None,             # <<<<<<<<<<<<<<
 *     tuple bbox=None,
 *     object mask=None,
*/
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1632
 *     int max_features=0,
 *     object where=None,
 *     tuple bbox=None,             # <<<<<<<<<<<<<<
 *     object mask=None,
 *     object fids=None,
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1633
 *     object where=None,
 *     tuple bbox=None,
 *     object mask=None,             # <<<<<<<<<<<<<<
 *     object fids=None,
 *     str sql=None,
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1634
 *     tuple bbox=None,
 *     object mask=None,
 *     object fids=None,             # <<<<<<<<<<<<<<
 *     str sql=None,
 *     str sql_dialect=None,
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1635
 *     object mask=None,
 *     object fids=None,
 *     str sql=None,             # <<<<<<<<<<<<<<
 *     str sql_dialect=None,
 *     int return_fids=False,
*/
      if (!values[13]) values[13] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1636
 *     object fids=None,
 *     str sql=None,
 *     str sql_dialect=None,             # <<<<<<<<<<<<<<
 *     int return_fids=False,
 *     bint datetime_as_string=False,
*/
      if (!values[14]) values[14] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_path_or_buffer = values[0];
    __pyx_v_dataset_kwargs = values[1];
    __pyx_v_layer = values[2];
    __pyx_v_encoding = values[3];
    if (values[4]) {
      __pyx_v_read_geometry = __Pyx_PyLong_As_int(values[4]); if (unlikely((__pyx_v_read_geometry == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1626, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":1626
 *     object layer=None,
 *     object encoding=None,
 *     int read_geometry=True,             # <<<<<<<<<<<<<<
 *     int force_2d=False,
 *     object columns=None,
*/
      __pyx_v_read_geometry = ((int)((int)1));
    }
    if (values[5]) {
      __pyx_v_force_2d = __Pyx_PyLong_As_int(values[5]); if (unlikely((__pyx_v_force_2d == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1627, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":1627
 *     object encoding=None,
 *     int read_geometry=True,
 *     int force_2d=False,             # <<<<<<<<<<<<<<
 *     object columns=None,
 *     int skip_features=0,
*/
      __pyx_v_force_2d = ((int)((int)0));
    }
    __pyx_v_columns = values[6];
    if (values[7]) {
      __pyx_v_skip_features = __Pyx_PyLong_As_int(values[7]); if (unlikely((__pyx_v_skip_features == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1629, __pyx_L3_error)
    } else {
      __pyx_v_skip_features = ((int)((int)0));
    }
    if (values[8]) {
      __pyx_v_max_features = __Pyx_PyLong_As_int(values[8]); if (unlikely((__pyx_v_max_features == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1630, __pyx_L3_error)
    } else {
      __pyx_v_max_features = ((int)((int)0));
    }
    __pyx_v_where = values[9];
    __pyx_v_bbox = ((PyObject*)values[10]);
    __pyx_v_mask = values[11];
    __pyx_v_fids = values[12];
    __pyx_v_sql = ((PyObject*)values[13]);
    __pyx_v_sql_dialect = ((PyObject*)values[14]);
    if (values[15]) {
      __pyx_v_return_fids = __Pyx_PyLong_As_int(values[15]); if (unlikely((__pyx_v_return_fids == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1637, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":1637
 *     str sql=None,
 *     str sql_dialect=None,
 *     int return_fids=False,             # <<<<<<<<<<<<<<
 *     bint datetime_as_string=False,
 * ):
*/
      __pyx_v_return_fids = ((int)((int)0));
    }
    if (values[16]) {
      __pyx_v_datetime_as_string = __Pyx_PyObject_IsTrue(values[16]); if (unlikely((__pyx_v_datetime_as_string == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1638, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":1638
 *     str sql_dialect=None,
 *     int return_fids=False,
 *     bint datetime_as_string=False,             # <<<<<<<<<<<<<<
 * ):
 *     """Read features from a dataset.
*/
      __pyx_v_datetime_as_string = ((int)((int)0));
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ogr_read", 0, 2, 17, __pyx_nargs); __PYX_ERR(0, 1621, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.ogr_read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_bbox), (&PyTuple_Type), 1, "bbox", 1))) __PYX_ERR(0, 1632, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sql), (&PyUnicode_Type), 1, "sql", 1))) __PYX_ERR(0, 1635, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sql_dialect), (&PyUnicode_Type), 1, "sql_dialect", 1))) __PYX_ERR(0, 1636, __pyx_L1_error)
  __pyx_r = __pyx_pf_7pyogrio_3_io_ogr_read(__pyx_self, __pyx_v_path_or_buffer, __pyx_v_dataset_kwargs, __pyx_v_layer, __pyx_v_encoding, __pyx_v_read_geometry, __pyx_v_force_2d, __pyx_v_columns, __pyx_v_skip_features, __pyx_v_max_features, __pyx_v_where, __pyx_v_bbox, __pyx_v_mask, __pyx_v_fids, __pyx_v_sql, __pyx_v_sql_dialect, __pyx_v_return_fids, __pyx_v_datetime_as_string);

  /* "pyogrio/_io.pyx":1621
 * 
 * 
 * def ogr_read(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object dataset_kwargs,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_ogr_read(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer, PyObject *__pyx_v_encoding, int __pyx_v_read_geometry, int __pyx_v_force_2d, PyObject *__pyx_v_columns, int __pyx_v_skip_features, int __pyx_v_max_features, PyObject *__pyx_v_where, PyObject *__pyx_v_bbox, PyObject *__pyx_v_mask, PyObject *__pyx_v_fids, PyObject *__pyx_v_sql, PyObject *__pyx_v_sql_dialect, int __pyx_v_return_fids, int __pyx_v_datetime_as_string) {
  int __pyx_v_use_tmp_vsimem;
  char **__pyx_v_dataset_options;
  char const *__pyx_v_field_c;
  char **__pyx_v_fields_c;
  OGRDataSourceH __pyx_v_ogr_dataset;
  OGRLayerH __pyx_v_ogr_layer;
  char const *__pyx_v_prev_shape_encoding;
  int __pyx_v_override_shape_encoding;
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_crs = NULL;
  PyObject *__pyx_v_fields = NULL;
  PyObject *__pyx_v_ignored_fields = NULL;
  PyObject *__pyx_v_idx = NULL;
  PyObject *__pyx_v_field = NULL;
  PyObject *__pyx_v_field_b = NULL;
  PyObject *__pyx_v_geometry_type = NULL;
  PyObject *__pyx_v_geometries = NULL;
  PyObject *__pyx_v_field_data = NULL;
  PyObject *__pyx_v_fid_data = NULL;
  PyObject *__pyx_v_num_features = NULL;
  PyObject *__pyx_v_ogr_types = NULL;
  PyObject *__pyx_v_ogr_subtypes = NULL;
  PyObject *__pyx_v_meta = NULL;
  PyObject *__pyx_8genexpr5__pyx_v_field = NULL;
  PyObject *__pyx_8genexpr6__pyx_v_field = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  char const *__pyx_t_10;
  char **__pyx_t_11;
  char const *__pyx_t_12;
  void *__pyx_t_13;
  OGRLayerH __pyx_t_14;
  struct __pyx_opt_args_7pyogrio_3_io_execute_sql __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  char const *__pyx_t_20;
  __Pyx_memviewslice __pyx_t_21 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_22 = { 0, 0, { 0 }, { 0 }, { 0 } };
  uint8_t __pyx_t_23;
  PyObject *(*__pyx_t_24)(PyObject *);
  struct __pyx_opt_args_7pyogrio_3_io_validate_feature_range __pyx_t_25;
  int __pyx_t_26;
  PyObject *(*__pyx_t_27)(PyObject *);
  int __pyx_t_28;
  char const *__pyx_t_29;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  PyObject *__pyx_t_35 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_read", 0);
  __Pyx_INCREF(__pyx_v_layer);
  __Pyx_INCREF(__pyx_v_encoding);
  __Pyx_INCREF(__pyx_v_fids);

  /* "pyogrio/_io.pyx":1649
 * 
 *     """
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)             # <<<<<<<<<<<<<<
 *     cdef char **dataset_options = NULL
 *     cdef const char *field_c = NULL
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_path_or_buffer); 
  __pyx_v_use_tmp_vsimem = __pyx_t_1;

  /* "pyogrio/_io.pyx":1650
 *     """
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef char **dataset_options = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *field_c = NULL
 *     cdef char **fields_c = NULL
*/
  __pyx_v_dataset_options = NULL;

  /* "pyogrio/_io.pyx":1651
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef char **dataset_options = NULL
 *     cdef const char *field_c = NULL             # <<<<<<<<<<<<<<
 *     cdef char **fields_c = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
*/
  __pyx_v_field_c = NULL;

  /* "pyogrio/_io.pyx":1652
 *     cdef char **dataset_options = NULL
 *     cdef const char *field_c = NULL
 *     cdef char **fields_c = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
*/
  __pyx_v_fields_c = NULL;

  /* "pyogrio/_io.pyx":1653
 *     cdef const char *field_c = NULL
 *     cdef char **fields_c = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef const char *prev_shape_encoding = NULL
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":1654
 *     cdef char **fields_c = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *prev_shape_encoding = NULL
 *     cdef bint override_shape_encoding = False
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":1655
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef const char *prev_shape_encoding = NULL             # <<<<<<<<<<<<<<
 *     cdef bint override_shape_encoding = False
 * 
*/
  __pyx_v_prev_shape_encoding = NULL;

  /* "pyogrio/_io.pyx":1656
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef const char *prev_shape_encoding = NULL
 *     cdef bint override_shape_encoding = False             # <<<<<<<<<<<<<<
 * 
 *     if fids is not None:
*/
  __pyx_v_override_shape_encoding = 0;

  /* "pyogrio/_io.pyx":1658
 *     cdef bint override_shape_encoding = False
 * 
 *     if fids is not None:             # <<<<<<<<<<<<<<
 *         if (
 *             where is not None
*/
  __pyx_t_1 = (__pyx_v_fids != Py_None);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1660
 *     if fids is not None:
 *         if (
 *             where is not None             # <<<<<<<<<<<<<<
 *             or bbox is not None
 *             or mask is not None
*/
    __pyx_t_2 = (__pyx_v_where != Py_None);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1661
 *         if (
 *             where is not None
 *             or bbox is not None             # <<<<<<<<<<<<<<
 *             or mask is not None
 *             or sql is not None
*/
    __pyx_t_2 = (__pyx_v_bbox != ((PyObject*)Py_None));
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1662
 *             where is not None
 *             or bbox is not None
 *             or mask is not None             # <<<<<<<<<<<<<<
 *             or sql is not None
 *             or skip_features
*/
    __pyx_t_2 = (__pyx_v_mask != Py_None);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1663
 *             or bbox is not None
 *             or mask is not None
 *             or sql is not None             # <<<<<<<<<<<<<<
 *             or skip_features
 *             or max_features
*/
    __pyx_t_2 = (__pyx_v_sql != ((PyObject*)Py_None));
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1664
 *             or mask is not None
 *             or sql is not None
 *             or skip_features             # <<<<<<<<<<<<<<
 *             or max_features
 *         ):
*/
    __pyx_t_2 = (__pyx_v_skip_features != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1665
 *             or sql is not None
 *             or skip_features
 *             or max_features             # <<<<<<<<<<<<<<
 *         ):
 *             raise ValueError(
*/
    __pyx_t_2 = (__pyx_v_max_features != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L5_bool_binop_done:;

    /* "pyogrio/_io.pyx":1659
 * 
 *     if fids is not None:
 *         if (             # <<<<<<<<<<<<<<
 *             where is not None
 *             or bbox is not None
*/
    if (unlikely(__pyx_t_1)) {

      /* "pyogrio/_io.pyx":1667
 *             or max_features
 *         ):
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "cannot set both 'fids' and any of 'where', 'bbox', 'mask', "
 *                 "'sql', 'skip_features' or 'max_features'"
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_cannot_set_both_fids_and_any_of};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1667, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 1667, __pyx_L1_error)

      /* "pyogrio/_io.pyx":1659
 * 
 *     if fids is not None:
 *         if (             # <<<<<<<<<<<<<<
 *             where is not None
 *             or bbox is not None
*/
    }

    /* "pyogrio/_io.pyx":1671
 *                 "'sql', 'skip_features' or 'max_features'"
 *             )
 *         fids = np.asarray(fids, dtype=np.intc)             # <<<<<<<<<<<<<<
 * 
 *     if sql is not None and layer is not None:
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_asarray); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_intc); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, __pyx_v_fids};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1671, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_8, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1671, __pyx_L1_error)
      __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1671, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF_SET(__pyx_v_fids, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":1658
 *     cdef bint override_shape_encoding = False
 * 
 *     if fids is not None:             # <<<<<<<<<<<<<<
 *         if (
 *             where is not None
*/
  }

  /* "pyogrio/_io.pyx":1673
 *         fids = np.asarray(fids, dtype=np.intc)
 * 
 *     if sql is not None and layer is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
*/
  __pyx_t_2 = (__pyx_v_sql != ((PyObject*)Py_None));
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_layer != Py_None);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":1674
 * 
 *     if sql is not None and layer is not None:
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")             # <<<<<<<<<<<<<<
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):
*/
    __pyx_t_7 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_sql_parameter_cannot_be_combine};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1674, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1674, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1673
 *         fids = np.asarray(fids, dtype=np.intc)
 * 
 *     if sql is not None and layer is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1676
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "at least one of read_geometry or return_fids must be True or columns must "
*/
  __pyx_t_2 = (__pyx_v_read_geometry != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_return_fids != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_columns == Py_None);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_9 = PyObject_Length(__pyx_v_columns); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1676, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_9 > 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":1677
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "at least one of read_geometry or return_fids must be True or columns must "
 *             "be None or non-empty"
*/
    __pyx_t_7 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_at_least_one_of_read_geometry_or};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1677, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1677, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1676
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "at least one of read_geometry or return_fids must be True or columns must "
*/
  }

  /* "pyogrio/_io.pyx":1682
 *         )
 * 
 *     if bbox and mask:             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
*/
  if (__pyx_v_bbox == Py_None) __pyx_t_1 = 0;
  else
  {
    Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_v_bbox);
    if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 1682, __pyx_L1_error)
    __pyx_t_1 = (__pyx_temp != 0);
  }

  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_mask); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1682, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __pyx_L20_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":1683
 * 
 *     if bbox and mask:
 *         raise ValueError("cannot set both 'bbox' and 'mask'")             # <<<<<<<<<<<<<<
 * 
 *     if skip_features < 0:
*/
    __pyx_t_7 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_cannot_set_both_bbox_and_mask};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1683, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1683, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1682
 *         )
 * 
 *     if bbox and mask:             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1685
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
 *     if skip_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'skip_features' must be >= 0")
 * 
*/
  __pyx_t_2 = (__pyx_v_skip_features < 0);
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":1686
 * 
 *     if skip_features < 0:
 *         raise ValueError("'skip_features' must be >= 0")             # <<<<<<<<<<<<<<
 * 
 *     if max_features < 0:
*/
    __pyx_t_7 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_skip_features_must_be_0};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1686, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1686, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1685
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
 *     if skip_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'skip_features' must be >= 0")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1688
 *         raise ValueError("'skip_features' must be >= 0")
 * 
 *     if max_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'max_features' must be >= 0")
 * 
*/
  __pyx_t_2 = (__pyx_v_max_features < 0);
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":1689
 * 
 *     if max_features < 0:
 *         raise ValueError("'max_features' must be >= 0")             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
    __pyx_t_7 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_max_features_must_be_0};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1689, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1689, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1688
 *         raise ValueError("'skip_features' must be >= 0")
 * 
 *     if max_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'max_features' must be >= 0")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1691
 *         raise ValueError("'max_features' must be >= 0")
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":1694
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 * 
 *         if encoding:
*/
    if (__pyx_v_use_tmp_vsimem) {

      /* "pyogrio/_io.pyx":1693
 *     try:
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer             # <<<<<<<<<<<<<<
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
*/
      __pyx_t_7 = __pyx_v_path_or_buffer;
      __Pyx_INCREF(__pyx_t_7);
      if (!(likely(PyBytes_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_7))) __PYX_ERR(0, 1693, __pyx_L25_error)

      /* "pyogrio/_io.pyx":1692
 * 
 *     try:
 *         path = read_buffer_to_vsimem(             # <<<<<<<<<<<<<<
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer
*/
      __pyx_t_6 = __pyx_f_7pyogrio_4_vsi_read_buffer_to_vsimem(((PyObject*)__pyx_t_7)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1692, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_3 = __pyx_t_6;
      __pyx_t_6 = 0;
    } else {

      /* "pyogrio/_io.pyx":1694
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 * 
 *         if encoding:
*/
      __Pyx_INCREF(__pyx_v_path_or_buffer);
      __pyx_t_3 = __pyx_v_path_or_buffer;
    }
    __pyx_v_path = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":1696
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
 *         if encoding:             # <<<<<<<<<<<<<<
 *             # for shapefiles, SHAPE_ENCODING must be set before opening the file
 *             # to prevent automatic decoding to UTF-8 by GDAL, so we save previous
*/
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1696, __pyx_L25_error)
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":1702
 *             # (we do this for all data sources where encoding is set because
 *             # we don't know the driver until after it is opened, which is too late)
 *             override_shape_encoding = True             # <<<<<<<<<<<<<<
 *             prev_shape_encoding = override_threadlocal_config_option(
 *                 "SHAPE_ENCODING", encoding
*/
      __pyx_v_override_shape_encoding = 1;

      /* "pyogrio/_io.pyx":1704
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(
 *                 "SHAPE_ENCODING", encoding             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_3 = __pyx_v_encoding;
      __Pyx_INCREF(__pyx_t_3);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 1704, __pyx_L25_error)

      /* "pyogrio/_io.pyx":1703
 *             # we don't know the driver until after it is opened, which is too late)
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(             # <<<<<<<<<<<<<<
 *                 "SHAPE_ENCODING", encoding
 *             )
*/
      __pyx_t_10 = __pyx_f_7pyogrio_3_io_override_threadlocal_config_option(__pyx_mstate_global->__pyx_n_u_SHAPE_ENCODING, ((PyObject*)__pyx_t_3)); if (unlikely(__pyx_t_10 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 1703, __pyx_L25_error)
      __pyx_v_prev_shape_encoding = __pyx_t_10;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyogrio/_io.pyx":1696
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
 *         if encoding:             # <<<<<<<<<<<<<<
 *             # for shapefiles, SHAPE_ENCODING must be set before opening the file
 *             # to prevent automatic decoding to UTF-8 by GDAL, so we save previous
*/
    }

    /* "pyogrio/_io.pyx":1707
 *             )
 * 
 *         dataset_options = dict_to_options(dataset_kwargs)             # <<<<<<<<<<<<<<
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
*/
    __pyx_t_11 = __pyx_f_7pyogrio_3_io_dict_to_options(__pyx_v_dataset_kwargs); if (unlikely(__pyx_t_11 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 1707, __pyx_L25_error)
    __pyx_v_dataset_options = __pyx_t_11;

    /* "pyogrio/_io.pyx":1708
 * 
 *         dataset_options = dict_to_options(dataset_kwargs)
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)             # <<<<<<<<<<<<<<
 * 
 *         if sql is None:
*/
    __pyx_t_6 = __pyx_v_path;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1708, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_12 = __Pyx_PyObject_AsString(__pyx_t_3); if (unlikely((!__pyx_t_12) && PyErr_Occurred())) __PYX_ERR(0, 1708, __pyx_L25_error)
    __pyx_t_13 = __pyx_f_7pyogrio_3_io_ogr_open(__pyx_t_12, 0, __pyx_v_dataset_options); if (unlikely(__pyx_t_13 == ((void *)NULL))) __PYX_ERR(0, 1708, __pyx_L25_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_ogr_dataset = __pyx_t_13;

    /* "pyogrio/_io.pyx":1710
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
 *         if sql is None:             # <<<<<<<<<<<<<<
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)
*/
    __pyx_t_2 = (__pyx_v_sql == ((PyObject*)Py_None));
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":1711
 * 
 *         if sql is None:
 *             if layer is None:             # <<<<<<<<<<<<<<
 *                 layer = get_default_layer(ogr_dataset)
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
*/
      __pyx_t_2 = (__pyx_v_layer == Py_None);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1712
 *         if sql is None:
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
 *         else:
*/
        __pyx_t_3 = __pyx_f_7pyogrio_3_io_get_default_layer(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1712, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF_SET(__pyx_v_layer, __pyx_t_3);
        __pyx_t_3 = 0;

        /* "pyogrio/_io.pyx":1711
 * 
 *         if sql is None:
 *             if layer is None:             # <<<<<<<<<<<<<<
 *                 layer = get_default_layer(ogr_dataset)
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
*/
      }

      /* "pyogrio/_io.pyx":1713
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)             # <<<<<<<<<<<<<<
 *         else:
 *             ogr_layer = execute_sql(ogr_dataset, sql, sql_dialect)
*/
      __pyx_t_14 = __pyx_f_7pyogrio_3_io_get_ogr_layer(__pyx_v_ogr_dataset, __pyx_v_layer); if (unlikely(__pyx_t_14 == ((void *)NULL))) __PYX_ERR(0, 1713, __pyx_L25_error)
      __pyx_v_ogr_layer = __pyx_t_14;

      /* "pyogrio/_io.pyx":1710
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
 *         if sql is None:             # <<<<<<<<<<<<<<
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)
*/
      goto __pyx_L28;
    }

    /* "pyogrio/_io.pyx":1715
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
 *         else:
 *             ogr_layer = execute_sql(ogr_dataset, sql, sql_dialect)             # <<<<<<<<<<<<<<
 * 
 *         crs = get_crs(ogr_layer)
*/
    /*else*/ {
      __pyx_t_15.__pyx_n = 1;
      __pyx_t_15.sql_dialect = __pyx_v_sql_dialect;
      __pyx_t_14 = __pyx_f_7pyogrio_3_io_execute_sql(__pyx_v_ogr_dataset, __pyx_v_sql, &__pyx_t_15); if (unlikely(__pyx_t_14 == ((void *)NULL))) __PYX_ERR(0, 1715, __pyx_L25_error)
      __pyx_v_ogr_layer = __pyx_t_14;
    }
    __pyx_L28:;

    /* "pyogrio/_io.pyx":1717
 *             ogr_layer = execute_sql(ogr_dataset, sql, sql_dialect)
 * 
 *         crs = get_crs(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         # Encoding is derived from the user, from the dataset capabilities / type,
*/
    __pyx_t_3 = __pyx_f_7pyogrio_3_io_get_crs(__pyx_v_ogr_layer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1717, __pyx_L25_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_crs = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":1721
 *         # Encoding is derived from the user, from the dataset capabilities / type,
 *         # or from the system locale
 *         if encoding:             # <<<<<<<<<<<<<<
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":
 *                 # NOTE: SHAPE_ENCODING is a configuration option whereas ENCODING is the
*/
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1721, __pyx_L25_error)
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":1722
 *         # or from the system locale
 *         if encoding:
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *                 # NOTE: SHAPE_ENCODING is a configuration option whereas ENCODING is the
 *                 # dataset open option
*/
      __pyx_t_3 = __pyx_f_7pyogrio_3_io_get_driver(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1722, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1722, __pyx_L25_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1725
 *                 # NOTE: SHAPE_ENCODING is a configuration option whereas ENCODING is the
 *                 # dataset open option
 *                 if "ENCODING" in dataset_kwargs:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'cannot provide both encoding parameter and "ENCODING" option; '
*/
        __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_mstate_global->__pyx_n_u_ENCODING, __pyx_v_dataset_kwargs, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 1725, __pyx_L25_error)
        if (unlikely(__pyx_t_2)) {

          /* "pyogrio/_io.pyx":1726
 *                 # dataset open option
 *                 if "ENCODING" in dataset_kwargs:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         'cannot provide both encoding parameter and "ENCODING" option; '
 *                         "use encoding parameter to specify correct encoding for data "
*/
          __pyx_t_6 = NULL;
          __pyx_t_5 = 1;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_cannot_provide_both_encoding_par};
            __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1726, __pyx_L25_error)
            __Pyx_GOTREF(__pyx_t_3);
          }
          __Pyx_Raise(__pyx_t_3, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __PYX_ERR(0, 1726, __pyx_L25_error)

          /* "pyogrio/_io.pyx":1725
 *                 # NOTE: SHAPE_ENCODING is a configuration option whereas ENCODING is the
 *                 # dataset open option
 *                 if "ENCODING" in dataset_kwargs:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'cannot provide both encoding parameter and "ENCODING" option; '
*/
        }

        /* "pyogrio/_io.pyx":1734
 *                 # Because SHAPE_ENCODING is set above, GDAL will automatically
 *                 # decode shapefiles to UTF-8; ignore any encoding set by user
 *                 encoding = "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *         else:
*/
        __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
        __Pyx_DECREF_SET(__pyx_v_encoding, __pyx_mstate_global->__pyx_kp_u_UTF_8);

        /* "pyogrio/_io.pyx":1722
 *         # or from the system locale
 *         if encoding:
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *                 # NOTE: SHAPE_ENCODING is a configuration option whereas ENCODING is the
 *                 # dataset open option
*/
      }

      /* "pyogrio/_io.pyx":1721
 *         # Encoding is derived from the user, from the dataset capabilities / type,
 *         # or from the system locale
 *         if encoding:             # <<<<<<<<<<<<<<
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":
 *                 # NOTE: SHAPE_ENCODING is a configuration option whereas ENCODING is the
*/
      goto __pyx_L30;
    }

    /* "pyogrio/_io.pyx":1737
 * 
 *         else:
 *             encoding = detect_encoding(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         fields = get_fields(ogr_layer, encoding)
*/
    /*else*/ {
      __pyx_t_3 = __pyx_f_7pyogrio_3_io_detect_encoding(__pyx_v_ogr_dataset, __pyx_v_ogr_layer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1737, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_encoding, __pyx_t_3);
      __pyx_t_3 = 0;
    }
    __pyx_L30:;

    /* "pyogrio/_io.pyx":1739
 *             encoding = detect_encoding(ogr_dataset, ogr_layer)
 * 
 *         fields = get_fields(ogr_layer, encoding)             # <<<<<<<<<<<<<<
 * 
 *         ignored_fields = []
*/
    __pyx_t_3 = __pyx_v_encoding;
    __Pyx_INCREF(__pyx_t_3);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 1739, __pyx_L25_error)
    __pyx_t_6 = __pyx_f_7pyogrio_3_io_get_fields(__pyx_v_ogr_layer, ((PyObject*)__pyx_t_3), NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1739, __pyx_L25_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_fields = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "pyogrio/_io.pyx":1741
 *         fields = get_fields(ogr_layer, encoding)
 * 
 *         ignored_fields = []             # <<<<<<<<<<<<<<
 *         if columns is not None:
 *             # identify ignored fields first
*/
    __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1741, __pyx_L25_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_ignored_fields = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "pyogrio/_io.pyx":1742
 * 
 *         ignored_fields = []
 *         if columns is not None:             # <<<<<<<<<<<<<<
 *             # identify ignored fields first
 *             ignored_fields = list(set(fields[:, 2]) - set(columns))
*/
    __pyx_t_2 = (__pyx_v_columns != Py_None);
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":1744
 *         if columns is not None:
 *             # identify ignored fields first
 *             ignored_fields = list(set(fields[:, 2]) - set(columns))             # <<<<<<<<<<<<<<
 * 
 *             # Fields are matched exactly by name, duplicates are dropped.
*/
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[5]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1744, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = PySet_New(__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1744, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PySet_New(__pyx_v_columns); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1744, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = PyNumber_Subtract(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1744, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PySequence_ListKeepNew(__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1744, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF_SET(__pyx_v_ignored_fields, ((PyObject*)__pyx_t_6));
      __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":1748
 *             # Fields are matched exactly by name, duplicates are dropped.
 *             # Find index of each field into fields
 *             idx = np.sort(np.intersect1d(fields[:, 2], columns, return_indices=True)[1])             # <<<<<<<<<<<<<<
 *             fields = fields[idx, :]
 * 
*/
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1748, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_sort); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1748, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_4 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1748, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_intersect1d); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1748, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_17);
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __pyx_t_16 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[5]); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1748, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_16);
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_17))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_17);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_17);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_17, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, __pyx_t_16, __pyx_v_columns};
        __pyx_t_18 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1748, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_18);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_return_indices, Py_True, __pyx_t_18, __pyx_callargs+3, 0) < (0)) __PYX_ERR(0, 1748, __pyx_L25_error)
        __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_17, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_18);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1748, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __pyx_t_17 = __Pyx_GetItemInt(__pyx_t_3, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1748, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_17);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_17};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1748, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __pyx_v_idx = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":1749
 *             # Find index of each field into fields
 *             idx = np.sort(np.intersect1d(fields[:, 2], columns, return_indices=True)[1])
 *             fields = fields[idx, :]             # <<<<<<<<<<<<<<
 * 
 *         if not read_geometry and bbox is None and mask is None:
*/
      __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1749, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_idx);
      __Pyx_GIVEREF(__pyx_v_idx);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_idx) != (0)) __PYX_ERR(0, 1749, __pyx_L25_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[0]);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_mstate_global->__pyx_slice[0]) != (0)) __PYX_ERR(0, 1749, __pyx_L25_error);
      __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1749, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF_SET(__pyx_v_fields, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":1742
 * 
 *         ignored_fields = []
 *         if columns is not None:             # <<<<<<<<<<<<<<
 *             # identify ignored fields first
 *             ignored_fields = list(set(fields[:, 2]) - set(columns))
*/
    }

    /* "pyogrio/_io.pyx":1751
 *             fields = fields[idx, :]
 * 
 *         if not read_geometry and bbox is None and mask is None:             # <<<<<<<<<<<<<<
 *             ignored_fields.append("OGR_GEOMETRY")
 * 
*/
    __pyx_t_1 = (!(__pyx_v_read_geometry != 0));
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L35_bool_binop_done;
    }
    __pyx_t_1 = (__pyx_v_bbox == ((PyObject*)Py_None));
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L35_bool_binop_done;
    }
    __pyx_t_1 = (__pyx_v_mask == Py_None);
    __pyx_t_2 = __pyx_t_1;
    __pyx_L35_bool_binop_done:;
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":1752
 * 
 *         if not read_geometry and bbox is None and mask is None:
 *             ignored_fields.append("OGR_GEOMETRY")             # <<<<<<<<<<<<<<
 * 
 *         # Instruct GDAL to ignore reading fields not
*/
      __pyx_t_19 = __Pyx_PyList_Append(__pyx_v_ignored_fields, __pyx_mstate_global->__pyx_n_u_OGR_GEOMETRY); if (unlikely(__pyx_t_19 == ((int)-1))) __PYX_ERR(0, 1752, __pyx_L25_error)

      /* "pyogrio/_io.pyx":1751
 *             fields = fields[idx, :]
 * 
 *         if not read_geometry and bbox is None and mask is None:             # <<<<<<<<<<<<<<
 *             ignored_fields.append("OGR_GEOMETRY")
 * 
*/
    }

    /* "pyogrio/_io.pyx":1756
 *         # Instruct GDAL to ignore reading fields not
 *         # included in output columns for faster I/O
 *         if ignored_fields:             # <<<<<<<<<<<<<<
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")
*/
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_v_ignored_fields);
      if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 1756, __pyx_L25_error)
      __pyx_t_2 = (__pyx_temp != 0);
    }

    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":1757
 *         # included in output columns for faster I/O
 *         if ignored_fields:
 *             for field in ignored_fields:             # <<<<<<<<<<<<<<
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b
*/
      __pyx_t_8 = __pyx_v_ignored_fields; __Pyx_INCREF(__pyx_t_8);
      __pyx_t_9 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1757, __pyx_L25_error)
          #endif
          if (__pyx_t_9 >= __pyx_temp) break;
        }
        __pyx_t_6 = __Pyx_PyList_GetItemRefFast(__pyx_t_8, __pyx_t_9, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_9;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1757, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_v_field, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "pyogrio/_io.pyx":1758
 *         if ignored_fields:
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")             # <<<<<<<<<<<<<<
 *                 field_c = field_b
 *                 fields_c = CSLAddString(fields_c, field_c)
*/
        __pyx_t_17 = __pyx_v_field;
        __Pyx_INCREF(__pyx_t_17);
        __pyx_t_5 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_mstate_global->__pyx_kp_u_utf_8};
          __pyx_t_6 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1758, __pyx_L25_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_XDECREF_SET(__pyx_v_field_b, __pyx_t_6);
        __pyx_t_6 = 0;

        /* "pyogrio/_io.pyx":1759
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b             # <<<<<<<<<<<<<<
 *                 fields_c = CSLAddString(fields_c, field_c)
 * 
*/
        __pyx_t_20 = __Pyx_PyObject_AsString(__pyx_v_field_b); if (unlikely((!__pyx_t_20) && PyErr_Occurred())) __PYX_ERR(0, 1759, __pyx_L25_error)
        __pyx_v_field_c = __pyx_t_20;

        /* "pyogrio/_io.pyx":1760
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b
 *                 fields_c = CSLAddString(fields_c, field_c)             # <<<<<<<<<<<<<<
 * 
 *             with nogil:
*/
        __pyx_v_fields_c = CSLAddString(__pyx_v_fields_c, __pyx_v_field_c);

        /* "pyogrio/_io.pyx":1757
 *         # included in output columns for faster I/O
 *         if ignored_fields:
 *             for field in ignored_fields:             # <<<<<<<<<<<<<<
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b
*/
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":1762
 *                 fields_c = CSLAddString(fields_c, field_c)
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)
 * 
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "pyogrio/_io.pyx":1763
 * 
 *             with nogil:
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)             # <<<<<<<<<<<<<<
 * 
 *         geometry_type = get_geometry_type(ogr_layer)
*/
            (void)(OGR_L_SetIgnoredFields(__pyx_v_ogr_layer, ((char const **)__pyx_v_fields_c)));
          }

          /* "pyogrio/_io.pyx":1762
 *                 fields_c = CSLAddString(fields_c, field_c)
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)
 * 
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L44;
            }
            __pyx_L44:;
          }
      }

      /* "pyogrio/_io.pyx":1756
 *         # Instruct GDAL to ignore reading fields not
 *         # included in output columns for faster I/O
 *         if ignored_fields:             # <<<<<<<<<<<<<<
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")
*/
    }

    /* "pyogrio/_io.pyx":1765
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)
 * 
 *         geometry_type = get_geometry_type(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         if fids is not None:
*/
    __pyx_t_8 = __pyx_f_7pyogrio_9_geometry_get_geometry_type(__pyx_v_ogr_layer); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1765, __pyx_L25_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_geometry_type = ((PyObject*)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":1767
 *         geometry_type = get_geometry_type(ogr_layer)
 * 
 *         if fids is not None:             # <<<<<<<<<<<<<<
 *             geometries, field_data = get_features_by_fid(
 *                 ogr_layer,
*/
    __pyx_t_2 = (__pyx_v_fids != Py_None);
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":1770
 *             geometries, field_data = get_features_by_fid(
 *                 ogr_layer,
 *                 fids,             # <<<<<<<<<<<<<<
 *                 fields,
 *                 encoding,
*/
      __pyx_t_21 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_v_fids, PyBUF_WRITABLE); if (unlikely(!__pyx_t_21.memview)) __PYX_ERR(0, 1770, __pyx_L25_error)

      /* "pyogrio/_io.pyx":1771
 *                 ogr_layer,
 *                 fids,
 *                 fields,             # <<<<<<<<<<<<<<
 *                 encoding,
 *                 read_geometry=read_geometry and geometry_type is not None,
*/
      __pyx_t_22 = __Pyx_PyObject_to_MemoryviewSlice_dsds_object(__pyx_v_fields, PyBUF_WRITABLE); if (unlikely(!__pyx_t_22.memview)) __PYX_ERR(0, 1771, __pyx_L25_error)

      /* "pyogrio/_io.pyx":1773
 *                 fields,
 *                 encoding,
 *                 read_geometry=read_geometry and geometry_type is not None,             # <<<<<<<<<<<<<<
 *                 force_2d=force_2d,
 *                 datetime_as_string=datetime_as_string,
*/
      if (__pyx_v_read_geometry) {
      } else {
        __pyx_t_23 = __pyx_v_read_geometry;
        goto __pyx_L46_bool_binop_done;
      }
      __pyx_t_2 = (__pyx_v_geometry_type != ((PyObject*)Py_None));
      __pyx_t_23 = __pyx_t_2;
      __pyx_L46_bool_binop_done:;

      /* "pyogrio/_io.pyx":1768
 * 
 *         if fids is not None:
 *             geometries, field_data = get_features_by_fid(             # <<<<<<<<<<<<<<
 *                 ogr_layer,
 *                 fids,
*/
      __pyx_t_8 = __pyx_f_7pyogrio_3_io_get_features_by_fid(__pyx_v_ogr_layer, __pyx_t_21, __pyx_t_22, __pyx_v_encoding, __pyx_t_23, __pyx_v_force_2d, __pyx_v_datetime_as_string); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1768, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_8);
      __PYX_XCLEAR_MEMVIEW(&__pyx_t_21, 1);
      __pyx_t_21.memview = NULL; __pyx_t_21.data = NULL;
      __PYX_XCLEAR_MEMVIEW(&__pyx_t_22, 1);
      __pyx_t_22.memview = NULL; __pyx_t_22.data = NULL;
      if ((likely(PyTuple_CheckExact(__pyx_t_8))) || (PyList_CheckExact(__pyx_t_8))) {
        PyObject* sequence = __pyx_t_8;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1768, __pyx_L25_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_6);
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_17);
        } else {
          __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1768, __pyx_L25_error)
          __Pyx_XGOTREF(__pyx_t_6);
          __pyx_t_17 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1768, __pyx_L25_error)
          __Pyx_XGOTREF(__pyx_t_17);
        }
        #else
        __pyx_t_6 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1768, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_17 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1768, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_7 = PyObject_GetIter(__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1768, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_24 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7);
        index = 0; __pyx_t_6 = __pyx_t_24(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L48_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        index = 1; __pyx_t_17 = __pyx_t_24(__pyx_t_7); if (unlikely(!__pyx_t_17)) goto __pyx_L48_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_24(__pyx_t_7), 2) < (0)) __PYX_ERR(0, 1768, __pyx_L25_error)
        __pyx_t_24 = NULL;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L49_unpacking_done;
        __pyx_L48_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_24 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1768, __pyx_L25_error)
        __pyx_L49_unpacking_done:;
      }
      __pyx_v_geometries = __pyx_t_6;
      __pyx_t_6 = 0;
      __pyx_v_field_data = __pyx_t_17;
      __pyx_t_17 = 0;

      /* "pyogrio/_io.pyx":1779
 * 
 *             # bypass reading fids since these should match fids used for read
 *             if return_fids:             # <<<<<<<<<<<<<<
 *                 fid_data = fids.astype(np.int64)
 *             else:
*/
      __pyx_t_2 = (__pyx_v_return_fids != 0);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1780
 *             # bypass reading fids since these should match fids used for read
 *             if return_fids:
 *                 fid_data = fids.astype(np.int64)             # <<<<<<<<<<<<<<
 *             else:
 *                 fid_data = None
*/
        __pyx_t_17 = __pyx_v_fids;
        __Pyx_INCREF(__pyx_t_17);
        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1780, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1780, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_5 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_t_7};
          __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1780, __pyx_L25_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __pyx_v_fid_data = __pyx_t_8;
        __pyx_t_8 = 0;

        /* "pyogrio/_io.pyx":1779
 * 
 *             # bypass reading fids since these should match fids used for read
 *             if return_fids:             # <<<<<<<<<<<<<<
 *                 fid_data = fids.astype(np.int64)
 *             else:
*/
        goto __pyx_L50;
      }

      /* "pyogrio/_io.pyx":1782
 *                 fid_data = fids.astype(np.int64)
 *             else:
 *                 fid_data = None             # <<<<<<<<<<<<<<
 *         else:
 *             # Apply the attribute filter
*/
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_fid_data = Py_None;
      }
      __pyx_L50:;

      /* "pyogrio/_io.pyx":1767
 *         geometry_type = get_geometry_type(ogr_layer)
 * 
 *         if fids is not None:             # <<<<<<<<<<<<<<
 *             geometries, field_data = get_features_by_fid(
 *                 ogr_layer,
*/
      goto __pyx_L45;
    }

    /* "pyogrio/_io.pyx":1785
 *         else:
 *             # Apply the attribute filter
 *             if where is not None and where != "":             # <<<<<<<<<<<<<<
 *                 apply_where_filter(ogr_layer, where)
 * 
*/
    /*else*/ {
      __pyx_t_1 = (__pyx_v_where != Py_None);
      if (__pyx_t_1) {
      } else {
        __pyx_t_2 = __pyx_t_1;
        goto __pyx_L52_bool_binop_done;
      }
      __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_where, __pyx_mstate_global->__pyx_kp_u__9, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1785, __pyx_L25_error)
      __pyx_t_2 = __pyx_t_1;
      __pyx_L52_bool_binop_done:;
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1786
 *             # Apply the attribute filter
 *             if where is not None and where != "":
 *                 apply_where_filter(ogr_layer, where)             # <<<<<<<<<<<<<<
 * 
 *             # Apply the spatial filter
*/
        __pyx_t_8 = __pyx_v_where;
        __Pyx_INCREF(__pyx_t_8);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_8))) __PYX_ERR(0, 1786, __pyx_L25_error)
        __pyx_t_7 = __pyx_f_7pyogrio_3_io_apply_where_filter(__pyx_v_ogr_layer, ((PyObject*)__pyx_t_8)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1786, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":1785
 *         else:
 *             # Apply the attribute filter
 *             if where is not None and where != "":             # <<<<<<<<<<<<<<
 *                 apply_where_filter(ogr_layer, where)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1789
 * 
 *             # Apply the spatial filter
 *             if bbox is not None:             # <<<<<<<<<<<<<<
 *                 apply_bbox_filter(ogr_layer, bbox)
 * 
*/
      __pyx_t_2 = (__pyx_v_bbox != ((PyObject*)Py_None));
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1790
 *             # Apply the spatial filter
 *             if bbox is not None:
 *                 apply_bbox_filter(ogr_layer, bbox)             # <<<<<<<<<<<<<<
 * 
 *             elif mask is not None:
*/
        __pyx_t_7 = __pyx_f_7pyogrio_3_io_apply_bbox_filter(__pyx_v_ogr_layer, __pyx_v_bbox); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1790, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":1789
 * 
 *             # Apply the spatial filter
 *             if bbox is not None:             # <<<<<<<<<<<<<<
 *                 apply_bbox_filter(ogr_layer, bbox)
 * 
*/
        goto __pyx_L54;
      }

      /* "pyogrio/_io.pyx":1792
 *                 apply_bbox_filter(ogr_layer, bbox)
 * 
 *             elif mask is not None:             # <<<<<<<<<<<<<<
 *                 apply_geometry_filter(ogr_layer, mask)
 * 
*/
      __pyx_t_2 = (__pyx_v_mask != Py_None);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1793
 * 
 *             elif mask is not None:
 *                 apply_geometry_filter(ogr_layer, mask)             # <<<<<<<<<<<<<<
 * 
 *             # Limit feature range to available range, but avoid doing an actual count if
*/
        __pyx_t_7 = __pyx_f_7pyogrio_3_io_apply_geometry_filter(__pyx_v_ogr_layer, __pyx_v_mask); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1793, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":1792
 *                 apply_bbox_filter(ogr_layer, bbox)
 * 
 *             elif mask is not None:             # <<<<<<<<<<<<<<
 *                 apply_geometry_filter(ogr_layer, mask)
 * 
*/
      }
      __pyx_L54:;

      /* "pyogrio/_io.pyx":1797
 *             # Limit feature range to available range, but avoid doing an actual count if
 *             # not needed (with num_features=-1, we will read in chunks in get_features)
 *             num_features = -1             # <<<<<<<<<<<<<<
 *             if max_features > 0:
 *                 num_features = max_features
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_neg_1);
      __pyx_v_num_features = __pyx_mstate_global->__pyx_int_neg_1;

      /* "pyogrio/_io.pyx":1798
 *             # not needed (with num_features=-1, we will read in chunks in get_features)
 *             num_features = -1
 *             if max_features > 0:             # <<<<<<<<<<<<<<
 *                 num_features = max_features
 * 
*/
      __pyx_t_2 = (__pyx_v_max_features > 0);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1799
 *             num_features = -1
 *             if max_features > 0:
 *                 num_features = max_features             # <<<<<<<<<<<<<<
 * 
 *             # When skipping features, we need to validate the value against the
*/
        __pyx_t_7 = __Pyx_PyLong_From_int(__pyx_v_max_features); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1799, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF_SET(__pyx_v_num_features, __pyx_t_7);
        __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":1798
 *             # not needed (with num_features=-1, we will read in chunks in get_features)
 *             num_features = -1
 *             if max_features > 0:             # <<<<<<<<<<<<<<
 *                 num_features = max_features
 * 
*/
      }

      /* "pyogrio/_io.pyx":1803
 *             # When skipping features, we need to validate the value against the
 *             # actual count to avoid out-of-bound index in apply_skip_features
 *             if skip_features > 0:             # <<<<<<<<<<<<<<
 *                 skip_features, num_features = validate_feature_range(
 *                     ogr_layer, skip_features, max_features
*/
      __pyx_t_2 = (__pyx_v_skip_features > 0);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1804
 *             # actual count to avoid out-of-bound index in apply_skip_features
 *             if skip_features > 0:
 *                 skip_features, num_features = validate_feature_range(             # <<<<<<<<<<<<<<
 *                     ogr_layer, skip_features, max_features
 *                 )
*/
        __pyx_t_25.__pyx_n = 2;
        __pyx_t_25.skip_features = __pyx_v_skip_features;
        __pyx_t_25.max_features = __pyx_v_max_features;
        __pyx_t_7 = __pyx_f_7pyogrio_3_io_validate_feature_range(__pyx_v_ogr_layer, &__pyx_t_25); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1804, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
          PyObject* sequence = __pyx_t_7;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 1804, __pyx_L25_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0);
            __Pyx_INCREF(__pyx_t_8);
            __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1);
            __Pyx_INCREF(__pyx_t_17);
          } else {
            __pyx_t_8 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1804, __pyx_L25_error)
            __Pyx_XGOTREF(__pyx_t_8);
            __pyx_t_17 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1804, __pyx_L25_error)
            __Pyx_XGOTREF(__pyx_t_17);
          }
          #else
          __pyx_t_8 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1804, __pyx_L25_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_17 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1804, __pyx_L25_error)
          __Pyx_GOTREF(__pyx_t_17);
          #endif
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_6 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1804, __pyx_L25_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_24 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6);
          index = 0; __pyx_t_8 = __pyx_t_24(__pyx_t_6); if (unlikely(!__pyx_t_8)) goto __pyx_L57_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_8);
          index = 1; __pyx_t_17 = __pyx_t_24(__pyx_t_6); if (unlikely(!__pyx_t_17)) goto __pyx_L57_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_17);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_24(__pyx_t_6), 2) < (0)) __PYX_ERR(0, 1804, __pyx_L25_error)
          __pyx_t_24 = NULL;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          goto __pyx_L58_unpacking_done;
          __pyx_L57_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_24 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 1804, __pyx_L25_error)
          __pyx_L58_unpacking_done:;
        }
        __pyx_t_26 = __Pyx_PyLong_As_int(__pyx_t_8); if (unlikely((__pyx_t_26 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1804, __pyx_L25_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_skip_features = __pyx_t_26;
        __Pyx_DECREF_SET(__pyx_v_num_features, __pyx_t_17);
        __pyx_t_17 = 0;

        /* "pyogrio/_io.pyx":1803
 *             # When skipping features, we need to validate the value against the
 *             # actual count to avoid out-of-bound index in apply_skip_features
 *             if skip_features > 0:             # <<<<<<<<<<<<<<
 *                 skip_features, num_features = validate_feature_range(
 *                     ogr_layer, skip_features, max_features
*/
      }

      /* "pyogrio/_io.pyx":1811
 *             # determine the feature count already which avoids chunking in get_features.
 *             if (
 *                 where is None             # <<<<<<<<<<<<<<
 *                 and bbox is None
 *                 and mask is None
*/
      __pyx_t_1 = (__pyx_v_where == Py_None);
      if (__pyx_t_1) {
      } else {
        __pyx_t_2 = __pyx_t_1;
        goto __pyx_L60_bool_binop_done;
      }

      /* "pyogrio/_io.pyx":1812
 *             if (
 *                 where is None
 *                 and bbox is None             # <<<<<<<<<<<<<<
 *                 and mask is None
 *                 and skip_features == 0
*/
      __pyx_t_1 = (__pyx_v_bbox == ((PyObject*)Py_None));
      if (__pyx_t_1) {
      } else {
        __pyx_t_2 = __pyx_t_1;
        goto __pyx_L60_bool_binop_done;
      }

      /* "pyogrio/_io.pyx":1813
 *                 where is None
 *                 and bbox is None
 *                 and mask is None             # <<<<<<<<<<<<<<
 *                 and skip_features == 0
 *                 and max_features == 0
*/
      __pyx_t_1 = (__pyx_v_mask == Py_None);
      if (__pyx_t_1) {
      } else {
        __pyx_t_2 = __pyx_t_1;
        goto __pyx_L60_bool_binop_done;
      }

      /* "pyogrio/_io.pyx":1814
 *                 and bbox is None
 *                 and mask is None
 *                 and skip_features == 0             # <<<<<<<<<<<<<<
 *                 and max_features == 0
 *                 and num_features < 0
*/
      __pyx_t_1 = (__pyx_v_skip_features == 0);
      if (__pyx_t_1) {
      } else {
        __pyx_t_2 = __pyx_t_1;
        goto __pyx_L60_bool_binop_done;
      }

      /* "pyogrio/_io.pyx":1815
 *                 and mask is None
 *                 and skip_features == 0
 *                 and max_features == 0             # <<<<<<<<<<<<<<
 *                 and num_features < 0
 *             ):
*/
      __pyx_t_1 = (__pyx_v_max_features == 0);
      if (__pyx_t_1) {
      } else {
        __pyx_t_2 = __pyx_t_1;
        goto __pyx_L60_bool_binop_done;
      }

      /* "pyogrio/_io.pyx":1816
 *                 and skip_features == 0
 *                 and max_features == 0
 *                 and num_features < 0             # <<<<<<<<<<<<<<
 *             ):
 *                 num_features = get_feature_count(ogr_layer, 0)
*/
      __pyx_t_7 = PyObject_RichCompare(__pyx_v_num_features, __pyx_mstate_global->__pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1816, __pyx_L25_error)
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1816, __pyx_L25_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_2 = __pyx_t_1;
      __pyx_L60_bool_binop_done:;

      /* "pyogrio/_io.pyx":1810
 *             # If there are no filters and the driver supports fast feature count, we can
 *             # determine the feature count already which avoids chunking in get_features.
 *             if (             # <<<<<<<<<<<<<<
 *                 where is None
 *                 and bbox is None
*/
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1818
 *                 and num_features < 0
 *             ):
 *                 num_features = get_feature_count(ogr_layer, 0)             # <<<<<<<<<<<<<<
 * 
 *             fid_data, geometries, field_data = get_features(
*/
        __pyx_t_7 = __pyx_f_7pyogrio_3_io_get_feature_count(__pyx_v_ogr_layer, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1818, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF_SET(__pyx_v_num_features, __pyx_t_7);
        __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":1810
 *             # If there are no filters and the driver supports fast feature count, we can
 *             # determine the feature count already which avoids chunking in get_features.
 *             if (             # <<<<<<<<<<<<<<
 *                 where is None
 *                 and bbox is None
*/
      }

      /* "pyogrio/_io.pyx":1822
 *             fid_data, geometries, field_data = get_features(
 *                 ogr_layer,
 *                 fields,             # <<<<<<<<<<<<<<
 *                 encoding,
 *                 read_geometry=read_geometry and geometry_type is not None,
*/
      __pyx_t_22 = __Pyx_PyObject_to_MemoryviewSlice_dsds_object(__pyx_v_fields, PyBUF_WRITABLE); if (unlikely(!__pyx_t_22.memview)) __PYX_ERR(0, 1822, __pyx_L25_error)

      /* "pyogrio/_io.pyx":1824
 *                 fields,
 *                 encoding,
 *                 read_geometry=read_geometry and geometry_type is not None,             # <<<<<<<<<<<<<<
 *                 force_2d=force_2d,
 *                 skip_features=skip_features,
*/
      if (__pyx_v_read_geometry) {
      } else {
        __pyx_t_23 = __pyx_v_read_geometry;
        goto __pyx_L66_bool_binop_done;
      }
      __pyx_t_2 = (__pyx_v_geometry_type != ((PyObject*)Py_None));
      __pyx_t_23 = __pyx_t_2;
      __pyx_L66_bool_binop_done:;

      /* "pyogrio/_io.pyx":1827
 *                 force_2d=force_2d,
 *                 skip_features=skip_features,
 *                 num_features=num_features,             # <<<<<<<<<<<<<<
 *                 return_fids=return_fids,
 *                 datetime_as_string=datetime_as_string
*/
      __pyx_t_26 = __Pyx_PyLong_As_int(__pyx_v_num_features); if (unlikely((__pyx_t_26 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1827, __pyx_L25_error)

      /* "pyogrio/_io.pyx":1820
 *                 num_features = get_feature_count(ogr_layer, 0)
 * 
 *             fid_data, geometries, field_data = get_features(             # <<<<<<<<<<<<<<
 *                 ogr_layer,
 *                 fields,
*/
      __pyx_t_7 = __pyx_f_7pyogrio_3_io_get_features(__pyx_v_ogr_layer, __pyx_t_22, __pyx_v_encoding, __pyx_t_23, __pyx_v_force_2d, __pyx_v_skip_features, __pyx_t_26, __pyx_v_return_fids, __pyx_v_datetime_as_string); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1820, __pyx_L25_error)
      __Pyx_GOTREF(__pyx_t_7);
      __PYX_XCLEAR_MEMVIEW(&__pyx_t_22, 1);
      __pyx_t_22.memview = NULL; __pyx_t_22.data = NULL;
      if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
        PyObject* sequence = __pyx_t_7;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1820, __pyx_L25_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_17);
          __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_8);
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2);
          __Pyx_INCREF(__pyx_t_6);
        } else {
          __pyx_t_17 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1820, __pyx_L25_error)
          __Pyx_XGOTREF(__pyx_t_17);
          __pyx_t_8 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1820, __pyx_L25_error)
          __Pyx_XGOTREF(__pyx_t_8);
          __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 2, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1820, __pyx_L25_error)
          __Pyx_XGOTREF(__pyx_t_6);
        }
        #else
        __pyx_t_17 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1820, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_8 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1820, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_6 = __Pyx_PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1820, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_3 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1820, __pyx_L25_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_24 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3);
        index = 0; __pyx_t_17 = __pyx_t_24(__pyx_t_3); if (unlikely(!__pyx_t_17)) goto __pyx_L68_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        index = 1; __pyx_t_8 = __pyx_t_24(__pyx_t_3); if (unlikely(!__pyx_t_8)) goto __pyx_L68_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_8);
        index = 2; __pyx_t_6 = __pyx_t_24(__pyx_t_3); if (unlikely(!__pyx_t_6)) goto __pyx_L68_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_24(__pyx_t_3), 3) < (0)) __PYX_ERR(0, 1820, __pyx_L25_error)
        __pyx_t_24 = NULL;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        goto __pyx_L69_unpacking_done;
        __pyx_L68_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_24 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1820, __pyx_L25_error)
        __pyx_L69_unpacking_done:;
      }
      __pyx_v_fid_data = __pyx_t_17;
      __pyx_t_17 = 0;
      __pyx_v_geometries = __pyx_t_8;
      __pyx_t_8 = 0;
      __pyx_v_field_data = __pyx_t_6;
      __pyx_t_6 = 0;
    }
    __pyx_L45:;

    /* "pyogrio/_io.pyx":1832
 *             )
 * 
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]             # <<<<<<<<<<<<<<
 *         ogr_subtypes = [
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields
*/
    { /* enter inner scope */
      __pyx_t_7 = PyList_New(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1832, __pyx_L72_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (likely(PyList_CheckExact(__pyx_v_fields)) || PyTuple_CheckExact(__pyx_v_fields)) {
        __pyx_t_6 = __pyx_v_fields; __Pyx_INCREF(__pyx_t_6);
        __pyx_t_9 = 0;
        __pyx_t_27 = NULL;
      } else {
        __pyx_t_9 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_v_fields); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1832, __pyx_L72_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_27 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1832, __pyx_L72_error)
      }
      for (;;) {
        if (likely(!__pyx_t_27)) {
          if (likely(PyList_CheckExact(__pyx_t_6))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_6);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1832, __pyx_L72_error)
              #endif
              if (__pyx_t_9 >= __pyx_temp) break;
            }
            __pyx_t_8 = __Pyx_PyList_GetItemRefFast(__pyx_t_6, __pyx_t_9, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_9;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_6);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1832, __pyx_L72_error)
              #endif
              if (__pyx_t_9 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_9));
            #else
            __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_9);
            #endif
            ++__pyx_t_9;
          }
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1832, __pyx_L72_error)
        } else {
          __pyx_t_8 = __pyx_t_27(__pyx_t_6);
          if (unlikely(!__pyx_t_8)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1832, __pyx_L72_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_XDECREF_SET(__pyx_8genexpr5__pyx_v_field, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_17 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_FIELD_TYPE_NAMES); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1832, __pyx_L72_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1832, __pyx_L72_error)
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_8genexpr5__pyx_v_field, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1832, __pyx_L72_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_18))) {
          __pyx_t_17 = PyMethod_GET_SELF(__pyx_t_18);
          assert(__pyx_t_17);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_18);
          __Pyx_INCREF(__pyx_t_17);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_18, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_17, __pyx_t_3, __pyx_mstate_global->__pyx_n_u_Unknown};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_18, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1832, __pyx_L72_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_7, (PyObject*)__pyx_t_8))) __PYX_ERR(0, 1832, __pyx_L72_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_field); __pyx_8genexpr5__pyx_v_field = 0;
      goto __pyx_L76_exit_scope;
      __pyx_L72_error:;
      __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_field); __pyx_8genexpr5__pyx_v_field = 0;
      goto __pyx_L25_error;
      __pyx_L76_exit_scope:;
    } /* exit inner scope */
    __pyx_v_ogr_types = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":1833
 * 
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
 *         ogr_subtypes = [             # <<<<<<<<<<<<<<
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields
 *         ]
*/
    { /* enter inner scope */
      __pyx_t_7 = PyList_New(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1833, __pyx_L79_error)
      __Pyx_GOTREF(__pyx_t_7);

      /* "pyogrio/_io.pyx":1834
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
 *         ogr_subtypes = [
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields             # <<<<<<<<<<<<<<
 *         ]
 * 
*/
      if (likely(PyList_CheckExact(__pyx_v_fields)) || PyTuple_CheckExact(__pyx_v_fields)) {
        __pyx_t_6 = __pyx_v_fields; __Pyx_INCREF(__pyx_t_6);
        __pyx_t_9 = 0;
        __pyx_t_27 = NULL;
      } else {
        __pyx_t_9 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_v_fields); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1834, __pyx_L79_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_27 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6); if (unlikely(!__pyx_t_27)) __PYX_ERR(0, 1834, __pyx_L79_error)
      }
      for (;;) {
        if (likely(!__pyx_t_27)) {
          if (likely(PyList_CheckExact(__pyx_t_6))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_6);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1834, __pyx_L79_error)
              #endif
              if (__pyx_t_9 >= __pyx_temp) break;
            }
            __pyx_t_8 = __Pyx_PyList_GetItemRefFast(__pyx_t_6, __pyx_t_9, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_9;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_6);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 1834, __pyx_L79_error)
              #endif
              if (__pyx_t_9 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_9));
            #else
            __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_9);
            #endif
            ++__pyx_t_9;
          }
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1834, __pyx_L79_error)
        } else {
          __pyx_t_8 = __pyx_t_27(__pyx_t_6);
          if (unlikely(!__pyx_t_8)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 1834, __pyx_L79_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_XDECREF_SET(__pyx_8genexpr6__pyx_v_field, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_18 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_FIELD_SUBTYPE_NAMES); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1834, __pyx_L79_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1834, __pyx_L79_error)
        __Pyx_GOTREF(__pyx_t_17);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_8genexpr6__pyx_v_field, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1834, __pyx_L79_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_17))) {
          __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_17);
          assert(__pyx_t_18);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_17);
          __Pyx_INCREF(__pyx_t_18);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_17, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_18, __pyx_t_3, __pyx_mstate_global->__pyx_n_u_Unknown};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_17, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1834, __pyx_L79_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_7, (PyObject*)__pyx_t_8))) __PYX_ERR(0, 1833, __pyx_L79_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_field); __pyx_8genexpr6__pyx_v_field = 0;
      goto __pyx_L83_exit_scope;
      __pyx_L79_error:;
      __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_field); __pyx_8genexpr6__pyx_v_field = 0;
      goto __pyx_L25_error;
      __pyx_L83_exit_scope:;
    } /* exit inner scope */
    __pyx_v_ogr_subtypes = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":1838
 * 
 *         meta = {
 *             "crs": crs,             # <<<<<<<<<<<<<<
 *             "encoding": encoding,
 *             "fields": fields[:, 2],
*/
    __pyx_t_7 = __Pyx_PyDict_NewPresized(7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1838, __pyx_L25_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_crs, __pyx_v_crs) < (0)) __PYX_ERR(0, 1838, __pyx_L25_error)

    /* "pyogrio/_io.pyx":1839
 *         meta = {
 *             "crs": crs,
 *             "encoding": encoding,             # <<<<<<<<<<<<<<
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],
*/
    if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_encoding, __pyx_v_encoding) < (0)) __PYX_ERR(0, 1838, __pyx_L25_error)

    /* "pyogrio/_io.pyx":1840
 *             "crs": crs,
 *             "encoding": encoding,
 *             "fields": fields[:, 2],             # <<<<<<<<<<<<<<
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,
*/
    __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[5]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1840, __pyx_L25_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fields, __pyx_t_6) < (0)) __PYX_ERR(0, 1838, __pyx_L25_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "pyogrio/_io.pyx":1841
 *             "encoding": encoding,
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],             # <<<<<<<<<<<<<<
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,
*/
    __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[6]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1841, __pyx_L25_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_dtypes, __pyx_t_6) < (0)) __PYX_ERR(0, 1838, __pyx_L25_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "pyogrio/_io.pyx":1842
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,             # <<<<<<<<<<<<<<
 *             "ogr_subtypes": ogr_subtypes,
 *             "geometry_type": geometry_type,
*/
    if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ogr_types, __pyx_v_ogr_types) < (0)) __PYX_ERR(0, 1838, __pyx_L25_error)

    /* "pyogrio/_io.pyx":1843
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,             # <<<<<<<<<<<<<<
 *             "geometry_type": geometry_type,
 *         }
*/
    if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ogr_subtypes, __pyx_v_ogr_subtypes) < (0)) __PYX_ERR(0, 1838, __pyx_L25_error)

    /* "pyogrio/_io.pyx":1844
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,
 *             "geometry_type": geometry_type,             # <<<<<<<<<<<<<<
 *         }
 * 
*/
    if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_geometry_type, __pyx_v_geometry_type) < (0)) __PYX_ERR(0, 1838, __pyx_L25_error)
    __pyx_v_meta = ((PyObject*)__pyx_t_7);
    __pyx_t_7 = 0;
  }

  /* "pyogrio/_io.pyx":1848
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = (__pyx_v_dataset_options != NULL);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1849
 *     finally:
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
        CSLDestroy(__pyx_v_dataset_options);

        /* "pyogrio/_io.pyx":1850
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
        __pyx_v_dataset_options = NULL;

        /* "pyogrio/_io.pyx":1848
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
      }

      /* "pyogrio/_io.pyx":1852
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
      __pyx_t_2 = (__pyx_v_ogr_dataset != NULL);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":1853
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
        __pyx_t_2 = (__pyx_v_sql != ((PyObject*)Py_None));
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":1854
 *         if ogr_dataset != NULL:
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *             GDALClose(ogr_dataset)
*/
          GDALDatasetReleaseResultSet(__pyx_v_ogr_dataset, __pyx_v_ogr_layer);

          /* "pyogrio/_io.pyx":1853
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
        }

        /* "pyogrio/_io.pyx":1856
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
        (void)(GDALClose(__pyx_v_ogr_dataset));

        /* "pyogrio/_io.pyx":1857
 * 
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
*/
        __pyx_v_ogr_dataset = NULL;

        /* "pyogrio/_io.pyx":1852
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
      }

      /* "pyogrio/_io.pyx":1860
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
      if (__pyx_v_override_shape_encoding) {

        /* "pyogrio/_io.pyx":1861
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *             if prev_shape_encoding != NULL:
*/
        CPLSetThreadLocalConfigOption(__pyx_k_SHAPE_ENCODING, __pyx_v_prev_shape_encoding);

        /* "pyogrio/_io.pyx":1863
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
        __pyx_t_2 = (__pyx_v_prev_shape_encoding != NULL);
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":1864
 * 
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)             # <<<<<<<<<<<<<<
 *                 prev_shape_encoding = NULL
 * 
*/
          CPLFree(((void *)__pyx_v_prev_shape_encoding));

          /* "pyogrio/_io.pyx":1865
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
          __pyx_v_prev_shape_encoding = NULL;

          /* "pyogrio/_io.pyx":1863
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
        }

        /* "pyogrio/_io.pyx":1860
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
      }

      /* "pyogrio/_io.pyx":1867
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      if (__pyx_v_use_tmp_vsimem) {

        /* "pyogrio/_io.pyx":1868
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return (
*/
        __pyx_t_7 = __pyx_v_path;
        __Pyx_INCREF(__pyx_t_7);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_7))) __PYX_ERR(0, 1868, __pyx_L1_error)
        __pyx_t_6 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_7), 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1868, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pyogrio/_io.pyx":1867
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      }
      goto __pyx_L26;
    }
    __pyx_L25_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __PYX_XCLEAR_MEMVIEW(&__pyx_t_21, 1);
      __pyx_t_21.memview = NULL; __pyx_t_21.data = NULL;
      __PYX_XCLEAR_MEMVIEW(&__pyx_t_22, 1);
      __pyx_t_22.memview = NULL; __pyx_t_22.data = NULL;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_33, &__pyx_t_34, &__pyx_t_35);
      if ( unlikely(__Pyx_GetException(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32) < 0)) __Pyx_ErrFetch(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32);
      __Pyx_XGOTREF(__pyx_t_30);
      __Pyx_XGOTREF(__pyx_t_31);
      __Pyx_XGOTREF(__pyx_t_32);
      __Pyx_XGOTREF(__pyx_t_33);
      __Pyx_XGOTREF(__pyx_t_34);
      __Pyx_XGOTREF(__pyx_t_35);
      __pyx_t_26 = __pyx_lineno; __pyx_t_28 = __pyx_clineno; __pyx_t_29 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":1848
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        __pyx_t_2 = (__pyx_v_dataset_options != NULL);
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":1849
 *     finally:
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
          CSLDestroy(__pyx_v_dataset_options);

          /* "pyogrio/_io.pyx":1850
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
          __pyx_v_dataset_options = NULL;

          /* "pyogrio/_io.pyx":1848
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        }

        /* "pyogrio/_io.pyx":1852
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
        __pyx_t_2 = (__pyx_v_ogr_dataset != NULL);
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":1853
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
          __pyx_t_2 = (__pyx_v_sql != ((PyObject*)Py_None));
          if (__pyx_t_2) {

            /* "pyogrio/_io.pyx":1854
 *         if ogr_dataset != NULL:
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *             GDALClose(ogr_dataset)
*/
            GDALDatasetReleaseResultSet(__pyx_v_ogr_dataset, __pyx_v_ogr_layer);

            /* "pyogrio/_io.pyx":1853
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
          }

          /* "pyogrio/_io.pyx":1856
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
          (void)(GDALClose(__pyx_v_ogr_dataset));

          /* "pyogrio/_io.pyx":1857
 * 
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
*/
          __pyx_v_ogr_dataset = NULL;

          /* "pyogrio/_io.pyx":1852
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
        }

        /* "pyogrio/_io.pyx":1860
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
        if (__pyx_v_override_shape_encoding) {

          /* "pyogrio/_io.pyx":1861
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *             if prev_shape_encoding != NULL:
*/
          CPLSetThreadLocalConfigOption(__pyx_k_SHAPE_ENCODING, __pyx_v_prev_shape_encoding);

          /* "pyogrio/_io.pyx":1863
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
          __pyx_t_2 = (__pyx_v_prev_shape_encoding != NULL);
          if (__pyx_t_2) {

            /* "pyogrio/_io.pyx":1864
 * 
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)             # <<<<<<<<<<<<<<
 *                 prev_shape_encoding = NULL
 * 
*/
            CPLFree(((void *)__pyx_v_prev_shape_encoding));

            /* "pyogrio/_io.pyx":1865
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
            __pyx_v_prev_shape_encoding = NULL;

            /* "pyogrio/_io.pyx":1863
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
          }

          /* "pyogrio/_io.pyx":1860
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
        }

        /* "pyogrio/_io.pyx":1867
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        if (__pyx_v_use_tmp_vsimem) {

          /* "pyogrio/_io.pyx":1868
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return (
*/
          if (unlikely(!__pyx_v_path)) { __Pyx_RaiseUnboundLocalError("path"); __PYX_ERR(0, 1868, __pyx_L91_error) }
          __pyx_t_6 = __pyx_v_path;
          __Pyx_INCREF(__pyx_t_6);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_6))) __PYX_ERR(0, 1868, __pyx_L91_error)
          __pyx_t_7 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_6), 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1868, __pyx_L91_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

          /* "pyogrio/_io.pyx":1867
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_35);
      __Pyx_ExceptionReset(__pyx_t_33, __pyx_t_34, __pyx_t_35);
      __Pyx_XGIVEREF(__pyx_t_30);
      __Pyx_XGIVEREF(__pyx_t_31);
      __Pyx_XGIVEREF(__pyx_t_32);
      __Pyx_ErrRestore(__pyx_t_30, __pyx_t_31, __pyx_t_32);
      __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
      __pyx_lineno = __pyx_t_26; __pyx_clineno = __pyx_t_28; __pyx_filename = __pyx_t_29;
      goto __pyx_L1_error;
      __pyx_L91_error:;
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_35);
      __Pyx_ExceptionReset(__pyx_t_33, __pyx_t_34, __pyx_t_35);
      __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
      __Pyx_XDECREF(__pyx_t_31); __pyx_t_31 = 0;
      __Pyx_XDECREF(__pyx_t_32); __pyx_t_32 = 0;
      __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L26:;
  }

  /* "pyogrio/_io.pyx":1870
 *             vsimem_rmtree_toplevel(path)
 * 
 *     return (             # <<<<<<<<<<<<<<
 *         meta,
 *         fid_data,
*/
  __Pyx_XDECREF(__pyx_r);

  /* "pyogrio/_io.pyx":1871
 * 
 *     return (
 *         meta,             # <<<<<<<<<<<<<<
 *         fid_data,
 *         geometries,
*/
  __pyx_t_7 = PyTuple_New(4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1871, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_INCREF(__pyx_v_meta);
  __Pyx_GIVEREF(__pyx_v_meta);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_v_meta) != (0)) __PYX_ERR(0, 1871, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_fid_data);
  __Pyx_GIVEREF(__pyx_v_fid_data);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_fid_data) != (0)) __PYX_ERR(0, 1871, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_geometries);
  __Pyx_GIVEREF(__pyx_v_geometries);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_geometries) != (0)) __PYX_ERR(0, 1871, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_field_data);
  __Pyx_GIVEREF(__pyx_v_field_data);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_v_field_data) != (0)) __PYX_ERR(0, 1871, __pyx_L1_error);
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":1621
 * 
 * 
 * def ogr_read(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object dataset_kwargs,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_21, 1);
  __PYX_XCLEAR_MEMVIEW(&__pyx_t_22, 1);
  __Pyx_AddTraceback("pyogrio._io.ogr_read", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_crs);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_ignored_fields);
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_XDECREF(__pyx_v_field);
  __Pyx_XDECREF(__pyx_v_field_b);
  __Pyx_XDECREF(__pyx_v_geometry_type);
  __Pyx_XDECREF(__pyx_v_geometries);
  __Pyx_XDECREF(__pyx_v_field_data);
  __Pyx_XDECREF(__pyx_v_fid_data);
  __Pyx_XDECREF(__pyx_v_num_features);
  __Pyx_XDECREF(__pyx_v_ogr_types);
  __Pyx_XDECREF(__pyx_v_ogr_subtypes);
  __Pyx_XDECREF(__pyx_v_meta);
  __Pyx_XDECREF(__pyx_8genexpr5__pyx_v_field);
  __Pyx_XDECREF(__pyx_8genexpr6__pyx_v_field);
  __Pyx_XDECREF(__pyx_v_layer);
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XDECREF(__pyx_v_fids);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1878
 * 
 * 
 * cdef void pycapsule_array_stream_deleter(object stream_capsule) noexcept:             # <<<<<<<<<<<<<<
 *     cdef ArrowArrayStream* stream = <ArrowArrayStream*>PyCapsule_GetPointer(
 *         stream_capsule, "arrow_array_stream"
*/

static void __pyx_f_7pyogrio_3_io_pycapsule_array_stream_deleter(PyObject *__pyx_v_stream_capsule) {
  struct ArrowArrayStream *__pyx_v_stream;
  void *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "pyogrio/_io.pyx":1879
 * 
 * cdef void pycapsule_array_stream_deleter(object stream_capsule) noexcept:
 *     cdef ArrowArrayStream* stream = <ArrowArrayStream*>PyCapsule_GetPointer(             # <<<<<<<<<<<<<<
 *         stream_capsule, "arrow_array_stream"
 *     )
*/
  __pyx_t_1 = PyCapsule_GetPointer(__pyx_v_stream_capsule, __pyx_k_arrow_array_stream); if (unlikely(__pyx_t_1 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 1879, __pyx_L1_error)
  __pyx_v_stream = ((struct ArrowArrayStream *)__pyx_t_1);

  /* "pyogrio/_io.pyx":1883
 *     )
 *     # Do not invoke the deleter on a used/moved capsule
 *     if stream.release != NULL:             # <<<<<<<<<<<<<<
 *         stream.release(stream)
 * 
*/
  __pyx_t_2 = (__pyx_v_stream->release != NULL);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":1884
 *     # Do not invoke the deleter on a used/moved capsule
 *     if stream.release != NULL:
 *         stream.release(stream)             # <<<<<<<<<<<<<<
 * 
 *     free(stream)
*/
    __pyx_v_stream->release(__pyx_v_stream);

    /* "pyogrio/_io.pyx":1883
 *     )
 *     # Do not invoke the deleter on a used/moved capsule
 *     if stream.release != NULL:             # <<<<<<<<<<<<<<
 *         stream.release(stream)
 * 
*/
  }

  /* "pyogrio/_io.pyx":1886
 *         stream.release(stream)
 * 
 *     free(stream)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  free(__pyx_v_stream);

  /* "pyogrio/_io.pyx":1878
 * 
 * 
 * cdef void pycapsule_array_stream_deleter(object stream_capsule) noexcept:             # <<<<<<<<<<<<<<
 *     cdef ArrowArrayStream* stream = <ArrowArrayStream*>PyCapsule_GetPointer(
 *         stream_capsule, "arrow_array_stream"
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pyogrio._io.pycapsule_array_stream_deleter", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
}

/* "pyogrio/_io.pyx":1889
 * 
 * 
 * cdef object alloc_c_stream(ArrowArrayStream** c_stream):             # <<<<<<<<<<<<<<
 *     c_stream[0] = <ArrowArrayStream*> malloc(sizeof(ArrowArrayStream))
 *     # Ensure the capsule destructor doesn't call a random release pointer
*/

static PyObject *__pyx_f_7pyogrio_3_io_alloc_c_stream(struct ArrowArrayStream **__pyx_v_c_stream) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("alloc_c_stream", 0);

  /* "pyogrio/_io.pyx":1890
 * 
 * cdef object alloc_c_stream(ArrowArrayStream** c_stream):
 *     c_stream[0] = <ArrowArrayStream*> malloc(sizeof(ArrowArrayStream))             # <<<<<<<<<<<<<<
 *     # Ensure the capsule destructor doesn't call a random release pointer
 *     c_stream[0].release = NULL
*/
  (__pyx_v_c_stream[0]) = ((struct ArrowArrayStream *)malloc((sizeof(struct ArrowArrayStream))));

  /* "pyogrio/_io.pyx":1892
 *     c_stream[0] = <ArrowArrayStream*> malloc(sizeof(ArrowArrayStream))
 *     # Ensure the capsule destructor doesn't call a random release pointer
 *     c_stream[0].release = NULL             # <<<<<<<<<<<<<<
 *     return PyCapsule_New(
 *         c_stream[0], "arrow_array_stream", &pycapsule_array_stream_deleter
*/
  (__pyx_v_c_stream[0])->release = NULL;

  /* "pyogrio/_io.pyx":1893
 *     # Ensure the capsule destructor doesn't call a random release pointer
 *     c_stream[0].release = NULL
 *     return PyCapsule_New(             # <<<<<<<<<<<<<<
 *         c_stream[0], "arrow_array_stream", &pycapsule_array_stream_deleter
 *     )
*/
  __Pyx_XDECREF(__pyx_r);

  /* "pyogrio/_io.pyx":1894
 *     c_stream[0].release = NULL
 *     return PyCapsule_New(
 *         c_stream[0], "arrow_array_stream", &pycapsule_array_stream_deleter             # <<<<<<<<<<<<<<
 *     )
 * 
*/
  __pyx_t_1 = PyCapsule_New((__pyx_v_c_stream[0]), __pyx_k_arrow_array_stream, (&__pyx_f_7pyogrio_3_io_pycapsule_array_stream_deleter)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":1889
 * 
 * 
 * cdef object alloc_c_stream(ArrowArrayStream** c_stream):             # <<<<<<<<<<<<<<
 *     c_stream[0] = <ArrowArrayStream*> malloc(sizeof(ArrowArrayStream))
 *     # Ensure the capsule destructor doesn't call a random release pointer
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyogrio._io.alloc_c_stream", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1899
 * 
 * class _ArrowStream:
 *     def __init__(self, capsule):             # <<<<<<<<<<<<<<
 *         self._capsule = capsule
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_12_ArrowStream_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7pyogrio_3_io_12_ArrowStream_1__init__ = {"__init__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_12_ArrowStream_1__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7pyogrio_3_io_12_ArrowStream_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_capsule = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_capsule,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1899, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1899, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1899, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 1899, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, i); __PYX_ERR(0, 1899, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1899, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1899, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_capsule = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1899, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io._ArrowStream.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyogrio_3_io_12_ArrowStream___init__(__pyx_self, __pyx_v_self, __pyx_v_capsule);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_12_ArrowStream___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_capsule) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyogrio/_io.pyx":1900
 * class _ArrowStream:
 *     def __init__(self, capsule):
 *         self._capsule = capsule             # <<<<<<<<<<<<<<
 * 
 *     def __arrow_c_stream__(self, requested_schema=None):
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_capsule_2, __pyx_v_capsule) < (0)) __PYX_ERR(0, 1900, __pyx_L1_error)

  /* "pyogrio/_io.pyx":1899
 * 
 * class _ArrowStream:
 *     def __init__(self, capsule):             # <<<<<<<<<<<<<<
 *         self._capsule = capsule
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyogrio._io._ArrowStream.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":1902
 *         self._capsule = capsule
 * 
 *     def __arrow_c_stream__(self, requested_schema=None):             # <<<<<<<<<<<<<<
 *         if requested_schema is not None:
 *             raise NotImplementedError("requested_schema is not supported")
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_12_ArrowStream_3__arrow_c_stream__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_7pyogrio_3_io_12_ArrowStream_3__arrow_c_stream__ = {"__arrow_c_stream__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_12_ArrowStream_3__arrow_c_stream__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7pyogrio_3_io_12_ArrowStream_3__arrow_c_stream__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_requested_schema = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__arrow_c_stream__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_requested_schema,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1902, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1902, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1902, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__arrow_c_stream__", 0) < (0)) __PYX_ERR(0, 1902, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__arrow_c_stream__", 0, 1, 2, i); __PYX_ERR(0, 1902, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1902, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1902, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_requested_schema = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__arrow_c_stream__", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 1902, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io._ArrowStream.__arrow_c_stream__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyogrio_3_io_12_ArrowStream_2__arrow_c_stream__(__pyx_self, __pyx_v_self, __pyx_v_requested_schema);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_12_ArrowStream_2__arrow_c_stream__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_requested_schema) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__arrow_c_stream__", 0);

  /* "pyogrio/_io.pyx":1903
 * 
 *     def __arrow_c_stream__(self, requested_schema=None):
 *         if requested_schema is not None:             # <<<<<<<<<<<<<<
 *             raise NotImplementedError("requested_schema is not supported")
 *         return self._capsule
*/
  __pyx_t_1 = (__pyx_v_requested_schema != Py_None);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":1904
 *     def __arrow_c_stream__(self, requested_schema=None):
 *         if requested_schema is not None:
 *             raise NotImplementedError("requested_schema is not supported")             # <<<<<<<<<<<<<<
 *         return self._capsule
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_requested_schema_is_not_supporte};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_NotImplementedError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1904, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1904, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1903
 * 
 *     def __arrow_c_stream__(self, requested_schema=None):
 *         if requested_schema is not None:             # <<<<<<<<<<<<<<
 *             raise NotImplementedError("requested_schema is not supported")
 *         return self._capsule
*/
  }

  /* "pyogrio/_io.pyx":1905
 *         if requested_schema is not None:
 *             raise NotImplementedError("requested_schema is not supported")
 *         return self._capsule             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_capsule_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":1902
 *         self._capsule = capsule
 * 
 *     def __arrow_c_stream__(self, requested_schema=None):             # <<<<<<<<<<<<<<
 *         if requested_schema is not None:
 *             raise NotImplementedError("requested_schema is not supported")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyogrio._io._ArrowStream.__arrow_c_stream__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_7pyogrio_3_io_4generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "pyogrio/_io.pyx":1908
 * 
 * 
 * @contextlib.contextmanager             # <<<<<<<<<<<<<<
 * def ogr_open_arrow(
 *     object path_or_buffer,
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_3ogr_open_arrow(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_2ogr_open_arrow, "Open a file via the GDAL arrow interface.\n\n    Returns\n    -------\n    tuple(dict, Arrow ArrayStream)\n        A tuple containing a dictionary with metadata about the file and an Arrow\n        ArrayStream object that can be used to read the data.\n    ");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_3ogr_open_arrow = {"ogr_open_arrow", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_3ogr_open_arrow, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_2ogr_open_arrow};
static PyObject *__pyx_pw_7pyogrio_3_io_3ogr_open_arrow(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path_or_buffer = 0;
  PyObject *__pyx_v_dataset_kwargs = 0;
  PyObject *__pyx_v_layer = 0;
  PyObject *__pyx_v_encoding = 0;
  int __pyx_v_read_geometry;
  int __pyx_v_force_2d;
  PyObject *__pyx_v_columns = 0;
  int __pyx_v_skip_features;
  int __pyx_v_max_features;
  PyObject *__pyx_v_where = 0;
  PyObject *__pyx_v_bbox = 0;
  PyObject *__pyx_v_mask = 0;
  PyObject *__pyx_v_fids = 0;
  PyObject *__pyx_v_sql = 0;
  PyObject *__pyx_v_sql_dialect = 0;
  int __pyx_v_return_fids;
  int __pyx_v_batch_size;
  PyObject *__pyx_v_use_pyarrow = 0;
  PyObject *__pyx_v_datetime_as_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[19] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_open_arrow (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path_or_buffer,&__pyx_mstate_global->__pyx_n_u_dataset_kwargs,&__pyx_mstate_global->__pyx_n_u_layer,&__pyx_mstate_global->__pyx_n_u_encoding,&__pyx_mstate_global->__pyx_n_u_read_geometry,&__pyx_mstate_global->__pyx_n_u_force_2d,&__pyx_mstate_global->__pyx_n_u_columns,&__pyx_mstate_global->__pyx_n_u_skip_features,&__pyx_mstate_global->__pyx_n_u_max_features,&__pyx_mstate_global->__pyx_n_u_where,&__pyx_mstate_global->__pyx_n_u_bbox,&__pyx_mstate_global->__pyx_n_u_mask,&__pyx_mstate_global->__pyx_n_u_fids,&__pyx_mstate_global->__pyx_n_u_sql,&__pyx_mstate_global->__pyx_n_u_sql_dialect,&__pyx_mstate_global->__pyx_n_u_return_fids,&__pyx_mstate_global->__pyx_n_u_batch_size,&__pyx_mstate_global->__pyx_n_u_use_pyarrow,&__pyx_mstate_global->__pyx_n_u_datetime_as_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1908, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case 19:
        values[18] = __Pyx_ArgRef_FASTCALL(__pyx_args, 18);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[18])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 18:
        values[17] = __Pyx_ArgRef_FASTCALL(__pyx_args, 17);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[17])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 17:
        values[16] = __Pyx_ArgRef_FASTCALL(__pyx_args, 16);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[16])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 16:
        values[15] = __Pyx_ArgRef_FASTCALL(__pyx_args, 15);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[15])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 15:
        values[14] = __Pyx_ArgRef_FASTCALL(__pyx_args, 14);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[14])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 14:
        values[13] = __Pyx_ArgRef_FASTCALL(__pyx_args, 13);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[13])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "ogr_open_arrow", 0) < (0)) __PYX_ERR(0, 1908, __pyx_L3_error)

      /* "pyogrio/_io.pyx":1912
 *     object path_or_buffer,
 *     dataset_kwargs,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int read_geometry=True,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1913
 *     dataset_kwargs,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int read_geometry=True,
 *     int force_2d=False,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1916
 *     int read_geometry=True,
 *     int force_2d=False,
 *     object columns=None,             # <<<<<<<<<<<<<<
 *     int skip_features=0,
 *     int max_features=0,
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1919
 *     int skip_features=0,
 *     int max_features=0,
 *     object where=None,             # <<<<<<<<<<<<<<
 *     tuple bbox=None,
 *     object mask=None,
*/
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1920
 *     int max_features=0,
 *     object where=None,
 *     tuple bbox=None,             # <<<<<<<<<<<<<<
 *     object mask=None,
 *     object fids=None,
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1921
 *     object where=None,
 *     tuple bbox=None,
 *     object mask=None,             # <<<<<<<<<<<<<<
 *     object fids=None,
 *     str sql=None,
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1922
 *     tuple bbox=None,
 *     object mask=None,
 *     object fids=None,             # <<<<<<<<<<<<<<
 *     str sql=None,
 *     str sql_dialect=None,
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1923
 *     object mask=None,
 *     object fids=None,
 *     str sql=None,             # <<<<<<<<<<<<<<
 *     str sql_dialect=None,
 *     int return_fids=False,
*/
      if (!values[13]) values[13] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1924
 *     object fids=None,
 *     str sql=None,
 *     str sql_dialect=None,             # <<<<<<<<<<<<<<
 *     int return_fids=False,
 *     int batch_size=0,
*/
      if (!values[14]) values[14] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1927
 *     int return_fids=False,
 *     int batch_size=0,
 *     use_pyarrow=False,             # <<<<<<<<<<<<<<
 *     datetime_as_string=False,
 * ):
*/
      if (!values[17]) values[17] = __Pyx_NewRef(((PyObject *)((PyObject*)Py_False)));

      /* "pyogrio/_io.pyx":1928
 *     int batch_size=0,
 *     use_pyarrow=False,
 *     datetime_as_string=False,             # <<<<<<<<<<<<<<
 * ):
 *     """Open a file via the GDAL arrow interface.
*/
      if (!values[18]) values[18] = __Pyx_NewRef(((PyObject *)((PyObject*)Py_False)));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("ogr_open_arrow", 0, 2, 19, i); __PYX_ERR(0, 1908, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case 19:
        values[18] = __Pyx_ArgRef_FASTCALL(__pyx_args, 18);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[18])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 18:
        values[17] = __Pyx_ArgRef_FASTCALL(__pyx_args, 17);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[17])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 17:
        values[16] = __Pyx_ArgRef_FASTCALL(__pyx_args, 16);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[16])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 16:
        values[15] = __Pyx_ArgRef_FASTCALL(__pyx_args, 15);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[15])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 15:
        values[14] = __Pyx_ArgRef_FASTCALL(__pyx_args, 14);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[14])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 14:
        values[13] = __Pyx_ArgRef_FASTCALL(__pyx_args, 13);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[13])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1908, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1908, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1908, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "pyogrio/_io.pyx":1912
 *     object path_or_buffer,
 *     dataset_kwargs,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int read_geometry=True,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1913
 *     dataset_kwargs,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int read_geometry=True,
 *     int force_2d=False,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1916
 *     int read_geometry=True,
 *     int force_2d=False,
 *     object columns=None,             # <<<<<<<<<<<<<<
 *     int skip_features=0,
 *     int max_features=0,
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1919
 *     int skip_features=0,
 *     int max_features=0,
 *     object where=None,             # <<<<<<<<<<<<<<
 *     tuple bbox=None,
 *     object mask=None,
*/
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1920
 *     int max_features=0,
 *     object where=None,
 *     tuple bbox=None,             # <<<<<<<<<<<<<<
 *     object mask=None,
 *     object fids=None,
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1921
 *     object where=None,
 *     tuple bbox=None,
 *     object mask=None,             # <<<<<<<<<<<<<<
 *     object fids=None,
 *     str sql=None,
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1922
 *     tuple bbox=None,
 *     object mask=None,
 *     object fids=None,             # <<<<<<<<<<<<<<
 *     str sql=None,
 *     str sql_dialect=None,
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":1923
 *     object mask=None,
 *     object fids=None,
 *     str sql=None,             # <<<<<<<<<<<<<<
 *     str sql_dialect=None,
 *     int return_fids=False,
*/
      if (!values[13]) values[13] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":1924
 *     object fids=None,
 *     str sql=None,
 *     str sql_dialect=None,             # <<<<<<<<<<<<<<
 *     int return_fids=False,
 *     int batch_size=0,
*/
      if (!values[14]) values[14] = __Pyx_NewRef(((PyObject*)Py_None));
      if (!values[17]) values[17] = __Pyx_NewRef(((PyObject *)((PyObject*)Py_False)));
      if (!values[18]) values[18] = __Pyx_NewRef(((PyObject *)((PyObject*)Py_False)));
    }
    __pyx_v_path_or_buffer = values[0];
    __pyx_v_dataset_kwargs = values[1];
    __pyx_v_layer = values[2];
    __pyx_v_encoding = values[3];
    if (values[4]) {
      __pyx_v_read_geometry = __Pyx_PyLong_As_int(values[4]); if (unlikely((__pyx_v_read_geometry == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1914, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":1914
 *     object layer=None,
 *     object encoding=None,
 *     int read_geometry=True,             # <<<<<<<<<<<<<<
 *     int force_2d=False,
 *     object columns=None,
*/
      __pyx_v_read_geometry = ((int)((int)1));
    }
    if (values[5]) {
      __pyx_v_force_2d = __Pyx_PyLong_As_int(values[5]); if (unlikely((__pyx_v_force_2d == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1915, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":1915
 *     object encoding=None,
 *     int read_geometry=True,
 *     int force_2d=False,             # <<<<<<<<<<<<<<
 *     object columns=None,
 *     int skip_features=0,
*/
      __pyx_v_force_2d = ((int)((int)0));
    }
    __pyx_v_columns = values[6];
    if (values[7]) {
      __pyx_v_skip_features = __Pyx_PyLong_As_int(values[7]); if (unlikely((__pyx_v_skip_features == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1917, __pyx_L3_error)
    } else {
      __pyx_v_skip_features = ((int)((int)0));
    }
    if (values[8]) {
      __pyx_v_max_features = __Pyx_PyLong_As_int(values[8]); if (unlikely((__pyx_v_max_features == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1918, __pyx_L3_error)
    } else {
      __pyx_v_max_features = ((int)((int)0));
    }
    __pyx_v_where = values[9];
    __pyx_v_bbox = ((PyObject*)values[10]);
    __pyx_v_mask = values[11];
    __pyx_v_fids = values[12];
    __pyx_v_sql = ((PyObject*)values[13]);
    __pyx_v_sql_dialect = ((PyObject*)values[14]);
    if (values[15]) {
      __pyx_v_return_fids = __Pyx_PyLong_As_int(values[15]); if (unlikely((__pyx_v_return_fids == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1925, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":1925
 *     str sql=None,
 *     str sql_dialect=None,
 *     int return_fids=False,             # <<<<<<<<<<<<<<
 *     int batch_size=0,
 *     use_pyarrow=False,
*/
      __pyx_v_return_fids = ((int)((int)0));
    }
    if (values[16]) {
      __pyx_v_batch_size = __Pyx_PyLong_As_int(values[16]); if (unlikely((__pyx_v_batch_size == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1926, __pyx_L3_error)
    } else {
      __pyx_v_batch_size = ((int)((int)0));
    }
    __pyx_v_use_pyarrow = values[17];
    __pyx_v_datetime_as_string = values[18];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ogr_open_arrow", 0, 2, 19, __pyx_nargs); __PYX_ERR(0, 1908, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.ogr_open_arrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_bbox), (&PyTuple_Type), 1, "bbox", 1))) __PYX_ERR(0, 1920, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sql), (&PyUnicode_Type), 1, "sql", 1))) __PYX_ERR(0, 1923, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sql_dialect), (&PyUnicode_Type), 1, "sql_dialect", 1))) __PYX_ERR(0, 1924, __pyx_L1_error)
  __pyx_r = __pyx_pf_7pyogrio_3_io_2ogr_open_arrow(__pyx_self, __pyx_v_path_or_buffer, __pyx_v_dataset_kwargs, __pyx_v_layer, __pyx_v_encoding, __pyx_v_read_geometry, __pyx_v_force_2d, __pyx_v_columns, __pyx_v_skip_features, __pyx_v_max_features, __pyx_v_where, __pyx_v_bbox, __pyx_v_mask, __pyx_v_fids, __pyx_v_sql, __pyx_v_sql_dialect, __pyx_v_return_fids, __pyx_v_batch_size, __pyx_v_use_pyarrow, __pyx_v_datetime_as_string);

  /* "pyogrio/_io.pyx":1908
 * 
 * 
 * @contextlib.contextmanager             # <<<<<<<<<<<<<<
 * def ogr_open_arrow(
 *     object path_or_buffer,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_2ogr_open_arrow(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer, PyObject *__pyx_v_encoding, int __pyx_v_read_geometry, int __pyx_v_force_2d, PyObject *__pyx_v_columns, int __pyx_v_skip_features, int __pyx_v_max_features, PyObject *__pyx_v_where, PyObject *__pyx_v_bbox, PyObject *__pyx_v_mask, PyObject *__pyx_v_fids, PyObject *__pyx_v_sql, PyObject *__pyx_v_sql_dialect, int __pyx_v_return_fids, int __pyx_v_batch_size, PyObject *__pyx_v_use_pyarrow, PyObject *__pyx_v_datetime_as_string) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_open_arrow", 0);
  __pyx_cur_scope = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow(__pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1908, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_path_or_buffer = __pyx_v_path_or_buffer;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_path_or_buffer);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_path_or_buffer);
  __pyx_cur_scope->__pyx_v_dataset_kwargs = __pyx_v_dataset_kwargs;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_dataset_kwargs);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_dataset_kwargs);
  __pyx_cur_scope->__pyx_v_layer = __pyx_v_layer;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_layer);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_layer);
  __pyx_cur_scope->__pyx_v_encoding = __pyx_v_encoding;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_encoding);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_encoding);
  __pyx_cur_scope->__pyx_v_read_geometry = __pyx_v_read_geometry;
  __pyx_cur_scope->__pyx_v_force_2d = __pyx_v_force_2d;
  __pyx_cur_scope->__pyx_v_columns = __pyx_v_columns;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_columns);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_columns);
  __pyx_cur_scope->__pyx_v_skip_features = __pyx_v_skip_features;
  __pyx_cur_scope->__pyx_v_max_features = __pyx_v_max_features;
  __pyx_cur_scope->__pyx_v_where = __pyx_v_where;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_where);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_where);
  __pyx_cur_scope->__pyx_v_bbox = __pyx_v_bbox;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_bbox);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_bbox);
  __pyx_cur_scope->__pyx_v_mask = __pyx_v_mask;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_mask);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_mask);
  __pyx_cur_scope->__pyx_v_fids = __pyx_v_fids;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_fids);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_fids);
  __pyx_cur_scope->__pyx_v_sql = __pyx_v_sql;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_sql);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_sql);
  __pyx_cur_scope->__pyx_v_sql_dialect = __pyx_v_sql_dialect;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_sql_dialect);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_sql_dialect);
  __pyx_cur_scope->__pyx_v_return_fids = __pyx_v_return_fids;
  __pyx_cur_scope->__pyx_v_batch_size = __pyx_v_batch_size;
  __pyx_cur_scope->__pyx_v_use_pyarrow = __pyx_v_use_pyarrow;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_use_pyarrow);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_use_pyarrow);
  __pyx_cur_scope->__pyx_v_datetime_as_string = __pyx_v_datetime_as_string;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_datetime_as_string);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_datetime_as_string);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_7pyogrio_3_io_4generator, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_ogr_open_arrow, __pyx_mstate_global->__pyx_n_u_ogr_open_arrow, __pyx_mstate_global->__pyx_n_u_pyogrio__io); if (unlikely(!gen)) __PYX_ERR(0, 1908, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyogrio._io.ogr_open_arrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_7pyogrio_3_io_4generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *__pyx_cur_scope = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  char const *__pyx_t_8;
  char **__pyx_t_9;
  char const *__pyx_t_10;
  void *__pyx_t_11;
  OGRLayerH __pyx_t_12;
  struct __pyx_opt_args_7pyogrio_3_io_execute_sql __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  struct __pyx_opt_args_7pyogrio_3_io_get_fields __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_t_20;
  PyObject *(*__pyx_t_21)(PyObject *);
  PyObject *__pyx_t_22[4];
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_t_26;
  PyObject *__pyx_t_27[3];
  int __pyx_t_28;
  char const *__pyx_t_29;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  PyObject *__pyx_t_35 = NULL;
  char const *__pyx_t_36;
  char const *__pyx_t_37;
  char const *__pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_open_arrow", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L96_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(0, 1908, __pyx_L1_error)
  }

  /* "pyogrio/_io.pyx":1938
 *         ArrayStream object that can be used to read the data.
 *     """
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)             # <<<<<<<<<<<<<<
 *     cdef char **dataset_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
*/
  __pyx_t_1 = PyBytes_Check(__pyx_cur_scope->__pyx_v_path_or_buffer); 
  __pyx_cur_scope->__pyx_v_use_tmp_vsimem = __pyx_t_1;

  /* "pyogrio/_io.pyx":1939
 *     """
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef char **dataset_options = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
*/
  __pyx_cur_scope->__pyx_v_dataset_options = NULL;

  /* "pyogrio/_io.pyx":1940
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef char **dataset_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef char **fields_c = NULL
*/
  __pyx_cur_scope->__pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":1941
 *     cdef char **dataset_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 *     cdef char **fields_c = NULL
 *     cdef const char *field_c = NULL
*/
  __pyx_cur_scope->__pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":1942
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef char **fields_c = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *field_c = NULL
 *     cdef char **options = NULL
*/
  __pyx_cur_scope->__pyx_v_fields_c = NULL;

  /* "pyogrio/_io.pyx":1943
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef char **fields_c = NULL
 *     cdef const char *field_c = NULL             # <<<<<<<<<<<<<<
 *     cdef char **options = NULL
 *     cdef const char *prev_shape_encoding = NULL
*/
  __pyx_cur_scope->__pyx_v_field_c = NULL;

  /* "pyogrio/_io.pyx":1944
 *     cdef char **fields_c = NULL
 *     cdef const char *field_c = NULL
 *     cdef char **options = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *prev_shape_encoding = NULL
 *     cdef bint override_shape_encoding = False
*/
  __pyx_cur_scope->__pyx_v_options = NULL;

  /* "pyogrio/_io.pyx":1945
 *     cdef const char *field_c = NULL
 *     cdef char **options = NULL
 *     cdef const char *prev_shape_encoding = NULL             # <<<<<<<<<<<<<<
 *     cdef bint override_shape_encoding = False
 *     cdef ArrowArrayStream* stream
*/
  __pyx_cur_scope->__pyx_v_prev_shape_encoding = NULL;

  /* "pyogrio/_io.pyx":1946
 *     cdef char **options = NULL
 *     cdef const char *prev_shape_encoding = NULL
 *     cdef bint override_shape_encoding = False             # <<<<<<<<<<<<<<
 *     cdef ArrowArrayStream* stream
 * 
*/
  __pyx_cur_scope->__pyx_v_override_shape_encoding = 0;

  /* "pyogrio/_io.pyx":1949
 *     cdef ArrowArrayStream* stream
 * 
 *     if force_2d:             # <<<<<<<<<<<<<<
 *         raise ValueError("forcing 2D is not supported for Arrow")
 * 
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_force_2d != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":1950
 * 
 *     if force_2d:
 *         raise ValueError("forcing 2D is not supported for Arrow")             # <<<<<<<<<<<<<<
 * 
 *     if fids is not None:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_forcing_2D_is_not_supported_for};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1950, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1950, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1949
 *     cdef ArrowArrayStream* stream
 * 
 *     if force_2d:             # <<<<<<<<<<<<<<
 *         raise ValueError("forcing 2D is not supported for Arrow")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1952
 *         raise ValueError("forcing 2D is not supported for Arrow")
 * 
 *     if fids is not None:             # <<<<<<<<<<<<<<
 *         if (
 *             where is not None
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_fids != Py_None);
  if (__pyx_t_1) {

    /* "pyogrio/_io.pyx":1954
 *     if fids is not None:
 *         if (
 *             where is not None             # <<<<<<<<<<<<<<
 *             or bbox is not None
 *             or mask is not None
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_where != Py_None);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L7_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1955
 *         if (
 *             where is not None
 *             or bbox is not None             # <<<<<<<<<<<<<<
 *             or mask is not None
 *             or sql is not None
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_bbox != ((PyObject*)Py_None));
    if (!__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L7_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1956
 *             where is not None
 *             or bbox is not None
 *             or mask is not None             # <<<<<<<<<<<<<<
 *             or sql is not None
 *             or skip_features
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_mask != Py_None);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L7_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1957
 *             or bbox is not None
 *             or mask is not None
 *             or sql is not None             # <<<<<<<<<<<<<<
 *             or skip_features
 *             or max_features
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_sql != ((PyObject*)Py_None));
    if (!__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L7_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1958
 *             or mask is not None
 *             or sql is not None
 *             or skip_features             # <<<<<<<<<<<<<<
 *             or max_features
 *         ):
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_skip_features != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L7_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":1959
 *             or sql is not None
 *             or skip_features
 *             or max_features             # <<<<<<<<<<<<<<
 *         ):
 *             raise ValueError(
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_max_features != 0);
    __pyx_t_1 = __pyx_t_5;
    __pyx_L7_bool_binop_done:;

    /* "pyogrio/_io.pyx":1953
 * 
 *     if fids is not None:
 *         if (             # <<<<<<<<<<<<<<
 *             where is not None
 *             or bbox is not None
*/
    if (unlikely(__pyx_t_1)) {

      /* "pyogrio/_io.pyx":1961
 *             or max_features
 *         ):
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "cannot set both 'fids' and any of 'where', 'bbox', 'mask', "
 *                 "'sql', 'skip_features', or 'max_features'"
*/
      __pyx_t_3 = NULL;
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_cannot_set_both_fids_and_any_of_2};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1961, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1961, __pyx_L1_error)

      /* "pyogrio/_io.pyx":1953
 * 
 *     if fids is not None:
 *         if (             # <<<<<<<<<<<<<<
 *             where is not None
 *             or bbox is not None
*/
    }

    /* "pyogrio/_io.pyx":1952
 *         raise ValueError("forcing 2D is not supported for Arrow")
 * 
 *     if fids is not None:             # <<<<<<<<<<<<<<
 *         if (
 *             where is not None
*/
  }

  /* "pyogrio/_io.pyx":1972
 *             )
 * 
 *     if skip_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'skip_features' must be >= 0")
 * 
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_skip_features < 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":1973
 * 
 *     if skip_features < 0:
 *         raise ValueError("'skip_features' must be >= 0")             # <<<<<<<<<<<<<<
 * 
 *     if max_features:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_skip_features_must_be_0};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1973, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1973, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1972
 *             )
 * 
 *     if skip_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'skip_features' must be >= 0")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1975
 *         raise ValueError("'skip_features' must be >= 0")
 * 
 *     if max_features:             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "specifying 'max_features' is not supported for Arrow"
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_max_features != 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":1976
 * 
 *     if max_features:
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "specifying 'max_features' is not supported for Arrow"
 *         )
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_specifying_max_features_is_not_s};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1976, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1976, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1975
 *         raise ValueError("'skip_features' must be >= 0")
 * 
 *     if max_features:             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "specifying 'max_features' is not supported for Arrow"
*/
  }

  /* "pyogrio/_io.pyx":1980
 *         )
 * 
 *     if sql is not None and layer is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
*/
  __pyx_t_5 = (__pyx_cur_scope->__pyx_v_sql != ((PyObject*)Py_None));
  if (__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_5 = (__pyx_cur_scope->__pyx_v_layer != Py_None);
  __pyx_t_1 = __pyx_t_5;
  __pyx_L16_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":1981
 * 
 *     if sql is not None and layer is not None:
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")             # <<<<<<<<<<<<<<
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_sql_parameter_cannot_be_combine};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1981, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1981, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1980
 *         )
 * 
 *     if sql is not None and layer is not None:             # <<<<<<<<<<<<<<
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1983
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "at least one of read_geometry or return_fids must be True or columns must "
*/
  __pyx_t_5 = (__pyx_cur_scope->__pyx_v_read_geometry != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L19_bool_binop_done;
  }
  __pyx_t_5 = (__pyx_cur_scope->__pyx_v_return_fids != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L19_bool_binop_done;
  }
  __pyx_t_5 = (__pyx_cur_scope->__pyx_v_columns == Py_None);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L19_bool_binop_done;
  }
  __pyx_t_6 = PyObject_Length(__pyx_cur_scope->__pyx_v_columns); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1983, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_6 > 0);
  __pyx_t_1 = __pyx_t_5;
  __pyx_L19_bool_binop_done:;
  __pyx_t_5 = (!__pyx_t_1);
  if (unlikely(__pyx_t_5)) {

    /* "pyogrio/_io.pyx":1984
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "at least one of read_geometry or return_fids must be True or columns must "
 *             "be None or non-empty"
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_at_least_one_of_read_geometry_or};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1984, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1984, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1983
 *         raise ValueError("'sql' parameter cannot be combined with 'layer'")
 * 
 *     if not (read_geometry or return_fids or columns is None or len(columns) > 0):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "at least one of read_geometry or return_fids must be True or columns must "
*/
  }

  /* "pyogrio/_io.pyx":1989
 *         )
 * 
 *     if bbox and mask:             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
*/
  if (__pyx_cur_scope->__pyx_v_bbox == Py_None) __pyx_t_1 = 0;
  else
  {
    Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_cur_scope->__pyx_v_bbox);
    if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 1989, __pyx_L1_error)
    __pyx_t_1 = (__pyx_temp != 0);
  }

  if (__pyx_t_1) {
  } else {
    __pyx_t_5 = __pyx_t_1;
    goto __pyx_L24_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_mask); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1989, __pyx_L1_error)
  __pyx_t_5 = __pyx_t_1;
  __pyx_L24_bool_binop_done:;
  if (unlikely(__pyx_t_5)) {

    /* "pyogrio/_io.pyx":1990
 * 
 *     if bbox and mask:
 *         raise ValueError("cannot set both 'bbox' and 'mask'")             # <<<<<<<<<<<<<<
 * 
 *     reader = None
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_cannot_set_both_bbox_and_mask};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1990, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1990, __pyx_L1_error)

    /* "pyogrio/_io.pyx":1989
 *         )
 * 
 *     if bbox and mask:             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
*/
  }

  /* "pyogrio/_io.pyx":1992
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
 *     reader = None             # <<<<<<<<<<<<<<
 *     try:
 *         path = read_buffer_to_vsimem(
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __pyx_cur_scope->__pyx_v_reader = Py_None;

  /* "pyogrio/_io.pyx":1993
 * 
 *     reader = None
 *     try:             # <<<<<<<<<<<<<<
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":1996
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 * 
 *         if encoding:
*/
    if (__pyx_cur_scope->__pyx_v_use_tmp_vsimem) {

      /* "pyogrio/_io.pyx":1995
 *     try:
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer             # <<<<<<<<<<<<<<
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
*/
      __pyx_t_3 = __pyx_cur_scope->__pyx_v_path_or_buffer;
      __Pyx_INCREF(__pyx_t_3);
      if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_3))) __PYX_ERR(0, 1995, __pyx_L27_error)

      /* "pyogrio/_io.pyx":1994
 *     reader = None
 *     try:
 *         path = read_buffer_to_vsimem(             # <<<<<<<<<<<<<<
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer
*/
      __pyx_t_7 = __pyx_f_7pyogrio_4_vsi_read_buffer_to_vsimem(((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1994, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_2 = __pyx_t_7;
      __pyx_t_7 = 0;
    } else {

      /* "pyogrio/_io.pyx":1996
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 * 
 *         if encoding:
*/
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_path_or_buffer);
      __pyx_t_2 = __pyx_cur_scope->__pyx_v_path_or_buffer;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_v_path = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":1998
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
 *         if encoding:             # <<<<<<<<<<<<<<
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_encoding); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1998, __pyx_L27_error)
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":1999
 * 
 *         if encoding:
 *             override_shape_encoding = True             # <<<<<<<<<<<<<<
 *             prev_shape_encoding = override_threadlocal_config_option(
 *                 "SHAPE_ENCODING", encoding
*/
      __pyx_cur_scope->__pyx_v_override_shape_encoding = 1;

      /* "pyogrio/_io.pyx":2001
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(
 *                 "SHAPE_ENCODING", encoding             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_2 = __pyx_cur_scope->__pyx_v_encoding;
      __Pyx_INCREF(__pyx_t_2);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 2001, __pyx_L27_error)

      /* "pyogrio/_io.pyx":2000
 *         if encoding:
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(             # <<<<<<<<<<<<<<
 *                 "SHAPE_ENCODING", encoding
 *             )
*/
      __pyx_t_8 = __pyx_f_7pyogrio_3_io_override_threadlocal_config_option(__pyx_mstate_global->__pyx_n_u_SHAPE_ENCODING, ((PyObject*)__pyx_t_2)); if (unlikely(__pyx_t_8 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 2000, __pyx_L27_error)
      __pyx_cur_scope->__pyx_v_prev_shape_encoding = __pyx_t_8;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyogrio/_io.pyx":1998
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
 *         if encoding:             # <<<<<<<<<<<<<<
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(
*/
    }

    /* "pyogrio/_io.pyx":2004
 *             )
 * 
 *         dataset_options = dict_to_options(dataset_kwargs)             # <<<<<<<<<<<<<<
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
*/
    __pyx_t_9 = __pyx_f_7pyogrio_3_io_dict_to_options(__pyx_cur_scope->__pyx_v_dataset_kwargs); if (unlikely(__pyx_t_9 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 2004, __pyx_L27_error)
    __pyx_cur_scope->__pyx_v_dataset_options = __pyx_t_9;

    /* "pyogrio/_io.pyx":2005
 * 
 *         dataset_options = dict_to_options(dataset_kwargs)
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)             # <<<<<<<<<<<<<<
 * 
 *         if sql is None:
*/
    __pyx_t_7 = __pyx_cur_scope->__pyx_v_path;
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2005, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_10 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_10) && PyErr_Occurred())) __PYX_ERR(0, 2005, __pyx_L27_error)
    __pyx_t_11 = __pyx_f_7pyogrio_3_io_ogr_open(__pyx_t_10, 0, __pyx_cur_scope->__pyx_v_dataset_options); if (unlikely(__pyx_t_11 == ((void *)NULL))) __PYX_ERR(0, 2005, __pyx_L27_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_cur_scope->__pyx_v_ogr_dataset = __pyx_t_11;

    /* "pyogrio/_io.pyx":2007
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
 *         if sql is None:             # <<<<<<<<<<<<<<
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_sql == ((PyObject*)Py_None));
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2008
 * 
 *         if sql is None:
 *             if layer is None:             # <<<<<<<<<<<<<<
 *                 layer = get_default_layer(ogr_dataset)
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
*/
      __pyx_t_5 = (__pyx_cur_scope->__pyx_v_layer == Py_None);
      if (__pyx_t_5) {

        /* "pyogrio/_io.pyx":2009
 *         if sql is None:
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
 *         else:
*/
        __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_default_layer(__pyx_cur_scope->__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2009, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_layer);
        __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_layer, __pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_2);
        __pyx_t_2 = 0;

        /* "pyogrio/_io.pyx":2008
 * 
 *         if sql is None:
 *             if layer is None:             # <<<<<<<<<<<<<<
 *                 layer = get_default_layer(ogr_dataset)
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
*/
      }

      /* "pyogrio/_io.pyx":2010
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)             # <<<<<<<<<<<<<<
 *         else:
 *             ogr_layer = execute_sql(ogr_dataset, sql, sql_dialect)
*/
      __pyx_t_12 = __pyx_f_7pyogrio_3_io_get_ogr_layer(__pyx_cur_scope->__pyx_v_ogr_dataset, __pyx_cur_scope->__pyx_v_layer); if (unlikely(__pyx_t_12 == ((void *)NULL))) __PYX_ERR(0, 2010, __pyx_L27_error)
      __pyx_cur_scope->__pyx_v_ogr_layer = __pyx_t_12;

      /* "pyogrio/_io.pyx":2007
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
 *         if sql is None:             # <<<<<<<<<<<<<<
 *             if layer is None:
 *                 layer = get_default_layer(ogr_dataset)
*/
      goto __pyx_L30;
    }

    /* "pyogrio/_io.pyx":2012
 *             ogr_layer = get_ogr_layer(ogr_dataset, layer)
 *         else:
 *             ogr_layer = execute_sql(ogr_dataset, sql, sql_dialect)             # <<<<<<<<<<<<<<
 * 
 *         crs = get_crs(ogr_layer)
*/
    /*else*/ {
      __pyx_t_13.__pyx_n = 1;
      __pyx_t_13.sql_dialect = __pyx_cur_scope->__pyx_v_sql_dialect;
      __pyx_t_12 = __pyx_f_7pyogrio_3_io_execute_sql(__pyx_cur_scope->__pyx_v_ogr_dataset, __pyx_cur_scope->__pyx_v_sql, &__pyx_t_13); if (unlikely(__pyx_t_12 == ((void *)NULL))) __PYX_ERR(0, 2012, __pyx_L27_error)
      __pyx_cur_scope->__pyx_v_ogr_layer = __pyx_t_12;
    }
    __pyx_L30:;

    /* "pyogrio/_io.pyx":2014
 *             ogr_layer = execute_sql(ogr_dataset, sql, sql_dialect)
 * 
 *         crs = get_crs(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         # Encoding is derived from the user, from the dataset capabilities / type,
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_crs(__pyx_cur_scope->__pyx_v_ogr_layer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2014, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __pyx_cur_scope->__pyx_v_crs = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2018
 *         # Encoding is derived from the user, from the dataset capabilities / type,
 *         # or from the system locale
 *         if encoding:             # <<<<<<<<<<<<<<
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":
 *                 if "ENCODING" in dataset_kwargs:
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_encoding); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2018, __pyx_L27_error)
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2019
 *         # or from the system locale
 *         if encoding:
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *                 if "ENCODING" in dataset_kwargs:
 *                     raise ValueError(
*/
      __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_driver(__pyx_cur_scope->__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2019, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2019, __pyx_L27_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_5) {

        /* "pyogrio/_io.pyx":2020
 *         if encoding:
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":
 *                 if "ENCODING" in dataset_kwargs:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'cannot provide both encoding parameter and "ENCODING" option; '
*/
        __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_mstate_global->__pyx_n_u_ENCODING, __pyx_cur_scope->__pyx_v_dataset_kwargs, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2020, __pyx_L27_error)
        if (unlikely(__pyx_t_5)) {

          /* "pyogrio/_io.pyx":2021
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":
 *                 if "ENCODING" in dataset_kwargs:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         'cannot provide both encoding parameter and "ENCODING" option; '
 *                         "use encoding parameter to specify correct encoding for data "
*/
          __pyx_t_7 = NULL;
          __pyx_t_4 = 1;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_cannot_provide_both_encoding_par};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2021, __pyx_L27_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 2021, __pyx_L27_error)

          /* "pyogrio/_io.pyx":2020
 *         if encoding:
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":
 *                 if "ENCODING" in dataset_kwargs:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         'cannot provide both encoding parameter and "ENCODING" option; '
*/
        }

        /* "pyogrio/_io.pyx":2027
 *                     )
 * 
 *                 encoding = "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *             elif encoding.replace("-", "").upper() != "UTF8":
*/
        __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
        __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_encoding);
        __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_encoding, __pyx_mstate_global->__pyx_kp_u_UTF_8);
        __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);

        /* "pyogrio/_io.pyx":2019
 *         # or from the system locale
 *         if encoding:
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *                 if "ENCODING" in dataset_kwargs:
 *                     raise ValueError(
*/
        goto __pyx_L33;
      }

      /* "pyogrio/_io.pyx":2029
 *                 encoding = "UTF-8"
 * 
 *             elif encoding.replace("-", "").upper() != "UTF8":             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "non-UTF-8 encoding is not supported for Arrow; use the non-Arrow "
*/
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_encoding, __pyx_mstate_global->__pyx_n_u_replace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2029, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_mstate_global->__pyx_tuple[7], NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2029, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_7 = __pyx_t_14;
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_upper, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2029, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_UTF8, Py_NE)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2029, __pyx_L27_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(__pyx_t_5)) {

        /* "pyogrio/_io.pyx":2030
 * 
 *             elif encoding.replace("-", "").upper() != "UTF8":
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "non-UTF-8 encoding is not supported for Arrow; use the non-Arrow "
 *                     "interface instead"
*/
        __pyx_t_14 = NULL;
        __pyx_t_4 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_mstate_global->__pyx_kp_u_non_UTF_8_encoding_is_not_suppor};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2030, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 2030, __pyx_L27_error)

        /* "pyogrio/_io.pyx":2029
 *                 encoding = "UTF-8"
 * 
 *             elif encoding.replace("-", "").upper() != "UTF8":             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "non-UTF-8 encoding is not supported for Arrow; use the non-Arrow "
*/
      }
      __pyx_L33:;

      /* "pyogrio/_io.pyx":2018
 *         # Encoding is derived from the user, from the dataset capabilities / type,
 *         # or from the system locale
 *         if encoding:             # <<<<<<<<<<<<<<
 *             if get_driver(ogr_dataset) == "ESRI Shapefile":
 *                 if "ENCODING" in dataset_kwargs:
*/
      goto __pyx_L32;
    }

    /* "pyogrio/_io.pyx":2036
 * 
 *         else:
 *             encoding = detect_encoding(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         fields = get_fields(ogr_layer, encoding, use_arrow=True)
*/
    /*else*/ {
      __pyx_t_2 = __pyx_f_7pyogrio_3_io_detect_encoding(__pyx_cur_scope->__pyx_v_ogr_dataset, __pyx_cur_scope->__pyx_v_ogr_layer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2036, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_encoding);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_encoding, __pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L32:;

    /* "pyogrio/_io.pyx":2038
 *             encoding = detect_encoding(ogr_dataset, ogr_layer)
 * 
 *         fields = get_fields(ogr_layer, encoding, use_arrow=True)             # <<<<<<<<<<<<<<
 * 
 *         ignored_fields = []
*/
    __pyx_t_2 = __pyx_cur_scope->__pyx_v_encoding;
    __Pyx_INCREF(__pyx_t_2);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 2038, __pyx_L27_error)
    __pyx_t_15.__pyx_n = 1;
    __pyx_t_15.use_arrow = Py_True;
    __pyx_t_14 = __pyx_f_7pyogrio_3_io_get_fields(__pyx_cur_scope->__pyx_v_ogr_layer, ((PyObject*)__pyx_t_2), &__pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2038, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GIVEREF(__pyx_t_14);
    __pyx_cur_scope->__pyx_v_fields = __pyx_t_14;
    __pyx_t_14 = 0;

    /* "pyogrio/_io.pyx":2040
 *         fields = get_fields(ogr_layer, encoding, use_arrow=True)
 * 
 *         ignored_fields = []             # <<<<<<<<<<<<<<
 *         if columns is not None:
 *             # Fields are matched exactly by name, duplicates are dropped.
*/
    __pyx_t_14 = PyList_New(0); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2040, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_14);
    __pyx_cur_scope->__pyx_v_ignored_fields = ((PyObject*)__pyx_t_14);
    __pyx_t_14 = 0;

    /* "pyogrio/_io.pyx":2041
 * 
 *         ignored_fields = []
 *         if columns is not None:             # <<<<<<<<<<<<<<
 *             # Fields are matched exactly by name, duplicates are dropped.
 *             ignored_fields = list(set(fields[:, 2]) - set(columns))
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_columns != Py_None);
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2043
 *         if columns is not None:
 *             # Fields are matched exactly by name, duplicates are dropped.
 *             ignored_fields = list(set(fields[:, 2]) - set(columns))             # <<<<<<<<<<<<<<
 * 
 *             # Find index of each field in columns, and only keep those
*/
      __pyx_t_14 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[5]); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2043, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_2 = PySet_New(__pyx_t_14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2043, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = PySet_New(__pyx_cur_scope->__pyx_v_columns); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2043, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_7 = PyNumber_Subtract(__pyx_t_2, __pyx_t_14); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2043, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = __Pyx_PySequence_ListKeepNew(__pyx_t_7); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2043, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ignored_fields);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_ignored_fields, ((PyObject*)__pyx_t_14));
      __Pyx_GIVEREF(__pyx_t_14);
      __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":2046
 * 
 *             # Find index of each field in columns, and only keep those
 *             idx = np.sort(np.intersect1d(fields[:, 2], columns, return_indices=True)[1])             # <<<<<<<<<<<<<<
 *             fields = fields[idx, :]
 * 
*/
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2046, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_sort); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2046, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_16 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2046, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_17);
      __pyx_t_18 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_intersect1d); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2046, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      __pyx_t_17 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[5]); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2046, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_17);
      __pyx_t_4 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_18))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_18);
        assert(__pyx_t_16);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_18);
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_18, __pyx__function);
        __pyx_t_4 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_16, __pyx_t_17, __pyx_cur_scope->__pyx_v_columns};
        __pyx_t_19 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 2046, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_19);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_return_indices, Py_True, __pyx_t_19, __pyx_callargs+3, 0) < (0)) __PYX_ERR(0, 2046, __pyx_L27_error)
        __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_18, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_19);
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2046, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_t_18 = __Pyx_GetItemInt(__pyx_t_2, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2046, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_4 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
        __pyx_t_4 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_18};
        __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2046, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_14);
      }
      __Pyx_GIVEREF(__pyx_t_14);
      __pyx_cur_scope->__pyx_v_idx = __pyx_t_14;
      __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":2047
 *             # Find index of each field in columns, and only keep those
 *             idx = np.sort(np.intersect1d(fields[:, 2], columns, return_indices=True)[1])
 *             fields = fields[idx, :]             # <<<<<<<<<<<<<<
 * 
 *         if not read_geometry:
*/
      __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2047, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_idx);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_idx);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_cur_scope->__pyx_v_idx) != (0)) __PYX_ERR(0, 2047, __pyx_L27_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_slice[0]);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_mstate_global->__pyx_slice[0]) != (0)) __PYX_ERR(0, 2047, __pyx_L27_error);
      __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_fields, __pyx_t_14); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2047, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_fields);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_fields, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyogrio/_io.pyx":2041
 * 
 *         ignored_fields = []
 *         if columns is not None:             # <<<<<<<<<<<<<<
 *             # Fields are matched exactly by name, duplicates are dropped.
 *             ignored_fields = list(set(fields[:, 2]) - set(columns))
*/
    }

    /* "pyogrio/_io.pyx":2049
 *             fields = fields[idx, :]
 * 
 *         if not read_geometry:             # <<<<<<<<<<<<<<
 *             ignored_fields.append("OGR_GEOMETRY")
 * 
*/
    __pyx_t_5 = (!(__pyx_cur_scope->__pyx_v_read_geometry != 0));
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2050
 * 
 *         if not read_geometry:
 *             ignored_fields.append("OGR_GEOMETRY")             # <<<<<<<<<<<<<<
 * 
 *         # raise error if schema has bool values for FGB / GPKG and GDAL <3.8.3
*/
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_ignored_fields, __pyx_mstate_global->__pyx_n_u_OGR_GEOMETRY); if (unlikely(__pyx_t_20 == ((int)-1))) __PYX_ERR(0, 2050, __pyx_L27_error)

      /* "pyogrio/_io.pyx":2049
 *             fields = fields[idx, :]
 * 
 *         if not read_geometry:             # <<<<<<<<<<<<<<
 *             ignored_fields.append("OGR_GEOMETRY")
 * 
*/
    }

    /* "pyogrio/_io.pyx":2066
 *                         )
 * 
 *         geometry_type = get_geometry_type(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         geometry_name = get_string(OGR_L_GetGeometryColumn(ogr_layer))
*/
    __pyx_t_3 = __pyx_f_7pyogrio_9_geometry_get_geometry_type(__pyx_cur_scope->__pyx_v_ogr_layer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2066, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_v_geometry_type = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2068
 *         geometry_type = get_geometry_type(ogr_layer)
 * 
 *         geometry_name = get_string(OGR_L_GetGeometryColumn(ogr_layer))             # <<<<<<<<<<<<<<
 * 
 *         fid_column = get_string(OGR_L_GetFIDColumn(ogr_layer))
*/
    __pyx_t_3 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetGeometryColumn(__pyx_cur_scope->__pyx_v_ogr_layer), NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2068, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_v_geometry_name = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2070
 *         geometry_name = get_string(OGR_L_GetGeometryColumn(ogr_layer))
 * 
 *         fid_column = get_string(OGR_L_GetFIDColumn(ogr_layer))             # <<<<<<<<<<<<<<
 *         fid_column_where = fid_column
 *         # OGR_L_GetFIDColumn returns the column name if it is a custom column,
*/
    __pyx_t_3 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetFIDColumn(__pyx_cur_scope->__pyx_v_ogr_layer), NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2070, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_v_fid_column = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2071
 * 
 *         fid_column = get_string(OGR_L_GetFIDColumn(ogr_layer))
 *         fid_column_where = fid_column             # <<<<<<<<<<<<<<
 *         # OGR_L_GetFIDColumn returns the column name if it is a custom column,
 *         # or "" if not. For arrow, the default column name used to return the FID data
*/
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_fid_column);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_fid_column);
    __pyx_cur_scope->__pyx_v_fid_column_where = __pyx_cur_scope->__pyx_v_fid_column;

    /* "pyogrio/_io.pyx":2076
 *         # read is "OGC_FID". When accessing the underlying datasource like when using a
 *         # where clause, the default column name is "FID".
 *         if fid_column == "":             # <<<<<<<<<<<<<<
 *             fid_column = "OGC_FID"
 *             fid_column_where = "FID"
*/
    __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v_fid_column, __pyx_mstate_global->__pyx_kp_u__9, Py_EQ)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2076, __pyx_L27_error)
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2077
 *         # where clause, the default column name is "FID".
 *         if fid_column == "":
 *             fid_column = "OGC_FID"             # <<<<<<<<<<<<<<
 *             fid_column_where = "FID"
 * 
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_OGC_FID);
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_fid_column);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_fid_column, __pyx_mstate_global->__pyx_n_u_OGC_FID);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_OGC_FID);

      /* "pyogrio/_io.pyx":2078
 *         if fid_column == "":
 *             fid_column = "OGC_FID"
 *             fid_column_where = "FID"             # <<<<<<<<<<<<<<
 * 
 *         # Use fids list to create a where clause, as arrow doesn't support direct fid
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_FID);
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_fid_column_where);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_fid_column_where, __pyx_mstate_global->__pyx_n_u_FID);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_u_FID);

      /* "pyogrio/_io.pyx":2076
 *         # read is "OGC_FID". When accessing the underlying datasource like when using a
 *         # where clause, the default column name is "FID".
 *         if fid_column == "":             # <<<<<<<<<<<<<<
 *             fid_column = "OGC_FID"
 *             fid_column_where = "FID"
*/
    }

    /* "pyogrio/_io.pyx":2082
 *         # Use fids list to create a where clause, as arrow doesn't support direct fid
 *         # filtering.
 *         if fids is not None:             # <<<<<<<<<<<<<<
 *             IF CTE_GDAL_VERSION < (3, 8, 0):
 *                 driver = get_driver(ogr_dataset)
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_fids != Py_None);
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2093
 *                     )
 * 
 *             fids_str = ",".join([str(fid) for fid in fids])             # <<<<<<<<<<<<<<
 *             where = f"{fid_column_where} IN ({fids_str})"
 * 
*/
      { /* enter inner scope */
        __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2093, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_fids)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_fids)) {
          __pyx_t_14 = __pyx_cur_scope->__pyx_v_fids; __Pyx_INCREF(__pyx_t_14);
          __pyx_t_6 = 0;
          __pyx_t_21 = NULL;
        } else {
          __pyx_t_6 = -1; __pyx_t_14 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_fids); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2093, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_21 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_14); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 2093, __pyx_L27_error)
        }
        for (;;) {
          if (likely(!__pyx_t_21)) {
            if (likely(PyList_CheckExact(__pyx_t_14))) {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_14);
                #if !CYTHON_ASSUME_SAFE_SIZE
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2093, __pyx_L27_error)
                #endif
                if (__pyx_t_6 >= __pyx_temp) break;
              }
              __pyx_t_18 = __Pyx_PyList_GetItemRefFast(__pyx_t_14, __pyx_t_6, __Pyx_ReferenceSharing_OwnStrongReference);
              ++__pyx_t_6;
            } else {
              {
                Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_14);
                #if !CYTHON_ASSUME_SAFE_SIZE
                if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2093, __pyx_L27_error)
                #endif
                if (__pyx_t_6 >= __pyx_temp) break;
              }
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_18 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_14, __pyx_t_6));
              #else
              __pyx_t_18 = __Pyx_PySequence_ITEM(__pyx_t_14, __pyx_t_6);
              #endif
              ++__pyx_t_6;
            }
            if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2093, __pyx_L27_error)
          } else {
            __pyx_t_18 = __pyx_t_21(__pyx_t_14);
            if (unlikely(!__pyx_t_18)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2093, __pyx_L27_error)
                PyErr_Clear();
              }
              break;
            }
          }
          __Pyx_GOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_cur_scope->__pyx_8genexpr7__pyx_v_fid);
          __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_8genexpr7__pyx_v_fid, __pyx_t_18);
          __Pyx_GIVEREF(__pyx_t_18);
          __pyx_t_18 = 0;
          __pyx_t_18 = __Pyx_PyObject_Unicode(__pyx_cur_scope->__pyx_8genexpr7__pyx_v_fid); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2093, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_18);
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_18))) __PYX_ERR(0, 2093, __pyx_L27_error)
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        }
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      } /* exit inner scope */
      __pyx_t_14 = PyUnicode_Join(__pyx_mstate_global->__pyx_kp_u__11, __pyx_t_3); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2093, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GIVEREF(__pyx_t_14);
      __pyx_cur_scope->__pyx_v_fids_str = ((PyObject*)__pyx_t_14);
      __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":2094
 * 
 *             fids_str = ",".join([str(fid) for fid in fids])
 *             where = f"{fid_column_where} IN ({fids_str})"             # <<<<<<<<<<<<<<
 * 
 *         # Apply the attribute filter
*/
      __pyx_t_14 = __Pyx_PyObject_FormatSimple(__pyx_cur_scope->__pyx_v_fid_column_where, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2094, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_3 = __Pyx_PyUnicode_Unicode(__pyx_cur_scope->__pyx_v_fids_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2094, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_22[0] = __pyx_t_14;
      __pyx_t_22[1] = __pyx_mstate_global->__pyx_kp_u_IN;
      __pyx_t_22[2] = __pyx_t_3;
      __pyx_t_22[3] = __pyx_mstate_global->__pyx_kp_u__5;
      __pyx_t_18 = __Pyx_PyUnicode_Join(__pyx_t_22, 4, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14) + 5 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3));
      if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2094, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_where);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_where, __pyx_t_18);
      __Pyx_GIVEREF(__pyx_t_18);
      __pyx_t_18 = 0;

      /* "pyogrio/_io.pyx":2082
 *         # Use fids list to create a where clause, as arrow doesn't support direct fid
 *         # filtering.
 *         if fids is not None:             # <<<<<<<<<<<<<<
 *             IF CTE_GDAL_VERSION < (3, 8, 0):
 *                 driver = get_driver(ogr_dataset)
*/
    }

    /* "pyogrio/_io.pyx":2097
 * 
 *         # Apply the attribute filter
 *         if where is not None and where != "":             # <<<<<<<<<<<<<<
 *             try:
 *                 apply_where_filter(ogr_layer, where)
*/
    __pyx_t_1 = (__pyx_cur_scope->__pyx_v_where != Py_None);
    if (__pyx_t_1) {
    } else {
      __pyx_t_5 = __pyx_t_1;
      goto __pyx_L43_bool_binop_done;
    }
    __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_cur_scope->__pyx_v_where, __pyx_mstate_global->__pyx_kp_u__9, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2097, __pyx_L27_error)
    __pyx_t_5 = __pyx_t_1;
    __pyx_L43_bool_binop_done:;
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2098
 *         # Apply the attribute filter
 *         if where is not None and where != "":
 *             try:             # <<<<<<<<<<<<<<
 *                 apply_where_filter(ogr_layer, where)
 *             except ValueError as ex:
*/
      {
        __Pyx_ExceptionSave(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
        __Pyx_XGOTREF(__pyx_t_23);
        __Pyx_XGOTREF(__pyx_t_24);
        __Pyx_XGOTREF(__pyx_t_25);
        /*try:*/ {

          /* "pyogrio/_io.pyx":2099
 *         if where is not None and where != "":
 *             try:
 *                 apply_where_filter(ogr_layer, where)             # <<<<<<<<<<<<<<
 *             except ValueError as ex:
 *                 if fids is not None and str(ex).startswith("Invalid SQL query"):
*/
          __pyx_t_18 = __pyx_cur_scope->__pyx_v_where;
          __Pyx_INCREF(__pyx_t_18);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_18))||((__pyx_t_18) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_18))) __PYX_ERR(0, 2099, __pyx_L45_error)
          __pyx_t_3 = __pyx_f_7pyogrio_3_io_apply_where_filter(__pyx_cur_scope->__pyx_v_ogr_layer, ((PyObject*)__pyx_t_18)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2099, __pyx_L45_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "pyogrio/_io.pyx":2098
 *         # Apply the attribute filter
 *         if where is not None and where != "":
 *             try:             # <<<<<<<<<<<<<<
 *                 apply_where_filter(ogr_layer, where)
 *             except ValueError as ex:
*/
        }
        __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
        __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
        __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
        goto __pyx_L50_try_end;
        __pyx_L45_error:;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":2100
 *             try:
 *                 apply_where_filter(ogr_layer, where)
 *             except ValueError as ex:             # <<<<<<<<<<<<<<
 *                 if fids is not None and str(ex).startswith("Invalid SQL query"):
 *                     # If fids is not None, the where being applied is the one formatted
*/
        __pyx_t_26 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_ValueError))));
        if (__pyx_t_26) {
          __Pyx_AddTraceback("pyogrio._io.ogr_open_arrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_18, &__pyx_t_14) < 0) __PYX_ERR(0, 2100, __pyx_L47_except_error)
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_18);
          __Pyx_GIVEREF(__pyx_t_18);
          __pyx_cur_scope->__pyx_v_ex = __pyx_t_18;
          /*try:*/ {

            /* "pyogrio/_io.pyx":2101
 *                 apply_where_filter(ogr_layer, where)
 *             except ValueError as ex:
 *                 if fids is not None and str(ex).startswith("Invalid SQL query"):             # <<<<<<<<<<<<<<
 *                     # If fids is not None, the where being applied is the one formatted
 *                     # above.
*/
            __pyx_t_1 = (__pyx_cur_scope->__pyx_v_fids != Py_None);
            if (__pyx_t_1) {
            } else {
              __pyx_t_5 = __pyx_t_1;
              goto __pyx_L59_bool_binop_done;
            }
            __pyx_t_7 = __Pyx_PyObject_Unicode(__pyx_cur_scope->__pyx_v_ex); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2101, __pyx_L56_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_1 = __Pyx_PyUnicode_Tailmatch(__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_Invalid_SQL_query_2, 0, PY_SSIZE_T_MAX, -1); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 2101, __pyx_L56_error)
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_5 = __pyx_t_1;
            __pyx_L59_bool_binop_done:;
            if (unlikely(__pyx_t_5)) {

              /* "pyogrio/_io.pyx":2104
 *                     # If fids is not None, the where being applied is the one formatted
 *                     # above.
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         f"error applying filter for {len(fids)} fids; max. number for "
 *                         f"drivers with default SQL dialect 'OGRSQL' is 4997"
*/
              __pyx_t_2 = NULL;

              /* "pyogrio/_io.pyx":2105
 *                     # above.
 *                     raise ValueError(
 *                         f"error applying filter for {len(fids)} fids; max. number for "             # <<<<<<<<<<<<<<
 *                         f"drivers with default SQL dialect 'OGRSQL' is 4997"
 *                     ) from ex
*/
              __pyx_t_6 = PyObject_Length(__pyx_cur_scope->__pyx_v_fids); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2105, __pyx_L56_error)
              __pyx_t_19 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_t_6, 0, ' ', 'd'); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 2105, __pyx_L56_error)
              __Pyx_GOTREF(__pyx_t_19);
              __pyx_t_27[0] = __pyx_mstate_global->__pyx_kp_u_error_applying_filter_for;
              __pyx_t_27[1] = __pyx_t_19;
              __pyx_t_27[2] = __pyx_mstate_global->__pyx_kp_u_fids_max_number_for_drivers_wit;
              __pyx_t_17 = __Pyx_PyUnicode_Join(__pyx_t_27, 3, 26 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_19) + 72, 127);
              if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2105, __pyx_L56_error)
              __Pyx_GOTREF(__pyx_t_17);
              __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
              __pyx_t_4 = 1;
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_17};
                __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2104, __pyx_L56_error)
                __Pyx_GOTREF(__pyx_t_7);
              }

              /* "pyogrio/_io.pyx":2107
 *                         f"error applying filter for {len(fids)} fids; max. number for "
 *                         f"drivers with default SQL dialect 'OGRSQL' is 4997"
 *                     ) from ex             # <<<<<<<<<<<<<<
 * 
 *                 raise
*/
              __Pyx_Raise(__pyx_t_7, 0, 0, __pyx_cur_scope->__pyx_v_ex);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __PYX_ERR(0, 2104, __pyx_L56_error)

              /* "pyogrio/_io.pyx":2101
 *                 apply_where_filter(ogr_layer, where)
 *             except ValueError as ex:
 *                 if fids is not None and str(ex).startswith("Invalid SQL query"):             # <<<<<<<<<<<<<<
 *                     # If fids is not None, the where being applied is the one formatted
 *                     # above.
*/
            }

            /* "pyogrio/_io.pyx":2109
 *                     ) from ex
 * 
 *                 raise             # <<<<<<<<<<<<<<
 * 
 *         # Apply the spatial filter
*/
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_14);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_18, __pyx_t_14);
            __pyx_t_3 = 0;  __pyx_t_18 = 0;  __pyx_t_14 = 0; 
            __PYX_ERR(0, 2109, __pyx_L56_error)
          }

          /* "pyogrio/_io.pyx":2100
 *             try:
 *                 apply_where_filter(ogr_layer, where)
 *             except ValueError as ex:             # <<<<<<<<<<<<<<
 *                 if fids is not None and str(ex).startswith("Invalid SQL query"):
 *                     # If fids is not None, the where being applied is the one formatted
*/
          /*finally:*/ {
            __pyx_L56_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_assign
              __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
              __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
               __Pyx_ExceptionSwap(&__pyx_t_33, &__pyx_t_34, &__pyx_t_35);
              if ( unlikely(__Pyx_GetException(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32) < 0)) __Pyx_ErrFetch(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32);
              __Pyx_XGOTREF(__pyx_t_30);
              __Pyx_XGOTREF(__pyx_t_31);
              __Pyx_XGOTREF(__pyx_t_32);
              __Pyx_XGOTREF(__pyx_t_33);
              __Pyx_XGOTREF(__pyx_t_34);
              __Pyx_XGOTREF(__pyx_t_35);
              __pyx_t_26 = __pyx_lineno; __pyx_t_28 = __pyx_clineno; __pyx_t_29 = __pyx_filename;
              {
                __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ex);
                __Pyx_DECREF(__pyx_cur_scope->__pyx_v_ex); __pyx_cur_scope->__pyx_v_ex = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_33);
              __Pyx_XGIVEREF(__pyx_t_34);
              __Pyx_XGIVEREF(__pyx_t_35);
              __Pyx_ExceptionReset(__pyx_t_33, __pyx_t_34, __pyx_t_35);
              __Pyx_XGIVEREF(__pyx_t_30);
              __Pyx_XGIVEREF(__pyx_t_31);
              __Pyx_XGIVEREF(__pyx_t_32);
              __Pyx_ErrRestore(__pyx_t_30, __pyx_t_31, __pyx_t_32);
              __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0; __pyx_t_33 = 0; __pyx_t_34 = 0; __pyx_t_35 = 0;
              __pyx_lineno = __pyx_t_26; __pyx_clineno = __pyx_t_28; __pyx_filename = __pyx_t_29;
              goto __pyx_L47_except_error;
            }
          }
        }
        goto __pyx_L47_except_error;

        /* "pyogrio/_io.pyx":2098
 *         # Apply the attribute filter
 *         if where is not None and where != "":
 *             try:             # <<<<<<<<<<<<<<
 *                 apply_where_filter(ogr_layer, where)
 *             except ValueError as ex:
*/
        __pyx_L47_except_error:;
        __Pyx_XGIVEREF(__pyx_t_23);
        __Pyx_XGIVEREF(__pyx_t_24);
        __Pyx_XGIVEREF(__pyx_t_25);
        __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
        goto __pyx_L27_error;
        __pyx_L50_try_end:;
      }

      /* "pyogrio/_io.pyx":2097
 * 
 *         # Apply the attribute filter
 *         if where is not None and where != "":             # <<<<<<<<<<<<<<
 *             try:
 *                 apply_where_filter(ogr_layer, where)
*/
    }

    /* "pyogrio/_io.pyx":2112
 * 
 *         # Apply the spatial filter
 *         if bbox is not None:             # <<<<<<<<<<<<<<
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_bbox != ((PyObject*)Py_None));
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2113
 *         # Apply the spatial filter
 *         if bbox is not None:
 *             apply_bbox_filter(ogr_layer, bbox)             # <<<<<<<<<<<<<<
 * 
 *         elif mask is not None:
*/
      __pyx_t_14 = __pyx_f_7pyogrio_3_io_apply_bbox_filter(__pyx_cur_scope->__pyx_v_ogr_layer, __pyx_cur_scope->__pyx_v_bbox); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2113, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":2112
 * 
 *         # Apply the spatial filter
 *         if bbox is not None:             # <<<<<<<<<<<<<<
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
*/
      goto __pyx_L65;
    }

    /* "pyogrio/_io.pyx":2115
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
 *         elif mask is not None:             # <<<<<<<<<<<<<<
 *             apply_geometry_filter(ogr_layer, mask)
 * 
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_mask != Py_None);
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2116
 * 
 *         elif mask is not None:
 *             apply_geometry_filter(ogr_layer, mask)             # <<<<<<<<<<<<<<
 * 
 *         # Limit feature range to available range (cannot use logic of
*/
      __pyx_t_14 = __pyx_f_7pyogrio_3_io_apply_geometry_filter(__pyx_cur_scope->__pyx_v_ogr_layer, __pyx_cur_scope->__pyx_v_mask); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2116, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":2115
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
 *         elif mask is not None:             # <<<<<<<<<<<<<<
 *             apply_geometry_filter(ogr_layer, mask)
 * 
*/
    }
    __pyx_L65:;

    /* "pyogrio/_io.pyx":2120
 *         # Limit feature range to available range (cannot use logic of
 *         # `validate_feature_range` because max_features is not supported)
 *         if skip_features > 0:             # <<<<<<<<<<<<<<
 *             feature_count = get_feature_count(ogr_layer, 1)
 *             if skip_features >= feature_count:
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_skip_features > 0);
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2121
 *         # `validate_feature_range` because max_features is not supported)
 *         if skip_features > 0:
 *             feature_count = get_feature_count(ogr_layer, 1)             # <<<<<<<<<<<<<<
 *             if skip_features >= feature_count:
 *                 skip_features = feature_count
*/
      __pyx_t_14 = __pyx_f_7pyogrio_3_io_get_feature_count(__pyx_cur_scope->__pyx_v_ogr_layer, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2121, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_GIVEREF(__pyx_t_14);
      __pyx_cur_scope->__pyx_v_feature_count = __pyx_t_14;
      __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":2122
 *         if skip_features > 0:
 *             feature_count = get_feature_count(ogr_layer, 1)
 *             if skip_features >= feature_count:             # <<<<<<<<<<<<<<
 *                 skip_features = feature_count
 * 
*/
      __pyx_t_14 = __Pyx_PyLong_From_int(__pyx_cur_scope->__pyx_v_skip_features); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2122, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_18 = PyObject_RichCompare(__pyx_t_14, __pyx_cur_scope->__pyx_v_feature_count, Py_GE); __Pyx_XGOTREF(__pyx_t_18); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2122, __pyx_L27_error)
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_18); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2122, __pyx_L27_error)
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      if (__pyx_t_5) {

        /* "pyogrio/_io.pyx":2123
 *             feature_count = get_feature_count(ogr_layer, 1)
 *             if skip_features >= feature_count:
 *                 skip_features = feature_count             # <<<<<<<<<<<<<<
 * 
 *         # Limit to specified columns
*/
        __pyx_t_28 = __Pyx_PyLong_As_int(__pyx_cur_scope->__pyx_v_feature_count); if (unlikely((__pyx_t_28 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2123, __pyx_L27_error)
        __pyx_cur_scope->__pyx_v_skip_features = __pyx_t_28;

        /* "pyogrio/_io.pyx":2122
 *         if skip_features > 0:
 *             feature_count = get_feature_count(ogr_layer, 1)
 *             if skip_features >= feature_count:             # <<<<<<<<<<<<<<
 *                 skip_features = feature_count
 * 
*/
      }

      /* "pyogrio/_io.pyx":2120
 *         # Limit feature range to available range (cannot use logic of
 *         # `validate_feature_range` because max_features is not supported)
 *         if skip_features > 0:             # <<<<<<<<<<<<<<
 *             feature_count = get_feature_count(ogr_layer, 1)
 *             if skip_features >= feature_count:
*/
    }

    /* "pyogrio/_io.pyx":2126
 * 
 *         # Limit to specified columns
 *         if ignored_fields:             # <<<<<<<<<<<<<<
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")
*/
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_ignored_fields);
      if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 2126, __pyx_L27_error)
      __pyx_t_5 = (__pyx_temp != 0);
    }

    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2127
 *         # Limit to specified columns
 *         if ignored_fields:
 *             for field in ignored_fields:             # <<<<<<<<<<<<<<
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b
*/
      __pyx_t_18 = __pyx_cur_scope->__pyx_v_ignored_fields; __Pyx_INCREF(__pyx_t_18);
      __pyx_t_6 = 0;
      for (;;) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_18);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2127, __pyx_L27_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        __pyx_t_14 = __Pyx_PyList_GetItemRefFast(__pyx_t_18, __pyx_t_6, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_6;
        if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2127, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_field);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_field, __pyx_t_14);
        __Pyx_GIVEREF(__pyx_t_14);
        __pyx_t_14 = 0;

        /* "pyogrio/_io.pyx":2128
 *         if ignored_fields:
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")             # <<<<<<<<<<<<<<
 *                 field_c = field_b
 *                 fields_c = CSLAddString(fields_c, field_c)
*/
        __pyx_t_3 = __pyx_cur_scope->__pyx_v_field;
        __Pyx_INCREF(__pyx_t_3);
        __pyx_t_4 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_utf_8};
          __pyx_t_14 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2128, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_14);
        }
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_field_b);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_field_b, __pyx_t_14);
        __Pyx_GIVEREF(__pyx_t_14);
        __pyx_t_14 = 0;

        /* "pyogrio/_io.pyx":2129
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b             # <<<<<<<<<<<<<<
 *                 fields_c = CSLAddString(fields_c, field_c)
 * 
*/
        __pyx_t_36 = __Pyx_PyObject_AsString(__pyx_cur_scope->__pyx_v_field_b); if (unlikely((!__pyx_t_36) && PyErr_Occurred())) __PYX_ERR(0, 2129, __pyx_L27_error)
        __pyx_cur_scope->__pyx_v_field_c = __pyx_t_36;

        /* "pyogrio/_io.pyx":2130
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b
 *                 fields_c = CSLAddString(fields_c, field_c)             # <<<<<<<<<<<<<<
 * 
 *             with nogil:
*/
        __pyx_cur_scope->__pyx_v_fields_c = CSLAddString(__pyx_cur_scope->__pyx_v_fields_c, __pyx_cur_scope->__pyx_v_field_c);

        /* "pyogrio/_io.pyx":2127
 *         # Limit to specified columns
 *         if ignored_fields:
 *             for field in ignored_fields:             # <<<<<<<<<<<<<<
 *                 field_b = field.encode("utf-8")
 *                 field_c = field_b
*/
      }
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

      /* "pyogrio/_io.pyx":2132
 *                 fields_c = CSLAddString(fields_c, field_c)
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)
 * 
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "pyogrio/_io.pyx":2133
 * 
 *             with nogil:
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)             # <<<<<<<<<<<<<<
 * 
 *         if not return_fids:
*/
            (void)(OGR_L_SetIgnoredFields(__pyx_cur_scope->__pyx_v_ogr_layer, ((char const **)__pyx_cur_scope->__pyx_v_fields_c)));
          }

          /* "pyogrio/_io.pyx":2132
 *                 fields_c = CSLAddString(fields_c, field_c)
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)
 * 
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L74;
            }
            __pyx_L74:;
          }
      }

      /* "pyogrio/_io.pyx":2126
 * 
 *         # Limit to specified columns
 *         if ignored_fields:             # <<<<<<<<<<<<<<
 *             for field in ignored_fields:
 *                 field_b = field.encode("utf-8")
*/
    }

    /* "pyogrio/_io.pyx":2135
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)
 * 
 *         if not return_fids:             # <<<<<<<<<<<<<<
 *             options = CSLSetNameValue(options, "INCLUDE_FID", "NO")
 * 
*/
    __pyx_t_5 = (!(__pyx_cur_scope->__pyx_v_return_fids != 0));
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2136
 * 
 *         if not return_fids:
 *             options = CSLSetNameValue(options, "INCLUDE_FID", "NO")             # <<<<<<<<<<<<<<
 * 
 *         if batch_size > 0:
*/
      __pyx_cur_scope->__pyx_v_options = CSLSetNameValue(__pyx_cur_scope->__pyx_v_options, __pyx_k_INCLUDE_FID, __pyx_k_NO);

      /* "pyogrio/_io.pyx":2135
 *                 OGR_L_SetIgnoredFields(ogr_layer, <const char**>fields_c)
 * 
 *         if not return_fids:             # <<<<<<<<<<<<<<
 *             options = CSLSetNameValue(options, "INCLUDE_FID", "NO")
 * 
*/
    }

    /* "pyogrio/_io.pyx":2138
 *             options = CSLSetNameValue(options, "INCLUDE_FID", "NO")
 * 
 *         if batch_size > 0:             # <<<<<<<<<<<<<<
 *             options = CSLSetNameValue(
 *                 options,
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_batch_size > 0);
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2142
 *                 options,
 *                 "MAX_FEATURES_IN_BATCH",
 *                 str(batch_size).encode("UTF-8")             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_18 = __Pyx_PyLong_From_int(__pyx_cur_scope->__pyx_v_batch_size); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2142, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_18);
      __pyx_t_14 = __Pyx_PyObject_Unicode(__pyx_t_18); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2142, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      __pyx_t_18 = PyUnicode_AsUTF8String(((PyObject*)__pyx_t_14)); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2142, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_37 = __Pyx_PyBytes_AsString(__pyx_t_18); if (unlikely((!__pyx_t_37) && PyErr_Occurred())) __PYX_ERR(0, 2142, __pyx_L27_error)

      /* "pyogrio/_io.pyx":2139
 * 
 *         if batch_size > 0:
 *             options = CSLSetNameValue(             # <<<<<<<<<<<<<<
 *                 options,
 *                 "MAX_FEATURES_IN_BATCH",
*/
      __pyx_cur_scope->__pyx_v_options = CSLSetNameValue(__pyx_cur_scope->__pyx_v_options, __pyx_k_MAX_FEATURES_IN_BATCH, __pyx_t_37);
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

      /* "pyogrio/_io.pyx":2138
 *             options = CSLSetNameValue(options, "INCLUDE_FID", "NO")
 * 
 *         if batch_size > 0:             # <<<<<<<<<<<<<<
 *             options = CSLSetNameValue(
 *                 options,
*/
    }

    /* "pyogrio/_io.pyx":2150
 *                 options,
 *                 "GEOMETRY_METADATA_ENCODING",
 *                 "GEOARROW".encode("UTF-8")             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_37 = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_n_b_GEOARROW); if (unlikely((!__pyx_t_37) && PyErr_Occurred())) __PYX_ERR(0, 2150, __pyx_L27_error)

    /* "pyogrio/_io.pyx":2147
 *         # Default to geoarrow metadata encoding
 *         IF CTE_GDAL_VERSION >= (3, 8, 0):
 *             options = CSLSetNameValue(             # <<<<<<<<<<<<<<
 *                 options,
 *                 "GEOMETRY_METADATA_ENCODING",
*/
    __pyx_cur_scope->__pyx_v_options = CSLSetNameValue(__pyx_cur_scope->__pyx_v_options, __pyx_k_GEOMETRY_METADATA_ENCODING, __pyx_t_37);

    /* "pyogrio/_io.pyx":2156
 *         # quite limited regarding support for mixed time zones,...
 *         IF CTE_GDAL_VERSION >= (3, 11, 0):
 *             if datetime_as_string:             # <<<<<<<<<<<<<<
 *                 options = CSLSetNameValue(options, "DATETIME_AS_STRING", "YES")
 * 
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_datetime_as_string); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2156, __pyx_L27_error)
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2157
 *         IF CTE_GDAL_VERSION >= (3, 11, 0):
 *             if datetime_as_string:
 *                 options = CSLSetNameValue(options, "DATETIME_AS_STRING", "YES")             # <<<<<<<<<<<<<<
 * 
 *         # make sure layer is read from beginning
*/
      __pyx_cur_scope->__pyx_v_options = CSLSetNameValue(__pyx_cur_scope->__pyx_v_options, __pyx_k_DATETIME_AS_STRING, __pyx_k_YES);

      /* "pyogrio/_io.pyx":2156
 *         # quite limited regarding support for mixed time zones,...
 *         IF CTE_GDAL_VERSION >= (3, 11, 0):
 *             if datetime_as_string:             # <<<<<<<<<<<<<<
 *                 options = CSLSetNameValue(options, "DATETIME_AS_STRING", "YES")
 * 
*/
    }

    /* "pyogrio/_io.pyx":2160
 * 
 *         # make sure layer is read from beginning
 *         with nogil:             # <<<<<<<<<<<<<<
 *             OGR_L_ResetReading(ogr_layer)
 * 
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "pyogrio/_io.pyx":2161
 *         # make sure layer is read from beginning
 *         with nogil:
 *             OGR_L_ResetReading(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         # allocate the stream struct and wrap in capsule to ensure clean-up on error
*/
          OGR_L_ResetReading(__pyx_cur_scope->__pyx_v_ogr_layer);
        }

        /* "pyogrio/_io.pyx":2160
 * 
 *         # make sure layer is read from beginning
 *         with nogil:             # <<<<<<<<<<<<<<
 *             OGR_L_ResetReading(ogr_layer)
 * 
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L80;
          }
          __pyx_L80:;
        }
    }

    /* "pyogrio/_io.pyx":2164
 * 
 *         # allocate the stream struct and wrap in capsule to ensure clean-up on error
 *         capsule = alloc_c_stream(&stream)             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_18 = __pyx_f_7pyogrio_3_io_alloc_c_stream((&__pyx_cur_scope->__pyx_v_stream)); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2164, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_18);
    __Pyx_GIVEREF(__pyx_t_18);
    __pyx_cur_scope->__pyx_v_capsule = __pyx_t_18;
    __pyx_t_18 = 0;

    /* "pyogrio/_io.pyx":2166
 *         capsule = alloc_c_stream(&stream)
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             reval = OGR_L_GetArrowStream(ogr_layer, stream, options)
 *         if not reval:
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "pyogrio/_io.pyx":2167
 * 
 *         with nogil:
 *             reval = OGR_L_GetArrowStream(ogr_layer, stream, options)             # <<<<<<<<<<<<<<
 *         if not reval:
 *             raise RuntimeError("Failed to open ArrowArrayStream from Layer")
*/
          __pyx_cur_scope->__pyx_v_reval = OGR_L_GetArrowStream(__pyx_cur_scope->__pyx_v_ogr_layer, __pyx_cur_scope->__pyx_v_stream, __pyx_cur_scope->__pyx_v_options);
        }

        /* "pyogrio/_io.pyx":2166
 *         capsule = alloc_c_stream(&stream)
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             reval = OGR_L_GetArrowStream(ogr_layer, stream, options)
 *         if not reval:
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L83;
          }
          __pyx_L83:;
        }
    }

    /* "pyogrio/_io.pyx":2168
 *         with nogil:
 *             reval = OGR_L_GetArrowStream(ogr_layer, stream, options)
 *         if not reval:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Failed to open ArrowArrayStream from Layer")
 * 
*/
    __pyx_t_5 = (!__pyx_cur_scope->__pyx_v_reval);
    if (unlikely(__pyx_t_5)) {

      /* "pyogrio/_io.pyx":2169
 *             reval = OGR_L_GetArrowStream(ogr_layer, stream, options)
 *         if not reval:
 *             raise RuntimeError("Failed to open ArrowArrayStream from Layer")             # <<<<<<<<<<<<<<
 * 
 *         if skip_features > 0:
*/
      __pyx_t_14 = NULL;
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_mstate_global->__pyx_kp_u_Failed_to_open_ArrowArrayStream};
        __pyx_t_18 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2169, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_18);
      }
      __Pyx_Raise(__pyx_t_18, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      __PYX_ERR(0, 2169, __pyx_L27_error)

      /* "pyogrio/_io.pyx":2168
 *         with nogil:
 *             reval = OGR_L_GetArrowStream(ogr_layer, stream, options)
 *         if not reval:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Failed to open ArrowArrayStream from Layer")
 * 
*/
    }

    /* "pyogrio/_io.pyx":2171
 *             raise RuntimeError("Failed to open ArrowArrayStream from Layer")
 * 
 *         if skip_features > 0:             # <<<<<<<<<<<<<<
 *             # only supported for GDAL >= 3.8.0; have to do this after getting
 *             # the Arrow stream
*/
    __pyx_t_5 = (__pyx_cur_scope->__pyx_v_skip_features > 0);
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2176
 *             # use `OGR_L_SetNextByIndex` directly and not `apply_skip_features`
 *             # to ignore errors in case skip_features == feature_count
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 OGR_L_SetNextByIndex(ogr_layer, skip_features)
 * 
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "pyogrio/_io.pyx":2177
 *             # to ignore errors in case skip_features == feature_count
 *             with nogil:
 *                 OGR_L_SetNextByIndex(ogr_layer, skip_features)             # <<<<<<<<<<<<<<
 * 
 *         if use_pyarrow:
*/
            (void)(OGR_L_SetNextByIndex(__pyx_cur_scope->__pyx_v_ogr_layer, __pyx_cur_scope->__pyx_v_skip_features));
          }

          /* "pyogrio/_io.pyx":2176
 *             # use `OGR_L_SetNextByIndex` directly and not `apply_skip_features`
 *             # to ignore errors in case skip_features == feature_count
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 OGR_L_SetNextByIndex(ogr_layer, skip_features)
 * 
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L88;
            }
            __pyx_L88:;
          }
      }

      /* "pyogrio/_io.pyx":2171
 *             raise RuntimeError("Failed to open ArrowArrayStream from Layer")
 * 
 *         if skip_features > 0:             # <<<<<<<<<<<<<<
 *             # only supported for GDAL >= 3.8.0; have to do this after getting
 *             # the Arrow stream
*/
    }

    /* "pyogrio/_io.pyx":2179
 *                 OGR_L_SetNextByIndex(ogr_layer, skip_features)
 * 
 *         if use_pyarrow:             # <<<<<<<<<<<<<<
 *             import pyarrow as pa
 * 
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_use_pyarrow); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 2179, __pyx_L27_error)
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":2180
 * 
 *         if use_pyarrow:
 *             import pyarrow as pa             # <<<<<<<<<<<<<<
 * 
 *             reader = pa.RecordBatchStreamReader._import_from_c(<uintptr_t> stream)
*/
      __pyx_t_25 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pyarrow, 0, 0, NULL, 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 2180, __pyx_L27_error)
      __pyx_t_18 = __pyx_t_25;
      __Pyx_GOTREF(__pyx_t_18);
      __Pyx_GIVEREF(__pyx_t_18);
      __pyx_cur_scope->__pyx_v_pa = __pyx_t_18;
      __pyx_t_18 = 0;

      /* "pyogrio/_io.pyx":2182
 *             import pyarrow as pa
 * 
 *             reader = pa.RecordBatchStreamReader._import_from_c(<uintptr_t> stream)             # <<<<<<<<<<<<<<
 *         else:
 *             reader = _ArrowStream(capsule)
*/
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_pa, __pyx_mstate_global->__pyx_n_u_RecordBatchStreamReader); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2182, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = __pyx_t_3;
      __Pyx_INCREF(__pyx_t_14);
      __pyx_t_7 = __Pyx_PyLong_FromSize_t(((uintptr_t)__pyx_cur_scope->__pyx_v_stream)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2182, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_7};
        __pyx_t_18 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_import_from_c, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2182, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_18);
      }
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_reader);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_reader, __pyx_t_18);
      __Pyx_GIVEREF(__pyx_t_18);
      __pyx_t_18 = 0;

      /* "pyogrio/_io.pyx":2179
 *                 OGR_L_SetNextByIndex(ogr_layer, skip_features)
 * 
 *         if use_pyarrow:             # <<<<<<<<<<<<<<
 *             import pyarrow as pa
 * 
*/
      goto __pyx_L89;
    }

    /* "pyogrio/_io.pyx":2184
 *             reader = pa.RecordBatchStreamReader._import_from_c(<uintptr_t> stream)
 *         else:
 *             reader = _ArrowStream(capsule)             # <<<<<<<<<<<<<<
 * 
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
*/
    /*else*/ {
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ArrowStream); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2184, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_4 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_4 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_capsule};
        __pyx_t_18 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2184, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_18);
      }
      __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_reader);
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_reader, __pyx_t_18);
      __Pyx_GIVEREF(__pyx_t_18);
      __pyx_t_18 = 0;
    }
    __pyx_L89:;

    /* "pyogrio/_io.pyx":2186
 *             reader = _ArrowStream(capsule)
 * 
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]             # <<<<<<<<<<<<<<
 *         ogr_subtypes = [
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields
*/
    { /* enter inner scope */
      __pyx_t_18 = PyList_New(0); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2186, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_18);
      if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_fields)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_fields)) {
        __pyx_t_7 = __pyx_cur_scope->__pyx_v_fields; __Pyx_INCREF(__pyx_t_7);
        __pyx_t_6 = 0;
        __pyx_t_21 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_fields); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2186, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_21 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 2186, __pyx_L27_error)
      }
      for (;;) {
        if (likely(!__pyx_t_21)) {
          if (likely(PyList_CheckExact(__pyx_t_7))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_7);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2186, __pyx_L27_error)
              #endif
              if (__pyx_t_6 >= __pyx_temp) break;
            }
            __pyx_t_3 = __Pyx_PyList_GetItemRefFast(__pyx_t_7, __pyx_t_6, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_6;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_7);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2186, __pyx_L27_error)
              #endif
              if (__pyx_t_6 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_6));
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_6);
            #endif
            ++__pyx_t_6;
          }
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2186, __pyx_L27_error)
        } else {
          __pyx_t_3 = __pyx_t_21(__pyx_t_7);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2186, __pyx_L27_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_8genexpr8__pyx_v_field);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_8genexpr8__pyx_v_field, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_14 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_FIELD_TYPE_NAMES); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2186, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2186, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        __pyx_t_17 = __Pyx_GetItemInt(__pyx_cur_scope->__pyx_8genexpr8__pyx_v_field, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2186, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_4 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_2);
          assert(__pyx_t_14);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
          __pyx_t_4 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_t_17, __pyx_mstate_global->__pyx_n_u_Unknown};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2186, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_18, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 2186, __pyx_L27_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } /* exit inner scope */
    __Pyx_GIVEREF(__pyx_t_18);
    __pyx_cur_scope->__pyx_v_ogr_types = ((PyObject*)__pyx_t_18);
    __pyx_t_18 = 0;

    /* "pyogrio/_io.pyx":2187
 * 
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
 *         ogr_subtypes = [             # <<<<<<<<<<<<<<
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields
 *         ]
*/
    { /* enter inner scope */
      __pyx_t_18 = PyList_New(0); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2187, __pyx_L27_error)
      __Pyx_GOTREF(__pyx_t_18);

      /* "pyogrio/_io.pyx":2188
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
 *         ogr_subtypes = [
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields             # <<<<<<<<<<<<<<
 *         ]
 * 
*/
      if (likely(PyList_CheckExact(__pyx_cur_scope->__pyx_v_fields)) || PyTuple_CheckExact(__pyx_cur_scope->__pyx_v_fields)) {
        __pyx_t_7 = __pyx_cur_scope->__pyx_v_fields; __Pyx_INCREF(__pyx_t_7);
        __pyx_t_6 = 0;
        __pyx_t_21 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_cur_scope->__pyx_v_fields); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2188, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_21 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_7); if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 2188, __pyx_L27_error)
      }
      for (;;) {
        if (likely(!__pyx_t_21)) {
          if (likely(PyList_CheckExact(__pyx_t_7))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_7);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2188, __pyx_L27_error)
              #endif
              if (__pyx_t_6 >= __pyx_temp) break;
            }
            __pyx_t_3 = __Pyx_PyList_GetItemRefFast(__pyx_t_7, __pyx_t_6, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_6;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_7);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2188, __pyx_L27_error)
              #endif
              if (__pyx_t_6 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_6));
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_7, __pyx_t_6);
            #endif
            ++__pyx_t_6;
          }
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2188, __pyx_L27_error)
        } else {
          __pyx_t_3 = __pyx_t_21(__pyx_t_7);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2188, __pyx_L27_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_cur_scope->__pyx_8genexpr9__pyx_v_field);
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_8genexpr9__pyx_v_field, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_2 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_FIELD_SUBTYPE_NAMES); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2188, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2188, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
        __pyx_t_17 = __Pyx_GetItemInt(__pyx_cur_scope->__pyx_8genexpr9__pyx_v_field, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2188, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_17);
        __pyx_t_4 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_14);
          assert(__pyx_t_2);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
          __pyx_t_4 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_17, __pyx_mstate_global->__pyx_n_u_Unknown};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2188, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_18, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 2187, __pyx_L27_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } /* exit inner scope */
    __Pyx_GIVEREF(__pyx_t_18);
    __pyx_cur_scope->__pyx_v_ogr_subtypes = ((PyObject*)__pyx_t_18);
    __pyx_t_18 = 0;

    /* "pyogrio/_io.pyx":2192
 * 
 *         meta = {
 *             "crs": crs,             # <<<<<<<<<<<<<<
 *             "encoding": encoding,
 *             "fields": fields[:, 2],
*/
    __pyx_t_18 = __Pyx_PyDict_NewPresized(9); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2192, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_18);
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_crs, __pyx_cur_scope->__pyx_v_crs) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)

    /* "pyogrio/_io.pyx":2193
 *         meta = {
 *             "crs": crs,
 *             "encoding": encoding,             # <<<<<<<<<<<<<<
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],
*/
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_encoding, __pyx_cur_scope->__pyx_v_encoding) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)

    /* "pyogrio/_io.pyx":2194
 *             "crs": crs,
 *             "encoding": encoding,
 *             "fields": fields[:, 2],             # <<<<<<<<<<<<<<
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,
*/
    __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[5]); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2194, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_fields, __pyx_t_7) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":2195
 *             "encoding": encoding,
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],             # <<<<<<<<<<<<<<
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,
*/
    __pyx_t_7 = __Pyx_PyObject_GetItem(__pyx_cur_scope->__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[6]); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2195, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_dtypes, __pyx_t_7) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "pyogrio/_io.pyx":2196
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,             # <<<<<<<<<<<<<<
 *             "ogr_subtypes": ogr_subtypes,
 *             "geometry_type": geometry_type,
*/
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_ogr_types, __pyx_cur_scope->__pyx_v_ogr_types) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)

    /* "pyogrio/_io.pyx":2197
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,             # <<<<<<<<<<<<<<
 *             "geometry_type": geometry_type,
 *             "geometry_name": geometry_name,
*/
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_ogr_subtypes, __pyx_cur_scope->__pyx_v_ogr_subtypes) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)

    /* "pyogrio/_io.pyx":2198
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,
 *             "geometry_type": geometry_type,             # <<<<<<<<<<<<<<
 *             "geometry_name": geometry_name,
 *             "fid_column": fid_column,
*/
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_geometry_type, __pyx_cur_scope->__pyx_v_geometry_type) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)

    /* "pyogrio/_io.pyx":2199
 *             "ogr_subtypes": ogr_subtypes,
 *             "geometry_type": geometry_type,
 *             "geometry_name": geometry_name,             # <<<<<<<<<<<<<<
 *             "fid_column": fid_column,
 *         }
*/
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_geometry_name, __pyx_cur_scope->__pyx_v_geometry_name) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)

    /* "pyogrio/_io.pyx":2200
 *             "geometry_type": geometry_type,
 *             "geometry_name": geometry_name,
 *             "fid_column": fid_column,             # <<<<<<<<<<<<<<
 *         }
 * 
*/
    if (PyDict_SetItem(__pyx_t_18, __pyx_mstate_global->__pyx_n_u_fid_column, __pyx_cur_scope->__pyx_v_fid_column) < (0)) __PYX_ERR(0, 2192, __pyx_L27_error)
    __Pyx_GIVEREF(__pyx_t_18);
    __pyx_cur_scope->__pyx_v_meta = ((PyObject*)__pyx_t_18);
    __pyx_t_18 = 0;

    /* "pyogrio/_io.pyx":2204
 * 
 *         # stream has to be consumed before the Dataset is closed
 *         yield meta, reader             # <<<<<<<<<<<<<<
 * 
 *     finally:
*/
    __pyx_t_18 = PyTuple_New(2); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2204, __pyx_L27_error)
    __Pyx_GOTREF(__pyx_t_18);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_meta);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_meta);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_18, 0, __pyx_cur_scope->__pyx_v_meta) != (0)) __PYX_ERR(0, 2204, __pyx_L27_error);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_reader);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_reader);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_18, 1, __pyx_cur_scope->__pyx_v_reader) != (0)) __PYX_ERR(0, 2204, __pyx_L27_error);
    __pyx_r = __pyx_t_18;
    __pyx_t_18 = 0;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L96_resume_from_yield:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 2204, __pyx_L27_error)
  }

  /* "pyogrio/_io.pyx":2207
 * 
 *     finally:
 *         if use_pyarrow and reader is not None:             # <<<<<<<<<<<<<<
 *             # Mark reader as closed to prevent reading batches
 *             reader.close()
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_use_pyarrow); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2207, __pyx_L1_error)
      if (__pyx_t_1) {
      } else {
        __pyx_t_5 = __pyx_t_1;
        goto __pyx_L98_bool_binop_done;
      }
      __pyx_t_1 = (__pyx_cur_scope->__pyx_v_reader != Py_None);
      __pyx_t_5 = __pyx_t_1;
      __pyx_L98_bool_binop_done:;
      if (__pyx_t_5) {

        /* "pyogrio/_io.pyx":2209
 *         if use_pyarrow and reader is not None:
 *             # Mark reader as closed to prevent reading batches
 *             reader.close()             # <<<<<<<<<<<<<<
 * 
 *         # `stream` will be freed through `capsule` destructor
*/
        __pyx_t_7 = __pyx_cur_scope->__pyx_v_reader;
        __Pyx_INCREF(__pyx_t_7);
        __pyx_t_4 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
          __pyx_t_18 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_close, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2209, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_18);
        }
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

        /* "pyogrio/_io.pyx":2207
 * 
 *     finally:
 *         if use_pyarrow and reader is not None:             # <<<<<<<<<<<<<<
 *             # Mark reader as closed to prevent reading batches
 *             reader.close()
*/
      }

      /* "pyogrio/_io.pyx":2213
 *         # `stream` will be freed through `capsule` destructor
 * 
 *         CSLDestroy(options)             # <<<<<<<<<<<<<<
 *         if fields_c != NULL:
 *             CSLDestroy(fields_c)
*/
      CSLDestroy(__pyx_cur_scope->__pyx_v_options);

      /* "pyogrio/_io.pyx":2214
 * 
 *         CSLDestroy(options)
 *         if fields_c != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(fields_c)
 *             fields_c = NULL
*/
      __pyx_t_5 = (__pyx_cur_scope->__pyx_v_fields_c != NULL);
      if (__pyx_t_5) {

        /* "pyogrio/_io.pyx":2215
 *         CSLDestroy(options)
 *         if fields_c != NULL:
 *             CSLDestroy(fields_c)             # <<<<<<<<<<<<<<
 *             fields_c = NULL
 * 
*/
        CSLDestroy(__pyx_cur_scope->__pyx_v_fields_c);

        /* "pyogrio/_io.pyx":2216
 *         if fields_c != NULL:
 *             CSLDestroy(fields_c)
 *             fields_c = NULL             # <<<<<<<<<<<<<<
 * 
 *         if dataset_options != NULL:
*/
        __pyx_cur_scope->__pyx_v_fields_c = NULL;

        /* "pyogrio/_io.pyx":2214
 * 
 *         CSLDestroy(options)
 *         if fields_c != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(fields_c)
 *             fields_c = NULL
*/
      }

      /* "pyogrio/_io.pyx":2218
 *             fields_c = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
      __pyx_t_5 = (__pyx_cur_scope->__pyx_v_dataset_options != NULL);
      if (__pyx_t_5) {

        /* "pyogrio/_io.pyx":2219
 * 
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
        CSLDestroy(__pyx_cur_scope->__pyx_v_dataset_options);

        /* "pyogrio/_io.pyx":2220
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
        __pyx_cur_scope->__pyx_v_dataset_options = NULL;

        /* "pyogrio/_io.pyx":2218
 *             fields_c = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
      }

      /* "pyogrio/_io.pyx":2222
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
      __pyx_t_5 = (__pyx_cur_scope->__pyx_v_ogr_dataset != NULL);
      if (__pyx_t_5) {

        /* "pyogrio/_io.pyx":2223
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
        __pyx_t_5 = (__pyx_cur_scope->__pyx_v_sql != ((PyObject*)Py_None));
        if (__pyx_t_5) {

          /* "pyogrio/_io.pyx":2224
 *         if ogr_dataset != NULL:
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *             GDALClose(ogr_dataset)
*/
          GDALDatasetReleaseResultSet(__pyx_cur_scope->__pyx_v_ogr_dataset, __pyx_cur_scope->__pyx_v_ogr_layer);

          /* "pyogrio/_io.pyx":2223
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
        }

        /* "pyogrio/_io.pyx":2226
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
        (void)(GDALClose(__pyx_cur_scope->__pyx_v_ogr_dataset));

        /* "pyogrio/_io.pyx":2227
 * 
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
*/
        __pyx_cur_scope->__pyx_v_ogr_dataset = NULL;

        /* "pyogrio/_io.pyx":2222
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
      }

      /* "pyogrio/_io.pyx":2230
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
      if (__pyx_cur_scope->__pyx_v_override_shape_encoding) {

        /* "pyogrio/_io.pyx":2231
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *             if prev_shape_encoding != NULL:
*/
        CPLSetThreadLocalConfigOption(__pyx_k_SHAPE_ENCODING, __pyx_cur_scope->__pyx_v_prev_shape_encoding);

        /* "pyogrio/_io.pyx":2233
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
        __pyx_t_5 = (__pyx_cur_scope->__pyx_v_prev_shape_encoding != NULL);
        if (__pyx_t_5) {

          /* "pyogrio/_io.pyx":2234
 * 
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)             # <<<<<<<<<<<<<<
 *                 prev_shape_encoding = NULL
 * 
*/
          CPLFree(((void *)__pyx_cur_scope->__pyx_v_prev_shape_encoding));

          /* "pyogrio/_io.pyx":2235
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
          __pyx_cur_scope->__pyx_v_prev_shape_encoding = NULL;

          /* "pyogrio/_io.pyx":2233
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
        }

        /* "pyogrio/_io.pyx":2230
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
      }

      /* "pyogrio/_io.pyx":2237
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      if (__pyx_cur_scope->__pyx_v_use_tmp_vsimem) {

        /* "pyogrio/_io.pyx":2238
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 * 
*/
        __pyx_t_18 = __pyx_cur_scope->__pyx_v_path;
        __Pyx_INCREF(__pyx_t_18);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_18))||((__pyx_t_18) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_18))) __PYX_ERR(0, 2238, __pyx_L1_error)
        __pyx_t_7 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_18), 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2238, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "pyogrio/_io.pyx":2237
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      }
      goto __pyx_L28;
    }
    __pyx_L27_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_assign
      __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_35, &__pyx_t_34, &__pyx_t_33);
      if ( unlikely(__Pyx_GetException(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23) < 0)) __Pyx_ErrFetch(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23);
      __Pyx_XGOTREF(__pyx_t_25);
      __Pyx_XGOTREF(__pyx_t_24);
      __Pyx_XGOTREF(__pyx_t_23);
      __Pyx_XGOTREF(__pyx_t_35);
      __Pyx_XGOTREF(__pyx_t_34);
      __Pyx_XGOTREF(__pyx_t_33);
      __pyx_t_28 = __pyx_lineno; __pyx_t_26 = __pyx_clineno; __pyx_t_38 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":2207
 * 
 *     finally:
 *         if use_pyarrow and reader is not None:             # <<<<<<<<<<<<<<
 *             # Mark reader as closed to prevent reading batches
 *             reader.close()
*/
        __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_use_pyarrow); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2207, __pyx_L108_error)
        if (__pyx_t_1) {
        } else {
          __pyx_t_5 = __pyx_t_1;
          goto __pyx_L110_bool_binop_done;
        }
        __pyx_t_1 = (__pyx_cur_scope->__pyx_v_reader != Py_None);
        __pyx_t_5 = __pyx_t_1;
        __pyx_L110_bool_binop_done:;
        if (__pyx_t_5) {

          /* "pyogrio/_io.pyx":2209
 *         if use_pyarrow and reader is not None:
 *             # Mark reader as closed to prevent reading batches
 *             reader.close()             # <<<<<<<<<<<<<<
 * 
 *         # `stream` will be freed through `capsule` destructor
*/
          __pyx_t_18 = __pyx_cur_scope->__pyx_v_reader;
          __Pyx_INCREF(__pyx_t_18);
          __pyx_t_4 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_18, NULL};
            __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_close, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2209, __pyx_L108_error)
            __Pyx_GOTREF(__pyx_t_7);
          }
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

          /* "pyogrio/_io.pyx":2207
 * 
 *     finally:
 *         if use_pyarrow and reader is not None:             # <<<<<<<<<<<<<<
 *             # Mark reader as closed to prevent reading batches
 *             reader.close()
*/
        }

        /* "pyogrio/_io.pyx":2213
 *         # `stream` will be freed through `capsule` destructor
 * 
 *         CSLDestroy(options)             # <<<<<<<<<<<<<<
 *         if fields_c != NULL:
 *             CSLDestroy(fields_c)
*/
        CSLDestroy(__pyx_cur_scope->__pyx_v_options);

        /* "pyogrio/_io.pyx":2214
 * 
 *         CSLDestroy(options)
 *         if fields_c != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(fields_c)
 *             fields_c = NULL
*/
        __pyx_t_5 = (__pyx_cur_scope->__pyx_v_fields_c != NULL);
        if (__pyx_t_5) {

          /* "pyogrio/_io.pyx":2215
 *         CSLDestroy(options)
 *         if fields_c != NULL:
 *             CSLDestroy(fields_c)             # <<<<<<<<<<<<<<
 *             fields_c = NULL
 * 
*/
          CSLDestroy(__pyx_cur_scope->__pyx_v_fields_c);

          /* "pyogrio/_io.pyx":2216
 *         if fields_c != NULL:
 *             CSLDestroy(fields_c)
 *             fields_c = NULL             # <<<<<<<<<<<<<<
 * 
 *         if dataset_options != NULL:
*/
          __pyx_cur_scope->__pyx_v_fields_c = NULL;

          /* "pyogrio/_io.pyx":2214
 * 
 *         CSLDestroy(options)
 *         if fields_c != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(fields_c)
 *             fields_c = NULL
*/
        }

        /* "pyogrio/_io.pyx":2218
 *             fields_c = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        __pyx_t_5 = (__pyx_cur_scope->__pyx_v_dataset_options != NULL);
        if (__pyx_t_5) {

          /* "pyogrio/_io.pyx":2219
 * 
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
          CSLDestroy(__pyx_cur_scope->__pyx_v_dataset_options);

          /* "pyogrio/_io.pyx":2220
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
          __pyx_cur_scope->__pyx_v_dataset_options = NULL;

          /* "pyogrio/_io.pyx":2218
 *             fields_c = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        }

        /* "pyogrio/_io.pyx":2222
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
        __pyx_t_5 = (__pyx_cur_scope->__pyx_v_ogr_dataset != NULL);
        if (__pyx_t_5) {

          /* "pyogrio/_io.pyx":2223
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
          __pyx_t_5 = (__pyx_cur_scope->__pyx_v_sql != ((PyObject*)Py_None));
          if (__pyx_t_5) {

            /* "pyogrio/_io.pyx":2224
 *         if ogr_dataset != NULL:
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *             GDALClose(ogr_dataset)
*/
            GDALDatasetReleaseResultSet(__pyx_cur_scope->__pyx_v_ogr_dataset, __pyx_cur_scope->__pyx_v_ogr_layer);

            /* "pyogrio/_io.pyx":2223
 * 
 *         if ogr_dataset != NULL:
 *             if sql is not None:             # <<<<<<<<<<<<<<
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
*/
          }

          /* "pyogrio/_io.pyx":2226
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
 * 
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
          (void)(GDALClose(__pyx_cur_scope->__pyx_v_ogr_dataset));

          /* "pyogrio/_io.pyx":2227
 * 
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
*/
          __pyx_cur_scope->__pyx_v_ogr_dataset = NULL;

          /* "pyogrio/_io.pyx":2222
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             if sql is not None:
 *                 GDALDatasetReleaseResultSet(ogr_dataset, ogr_layer)
*/
        }

        /* "pyogrio/_io.pyx":2230
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
        if (__pyx_cur_scope->__pyx_v_override_shape_encoding) {

          /* "pyogrio/_io.pyx":2231
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *             if prev_shape_encoding != NULL:
*/
          CPLSetThreadLocalConfigOption(__pyx_k_SHAPE_ENCODING, __pyx_cur_scope->__pyx_v_prev_shape_encoding);

          /* "pyogrio/_io.pyx":2233
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
          __pyx_t_5 = (__pyx_cur_scope->__pyx_v_prev_shape_encoding != NULL);
          if (__pyx_t_5) {

            /* "pyogrio/_io.pyx":2234
 * 
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)             # <<<<<<<<<<<<<<
 *                 prev_shape_encoding = NULL
 * 
*/
            CPLFree(((void *)__pyx_cur_scope->__pyx_v_prev_shape_encoding));

            /* "pyogrio/_io.pyx":2235
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
            __pyx_cur_scope->__pyx_v_prev_shape_encoding = NULL;

            /* "pyogrio/_io.pyx":2233
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 *                 prev_shape_encoding = NULL
*/
          }

          /* "pyogrio/_io.pyx":2230
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
        }

        /* "pyogrio/_io.pyx":2237
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        if (__pyx_cur_scope->__pyx_v_use_tmp_vsimem) {

          /* "pyogrio/_io.pyx":2238
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 * 
*/
          if (unlikely(!__pyx_cur_scope->__pyx_v_path)) { __Pyx_RaiseUnboundLocalError("path"); __PYX_ERR(0, 2238, __pyx_L108_error) }
          __pyx_t_7 = __pyx_cur_scope->__pyx_v_path;
          __Pyx_INCREF(__pyx_t_7);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_7))) __PYX_ERR(0, 2238, __pyx_L108_error)
          __pyx_t_18 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_7), 0); if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 2238, __pyx_L108_error)
          __Pyx_GOTREF(__pyx_t_18);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;

          /* "pyogrio/_io.pyx":2237
 *                 prev_shape_encoding = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_35);
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_ExceptionReset(__pyx_t_35, __pyx_t_34, __pyx_t_33);
      __Pyx_XGIVEREF(__pyx_t_25);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_ErrRestore(__pyx_t_25, __pyx_t_24, __pyx_t_23);
      __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
      __pyx_lineno = __pyx_t_28; __pyx_clineno = __pyx_t_26; __pyx_filename = __pyx_t_38;
      goto __pyx_L1_error;
      __pyx_L108_error:;
      __Pyx_XGIVEREF(__pyx_t_35);
      __Pyx_XGIVEREF(__pyx_t_34);
      __Pyx_XGIVEREF(__pyx_t_33);
      __Pyx_ExceptionReset(__pyx_t_35, __pyx_t_34, __pyx_t_33);
      __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
      __Pyx_XDECREF(__pyx_t_24); __pyx_t_24 = 0;
      __Pyx_XDECREF(__pyx_t_23); __pyx_t_23 = 0;
      __pyx_t_35 = 0; __pyx_t_34 = 0; __pyx_t_33 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L28:;
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "pyogrio/_io.pyx":1908
 * 
 * 
 * @contextlib.contextmanager             # <<<<<<<<<<<<<<
 * def ogr_open_arrow(
 *     object path_or_buffer,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("ogr_open_arrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2241
 * 
 * 
 * def ogr_read_bounds(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object layer=None,
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_6ogr_read_bounds(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_5ogr_read_bounds, "Read bounds of features in the dataset.");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_6ogr_read_bounds = {"ogr_read_bounds", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_6ogr_read_bounds, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_5ogr_read_bounds};
static PyObject *__pyx_pw_7pyogrio_3_io_6ogr_read_bounds(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path_or_buffer = 0;
  PyObject *__pyx_v_layer = 0;
  CYTHON_UNUSED PyObject *__pyx_v_encoding = 0;
  CYTHON_UNUSED int __pyx_v_read_geometry;
  CYTHON_UNUSED int __pyx_v_force_2d;
  CYTHON_UNUSED PyObject *__pyx_v_columns = 0;
  int __pyx_v_skip_features;
  int __pyx_v_max_features;
  PyObject *__pyx_v_where = 0;
  PyObject *__pyx_v_bbox = 0;
  PyObject *__pyx_v_mask = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_read_bounds (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path_or_buffer,&__pyx_mstate_global->__pyx_n_u_layer,&__pyx_mstate_global->__pyx_n_u_encoding,&__pyx_mstate_global->__pyx_n_u_read_geometry,&__pyx_mstate_global->__pyx_n_u_force_2d,&__pyx_mstate_global->__pyx_n_u_columns,&__pyx_mstate_global->__pyx_n_u_skip_features,&__pyx_mstate_global->__pyx_n_u_max_features,&__pyx_mstate_global->__pyx_n_u_where,&__pyx_mstate_global->__pyx_n_u_bbox,&__pyx_mstate_global->__pyx_n_u_mask,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2241, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "ogr_read_bounds", 0) < (0)) __PYX_ERR(0, 2241, __pyx_L3_error)

      /* "pyogrio/_io.pyx":2243
 * def ogr_read_bounds(
 *     object path_or_buffer,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int read_geometry=True,
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2244
 *     object path_or_buffer,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int read_geometry=True,
 *     int force_2d=False,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2247
 *     int read_geometry=True,
 *     int force_2d=False,
 *     object columns=None,             # <<<<<<<<<<<<<<
 *     int skip_features=0,
 *     int max_features=0,
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2250
 *     int skip_features=0,
 *     int max_features=0,
 *     object where=None,             # <<<<<<<<<<<<<<
 *     tuple bbox=None,
 *     object mask=None,
*/
      if (!values[8]) values[8] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2251
 *     int max_features=0,
 *     object where=None,
 *     tuple bbox=None,             # <<<<<<<<<<<<<<
 *     object mask=None,
 * ):
*/
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":2252
 *     object where=None,
 *     tuple bbox=None,
 *     object mask=None,             # <<<<<<<<<<<<<<
 * ):
 *     """Read bounds of features in the dataset."""
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("ogr_read_bounds", 0, 1, 11, i); __PYX_ERR(0, 2241, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2241, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "pyogrio/_io.pyx":2243
 * def ogr_read_bounds(
 *     object path_or_buffer,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int read_geometry=True,
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2244
 *     object path_or_buffer,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int read_geometry=True,
 *     int force_2d=False,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2247
 *     int read_geometry=True,
 *     int force_2d=False,
 *     object columns=None,             # <<<<<<<<<<<<<<
 *     int skip_features=0,
 *     int max_features=0,
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2250
 *     int skip_features=0,
 *     int max_features=0,
 *     object where=None,             # <<<<<<<<<<<<<<
 *     tuple bbox=None,
 *     object mask=None,
*/
      if (!values[8]) values[8] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2251
 *     int max_features=0,
 *     object where=None,
 *     tuple bbox=None,             # <<<<<<<<<<<<<<
 *     object mask=None,
 * ):
*/
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject*)Py_None));

      /* "pyogrio/_io.pyx":2252
 *     object where=None,
 *     tuple bbox=None,
 *     object mask=None,             # <<<<<<<<<<<<<<
 * ):
 *     """Read bounds of features in the dataset."""
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_path_or_buffer = values[0];
    __pyx_v_layer = values[1];
    __pyx_v_encoding = values[2];
    if (values[3]) {
      __pyx_v_read_geometry = __Pyx_PyLong_As_int(values[3]); if (unlikely((__pyx_v_read_geometry == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2245, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":2245
 *     object layer=None,
 *     object encoding=None,
 *     int read_geometry=True,             # <<<<<<<<<<<<<<
 *     int force_2d=False,
 *     object columns=None,
*/
      __pyx_v_read_geometry = ((int)((int)1));
    }
    if (values[4]) {
      __pyx_v_force_2d = __Pyx_PyLong_As_int(values[4]); if (unlikely((__pyx_v_force_2d == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2246, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":2246
 *     object encoding=None,
 *     int read_geometry=True,
 *     int force_2d=False,             # <<<<<<<<<<<<<<
 *     object columns=None,
 *     int skip_features=0,
*/
      __pyx_v_force_2d = ((int)((int)0));
    }
    __pyx_v_columns = values[5];
    if (values[6]) {
      __pyx_v_skip_features = __Pyx_PyLong_As_int(values[6]); if (unlikely((__pyx_v_skip_features == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2248, __pyx_L3_error)
    } else {
      __pyx_v_skip_features = ((int)((int)0));
    }
    if (values[7]) {
      __pyx_v_max_features = __Pyx_PyLong_As_int(values[7]); if (unlikely((__pyx_v_max_features == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2249, __pyx_L3_error)
    } else {
      __pyx_v_max_features = ((int)((int)0));
    }
    __pyx_v_where = values[8];
    __pyx_v_bbox = ((PyObject*)values[9]);
    __pyx_v_mask = values[10];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ogr_read_bounds", 0, 1, 11, __pyx_nargs); __PYX_ERR(0, 2241, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.ogr_read_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_bbox), (&PyTuple_Type), 1, "bbox", 1))) __PYX_ERR(0, 2251, __pyx_L1_error)
  __pyx_r = __pyx_pf_7pyogrio_3_io_5ogr_read_bounds(__pyx_self, __pyx_v_path_or_buffer, __pyx_v_layer, __pyx_v_encoding, __pyx_v_read_geometry, __pyx_v_force_2d, __pyx_v_columns, __pyx_v_skip_features, __pyx_v_max_features, __pyx_v_where, __pyx_v_bbox, __pyx_v_mask);

  /* "pyogrio/_io.pyx":2241
 * 
 * 
 * def ogr_read_bounds(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object layer=None,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_5ogr_read_bounds(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_layer, CYTHON_UNUSED PyObject *__pyx_v_encoding, CYTHON_UNUSED int __pyx_v_read_geometry, CYTHON_UNUSED int __pyx_v_force_2d, CYTHON_UNUSED PyObject *__pyx_v_columns, int __pyx_v_skip_features, int __pyx_v_max_features, PyObject *__pyx_v_where, PyObject *__pyx_v_bbox, PyObject *__pyx_v_mask) {
  int __pyx_v_use_tmp_vsimem;
  OGRDataSourceH __pyx_v_ogr_dataset;
  OGRLayerH __pyx_v_ogr_layer;
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_num_features = NULL;
  PyObject *__pyx_v_bounds = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char const *__pyx_t_7;
  void *__pyx_t_8;
  OGRLayerH __pyx_t_9;
  struct __pyx_opt_args_7pyogrio_3_io_validate_feature_range __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *(*__pyx_t_12)(PyObject *);
  int __pyx_t_13;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_read_bounds", 0);
  __Pyx_INCREF(__pyx_v_layer);

  /* "pyogrio/_io.pyx":2255
 * ):
 *     """Read bounds of features in the dataset."""
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)             # <<<<<<<<<<<<<<
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_path_or_buffer); 
  __pyx_v_use_tmp_vsimem = __pyx_t_1;

  /* "pyogrio/_io.pyx":2256
 *     """Read bounds of features in the dataset."""
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRLayerH ogr_layer = NULL
 * 
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":2257
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 * 
 *     if bbox and mask:
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":2259
 *     cdef OGRLayerH ogr_layer = NULL
 * 
 *     if bbox and mask:             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
*/
  if (__pyx_v_bbox == Py_None) __pyx_t_2 = 0;
  else
  {
    Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_v_bbox);
    if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 2259, __pyx_L1_error)
    __pyx_t_2 = (__pyx_temp != 0);
  }

  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_mask); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 2259, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":2260
 * 
 *     if bbox and mask:
 *         raise ValueError("cannot set both 'bbox' and 'mask'")             # <<<<<<<<<<<<<<
 * 
 *     if skip_features < 0:
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_cannot_set_both_bbox_and_mask};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2260, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 2260, __pyx_L1_error)

    /* "pyogrio/_io.pyx":2259
 *     cdef OGRLayerH ogr_layer = NULL
 * 
 *     if bbox and mask:             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
*/
  }

  /* "pyogrio/_io.pyx":2262
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
 *     if skip_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'skip_features' must be >= 0")
 * 
*/
  __pyx_t_1 = (__pyx_v_skip_features < 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":2263
 * 
 *     if skip_features < 0:
 *         raise ValueError("'skip_features' must be >= 0")             # <<<<<<<<<<<<<<
 * 
 *     if max_features < 0:
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_skip_features_must_be_0};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 2263, __pyx_L1_error)

    /* "pyogrio/_io.pyx":2262
 *         raise ValueError("cannot set both 'bbox' and 'mask'")
 * 
 *     if skip_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'skip_features' must be >= 0")
 * 
*/
  }

  /* "pyogrio/_io.pyx":2265
 *         raise ValueError("'skip_features' must be >= 0")
 * 
 *     if max_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'max_features' must be >= 0")
 * 
*/
  __pyx_t_1 = (__pyx_v_max_features < 0);
  if (unlikely(__pyx_t_1)) {

    /* "pyogrio/_io.pyx":2266
 * 
 *     if max_features < 0:
 *         raise ValueError("'max_features' must be >= 0")             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_max_features_must_be_0};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 2266, __pyx_L1_error)

    /* "pyogrio/_io.pyx":2265
 *         raise ValueError("'skip_features' must be >= 0")
 * 
 *     if max_features < 0:             # <<<<<<<<<<<<<<
 *         raise ValueError("'max_features' must be >= 0")
 * 
*/
  }

  /* "pyogrio/_io.pyx":2268
 *         raise ValueError("'max_features' must be >= 0")
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":2271
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)
 * 
*/
    if (__pyx_v_use_tmp_vsimem) {

      /* "pyogrio/_io.pyx":2270
 *     try:
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer             # <<<<<<<<<<<<<<
 *         ) if use_tmp_vsimem else path_or_buffer
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)
*/
      __pyx_t_4 = __pyx_v_path_or_buffer;
      __Pyx_INCREF(__pyx_t_4);
      if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_4))) __PYX_ERR(0, 2270, __pyx_L9_error)

      /* "pyogrio/_io.pyx":2269
 * 
 *     try:
 *         path = read_buffer_to_vsimem(             # <<<<<<<<<<<<<<
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer
*/
      __pyx_t_6 = __pyx_f_7pyogrio_4_vsi_read_buffer_to_vsimem(((PyObject*)__pyx_t_4)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2269, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = __pyx_t_6;
      __pyx_t_6 = 0;
    } else {

      /* "pyogrio/_io.pyx":2271
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)
 * 
*/
      __Pyx_INCREF(__pyx_v_path_or_buffer);
      __pyx_t_3 = __pyx_v_path_or_buffer;
    }
    __pyx_v_path = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2272
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)             # <<<<<<<<<<<<<<
 * 
 *         if layer is None:
*/
    __pyx_t_6 = __pyx_v_path;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2272, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_7 = __Pyx_PyObject_AsString(__pyx_t_3); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(0, 2272, __pyx_L9_error)
    __pyx_t_8 = __pyx_f_7pyogrio_3_io_ogr_open(__pyx_t_7, 0, NULL); if (unlikely(__pyx_t_8 == ((void *)NULL))) __PYX_ERR(0, 2272, __pyx_L9_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_ogr_dataset = __pyx_t_8;

    /* "pyogrio/_io.pyx":2274
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)
 * 
 *         if layer is None:             # <<<<<<<<<<<<<<
 *             layer = get_default_layer(ogr_dataset)
 * 
*/
    __pyx_t_1 = (__pyx_v_layer == Py_None);
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":2275
 * 
 *         if layer is None:
 *             layer = get_default_layer(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)
*/
      __pyx_t_3 = __pyx_f_7pyogrio_3_io_get_default_layer(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2275, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF_SET(__pyx_v_layer, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyogrio/_io.pyx":2274
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)
 * 
 *         if layer is None:             # <<<<<<<<<<<<<<
 *             layer = get_default_layer(ogr_dataset)
 * 
*/
    }

    /* "pyogrio/_io.pyx":2277
 *             layer = get_default_layer(ogr_dataset)
 * 
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)             # <<<<<<<<<<<<<<
 * 
 *         # Apply the attribute filter
*/
    __pyx_t_9 = __pyx_f_7pyogrio_3_io_get_ogr_layer(__pyx_v_ogr_dataset, __pyx_v_layer); if (unlikely(__pyx_t_9 == ((void *)NULL))) __PYX_ERR(0, 2277, __pyx_L9_error)
    __pyx_v_ogr_layer = __pyx_t_9;

    /* "pyogrio/_io.pyx":2280
 * 
 *         # Apply the attribute filter
 *         if where is not None and where != "":             # <<<<<<<<<<<<<<
 *             apply_where_filter(ogr_layer, where)
 * 
*/
    __pyx_t_2 = (__pyx_v_where != Py_None);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L13_bool_binop_done;
    }
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_where, __pyx_mstate_global->__pyx_kp_u__9, Py_NE)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 2280, __pyx_L9_error)
    __pyx_t_1 = __pyx_t_2;
    __pyx_L13_bool_binop_done:;
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":2281
 *         # Apply the attribute filter
 *         if where is not None and where != "":
 *             apply_where_filter(ogr_layer, where)             # <<<<<<<<<<<<<<
 * 
 *         # Apply the spatial filter
*/
      __pyx_t_3 = __pyx_v_where;
      __Pyx_INCREF(__pyx_t_3);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 2281, __pyx_L9_error)
      __pyx_t_6 = __pyx_f_7pyogrio_3_io_apply_where_filter(__pyx_v_ogr_layer, ((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2281, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":2280
 * 
 *         # Apply the attribute filter
 *         if where is not None and where != "":             # <<<<<<<<<<<<<<
 *             apply_where_filter(ogr_layer, where)
 * 
*/
    }

    /* "pyogrio/_io.pyx":2284
 * 
 *         # Apply the spatial filter
 *         if bbox is not None:             # <<<<<<<<<<<<<<
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
*/
    __pyx_t_1 = (__pyx_v_bbox != ((PyObject*)Py_None));
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":2285
 *         # Apply the spatial filter
 *         if bbox is not None:
 *             apply_bbox_filter(ogr_layer, bbox)             # <<<<<<<<<<<<<<
 * 
 *         elif mask is not None:
*/
      __pyx_t_6 = __pyx_f_7pyogrio_3_io_apply_bbox_filter(__pyx_v_ogr_layer, __pyx_v_bbox); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2285, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":2284
 * 
 *         # Apply the spatial filter
 *         if bbox is not None:             # <<<<<<<<<<<<<<
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
*/
      goto __pyx_L15;
    }

    /* "pyogrio/_io.pyx":2287
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
 *         elif mask is not None:             # <<<<<<<<<<<<<<
 *             apply_geometry_filter(ogr_layer, mask)
 * 
*/
    __pyx_t_1 = (__pyx_v_mask != Py_None);
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":2288
 * 
 *         elif mask is not None:
 *             apply_geometry_filter(ogr_layer, mask)             # <<<<<<<<<<<<<<
 * 
 *         # Limit feature range to available range
*/
      __pyx_t_6 = __pyx_f_7pyogrio_3_io_apply_geometry_filter(__pyx_v_ogr_layer, __pyx_v_mask); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2288, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":2287
 *             apply_bbox_filter(ogr_layer, bbox)
 * 
 *         elif mask is not None:             # <<<<<<<<<<<<<<
 *             apply_geometry_filter(ogr_layer, mask)
 * 
*/
    }
    __pyx_L15:;

    /* "pyogrio/_io.pyx":2291
 * 
 *         # Limit feature range to available range
 *         skip_features, num_features = validate_feature_range(             # <<<<<<<<<<<<<<
 *             ogr_layer, skip_features, max_features
 *         )
*/
    __pyx_t_10.__pyx_n = 2;
    __pyx_t_10.skip_features = __pyx_v_skip_features;
    __pyx_t_10.max_features = __pyx_v_max_features;
    __pyx_t_6 = __pyx_f_7pyogrio_3_io_validate_feature_range(__pyx_v_ogr_layer, &__pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2291, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_6);
    if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
      PyObject* sequence = __pyx_t_6;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 2291, __pyx_L9_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_4);
      } else {
        __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2291, __pyx_L9_error)
        __Pyx_XGOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2291, __pyx_L9_error)
        __Pyx_XGOTREF(__pyx_t_4);
      }
      #else
      __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2291, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2291, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_11 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2291, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_12 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_11);
      index = 0; __pyx_t_3 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_3)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_4 = __pyx_t_12(__pyx_t_11); if (unlikely(!__pyx_t_4)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_12(__pyx_t_11), 2) < (0)) __PYX_ERR(0, 2291, __pyx_L9_error)
      __pyx_t_12 = NULL;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L17_unpacking_done;
      __pyx_L16_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 2291, __pyx_L9_error)
      __pyx_L17_unpacking_done:;
    }
    __pyx_t_13 = __Pyx_PyLong_As_int(__pyx_t_3); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2291, __pyx_L9_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_skip_features = __pyx_t_13;
    __pyx_v_num_features = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "pyogrio/_io.pyx":2295
 *         )
 * 
 *         bounds = get_bounds(ogr_layer, skip_features, num_features)             # <<<<<<<<<<<<<<
 * 
 *     finally:
*/
    __pyx_t_13 = __Pyx_PyLong_As_int(__pyx_v_num_features); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2295, __pyx_L9_error)
    __pyx_t_6 = __pyx_f_7pyogrio_3_io_get_bounds(__pyx_v_ogr_layer, __pyx_v_skip_features, __pyx_t_13); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2295, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_bounds = __pyx_t_6;
    __pyx_t_6 = 0;
  }

  /* "pyogrio/_io.pyx":2298
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_1 = (__pyx_v_ogr_dataset != NULL);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":2299
 *     finally:
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
        (void)(GDALClose(__pyx_v_ogr_dataset));

        /* "pyogrio/_io.pyx":2300
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
        __pyx_v_ogr_dataset = NULL;

        /* "pyogrio/_io.pyx":2298
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
      }

      /* "pyogrio/_io.pyx":2302
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      if (__pyx_v_use_tmp_vsimem) {

        /* "pyogrio/_io.pyx":2303
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return bounds
*/
        __pyx_t_6 = __pyx_v_path;
        __Pyx_INCREF(__pyx_t_6);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_6))) __PYX_ERR(0, 2303, __pyx_L1_error)
        __pyx_t_4 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_6), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2303, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pyogrio/_io.pyx":2302
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      }
      goto __pyx_L10;
    }
    __pyx_L9_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
      if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_21);
      __pyx_t_13 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":2298
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
        __pyx_t_1 = (__pyx_v_ogr_dataset != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":2299
 *     finally:
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
          (void)(GDALClose(__pyx_v_ogr_dataset));

          /* "pyogrio/_io.pyx":2300
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
          __pyx_v_ogr_dataset = NULL;

          /* "pyogrio/_io.pyx":2298
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
        }

        /* "pyogrio/_io.pyx":2302
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        if (__pyx_v_use_tmp_vsimem) {

          /* "pyogrio/_io.pyx":2303
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return bounds
*/
          if (unlikely(!__pyx_v_path)) { __Pyx_RaiseUnboundLocalError("path"); __PYX_ERR(0, 2303, __pyx_L21_error) }
          __pyx_t_4 = __pyx_v_path;
          __Pyx_INCREF(__pyx_t_4);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_4))) __PYX_ERR(0, 2303, __pyx_L21_error)
          __pyx_t_6 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_4), 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2303, __pyx_L21_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

          /* "pyogrio/_io.pyx":2302
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
      __pyx_lineno = __pyx_t_13; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
      goto __pyx_L1_error;
      __pyx_L21_error:;
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L10:;
  }

  /* "pyogrio/_io.pyx":2305
 *             vsimem_rmtree_toplevel(path)
 * 
 *     return bounds             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_bounds);
  __pyx_r = __pyx_v_bounds;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2241
 * 
 * 
 * def ogr_read_bounds(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object layer=None,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pyogrio._io.ogr_read_bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_num_features);
  __Pyx_XDECREF(__pyx_v_bounds);
  __Pyx_XDECREF(__pyx_v_layer);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2308
 * 
 * 
 * def ogr_read_info(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     dataset_kwargs,
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_8ogr_read_info(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_7ogr_read_info, "Read metadata information about the dataset.");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_8ogr_read_info = {"ogr_read_info", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_8ogr_read_info, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_7ogr_read_info};
static PyObject *__pyx_pw_7pyogrio_3_io_8ogr_read_info(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path_or_buffer = 0;
  PyObject *__pyx_v_dataset_kwargs = 0;
  PyObject *__pyx_v_layer = 0;
  PyObject *__pyx_v_encoding = 0;
  int __pyx_v_force_feature_count;
  int __pyx_v_force_total_bounds;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_read_info (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path_or_buffer,&__pyx_mstate_global->__pyx_n_u_dataset_kwargs,&__pyx_mstate_global->__pyx_n_u_layer,&__pyx_mstate_global->__pyx_n_u_encoding,&__pyx_mstate_global->__pyx_n_u_force_feature_count,&__pyx_mstate_global->__pyx_n_u_force_total_bounds,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2308, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "ogr_read_info", 0) < (0)) __PYX_ERR(0, 2308, __pyx_L3_error)

      /* "pyogrio/_io.pyx":2311
 *     object path_or_buffer,
 *     dataset_kwargs,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int force_feature_count=False,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2312
 *     dataset_kwargs,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int force_feature_count=False,
 *     int force_total_bounds=False
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("ogr_read_info", 0, 2, 6, i); __PYX_ERR(0, 2308, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2308, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2308, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2308, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "pyogrio/_io.pyx":2311
 *     object path_or_buffer,
 *     dataset_kwargs,
 *     object layer=None,             # <<<<<<<<<<<<<<
 *     object encoding=None,
 *     int force_feature_count=False,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2312
 *     dataset_kwargs,
 *     object layer=None,
 *     object encoding=None,             # <<<<<<<<<<<<<<
 *     int force_feature_count=False,
 *     int force_total_bounds=False
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_path_or_buffer = values[0];
    __pyx_v_dataset_kwargs = values[1];
    __pyx_v_layer = values[2];
    __pyx_v_encoding = values[3];
    if (values[4]) {
      __pyx_v_force_feature_count = __Pyx_PyLong_As_int(values[4]); if (unlikely((__pyx_v_force_feature_count == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2313, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":2313
 *     object layer=None,
 *     object encoding=None,
 *     int force_feature_count=False,             # <<<<<<<<<<<<<<
 *     int force_total_bounds=False
 * ):
*/
      __pyx_v_force_feature_count = ((int)((int)0));
    }
    if (values[5]) {
      __pyx_v_force_total_bounds = __Pyx_PyLong_As_int(values[5]); if (unlikely((__pyx_v_force_total_bounds == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2314, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":2314
 *     object encoding=None,
 *     int force_feature_count=False,
 *     int force_total_bounds=False             # <<<<<<<<<<<<<<
 * ):
 *     """Read metadata information about the dataset."""
*/
      __pyx_v_force_total_bounds = ((int)((int)0));
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ogr_read_info", 0, 2, 6, __pyx_nargs); __PYX_ERR(0, 2308, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.ogr_read_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyogrio_3_io_7ogr_read_info(__pyx_self, __pyx_v_path_or_buffer, __pyx_v_dataset_kwargs, __pyx_v_layer, __pyx_v_encoding, __pyx_v_force_feature_count, __pyx_v_force_total_bounds);

  /* "pyogrio/_io.pyx":2308
 * 
 * 
 * def ogr_read_info(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     dataset_kwargs,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_7ogr_read_info(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer, PyObject *__pyx_v_encoding, int __pyx_v_force_feature_count, int __pyx_v_force_total_bounds) {
  int __pyx_v_use_tmp_vsimem;
  char **__pyx_v_dataset_options;
  OGRDataSourceH __pyx_v_ogr_dataset;
  OGRLayerH __pyx_v_ogr_layer;
  char const *__pyx_v_prev_shape_encoding;
  int __pyx_v_override_shape_encoding;
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_fields = NULL;
  PyObject *__pyx_v_ogr_types = NULL;
  PyObject *__pyx_v_ogr_subtypes = NULL;
  PyObject *__pyx_v_meta = NULL;
  PyObject *__pyx_9genexpr10__pyx_v_field = NULL;
  PyObject *__pyx_9genexpr11__pyx_v_field = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  char const *__pyx_t_5;
  char **__pyx_t_6;
  size_t __pyx_t_7;
  char const *__pyx_t_8;
  void *__pyx_t_9;
  OGRLayerH __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_read_info", 0);
  __Pyx_INCREF(__pyx_v_layer);
  __Pyx_INCREF(__pyx_v_encoding);

  /* "pyogrio/_io.pyx":2317
 * ):
 *     """Read metadata information about the dataset."""
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)             # <<<<<<<<<<<<<<
 *     cdef char **dataset_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_path_or_buffer); 
  __pyx_v_use_tmp_vsimem = __pyx_t_1;

  /* "pyogrio/_io.pyx":2318
 *     """Read metadata information about the dataset."""
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef char **dataset_options = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
*/
  __pyx_v_dataset_options = NULL;

  /* "pyogrio/_io.pyx":2319
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef char **dataset_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef const char *prev_shape_encoding = NULL
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":2320
 *     cdef char **dataset_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *prev_shape_encoding = NULL
 *     cdef bint override_shape_encoding = False
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":2321
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef const char *prev_shape_encoding = NULL             # <<<<<<<<<<<<<<
 *     cdef bint override_shape_encoding = False
 * 
*/
  __pyx_v_prev_shape_encoding = NULL;

  /* "pyogrio/_io.pyx":2322
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef const char *prev_shape_encoding = NULL
 *     cdef bint override_shape_encoding = False             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
  __pyx_v_override_shape_encoding = 0;

  /* "pyogrio/_io.pyx":2324
 *     cdef bint override_shape_encoding = False
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":2327
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 * 
 *         if encoding:
*/
    if (__pyx_v_use_tmp_vsimem) {

      /* "pyogrio/_io.pyx":2326
 *     try:
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer             # <<<<<<<<<<<<<<
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
*/
      __pyx_t_3 = __pyx_v_path_or_buffer;
      __Pyx_INCREF(__pyx_t_3);
      if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_3))) __PYX_ERR(0, 2326, __pyx_L4_error)

      /* "pyogrio/_io.pyx":2325
 * 
 *     try:
 *         path = read_buffer_to_vsimem(             # <<<<<<<<<<<<<<
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer
*/
      __pyx_t_4 = __pyx_f_7pyogrio_4_vsi_read_buffer_to_vsimem(((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2325, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_2 = __pyx_t_4;
      __pyx_t_4 = 0;
    } else {

      /* "pyogrio/_io.pyx":2327
 *         path = read_buffer_to_vsimem(
 *             path_or_buffer
 *         ) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 * 
 *         if encoding:
*/
      __Pyx_INCREF(__pyx_v_path_or_buffer);
      __pyx_t_2 = __pyx_v_path_or_buffer;
    }
    __pyx_v_path = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2329
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
 *         if encoding:             # <<<<<<<<<<<<<<
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2329, __pyx_L4_error)
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":2330
 * 
 *         if encoding:
 *             override_shape_encoding = True             # <<<<<<<<<<<<<<
 *             prev_shape_encoding = override_threadlocal_config_option(
 *                 "SHAPE_ENCODING", encoding
*/
      __pyx_v_override_shape_encoding = 1;

      /* "pyogrio/_io.pyx":2332
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(
 *                 "SHAPE_ENCODING", encoding             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_2 = __pyx_v_encoding;
      __Pyx_INCREF(__pyx_t_2);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 2332, __pyx_L4_error)

      /* "pyogrio/_io.pyx":2331
 *         if encoding:
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(             # <<<<<<<<<<<<<<
 *                 "SHAPE_ENCODING", encoding
 *             )
*/
      __pyx_t_5 = __pyx_f_7pyogrio_3_io_override_threadlocal_config_option(__pyx_mstate_global->__pyx_n_u_SHAPE_ENCODING, ((PyObject*)__pyx_t_2)); if (unlikely(__pyx_t_5 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 2331, __pyx_L4_error)
      __pyx_v_prev_shape_encoding = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pyogrio/_io.pyx":2329
 *         ) if use_tmp_vsimem else path_or_buffer
 * 
 *         if encoding:             # <<<<<<<<<<<<<<
 *             override_shape_encoding = True
 *             prev_shape_encoding = override_threadlocal_config_option(
*/
    }

    /* "pyogrio/_io.pyx":2335
 *             )
 * 
 *         dataset_options = dict_to_options(dataset_kwargs)             # <<<<<<<<<<<<<<
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
*/
    __pyx_t_6 = __pyx_f_7pyogrio_3_io_dict_to_options(__pyx_v_dataset_kwargs); if (unlikely(__pyx_t_6 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 2335, __pyx_L4_error)
    __pyx_v_dataset_options = __pyx_t_6;

    /* "pyogrio/_io.pyx":2336
 * 
 *         dataset_options = dict_to_options(dataset_kwargs)
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)             # <<<<<<<<<<<<<<
 * 
 *         if layer is None:
*/
    __pyx_t_4 = __pyx_v_path;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2336, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_8 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 2336, __pyx_L4_error)
    __pyx_t_9 = __pyx_f_7pyogrio_3_io_ogr_open(__pyx_t_8, 0, __pyx_v_dataset_options); if (unlikely(__pyx_t_9 == ((void *)NULL))) __PYX_ERR(0, 2336, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ogr_dataset = __pyx_t_9;

    /* "pyogrio/_io.pyx":2338
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
 *         if layer is None:             # <<<<<<<<<<<<<<
 *             layer = get_default_layer(ogr_dataset)
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)
*/
    __pyx_t_1 = (__pyx_v_layer == Py_None);
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":2339
 * 
 *         if layer is None:
 *             layer = get_default_layer(ogr_dataset)             # <<<<<<<<<<<<<<
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)
 * 
*/
      __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_default_layer(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2339, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_layer, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyogrio/_io.pyx":2338
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, dataset_options)
 * 
 *         if layer is None:             # <<<<<<<<<<<<<<
 *             layer = get_default_layer(ogr_dataset)
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)
*/
    }

    /* "pyogrio/_io.pyx":2340
 *         if layer is None:
 *             layer = get_default_layer(ogr_dataset)
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)             # <<<<<<<<<<<<<<
 * 
 *         if encoding and get_driver(ogr_dataset) == "ESRI Shapefile":
*/
    __pyx_t_10 = __pyx_f_7pyogrio_3_io_get_ogr_layer(__pyx_v_ogr_dataset, __pyx_v_layer); if (unlikely(__pyx_t_10 == ((void *)NULL))) __PYX_ERR(0, 2340, __pyx_L4_error)
    __pyx_v_ogr_layer = __pyx_t_10;

    /* "pyogrio/_io.pyx":2342
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)
 * 
 *         if encoding and get_driver(ogr_dataset) == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *             encoding = "UTF-8"
 *         else:
*/
    __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 2342, __pyx_L4_error)
    if (__pyx_t_11) {
    } else {
      __pyx_t_1 = __pyx_t_11;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_driver(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2342, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_EQ)); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 2342, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __pyx_t_11;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_1) {

      /* "pyogrio/_io.pyx":2343
 * 
 *         if encoding and get_driver(ogr_dataset) == "ESRI Shapefile":
 *             encoding = "UTF-8"             # <<<<<<<<<<<<<<
 *         else:
 *             encoding = encoding or detect_encoding(ogr_dataset, ogr_layer)
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
      __Pyx_DECREF_SET(__pyx_v_encoding, __pyx_mstate_global->__pyx_kp_u_UTF_8);

      /* "pyogrio/_io.pyx":2342
 *         ogr_layer = get_ogr_layer(ogr_dataset, layer)
 * 
 *         if encoding and get_driver(ogr_dataset) == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *             encoding = "UTF-8"
 *         else:
*/
      goto __pyx_L8;
    }

    /* "pyogrio/_io.pyx":2345
 *             encoding = "UTF-8"
 *         else:
 *             encoding = encoding or detect_encoding(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         fields = get_fields(ogr_layer, encoding)
*/
    /*else*/ {
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2345, __pyx_L4_error)
      if (!__pyx_t_1) {
      } else {
        __Pyx_INCREF(__pyx_v_encoding);
        __pyx_t_2 = __pyx_v_encoding;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_4 = __pyx_f_7pyogrio_3_io_detect_encoding(__pyx_v_ogr_dataset, __pyx_v_ogr_layer); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2345, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_2 = __pyx_t_4;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_L11_bool_binop_done:;
      __Pyx_DECREF_SET(__pyx_v_encoding, __pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L8:;

    /* "pyogrio/_io.pyx":2347
 *             encoding = encoding or detect_encoding(ogr_dataset, ogr_layer)
 * 
 *         fields = get_fields(ogr_layer, encoding)             # <<<<<<<<<<<<<<
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
 *         ogr_subtypes = [
*/
    __pyx_t_2 = __pyx_v_encoding;
    __Pyx_INCREF(__pyx_t_2);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 2347, __pyx_L4_error)
    __pyx_t_4 = __pyx_f_7pyogrio_3_io_get_fields(__pyx_v_ogr_layer, ((PyObject*)__pyx_t_2), NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2347, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_fields = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "pyogrio/_io.pyx":2348
 * 
 *         fields = get_fields(ogr_layer, encoding)
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]             # <<<<<<<<<<<<<<
 *         ogr_subtypes = [
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields
*/
    { /* enter inner scope */
      __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2348, __pyx_L15_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (likely(PyList_CheckExact(__pyx_v_fields)) || PyTuple_CheckExact(__pyx_v_fields)) {
        __pyx_t_2 = __pyx_v_fields; __Pyx_INCREF(__pyx_t_2);
        __pyx_t_12 = 0;
        __pyx_t_13 = NULL;
      } else {
        __pyx_t_12 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2348, __pyx_L15_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_13 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2348, __pyx_L15_error)
      }
      for (;;) {
        if (likely(!__pyx_t_13)) {
          if (likely(PyList_CheckExact(__pyx_t_2))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2348, __pyx_L15_error)
              #endif
              if (__pyx_t_12 >= __pyx_temp) break;
            }
            __pyx_t_3 = __Pyx_PyList_GetItemRefFast(__pyx_t_2, __pyx_t_12, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_12;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2348, __pyx_L15_error)
              #endif
              if (__pyx_t_12 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_12));
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_12);
            #endif
            ++__pyx_t_12;
          }
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2348, __pyx_L15_error)
        } else {
          __pyx_t_3 = __pyx_t_13(__pyx_t_2);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2348, __pyx_L15_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_XDECREF_SET(__pyx_9genexpr10__pyx_v_field, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_14 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_FIELD_TYPE_NAMES); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2348, __pyx_L15_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2348, __pyx_L15_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = __Pyx_GetItemInt(__pyx_9genexpr10__pyx_v_field, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2348, __pyx_L15_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_16))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_16);
          assert(__pyx_t_14);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
          __pyx_t_7 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_14, __pyx_t_15, __pyx_mstate_global->__pyx_n_u_Unknown};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2348, __pyx_L15_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_4, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 2348, __pyx_L15_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_field); __pyx_9genexpr10__pyx_v_field = 0;
      goto __pyx_L19_exit_scope;
      __pyx_L15_error:;
      __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_field); __pyx_9genexpr10__pyx_v_field = 0;
      goto __pyx_L4_error;
      __pyx_L19_exit_scope:;
    } /* exit inner scope */
    __pyx_v_ogr_types = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyogrio/_io.pyx":2349
 *         fields = get_fields(ogr_layer, encoding)
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
 *         ogr_subtypes = [             # <<<<<<<<<<<<<<
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields
 *         ]
*/
    { /* enter inner scope */
      __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2349, __pyx_L22_error)
      __Pyx_GOTREF(__pyx_t_4);

      /* "pyogrio/_io.pyx":2350
 *         ogr_types = [FIELD_TYPE_NAMES.get(field[1], "Unknown") for field in fields]
 *         ogr_subtypes = [
 *             FIELD_SUBTYPE_NAMES.get(field[4], "Unknown") for field in fields             # <<<<<<<<<<<<<<
 *         ]
 * 
*/
      if (likely(PyList_CheckExact(__pyx_v_fields)) || PyTuple_CheckExact(__pyx_v_fields)) {
        __pyx_t_2 = __pyx_v_fields; __Pyx_INCREF(__pyx_t_2);
        __pyx_t_12 = 0;
        __pyx_t_13 = NULL;
      } else {
        __pyx_t_12 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2350, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_13 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2350, __pyx_L22_error)
      }
      for (;;) {
        if (likely(!__pyx_t_13)) {
          if (likely(PyList_CheckExact(__pyx_t_2))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_2);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2350, __pyx_L22_error)
              #endif
              if (__pyx_t_12 >= __pyx_temp) break;
            }
            __pyx_t_3 = __Pyx_PyList_GetItemRefFast(__pyx_t_2, __pyx_t_12, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_12;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_2);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2350, __pyx_L22_error)
              #endif
              if (__pyx_t_12 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_12));
            #else
            __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_2, __pyx_t_12);
            #endif
            ++__pyx_t_12;
          }
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2350, __pyx_L22_error)
        } else {
          __pyx_t_3 = __pyx_t_13(__pyx_t_2);
          if (unlikely(!__pyx_t_3)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2350, __pyx_L22_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_XDECREF_SET(__pyx_9genexpr11__pyx_v_field, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_16 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_FIELD_SUBTYPE_NAMES); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2350, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2350, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_15 = __Pyx_GetItemInt(__pyx_9genexpr11__pyx_v_field, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2350, __pyx_L22_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_7 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_14);
          assert(__pyx_t_16);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_16);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
          __pyx_t_7 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_16, __pyx_t_15, __pyx_mstate_global->__pyx_n_u_Unknown};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2350, __pyx_L22_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_4, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 2349, __pyx_L22_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_9genexpr11__pyx_v_field); __pyx_9genexpr11__pyx_v_field = 0;
      goto __pyx_L26_exit_scope;
      __pyx_L22_error:;
      __Pyx_XDECREF(__pyx_9genexpr11__pyx_v_field); __pyx_9genexpr11__pyx_v_field = 0;
      goto __pyx_L4_error;
      __pyx_L26_exit_scope:;
    } /* exit inner scope */
    __pyx_v_ogr_subtypes = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyogrio/_io.pyx":2354
 * 
 *         meta = {
 *             "layer_name": get_string(OGR_L_GetName(ogr_layer)),             # <<<<<<<<<<<<<<
 *             "crs": get_crs(ogr_layer),
 *             "encoding": encoding,
*/
    __pyx_t_4 = __Pyx_PyDict_NewPresized(16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetName(__pyx_v_ogr_layer), NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_layer_name, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2355
 *         meta = {
 *             "layer_name": get_string(OGR_L_GetName(ogr_layer)),
 *             "crs": get_crs(ogr_layer),             # <<<<<<<<<<<<<<
 *             "encoding": encoding,
 *             "fields": fields[:, 2],
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_crs(__pyx_v_ogr_layer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2355, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_crs, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2356
 *             "layer_name": get_string(OGR_L_GetName(ogr_layer)),
 *             "crs": get_crs(ogr_layer),
 *             "encoding": encoding,             # <<<<<<<<<<<<<<
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],
*/
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_encoding, __pyx_v_encoding) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)

    /* "pyogrio/_io.pyx":2357
 *             "crs": get_crs(ogr_layer),
 *             "encoding": encoding,
 *             "fields": fields[:, 2],             # <<<<<<<<<<<<<<
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,
*/
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[5]); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2357, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_fields, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2358
 *             "encoding": encoding,
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],             # <<<<<<<<<<<<<<
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,
*/
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_fields, __pyx_mstate_global->__pyx_tuple[6]); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2358, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_dtypes, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2359
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,             # <<<<<<<<<<<<<<
 *             "ogr_subtypes": ogr_subtypes,
 *             "fid_column": get_string(OGR_L_GetFIDColumn(ogr_layer)),
*/
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ogr_types, __pyx_v_ogr_types) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)

    /* "pyogrio/_io.pyx":2360
 *             "dtypes": fields[:, 3],
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,             # <<<<<<<<<<<<<<
 *             "fid_column": get_string(OGR_L_GetFIDColumn(ogr_layer)),
 *             "geometry_name": get_string(OGR_L_GetGeometryColumn(ogr_layer)),
*/
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ogr_subtypes, __pyx_v_ogr_subtypes) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)

    /* "pyogrio/_io.pyx":2361
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,
 *             "fid_column": get_string(OGR_L_GetFIDColumn(ogr_layer)),             # <<<<<<<<<<<<<<
 *             "geometry_name": get_string(OGR_L_GetGeometryColumn(ogr_layer)),
 *             "geometry_type": get_geometry_type(ogr_layer),
*/
    __pyx_t_2 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetFIDColumn(__pyx_v_ogr_layer), NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2361, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_fid_column, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2362
 *             "ogr_subtypes": ogr_subtypes,
 *             "fid_column": get_string(OGR_L_GetFIDColumn(ogr_layer)),
 *             "geometry_name": get_string(OGR_L_GetGeometryColumn(ogr_layer)),             # <<<<<<<<<<<<<<
 *             "geometry_type": get_geometry_type(ogr_layer),
 *             "features": get_feature_count(ogr_layer, force_feature_count),
*/
    __pyx_t_2 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetGeometryColumn(__pyx_v_ogr_layer), NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2362, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_geometry_name, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2363
 *             "fid_column": get_string(OGR_L_GetFIDColumn(ogr_layer)),
 *             "geometry_name": get_string(OGR_L_GetGeometryColumn(ogr_layer)),
 *             "geometry_type": get_geometry_type(ogr_layer),             # <<<<<<<<<<<<<<
 *             "features": get_feature_count(ogr_layer, force_feature_count),
 *             "total_bounds": get_total_bounds(ogr_layer, force_total_bounds),
*/
    __pyx_t_2 = __pyx_f_7pyogrio_9_geometry_get_geometry_type(__pyx_v_ogr_layer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2363, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_geometry_type, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2364
 *             "geometry_name": get_string(OGR_L_GetGeometryColumn(ogr_layer)),
 *             "geometry_type": get_geometry_type(ogr_layer),
 *             "features": get_feature_count(ogr_layer, force_feature_count),             # <<<<<<<<<<<<<<
 *             "total_bounds": get_total_bounds(ogr_layer, force_total_bounds),
 *             "driver": get_driver(ogr_dataset),
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_feature_count(__pyx_v_ogr_layer, __pyx_v_force_feature_count); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2364, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_features, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2365
 *             "geometry_type": get_geometry_type(ogr_layer),
 *             "features": get_feature_count(ogr_layer, force_feature_count),
 *             "total_bounds": get_total_bounds(ogr_layer, force_total_bounds),             # <<<<<<<<<<<<<<
 *             "driver": get_driver(ogr_dataset),
 *             "capabilities": {
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_total_bounds(__pyx_v_ogr_layer, __pyx_v_force_total_bounds); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2365, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_total_bounds, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2366
 *             "features": get_feature_count(ogr_layer, force_feature_count),
 *             "total_bounds": get_total_bounds(ogr_layer, force_total_bounds),
 *             "driver": get_driver(ogr_dataset),             # <<<<<<<<<<<<<<
 *             "capabilities": {
 *                 "random_read": OGR_L_TestCapability(ogr_layer, OLCRandomRead) == 1,
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_driver(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2366, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_driver, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2368
 *             "driver": get_driver(ogr_dataset),
 *             "capabilities": {
 *                 "random_read": OGR_L_TestCapability(ogr_layer, OLCRandomRead) == 1,             # <<<<<<<<<<<<<<
 *                 "fast_set_next_by_index": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastSetNextByIndex
*/
    __pyx_t_2 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2368, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyBool_FromLong((OGR_L_TestCapability(__pyx_v_ogr_layer, OLCRandomRead) == 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2368, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_random_read, __pyx_t_3) < (0)) __PYX_ERR(0, 2368, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2371
 *                 "fast_set_next_by_index": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastSetNextByIndex
 *                 ) == 1,             # <<<<<<<<<<<<<<
 *                 "fast_spatial_filter": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastSpatialFilter
*/
    __pyx_t_3 = __Pyx_PyBool_FromLong((OGR_L_TestCapability(__pyx_v_ogr_layer, OLCFastSetNextByIndex) == 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2371, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fast_set_next_by_index, __pyx_t_3) < (0)) __PYX_ERR(0, 2368, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2374
 *                 "fast_spatial_filter": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastSpatialFilter
 *                 ) == 1,             # <<<<<<<<<<<<<<
 *                 "fast_feature_count": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastFeatureCount
*/
    __pyx_t_3 = __Pyx_PyBool_FromLong((OGR_L_TestCapability(__pyx_v_ogr_layer, OLCFastSpatialFilter) == 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2374, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fast_spatial_filter, __pyx_t_3) < (0)) __PYX_ERR(0, 2368, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2377
 *                 "fast_feature_count": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastFeatureCount
 *                 ) == 1,             # <<<<<<<<<<<<<<
 *                 "fast_total_bounds": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastGetExtent
*/
    __pyx_t_3 = __Pyx_PyBool_FromLong((OGR_L_TestCapability(__pyx_v_ogr_layer, OLCFastFeatureCount) == 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2377, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fast_feature_count, __pyx_t_3) < (0)) __PYX_ERR(0, 2368, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":2380
 *                 "fast_total_bounds": OGR_L_TestCapability(
 *                     ogr_layer, OLCFastGetExtent
 *                 ) == 1,             # <<<<<<<<<<<<<<
 *             },
 *             "layer_metadata": get_metadata(ogr_layer),
*/
    __pyx_t_3 = __Pyx_PyBool_FromLong((OGR_L_TestCapability(__pyx_v_ogr_layer, OLCFastGetExtent) == 1)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2380, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fast_total_bounds, __pyx_t_3) < (0)) __PYX_ERR(0, 2368, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_capabilities, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2382
 *                 ) == 1,
 *             },
 *             "layer_metadata": get_metadata(ogr_layer),             # <<<<<<<<<<<<<<
 *             "dataset_metadata": get_metadata(ogr_dataset),
 *         }
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_metadata(__pyx_v_ogr_layer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2382, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_layer_metadata, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2383
 *             },
 *             "layer_metadata": get_metadata(ogr_layer),
 *             "dataset_metadata": get_metadata(ogr_dataset),             # <<<<<<<<<<<<<<
 *         }
 * 
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_metadata(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2383, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_dataset_metadata, __pyx_t_2) < (0)) __PYX_ERR(0, 2354, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_meta = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
  }

  /* "pyogrio/_io.pyx":2387
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_1 = (__pyx_v_dataset_options != NULL);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":2388
 *     finally:
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
        CSLDestroy(__pyx_v_dataset_options);

        /* "pyogrio/_io.pyx":2389
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
        __pyx_v_dataset_options = NULL;

        /* "pyogrio/_io.pyx":2387
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
      }

      /* "pyogrio/_io.pyx":2391
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
      __pyx_t_1 = (__pyx_v_ogr_dataset != NULL);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":2392
 * 
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
        (void)(GDALClose(__pyx_v_ogr_dataset));

        /* "pyogrio/_io.pyx":2393
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
*/
        __pyx_v_ogr_dataset = NULL;

        /* "pyogrio/_io.pyx":2391
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
      }

      /* "pyogrio/_io.pyx":2396
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
      if (__pyx_v_override_shape_encoding) {

        /* "pyogrio/_io.pyx":2397
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *             if prev_shape_encoding != NULL:
*/
        CPLSetThreadLocalConfigOption(__pyx_k_SHAPE_ENCODING, __pyx_v_prev_shape_encoding);

        /* "pyogrio/_io.pyx":2399
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
*/
        __pyx_t_1 = (__pyx_v_prev_shape_encoding != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":2400
 * 
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
          CPLFree(((void *)__pyx_v_prev_shape_encoding));

          /* "pyogrio/_io.pyx":2399
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
*/
        }

        /* "pyogrio/_io.pyx":2396
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
      }

      /* "pyogrio/_io.pyx":2402
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      if (__pyx_v_use_tmp_vsimem) {

        /* "pyogrio/_io.pyx":2403
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return meta
*/
        __pyx_t_4 = __pyx_v_path;
        __Pyx_INCREF(__pyx_t_4);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_4))) __PYX_ERR(0, 2403, __pyx_L1_error)
        __pyx_t_2 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_4), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2403, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pyogrio/_io.pyx":2402
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      }
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
      if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_22);
      __Pyx_XGOTREF(__pyx_t_23);
      __Pyx_XGOTREF(__pyx_t_24);
      __Pyx_XGOTREF(__pyx_t_25);
      __pyx_t_17 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":2387
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        __pyx_t_1 = (__pyx_v_dataset_options != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":2388
 *     finally:
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
          CSLDestroy(__pyx_v_dataset_options);

          /* "pyogrio/_io.pyx":2389
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
          __pyx_v_dataset_options = NULL;

          /* "pyogrio/_io.pyx":2387
 * 
 *     finally:
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        }

        /* "pyogrio/_io.pyx":2391
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
        __pyx_t_1 = (__pyx_v_ogr_dataset != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":2392
 * 
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
          (void)(GDALClose(__pyx_v_ogr_dataset));

          /* "pyogrio/_io.pyx":2393
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
*/
          __pyx_v_ogr_dataset = NULL;

          /* "pyogrio/_io.pyx":2391
 *             dataset_options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
        }

        /* "pyogrio/_io.pyx":2396
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
        if (__pyx_v_override_shape_encoding) {

          /* "pyogrio/_io.pyx":2397
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *             if prev_shape_encoding != NULL:
*/
          CPLSetThreadLocalConfigOption(__pyx_k_SHAPE_ENCODING, __pyx_v_prev_shape_encoding);

          /* "pyogrio/_io.pyx":2399
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
*/
          __pyx_t_1 = (__pyx_v_prev_shape_encoding != NULL);
          if (__pyx_t_1) {

            /* "pyogrio/_io.pyx":2400
 * 
 *             if prev_shape_encoding != NULL:
 *                 CPLFree(<void*>prev_shape_encoding)             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
            CPLFree(((void *)__pyx_v_prev_shape_encoding));

            /* "pyogrio/_io.pyx":2399
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
 *             if prev_shape_encoding != NULL:             # <<<<<<<<<<<<<<
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
*/
          }

          /* "pyogrio/_io.pyx":2396
 * 
 *         # reset SHAPE_ENCODING config parameter if temporarily set above
 *         if override_shape_encoding:             # <<<<<<<<<<<<<<
 *             CPLSetThreadLocalConfigOption("SHAPE_ENCODING", prev_shape_encoding)
 * 
*/
        }

        /* "pyogrio/_io.pyx":2402
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        if (__pyx_v_use_tmp_vsimem) {

          /* "pyogrio/_io.pyx":2403
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return meta
*/
          if (unlikely(!__pyx_v_path)) { __Pyx_RaiseUnboundLocalError("path"); __PYX_ERR(0, 2403, __pyx_L33_error) }
          __pyx_t_2 = __pyx_v_path;
          __Pyx_INCREF(__pyx_t_2);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 2403, __pyx_L33_error)
          __pyx_t_4 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_2), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2403, __pyx_L33_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "pyogrio/_io.pyx":2402
 *                 CPLFree(<void*>prev_shape_encoding)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_XGIVEREF(__pyx_t_25);
      __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_22);
      __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
      __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
      __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
      goto __pyx_L1_error;
      __pyx_L33_error:;
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_XGIVEREF(__pyx_t_25);
      __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
      __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
      __Pyx_XDECREF(__pyx_t_22); __pyx_t_22 = 0;
      __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "pyogrio/_io.pyx":2405
 *             vsimem_rmtree_toplevel(path)
 * 
 *     return meta             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_meta);
  __pyx_r = __pyx_v_meta;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2308
 * 
 * 
 * def ogr_read_info(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     dataset_kwargs,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("pyogrio._io.ogr_read_info", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_ogr_types);
  __Pyx_XDECREF(__pyx_v_ogr_subtypes);
  __Pyx_XDECREF(__pyx_v_meta);
  __Pyx_XDECREF(__pyx_9genexpr10__pyx_v_field);
  __Pyx_XDECREF(__pyx_9genexpr11__pyx_v_field);
  __Pyx_XDECREF(__pyx_v_layer);
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2408
 * 
 * 
 * def ogr_list_layers(object path_or_buffer):             # <<<<<<<<<<<<<<
 *     """List the layers in the dataset.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_10ogr_list_layers(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_9ogr_list_layers, "List the layers in the dataset.\n\n    Parameters\n    ----------\n    path_or_buffer : str or bytes\n        The path to the dataset, or a bytes buffer containing the dataset.\n\n    Returns\n    -------\n    ndarray(n)\n        An array with all layer names.\n\n    ");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_10ogr_list_layers = {"ogr_list_layers", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_10ogr_list_layers, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_9ogr_list_layers};
static PyObject *__pyx_pw_7pyogrio_3_io_10ogr_list_layers(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path_or_buffer = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_list_layers (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path_or_buffer,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2408, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2408, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "ogr_list_layers", 0) < (0)) __PYX_ERR(0, 2408, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("ogr_list_layers", 1, 1, 1, i); __PYX_ERR(0, 2408, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2408, __pyx_L3_error)
    }
    __pyx_v_path_or_buffer = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ogr_list_layers", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 2408, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.ogr_list_layers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7pyogrio_3_io_9ogr_list_layers(__pyx_self, __pyx_v_path_or_buffer);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_9ogr_list_layers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_buffer) {
  int __pyx_v_use_tmp_vsimem;
  OGRDataSourceH __pyx_v_ogr_dataset;
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_layers = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  char const *__pyx_t_6;
  void *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  char const *__pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_list_layers", 0);

  /* "pyogrio/_io.pyx":2422
 * 
 *     """
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)             # <<<<<<<<<<<<<<
 *     cdef OGRDataSourceH ogr_dataset = NULL
 * 
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_path_or_buffer); 
  __pyx_v_use_tmp_vsimem = __pyx_t_1;

  /* "pyogrio/_io.pyx":2423
 *     """
 *     cdef bint use_tmp_vsimem = isinstance(path_or_buffer, bytes)
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":2425
 *     cdef OGRDataSourceH ogr_dataset = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         path = (
 *             read_buffer_to_vsimem(path_or_buffer) if use_tmp_vsimem else path_or_buffer
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":2427
 *     try:
 *         path = (
 *             read_buffer_to_vsimem(path_or_buffer) if use_tmp_vsimem else path_or_buffer             # <<<<<<<<<<<<<<
 *         )
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)
*/
    if (__pyx_v_use_tmp_vsimem) {
      __pyx_t_3 = __pyx_v_path_or_buffer;
      __Pyx_INCREF(__pyx_t_3);
      if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_3))) __PYX_ERR(0, 2427, __pyx_L4_error)
      __pyx_t_4 = __pyx_f_7pyogrio_4_vsi_read_buffer_to_vsimem(((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2427, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_2 = __pyx_t_4;
      __pyx_t_4 = 0;
    } else {
      __Pyx_INCREF(__pyx_v_path_or_buffer);
      __pyx_t_2 = __pyx_v_path_or_buffer;
    }
    __pyx_v_path = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2429
 *             read_buffer_to_vsimem(path_or_buffer) if use_tmp_vsimem else path_or_buffer
 *         )
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)             # <<<<<<<<<<<<<<
 *         layers = get_layer_names(ogr_dataset)
 * 
*/
    __pyx_t_4 = __pyx_v_path;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_UTF_8};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2429, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_6 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 2429, __pyx_L4_error)
    __pyx_t_7 = __pyx_f_7pyogrio_3_io_ogr_open(__pyx_t_6, 0, NULL); if (unlikely(__pyx_t_7 == ((void *)NULL))) __PYX_ERR(0, 2429, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ogr_dataset = __pyx_t_7;

    /* "pyogrio/_io.pyx":2430
 *         )
 *         ogr_dataset = ogr_open(path.encode("UTF-8"), 0, NULL)
 *         layers = get_layer_names(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *     finally:
*/
    __pyx_t_2 = __pyx_f_7pyogrio_3_io_get_layer_names(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2430, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_layers = __pyx_t_2;
    __pyx_t_2 = 0;
  }

  /* "pyogrio/_io.pyx":2433
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_1 = (__pyx_v_ogr_dataset != NULL);
      if (__pyx_t_1) {

        /* "pyogrio/_io.pyx":2434
 *     finally:
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
        (void)(GDALClose(__pyx_v_ogr_dataset));

        /* "pyogrio/_io.pyx":2435
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
        __pyx_v_ogr_dataset = NULL;

        /* "pyogrio/_io.pyx":2433
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
      }

      /* "pyogrio/_io.pyx":2437
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      if (__pyx_v_use_tmp_vsimem) {

        /* "pyogrio/_io.pyx":2438
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return layers
*/
        __pyx_t_2 = __pyx_v_path;
        __Pyx_INCREF(__pyx_t_2);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 2438, __pyx_L1_error)
        __pyx_t_4 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_2), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2438, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pyogrio/_io.pyx":2437
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      }
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      if ( unlikely(__Pyx_GetException(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __pyx_t_8 = __pyx_lineno; __pyx_t_9 = __pyx_clineno; __pyx_t_10 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":2433
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
        __pyx_t_1 = (__pyx_v_ogr_dataset != NULL);
        if (__pyx_t_1) {

          /* "pyogrio/_io.pyx":2434
 *     finally:
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *             ogr_dataset = NULL
 * 
*/
          (void)(GDALClose(__pyx_v_ogr_dataset));

          /* "pyogrio/_io.pyx":2435
 *         if ogr_dataset != NULL:
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
          __pyx_v_ogr_dataset = NULL;

          /* "pyogrio/_io.pyx":2433
 * 
 *     finally:
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             GDALClose(ogr_dataset)
 *             ogr_dataset = NULL
*/
        }

        /* "pyogrio/_io.pyx":2437
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        if (__pyx_v_use_tmp_vsimem) {

          /* "pyogrio/_io.pyx":2438
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 *     return layers
*/
          if (unlikely(!__pyx_v_path)) { __Pyx_RaiseUnboundLocalError("path"); __PYX_ERR(0, 2438, __pyx_L9_error) }
          __pyx_t_4 = __pyx_v_path;
          __Pyx_INCREF(__pyx_t_4);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_4))) __PYX_ERR(0, 2438, __pyx_L9_error)
          __pyx_t_2 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_4), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2438, __pyx_L9_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyogrio/_io.pyx":2437
 *             ogr_dataset = NULL
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ErrRestore(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __pyx_lineno = __pyx_t_8; __pyx_clineno = __pyx_t_9; __pyx_filename = __pyx_t_10;
      goto __pyx_L1_error;
      __pyx_L9_error:;
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "pyogrio/_io.pyx":2440
 *             vsimem_rmtree_toplevel(path)
 * 
 *     return layers             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_layers);
  __pyx_r = __pyx_v_layers;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2408
 * 
 * 
 * def ogr_list_layers(object path_or_buffer):             # <<<<<<<<<<<<<<
 *     """List the layers in the dataset.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyogrio._io.ogr_list_layers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_layers);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2443
 * 
 * 
 * cdef str get_default_layer(OGRDataSourceH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Get the layer in the dataset that is read by default.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_default_layer(OGRDataSourceH __pyx_v_ogr_dataset) {
  PyObject *__pyx_v_layers = NULL;
  PyObject *__pyx_v_first_layer_name = NULL;
  PyObject *__pyx_v_dataset_name = NULL;
  PyObject *__pyx_v_other_layer_names = NULL;
  PyObject *__pyx_9genexpr12__pyx_v_lyr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9[3];
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12[7];
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_default_layer", 0);

  /* "pyogrio/_io.pyx":2459
 * 
 *     """
 *     layers = get_layer_names(ogr_dataset)             # <<<<<<<<<<<<<<
 *     first_layer_name = layers[0][0]
 * 
*/
  __pyx_t_1 = __pyx_f_7pyogrio_3_io_get_layer_names(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_layers = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2460
 *     """
 *     layers = get_layer_names(ogr_dataset)
 *     first_layer_name = layers[0][0]             # <<<<<<<<<<<<<<
 * 
 *     if len(layers) > 1:
*/
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_layers, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_first_layer_name = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyogrio/_io.pyx":2462
 *     first_layer_name = layers[0][0]
 * 
 *     if len(layers) > 1:             # <<<<<<<<<<<<<<
 *         dataset_name = os.path.basename(get_string(OGR_DS_GetName(ogr_dataset)))
 * 
*/
  __pyx_t_3 = PyObject_Length(__pyx_v_layers); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2462, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_3 > 1);
  if (__pyx_t_4) {

    /* "pyogrio/_io.pyx":2463
 * 
 *     if len(layers) > 1:
 *         dataset_name = os.path.basename(get_string(OGR_DS_GetName(ogr_dataset)))             # <<<<<<<<<<<<<<
 * 
 *         other_layer_names = ", ".join([f"'{lyr}'" for lyr in layers[1:, 0]])
*/
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __pyx_t_6;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_5 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_DS_GetName(__pyx_v_ogr_dataset), NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_basename, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2463, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_dataset_name = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pyogrio/_io.pyx":2465
 *         dataset_name = os.path.basename(get_string(OGR_DS_GetName(ogr_dataset)))
 * 
 *         other_layer_names = ", ".join([f"'{lyr}'" for lyr in layers[1:, 0]])             # <<<<<<<<<<<<<<
 *         warnings.warn(
 *             f"More than one layer found in '{dataset_name}': '{first_layer_name}' "
*/
    { /* enter inner scope */
      __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2465, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_layers, __pyx_mstate_global->__pyx_tuple[8]); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2465, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
        __pyx_t_5 = __pyx_t_6; __Pyx_INCREF(__pyx_t_5);
        __pyx_t_3 = 0;
        __pyx_t_8 = NULL;
      } else {
        __pyx_t_3 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2465, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2465, __pyx_L6_error)
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      for (;;) {
        if (likely(!__pyx_t_8)) {
          if (likely(PyList_CheckExact(__pyx_t_5))) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_5);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2465, __pyx_L6_error)
              #endif
              if (__pyx_t_3 >= __pyx_temp) break;
            }
            __pyx_t_6 = __Pyx_PyList_GetItemRefFast(__pyx_t_5, __pyx_t_3, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_3;
          } else {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_5);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2465, __pyx_L6_error)
              #endif
              if (__pyx_t_3 >= __pyx_temp) break;
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_6 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_3));
            #else
            __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_5, __pyx_t_3);
            #endif
            ++__pyx_t_3;
          }
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2465, __pyx_L6_error)
        } else {
          __pyx_t_6 = __pyx_t_8(__pyx_t_5);
          if (unlikely(!__pyx_t_6)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2465, __pyx_L6_error)
              PyErr_Clear();
            }
            break;
          }
        }
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_XDECREF_SET(__pyx_9genexpr12__pyx_v_lyr, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_9genexpr12__pyx_v_lyr, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2465, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_9[0] = __pyx_mstate_global->__pyx_kp_u__4;
        __pyx_t_9[1] = __pyx_t_6;
        __pyx_t_9[2] = __pyx_mstate_global->__pyx_kp_u__4;
        __pyx_t_1 = __Pyx_PyUnicode_Join(__pyx_t_9, 3, 1 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6));
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2465, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_1))) __PYX_ERR(0, 2465, __pyx_L6_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_9genexpr12__pyx_v_lyr); __pyx_9genexpr12__pyx_v_lyr = 0;
      goto __pyx_L10_exit_scope;
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_9genexpr12__pyx_v_lyr); __pyx_9genexpr12__pyx_v_lyr = 0;
      goto __pyx_L1_error;
      __pyx_L10_exit_scope:;
    } /* exit inner scope */
    __pyx_t_5 = PyUnicode_Join(__pyx_mstate_global->__pyx_kp_u__12, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_other_layer_names = ((PyObject*)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":2466
 * 
 *         other_layer_names = ", ".join([f"'{lyr}'" for lyr in layers[1:, 0]])
 *         warnings.warn(             # <<<<<<<<<<<<<<
 *             f"More than one layer found in '{dataset_name}': '{first_layer_name}' "
 *             f"(default), {other_layer_names}. Specify layer parameter to avoid this "
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2466, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2466, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyogrio/_io.pyx":2467
 *         other_layer_names = ", ".join([f"'{lyr}'" for lyr in layers[1:, 0]])
 *         warnings.warn(
 *             f"More than one layer found in '{dataset_name}': '{first_layer_name}' "             # <<<<<<<<<<<<<<
 *             f"(default), {other_layer_names}. Specify layer parameter to avoid this "
 *             "warning.",
*/
    __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_dataset_name, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_v_first_layer_name, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);

    /* "pyogrio/_io.pyx":2468
 *         warnings.warn(
 *             f"More than one layer found in '{dataset_name}': '{first_layer_name}' "
 *             f"(default), {other_layer_names}. Specify layer parameter to avoid this "             # <<<<<<<<<<<<<<
 *             "warning.",
 *             stacklevel=2,
*/
    __pyx_t_11 = __Pyx_PyUnicode_Unicode(__pyx_v_other_layer_names); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_12[0] = __pyx_mstate_global->__pyx_kp_u_More_than_one_layer_found_in;
    __pyx_t_12[1] = __pyx_t_1;
    __pyx_t_12[2] = __pyx_mstate_global->__pyx_kp_u__8;
    __pyx_t_12[3] = __pyx_t_10;
    __pyx_t_12[4] = __pyx_mstate_global->__pyx_kp_u_default;
    __pyx_t_12[5] = __pyx_t_11;
    __pyx_t_12[6] = __pyx_mstate_global->__pyx_kp_u_Specify_layer_parameter_to_avoi;

    /* "pyogrio/_io.pyx":2467
 *         other_layer_names = ", ".join([f"'{lyr}'" for lyr in layers[1:, 0]])
 *         warnings.warn(
 *             f"More than one layer found in '{dataset_name}': '{first_layer_name}' "             # <<<<<<<<<<<<<<
 *             f"(default), {other_layer_names}. Specify layer parameter to avoid this "
 *             "warning.",
*/
    __pyx_t_13 = __Pyx_PyUnicode_Join(__pyx_t_12, 7, 30 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 4 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10) + 13 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11) + 48, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11));
    if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_t_13};
      __pyx_t_11 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2466, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_stacklevel, __pyx_mstate_global->__pyx_int_2, __pyx_t_11, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 2466, __pyx_L1_error)
      __pyx_t_5 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_11);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2466, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":2462
 *     first_layer_name = layers[0][0]
 * 
 *     if len(layers) > 1:             # <<<<<<<<<<<<<<
 *         dataset_name = os.path.basename(get_string(OGR_DS_GetName(ogr_dataset)))
 * 
*/
  }

  /* "pyogrio/_io.pyx":2473
 *         )
 * 
 *     return first_layer_name             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_v_first_layer_name;
  __Pyx_INCREF(__pyx_t_5);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_5))) __PYX_ERR(0, 2473, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2443
 * 
 * 
 * cdef str get_default_layer(OGRDataSourceH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Get the layer in the dataset that is read by default.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyogrio._io.get_default_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_layers);
  __Pyx_XDECREF(__pyx_v_first_layer_name);
  __Pyx_XDECREF(__pyx_v_dataset_name);
  __Pyx_XDECREF(__pyx_v_other_layer_names);
  __Pyx_XDECREF(__pyx_9genexpr12__pyx_v_lyr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2476
 * 
 * 
 * cdef get_layer_names(OGRDataSourceH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Get the layers in the dataset.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_layer_names(OGRDataSourceH __pyx_v_ogr_dataset) {
  OGRLayerH __pyx_v_ogr_layer;
  int __pyx_v_layer_count;
  PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_data_view = NULL;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_layer_names", 0);

  /* "pyogrio/_io.pyx":2492
 * 
 *     """
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":2494
 *     cdef OGRLayerH ogr_layer = NULL
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         layer_count = GDALDatasetGetLayerCount(ogr_dataset)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "pyogrio/_io.pyx":2495
 * 
 *     with nogil:
 *         layer_count = GDALDatasetGetLayerCount(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *     data = np.empty(shape=(layer_count, 2), dtype=object)
*/
        __pyx_v_layer_count = GDALDatasetGetLayerCount(__pyx_v_ogr_dataset);
      }

      /* "pyogrio/_io.pyx":2494
 *     cdef OGRLayerH ogr_layer = NULL
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         layer_count = GDALDatasetGetLayerCount(ogr_dataset)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "pyogrio/_io.pyx":2497
 *         layer_count = GDALDatasetGetLayerCount(ogr_dataset)
 * 
 *     data = np.empty(shape=(layer_count, 2), dtype=object)             # <<<<<<<<<<<<<<
 *     data_view = data[:]
 *     for i in range(layer_count):
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_layer_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 2497, __pyx_L1_error);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_2);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_mstate_global->__pyx_int_2) != (0)) __PYX_ERR(0, 2497, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_5, __pyx_t_3, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 2497, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_builtin_object, __pyx_t_3, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 2497, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2497, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_data = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2498
 * 
 *     data = np.empty(shape=(layer_count, 2), dtype=object)
 *     data_view = data[:]             # <<<<<<<<<<<<<<
 *     for i in range(layer_count):
 *         with nogil:
*/
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_data, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_view = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2499
 *     data = np.empty(shape=(layer_count, 2), dtype=object)
 *     data_view = data[:]
 *     for i in range(layer_count):             # <<<<<<<<<<<<<<
 *         with nogil:
 *             ogr_layer = GDALDatasetGetLayer(ogr_dataset, i)
*/
  __pyx_t_7 = __pyx_v_layer_count;
  __pyx_t_8 = __pyx_t_7;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "pyogrio/_io.pyx":2500
 *     data_view = data[:]
 *     for i in range(layer_count):
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_layer = GDALDatasetGetLayer(ogr_dataset, i)
 * 
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "pyogrio/_io.pyx":2501
 *     for i in range(layer_count):
 *         with nogil:
 *             ogr_layer = GDALDatasetGetLayer(ogr_dataset, i)             # <<<<<<<<<<<<<<
 * 
 *         data_view[i, 0] = get_string(OGR_L_GetName(ogr_layer))
*/
          __pyx_v_ogr_layer = GDALDatasetGetLayer(__pyx_v_ogr_dataset, __pyx_v_i);
        }

        /* "pyogrio/_io.pyx":2500
 *     data_view = data[:]
 *     for i in range(layer_count):
 *         with nogil:             # <<<<<<<<<<<<<<
 *             ogr_layer = GDALDatasetGetLayer(ogr_dataset, i)
 * 
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L12;
          }
          __pyx_L12:;
        }
    }

    /* "pyogrio/_io.pyx":2503
 *             ogr_layer = GDALDatasetGetLayer(ogr_dataset, i)
 * 
 *         data_view[i, 0] = get_string(OGR_L_GetName(ogr_layer))             # <<<<<<<<<<<<<<
 *         data_view[i, 1] = get_geometry_type(ogr_layer)
 * 
*/
    __pyx_t_1 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetName(__pyx_v_ogr_layer), NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4) != (0)) __PYX_ERR(0, 2503, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(0, 2503, __pyx_L1_error);
    __pyx_t_4 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_data_view, __pyx_t_3, __pyx_t_1) < 0))) __PYX_ERR(0, 2503, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pyogrio/_io.pyx":2504
 * 
 *         data_view[i, 0] = get_string(OGR_L_GetName(ogr_layer))
 *         data_view[i, 1] = get_geometry_type(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *     return data
*/
    __pyx_t_1 = __pyx_f_7pyogrio_9_geometry_get_geometry_type(__pyx_v_ogr_layer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2504, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2504, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2504, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 2504, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_1) != (0)) __PYX_ERR(0, 2504, __pyx_L1_error);
    __pyx_t_3 = 0;
    if (unlikely((PyObject_SetItem(__pyx_v_data_view, __pyx_t_4, __pyx_t_1) < 0))) __PYX_ERR(0, 2504, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pyogrio/_io.pyx":2506
 *         data_view[i, 1] = get_geometry_type(ogr_layer)
 * 
 *     return data             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2476
 * 
 * 
 * cdef get_layer_names(OGRDataSourceH ogr_dataset):             # <<<<<<<<<<<<<<
 *     """Get the layers in the dataset.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyogrio._io.get_layer_names", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_data_view);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2509
 * 
 * 
 * cdef void * ogr_create(             # <<<<<<<<<<<<<<
 *     const char* path_c, const char* driver_c, char** options
 * ) except NULL:
*/

static void *__pyx_f_7pyogrio_3_io_ogr_create(char const *__pyx_v_path_c, char const *__pyx_v_driver_c, char **__pyx_v_options) {
  void *__pyx_v_ogr_driver;
  OGRDataSourceH __pyx_v_ogr_dataset;
  PyObject *__pyx_v_exc = NULL;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  void *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14[3];
  PyObject *__pyx_t_15 = NULL;
  size_t __pyx_t_16;
  int __pyx_t_17;
  char const *__pyx_t_18;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25[4];
  PyObject *__pyx_t_26 = NULL;
  char const *__pyx_t_27;
  char const *__pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_create", 0);

  /* "pyogrio/_io.pyx":2533
 * 
 *     """
 *     cdef void *ogr_driver = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRDataSourceH ogr_dataset = NULL
 * 
*/
  __pyx_v_ogr_driver = NULL;

  /* "pyogrio/_io.pyx":2534
 *     """
 *     cdef void *ogr_driver = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *     # Get the driver
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":2537
 * 
 *     # Get the driver
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_driver = check_pointer(GDALGetDriverByName(driver_c))
 * 
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "pyogrio/_io.pyx":2538
 *     # Get the driver
 *     try:
 *         ogr_driver = check_pointer(GDALGetDriverByName(driver_c))             # <<<<<<<<<<<<<<
 * 
 *     except NullPointerError:
*/
      __pyx_t_4 = __pyx_f_7pyogrio_4_err_check_pointer(GDALGetDriverByName(__pyx_v_driver_c)); if (unlikely(__pyx_t_4 == ((void *)NULL))) __PYX_ERR(0, 2538, __pyx_L3_error)
      __pyx_v_ogr_driver = __pyx_t_4;

      /* "pyogrio/_io.pyx":2537
 * 
 *     # Get the driver
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_driver = check_pointer(GDALGetDriverByName(driver_c))
 * 
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "pyogrio/_io.pyx":2540
 *         ogr_driver = check_pointer(GDALGetDriverByName(driver_c))
 * 
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(
 *             f"Could not obtain driver: {driver_c.decode('utf-8')} "
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2540, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.ogr_create", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 2540, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);

      /* "pyogrio/_io.pyx":2541
 * 
 *     except NullPointerError:
 *         raise DataSourceError(             # <<<<<<<<<<<<<<
 *             f"Could not obtain driver: {driver_c.decode('utf-8')} "
 *             "(check that it was installed correctly into GDAL)"
*/
      __pyx_t_10 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2541, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_11);

      /* "pyogrio/_io.pyx":2542
 *     except NullPointerError:
 *         raise DataSourceError(
 *             f"Could not obtain driver: {driver_c.decode('utf-8')} "             # <<<<<<<<<<<<<<
 *             "(check that it was installed correctly into GDAL)"
 *         )
*/
      __pyx_t_12 = __Pyx_ssize_strlen(__pyx_v_driver_c); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2542, __pyx_L5_except_error)
      __pyx_t_13 = __Pyx_decode_c_string(__pyx_v_driver_c, 0, __pyx_t_12, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2542, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_14[0] = __pyx_mstate_global->__pyx_kp_u_Could_not_obtain_driver;
      __pyx_t_14[1] = __pyx_t_13;
      __pyx_t_14[2] = __pyx_mstate_global->__pyx_kp_u_check_that_it_was_installed_cor;
      __pyx_t_15 = __Pyx_PyUnicode_Join(__pyx_t_14, 3, 25 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13) + 50, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13));
      if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2542, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_16 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
        assert(__pyx_t_10);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
        __pyx_t_16 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_15};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2541, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 2541, __pyx_L5_except_error)
    }

    /* "pyogrio/_io.pyx":2546
 *         )
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(str(exc))
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2546, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.ogr_create", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 2546, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_exc = __pyx_t_6;
      /*try:*/ {

        /* "pyogrio/_io.pyx":2547
 * 
 *     except CPLE_BaseError as exc:
 *         raise DataSourceError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *     # For /vsimem/ files, with GDAL >= 3.8 parent directories are created automatically.
*/
        __pyx_t_11 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2547, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_10 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2547, __pyx_L16_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_16 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_15))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_15);
          assert(__pyx_t_11);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
          __pyx_t_16 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_10};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2547, __pyx_L16_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 2547, __pyx_L16_error)
      }

      /* "pyogrio/_io.pyx":2546
 *         )
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(str(exc))
 * 
*/
      /*finally:*/ {
        __pyx_L16_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
          if ( unlikely(__Pyx_GetException(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21) < 0)) __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __pyx_t_9 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_18 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_23, __pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ErrRestore(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
          __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_18;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "pyogrio/_io.pyx":2537
 * 
 *     # Get the driver
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_driver = check_pointer(GDALGetDriverByName(driver_c))
 * 
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "pyogrio/_io.pyx":2560
 * 
 *     # Create the dataset
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_dataset = check_pointer(
 *             GDALCreate(ogr_driver, path_c, 0, 0, 0, GDT_Unknown, options)
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_1);
    /*try:*/ {

      /* "pyogrio/_io.pyx":2561
 *     # Create the dataset
 *     try:
 *         ogr_dataset = check_pointer(             # <<<<<<<<<<<<<<
 *             GDALCreate(ogr_driver, path_c, 0, 0, 0, GDT_Unknown, options)
 *         )
*/
      __pyx_t_4 = __pyx_f_7pyogrio_4_err_check_pointer(GDALCreate(__pyx_v_ogr_driver, __pyx_v_path_c, 0, 0, 0, GDT_Unknown, __pyx_v_options)); if (unlikely(__pyx_t_4 == ((void *)NULL))) __PYX_ERR(0, 2561, __pyx_L22_error)
      __pyx_v_ogr_dataset = __pyx_t_4;

      /* "pyogrio/_io.pyx":2560
 * 
 *     # Create the dataset
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_dataset = check_pointer(
 *             GDALCreate(ogr_driver, path_c, 0, 0, 0, GDT_Unknown, options)
*/
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L27_try_end;
    __pyx_L22_error:;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "pyogrio/_io.pyx":2565
 *         )
 * 
 *     except NullPointerError:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(
 *             f"Failed to create dataset with driver: {path_c.decode('utf-8')} "
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_NullPointerError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2565, __pyx_L24_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_17 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_17) {
      __Pyx_AddTraceback("pyogrio._io.ogr_create", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 2565, __pyx_L24_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);

      /* "pyogrio/_io.pyx":2566
 * 
 *     except NullPointerError:
 *         raise DataSourceError(             # <<<<<<<<<<<<<<
 *             f"Failed to create dataset with driver: {path_c.decode('utf-8')} "
 *             f"{driver_c.decode('utf-8')}"
*/
      __pyx_t_15 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2566, __pyx_L24_except_error)
      __Pyx_GOTREF(__pyx_t_10);

      /* "pyogrio/_io.pyx":2567
 *     except NullPointerError:
 *         raise DataSourceError(
 *             f"Failed to create dataset with driver: {path_c.decode('utf-8')} "             # <<<<<<<<<<<<<<
 *             f"{driver_c.decode('utf-8')}"
 *         ) from None
*/
      __pyx_t_12 = __Pyx_ssize_strlen(__pyx_v_path_c); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2567, __pyx_L24_except_error)
      __pyx_t_11 = __Pyx_decode_c_string(__pyx_v_path_c, 0, __pyx_t_12, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2567, __pyx_L24_except_error)
      __Pyx_GOTREF(__pyx_t_11);

      /* "pyogrio/_io.pyx":2568
 *         raise DataSourceError(
 *             f"Failed to create dataset with driver: {path_c.decode('utf-8')} "
 *             f"{driver_c.decode('utf-8')}"             # <<<<<<<<<<<<<<
 *         ) from None
 * 
*/
      __pyx_t_12 = __Pyx_ssize_strlen(__pyx_v_driver_c); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2568, __pyx_L24_except_error)
      __pyx_t_13 = __Pyx_decode_c_string(__pyx_v_driver_c, 0, __pyx_t_12, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2568, __pyx_L24_except_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_25[0] = __pyx_mstate_global->__pyx_kp_u_Failed_to_create_dataset_with_dr;
      __pyx_t_25[1] = __pyx_t_11;
      __pyx_t_25[2] = __pyx_mstate_global->__pyx_kp_u__13;
      __pyx_t_25[3] = __pyx_t_13;

      /* "pyogrio/_io.pyx":2567
 *     except NullPointerError:
 *         raise DataSourceError(
 *             f"Failed to create dataset with driver: {path_c.decode('utf-8')} "             # <<<<<<<<<<<<<<
 *             f"{driver_c.decode('utf-8')}"
 *         ) from None
*/
      __pyx_t_26 = __Pyx_PyUnicode_Join(__pyx_t_25, 4, 38 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11) + 1 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13));
      if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 2567, __pyx_L24_except_error)
      __Pyx_GOTREF(__pyx_t_26);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_16 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_10);
        assert(__pyx_t_15);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_15);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
        __pyx_t_16 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_t_26};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2566, __pyx_L24_except_error)
        __Pyx_GOTREF(__pyx_t_8);
      }

      /* "pyogrio/_io.pyx":2569
 *             f"Failed to create dataset with driver: {path_c.decode('utf-8')} "
 *             f"{driver_c.decode('utf-8')}"
 *         ) from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_NotSupportedError as exc:
*/
      __Pyx_Raise(__pyx_t_8, 0, 0, Py_None);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 2566, __pyx_L24_except_error)
    }

    /* "pyogrio/_io.pyx":2571
 *         ) from None
 * 
 *     except CPLE_NotSupportedError as exc:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(
 *             f"Driver {driver_c.decode('utf-8')} does not support write functionality"
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_NotSupportedError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2571, __pyx_L24_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_17 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_17) {
      __Pyx_AddTraceback("pyogrio._io.ogr_create", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 2571, __pyx_L24_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_exc = __pyx_t_6;
      /*try:*/ {

        /* "pyogrio/_io.pyx":2572
 * 
 *     except CPLE_NotSupportedError as exc:
 *         raise DataSourceError(             # <<<<<<<<<<<<<<
 *             f"Driver {driver_c.decode('utf-8')} does not support write functionality"
 *         ) from None
*/
        __pyx_t_10 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_26, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 2572, __pyx_L35_error)
        __Pyx_GOTREF(__pyx_t_26);

        /* "pyogrio/_io.pyx":2573
 *     except CPLE_NotSupportedError as exc:
 *         raise DataSourceError(
 *             f"Driver {driver_c.decode('utf-8')} does not support write functionality"             # <<<<<<<<<<<<<<
 *         ) from None
 * 
*/
        __pyx_t_12 = __Pyx_ssize_strlen(__pyx_v_driver_c); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2573, __pyx_L35_error)
        __pyx_t_15 = __Pyx_decode_c_string(__pyx_v_driver_c, 0, __pyx_t_12, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2573, __pyx_L35_error)
        __Pyx_GOTREF(__pyx_t_15);
        __pyx_t_14[0] = __pyx_mstate_global->__pyx_kp_u_Driver;
        __pyx_t_14[1] = __pyx_t_15;
        __pyx_t_14[2] = __pyx_mstate_global->__pyx_kp_u_does_not_support_write_function;
        __pyx_t_13 = __Pyx_PyUnicode_Join(__pyx_t_14, 3, 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 37, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15));
        if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2573, __pyx_L35_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_16 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_26))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_26);
          assert(__pyx_t_10);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_26);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_26, __pyx__function);
          __pyx_t_16 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_13};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_26, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2572, __pyx_L35_error)
          __Pyx_GOTREF(__pyx_t_8);
        }

        /* "pyogrio/_io.pyx":2574
 *         raise DataSourceError(
 *             f"Driver {driver_c.decode('utf-8')} does not support write functionality"
 *         ) from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
        __Pyx_Raise(__pyx_t_8, 0, 0, Py_None);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 2572, __pyx_L35_error)
      }

      /* "pyogrio/_io.pyx":2571
 *         ) from None
 * 
 *     except CPLE_NotSupportedError as exc:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(
 *             f"Driver {driver_c.decode('utf-8')} does not support write functionality"
*/
      /*finally:*/ {
        __pyx_L35_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_21, &__pyx_t_20, &__pyx_t_19);
          if ( unlikely(__Pyx_GetException(&__pyx_t_24, &__pyx_t_23, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_24, &__pyx_t_23, &__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_24);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_19);
          __pyx_t_17 = __pyx_lineno; __pyx_t_9 = __pyx_clineno; __pyx_t_27 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_20, __pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_ErrRestore(__pyx_t_24, __pyx_t_23, __pyx_t_22);
          __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0;
          __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_9; __pyx_filename = __pyx_t_27;
          goto __pyx_L24_except_error;
        }
      }
    }

    /* "pyogrio/_io.pyx":2576
 *         ) from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(str(exc))
 * 
*/
    __Pyx_ErrFetch(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2576, __pyx_L24_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_5, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    __pyx_t_5 = 0; __pyx_t_6 = 0; __pyx_t_7 = 0;
    if (__pyx_t_9) {
      __Pyx_AddTraceback("pyogrio._io.ogr_create", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0) __PYX_ERR(0, 2576, __pyx_L24_except_error)
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_v_exc = __pyx_t_6;
      /*try:*/ {

        /* "pyogrio/_io.pyx":2577
 * 
 *     except CPLE_BaseError as exc:
 *         raise DataSourceError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *     return ogr_dataset
*/
        __pyx_t_26 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2577, __pyx_L46_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_10 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2577, __pyx_L46_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_16 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_13))) {
          __pyx_t_26 = PyMethod_GET_SELF(__pyx_t_13);
          assert(__pyx_t_26);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_26);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
          __pyx_t_16 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_26, __pyx_t_10};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_16, (2-__pyx_t_16) | (__pyx_t_16*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2577, __pyx_L46_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 2577, __pyx_L46_error)
      }

      /* "pyogrio/_io.pyx":2576
 *         ) from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         raise DataSourceError(str(exc))
 * 
*/
      /*finally:*/ {
        __pyx_L46_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
          if ( unlikely(__Pyx_GetException(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21) < 0)) __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __pyx_t_9 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_28 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_23, __pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ErrRestore(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
          __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_28;
          goto __pyx_L24_except_error;
        }
      }
    }
    goto __pyx_L24_except_error;

    /* "pyogrio/_io.pyx":2560
 * 
 *     # Create the dataset
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_dataset = check_pointer(
 *             GDALCreate(ogr_driver, path_c, 0, 0, 0, GDT_Unknown, options)
*/
    __pyx_L24_except_error:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_2, __pyx_t_1);
    goto __pyx_L1_error;
    __pyx_L27_try_end:;
  }

  /* "pyogrio/_io.pyx":2579
 *         raise DataSourceError(str(exc))
 * 
 *     return ogr_dataset             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_ogr_dataset;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2509
 * 
 * 
 * cdef void * ogr_create(             # <<<<<<<<<<<<<<
 *     const char* path_c, const char* driver_c, char** options
 * ) except NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_AddTraceback("pyogrio._io.ogr_create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2582
 * 
 * 
 * cdef void * create_crs(str crs) except NULL:             # <<<<<<<<<<<<<<
 *     """Create an OGRSpatialReferenceH object from a CRS string.
 * 
*/

static void *__pyx_f_7pyogrio_3_io_create_crs(PyObject *__pyx_v_crs) {
  char *__pyx_v_crs_c;
  void *__pyx_v_ogr_crs;
  PyObject *__pyx_v_crs_b = NULL;
  int __pyx_v_err;
  PyObject *__pyx_v_exc = NULL;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  void *__pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  size_t __pyx_t_14;
  int __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_crs", 0);

  /* "pyogrio/_io.pyx":2596
 * 
 *     """
 *     cdef char *crs_c = NULL             # <<<<<<<<<<<<<<
 *     cdef void *ogr_crs = NULL
 * 
*/
  __pyx_v_crs_c = NULL;

  /* "pyogrio/_io.pyx":2597
 *     """
 *     cdef char *crs_c = NULL
 *     cdef void *ogr_crs = NULL             # <<<<<<<<<<<<<<
 * 
 *     crs_b = crs.encode("UTF-8")
*/
  __pyx_v_ogr_crs = NULL;

  /* "pyogrio/_io.pyx":2599
 *     cdef void *ogr_crs = NULL
 * 
 *     crs_b = crs.encode("UTF-8")             # <<<<<<<<<<<<<<
 *     crs_c = crs_b
 * 
*/
  if (unlikely(__pyx_v_crs == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 2599, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_crs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_crs_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2600
 * 
 *     crs_b = crs.encode("UTF-8")
 *     crs_c = crs_b             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
  __pyx_t_2 = __Pyx_PyBytes_AsWritableString(__pyx_v_crs_b); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 2600, __pyx_L1_error)
  __pyx_v_crs_c = __pyx_t_2;

  /* "pyogrio/_io.pyx":2602
 *     crs_c = crs_b
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_crs = check_pointer(OSRNewSpatialReference(NULL))
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "pyogrio/_io.pyx":2603
 * 
 *     try:
 *         ogr_crs = check_pointer(OSRNewSpatialReference(NULL))             # <<<<<<<<<<<<<<
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)
 *         if err:
*/
      __pyx_t_6 = __pyx_f_7pyogrio_4_err_check_pointer(OSRNewSpatialReference(NULL)); if (unlikely(__pyx_t_6 == ((void *)NULL))) __PYX_ERR(0, 2603, __pyx_L3_error)
      __pyx_v_ogr_crs = __pyx_t_6;

      /* "pyogrio/_io.pyx":2604
 *     try:
 *         ogr_crs = check_pointer(OSRNewSpatialReference(NULL))
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)             # <<<<<<<<<<<<<<
 *         if err:
 *             raise CRSError(
*/
      __pyx_v_err = OSRSetFromUserInput(__pyx_v_ogr_crs, __pyx_v_crs_c);

      /* "pyogrio/_io.pyx":2605
 *         ogr_crs = check_pointer(OSRNewSpatialReference(NULL))
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)
 *         if err:             # <<<<<<<<<<<<<<
 *             raise CRSError(
 *                 "Could not set CRS: {}".format(crs_c.decode("UTF-8"))
*/
      __pyx_t_7 = (__pyx_v_err != 0);
      if (unlikely(__pyx_t_7)) {

        /* "pyogrio/_io.pyx":2606
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)
 *         if err:
 *             raise CRSError(             # <<<<<<<<<<<<<<
 *                 "Could not set CRS: {}".format(crs_c.decode("UTF-8"))
 *             ) from None
*/
        __pyx_t_8 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_CRSError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2606, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_9);

        /* "pyogrio/_io.pyx":2607
 *         if err:
 *             raise CRSError(
 *                 "Could not set CRS: {}".format(crs_c.decode("UTF-8"))             # <<<<<<<<<<<<<<
 *             ) from None
 * 
*/
        __pyx_t_11 = __pyx_mstate_global->__pyx_kp_u_Could_not_set_CRS;
        __Pyx_INCREF(__pyx_t_11);
        __pyx_t_12 = __Pyx_ssize_strlen(__pyx_v_crs_c); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2607, __pyx_L3_error)
        __pyx_t_13 = __Pyx_decode_c_string(__pyx_v_crs_c, 0, __pyx_t_12, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2607, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_14 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_13};
          __pyx_t_10 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2607, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_10);
        }
        __pyx_t_14 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_9))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_9);
          assert(__pyx_t_8);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
          __pyx_t_14 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_10};
          __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2606, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);
        }

        /* "pyogrio/_io.pyx":2608
 *             raise CRSError(
 *                 "Could not set CRS: {}".format(crs_c.decode("UTF-8"))
 *             ) from None             # <<<<<<<<<<<<<<
 * 
 *     except CPLE_BaseError as exc:
*/
        __Pyx_Raise(__pyx_t_1, 0, 0, Py_None);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 2606, __pyx_L3_error)

        /* "pyogrio/_io.pyx":2605
 *         ogr_crs = check_pointer(OSRNewSpatialReference(NULL))
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)
 *         if err:             # <<<<<<<<<<<<<<
 *             raise CRSError(
 *                 "Could not set CRS: {}".format(crs_c.decode("UTF-8"))
*/
      }

      /* "pyogrio/_io.pyx":2602
 *     crs_c = crs_b
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_crs = check_pointer(OSRNewSpatialReference(NULL))
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)
*/
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "pyogrio/_io.pyx":2610
 *             ) from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         OSRRelease(ogr_crs)
 *         raise CRSError("Could not set CRS: {}".format(exc))
*/
    __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_9, &__pyx_t_10);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2610, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_15 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_ErrRestore(__pyx_t_1, __pyx_t_9, __pyx_t_10);
    __pyx_t_1 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0;
    if (__pyx_t_15) {
      __Pyx_AddTraceback("pyogrio._io.create_crs", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_10, &__pyx_t_9, &__pyx_t_1) < 0) __PYX_ERR(0, 2610, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_9);
      __pyx_v_exc = __pyx_t_9;
      /*try:*/ {

        /* "pyogrio/_io.pyx":2611
 * 
 *     except CPLE_BaseError as exc:
 *         OSRRelease(ogr_crs)             # <<<<<<<<<<<<<<
 *         raise CRSError("Could not set CRS: {}".format(exc))
 * 
*/
        OSRRelease(__pyx_v_ogr_crs);

        /* "pyogrio/_io.pyx":2612
 *     except CPLE_BaseError as exc:
 *         OSRRelease(ogr_crs)
 *         raise CRSError("Could not set CRS: {}".format(exc))             # <<<<<<<<<<<<<<
 * 
 *     return ogr_crs
*/
        __pyx_t_13 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_CRSError); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 2612, __pyx_L15_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_17 = __pyx_mstate_global->__pyx_kp_u_Could_not_set_CRS;
        __Pyx_INCREF(__pyx_t_17);
        __pyx_t_14 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_17, __pyx_v_exc};
          __pyx_t_16 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_format, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2612, __pyx_L15_error)
          __Pyx_GOTREF(__pyx_t_16);
        }
        __pyx_t_14 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_11);
          assert(__pyx_t_13);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
          __pyx_t_14 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_16};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_14, (2-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2612, __pyx_L15_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 2612, __pyx_L15_error)
      }

      /* "pyogrio/_io.pyx":2610
 *             ) from None
 * 
 *     except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *         OSRRelease(ogr_crs)
 *         raise CRSError("Could not set CRS: {}".format(exc))
*/
      /*finally:*/ {
        __pyx_L15_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
          if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __Pyx_XGOTREF(__pyx_t_25);
          __pyx_t_15 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_25);
          __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __pyx_lineno = __pyx_t_15; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
          goto __pyx_L5_except_error;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "pyogrio/_io.pyx":2602
 *     crs_c = crs_b
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         ogr_crs = check_pointer(OSRNewSpatialReference(NULL))
 *         err = OSRSetFromUserInput(ogr_crs, crs_c)
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "pyogrio/_io.pyx":2614
 *         raise CRSError("Could not set CRS: {}".format(exc))
 * 
 *     return ogr_crs             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_ogr_crs;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2582
 * 
 * 
 * cdef void * create_crs(str crs) except NULL:             # <<<<<<<<<<<<<<
 *     """Create an OGRSpatialReferenceH object from a CRS string.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("pyogrio._io.create_crs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_crs_b);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2617
 * 
 * 
 * cdef infer_field_types(list dtypes):             # <<<<<<<<<<<<<<
 *     """Infer OGR field types from numpy dtypes.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_infer_field_types(PyObject *__pyx_v_dtypes) {
  int __pyx_v_field_type;
  int __pyx_v_field_subtype;
  PyObject *__pyx_v_field_types = NULL;
  PyObject *__pyx_v_field_types_view = NULL;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  PyObject *__pyx_t_15[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("infer_field_types", 0);

  /* "pyogrio/_io.pyx":2632
 * 
 *     """
 *     cdef int field_type = 0             # <<<<<<<<<<<<<<
 *     cdef int field_subtype = 0
 * 
*/
  __pyx_v_field_type = 0;

  /* "pyogrio/_io.pyx":2633
 *     """
 *     cdef int field_type = 0
 *     cdef int field_subtype = 0             # <<<<<<<<<<<<<<
 * 
 *     field_types = np.zeros(shape=(len(dtypes), 4), dtype=int)
*/
  __pyx_v_field_subtype = 0;

  /* "pyogrio/_io.pyx":2635
 *     cdef int field_subtype = 0
 * 
 *     field_types = np.zeros(shape=(len(dtypes), 4), dtype=int)             # <<<<<<<<<<<<<<
 *     field_types_view = field_types[:]
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_zeros); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_v_dtypes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 2635, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyList_GET_SIZE(__pyx_v_dtypes); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2635, __pyx_L1_error)
  __pyx_t_3 = PyLong_FromSsize_t(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 2635, __pyx_L1_error);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_4);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_mstate_global->__pyx_int_4) != (0)) __PYX_ERR(0, 2635, __pyx_L1_error);
  __pyx_t_3 = 0;
  __pyx_t_7 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_7 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_shape, __pyx_t_6, __pyx_t_3, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 2635, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, ((PyObject *)(&PyLong_Type)), __pyx_t_3, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 2635, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (1-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2635, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_field_types = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2636
 * 
 *     field_types = np.zeros(shape=(len(dtypes), 4), dtype=int)
 *     field_types_view = field_types[:]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(len(dtypes)):
*/
  __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_field_types, 0, 0, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_field_types_view = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2638
 *     field_types_view = field_types[:]
 * 
 *     for i in range(len(dtypes)):             # <<<<<<<<<<<<<<
 *         dtype = dtypes[i]
 * 
*/
  if (unlikely(__pyx_v_dtypes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 2638, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyList_GET_SIZE(__pyx_v_dtypes); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2638, __pyx_L1_error)
  __pyx_t_8 = __pyx_t_5;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "pyogrio/_io.pyx":2639
 * 
 *     for i in range(len(dtypes)):
 *         dtype = dtypes[i]             # <<<<<<<<<<<<<<
 * 
 *         if dtype.name in DTYPE_OGR_FIELD_TYPES:
*/
    if (unlikely(__pyx_v_dtypes == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 2639, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_dtypes, __pyx_v_i, Py_ssize_t, 1, PyLong_FromSsize_t, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2639, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyogrio/_io.pyx":2641
 *         dtype = dtypes[i]
 * 
 *         if dtype.name in DTYPE_OGR_FIELD_TYPES:             # <<<<<<<<<<<<<<
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES[dtype.name]
 *             field_types_view[i, 0] = field_type
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_DTYPE_OGR_FIELD_TYPES); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_t_4, Py_EQ)); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 2641, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_10) {

      /* "pyogrio/_io.pyx":2642
 * 
 *         if dtype.name in DTYPE_OGR_FIELD_TYPES:
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES[dtype.name]             # <<<<<<<<<<<<<<
 *             field_types_view[i, 0] = field_type
 *             field_types_view[i, 1] = field_subtype
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_DTYPE_OGR_FIELD_TYPES); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2642, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2642, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2642, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
        PyObject* sequence = __pyx_t_3;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 2642, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_1);
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_4);
        } else {
          __pyx_t_1 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2642, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2642, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_4);
        }
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2642, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2642, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_6 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2642, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6);
        index = 0; __pyx_t_1 = __pyx_t_11(__pyx_t_6); if (unlikely(!__pyx_t_1)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        index = 1; __pyx_t_4 = __pyx_t_11(__pyx_t_6); if (unlikely(!__pyx_t_4)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_6), 2) < (0)) __PYX_ERR(0, 2642, __pyx_L1_error)
        __pyx_t_11 = NULL;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_11 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 2642, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __pyx_t_12 = __Pyx_PyLong_As_int(__pyx_t_1); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2642, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2642, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_v_field_type = __pyx_t_12;
      __pyx_v_field_subtype = __pyx_t_13;

      /* "pyogrio/_io.pyx":2643
 *         if dtype.name in DTYPE_OGR_FIELD_TYPES:
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES[dtype.name]
 *             field_types_view[i, 0] = field_type             # <<<<<<<<<<<<<<
 *             field_types_view[i, 1] = field_subtype
 * 
*/
      __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_field_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2643, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyLong_FromSsize_t(__pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2643, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2643, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_4);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4) != (0)) __PYX_ERR(0, 2643, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(0, 2643, __pyx_L1_error);
      __pyx_t_4 = 0;
      if (unlikely((PyObject_SetItem(__pyx_v_field_types_view, __pyx_t_1, __pyx_t_3) < 0))) __PYX_ERR(0, 2643, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyogrio/_io.pyx":2644
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES[dtype.name]
 *             field_types_view[i, 0] = field_type
 *             field_types_view[i, 1] = field_subtype             # <<<<<<<<<<<<<<
 * 
 *         # Determine field type from ndarray values
*/
      __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_field_subtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2644, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_1);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 2644, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_1);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_1) != (0)) __PYX_ERR(0, 2644, __pyx_L1_error);
      __pyx_t_1 = 0;
      if (unlikely((PyObject_SetItem(__pyx_v_field_types_view, __pyx_t_4, __pyx_t_3) < 0))) __PYX_ERR(0, 2644, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyogrio/_io.pyx":2641
 *         dtype = dtypes[i]
 * 
 *         if dtype.name in DTYPE_OGR_FIELD_TYPES:             # <<<<<<<<<<<<<<
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES[dtype.name]
 *             field_types_view[i, 0] = field_type
*/
      goto __pyx_L5;
    }

    /* "pyogrio/_io.pyx":2647
 * 
 *         # Determine field type from ndarray values
 *         elif dtype == np.dtype("O"):             # <<<<<<<<<<<<<<
 *             # Object type is ambiguous: could be a string or binary data
 *             # TODO: handle binary or other types
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_n_u_O};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2647, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_6 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2647, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 2647, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_10) {

      /* "pyogrio/_io.pyx":2651
 *             # TODO: handle binary or other types
 *             # for now fall back to string (same as Geopandas)
 *             field_types_view[i, 0] = OFTString             # <<<<<<<<<<<<<<
 *             # Convert to unicode string then take itemsize
 *             # TODO: better implementation of this
*/
      __pyx_t_6 = __Pyx_PyLong_From_OGRFieldType(OFTString); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 2651, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(0, 2651, __pyx_L1_error);
      __pyx_t_3 = 0;
      if (unlikely((PyObject_SetItem(__pyx_v_field_types_view, __pyx_t_4, __pyx_t_6) < 0))) __PYX_ERR(0, 2651, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":2647
 * 
 *         # Determine field type from ndarray values
 *         elif dtype == np.dtype("O"):             # <<<<<<<<<<<<<<
 *             # Object type is ambiguous: could be a string or binary data
 *             # TODO: handle binary or other types
*/
      goto __pyx_L5;
    }

    /* "pyogrio/_io.pyx":2657
 *             # DO WE NEED WIDTH HERE?
 * 
 *         elif dtype.type is np.str_ or dtype.type is np.bytes_:             # <<<<<<<<<<<<<<
 *             field_types_view[i, 0] = OFTString
 *             field_types_view[i, 2] = int(dtype.itemsize // 4)
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_14 = (__pyx_t_6 == __pyx_t_3);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_14) {
    } else {
      __pyx_t_10 = __pyx_t_14;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_bytes); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2657, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_14 = (__pyx_t_3 == __pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = __pyx_t_14;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_10) {

      /* "pyogrio/_io.pyx":2658
 * 
 *         elif dtype.type is np.str_ or dtype.type is np.bytes_:
 *             field_types_view[i, 0] = OFTString             # <<<<<<<<<<<<<<
 *             field_types_view[i, 2] = int(dtype.itemsize // 4)
 * 
*/
      __pyx_t_4 = __Pyx_PyLong_From_OGRFieldType(OFTString); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2658, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2658, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2658, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 2658, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(0, 2658, __pyx_L1_error);
      __pyx_t_3 = 0;
      if (unlikely((PyObject_SetItem(__pyx_v_field_types_view, __pyx_t_6, __pyx_t_4) < 0))) __PYX_ERR(0, 2658, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyogrio/_io.pyx":2659
 *         elif dtype.type is np.str_ or dtype.type is np.bytes_:
 *             field_types_view[i, 0] = OFTString
 *             field_types_view[i, 2] = int(dtype.itemsize // 4)             # <<<<<<<<<<<<<<
 * 
 *         elif dtype.name.startswith("datetime64"):
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2659, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyLong_FloorDivideObjC(__pyx_t_4, __pyx_mstate_global->__pyx_int_4, 4, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2659, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyNumber_Int(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2659, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyLong_FromSsize_t(__pyx_v_i); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2659, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2659, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6) != (0)) __PYX_ERR(0, 2659, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_2);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_2);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_2) != (0)) __PYX_ERR(0, 2659, __pyx_L1_error);
      __pyx_t_6 = 0;
      if (unlikely((PyObject_SetItem(__pyx_v_field_types_view, __pyx_t_3, __pyx_t_4) < 0))) __PYX_ERR(0, 2659, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pyogrio/_io.pyx":2657
 *             # DO WE NEED WIDTH HERE?
 * 
 *         elif dtype.type is np.str_ or dtype.type is np.bytes_:             # <<<<<<<<<<<<<<
 *             field_types_view[i, 0] = OFTString
 *             field_types_view[i, 2] = int(dtype.itemsize // 4)
*/
      goto __pyx_L5;
    }

    /* "pyogrio/_io.pyx":2661
 *             field_types_view[i, 2] = int(dtype.itemsize // 4)
 * 
 *         elif dtype.name.startswith("datetime64"):             # <<<<<<<<<<<<<<
 *             # datetime dtype precision is specified with eg. [ms], but this isn't
 *             # usefull when writing to gdal.
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = __pyx_t_6;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_n_u_datetime64};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_startswith, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2661, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 2661, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (likely(__pyx_t_10)) {

      /* "pyogrio/_io.pyx":2664
 *             # datetime dtype precision is specified with eg. [ms], but this isn't
 *             # usefull when writing to gdal.
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES["datetime64"]             # <<<<<<<<<<<<<<
 *             field_types_view[i, 0] = field_type
 *             field_types_view[i, 1] = field_subtype
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_DTYPE_OGR_FIELD_TYPES); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_datetime64); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
        PyObject* sequence = __pyx_t_6;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 2664, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_4);
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_3);
        } else {
          __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2664, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_4);
          __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2664, __pyx_L1_error)
          __Pyx_XGOTREF(__pyx_t_3);
        }
        #else
        __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2664, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2664, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_1 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2664, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1);
        index = 0; __pyx_t_4 = __pyx_t_11(__pyx_t_1); if (unlikely(!__pyx_t_4)) goto __pyx_L10_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        index = 1; __pyx_t_3 = __pyx_t_11(__pyx_t_1); if (unlikely(!__pyx_t_3)) goto __pyx_L10_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_1), 2) < (0)) __PYX_ERR(0, 2664, __pyx_L1_error)
        __pyx_t_11 = NULL;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L11_unpacking_done;
        __pyx_L10_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_11 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 2664, __pyx_L1_error)
        __pyx_L11_unpacking_done:;
      }
      __pyx_t_13 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2664, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_12 = __Pyx_PyLong_As_int(__pyx_t_3); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2664, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_field_type = __pyx_t_13;
      __pyx_v_field_subtype = __pyx_t_12;

      /* "pyogrio/_io.pyx":2665
 *             # usefull when writing to gdal.
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES["datetime64"]
 *             field_types_view[i, 0] = field_type             # <<<<<<<<<<<<<<
 *             field_types_view[i, 1] = field_subtype
 * 
*/
      __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_field_type); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2665, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = PyLong_FromSsize_t(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2665, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2665, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 2665, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_0);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_mstate_global->__pyx_int_0) != (0)) __PYX_ERR(0, 2665, __pyx_L1_error);
      __pyx_t_3 = 0;
      if (unlikely((PyObject_SetItem(__pyx_v_field_types_view, __pyx_t_4, __pyx_t_6) < 0))) __PYX_ERR(0, 2665, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":2666
 *             field_type, field_subtype = DTYPE_OGR_FIELD_TYPES["datetime64"]
 *             field_types_view[i, 0] = field_type
 *             field_types_view[i, 1] = field_subtype             # <<<<<<<<<<<<<<
 * 
 *         else:
*/
      __pyx_t_6 = __Pyx_PyLong_From_int(__pyx_v_field_subtype); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2666, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_4 = PyLong_FromSsize_t(__pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2666, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2666, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4) != (0)) __PYX_ERR(0, 2666, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_1);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_1);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_mstate_global->__pyx_int_1) != (0)) __PYX_ERR(0, 2666, __pyx_L1_error);
      __pyx_t_4 = 0;
      if (unlikely((PyObject_SetItem(__pyx_v_field_types_view, __pyx_t_3, __pyx_t_6) < 0))) __PYX_ERR(0, 2666, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":2661
 *             field_types_view[i, 2] = int(dtype.itemsize // 4)
 * 
 *         elif dtype.name.startswith("datetime64"):             # <<<<<<<<<<<<<<
 *             # datetime dtype precision is specified with eg. [ms], but this isn't
 *             # usefull when writing to gdal.
*/
      goto __pyx_L5;
    }

    /* "pyogrio/_io.pyx":2669
 * 
 *         else:
 *             raise NotImplementedError(             # <<<<<<<<<<<<<<
 *                 f"field type is not supported {dtype.name} (field index: {i})"
 *             )
*/
    /*else*/ {
      __pyx_t_3 = NULL;

      /* "pyogrio/_io.pyx":2670
 *         else:
 *             raise NotImplementedError(
 *                 f"field type is not supported {dtype.name} (field index: {i})"             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2670, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2670, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyUnicode_From_Py_ssize_t(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2670, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_15[0] = __pyx_mstate_global->__pyx_kp_u_field_type_is_not_supported;
      __pyx_t_15[1] = __pyx_t_1;
      __pyx_t_15[2] = __pyx_mstate_global->__pyx_kp_u_field_index;
      __pyx_t_15[3] = __pyx_t_4;
      __pyx_t_15[4] = __pyx_mstate_global->__pyx_kp_u__5;
      __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_15, 5, 28 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 15 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2670, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_7 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_NotImplementedError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2669, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 2669, __pyx_L1_error)
    }
    __pyx_L5:;
  }

  /* "pyogrio/_io.pyx":2673
 *             )
 * 
 *     return field_types             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_field_types);
  __pyx_r = __pyx_v_field_types;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2617
 * 
 * 
 * cdef infer_field_types(list dtypes):             # <<<<<<<<<<<<<<
 *     """Infer OGR field types from numpy dtypes.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyogrio._io.infer_field_types", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_field_types);
  __Pyx_XDECREF(__pyx_v_field_types_view);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2676
 * 
 * 
 * cdef create_ogr_dataset_layer(             # <<<<<<<<<<<<<<
 *     str path,
 *     bint use_tmp_vsimem,
*/

static PyObject *__pyx_f_7pyogrio_3_io_create_ogr_dataset_layer(PyObject *__pyx_v_path, int __pyx_v_use_tmp_vsimem, PyObject *__pyx_v_layer, PyObject *__pyx_v_driver, PyObject *__pyx_v_crs, PyObject *__pyx_v_geometry_type, PyObject *__pyx_v_encoding, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer_kwargs, int __pyx_v_append, PyObject *__pyx_v_dataset_metadata, PyObject *__pyx_v_layer_metadata, OGRDataSourceH *__pyx_v_ogr_dataset_out, OGRLayerH *__pyx_v_ogr_layer_out) {
  char const *__pyx_v_path_c;
  char const *__pyx_v_layer_c;
  char const *__pyx_v_driver_c;
  char const *__pyx_v_encoding_c;
  char **__pyx_v_dataset_options;
  char **__pyx_v_layer_options;
  OGRDataSourceH __pyx_v_ogr_dataset;
  OGRLayerH __pyx_v_ogr_layer;
  OGRSpatialReferenceH __pyx_v_ogr_crs;
  OGRwkbGeometryType __pyx_v_geometry_code;
  int __pyx_v_layer_idx;
  PyObject *__pyx_v_path_b = NULL;
  PyObject *__pyx_v_driver_b = NULL;
  PyObject *__pyx_v_path_exists = NULL;
  PyObject *__pyx_v_multi_ext = NULL;
  int __pyx_v_layer_exists;
  int __pyx_v_i;
  char const *__pyx_v_name;
  PyObject *__pyx_v_exc = NULL;
  int __pyx_v_create_layer;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_encoding_b = NULL;
  PyObject *__pyx_v_layer_b = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char const *__pyx_t_2;
  char const *__pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  Py_ssize_t __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  void *__pyx_t_21;
  int __pyx_t_22;
  int __pyx_t_23;
  int __pyx_t_24;
  char const *__pyx_t_25;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27 = NULL;
  PyObject *__pyx_t_28 = NULL;
  PyObject *__pyx_t_29 = NULL;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  char **__pyx_t_32;
  char const *__pyx_t_33;
  char const *__pyx_t_34;
  char const *__pyx_t_35;
  char const *__pyx_t_36;
  OGRwkbGeometryType __pyx_t_37;
  char const *__pyx_t_38;
  OGRLayerH __pyx_t_39;
  char const *__pyx_t_40;
  char const *__pyx_t_41;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_ogr_dataset_layer", 0);
  __Pyx_INCREF(__pyx_v_layer);
  __Pyx_INCREF(__pyx_v_encoding);

  /* "pyogrio/_io.pyx":2720
 * 
 *     """
 *     cdef const char *path_c = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *layer_c = NULL
 *     cdef const char *driver_c = NULL
*/
  __pyx_v_path_c = NULL;

  /* "pyogrio/_io.pyx":2721
 *     """
 *     cdef const char *path_c = NULL
 *     cdef const char *layer_c = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *driver_c = NULL
 *     cdef const char *encoding_c = NULL
*/
  __pyx_v_layer_c = NULL;

  /* "pyogrio/_io.pyx":2722
 *     cdef const char *path_c = NULL
 *     cdef const char *layer_c = NULL
 *     cdef const char *driver_c = NULL             # <<<<<<<<<<<<<<
 *     cdef const char *encoding_c = NULL
 *     cdef char **dataset_options = NULL
*/
  __pyx_v_driver_c = NULL;

  /* "pyogrio/_io.pyx":2723
 *     cdef const char *layer_c = NULL
 *     cdef const char *driver_c = NULL
 *     cdef const char *encoding_c = NULL             # <<<<<<<<<<<<<<
 *     cdef char **dataset_options = NULL
 *     cdef char **layer_options = NULL
*/
  __pyx_v_encoding_c = NULL;

  /* "pyogrio/_io.pyx":2724
 *     cdef const char *driver_c = NULL
 *     cdef const char *encoding_c = NULL
 *     cdef char **dataset_options = NULL             # <<<<<<<<<<<<<<
 *     cdef char **layer_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
*/
  __pyx_v_dataset_options = NULL;

  /* "pyogrio/_io.pyx":2725
 *     cdef const char *encoding_c = NULL
 *     cdef char **dataset_options = NULL
 *     cdef char **layer_options = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
*/
  __pyx_v_layer_options = NULL;

  /* "pyogrio/_io.pyx":2726
 *     cdef char **dataset_options = NULL
 *     cdef char **layer_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef OGRSpatialReferenceH ogr_crs = NULL
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":2727
 *     cdef char **layer_options = NULL
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRSpatialReferenceH ogr_crs = NULL
 *     cdef OGRwkbGeometryType geometry_code
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":2728
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef OGRSpatialReferenceH ogr_crs = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRwkbGeometryType geometry_code
 *     cdef int layer_idx = -1
*/
  __pyx_v_ogr_crs = NULL;

  /* "pyogrio/_io.pyx":2730
 *     cdef OGRSpatialReferenceH ogr_crs = NULL
 *     cdef OGRwkbGeometryType geometry_code
 *     cdef int layer_idx = -1             # <<<<<<<<<<<<<<
 * 
 *     path_b = path.encode("UTF-8")
*/
  __pyx_v_layer_idx = -1;

  /* "pyogrio/_io.pyx":2732
 *     cdef int layer_idx = -1
 * 
 *     path_b = path.encode("UTF-8")             # <<<<<<<<<<<<<<
 *     path_c = path_b
 * 
*/
  if (unlikely(__pyx_v_path == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 2732, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_path); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2733
 * 
 *     path_b = path.encode("UTF-8")
 *     path_c = path_b             # <<<<<<<<<<<<<<
 * 
 *     driver_b = driver.encode("UTF-8")
*/
  __pyx_t_2 = __Pyx_PyBytes_AsString(__pyx_v_path_b); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 2733, __pyx_L1_error)
  __pyx_v_path_c = __pyx_t_2;

  /* "pyogrio/_io.pyx":2735
 *     path_c = path_b
 * 
 *     driver_b = driver.encode("UTF-8")             # <<<<<<<<<<<<<<
 *     driver_c = driver_b
 * 
*/
  if (unlikely(__pyx_v_driver == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 2735, __pyx_L1_error)
  }
  __pyx_t_1 = PyUnicode_AsUTF8String(__pyx_v_driver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_driver_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2736
 * 
 *     driver_b = driver.encode("UTF-8")
 *     driver_c = driver_b             # <<<<<<<<<<<<<<
 * 
 *     # temporary in-memory dataset is always created from scratch
*/
  __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_driver_b); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 2736, __pyx_L1_error)
  __pyx_v_driver_c = __pyx_t_3;

  /* "pyogrio/_io.pyx":2739
 * 
 *     # temporary in-memory dataset is always created from scratch
 *     path_exists = os.path.exists(path) if not use_tmp_vsimem else False             # <<<<<<<<<<<<<<
 * 
 *     if not layer:
*/
  __pyx_t_4 = (!__pyx_v_use_tmp_vsimem);
  if (__pyx_t_4) {
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2739, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2739, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __pyx_t_8;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_9 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_path};
      __pyx_t_5 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_exists, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2739, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __pyx_t_1 = __pyx_t_5;
    __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(Py_False);
    __pyx_t_1 = Py_False;
  }
  __pyx_v_path_exists = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":2741
 *     path_exists = os.path.exists(path) if not use_tmp_vsimem else False
 * 
 *     if not layer:             # <<<<<<<<<<<<<<
 *         # For multi extensions (e.g. ".shp.zip"), strip the full extension
 *         for multi_ext in MULTI_EXTENSIONS:
*/
  if (__pyx_v_layer == Py_None) __pyx_t_4 = 0;
  else
  {
    Py_ssize_t __pyx_temp = __Pyx_PyUnicode_IS_TRUE(__pyx_v_layer);
    if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 2741, __pyx_L1_error)
    __pyx_t_4 = (__pyx_temp != 0);
  }

  __pyx_t_10 = (!__pyx_t_4);
  if (__pyx_t_10) {

    /* "pyogrio/_io.pyx":2743
 *     if not layer:
 *         # For multi extensions (e.g. ".shp.zip"), strip the full extension
 *         for multi_ext in MULTI_EXTENSIONS:             # <<<<<<<<<<<<<<
 *             if path.endswith(multi_ext):
 *                 layer = os.path.split(path)[1][:-len(multi_ext)]
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_MULTI_EXTENSIONS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_5 = __pyx_t_1; __Pyx_INCREF(__pyx_t_5);
      __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_11 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2743, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_12 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_5); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 2743, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_5);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2743, __pyx_L1_error)
            #endif
            if (__pyx_t_11 >= __pyx_temp) break;
          }
          __pyx_t_1 = __Pyx_PyList_GetItemRefFast(__pyx_t_5, __pyx_t_11, __Pyx_ReferenceSharing_OwnStrongReference);
          ++__pyx_t_11;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_5);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 2743, __pyx_L1_error)
            #endif
            if (__pyx_t_11 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_11));
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_5, __pyx_t_11);
          #endif
          ++__pyx_t_11;
        }
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2743, __pyx_L1_error)
      } else {
        __pyx_t_1 = __pyx_t_12(__pyx_t_5);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 2743, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_multi_ext, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "pyogrio/_io.pyx":2744
 *         # For multi extensions (e.g. ".shp.zip"), strip the full extension
 *         for multi_ext in MULTI_EXTENSIONS:
 *             if path.endswith(multi_ext):             # <<<<<<<<<<<<<<
 *                 layer = os.path.split(path)[1][:-len(multi_ext)]
 *                 break
*/
      if (unlikely(__pyx_v_path == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
        __PYX_ERR(0, 2744, __pyx_L1_error)
      }
      __pyx_t_10 = __Pyx_PyUnicode_Tailmatch(__pyx_v_path, __pyx_v_multi_ext, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 2744, __pyx_L1_error)
      if (__pyx_t_10) {

        /* "pyogrio/_io.pyx":2745
 *         for multi_ext in MULTI_EXTENSIONS:
 *             if path.endswith(multi_ext):
 *                 layer = os.path.split(path)[1][:-len(multi_ext)]             # <<<<<<<<<<<<<<
 *                 break
 * 
*/
        __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2745, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2745, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_8 = __pyx_t_7;
        __Pyx_INCREF(__pyx_t_8);
        __pyx_t_9 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_path};
          __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_split, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2745, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
        }
        __pyx_t_7 = __Pyx_GetItemInt(__pyx_t_1, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2745, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_13 = PyObject_Length(__pyx_v_multi_ext); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2745, __pyx_L1_error)
        __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_t_7, 0, (-__pyx_t_13), NULL, NULL, NULL, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2745, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 2745, __pyx_L1_error)
        __Pyx_DECREF_SET(__pyx_v_layer, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "pyogrio/_io.pyx":2746
 *             if path.endswith(multi_ext):
 *                 layer = os.path.split(path)[1][:-len(multi_ext)]
 *                 break             # <<<<<<<<<<<<<<
 * 
 *         # If it wasn't a multi-extension, use the file stem
*/
        goto __pyx_L5_break;

        /* "pyogrio/_io.pyx":2744
 *         # For multi extensions (e.g. ".shp.zip"), strip the full extension
 *         for multi_ext in MULTI_EXTENSIONS:
 *             if path.endswith(multi_ext):             # <<<<<<<<<<<<<<
 *                 layer = os.path.split(path)[1][:-len(multi_ext)]
 *                 break
*/
      }

      /* "pyogrio/_io.pyx":2743
 *     if not layer:
 *         # For multi extensions (e.g. ".shp.zip"), strip the full extension
 *         for multi_ext in MULTI_EXTENSIONS:             # <<<<<<<<<<<<<<
 *             if path.endswith(multi_ext):
 *                 layer = os.path.split(path)[1][:-len(multi_ext)]
*/
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L7_for_end;
    __pyx_L5_break:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L7_for_end;
    __pyx_L7_for_end:;

    /* "pyogrio/_io.pyx":2749
 * 
 *         # If it wasn't a multi-extension, use the file stem
 *         if not layer:             # <<<<<<<<<<<<<<
 *             layer = os.path.splitext(os.path.split(path)[1])[0]
 * 
*/
    if (__pyx_v_layer == Py_None) __pyx_t_10 = 0;
    else
    {
      Py_ssize_t __pyx_temp = __Pyx_PyUnicode_IS_TRUE(__pyx_v_layer);
      if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 2749, __pyx_L1_error)
      __pyx_t_10 = (__pyx_temp != 0);
    }

    __pyx_t_4 = (!__pyx_t_10);
    if (__pyx_t_4) {

      /* "pyogrio/_io.pyx":2750
 *         # If it wasn't a multi-extension, use the file stem
 *         if not layer:
 *             layer = os.path.splitext(os.path.split(path)[1])[0]             # <<<<<<<<<<<<<<
 * 
 *     # if shapefile, GeoJSON, or FlatGeobuf, always delete first
*/
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2750, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2750, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_1 = __pyx_t_8;
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2750, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2750, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_6 = __pyx_t_15;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_9 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_path};
        __pyx_t_7 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_split, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2750, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __pyx_t_15 = __Pyx_GetItemInt(__pyx_t_7, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2750, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_9 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_15};
        __pyx_t_5 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_splitext, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2750, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __pyx_t_8 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2750, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (!(likely(PyUnicode_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_8))) __PYX_ERR(0, 2750, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_layer, ((PyObject*)__pyx_t_8));
      __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2749
 * 
 *         # If it wasn't a multi-extension, use the file stem
 *         if not layer:             # <<<<<<<<<<<<<<
 *             layer = os.path.splitext(os.path.split(path)[1])[0]
 * 
*/
    }

    /* "pyogrio/_io.pyx":2741
 *     path_exists = os.path.exists(path) if not use_tmp_vsimem else False
 * 
 *     if not layer:             # <<<<<<<<<<<<<<
 *         # For multi extensions (e.g. ".shp.zip"), strip the full extension
 *         for multi_ext in MULTI_EXTENSIONS:
*/
  }

  /* "pyogrio/_io.pyx":2756
 *     # GPKG might be the only multi-layer writeable type.  TODO: check this
 *     if (
 *         driver in ("ESRI Shapefile", "GeoJSON", "GeoJSONSeq", "FlatGeobuf")             # <<<<<<<<<<<<<<
 *         and path_exists
 *     ):
*/
  __Pyx_INCREF(__pyx_v_driver);
  __pyx_t_16 = __pyx_v_driver;
  __pyx_t_17 = (__Pyx_PyUnicode_Equals(__pyx_t_16, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_EQ)); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2756, __pyx_L1_error)
  if (!__pyx_t_17) {
  } else {
    __pyx_t_10 = __pyx_t_17;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_17 = (__Pyx_PyUnicode_Equals(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_GeoJSON, Py_EQ)); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2756, __pyx_L1_error)
  if (!__pyx_t_17) {
  } else {
    __pyx_t_10 = __pyx_t_17;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_17 = (__Pyx_PyUnicode_Equals(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_GeoJSONSeq, Py_EQ)); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2756, __pyx_L1_error)
  if (!__pyx_t_17) {
  } else {
    __pyx_t_10 = __pyx_t_17;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_17 = (__Pyx_PyUnicode_Equals(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_FlatGeobuf, Py_EQ)); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2756, __pyx_L1_error)
  __pyx_t_10 = __pyx_t_17;
  __pyx_L12_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_17 = __pyx_t_10;
  if (__pyx_t_17) {
  } else {
    __pyx_t_4 = __pyx_t_17;
    goto __pyx_L10_bool_binop_done;
  }

  /* "pyogrio/_io.pyx":2757
 *     if (
 *         driver in ("ESRI Shapefile", "GeoJSON", "GeoJSONSeq", "FlatGeobuf")
 *         and path_exists             # <<<<<<<<<<<<<<
 *     ):
 *         if not append:
*/
  __pyx_t_17 = __Pyx_PyObject_IsTrue(__pyx_v_path_exists); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2757, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_17;
  __pyx_L10_bool_binop_done:;

  /* "pyogrio/_io.pyx":2755
 *     # for other types, check if we can create layers
 *     # GPKG might be the only multi-layer writeable type.  TODO: check this
 *     if (             # <<<<<<<<<<<<<<
 *         driver in ("ESRI Shapefile", "GeoJSON", "GeoJSONSeq", "FlatGeobuf")
 *         and path_exists
*/
  if (__pyx_t_4) {

    /* "pyogrio/_io.pyx":2759
 *         and path_exists
 *     ):
 *         if not append:             # <<<<<<<<<<<<<<
 *             os.unlink(path)
 *             path_exists = False
*/
    __pyx_t_4 = (!__pyx_v_append);
    if (__pyx_t_4) {

      /* "pyogrio/_io.pyx":2760
 *     ):
 *         if not append:
 *             os.unlink(path)             # <<<<<<<<<<<<<<
 *             path_exists = False
 * 
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2760, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_unlink); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2760, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_9 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
        __pyx_t_9 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_path};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2760, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2761
 *         if not append:
 *             os.unlink(path)
 *             path_exists = False             # <<<<<<<<<<<<<<
 * 
 *     layer_exists = False
*/
      __Pyx_INCREF(Py_False);
      __Pyx_DECREF_SET(__pyx_v_path_exists, Py_False);

      /* "pyogrio/_io.pyx":2759
 *         and path_exists
 *     ):
 *         if not append:             # <<<<<<<<<<<<<<
 *             os.unlink(path)
 *             path_exists = False
*/
    }

    /* "pyogrio/_io.pyx":2755
 *     # for other types, check if we can create layers
 *     # GPKG might be the only multi-layer writeable type.  TODO: check this
 *     if (             # <<<<<<<<<<<<<<
 *         driver in ("ESRI Shapefile", "GeoJSON", "GeoJSONSeq", "FlatGeobuf")
 *         and path_exists
*/
  }

  /* "pyogrio/_io.pyx":2763
 *             path_exists = False
 * 
 *     layer_exists = False             # <<<<<<<<<<<<<<
 *     if path_exists:
 *         try:
*/
  __pyx_v_layer_exists = 0;

  /* "pyogrio/_io.pyx":2764
 * 
 *     layer_exists = False
 *     if path_exists:             # <<<<<<<<<<<<<<
 *         try:
 *             ogr_dataset = ogr_open(path_c, 1, NULL)
*/
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_path_exists); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2764, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "pyogrio/_io.pyx":2765
 *     layer_exists = False
 *     if path_exists:
 *         try:             # <<<<<<<<<<<<<<
 *             ogr_dataset = ogr_open(path_c, 1, NULL)
 * 
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      /*try:*/ {

        /* "pyogrio/_io.pyx":2766
 *     if path_exists:
 *         try:
 *             ogr_dataset = ogr_open(path_c, 1, NULL)             # <<<<<<<<<<<<<<
 * 
 *             for i in range(GDALDatasetGetLayerCount(ogr_dataset)):
*/
        __pyx_t_21 = __pyx_f_7pyogrio_3_io_ogr_open(__pyx_v_path_c, 1, NULL); if (unlikely(__pyx_t_21 == ((void *)NULL))) __PYX_ERR(0, 2766, __pyx_L18_error)
        __pyx_v_ogr_dataset = __pyx_t_21;

        /* "pyogrio/_io.pyx":2768
 *             ogr_dataset = ogr_open(path_c, 1, NULL)
 * 
 *             for i in range(GDALDatasetGetLayerCount(ogr_dataset)):             # <<<<<<<<<<<<<<
 *                 name = OGR_L_GetName(GDALDatasetGetLayer(ogr_dataset, i))
 *                 if layer == name.decode("UTF-8"):
*/
        __pyx_t_22 = GDALDatasetGetLayerCount(__pyx_v_ogr_dataset);
        __pyx_t_23 = __pyx_t_22;
        for (__pyx_t_24 = 0; __pyx_t_24 < __pyx_t_23; __pyx_t_24+=1) {
          __pyx_v_i = __pyx_t_24;

          /* "pyogrio/_io.pyx":2769
 * 
 *             for i in range(GDALDatasetGetLayerCount(ogr_dataset)):
 *                 name = OGR_L_GetName(GDALDatasetGetLayer(ogr_dataset, i))             # <<<<<<<<<<<<<<
 *                 if layer == name.decode("UTF-8"):
 *                     layer_idx = i
*/
          __pyx_v_name = OGR_L_GetName(GDALDatasetGetLayer(__pyx_v_ogr_dataset, __pyx_v_i));

          /* "pyogrio/_io.pyx":2770
 *             for i in range(GDALDatasetGetLayerCount(ogr_dataset)):
 *                 name = OGR_L_GetName(GDALDatasetGetLayer(ogr_dataset, i))
 *                 if layer == name.decode("UTF-8"):             # <<<<<<<<<<<<<<
 *                     layer_idx = i
 *                     break
*/
          __pyx_t_11 = __Pyx_ssize_strlen(__pyx_v_name); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2770, __pyx_L18_error)
          __pyx_t_8 = __Pyx_decode_c_string(__pyx_v_name, 0, __pyx_t_11, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2770, __pyx_L18_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_layer, __pyx_t_8, Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2770, __pyx_L18_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (__pyx_t_4) {

            /* "pyogrio/_io.pyx":2771
 *                 name = OGR_L_GetName(GDALDatasetGetLayer(ogr_dataset, i))
 *                 if layer == name.decode("UTF-8"):
 *                     layer_idx = i             # <<<<<<<<<<<<<<
 *                     break
 * 
*/
            __pyx_v_layer_idx = __pyx_v_i;

            /* "pyogrio/_io.pyx":2772
 *                 if layer == name.decode("UTF-8"):
 *                     layer_idx = i
 *                     break             # <<<<<<<<<<<<<<
 * 
 *             if layer_idx >= 0:
*/
            goto __pyx_L25_break;

            /* "pyogrio/_io.pyx":2770
 *             for i in range(GDALDatasetGetLayerCount(ogr_dataset)):
 *                 name = OGR_L_GetName(GDALDatasetGetLayer(ogr_dataset, i))
 *                 if layer == name.decode("UTF-8"):             # <<<<<<<<<<<<<<
 *                     layer_idx = i
 *                     break
*/
          }
        }
        __pyx_L25_break:;

        /* "pyogrio/_io.pyx":2774
 *                     break
 * 
 *             if layer_idx >= 0:             # <<<<<<<<<<<<<<
 *                 layer_exists = True
 * 
*/
        __pyx_t_4 = (__pyx_v_layer_idx >= 0);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":2775
 * 
 *             if layer_idx >= 0:
 *                 layer_exists = True             # <<<<<<<<<<<<<<
 * 
 *                 if not append:
*/
          __pyx_v_layer_exists = 1;

          /* "pyogrio/_io.pyx":2777
 *                 layer_exists = True
 * 
 *                 if not append:             # <<<<<<<<<<<<<<
 *                     GDALDatasetDeleteLayer(ogr_dataset, layer_idx)
 * 
*/
          __pyx_t_4 = (!__pyx_v_append);
          if (__pyx_t_4) {

            /* "pyogrio/_io.pyx":2778
 * 
 *                 if not append:
 *                     GDALDatasetDeleteLayer(ogr_dataset, layer_idx)             # <<<<<<<<<<<<<<
 * 
 *         except DataSourceError as exc:
*/
            (void)(GDALDatasetDeleteLayer(__pyx_v_ogr_dataset, __pyx_v_layer_idx));

            /* "pyogrio/_io.pyx":2777
 *                 layer_exists = True
 * 
 *                 if not append:             # <<<<<<<<<<<<<<
 *                     GDALDatasetDeleteLayer(ogr_dataset, layer_idx)
 * 
*/
          }

          /* "pyogrio/_io.pyx":2774
 *                     break
 * 
 *             if layer_idx >= 0:             # <<<<<<<<<<<<<<
 *                 layer_exists = True
 * 
*/
        }

        /* "pyogrio/_io.pyx":2765
 *     layer_exists = False
 *     if path_exists:
 *         try:             # <<<<<<<<<<<<<<
 *             ogr_dataset = ogr_open(path_c, 1, NULL)
 * 
*/
      }
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
      goto __pyx_L23_try_end;
      __pyx_L18_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2780
 *                     GDALDatasetDeleteLayer(ogr_dataset, layer_idx)
 * 
 *         except DataSourceError as exc:             # <<<<<<<<<<<<<<
 *             # open failed
 *             if append:
*/
      __Pyx_ErrFetch(&__pyx_t_8, &__pyx_t_1, &__pyx_t_5);
      __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2780, __pyx_L20_except_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_22 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_8, __pyx_t_15);
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_ErrRestore(__pyx_t_8, __pyx_t_1, __pyx_t_5);
      __pyx_t_8 = 0; __pyx_t_1 = 0; __pyx_t_5 = 0;
      if (__pyx_t_22) {
        __Pyx_AddTraceback("pyogrio._io.create_ogr_dataset_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_1, &__pyx_t_8) < 0) __PYX_ERR(0, 2780, __pyx_L20_except_error)
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_v_exc = __pyx_t_1;
        /*try:*/ {

          /* "pyogrio/_io.pyx":2782
 *         except DataSourceError as exc:
 *             # open failed
 *             if append:             # <<<<<<<<<<<<<<
 *                 raise exc
 * 
*/
          if (unlikely(__pyx_v_append)) {

            /* "pyogrio/_io.pyx":2783
 *             # open failed
 *             if append:
 *                 raise exc             # <<<<<<<<<<<<<<
 * 
 *             # otherwise create from scratch
*/
            __Pyx_Raise(__pyx_v_exc, 0, 0, 0);
            __PYX_ERR(0, 2783, __pyx_L34_error)

            /* "pyogrio/_io.pyx":2782
 *         except DataSourceError as exc:
 *             # open failed
 *             if append:             # <<<<<<<<<<<<<<
 *                 raise exc
 * 
*/
          }

          /* "pyogrio/_io.pyx":2787
 *             # otherwise create from scratch
 *             if (
 *                 driver == "OpenFileGDB"             # <<<<<<<<<<<<<<
 *                 and os.path.isdir(path)
 *                 and os.path.splitext(path)[1] == ".gdb"
*/
          __pyx_t_17 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_n_u_OpenFileGDB, Py_EQ)); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2787, __pyx_L34_error)
          if (__pyx_t_17) {
          } else {
            __pyx_t_4 = __pyx_t_17;
            goto __pyx_L38_bool_binop_done;
          }

          /* "pyogrio/_io.pyx":2788
 *             if (
 *                 driver == "OpenFileGDB"
 *                 and os.path.isdir(path)             # <<<<<<<<<<<<<<
 *                 and os.path.splitext(path)[1] == ".gdb"
 *             ):
*/
          __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2788, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2788, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_7 = __pyx_t_14;
          __Pyx_INCREF(__pyx_t_7);
          __pyx_t_9 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_path};
            __pyx_t_15 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_isdir, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2788, __pyx_L34_error)
            __Pyx_GOTREF(__pyx_t_15);
          }
          __pyx_t_17 = __Pyx_PyObject_IsTrue(__pyx_t_15); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2788, __pyx_L34_error)
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (__pyx_t_17) {
          } else {
            __pyx_t_4 = __pyx_t_17;
            goto __pyx_L38_bool_binop_done;
          }

          /* "pyogrio/_io.pyx":2789
 *                 driver == "OpenFileGDB"
 *                 and os.path.isdir(path)
 *                 and os.path.splitext(path)[1] == ".gdb"             # <<<<<<<<<<<<<<
 *             ):
 *                 # An FileGDB "file" is a directory instead of a file, so use rmtree
*/
          __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2789, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2789, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_14 = __pyx_t_6;
          __Pyx_INCREF(__pyx_t_14);
          __pyx_t_9 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_path};
            __pyx_t_15 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_splitext, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2789, __pyx_L34_error)
            __Pyx_GOTREF(__pyx_t_15);
          }
          __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_15, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2789, __pyx_L34_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_17 = (__Pyx_PyUnicode_Equals(__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_gdb, Py_EQ)); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2789, __pyx_L34_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_4 = __pyx_t_17;
          __pyx_L38_bool_binop_done:;

          /* "pyogrio/_io.pyx":2786
 * 
 *             # otherwise create from scratch
 *             if (             # <<<<<<<<<<<<<<
 *                 driver == "OpenFileGDB"
 *                 and os.path.isdir(path)
*/
          if (__pyx_t_4) {

            /* "pyogrio/_io.pyx":2792
 *             ):
 *                 # An FileGDB "file" is a directory instead of a file, so use rmtree
 *                 shutil.rmtree(path)             # <<<<<<<<<<<<<<
 *             else:
 *                 os.unlink(path)
*/
            __pyx_t_15 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_shutil); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2792, __pyx_L34_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_rmtree); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2792, __pyx_L34_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __pyx_t_9 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_7))) {
              __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_7);
              assert(__pyx_t_15);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_15);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
              __pyx_t_9 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_v_path};
              __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2792, __pyx_L34_error)
              __Pyx_GOTREF(__pyx_t_6);
            }
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

            /* "pyogrio/_io.pyx":2786
 * 
 *             # otherwise create from scratch
 *             if (             # <<<<<<<<<<<<<<
 *                 driver == "OpenFileGDB"
 *                 and os.path.isdir(path)
*/
            goto __pyx_L37;
          }

          /* "pyogrio/_io.pyx":2794
 *                 shutil.rmtree(path)
 *             else:
 *                 os.unlink(path)             # <<<<<<<<<<<<<<
 * 
 *             ogr_dataset = NULL
*/
          /*else*/ {
            __pyx_t_7 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2794, __pyx_L34_error)
            __Pyx_GOTREF(__pyx_t_15);
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_unlink); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2794, __pyx_L34_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            __pyx_t_9 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_14);
              assert(__pyx_t_7);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
              __Pyx_INCREF(__pyx_t_7);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
              __pyx_t_9 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_path};
              __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2794, __pyx_L34_error)
              __Pyx_GOTREF(__pyx_t_6);
            }
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
          __pyx_L37:;

          /* "pyogrio/_io.pyx":2796
 *                 os.unlink(path)
 * 
 *             ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *     # either it didn't exist or could not open it in write mode
*/
          __pyx_v_ogr_dataset = NULL;
        }

        /* "pyogrio/_io.pyx":2780
 *                     GDALDatasetDeleteLayer(ogr_dataset, layer_idx)
 * 
 *         except DataSourceError as exc:             # <<<<<<<<<<<<<<
 *             # open failed
 *             if append:
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
            goto __pyx_L35;
          }
          __pyx_L34_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_26 = 0; __pyx_t_27 = 0; __pyx_t_28 = 0; __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_29, &__pyx_t_30, &__pyx_t_31);
            if ( unlikely(__Pyx_GetException(&__pyx_t_26, &__pyx_t_27, &__pyx_t_28) < 0)) __Pyx_ErrFetch(&__pyx_t_26, &__pyx_t_27, &__pyx_t_28);
            __Pyx_XGOTREF(__pyx_t_26);
            __Pyx_XGOTREF(__pyx_t_27);
            __Pyx_XGOTREF(__pyx_t_28);
            __Pyx_XGOTREF(__pyx_t_29);
            __Pyx_XGOTREF(__pyx_t_30);
            __Pyx_XGOTREF(__pyx_t_31);
            __pyx_t_22 = __pyx_lineno; __pyx_t_23 = __pyx_clineno; __pyx_t_25 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_29);
            __Pyx_XGIVEREF(__pyx_t_30);
            __Pyx_XGIVEREF(__pyx_t_31);
            __Pyx_ExceptionReset(__pyx_t_29, __pyx_t_30, __pyx_t_31);
            __Pyx_XGIVEREF(__pyx_t_26);
            __Pyx_XGIVEREF(__pyx_t_27);
            __Pyx_XGIVEREF(__pyx_t_28);
            __Pyx_ErrRestore(__pyx_t_26, __pyx_t_27, __pyx_t_28);
            __pyx_t_26 = 0; __pyx_t_27 = 0; __pyx_t_28 = 0; __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0;
            __pyx_lineno = __pyx_t_22; __pyx_clineno = __pyx_t_23; __pyx_filename = __pyx_t_25;
            goto __pyx_L20_except_error;
          }
          __pyx_L35:;
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L19_exception_handled;
      }
      goto __pyx_L20_except_error;

      /* "pyogrio/_io.pyx":2765
 *     layer_exists = False
 *     if path_exists:
 *         try:             # <<<<<<<<<<<<<<
 *             ogr_dataset = ogr_open(path_c, 1, NULL)
 * 
*/
      __pyx_L20_except_error:;
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      goto __pyx_L1_error;
      __pyx_L19_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      __pyx_L23_try_end:;
    }

    /* "pyogrio/_io.pyx":2764
 * 
 *     layer_exists = False
 *     if path_exists:             # <<<<<<<<<<<<<<
 *         try:
 *             ogr_dataset = ogr_open(path_c, 1, NULL)
*/
  }

  /* "pyogrio/_io.pyx":2799
 * 
 *     # either it didn't exist or could not open it in write mode
 *     if ogr_dataset == NULL:             # <<<<<<<<<<<<<<
 *         dataset_options = dict_to_options(dataset_kwargs)
 *         ogr_dataset = ogr_create(path_c, driver_c, dataset_options)
*/
  __pyx_t_4 = (__pyx_v_ogr_dataset == NULL);
  if (__pyx_t_4) {

    /* "pyogrio/_io.pyx":2800
 *     # either it didn't exist or could not open it in write mode
 *     if ogr_dataset == NULL:
 *         dataset_options = dict_to_options(dataset_kwargs)             # <<<<<<<<<<<<<<
 *         ogr_dataset = ogr_create(path_c, driver_c, dataset_options)
 * 
*/
    __pyx_t_32 = __pyx_f_7pyogrio_3_io_dict_to_options(__pyx_v_dataset_kwargs); if (unlikely(__pyx_t_32 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 2800, __pyx_L1_error)
    __pyx_v_dataset_options = __pyx_t_32;

    /* "pyogrio/_io.pyx":2801
 *     if ogr_dataset == NULL:
 *         dataset_options = dict_to_options(dataset_kwargs)
 *         ogr_dataset = ogr_create(path_c, driver_c, dataset_options)             # <<<<<<<<<<<<<<
 * 
 *     # if we are not appending to an existing layer, we need to create
*/
    __pyx_t_21 = __pyx_f_7pyogrio_3_io_ogr_create(__pyx_v_path_c, __pyx_v_driver_c, __pyx_v_dataset_options); if (unlikely(__pyx_t_21 == ((void *)NULL))) __PYX_ERR(0, 2801, __pyx_L1_error)
    __pyx_v_ogr_dataset = __pyx_t_21;

    /* "pyogrio/_io.pyx":2799
 * 
 *     # either it didn't exist or could not open it in write mode
 *     if ogr_dataset == NULL:             # <<<<<<<<<<<<<<
 *         dataset_options = dict_to_options(dataset_kwargs)
 *         ogr_dataset = ogr_create(path_c, driver_c, dataset_options)
*/
  }

  /* "pyogrio/_io.pyx":2805
 *     # if we are not appending to an existing layer, we need to create
 *     # the layer and all associated properties (CRS, field defs, etc)
 *     create_layer = not (append and layer_exists)             # <<<<<<<<<<<<<<
 * 
 *     # Create the layer
*/
  if (__pyx_v_append) {
  } else {
    __pyx_t_4 = __pyx_v_append;
    goto __pyx_L46_bool_binop_done;
  }
  __pyx_t_4 = __pyx_v_layer_exists;
  __pyx_L46_bool_binop_done:;
  __pyx_v_create_layer = (!__pyx_t_4);

  /* "pyogrio/_io.pyx":2808
 * 
 *     # Create the layer
 *     if create_layer:             # <<<<<<<<<<<<<<
 *         # Create the CRS
 *         if crs is not None:
*/
  if (__pyx_v_create_layer) {

    /* "pyogrio/_io.pyx":2810
 *     if create_layer:
 *         # Create the CRS
 *         if crs is not None:             # <<<<<<<<<<<<<<
 *             try:
 *                 ogr_crs = create_crs(crs)
*/
    __pyx_t_4 = (__pyx_v_crs != ((PyObject*)Py_None));
    if (__pyx_t_4) {

      /* "pyogrio/_io.pyx":2811
 *         # Create the CRS
 *         if crs is not None:
 *             try:             # <<<<<<<<<<<<<<
 *                 ogr_crs = create_crs(crs)
 *                 # force geographic CRS to use lon, lat order and ignore axis order
*/
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_20, &__pyx_t_19, &__pyx_t_18);
        __Pyx_XGOTREF(__pyx_t_20);
        __Pyx_XGOTREF(__pyx_t_19);
        __Pyx_XGOTREF(__pyx_t_18);
        /*try:*/ {

          /* "pyogrio/_io.pyx":2812
 *         if crs is not None:
 *             try:
 *                 ogr_crs = create_crs(crs)             # <<<<<<<<<<<<<<
 *                 # force geographic CRS to use lon, lat order and ignore axis order
 *                 # specified by CRS, in order to correctly write KML and GeoJSON
*/
          __pyx_t_21 = __pyx_f_7pyogrio_3_io_create_crs(__pyx_v_crs); if (unlikely(__pyx_t_21 == ((void *)NULL))) __PYX_ERR(0, 2812, __pyx_L50_error)
          __pyx_v_ogr_crs = __pyx_t_21;

          /* "pyogrio/_io.pyx":2816
 *                 # specified by CRS, in order to correctly write KML and GeoJSON
 *                 # coordinates in correct order
 *                 OSRSetAxisMappingStrategy(ogr_crs, OAMS_TRADITIONAL_GIS_ORDER)             # <<<<<<<<<<<<<<
 * 
 *             except Exception as exc:
*/
          OSRSetAxisMappingStrategy(__pyx_v_ogr_crs, OAMS_TRADITIONAL_GIS_ORDER);

          /* "pyogrio/_io.pyx":2811
 *         # Create the CRS
 *         if crs is not None:
 *             try:             # <<<<<<<<<<<<<<
 *                 ogr_crs = create_crs(crs)
 *                 # force geographic CRS to use lon, lat order and ignore axis order
*/
        }
        __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
        __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L55_try_end;
        __pyx_L50_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyogrio/_io.pyx":2818
 *                 OSRSetAxisMappingStrategy(ogr_crs, OAMS_TRADITIONAL_GIS_ORDER)
 * 
 *             except Exception as exc:             # <<<<<<<<<<<<<<
 *                 if dataset_options != NULL:
 *                     CSLDestroy(dataset_options)
*/
        __pyx_t_23 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
        if (__pyx_t_23) {
          __Pyx_AddTraceback("pyogrio._io.create_ogr_dataset_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_1, &__pyx_t_5) < 0) __PYX_ERR(0, 2818, __pyx_L52_except_error)
          __Pyx_XGOTREF(__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_1);
          __pyx_v_exc = __pyx_t_1;
          /*try:*/ {

            /* "pyogrio/_io.pyx":2819
 * 
 *             except Exception as exc:
 *                 if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *                     CSLDestroy(dataset_options)
 *                     dataset_options = NULL
*/
            __pyx_t_4 = (__pyx_v_dataset_options != NULL);
            if (__pyx_t_4) {

              /* "pyogrio/_io.pyx":2820
 *             except Exception as exc:
 *                 if dataset_options != NULL:
 *                     CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *                     dataset_options = NULL
 * 
*/
              CSLDestroy(__pyx_v_dataset_options);

              /* "pyogrio/_io.pyx":2821
 *                 if dataset_options != NULL:
 *                     CSLDestroy(dataset_options)
 *                     dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *                 GDALClose(ogr_dataset)
*/
              __pyx_v_dataset_options = NULL;

              /* "pyogrio/_io.pyx":2819
 * 
 *             except Exception as exc:
 *                 if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *                     CSLDestroy(dataset_options)
 *                     dataset_options = NULL
*/
            }

            /* "pyogrio/_io.pyx":2823
 *                     dataset_options = NULL
 * 
 *                 GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *                 ogr_dataset = NULL
 * 
*/
            (void)(GDALClose(__pyx_v_ogr_dataset));

            /* "pyogrio/_io.pyx":2824
 * 
 *                 GDALClose(ogr_dataset)
 *                 ogr_dataset = NULL             # <<<<<<<<<<<<<<
 * 
 *                 raise exc
*/
            __pyx_v_ogr_dataset = NULL;

            /* "pyogrio/_io.pyx":2826
 *                 ogr_dataset = NULL
 * 
 *                 raise exc             # <<<<<<<<<<<<<<
 * 
 *         # Setup other layer creation options
*/
            __Pyx_Raise(__pyx_v_exc, 0, 0, 0);
            __PYX_ERR(0, 2826, __pyx_L61_error)
          }

          /* "pyogrio/_io.pyx":2818
 *                 OSRSetAxisMappingStrategy(ogr_crs, OAMS_TRADITIONAL_GIS_ORDER)
 * 
 *             except Exception as exc:             # <<<<<<<<<<<<<<
 *                 if dataset_options != NULL:
 *                     CSLDestroy(dataset_options)
*/
          /*finally:*/ {
            __pyx_L61_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_31 = 0; __pyx_t_30 = 0; __pyx_t_29 = 0; __pyx_t_28 = 0; __pyx_t_27 = 0; __pyx_t_26 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
               __Pyx_ExceptionSwap(&__pyx_t_28, &__pyx_t_27, &__pyx_t_26);
              if ( unlikely(__Pyx_GetException(&__pyx_t_31, &__pyx_t_30, &__pyx_t_29) < 0)) __Pyx_ErrFetch(&__pyx_t_31, &__pyx_t_30, &__pyx_t_29);
              __Pyx_XGOTREF(__pyx_t_31);
              __Pyx_XGOTREF(__pyx_t_30);
              __Pyx_XGOTREF(__pyx_t_29);
              __Pyx_XGOTREF(__pyx_t_28);
              __Pyx_XGOTREF(__pyx_t_27);
              __Pyx_XGOTREF(__pyx_t_26);
              __pyx_t_23 = __pyx_lineno; __pyx_t_22 = __pyx_clineno; __pyx_t_33 = __pyx_filename;
              {
                __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_28);
              __Pyx_XGIVEREF(__pyx_t_27);
              __Pyx_XGIVEREF(__pyx_t_26);
              __Pyx_ExceptionReset(__pyx_t_28, __pyx_t_27, __pyx_t_26);
              __Pyx_XGIVEREF(__pyx_t_31);
              __Pyx_XGIVEREF(__pyx_t_30);
              __Pyx_XGIVEREF(__pyx_t_29);
              __Pyx_ErrRestore(__pyx_t_31, __pyx_t_30, __pyx_t_29);
              __pyx_t_31 = 0; __pyx_t_30 = 0; __pyx_t_29 = 0; __pyx_t_28 = 0; __pyx_t_27 = 0; __pyx_t_26 = 0;
              __pyx_lineno = __pyx_t_23; __pyx_clineno = __pyx_t_22; __pyx_filename = __pyx_t_33;
              goto __pyx_L52_except_error;
            }
          }
        }
        goto __pyx_L52_except_error;

        /* "pyogrio/_io.pyx":2811
 *         # Create the CRS
 *         if crs is not None:
 *             try:             # <<<<<<<<<<<<<<
 *                 ogr_crs = create_crs(crs)
 *                 # force geographic CRS to use lon, lat order and ignore axis order
*/
        __pyx_L52_except_error:;
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_20, __pyx_t_19, __pyx_t_18);
        goto __pyx_L1_error;
        __pyx_L55_try_end:;
      }

      /* "pyogrio/_io.pyx":2810
 *     if create_layer:
 *         # Create the CRS
 *         if crs is not None:             # <<<<<<<<<<<<<<
 *             try:
 *                 ogr_crs = create_crs(crs)
*/
    }

    /* "pyogrio/_io.pyx":2829
 * 
 *         # Setup other layer creation options
 *         for k, v in layer_kwargs.items():             # <<<<<<<<<<<<<<
 *             k = k.encode("UTF-8")
 *             v = v.encode("UTF-8")
*/
    __pyx_t_11 = 0;
    if (unlikely(__pyx_v_layer_kwargs == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 2829, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_dict_iterator(__pyx_v_layer_kwargs, 0, __pyx_mstate_global->__pyx_n_u_items, (&__pyx_t_13), (&__pyx_t_22)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_5);
    __pyx_t_5 = __pyx_t_1;
    __pyx_t_1 = 0;
    while (1) {
      __pyx_t_23 = __Pyx_dict_iter_next(__pyx_t_5, __pyx_t_13, &__pyx_t_11, &__pyx_t_1, &__pyx_t_8, NULL, __pyx_t_22);
      if (unlikely(__pyx_t_23 == 0)) break;
      if (unlikely(__pyx_t_23 == -1)) __PYX_ERR(0, 2829, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2830
 *         # Setup other layer creation options
 *         for k, v in layer_kwargs.items():
 *             k = k.encode("UTF-8")             # <<<<<<<<<<<<<<
 *             v = v.encode("UTF-8")
 *             layer_options = CSLAddNameValue(
*/
      __pyx_t_1 = __pyx_v_k;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_9 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_UTF_8};
        __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2830, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_DECREF_SET(__pyx_v_k, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2831
 *         for k, v in layer_kwargs.items():
 *             k = k.encode("UTF-8")
 *             v = v.encode("UTF-8")             # <<<<<<<<<<<<<<
 *             layer_options = CSLAddNameValue(
 *                 layer_options, <const char *>k, <const char *>v
*/
      __pyx_t_1 = __pyx_v_v;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_9 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_UTF_8};
        __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2831, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2833
 *             v = v.encode("UTF-8")
 *             layer_options = CSLAddNameValue(
 *                 layer_options, <const char *>k, <const char *>v             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_34 = __Pyx_PyObject_AsString(__pyx_v_k); if (unlikely((!__pyx_t_34) && PyErr_Occurred())) __PYX_ERR(0, 2833, __pyx_L1_error)
      __pyx_t_35 = __Pyx_PyObject_AsString(__pyx_v_v); if (unlikely((!__pyx_t_35) && PyErr_Occurred())) __PYX_ERR(0, 2833, __pyx_L1_error)

      /* "pyogrio/_io.pyx":2832
 *             k = k.encode("UTF-8")
 *             v = v.encode("UTF-8")
 *             layer_options = CSLAddNameValue(             # <<<<<<<<<<<<<<
 *                 layer_options, <const char *>k, <const char *>v
 *             )
*/
      __pyx_v_layer_options = CSLAddNameValue(__pyx_v_layer_options, ((char const *)__pyx_t_34), ((char const *)__pyx_t_35));
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pyogrio/_io.pyx":2836
 *             )
 * 
 *         if driver == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *             # ENCODING option must be set for shapefiles to properly write *.cpg
 *             # file containing the encoding; this is not a supported option for
*/
    __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2836, __pyx_L1_error)
    if (__pyx_t_4) {

      /* "pyogrio/_io.pyx":2841
 *             # other drivers.  This is done after setting general options above
 *             # to override ENCODING if passed by the user as a layer option.
 *             if encoding and "ENCODING" in layer_kwargs:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     'cannot provide both encoding parameter and "ENCODING" layer '
*/
      if (__pyx_v_encoding == Py_None) __pyx_t_17 = 0;
      else
      {
        Py_ssize_t __pyx_temp = __Pyx_PyUnicode_IS_TRUE(__pyx_v_encoding);
        if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 2841, __pyx_L1_error)
        __pyx_t_17 = (__pyx_temp != 0);
      }

      if (__pyx_t_17) {
      } else {
        __pyx_t_4 = __pyx_t_17;
        goto __pyx_L72_bool_binop_done;
      }
      __pyx_t_17 = (__Pyx_PySequence_ContainsTF(__pyx_mstate_global->__pyx_n_u_ENCODING, __pyx_v_layer_kwargs, Py_EQ)); if (unlikely((__pyx_t_17 < 0))) __PYX_ERR(0, 2841, __pyx_L1_error)
      __pyx_t_4 = __pyx_t_17;
      __pyx_L72_bool_binop_done:;
      if (unlikely(__pyx_t_4)) {

        /* "pyogrio/_io.pyx":2842
 *             # to override ENCODING if passed by the user as a layer option.
 *             if encoding and "ENCODING" in layer_kwargs:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     'cannot provide both encoding parameter and "ENCODING" layer '
 *                     "creation option; use the encoding parameter"
*/
        __pyx_t_8 = NULL;
        __pyx_t_9 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_mstate_global->__pyx_kp_u_cannot_provide_both_encoding_par_2};
          __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2842, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(0, 2842, __pyx_L1_error)

        /* "pyogrio/_io.pyx":2841
 *             # other drivers.  This is done after setting general options above
 *             # to override ENCODING if passed by the user as a layer option.
 *             if encoding and "ENCODING" in layer_kwargs:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     'cannot provide both encoding parameter and "ENCODING" layer '
*/
      }

      /* "pyogrio/_io.pyx":2848
 * 
 *             # always write to UTF-8 if encoding is not set
 *             encoding = encoding or "UTF-8"             # <<<<<<<<<<<<<<
 *             encoding_b = encoding.upper().encode("UTF-8")
 *             encoding_c = encoding_b
*/
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 2848, __pyx_L1_error)
      if (!__pyx_t_4) {
      } else {
        __Pyx_INCREF(__pyx_v_encoding);
        __pyx_t_5 = __pyx_v_encoding;
        goto __pyx_L74_bool_binop_done;
      }
      __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
      __pyx_t_5 = __pyx_mstate_global->__pyx_kp_u_UTF_8;
      __pyx_L74_bool_binop_done:;
      __Pyx_DECREF_SET(__pyx_v_encoding, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "pyogrio/_io.pyx":2849
 *             # always write to UTF-8 if encoding is not set
 *             encoding = encoding or "UTF-8"
 *             encoding_b = encoding.upper().encode("UTF-8")             # <<<<<<<<<<<<<<
 *             encoding_c = encoding_b
 *             layer_options = CSLSetNameValue(layer_options, "ENCODING", encoding_c)
*/
      __pyx_t_5 = __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyUnicode_Type__upper, __pyx_v_encoding); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2849, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = PyUnicode_AsUTF8String(((PyObject*)__pyx_t_5)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2849, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_encoding_b = ((PyObject*)__pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2850
 *             encoding = encoding or "UTF-8"
 *             encoding_b = encoding.upper().encode("UTF-8")
 *             encoding_c = encoding_b             # <<<<<<<<<<<<<<
 *             layer_options = CSLSetNameValue(layer_options, "ENCODING", encoding_c)
 * 
*/
      __pyx_t_36 = __Pyx_PyBytes_AsString(__pyx_v_encoding_b); if (unlikely((!__pyx_t_36) && PyErr_Occurred())) __PYX_ERR(0, 2850, __pyx_L1_error)
      __pyx_v_encoding_c = __pyx_t_36;

      /* "pyogrio/_io.pyx":2851
 *             encoding_b = encoding.upper().encode("UTF-8")
 *             encoding_c = encoding_b
 *             layer_options = CSLSetNameValue(layer_options, "ENCODING", encoding_c)             # <<<<<<<<<<<<<<
 * 
 *         # Get geometry type
*/
      __pyx_v_layer_options = CSLSetNameValue(__pyx_v_layer_options, __pyx_k_ENCODING, __pyx_v_encoding_c);

      /* "pyogrio/_io.pyx":2836
 *             )
 * 
 *         if driver == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *             # ENCODING option must be set for shapefiles to properly write *.cpg
 *             # file containing the encoding; this is not a supported option for
*/
    }

    /* "pyogrio/_io.pyx":2856
 *         # TODO: this is brittle for 3D / ZM / M types
 *         # TODO: fail on M / ZM types
 *         geometry_code = get_geometry_type_code(geometry_type)             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
    __pyx_t_37 = __pyx_f_7pyogrio_9_geometry_get_geometry_type_code(__pyx_v_geometry_type); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2856, __pyx_L1_error)
    __pyx_v_geometry_code = __pyx_t_37;

    /* "pyogrio/_io.pyx":2808
 * 
 *     # Create the layer
 *     if create_layer:             # <<<<<<<<<<<<<<
 *         # Create the CRS
 *         if crs is not None:
*/
  }

  /* "pyogrio/_io.pyx":2858
 *         geometry_code = get_geometry_type_code(geometry_type)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         if create_layer:
 *             layer_b = layer.encode("UTF-8")
*/
  /*try:*/ {
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      /*try:*/ {

        /* "pyogrio/_io.pyx":2859
 * 
 *     try:
 *         if create_layer:             # <<<<<<<<<<<<<<
 *             layer_b = layer.encode("UTF-8")
 *             layer_c = layer_b
*/
        if (__pyx_v_create_layer) {

          /* "pyogrio/_io.pyx":2860
 *     try:
 *         if create_layer:
 *             layer_b = layer.encode("UTF-8")             # <<<<<<<<<<<<<<
 *             layer_c = layer_b
 * 
*/
          if (unlikely(__pyx_v_layer == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
            __PYX_ERR(0, 2860, __pyx_L79_error)
          }
          __pyx_t_8 = PyUnicode_AsUTF8String(__pyx_v_layer); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2860, __pyx_L79_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_layer_b = ((PyObject*)__pyx_t_8);
          __pyx_t_8 = 0;

          /* "pyogrio/_io.pyx":2861
 *         if create_layer:
 *             layer_b = layer.encode("UTF-8")
 *             layer_c = layer_b             # <<<<<<<<<<<<<<
 * 
 *             ogr_layer = check_pointer(
*/
          __pyx_t_38 = __Pyx_PyBytes_AsString(__pyx_v_layer_b); if (unlikely((!__pyx_t_38) && PyErr_Occurred())) __PYX_ERR(0, 2861, __pyx_L79_error)
          __pyx_v_layer_c = __pyx_t_38;

          /* "pyogrio/_io.pyx":2863
 *             layer_c = layer_b
 * 
 *             ogr_layer = check_pointer(             # <<<<<<<<<<<<<<
 *                 GDALDatasetCreateLayer(
 *                     ogr_dataset, layer_c, ogr_crs, geometry_code, layer_options
*/
          __pyx_t_21 = __pyx_f_7pyogrio_4_err_check_pointer(GDALDatasetCreateLayer(__pyx_v_ogr_dataset, __pyx_v_layer_c, __pyx_v_ogr_crs, __pyx_v_geometry_code, __pyx_v_layer_options)); if (unlikely(__pyx_t_21 == ((void *)NULL))) __PYX_ERR(0, 2863, __pyx_L79_error)
          __pyx_v_ogr_layer = __pyx_t_21;

          /* "pyogrio/_io.pyx":2859
 * 
 *     try:
 *         if create_layer:             # <<<<<<<<<<<<<<
 *             layer_b = layer.encode("UTF-8")
 *             layer_c = layer_b
*/
          goto __pyx_L85;
        }

        /* "pyogrio/_io.pyx":2870
 * 
 *         else:
 *             ogr_layer = check_pointer(get_ogr_layer(ogr_dataset, layer))             # <<<<<<<<<<<<<<
 * 
 *         # Set dataset and layer metadata
*/
        /*else*/ {
          __pyx_t_39 = __pyx_f_7pyogrio_3_io_get_ogr_layer(__pyx_v_ogr_dataset, __pyx_v_layer); if (unlikely(__pyx_t_39 == ((void *)NULL))) __PYX_ERR(0, 2870, __pyx_L79_error)
          __pyx_t_21 = __pyx_f_7pyogrio_4_err_check_pointer(__pyx_t_39); if (unlikely(__pyx_t_21 == ((void *)NULL))) __PYX_ERR(0, 2870, __pyx_L79_error)
          __pyx_v_ogr_layer = __pyx_t_21;
        }
        __pyx_L85:;

        /* "pyogrio/_io.pyx":2873
 * 
 *         # Set dataset and layer metadata
 *         set_metadata(ogr_dataset, dataset_metadata)             # <<<<<<<<<<<<<<
 *         set_metadata(ogr_layer, layer_metadata)
 * 
*/
        __pyx_t_8 = __pyx_f_7pyogrio_3_io_set_metadata(__pyx_v_ogr_dataset, __pyx_v_dataset_metadata); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2873, __pyx_L79_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyogrio/_io.pyx":2874
 *         # Set dataset and layer metadata
 *         set_metadata(ogr_dataset, dataset_metadata)
 *         set_metadata(ogr_layer, layer_metadata)             # <<<<<<<<<<<<<<
 * 
 *     except Exception as exc:
*/
        __pyx_t_8 = __pyx_f_7pyogrio_3_io_set_metadata(__pyx_v_ogr_layer, __pyx_v_layer_metadata); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2874, __pyx_L79_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "pyogrio/_io.pyx":2858
 *         geometry_code = get_geometry_type_code(geometry_type)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         if create_layer:
 *             layer_b = layer.encode("UTF-8")
*/
      }
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
      goto __pyx_L84_try_end;
      __pyx_L79_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyogrio/_io.pyx":2876
 *         set_metadata(ogr_layer, layer_metadata)
 * 
 *     except Exception as exc:             # <<<<<<<<<<<<<<
 *         GDALClose(ogr_dataset)
 *         ogr_dataset = NULL
*/
      __pyx_t_22 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
      if (__pyx_t_22) {
        __Pyx_AddTraceback("pyogrio._io.create_ogr_dataset_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_5, &__pyx_t_1) < 0) __PYX_ERR(0, 2876, __pyx_L81_except_error)
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_5);
        __pyx_v_exc = __pyx_t_5;
        /*try:*/ {

          /* "pyogrio/_io.pyx":2877
 * 
 *     except Exception as exc:
 *         GDALClose(ogr_dataset)             # <<<<<<<<<<<<<<
 *         ogr_dataset = NULL
 *         raise DataLayerError(str(exc))
*/
          (void)(GDALClose(__pyx_v_ogr_dataset));

          /* "pyogrio/_io.pyx":2878
 *     except Exception as exc:
 *         GDALClose(ogr_dataset)
 *         ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *         raise DataLayerError(str(exc))
 * 
*/
          __pyx_v_ogr_dataset = NULL;

          /* "pyogrio/_io.pyx":2879
 *         GDALClose(ogr_dataset)
 *         ogr_dataset = NULL
 *         raise DataLayerError(str(exc))             # <<<<<<<<<<<<<<
 * 
 *     finally:
*/
          __pyx_t_14 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2879, __pyx_L91_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_15 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 2879, __pyx_L91_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_9 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_7);
            assert(__pyx_t_14);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
            __pyx_t_9 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_15};
            __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2879, __pyx_L91_error)
            __Pyx_GOTREF(__pyx_t_6);
          }
          __Pyx_Raise(__pyx_t_6, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __PYX_ERR(0, 2879, __pyx_L91_error)
        }

        /* "pyogrio/_io.pyx":2876
 *         set_metadata(ogr_layer, layer_metadata)
 * 
 *     except Exception as exc:             # <<<<<<<<<<<<<<
 *         GDALClose(ogr_dataset)
 *         ogr_dataset = NULL
*/
        /*finally:*/ {
          __pyx_L91_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_26 = 0; __pyx_t_27 = 0; __pyx_t_28 = 0; __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_29, &__pyx_t_30, &__pyx_t_31);
            if ( unlikely(__Pyx_GetException(&__pyx_t_26, &__pyx_t_27, &__pyx_t_28) < 0)) __Pyx_ErrFetch(&__pyx_t_26, &__pyx_t_27, &__pyx_t_28);
            __Pyx_XGOTREF(__pyx_t_26);
            __Pyx_XGOTREF(__pyx_t_27);
            __Pyx_XGOTREF(__pyx_t_28);
            __Pyx_XGOTREF(__pyx_t_29);
            __Pyx_XGOTREF(__pyx_t_30);
            __Pyx_XGOTREF(__pyx_t_31);
            __pyx_t_22 = __pyx_lineno; __pyx_t_23 = __pyx_clineno; __pyx_t_40 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_29);
            __Pyx_XGIVEREF(__pyx_t_30);
            __Pyx_XGIVEREF(__pyx_t_31);
            __Pyx_ExceptionReset(__pyx_t_29, __pyx_t_30, __pyx_t_31);
            __Pyx_XGIVEREF(__pyx_t_26);
            __Pyx_XGIVEREF(__pyx_t_27);
            __Pyx_XGIVEREF(__pyx_t_28);
            __Pyx_ErrRestore(__pyx_t_26, __pyx_t_27, __pyx_t_28);
            __pyx_t_26 = 0; __pyx_t_27 = 0; __pyx_t_28 = 0; __pyx_t_29 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0;
            __pyx_lineno = __pyx_t_22; __pyx_clineno = __pyx_t_23; __pyx_filename = __pyx_t_40;
            goto __pyx_L81_except_error;
          }
        }
      }
      goto __pyx_L81_except_error;

      /* "pyogrio/_io.pyx":2858
 *         geometry_code = get_geometry_type_code(geometry_type)
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         if create_layer:
 *             layer_b = layer.encode("UTF-8")
*/
      __pyx_L81_except_error:;
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      goto __pyx_L77_error;
      __pyx_L84_try_end:;
    }
  }

  /* "pyogrio/_io.pyx":2882
 * 
 *     finally:
 *         if ogr_crs != NULL:             # <<<<<<<<<<<<<<
 *             OSRRelease(ogr_crs)
 *             ogr_crs = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_4 = (__pyx_v_ogr_crs != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":2883
 *     finally:
 *         if ogr_crs != NULL:
 *             OSRRelease(ogr_crs)             # <<<<<<<<<<<<<<
 *             ogr_crs = NULL
 * 
*/
        OSRRelease(__pyx_v_ogr_crs);

        /* "pyogrio/_io.pyx":2884
 *         if ogr_crs != NULL:
 *             OSRRelease(ogr_crs)
 *             ogr_crs = NULL             # <<<<<<<<<<<<<<
 * 
 *         if dataset_options != NULL:
*/
        __pyx_v_ogr_crs = NULL;

        /* "pyogrio/_io.pyx":2882
 * 
 *     finally:
 *         if ogr_crs != NULL:             # <<<<<<<<<<<<<<
 *             OSRRelease(ogr_crs)
 *             ogr_crs = NULL
*/
      }

      /* "pyogrio/_io.pyx":2886
 *             ogr_crs = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
      __pyx_t_4 = (__pyx_v_dataset_options != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":2887
 * 
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
        CSLDestroy(__pyx_v_dataset_options);

        /* "pyogrio/_io.pyx":2888
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if layer_options != NULL:
*/
        __pyx_v_dataset_options = NULL;

        /* "pyogrio/_io.pyx":2886
 *             ogr_crs = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
      }

      /* "pyogrio/_io.pyx":2890
 *             dataset_options = NULL
 * 
 *         if layer_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(layer_options)
 *             layer_options = NULL
*/
      __pyx_t_4 = (__pyx_v_layer_options != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":2891
 * 
 *         if layer_options != NULL:
 *             CSLDestroy(layer_options)             # <<<<<<<<<<<<<<
 *             layer_options = NULL
 * 
*/
        CSLDestroy(__pyx_v_layer_options);

        /* "pyogrio/_io.pyx":2892
 *         if layer_options != NULL:
 *             CSLDestroy(layer_options)
 *             layer_options = NULL             # <<<<<<<<<<<<<<
 * 
 *     ogr_dataset_out[0] = ogr_dataset
*/
        __pyx_v_layer_options = NULL;

        /* "pyogrio/_io.pyx":2890
 *             dataset_options = NULL
 * 
 *         if layer_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(layer_options)
 *             layer_options = NULL
*/
      }
      goto __pyx_L78;
    }
    __pyx_L77_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_31 = 0; __pyx_t_30 = 0; __pyx_t_29 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_31, &__pyx_t_30, &__pyx_t_29);
      if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_19, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_19, &__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_31);
      __Pyx_XGOTREF(__pyx_t_30);
      __Pyx_XGOTREF(__pyx_t_29);
      __pyx_t_23 = __pyx_lineno; __pyx_t_22 = __pyx_clineno; __pyx_t_41 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":2882
 * 
 *     finally:
 *         if ogr_crs != NULL:             # <<<<<<<<<<<<<<
 *             OSRRelease(ogr_crs)
 *             ogr_crs = NULL
*/
        __pyx_t_4 = (__pyx_v_ogr_crs != NULL);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":2883
 *     finally:
 *         if ogr_crs != NULL:
 *             OSRRelease(ogr_crs)             # <<<<<<<<<<<<<<
 *             ogr_crs = NULL
 * 
*/
          OSRRelease(__pyx_v_ogr_crs);

          /* "pyogrio/_io.pyx":2884
 *         if ogr_crs != NULL:
 *             OSRRelease(ogr_crs)
 *             ogr_crs = NULL             # <<<<<<<<<<<<<<
 * 
 *         if dataset_options != NULL:
*/
          __pyx_v_ogr_crs = NULL;

          /* "pyogrio/_io.pyx":2882
 * 
 *     finally:
 *         if ogr_crs != NULL:             # <<<<<<<<<<<<<<
 *             OSRRelease(ogr_crs)
 *             ogr_crs = NULL
*/
        }

        /* "pyogrio/_io.pyx":2886
 *             ogr_crs = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        __pyx_t_4 = (__pyx_v_dataset_options != NULL);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":2887
 * 
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)             # <<<<<<<<<<<<<<
 *             dataset_options = NULL
 * 
*/
          CSLDestroy(__pyx_v_dataset_options);

          /* "pyogrio/_io.pyx":2888
 *         if dataset_options != NULL:
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if layer_options != NULL:
*/
          __pyx_v_dataset_options = NULL;

          /* "pyogrio/_io.pyx":2886
 *             ogr_crs = NULL
 * 
 *         if dataset_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(dataset_options)
 *             dataset_options = NULL
*/
        }

        /* "pyogrio/_io.pyx":2890
 *             dataset_options = NULL
 * 
 *         if layer_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(layer_options)
 *             layer_options = NULL
*/
        __pyx_t_4 = (__pyx_v_layer_options != NULL);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":2891
 * 
 *         if layer_options != NULL:
 *             CSLDestroy(layer_options)             # <<<<<<<<<<<<<<
 *             layer_options = NULL
 * 
*/
          CSLDestroy(__pyx_v_layer_options);

          /* "pyogrio/_io.pyx":2892
 *         if layer_options != NULL:
 *             CSLDestroy(layer_options)
 *             layer_options = NULL             # <<<<<<<<<<<<<<
 * 
 *     ogr_dataset_out[0] = ogr_dataset
*/
          __pyx_v_layer_options = NULL;

          /* "pyogrio/_io.pyx":2890
 *             dataset_options = NULL
 * 
 *         if layer_options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(layer_options)
 *             layer_options = NULL
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_31);
      __Pyx_XGIVEREF(__pyx_t_30);
      __Pyx_XGIVEREF(__pyx_t_29);
      __Pyx_ExceptionReset(__pyx_t_31, __pyx_t_30, __pyx_t_29);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_ErrRestore(__pyx_t_20, __pyx_t_19, __pyx_t_18);
      __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_31 = 0; __pyx_t_30 = 0; __pyx_t_29 = 0;
      __pyx_lineno = __pyx_t_23; __pyx_clineno = __pyx_t_22; __pyx_filename = __pyx_t_41;
      goto __pyx_L1_error;
    }
    __pyx_L78:;
  }

  /* "pyogrio/_io.pyx":2894
 *             layer_options = NULL
 * 
 *     ogr_dataset_out[0] = ogr_dataset             # <<<<<<<<<<<<<<
 *     ogr_layer_out[0] = ogr_layer
 * 
*/
  (__pyx_v_ogr_dataset_out[0]) = __pyx_v_ogr_dataset;

  /* "pyogrio/_io.pyx":2895
 * 
 *     ogr_dataset_out[0] = ogr_dataset
 *     ogr_layer_out[0] = ogr_layer             # <<<<<<<<<<<<<<
 * 
 *     return create_layer
*/
  (__pyx_v_ogr_layer_out[0]) = __pyx_v_ogr_layer;

  /* "pyogrio/_io.pyx":2897
 *     ogr_layer_out[0] = ogr_layer
 * 
 *     return create_layer             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_create_layer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":2676
 * 
 * 
 * cdef create_ogr_dataset_layer(             # <<<<<<<<<<<<<<
 *     str path,
 *     bint use_tmp_vsimem,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("pyogrio._io.create_ogr_dataset_layer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path_b);
  __Pyx_XDECREF(__pyx_v_driver_b);
  __Pyx_XDECREF(__pyx_v_path_exists);
  __Pyx_XDECREF(__pyx_v_multi_ext);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_encoding_b);
  __Pyx_XDECREF(__pyx_v_layer_b);
  __Pyx_XDECREF(__pyx_v_layer);
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":2901
 * 
 * # TODO: set geometry and field data as memory views?
 * def ogr_write(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_12ogr_write(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_11ogr_write, "Write the data to a file.");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_12ogr_write = {"ogr_write", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_12ogr_write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_11ogr_write};
static PyObject *__pyx_pw_7pyogrio_3_io_12ogr_write(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path_or_fp = 0;
  PyObject *__pyx_v_layer = 0;
  PyObject *__pyx_v_driver = 0;
  PyObject *__pyx_v_geometry = 0;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_field_data = 0;
  PyObject *__pyx_v_field_mask = 0;
  PyObject *__pyx_v_crs = 0;
  PyObject *__pyx_v_geometry_type = 0;
  PyObject *__pyx_v_encoding = 0;
  PyObject *__pyx_v_dataset_kwargs = 0;
  PyObject *__pyx_v_layer_kwargs = 0;
  int __pyx_v_promote_to_multi;
  int __pyx_v_nan_as_null;
  int __pyx_v_append;
  PyObject *__pyx_v_dataset_metadata = 0;
  PyObject *__pyx_v_layer_metadata = 0;
  PyObject *__pyx_v_gdal_tz_offsets = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[18] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_write (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path_or_fp,&__pyx_mstate_global->__pyx_n_u_layer,&__pyx_mstate_global->__pyx_n_u_driver,&__pyx_mstate_global->__pyx_n_u_geometry,&__pyx_mstate_global->__pyx_n_u_fields,&__pyx_mstate_global->__pyx_n_u_field_data,&__pyx_mstate_global->__pyx_n_u_field_mask,&__pyx_mstate_global->__pyx_n_u_crs,&__pyx_mstate_global->__pyx_n_u_geometry_type,&__pyx_mstate_global->__pyx_n_u_encoding,&__pyx_mstate_global->__pyx_n_u_dataset_kwargs,&__pyx_mstate_global->__pyx_n_u_layer_kwargs,&__pyx_mstate_global->__pyx_n_u_promote_to_multi,&__pyx_mstate_global->__pyx_n_u_nan_as_null,&__pyx_mstate_global->__pyx_n_u_append,&__pyx_mstate_global->__pyx_n_u_dataset_metadata,&__pyx_mstate_global->__pyx_n_u_layer_metadata,&__pyx_mstate_global->__pyx_n_u_gdal_tz_offsets,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2901, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case 18:
        values[17] = __Pyx_ArgRef_FASTCALL(__pyx_args, 17);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[17])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 17:
        values[16] = __Pyx_ArgRef_FASTCALL(__pyx_args, 16);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[16])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 16:
        values[15] = __Pyx_ArgRef_FASTCALL(__pyx_args, 15);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[15])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 15:
        values[14] = __Pyx_ArgRef_FASTCALL(__pyx_args, 14);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[14])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 14:
        values[13] = __Pyx_ArgRef_FASTCALL(__pyx_args, 13);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[13])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "ogr_write", 0) < (0)) __PYX_ERR(0, 2901, __pyx_L3_error)

      /* "pyogrio/_io.pyx":2917
 *     bint nan_as_null=True,
 *     bint append=False,
 *     dataset_metadata=None,             # <<<<<<<<<<<<<<
 *     layer_metadata=None,
 *     gdal_tz_offsets=None
*/
      if (!values[15]) values[15] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2918
 *     bint append=False,
 *     dataset_metadata=None,
 *     layer_metadata=None,             # <<<<<<<<<<<<<<
 *     gdal_tz_offsets=None
 * ):
*/
      if (!values[16]) values[16] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2919
 *     dataset_metadata=None,
 *     layer_metadata=None,
 *     gdal_tz_offsets=None             # <<<<<<<<<<<<<<
 * ):
 *     """Write the data to a file."""
*/
      if (!values[17]) values[17] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 12; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("ogr_write", 0, 12, 18, i); __PYX_ERR(0, 2901, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case 18:
        values[17] = __Pyx_ArgRef_FASTCALL(__pyx_args, 17);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[17])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 17:
        values[16] = __Pyx_ArgRef_FASTCALL(__pyx_args, 16);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[16])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 16:
        values[15] = __Pyx_ArgRef_FASTCALL(__pyx_args, 15);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[15])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 15:
        values[14] = __Pyx_ArgRef_FASTCALL(__pyx_args, 14);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[14])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 14:
        values[13] = __Pyx_ArgRef_FASTCALL(__pyx_args, 13);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[13])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 2901, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2901, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2901, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "pyogrio/_io.pyx":2917
 *     bint nan_as_null=True,
 *     bint append=False,
 *     dataset_metadata=None,             # <<<<<<<<<<<<<<
 *     layer_metadata=None,
 *     gdal_tz_offsets=None
*/
      if (!values[15]) values[15] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2918
 *     bint append=False,
 *     dataset_metadata=None,
 *     layer_metadata=None,             # <<<<<<<<<<<<<<
 *     gdal_tz_offsets=None
 * ):
*/
      if (!values[16]) values[16] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":2919
 *     dataset_metadata=None,
 *     layer_metadata=None,
 *     gdal_tz_offsets=None             # <<<<<<<<<<<<<<
 * ):
 *     """Write the data to a file."""
*/
      if (!values[17]) values[17] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_path_or_fp = values[0];
    __pyx_v_layer = ((PyObject*)values[1]);
    __pyx_v_driver = ((PyObject*)values[2]);
    __pyx_v_geometry = values[3];
    __pyx_v_fields = ((PyObject*)values[4]);
    __pyx_v_field_data = ((PyObject*)values[5]);
    __pyx_v_field_mask = ((PyObject*)values[6]);
    __pyx_v_crs = ((PyObject*)values[7]);
    __pyx_v_geometry_type = ((PyObject*)values[8]);
    __pyx_v_encoding = ((PyObject*)values[9]);
    __pyx_v_dataset_kwargs = values[10];
    __pyx_v_layer_kwargs = values[11];
    if (values[12]) {
      __pyx_v_promote_to_multi = __Pyx_PyObject_IsTrue(values[12]); if (unlikely((__pyx_v_promote_to_multi == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2914, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":2914
 *     object dataset_kwargs,
 *     object layer_kwargs,
 *     bint promote_to_multi=False,             # <<<<<<<<<<<<<<
 *     bint nan_as_null=True,
 *     bint append=False,
*/
      __pyx_v_promote_to_multi = ((int)((int)0));
    }
    if (values[13]) {
      __pyx_v_nan_as_null = __Pyx_PyObject_IsTrue(values[13]); if (unlikely((__pyx_v_nan_as_null == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2915, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":2915
 *     object layer_kwargs,
 *     bint promote_to_multi=False,
 *     bint nan_as_null=True,             # <<<<<<<<<<<<<<
 *     bint append=False,
 *     dataset_metadata=None,
*/
      __pyx_v_nan_as_null = ((int)((int)1));
    }
    if (values[14]) {
      __pyx_v_append = __Pyx_PyObject_IsTrue(values[14]); if (unlikely((__pyx_v_append == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2916, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":2916
 *     bint promote_to_multi=False,
 *     bint nan_as_null=True,
 *     bint append=False,             # <<<<<<<<<<<<<<
 *     dataset_metadata=None,
 *     layer_metadata=None,
*/
      __pyx_v_append = ((int)((int)0));
    }
    __pyx_v_dataset_metadata = values[15];
    __pyx_v_layer_metadata = values[16];
    __pyx_v_gdal_tz_offsets = values[17];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ogr_write", 0, 12, 18, __pyx_nargs); __PYX_ERR(0, 2901, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.ogr_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_layer), (&PyUnicode_Type), 1, "layer", 1))) __PYX_ERR(0, 2903, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_driver), (&PyUnicode_Type), 1, "driver", 1))) __PYX_ERR(0, 2904, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fields), (&PyList_Type), 1, "fields", 1))) __PYX_ERR(0, 2906, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_field_data), (&PyList_Type), 1, "field_data", 1))) __PYX_ERR(0, 2907, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_field_mask), (&PyList_Type), 1, "field_mask", 1))) __PYX_ERR(0, 2908, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_crs), (&PyUnicode_Type), 1, "crs", 1))) __PYX_ERR(0, 2909, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_geometry_type), (&PyUnicode_Type), 1, "geometry_type", 1))) __PYX_ERR(0, 2910, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_encoding), (&PyUnicode_Type), 1, "encoding", 1))) __PYX_ERR(0, 2911, __pyx_L1_error)
  __pyx_r = __pyx_pf_7pyogrio_3_io_11ogr_write(__pyx_self, __pyx_v_path_or_fp, __pyx_v_layer, __pyx_v_driver, __pyx_v_geometry, __pyx_v_fields, __pyx_v_field_data, __pyx_v_field_mask, __pyx_v_crs, __pyx_v_geometry_type, __pyx_v_encoding, __pyx_v_dataset_kwargs, __pyx_v_layer_kwargs, __pyx_v_promote_to_multi, __pyx_v_nan_as_null, __pyx_v_append, __pyx_v_dataset_metadata, __pyx_v_layer_metadata, __pyx_v_gdal_tz_offsets);

  /* "pyogrio/_io.pyx":2901
 * 
 * # TODO: set geometry and field data as memory views?
 * def ogr_write(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_11ogr_write(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_fp, PyObject *__pyx_v_layer, PyObject *__pyx_v_driver, PyObject *__pyx_v_geometry, PyObject *__pyx_v_fields, PyObject *__pyx_v_field_data, PyObject *__pyx_v_field_mask, PyObject *__pyx_v_crs, PyObject *__pyx_v_geometry_type, PyObject *__pyx_v_encoding, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer_kwargs, int __pyx_v_promote_to_multi, int __pyx_v_nan_as_null, int __pyx_v_append, PyObject *__pyx_v_dataset_metadata, PyObject *__pyx_v_layer_metadata, PyObject *__pyx_v_gdal_tz_offsets) {
  OGRDataSourceH __pyx_v_ogr_dataset;
  OGRLayerH __pyx_v_ogr_layer;
  OGRFeatureH __pyx_v_ogr_feature;
  OGRGeometryH __pyx_v_ogr_geometry;
  OGRFeatureDefnH __pyx_v_ogr_featuredef;
  OGRFieldDefnH __pyx_v_ogr_fielddef;
  unsigned char const *__pyx_v_wkb_buffer;
  unsigned int __pyx_v_wkbtype;
  int __pyx_v_supports_transactions;
  int __pyx_v_err;
  int __pyx_v_i;
  int __pyx_v_num_records;
  int __pyx_v_num_field_data;
  int __pyx_v_num_fields;
  PyObject *__pyx_v_field_indexes = 0;
  PyObject *__pyx_v_field_ogr_types = 0;
  int __pyx_v_field_index;
  int __pyx_v_field_type;
  int __pyx_v_use_tmp_vsimem;
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_layer_created = NULL;
  PyObject *__pyx_v_field_types = NULL;
  PyObject *__pyx_v_field_ogr_type = NULL;
  PyObject *__pyx_v_field_subtype = NULL;
  PyObject *__pyx_v_width = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__precision = NULL;
  PyObject *__pyx_v_name_b = NULL;
  int __pyx_v_index;
  PyObject *__pyx_v_wkb = NULL;
  int __pyx_v_field_idx;
  PyObject *__pyx_v_field_value = NULL;
  PyObject *__pyx_v_mask = NULL;
  PyObject *__pyx_v_value_b = NULL;
  PyObject *__pyx_v_datetime_value = NULL;
  PyObject *__pyx_v_tz_array = NULL;
  PyObject *__pyx_v_gdal_tz = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_9genexpr13__pyx_v_field = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *(*__pyx_t_16)(PyObject *);
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  char const *__pyx_t_20;
  OGRFieldType __pyx_t_21;
  void *__pyx_t_22;
  OGRFieldSubType __pyx_t_23;
  int __pyx_t_24;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27[3];
  int __pyx_t_28;
  char const *__pyx_t_29;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  char const *__pyx_t_33;
  int __pyx_t_34;
  unsigned char const *__pyx_t_35;
  PyObject *__pyx_t_36[4];
  int __pyx_t_37;
  int __pyx_t_38;
  long double __pyx_t_39;
  char *__pyx_t_40;
  PyObject *__pyx_t_41[5];
  int64_t __pyx_t_42;
  double __pyx_t_43;
  int __pyx_t_44;
  int __pyx_t_45;
  int __pyx_t_46;
  int __pyx_t_47;
  float __pyx_t_48;
  int __pyx_t_49;
  PyObject *__pyx_t_50 = NULL;
  char const *__pyx_t_51;
  PyObject *__pyx_t_52 = NULL;
  PyObject *__pyx_t_53 = NULL;
  PyObject *__pyx_t_54 = NULL;
  char const *__pyx_t_55;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_write", 0);
  __Pyx_INCREF(__pyx_v_field_mask);
  __Pyx_INCREF(__pyx_v_geometry_type);
  __Pyx_INCREF(__pyx_v_encoding);
  __Pyx_INCREF(__pyx_v_gdal_tz_offsets);

  /* "pyogrio/_io.pyx":2922
 * ):
 *     """Write the data to a file."""
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef OGRFeatureH ogr_feature = NULL
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":2923
 *     """Write the data to a file."""
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRFeatureH ogr_feature = NULL
 *     cdef OGRGeometryH ogr_geometry = NULL
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":2924
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef OGRFeatureH ogr_feature = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRGeometryH ogr_geometry = NULL
 *     cdef OGRFeatureDefnH ogr_featuredef = NULL
*/
  __pyx_v_ogr_feature = NULL;

  /* "pyogrio/_io.pyx":2925
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef OGRFeatureH ogr_feature = NULL
 *     cdef OGRGeometryH ogr_geometry = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRFeatureDefnH ogr_featuredef = NULL
 *     cdef OGRFieldDefnH ogr_fielddef = NULL
*/
  __pyx_v_ogr_geometry = NULL;

  /* "pyogrio/_io.pyx":2926
 *     cdef OGRFeatureH ogr_feature = NULL
 *     cdef OGRGeometryH ogr_geometry = NULL
 *     cdef OGRFeatureDefnH ogr_featuredef = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRFieldDefnH ogr_fielddef = NULL
 *     cdef const unsigned char *wkb_buffer = NULL
*/
  __pyx_v_ogr_featuredef = NULL;

  /* "pyogrio/_io.pyx":2927
 *     cdef OGRGeometryH ogr_geometry = NULL
 *     cdef OGRFeatureDefnH ogr_featuredef = NULL
 *     cdef OGRFieldDefnH ogr_fielddef = NULL             # <<<<<<<<<<<<<<
 *     cdef const unsigned char *wkb_buffer = NULL
 *     cdef unsigned int wkbtype = 0
*/
  __pyx_v_ogr_fielddef = NULL;

  /* "pyogrio/_io.pyx":2928
 *     cdef OGRFeatureDefnH ogr_featuredef = NULL
 *     cdef OGRFieldDefnH ogr_fielddef = NULL
 *     cdef const unsigned char *wkb_buffer = NULL             # <<<<<<<<<<<<<<
 *     cdef unsigned int wkbtype = 0
 *     cdef int supports_transactions = 0
*/
  __pyx_v_wkb_buffer = NULL;

  /* "pyogrio/_io.pyx":2929
 *     cdef OGRFieldDefnH ogr_fielddef = NULL
 *     cdef const unsigned char *wkb_buffer = NULL
 *     cdef unsigned int wkbtype = 0             # <<<<<<<<<<<<<<
 *     cdef int supports_transactions = 0
 *     cdef int err = 0
*/
  __pyx_v_wkbtype = 0;

  /* "pyogrio/_io.pyx":2930
 *     cdef const unsigned char *wkb_buffer = NULL
 *     cdef unsigned int wkbtype = 0
 *     cdef int supports_transactions = 0             # <<<<<<<<<<<<<<
 *     cdef int err = 0
 *     cdef int i = 0
*/
  __pyx_v_supports_transactions = 0;

  /* "pyogrio/_io.pyx":2931
 *     cdef unsigned int wkbtype = 0
 *     cdef int supports_transactions = 0
 *     cdef int err = 0             # <<<<<<<<<<<<<<
 *     cdef int i = 0
 *     cdef int num_records = -1
*/
  __pyx_v_err = 0;

  /* "pyogrio/_io.pyx":2932
 *     cdef int supports_transactions = 0
 *     cdef int err = 0
 *     cdef int i = 0             # <<<<<<<<<<<<<<
 *     cdef int num_records = -1
 *     cdef int num_field_data = len(field_data) if field_data is not None else 0
*/
  __pyx_v_i = 0;

  /* "pyogrio/_io.pyx":2933
 *     cdef int err = 0
 *     cdef int i = 0
 *     cdef int num_records = -1             # <<<<<<<<<<<<<<
 *     cdef int num_field_data = len(field_data) if field_data is not None else 0
 *     cdef int num_fields = len(fields) if fields is not None else 0
*/
  __pyx_v_num_records = -1;

  /* "pyogrio/_io.pyx":2934
 *     cdef int i = 0
 *     cdef int num_records = -1
 *     cdef int num_field_data = len(field_data) if field_data is not None else 0             # <<<<<<<<<<<<<<
 *     cdef int num_fields = len(fields) if fields is not None else 0
 *     cdef list field_indexes = []
*/
  __pyx_t_2 = (__pyx_v_field_data != ((PyObject*)Py_None));
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_field_data == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2934, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyList_GET_SIZE(__pyx_v_field_data); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2934, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_3;
  } else {
    __pyx_t_1 = 0;
  }
  __pyx_v_num_field_data = __pyx_t_1;

  /* "pyogrio/_io.pyx":2935
 *     cdef int num_records = -1
 *     cdef int num_field_data = len(field_data) if field_data is not None else 0
 *     cdef int num_fields = len(fields) if fields is not None else 0             # <<<<<<<<<<<<<<
 *     cdef list field_indexes = []
 *     cdef list field_ogr_types = []
*/
  __pyx_t_2 = (__pyx_v_fields != ((PyObject*)Py_None));
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2935, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyList_GET_SIZE(__pyx_v_fields); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2935, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_3;
  } else {
    __pyx_t_1 = 0;
  }
  __pyx_v_num_fields = __pyx_t_1;

  /* "pyogrio/_io.pyx":2936
 *     cdef int num_field_data = len(field_data) if field_data is not None else 0
 *     cdef int num_fields = len(fields) if fields is not None else 0
 *     cdef list field_indexes = []             # <<<<<<<<<<<<<<
 *     cdef list field_ogr_types = []
 *     cdef int field_index
*/
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_field_indexes = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":2937
 *     cdef int num_fields = len(fields) if fields is not None else 0
 *     cdef list field_indexes = []
 *     cdef list field_ogr_types = []             # <<<<<<<<<<<<<<
 *     cdef int field_index
 *     cdef int field_type
*/
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_field_ogr_types = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":2940
 *     cdef int field_index
 *     cdef int field_type
 *     cdef bint use_tmp_vsimem = False             # <<<<<<<<<<<<<<
 * 
 *     if num_fields != num_field_data:
*/
  __pyx_v_use_tmp_vsimem = 0;

  /* "pyogrio/_io.pyx":2942
 *     cdef bint use_tmp_vsimem = False
 * 
 *     if num_fields != num_field_data:             # <<<<<<<<<<<<<<
 *         raise ValueError("field_data array needs to be same length as fields array")
 * 
*/
  __pyx_t_2 = (__pyx_v_num_fields != __pyx_v_num_field_data);
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":2943
 * 
 *     if num_fields != num_field_data:
 *         raise ValueError("field_data array needs to be same length as fields array")             # <<<<<<<<<<<<<<
 * 
 *     if num_fields == 0 and geometry is None:
*/
    __pyx_t_5 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_field_data_array_needs_to_be_sam};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2943, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 2943, __pyx_L1_error)

    /* "pyogrio/_io.pyx":2942
 *     cdef bint use_tmp_vsimem = False
 * 
 *     if num_fields != num_field_data:             # <<<<<<<<<<<<<<
 *         raise ValueError("field_data array needs to be same length as fields array")
 * 
*/
  }

  /* "pyogrio/_io.pyx":2945
 *         raise ValueError("field_data array needs to be same length as fields array")
 * 
 *     if num_fields == 0 and geometry is None:             # <<<<<<<<<<<<<<
 *         raise ValueError("You must provide at least a geometry column or a field")
 * 
*/
  __pyx_t_7 = (__pyx_v_num_fields == 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_2 = __pyx_t_7;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_7 = (__pyx_v_geometry == Py_None);
  __pyx_t_2 = __pyx_t_7;
  __pyx_L5_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "pyogrio/_io.pyx":2946
 * 
 *     if num_fields == 0 and geometry is None:
 *         raise ValueError("You must provide at least a geometry column or a field")             # <<<<<<<<<<<<<<
 * 
 *     if num_fields > 0:
*/
    __pyx_t_5 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_You_must_provide_at_least_a_geom};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2946, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 2946, __pyx_L1_error)

    /* "pyogrio/_io.pyx":2945
 *         raise ValueError("field_data array needs to be same length as fields array")
 * 
 *     if num_fields == 0 and geometry is None:             # <<<<<<<<<<<<<<
 *         raise ValueError("You must provide at least a geometry column or a field")
 * 
*/
  }

  /* "pyogrio/_io.pyx":2948
 *         raise ValueError("You must provide at least a geometry column or a field")
 * 
 *     if num_fields > 0:             # <<<<<<<<<<<<<<
 *         num_records = len(field_data[0])
 *         for i in range(1, len(field_data)):
*/
  __pyx_t_2 = (__pyx_v_num_fields > 0);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":2949
 * 
 *     if num_fields > 0:
 *         num_records = len(field_data[0])             # <<<<<<<<<<<<<<
 *         for i in range(1, len(field_data)):
 *             if len(field_data[i]) != num_records:
*/
    if (unlikely(__pyx_v_field_data == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 2949, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_field_data, 0, long, 1, __Pyx_PyLong_From_long, 1, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2949, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2949, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_num_records = __pyx_t_1;

    /* "pyogrio/_io.pyx":2950
 *     if num_fields > 0:
 *         num_records = len(field_data[0])
 *         for i in range(1, len(field_data)):             # <<<<<<<<<<<<<<
 *             if len(field_data[i]) != num_records:
 *                 raise ValueError("field_data arrays must be same length")
*/
    if (unlikely(__pyx_v_field_data == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2950, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyList_GET_SIZE(__pyx_v_field_data); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2950, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_1;
    for (__pyx_t_8 = 1; __pyx_t_8 < __pyx_t_3; __pyx_t_8+=1) {
      __pyx_v_i = __pyx_t_8;

      /* "pyogrio/_io.pyx":2951
 *         num_records = len(field_data[0])
 *         for i in range(1, len(field_data)):
 *             if len(field_data[i]) != num_records:             # <<<<<<<<<<<<<<
 *                 raise ValueError("field_data arrays must be same length")
 * 
*/
      if (unlikely(__pyx_v_field_data == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 2951, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_field_data, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2951, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_9 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2951, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_2 = (__pyx_t_9 != __pyx_v_num_records);
      if (unlikely(__pyx_t_2)) {

        /* "pyogrio/_io.pyx":2952
 *         for i in range(1, len(field_data)):
 *             if len(field_data[i]) != num_records:
 *                 raise ValueError("field_data arrays must be same length")             # <<<<<<<<<<<<<<
 * 
 *     if geometry is None:
*/
        __pyx_t_5 = NULL;
        __pyx_t_6 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_field_data_arrays_must_be_same_l};
          __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2952, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(0, 2952, __pyx_L1_error)

        /* "pyogrio/_io.pyx":2951
 *         num_records = len(field_data[0])
 *         for i in range(1, len(field_data)):
 *             if len(field_data[i]) != num_records:             # <<<<<<<<<<<<<<
 *                 raise ValueError("field_data arrays must be same length")
 * 
*/
      }
    }

    /* "pyogrio/_io.pyx":2948
 *         raise ValueError("You must provide at least a geometry column or a field")
 * 
 *     if num_fields > 0:             # <<<<<<<<<<<<<<
 *         num_records = len(field_data[0])
 *         for i in range(1, len(field_data)):
*/
  }

  /* "pyogrio/_io.pyx":2954
 *                 raise ValueError("field_data arrays must be same length")
 * 
 *     if geometry is None:             # <<<<<<<<<<<<<<
 *         # If no geometry data, we ignore the geometry_type and don't create a geometry
 *         # column
*/
  __pyx_t_2 = (__pyx_v_geometry == Py_None);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":2957
 *         # If no geometry data, we ignore the geometry_type and don't create a geometry
 *         # column
 *         geometry_type = None             # <<<<<<<<<<<<<<
 *     else:
 *         if num_fields > 0:
*/
    __Pyx_INCREF(Py_None);
    __Pyx_DECREF_SET(__pyx_v_geometry_type, ((PyObject*)Py_None));

    /* "pyogrio/_io.pyx":2954
 *                 raise ValueError("field_data arrays must be same length")
 * 
 *     if geometry is None:             # <<<<<<<<<<<<<<
 *         # If no geometry data, we ignore the geometry_type and don't create a geometry
 *         # column
*/
    goto __pyx_L11;
  }

  /* "pyogrio/_io.pyx":2959
 *         geometry_type = None
 *     else:
 *         if num_fields > 0:             # <<<<<<<<<<<<<<
 *             if len(geometry) != num_records:
 *                 raise ValueError(
*/
  /*else*/ {
    __pyx_t_2 = (__pyx_v_num_fields > 0);
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":2960
 *     else:
 *         if num_fields > 0:
 *             if len(geometry) != num_records:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "field_data arrays must be same length as geometry array"
*/
      __pyx_t_1 = PyObject_Length(__pyx_v_geometry); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2960, __pyx_L1_error)
      __pyx_t_2 = (__pyx_t_1 != __pyx_v_num_records);
      if (unlikely(__pyx_t_2)) {

        /* "pyogrio/_io.pyx":2961
 *         if num_fields > 0:
 *             if len(geometry) != num_records:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "field_data arrays must be same length as geometry array"
 *                 )
*/
        __pyx_t_5 = NULL;
        __pyx_t_6 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_field_data_arrays_must_be_same_l_2};
          __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2961, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(0, 2961, __pyx_L1_error)

        /* "pyogrio/_io.pyx":2960
 *     else:
 *         if num_fields > 0:
 *             if len(geometry) != num_records:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "field_data arrays must be same length as geometry array"
*/
      }

      /* "pyogrio/_io.pyx":2959
 *         geometry_type = None
 *     else:
 *         if num_fields > 0:             # <<<<<<<<<<<<<<
 *             if len(geometry) != num_records:
 *                 raise ValueError(
*/
      goto __pyx_L12;
    }

    /* "pyogrio/_io.pyx":2965
 *                 )
 *         else:
 *             num_records = len(geometry)             # <<<<<<<<<<<<<<
 * 
 *     if field_mask is not None:
*/
    /*else*/ {
      __pyx_t_1 = PyObject_Length(__pyx_v_geometry); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2965, __pyx_L1_error)
      __pyx_v_num_records = __pyx_t_1;
    }
    __pyx_L12:;
  }
  __pyx_L11:;

  /* "pyogrio/_io.pyx":2967
 *             num_records = len(geometry)
 * 
 *     if field_mask is not None:             # <<<<<<<<<<<<<<
 *         if len(field_data) != len(field_mask):
 *             raise ValueError("field_data and field_mask must be same length")
*/
  __pyx_t_2 = (__pyx_v_field_mask != ((PyObject*)Py_None));
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":2968
 * 
 *     if field_mask is not None:
 *         if len(field_data) != len(field_mask):             # <<<<<<<<<<<<<<
 *             raise ValueError("field_data and field_mask must be same length")
 *         for i in range(0, len(field_mask)):
*/
    if (unlikely(__pyx_v_field_data == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2968, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyList_GET_SIZE(__pyx_v_field_data); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2968, __pyx_L1_error)
    if (unlikely(__pyx_v_field_mask == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2968, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyList_GET_SIZE(__pyx_v_field_mask); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2968, __pyx_L1_error)
    __pyx_t_2 = (__pyx_t_1 != __pyx_t_3);
    if (unlikely(__pyx_t_2)) {

      /* "pyogrio/_io.pyx":2969
 *     if field_mask is not None:
 *         if len(field_data) != len(field_mask):
 *             raise ValueError("field_data and field_mask must be same length")             # <<<<<<<<<<<<<<
 *         for i in range(0, len(field_mask)):
 *             if field_mask[i] is not None and len(field_mask[i]) != num_records:
*/
      __pyx_t_5 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_field_data_and_field_mask_must_b};
        __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2969, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(0, 2969, __pyx_L1_error)

      /* "pyogrio/_io.pyx":2968
 * 
 *     if field_mask is not None:
 *         if len(field_data) != len(field_mask):             # <<<<<<<<<<<<<<
 *             raise ValueError("field_data and field_mask must be same length")
 *         for i in range(0, len(field_mask)):
*/
    }

    /* "pyogrio/_io.pyx":2970
 *         if len(field_data) != len(field_mask):
 *             raise ValueError("field_data and field_mask must be same length")
 *         for i in range(0, len(field_mask)):             # <<<<<<<<<<<<<<
 *             if field_mask[i] is not None and len(field_mask[i]) != num_records:
 *                 raise ValueError(
*/
    if (unlikely(__pyx_v_field_mask == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 2970, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyList_GET_SIZE(__pyx_v_field_mask); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2970, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_3;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_1; __pyx_t_8+=1) {
      __pyx_v_i = __pyx_t_8;

      /* "pyogrio/_io.pyx":2971
 *             raise ValueError("field_data and field_mask must be same length")
 *         for i in range(0, len(field_mask)):
 *             if field_mask[i] is not None and len(field_mask[i]) != num_records:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "field_mask arrays must be same length as geometry array"
*/
      if (unlikely(__pyx_v_field_mask == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 2971, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_field_mask, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2971, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = (__pyx_t_4 != Py_None);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_7) {
      } else {
        __pyx_t_2 = __pyx_t_7;
        goto __pyx_L19_bool_binop_done;
      }
      if (unlikely(__pyx_v_field_mask == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 2971, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_field_mask, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2971, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_9 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 2971, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_7 = (__pyx_t_9 != __pyx_v_num_records);
      __pyx_t_2 = __pyx_t_7;
      __pyx_L19_bool_binop_done:;
      if (unlikely(__pyx_t_2)) {

        /* "pyogrio/_io.pyx":2972
 *         for i in range(0, len(field_mask)):
 *             if field_mask[i] is not None and len(field_mask[i]) != num_records:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "field_mask arrays must be same length as geometry array"
 *                 )
*/
        __pyx_t_5 = NULL;
        __pyx_t_6 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_field_mask_arrays_must_be_same_l};
          __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2972, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __PYX_ERR(0, 2972, __pyx_L1_error)

        /* "pyogrio/_io.pyx":2971
 *             raise ValueError("field_data and field_mask must be same length")
 *         for i in range(0, len(field_mask)):
 *             if field_mask[i] is not None and len(field_mask[i]) != num_records:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "field_mask arrays must be same length as geometry array"
*/
      }
    }

    /* "pyogrio/_io.pyx":2967
 *             num_records = len(geometry)
 * 
 *     if field_mask is not None:             # <<<<<<<<<<<<<<
 *         if len(field_data) != len(field_mask):
 *             raise ValueError("field_data and field_mask must be same length")
*/
    goto __pyx_L14;
  }

  /* "pyogrio/_io.pyx":2976
 *                 )
 *     else:
 *         field_mask = [None] * num_fields             # <<<<<<<<<<<<<<
 * 
 *     if gdal_tz_offsets is None:
*/
  /*else*/ {
    __pyx_t_4 = PyList_New(1 * ((__pyx_v_num_fields<0) ? 0:__pyx_v_num_fields)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2976, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    { Py_ssize_t __pyx_temp;
      for (__pyx_temp=0; __pyx_temp < __pyx_v_num_fields; __pyx_temp++) {
        __Pyx_INCREF(Py_None);
        __Pyx_GIVEREF(Py_None);
        if (__Pyx_PyList_SET_ITEM(__pyx_t_4, __pyx_temp, Py_None) != (0)) __PYX_ERR(0, 2976, __pyx_L1_error);
      }
    }
    __Pyx_DECREF_SET(__pyx_v_field_mask, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;
  }
  __pyx_L14:;

  /* "pyogrio/_io.pyx":2978
 *         field_mask = [None] * num_fields
 * 
 *     if gdal_tz_offsets is None:             # <<<<<<<<<<<<<<
 *         gdal_tz_offsets = {}
 * 
*/
  __pyx_t_2 = (__pyx_v_gdal_tz_offsets == Py_None);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":2979
 * 
 *     if gdal_tz_offsets is None:
 *         gdal_tz_offsets = {}             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
    __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2979, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_gdal_tz_offsets, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyogrio/_io.pyx":2978
 *         field_mask = [None] * num_fields
 * 
 *     if gdal_tz_offsets is None:             # <<<<<<<<<<<<<<
 *         gdal_tz_offsets = {}
 * 
*/
  }

  /* "pyogrio/_io.pyx":2981
 *         gdal_tz_offsets = {}
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         # Setup in-memory handler if needed
 *         path, use_tmp_vsimem = get_ogr_vsimem_write_path(path_or_fp, driver)
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":2983
 *     try:
 *         # Setup in-memory handler if needed
 *         path, use_tmp_vsimem = get_ogr_vsimem_write_path(path_or_fp, driver)             # <<<<<<<<<<<<<<
 * 
 *         # Setup dataset and layer
*/
    __pyx_t_4 = __pyx_f_7pyogrio_4_vsi_get_ogr_vsimem_write_path(__pyx_v_path_or_fp, __pyx_v_driver); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2983, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (likely(__pyx_t_4 != Py_None)) {
      PyObject* sequence = __pyx_t_4;
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 2983, __pyx_L23_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_10);
      #else
      __pyx_t_5 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2983, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2983, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_10);
      #endif
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 2983, __pyx_L23_error)
    }
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 2983, __pyx_L23_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_v_path = __pyx_t_5;
    __pyx_t_5 = 0;
    __pyx_v_use_tmp_vsimem = __pyx_t_2;

    /* "pyogrio/_io.pyx":2987
 *         # Setup dataset and layer
 *         layer_created = create_ogr_dataset_layer(
 *             path, use_tmp_vsimem, layer, driver, crs, geometry_type, encoding,             # <<<<<<<<<<<<<<
 *             dataset_kwargs, layer_kwargs, append,
 *             dataset_metadata, layer_metadata,
*/
    __pyx_t_4 = __pyx_v_path;
    __Pyx_INCREF(__pyx_t_4);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_4))) __PYX_ERR(0, 2987, __pyx_L23_error)

    /* "pyogrio/_io.pyx":2986
 * 
 *         # Setup dataset and layer
 *         layer_created = create_ogr_dataset_layer(             # <<<<<<<<<<<<<<
 *             path, use_tmp_vsimem, layer, driver, crs, geometry_type, encoding,
 *             dataset_kwargs, layer_kwargs, append,
*/
    __pyx_t_10 = __pyx_f_7pyogrio_3_io_create_ogr_dataset_layer(((PyObject*)__pyx_t_4), __pyx_v_use_tmp_vsimem, __pyx_v_layer, __pyx_v_driver, __pyx_v_crs, __pyx_v_geometry_type, __pyx_v_encoding, __pyx_v_dataset_kwargs, __pyx_v_layer_kwargs, __pyx_v_append, __pyx_v_dataset_metadata, __pyx_v_layer_metadata, (&__pyx_v_ogr_dataset), (&__pyx_v_ogr_layer)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2986, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_layer_created = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "pyogrio/_io.pyx":2993
 *         )
 * 
 *         if driver == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *             # force encoding for remaining operations to be in UTF-8 (even if
 *             # user provides an encoding) because GDAL will automatically
*/
    __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 2993, __pyx_L23_error)
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":2998
 *             # convert those to the target encoding because ENCODING is set as a
 *             # layer creation option
 *             encoding = "UTF-8"             # <<<<<<<<<<<<<<
 * 
 *         else:
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_UTF_8);
      __Pyx_DECREF_SET(__pyx_v_encoding, __pyx_mstate_global->__pyx_kp_u_UTF_8);

      /* "pyogrio/_io.pyx":2993
 *         )
 * 
 *         if driver == "ESRI Shapefile":             # <<<<<<<<<<<<<<
 *             # force encoding for remaining operations to be in UTF-8 (even if
 *             # user provides an encoding) because GDAL will automatically
*/
      goto __pyx_L25;
    }

    /* "pyogrio/_io.pyx":3004
 *             # determine the encoding. It is derived from the user, from the
 *             # dataset capabilities / type, or from the system locale
 *             encoding = encoding or detect_encoding(ogr_dataset, ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         # Create the fields
*/
    /*else*/ {
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_encoding); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 3004, __pyx_L23_error)
      if (!__pyx_t_2) {
      } else {
        __Pyx_INCREF(__pyx_v_encoding);
        __pyx_t_10 = __pyx_v_encoding;
        goto __pyx_L26_bool_binop_done;
      }
      __pyx_t_4 = __pyx_f_7pyogrio_3_io_detect_encoding(__pyx_v_ogr_dataset, __pyx_v_ogr_layer); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3004, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __pyx_t_4;
      __Pyx_INCREF(__pyx_t_5);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_5))) __PYX_ERR(0, 3004, __pyx_L23_error)
      __pyx_t_10 = __pyx_t_5;
      __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_L26_bool_binop_done:;
      __Pyx_DECREF_SET(__pyx_v_encoding, ((PyObject*)__pyx_t_10));
      __pyx_t_10 = 0;
    }
    __pyx_L25:;

    /* "pyogrio/_io.pyx":3007
 * 
 *         # Create the fields
 *         field_types = None             # <<<<<<<<<<<<<<
 *         if num_fields > 0:
 *             field_types = infer_field_types([field.dtype for field in field_data])
*/
    __Pyx_INCREF(Py_None);
    __pyx_v_field_types = Py_None;

    /* "pyogrio/_io.pyx":3008
 *         # Create the fields
 *         field_types = None
 *         if num_fields > 0:             # <<<<<<<<<<<<<<
 *             field_types = infer_field_types([field.dtype for field in field_data])
 * 
*/
    __pyx_t_2 = (__pyx_v_num_fields > 0);
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3009
 *         field_types = None
 *         if num_fields > 0:
 *             field_types = infer_field_types([field.dtype for field in field_data])             # <<<<<<<<<<<<<<
 * 
 *         if layer_created:
*/
      { /* enter inner scope */
        __pyx_t_10 = PyList_New(0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3009, __pyx_L31_error)
        __Pyx_GOTREF(__pyx_t_10);
        if (unlikely(__pyx_v_field_data == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(0, 3009, __pyx_L31_error)
        }
        __pyx_t_4 = __pyx_v_field_data; __Pyx_INCREF(__pyx_t_4);
        __pyx_t_3 = 0;
        for (;;) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 3009, __pyx_L31_error)
            #endif
            if (__pyx_t_3 >= __pyx_temp) break;
          }
          __pyx_t_5 = __Pyx_PyList_GetItemRefFast(__pyx_t_4, __pyx_t_3, __Pyx_ReferenceSharing_OwnStrongReference);
          ++__pyx_t_3;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3009, __pyx_L31_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_XDECREF_SET(__pyx_9genexpr13__pyx_v_field, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_9genexpr13__pyx_v_field, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3009, __pyx_L31_error)
          __Pyx_GOTREF(__pyx_t_5);
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_10, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 3009, __pyx_L31_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_9genexpr13__pyx_v_field); __pyx_9genexpr13__pyx_v_field = 0;
        goto __pyx_L35_exit_scope;
        __pyx_L31_error:;
        __Pyx_XDECREF(__pyx_9genexpr13__pyx_v_field); __pyx_9genexpr13__pyx_v_field = 0;
        goto __pyx_L23_error;
        __pyx_L35_exit_scope:;
      } /* exit inner scope */
      __pyx_t_4 = __pyx_f_7pyogrio_3_io_infer_field_types(((PyObject*)__pyx_t_10)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3009, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF_SET(__pyx_v_field_types, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyogrio/_io.pyx":3008
 *         # Create the fields
 *         field_types = None
 *         if num_fields > 0:             # <<<<<<<<<<<<<<
 *             field_types = infer_field_types([field.dtype for field in field_data])
 * 
*/
    }

    /* "pyogrio/_io.pyx":3011
 *             field_types = infer_field_types([field.dtype for field in field_data])
 * 
 *         if layer_created:             # <<<<<<<<<<<<<<
 *             for i in range(num_fields):
 *                 field_ogr_type, field_subtype, width, _precision = field_types[i]
*/
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_layer_created); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 3011, __pyx_L23_error)
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3012
 * 
 *         if layer_created:
 *             for i in range(num_fields):             # <<<<<<<<<<<<<<
 *                 field_ogr_type, field_subtype, width, _precision = field_types[i]
 * 
*/
      __pyx_t_8 = __pyx_v_num_fields;
      __pyx_t_11 = __pyx_t_8;
      for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
        __pyx_v_i = __pyx_t_12;

        /* "pyogrio/_io.pyx":3013
 *         if layer_created:
 *             for i in range(num_fields):
 *                 field_ogr_type, field_subtype, width, _precision = field_types[i]             # <<<<<<<<<<<<<<
 * 
 *                 name_b = fields[i].encode(encoding)
*/
        __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_field_types, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3013, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
          PyObject* sequence = __pyx_t_4;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 4)) {
            if (size > 4) __Pyx_RaiseTooManyValuesError(4);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 3013, __pyx_L23_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0);
            __Pyx_INCREF(__pyx_t_10);
            __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1);
            __Pyx_INCREF(__pyx_t_5);
            __pyx_t_13 = PyTuple_GET_ITEM(sequence, 2);
            __Pyx_INCREF(__pyx_t_13);
            __pyx_t_14 = PyTuple_GET_ITEM(sequence, 3);
            __Pyx_INCREF(__pyx_t_14);
          } else {
            __pyx_t_10 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3013, __pyx_L23_error)
            __Pyx_XGOTREF(__pyx_t_10);
            __pyx_t_5 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3013, __pyx_L23_error)
            __Pyx_XGOTREF(__pyx_t_5);
            __pyx_t_13 = __Pyx_PyList_GetItemRefFast(sequence, 2, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3013, __pyx_L23_error)
            __Pyx_XGOTREF(__pyx_t_13);
            __pyx_t_14 = __Pyx_PyList_GetItemRefFast(sequence, 3, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3013, __pyx_L23_error)
            __Pyx_XGOTREF(__pyx_t_14);
          }
          #else
          {
            Py_ssize_t i;
            PyObject** temps[4] = {&__pyx_t_10,&__pyx_t_5,&__pyx_t_13,&__pyx_t_14};
            for (i=0; i < 4; i++) {
              PyObject* item = __Pyx_PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 3013, __pyx_L23_error)
              __Pyx_GOTREF(item);
              *(temps[i]) = item;
            }
          }
          #endif
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else {
          Py_ssize_t index = -1;
          PyObject** temps[4] = {&__pyx_t_10,&__pyx_t_5,&__pyx_t_13,&__pyx_t_14};
          __pyx_t_15 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3013, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_16 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_15);
          for (index=0; index < 4; index++) {
            PyObject* item = __pyx_t_16(__pyx_t_15); if (unlikely(!item)) goto __pyx_L39_unpacking_failed;
            __Pyx_GOTREF(item);
            *(temps[index]) = item;
          }
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_16(__pyx_t_15), 4) < (0)) __PYX_ERR(0, 3013, __pyx_L23_error)
          __pyx_t_16 = NULL;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          goto __pyx_L40_unpacking_done;
          __pyx_L39_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_16 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 3013, __pyx_L23_error)
          __pyx_L40_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_field_ogr_type, __pyx_t_10);
        __pyx_t_10 = 0;
        __Pyx_XDECREF_SET(__pyx_v_field_subtype, __pyx_t_5);
        __pyx_t_5 = 0;
        __Pyx_XDECREF_SET(__pyx_v_width, __pyx_t_13);
        __pyx_t_13 = 0;
        __Pyx_XDECREF_SET(__pyx_v__precision, __pyx_t_14);
        __pyx_t_14 = 0;

        /* "pyogrio/_io.pyx":3015
 *                 field_ogr_type, field_subtype, width, _precision = field_types[i]
 * 
 *                 name_b = fields[i].encode(encoding)             # <<<<<<<<<<<<<<
 *                 try:
 *                     ogr_fielddef = check_pointer(OGR_Fld_Create(name_b, field_ogr_type))
*/
        if (unlikely(__pyx_v_fields == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 3015, __pyx_L23_error)
        }
        __pyx_t_13 = __Pyx_GetItemInt_List(__pyx_v_fields, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3015, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_14 = __pyx_t_13;
        __Pyx_INCREF(__pyx_t_14);
        __pyx_t_6 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_encoding};
          __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3015, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_XDECREF_SET(__pyx_v_name_b, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "pyogrio/_io.pyx":3016
 * 
 *                 name_b = fields[i].encode(encoding)
 *                 try:             # <<<<<<<<<<<<<<
 *                     ogr_fielddef = check_pointer(OGR_Fld_Create(name_b, field_ogr_type))
 * 
*/
        /*try:*/ {
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_19);
            /*try:*/ {

              /* "pyogrio/_io.pyx":3017
 *                 name_b = fields[i].encode(encoding)
 *                 try:
 *                     ogr_fielddef = check_pointer(OGR_Fld_Create(name_b, field_ogr_type))             # <<<<<<<<<<<<<<
 * 
 *                     # subtypes, see: https://gdal.org/development/rfc/rfc50_ogr_field_subtype.html  # noqa: E501
*/
              __pyx_t_20 = __Pyx_PyObject_AsString(__pyx_v_name_b); if (unlikely((!__pyx_t_20) && PyErr_Occurred())) __PYX_ERR(0, 3017, __pyx_L46_error)
              __pyx_t_21 = ((OGRFieldType)__Pyx_PyLong_As_OGRFieldType(__pyx_v_field_ogr_type)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 3017, __pyx_L46_error)
              __pyx_t_22 = __pyx_f_7pyogrio_4_err_check_pointer(OGR_Fld_Create(__pyx_t_20, __pyx_t_21)); if (unlikely(__pyx_t_22 == ((void *)NULL))) __PYX_ERR(0, 3017, __pyx_L46_error)
              __pyx_v_ogr_fielddef = __pyx_t_22;

              /* "pyogrio/_io.pyx":3020
 * 
 *                     # subtypes, see: https://gdal.org/development/rfc/rfc50_ogr_field_subtype.html  # noqa: E501
 *                     if field_subtype != OFSTNone:             # <<<<<<<<<<<<<<
 *                         OGR_Fld_SetSubType(ogr_fielddef, field_subtype)
 * 
*/
              __pyx_t_4 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3020, __pyx_L46_error)
              __Pyx_GOTREF(__pyx_t_4);
              __pyx_t_13 = PyObject_RichCompare(__pyx_v_field_subtype, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_13); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3020, __pyx_L46_error)
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 3020, __pyx_L46_error)
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              if (__pyx_t_2) {

                /* "pyogrio/_io.pyx":3021
 *                     # subtypes, see: https://gdal.org/development/rfc/rfc50_ogr_field_subtype.html  # noqa: E501
 *                     if field_subtype != OFSTNone:
 *                         OGR_Fld_SetSubType(ogr_fielddef, field_subtype)             # <<<<<<<<<<<<<<
 * 
 *                     if width:
*/
                __pyx_t_23 = ((OGRFieldSubType)__Pyx_PyLong_As_OGRFieldSubType(__pyx_v_field_subtype)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 3021, __pyx_L46_error)
                OGR_Fld_SetSubType(__pyx_v_ogr_fielddef, __pyx_t_23);

                /* "pyogrio/_io.pyx":3020
 * 
 *                     # subtypes, see: https://gdal.org/development/rfc/rfc50_ogr_field_subtype.html  # noqa: E501
 *                     if field_subtype != OFSTNone:             # <<<<<<<<<<<<<<
 *                         OGR_Fld_SetSubType(ogr_fielddef, field_subtype)
 * 
*/
              }

              /* "pyogrio/_io.pyx":3023
 *                         OGR_Fld_SetSubType(ogr_fielddef, field_subtype)
 * 
 *                     if width:             # <<<<<<<<<<<<<<
 *                         OGR_Fld_SetWidth(ogr_fielddef, width)
 * 
*/
              __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_width); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 3023, __pyx_L46_error)
              if (__pyx_t_2) {

                /* "pyogrio/_io.pyx":3024
 * 
 *                     if width:
 *                         OGR_Fld_SetWidth(ogr_fielddef, width)             # <<<<<<<<<<<<<<
 * 
 *                     # TODO: set precision
*/
                __pyx_t_24 = __Pyx_PyLong_As_int(__pyx_v_width); if (unlikely((__pyx_t_24 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3024, __pyx_L46_error)
                OGR_Fld_SetWidth(__pyx_v_ogr_fielddef, __pyx_t_24);

                /* "pyogrio/_io.pyx":3023
 *                         OGR_Fld_SetSubType(ogr_fielddef, field_subtype)
 * 
 *                     if width:             # <<<<<<<<<<<<<<
 *                         OGR_Fld_SetWidth(ogr_fielddef, width)
 * 
*/
              }

              /* "pyogrio/_io.pyx":3028
 *                     # TODO: set precision
 * 
 *                     check_int(OGR_L_CreateField(ogr_layer, ogr_fielddef, 1))             # <<<<<<<<<<<<<<
 * 
 *                 except Exception:
*/
              __pyx_t_24 = __pyx_f_7pyogrio_4_err_check_int(OGR_L_CreateField(__pyx_v_ogr_layer, __pyx_v_ogr_fielddef, 1)); if (unlikely(__pyx_t_24 == ((int)-1))) __PYX_ERR(0, 3028, __pyx_L46_error)

              /* "pyogrio/_io.pyx":3016
 * 
 *                 name_b = fields[i].encode(encoding)
 *                 try:             # <<<<<<<<<<<<<<
 *                     ogr_fielddef = check_pointer(OGR_Fld_Create(name_b, field_ogr_type))
 * 
*/
            }
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
            goto __pyx_L53_try_end;
            __pyx_L46_error:;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "pyogrio/_io.pyx":3030
 *                     check_int(OGR_L_CreateField(ogr_layer, ogr_fielddef, 1))
 * 
 *                 except Exception:             # <<<<<<<<<<<<<<
 *                     raise FieldError(
 *                         f"Error adding field '{fields[i]}' to layer"
*/
            __pyx_t_24 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
            if (__pyx_t_24) {
              __Pyx_AddTraceback("pyogrio._io.ogr_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_13, &__pyx_t_4, &__pyx_t_14) < 0) __PYX_ERR(0, 3030, __pyx_L48_except_error)
              __Pyx_XGOTREF(__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_4);
              __Pyx_XGOTREF(__pyx_t_14);

              /* "pyogrio/_io.pyx":3031
 * 
 *                 except Exception:
 *                     raise FieldError(             # <<<<<<<<<<<<<<
 *                         f"Error adding field '{fields[i]}' to layer"
 *                     ) from None
*/
              __pyx_t_10 = NULL;
              __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_FieldError); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3031, __pyx_L48_except_error)
              __Pyx_GOTREF(__pyx_t_15);

              /* "pyogrio/_io.pyx":3032
 *                 except Exception:
 *                     raise FieldError(
 *                         f"Error adding field '{fields[i]}' to layer"             # <<<<<<<<<<<<<<
 *                     ) from None
 * 
*/
              if (unlikely(__pyx_v_fields == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                __PYX_ERR(0, 3032, __pyx_L48_except_error)
              }
              __pyx_t_25 = __Pyx_GetItemInt_List(__pyx_v_fields, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3032, __pyx_L48_except_error)
              __Pyx_GOTREF(__pyx_t_25);
              __pyx_t_26 = __Pyx_PyObject_FormatSimple(__pyx_t_25, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 3032, __pyx_L48_except_error)
              __Pyx_GOTREF(__pyx_t_26);
              __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
              __pyx_t_27[0] = __pyx_mstate_global->__pyx_kp_u_Error_adding_field;
              __pyx_t_27[1] = __pyx_t_26;
              __pyx_t_27[2] = __pyx_mstate_global->__pyx_kp_u_to_layer;
              __pyx_t_25 = __Pyx_PyUnicode_Join(__pyx_t_27, 3, 20 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_26) + 10, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_26));
              if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3032, __pyx_L48_except_error)
              __Pyx_GOTREF(__pyx_t_25);
              __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
              __pyx_t_6 = 1;
              #if CYTHON_UNPACK_METHODS
              if (unlikely(PyMethod_Check(__pyx_t_15))) {
                __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_15);
                assert(__pyx_t_10);
                PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
                __Pyx_INCREF(__pyx_t_10);
                __Pyx_INCREF(__pyx__function);
                __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
                __pyx_t_6 = 0;
              }
              #endif
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_25};
                __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3031, __pyx_L48_except_error)
                __Pyx_GOTREF(__pyx_t_5);
              }

              /* "pyogrio/_io.pyx":3033
 *                     raise FieldError(
 *                         f"Error adding field '{fields[i]}' to layer"
 *                     ) from None             # <<<<<<<<<<<<<<
 * 
 *                 finally:
*/
              __Pyx_Raise(__pyx_t_5, 0, 0, Py_None);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __PYX_ERR(0, 3031, __pyx_L48_except_error)
            }
            goto __pyx_L48_except_error;

            /* "pyogrio/_io.pyx":3016
 * 
 *                 name_b = fields[i].encode(encoding)
 *                 try:             # <<<<<<<<<<<<<<
 *                     ogr_fielddef = check_pointer(OGR_Fld_Create(name_b, field_ogr_type))
 * 
*/
            __pyx_L48_except_error:;
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
            goto __pyx_L44_error;
            __pyx_L53_try_end:;
          }
        }

        /* "pyogrio/_io.pyx":3036
 * 
 *                 finally:
 *                     if ogr_fielddef != NULL:             # <<<<<<<<<<<<<<
 *                         OGR_Fld_Destroy(ogr_fielddef)
 *                         ogr_fielddef = NULL
*/
        /*finally:*/ {
          /*normal exit:*/{
            __pyx_t_2 = (__pyx_v_ogr_fielddef != NULL);
            if (__pyx_t_2) {

              /* "pyogrio/_io.pyx":3037
 *                 finally:
 *                     if ogr_fielddef != NULL:
 *                         OGR_Fld_Destroy(ogr_fielddef)             # <<<<<<<<<<<<<<
 *                         ogr_fielddef = NULL
 * 
*/
              OGR_Fld_Destroy(__pyx_v_ogr_fielddef);

              /* "pyogrio/_io.pyx":3038
 *                     if ogr_fielddef != NULL:
 *                         OGR_Fld_Destroy(ogr_fielddef)
 *                         ogr_fielddef = NULL             # <<<<<<<<<<<<<<
 * 
 *         # Get the field indexes for the fields we need to write to, as they might be
*/
              __pyx_v_ogr_fielddef = NULL;

              /* "pyogrio/_io.pyx":3036
 * 
 *                 finally:
 *                     if ogr_fielddef != NULL:             # <<<<<<<<<<<<<<
 *                         OGR_Fld_Destroy(ogr_fielddef)
 *                         ogr_fielddef = NULL
*/
            }
            goto __pyx_L45;
          }
          __pyx_L44_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
            __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32);
            if ( unlikely(__Pyx_GetException(&__pyx_t_19, &__pyx_t_18, &__pyx_t_17) < 0)) __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_18, &__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_30);
            __Pyx_XGOTREF(__pyx_t_31);
            __Pyx_XGOTREF(__pyx_t_32);
            __pyx_t_24 = __pyx_lineno; __pyx_t_28 = __pyx_clineno; __pyx_t_29 = __pyx_filename;
            {
              __pyx_t_2 = (__pyx_v_ogr_fielddef != NULL);
              if (__pyx_t_2) {

                /* "pyogrio/_io.pyx":3037
 *                 finally:
 *                     if ogr_fielddef != NULL:
 *                         OGR_Fld_Destroy(ogr_fielddef)             # <<<<<<<<<<<<<<
 *                         ogr_fielddef = NULL
 * 
*/
                OGR_Fld_Destroy(__pyx_v_ogr_fielddef);

                /* "pyogrio/_io.pyx":3038
 *                     if ogr_fielddef != NULL:
 *                         OGR_Fld_Destroy(ogr_fielddef)
 *                         ogr_fielddef = NULL             # <<<<<<<<<<<<<<
 * 
 *         # Get the field indexes for the fields we need to write to, as they might be
*/
                __pyx_v_ogr_fielddef = NULL;

                /* "pyogrio/_io.pyx":3036
 * 
 *                 finally:
 *                     if ogr_fielddef != NULL:             # <<<<<<<<<<<<<<
 *                         OGR_Fld_Destroy(ogr_fielddef)
 *                         ogr_fielddef = NULL
*/
              }
            }
            __Pyx_XGIVEREF(__pyx_t_30);
            __Pyx_XGIVEREF(__pyx_t_31);
            __Pyx_XGIVEREF(__pyx_t_32);
            __Pyx_ExceptionReset(__pyx_t_30, __pyx_t_31, __pyx_t_32);
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_ErrRestore(__pyx_t_19, __pyx_t_18, __pyx_t_17);
            __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_17 = 0; __pyx_t_30 = 0; __pyx_t_31 = 0; __pyx_t_32 = 0;
            __pyx_lineno = __pyx_t_24; __pyx_clineno = __pyx_t_28; __pyx_filename = __pyx_t_29;
            goto __pyx_L23_error;
          }
          __pyx_L45:;
        }
      }

      /* "pyogrio/_io.pyx":3011
 *             field_types = infer_field_types([field.dtype for field in field_data])
 * 
 *         if layer_created:             # <<<<<<<<<<<<<<
 *             for i in range(num_fields):
 *                 field_ogr_type, field_subtype, width, _precision = field_types[i]
*/
    }

    /* "pyogrio/_io.pyx":3042
 *         # Get the field indexes for the fields we need to write to, as they might be
 *         # different than the order they were created, e.g. when using the KML driver.
 *         ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)             # <<<<<<<<<<<<<<
 * 
 *         if OGR_FD_GetFieldCount(ogr_featuredef) == num_fields:
*/
    __pyx_v_ogr_featuredef = OGR_L_GetLayerDefn(__pyx_v_ogr_layer);

    /* "pyogrio/_io.pyx":3044
 *         ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
 * 
 *         if OGR_FD_GetFieldCount(ogr_featuredef) == num_fields:             # <<<<<<<<<<<<<<
 *             field_indexes = list(range(num_fields))
 *         else:
*/
    __pyx_t_2 = (OGR_FD_GetFieldCount(__pyx_v_ogr_featuredef) == __pyx_v_num_fields);
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3045
 * 
 *         if OGR_FD_GetFieldCount(ogr_featuredef) == num_fields:
 *             field_indexes = list(range(num_fields))             # <<<<<<<<<<<<<<
 *         else:
 *             for i in range(num_fields):
*/
      __pyx_t_4 = NULL;
      __pyx_t_13 = __Pyx_PyLong_From_int(__pyx_v_num_fields); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3045, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_13};
        __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)(&PyRange_Type), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3045, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_14);
      }
      __pyx_t_13 = PySequence_List(__pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3045, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF_SET(__pyx_v_field_indexes, ((PyObject*)__pyx_t_13));
      __pyx_t_13 = 0;

      /* "pyogrio/_io.pyx":3044
 *         ogr_featuredef = OGR_L_GetLayerDefn(ogr_layer)
 * 
 *         if OGR_FD_GetFieldCount(ogr_featuredef) == num_fields:             # <<<<<<<<<<<<<<
 *             field_indexes = list(range(num_fields))
 *         else:
*/
      goto __pyx_L64;
    }

    /* "pyogrio/_io.pyx":3047
 *             field_indexes = list(range(num_fields))
 *         else:
 *             for i in range(num_fields):             # <<<<<<<<<<<<<<
 *                 index = OGR_FD_GetFieldIndex(ogr_featuredef, fields[i].encode(encoding))
 *                 if index < 0:
*/
    /*else*/ {
      __pyx_t_8 = __pyx_v_num_fields;
      __pyx_t_11 = __pyx_t_8;
      for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
        __pyx_v_i = __pyx_t_12;

        /* "pyogrio/_io.pyx":3048
 *         else:
 *             for i in range(num_fields):
 *                 index = OGR_FD_GetFieldIndex(ogr_featuredef, fields[i].encode(encoding))             # <<<<<<<<<<<<<<
 *                 if index < 0:
 *                     raise FieldError(
*/
        if (unlikely(__pyx_v_fields == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 3048, __pyx_L23_error)
        }
        __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_fields, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3048, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_14 = __pyx_t_4;
        __Pyx_INCREF(__pyx_t_14);
        __pyx_t_6 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_encoding};
          __pyx_t_13 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3048, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_13);
        }
        __pyx_t_33 = __Pyx_PyObject_AsString(__pyx_t_13); if (unlikely((!__pyx_t_33) && PyErr_Occurred())) __PYX_ERR(0, 3048, __pyx_L23_error)
        __pyx_v_index = OGR_FD_GetFieldIndex(__pyx_v_ogr_featuredef, __pyx_t_33);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

        /* "pyogrio/_io.pyx":3049
 *             for i in range(num_fields):
 *                 index = OGR_FD_GetFieldIndex(ogr_featuredef, fields[i].encode(encoding))
 *                 if index < 0:             # <<<<<<<<<<<<<<
 *                     raise FieldError(
 *                         f"Could not find field index for field '{fields[i]}'"
*/
        __pyx_t_2 = (__pyx_v_index < 0);
        if (unlikely(__pyx_t_2)) {

          /* "pyogrio/_io.pyx":3050
 *                 index = OGR_FD_GetFieldIndex(ogr_featuredef, fields[i].encode(encoding))
 *                 if index < 0:
 *                     raise FieldError(             # <<<<<<<<<<<<<<
 *                         f"Could not find field index for field '{fields[i]}'"
 *                     )
*/
          __pyx_t_4 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_FieldError); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3050, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_14);

          /* "pyogrio/_io.pyx":3051
 *                 if index < 0:
 *                     raise FieldError(
 *                         f"Could not find field index for field '{fields[i]}'"             # <<<<<<<<<<<<<<
 *                     )
 *                 field_indexes.append(index)
*/
          if (unlikely(__pyx_v_fields == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 3051, __pyx_L23_error)
          }
          __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_fields, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3051, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_t_5, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3051, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_27[0] = __pyx_mstate_global->__pyx_kp_u_Could_not_find_field_index_for_f;
          __pyx_t_27[1] = __pyx_t_15;
          __pyx_t_27[2] = __pyx_mstate_global->__pyx_kp_u__4;
          __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_27, 3, 38 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15));
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3051, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_14))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_14);
            assert(__pyx_t_4);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_5};
            __pyx_t_13 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3050, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_13);
          }
          __Pyx_Raise(__pyx_t_13, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __PYX_ERR(0, 3050, __pyx_L23_error)

          /* "pyogrio/_io.pyx":3049
 *             for i in range(num_fields):
 *                 index = OGR_FD_GetFieldIndex(ogr_featuredef, fields[i].encode(encoding))
 *                 if index < 0:             # <<<<<<<<<<<<<<
 *                     raise FieldError(
 *                         f"Could not find field index for field '{fields[i]}'"
*/
        }

        /* "pyogrio/_io.pyx":3053
 *                         f"Could not find field index for field '{fields[i]}'"
 *                     )
 *                 field_indexes.append(index)             # <<<<<<<<<<<<<<
 * 
 *         if field_types is not None:
*/
        __pyx_t_13 = __Pyx_PyLong_From_int(__pyx_v_index); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3053, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_34 = __Pyx_PyList_Append(__pyx_v_field_indexes, __pyx_t_13); if (unlikely(__pyx_t_34 == ((int)-1))) __PYX_ERR(0, 3053, __pyx_L23_error)
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      }
    }
    __pyx_L64:;

    /* "pyogrio/_io.pyx":3055
 *                 field_indexes.append(index)
 * 
 *         if field_types is not None:             # <<<<<<<<<<<<<<
 *             field_ogr_types = list(field_types[:, 0])
 * 
*/
    __pyx_t_2 = (__pyx_v_field_types != Py_None);
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3056
 * 
 *         if field_types is not None:
 *             field_ogr_types = list(field_types[:, 0])             # <<<<<<<<<<<<<<
 * 
 *         # Create the features
*/
      __pyx_t_13 = __Pyx_PyObject_GetItem(__pyx_v_field_types, __pyx_mstate_global->__pyx_tuple[3]); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3056, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_14 = __Pyx_PySequence_ListKeepNew(__pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3056, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF_SET(__pyx_v_field_ogr_types, ((PyObject*)__pyx_t_14));
      __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":3055
 *                 field_indexes.append(index)
 * 
 *         if field_types is not None:             # <<<<<<<<<<<<<<
 *             field_ogr_types = list(field_types[:, 0])
 * 
*/
    }

    /* "pyogrio/_io.pyx":3059
 * 
 *         # Create the features
 *         supports_transactions = OGR_L_TestCapability(ogr_layer, OLCTransactions)             # <<<<<<<<<<<<<<
 *         if supports_transactions:
 *             start_transaction(ogr_dataset, 0)
*/
    __pyx_v_supports_transactions = OGR_L_TestCapability(__pyx_v_ogr_layer, OLCTransactions);

    /* "pyogrio/_io.pyx":3060
 *         # Create the features
 *         supports_transactions = OGR_L_TestCapability(ogr_layer, OLCTransactions)
 *         if supports_transactions:             # <<<<<<<<<<<<<<
 *             start_transaction(ogr_dataset, 0)
 * 
*/
    __pyx_t_2 = (__pyx_v_supports_transactions != 0);
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3061
 *         supports_transactions = OGR_L_TestCapability(ogr_layer, OLCTransactions)
 *         if supports_transactions:
 *             start_transaction(ogr_dataset, 0)             # <<<<<<<<<<<<<<
 * 
 *         for i in range(num_records):
*/
      __pyx_t_8 = __pyx_f_7pyogrio_3_io_start_transaction(__pyx_v_ogr_dataset, 0); if (unlikely(__pyx_t_8 == ((int)1))) __PYX_ERR(0, 3061, __pyx_L23_error)

      /* "pyogrio/_io.pyx":3060
 *         # Create the features
 *         supports_transactions = OGR_L_TestCapability(ogr_layer, OLCTransactions)
 *         if supports_transactions:             # <<<<<<<<<<<<<<
 *             start_transaction(ogr_dataset, 0)
 * 
*/
    }

    /* "pyogrio/_io.pyx":3063
 *             start_transaction(ogr_dataset, 0)
 * 
 *         for i in range(num_records):             # <<<<<<<<<<<<<<
 *             # create the feature
 *             ogr_feature = OGR_F_Create(ogr_featuredef)
*/
    __pyx_t_8 = __pyx_v_num_records;
    __pyx_t_11 = __pyx_t_8;
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_v_i = __pyx_t_12;

      /* "pyogrio/_io.pyx":3065
 *         for i in range(num_records):
 *             # create the feature
 *             ogr_feature = OGR_F_Create(ogr_featuredef)             # <<<<<<<<<<<<<<
 *             if ogr_feature == NULL:
 *                 raise FeatureError(f"Could not create feature at index {i}") from None
*/
      __pyx_v_ogr_feature = OGR_F_Create(__pyx_v_ogr_featuredef);

      /* "pyogrio/_io.pyx":3066
 *             # create the feature
 *             ogr_feature = OGR_F_Create(ogr_featuredef)
 *             if ogr_feature == NULL:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(f"Could not create feature at index {i}") from None
 * 
*/
      __pyx_t_2 = (__pyx_v_ogr_feature == NULL);
      if (unlikely(__pyx_t_2)) {

        /* "pyogrio/_io.pyx":3067
 *             ogr_feature = OGR_F_Create(ogr_featuredef)
 *             if ogr_feature == NULL:
 *                 raise FeatureError(f"Could not create feature at index {i}") from None             # <<<<<<<<<<<<<<
 * 
 *             # create the geometry based on specific WKB type
*/
        __pyx_t_13 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3067, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3067, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_15 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Could_not_create_feature_at_inde, __pyx_t_4); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3067, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_5);
          assert(__pyx_t_13);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_15};
          __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3067, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_14);
        }
        __Pyx_Raise(__pyx_t_14, 0, 0, Py_None);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __PYX_ERR(0, 3067, __pyx_L23_error)

        /* "pyogrio/_io.pyx":3066
 *             # create the feature
 *             ogr_feature = OGR_F_Create(ogr_featuredef)
 *             if ogr_feature == NULL:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(f"Could not create feature at index {i}") from None
 * 
*/
      }

      /* "pyogrio/_io.pyx":3072
 *             # (there might be mixed types in geometries)
 *             # TODO: geometry must not be null or errors
 *             wkb = None if geometry is None else geometry[i]             # <<<<<<<<<<<<<<
 *             if wkb is not None:
 *                 wkb_buffer = wkb
*/
      __pyx_t_2 = (__pyx_v_geometry == Py_None);
      if (__pyx_t_2) {
        __Pyx_INCREF(Py_None);
        __pyx_t_14 = Py_None;
      } else {
        __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_geometry, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3072, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_14 = __pyx_t_5;
        __pyx_t_5 = 0;
      }
      __Pyx_XDECREF_SET(__pyx_v_wkb, __pyx_t_14);
      __pyx_t_14 = 0;

      /* "pyogrio/_io.pyx":3073
 *             # TODO: geometry must not be null or errors
 *             wkb = None if geometry is None else geometry[i]
 *             if wkb is not None:             # <<<<<<<<<<<<<<
 *                 wkb_buffer = wkb
 *                 if wkb_buffer[0] == 1:
*/
      __pyx_t_2 = (__pyx_v_wkb != Py_None);
      if (__pyx_t_2) {

        /* "pyogrio/_io.pyx":3074
 *             wkb = None if geometry is None else geometry[i]
 *             if wkb is not None:
 *                 wkb_buffer = wkb             # <<<<<<<<<<<<<<
 *                 if wkb_buffer[0] == 1:
 *                     # Little endian WKB type.
*/
        __pyx_t_35 = __Pyx_PyObject_AsUString(__pyx_v_wkb); if (unlikely((!__pyx_t_35) && PyErr_Occurred())) __PYX_ERR(0, 3074, __pyx_L23_error)
        __pyx_v_wkb_buffer = __pyx_t_35;

        /* "pyogrio/_io.pyx":3075
 *             if wkb is not None:
 *                 wkb_buffer = wkb
 *                 if wkb_buffer[0] == 1:             # <<<<<<<<<<<<<<
 *                     # Little endian WKB type.
 *                     wkbtype = (
*/
        __pyx_t_2 = ((__pyx_v_wkb_buffer[0]) == 1);
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":3078
 *                     # Little endian WKB type.
 *                     wkbtype = (
 *                         wkb_buffer[1] + (wkb_buffer[2] << 8) + (wkb_buffer[3] << 16) +             # <<<<<<<<<<<<<<
 *                         (<unsigned int>wkb_buffer[4] << 24)
 *                     )
*/
          __pyx_v_wkbtype = ((((__pyx_v_wkb_buffer[1]) + ((__pyx_v_wkb_buffer[2]) << 8)) + ((__pyx_v_wkb_buffer[3]) << 16)) + (((unsigned int)(__pyx_v_wkb_buffer[4])) << 24));

          /* "pyogrio/_io.pyx":3075
 *             if wkb is not None:
 *                 wkb_buffer = wkb
 *                 if wkb_buffer[0] == 1:             # <<<<<<<<<<<<<<
 *                     # Little endian WKB type.
 *                     wkbtype = (
*/
          goto __pyx_L74;
        }

        /* "pyogrio/_io.pyx":3085
 *                     wkbtype = (
 *                         (<unsigned int>(wkb_buffer[1]) << 24) + (wkb_buffer[2] << 16) +
 *                         (wkb_buffer[3] << 8) + wkb_buffer[4]             # <<<<<<<<<<<<<<
 *                     )
 *                 ogr_geometry = OGR_G_CreateGeometry(<OGRwkbGeometryType>wkbtype)
*/
        /*else*/ {

          /* "pyogrio/_io.pyx":3084
 *                     # Big endian WKB type.
 *                     wkbtype = (
 *                         (<unsigned int>(wkb_buffer[1]) << 24) + (wkb_buffer[2] << 16) +             # <<<<<<<<<<<<<<
 *                         (wkb_buffer[3] << 8) + wkb_buffer[4]
 *                     )
*/
          __pyx_v_wkbtype = ((((((unsigned int)(__pyx_v_wkb_buffer[1])) << 24) + ((__pyx_v_wkb_buffer[2]) << 16)) + ((__pyx_v_wkb_buffer[3]) << 8)) + (__pyx_v_wkb_buffer[4]));
        }
        __pyx_L74:;

        /* "pyogrio/_io.pyx":3087
 *                         (wkb_buffer[3] << 8) + wkb_buffer[4]
 *                     )
 *                 ogr_geometry = OGR_G_CreateGeometry(<OGRwkbGeometryType>wkbtype)             # <<<<<<<<<<<<<<
 *                 if ogr_geometry == NULL:
 *                     raise GeometryError(
*/
        __pyx_v_ogr_geometry = OGR_G_CreateGeometry(((OGRwkbGeometryType)__pyx_v_wkbtype));

        /* "pyogrio/_io.pyx":3088
 *                     )
 *                 ogr_geometry = OGR_G_CreateGeometry(<OGRwkbGeometryType>wkbtype)
 *                 if ogr_geometry == NULL:             # <<<<<<<<<<<<<<
 *                     raise GeometryError(
 *                         f"Could not create geometry at index {i} for WKB type {wkbtype}"
*/
        __pyx_t_2 = (__pyx_v_ogr_geometry == NULL);
        if (unlikely(__pyx_t_2)) {

          /* "pyogrio/_io.pyx":3089
 *                 ogr_geometry = OGR_G_CreateGeometry(<OGRwkbGeometryType>wkbtype)
 *                 if ogr_geometry == NULL:
 *                     raise GeometryError(             # <<<<<<<<<<<<<<
 *                         f"Could not create geometry at index {i} for WKB type {wkbtype}"
 *                     ) from None
*/
          __pyx_t_5 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_GeometryError); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3089, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_15);

          /* "pyogrio/_io.pyx":3090
 *                 if ogr_geometry == NULL:
 *                     raise GeometryError(
 *                         f"Could not create geometry at index {i} for WKB type {wkbtype}"             # <<<<<<<<<<<<<<
 *                     ) from None
 * 
*/
          __pyx_t_13 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3090, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_4 = __Pyx_PyUnicode_From_unsigned_int(__pyx_v_wkbtype, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3090, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_36[0] = __pyx_mstate_global->__pyx_kp_u_Could_not_create_geometry_at_ind;
          __pyx_t_36[1] = __pyx_t_13;
          __pyx_t_36[2] = __pyx_mstate_global->__pyx_kp_u_for_WKB_type;
          __pyx_t_36[3] = __pyx_t_4;
          __pyx_t_25 = __Pyx_PyUnicode_Join(__pyx_t_36, 4, 35 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13) + 14 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4), 127);
          if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3090, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_25);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_15))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_15);
            assert(__pyx_t_5);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_25};
            __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3089, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_14);
          }

          /* "pyogrio/_io.pyx":3091
 *                     raise GeometryError(
 *                         f"Could not create geometry at index {i} for WKB type {wkbtype}"
 *                     ) from None             # <<<<<<<<<<<<<<
 * 
 *                 # import the WKB
*/
          __Pyx_Raise(__pyx_t_14, 0, 0, Py_None);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __PYX_ERR(0, 3089, __pyx_L23_error)

          /* "pyogrio/_io.pyx":3088
 *                     )
 *                 ogr_geometry = OGR_G_CreateGeometry(<OGRwkbGeometryType>wkbtype)
 *                 if ogr_geometry == NULL:             # <<<<<<<<<<<<<<
 *                     raise GeometryError(
 *                         f"Could not create geometry at index {i} for WKB type {wkbtype}"
*/
        }

        /* "pyogrio/_io.pyx":3094
 * 
 *                 # import the WKB
 *                 err = OGR_G_ImportFromWkb(ogr_geometry, wkb_buffer, len(wkb))             # <<<<<<<<<<<<<<
 *                 if err:
 *                     raise GeometryError(
*/
        __pyx_t_3 = PyObject_Length(__pyx_v_wkb); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 3094, __pyx_L23_error)
        __pyx_v_err = OGR_G_ImportFromWkb(__pyx_v_ogr_geometry, __pyx_v_wkb_buffer, __pyx_t_3);

        /* "pyogrio/_io.pyx":3095
 *                 # import the WKB
 *                 err = OGR_G_ImportFromWkb(ogr_geometry, wkb_buffer, len(wkb))
 *                 if err:             # <<<<<<<<<<<<<<
 *                     raise GeometryError(
 *                         f"Could not create geometry from WKB at index {i}"
*/
        __pyx_t_2 = (__pyx_v_err != 0);
        if (unlikely(__pyx_t_2)) {

          /* "pyogrio/_io.pyx":3096
 *                 err = OGR_G_ImportFromWkb(ogr_geometry, wkb_buffer, len(wkb))
 *                 if err:
 *                     raise GeometryError(             # <<<<<<<<<<<<<<
 *                         f"Could not create geometry from WKB at index {i}"
 *                     ) from None
*/
          __pyx_t_15 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_25, __pyx_mstate_global->__pyx_n_u_GeometryError); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3096, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_25);

          /* "pyogrio/_io.pyx":3097
 *                 if err:
 *                     raise GeometryError(
 *                         f"Could not create geometry from WKB at index {i}"             # <<<<<<<<<<<<<<
 *                     ) from None
 * 
*/
          __pyx_t_5 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3097, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Could_not_create_geometry_from_W, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3097, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_25))) {
            __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_25);
            assert(__pyx_t_15);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_25);
            __Pyx_INCREF(__pyx_t_15);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_25, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_t_4};
            __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_25, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
            if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3096, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_14);
          }

          /* "pyogrio/_io.pyx":3098
 *                     raise GeometryError(
 *                         f"Could not create geometry from WKB at index {i}"
 *                     ) from None             # <<<<<<<<<<<<<<
 * 
 *                 # Convert to multi type
*/
          __Pyx_Raise(__pyx_t_14, 0, 0, Py_None);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __PYX_ERR(0, 3096, __pyx_L23_error)

          /* "pyogrio/_io.pyx":3095
 *                 # import the WKB
 *                 err = OGR_G_ImportFromWkb(ogr_geometry, wkb_buffer, len(wkb))
 *                 if err:             # <<<<<<<<<<<<<<
 *                     raise GeometryError(
 *                         f"Could not create geometry from WKB at index {i}"
*/
        }

        /* "pyogrio/_io.pyx":3101
 * 
 *                 # Convert to multi type
 *                 if promote_to_multi:             # <<<<<<<<<<<<<<
 *                     if wkbtype in (wkbPoint, wkbPoint25D, wkbPointM, wkbPointZM):
 *                         ogr_geometry = OGR_G_ForceToMultiPoint(ogr_geometry)
*/
        if (__pyx_v_promote_to_multi) {

          /* "pyogrio/_io.pyx":3102
 *                 # Convert to multi type
 *                 if promote_to_multi:
 *                     if wkbtype in (wkbPoint, wkbPoint25D, wkbPointM, wkbPointZM):             # <<<<<<<<<<<<<<
 *                         ogr_geometry = OGR_G_ForceToMultiPoint(ogr_geometry)
 *                     elif wkbtype in (
*/
          switch (__pyx_v_wkbtype) {
            case wkbPoint:
            case wkbPoint25D:
            case wkbPointM:
            case wkbPointZM:

            /* "pyogrio/_io.pyx":3103
 *                 if promote_to_multi:
 *                     if wkbtype in (wkbPoint, wkbPoint25D, wkbPointM, wkbPointZM):
 *                         ogr_geometry = OGR_G_ForceToMultiPoint(ogr_geometry)             # <<<<<<<<<<<<<<
 *                     elif wkbtype in (
 *                         wkbLineString, wkbLineString25D, wkbLineStringM, wkbLineStringZM
*/
            __pyx_v_ogr_geometry = OGR_G_ForceToMultiPoint(__pyx_v_ogr_geometry);

            /* "pyogrio/_io.pyx":3102
 *                 # Convert to multi type
 *                 if promote_to_multi:
 *                     if wkbtype in (wkbPoint, wkbPoint25D, wkbPointM, wkbPointZM):             # <<<<<<<<<<<<<<
 *                         ogr_geometry = OGR_G_ForceToMultiPoint(ogr_geometry)
 *                     elif wkbtype in (
*/
            break;
            case wkbLineString:

            /* "pyogrio/_io.pyx":3105
 *                         ogr_geometry = OGR_G_ForceToMultiPoint(ogr_geometry)
 *                     elif wkbtype in (
 *                         wkbLineString, wkbLineString25D, wkbLineStringM, wkbLineStringZM             # <<<<<<<<<<<<<<
 *                     ):
 *                         ogr_geometry = OGR_G_ForceToMultiLineString(ogr_geometry)
*/
            case wkbLineString25D:
            case wkbLineStringM:
            case wkbLineStringZM:

            /* "pyogrio/_io.pyx":3107
 *                         wkbLineString, wkbLineString25D, wkbLineStringM, wkbLineStringZM
 *                     ):
 *                         ogr_geometry = OGR_G_ForceToMultiLineString(ogr_geometry)             # <<<<<<<<<<<<<<
 *                     elif wkbtype in (
 *                         wkbPolygon, wkbPolygon25D, wkbPolygonM, wkbPolygonZM
*/
            __pyx_v_ogr_geometry = OGR_G_ForceToMultiLineString(__pyx_v_ogr_geometry);

            /* "pyogrio/_io.pyx":3104
 *                     if wkbtype in (wkbPoint, wkbPoint25D, wkbPointM, wkbPointZM):
 *                         ogr_geometry = OGR_G_ForceToMultiPoint(ogr_geometry)
 *                     elif wkbtype in (             # <<<<<<<<<<<<<<
 *                         wkbLineString, wkbLineString25D, wkbLineStringM, wkbLineStringZM
 *                     ):
*/
            break;
            case wkbPolygon:

            /* "pyogrio/_io.pyx":3109
 *                         ogr_geometry = OGR_G_ForceToMultiLineString(ogr_geometry)
 *                     elif wkbtype in (
 *                         wkbPolygon, wkbPolygon25D, wkbPolygonM, wkbPolygonZM             # <<<<<<<<<<<<<<
 *                     ):
 *                         ogr_geometry = OGR_G_ForceToMultiPolygon(ogr_geometry)
*/
            case wkbPolygon25D:
            case wkbPolygonM:
            case wkbPolygonZM:

            /* "pyogrio/_io.pyx":3111
 *                         wkbPolygon, wkbPolygon25D, wkbPolygonM, wkbPolygonZM
 *                     ):
 *                         ogr_geometry = OGR_G_ForceToMultiPolygon(ogr_geometry)             # <<<<<<<<<<<<<<
 * 
 *                 # Set the geometry on the feature
*/
            __pyx_v_ogr_geometry = OGR_G_ForceToMultiPolygon(__pyx_v_ogr_geometry);

            /* "pyogrio/_io.pyx":3108
 *                     ):
 *                         ogr_geometry = OGR_G_ForceToMultiLineString(ogr_geometry)
 *                     elif wkbtype in (             # <<<<<<<<<<<<<<
 *                         wkbPolygon, wkbPolygon25D, wkbPolygonM, wkbPolygonZM
 *                     ):
*/
            break;
            default: break;
          }

          /* "pyogrio/_io.pyx":3101
 * 
 *                 # Convert to multi type
 *                 if promote_to_multi:             # <<<<<<<<<<<<<<
 *                     if wkbtype in (wkbPoint, wkbPoint25D, wkbPointM, wkbPointZM):
 *                         ogr_geometry = OGR_G_ForceToMultiPoint(ogr_geometry)
*/
        }

        /* "pyogrio/_io.pyx":3115
 *                 # Set the geometry on the feature
 *                 # this assumes ownership of the geometry and it's cleanup
 *                 err = OGR_F_SetGeometryDirectly(ogr_feature, ogr_geometry)             # <<<<<<<<<<<<<<
 *                 ogr_geometry = NULL  # to prevent cleanup after this point
 *                 if err:
*/
        __pyx_v_err = OGR_F_SetGeometryDirectly(__pyx_v_ogr_feature, __pyx_v_ogr_geometry);

        /* "pyogrio/_io.pyx":3116
 *                 # this assumes ownership of the geometry and it's cleanup
 *                 err = OGR_F_SetGeometryDirectly(ogr_feature, ogr_geometry)
 *                 ogr_geometry = NULL  # to prevent cleanup after this point             # <<<<<<<<<<<<<<
 *                 if err:
 *                     raise GeometryError(
*/
        __pyx_v_ogr_geometry = NULL;

        /* "pyogrio/_io.pyx":3117
 *                 err = OGR_F_SetGeometryDirectly(ogr_feature, ogr_geometry)
 *                 ogr_geometry = NULL  # to prevent cleanup after this point
 *                 if err:             # <<<<<<<<<<<<<<
 *                     raise GeometryError(
 *                         f"Could not set geometry for feature at index {i}"
*/
        __pyx_t_2 = (__pyx_v_err != 0);
        if (unlikely(__pyx_t_2)) {

          /* "pyogrio/_io.pyx":3118
 *                 ogr_geometry = NULL  # to prevent cleanup after this point
 *                 if err:
 *                     raise GeometryError(             # <<<<<<<<<<<<<<
 *                         f"Could not set geometry for feature at index {i}"
 *                     ) from None
*/
          __pyx_t_25 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_GeometryError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3118, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_4);

          /* "pyogrio/_io.pyx":3119
 *                 if err:
 *                     raise GeometryError(
 *                         f"Could not set geometry for feature at index {i}"             # <<<<<<<<<<<<<<
 *                     ) from None
 * 
*/
          __pyx_t_15 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3119, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_5 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Could_not_set_geometry_for_featu, __pyx_t_15); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3119, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_25 = PyMethod_GET_SELF(__pyx_t_4);
            assert(__pyx_t_25);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_25);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_25, __pyx_t_5};
            __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3118, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_14);
          }

          /* "pyogrio/_io.pyx":3120
 *                     raise GeometryError(
 *                         f"Could not set geometry for feature at index {i}"
 *                     ) from None             # <<<<<<<<<<<<<<
 * 
 *             # Set field values
*/
          __Pyx_Raise(__pyx_t_14, 0, 0, Py_None);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __PYX_ERR(0, 3118, __pyx_L23_error)

          /* "pyogrio/_io.pyx":3117
 *                 err = OGR_F_SetGeometryDirectly(ogr_feature, ogr_geometry)
 *                 ogr_geometry = NULL  # to prevent cleanup after this point
 *                 if err:             # <<<<<<<<<<<<<<
 *                     raise GeometryError(
 *                         f"Could not set geometry for feature at index {i}"
*/
        }

        /* "pyogrio/_io.pyx":3073
 *             # TODO: geometry must not be null or errors
 *             wkb = None if geometry is None else geometry[i]
 *             if wkb is not None:             # <<<<<<<<<<<<<<
 *                 wkb_buffer = wkb
 *                 if wkb_buffer[0] == 1:
*/
      }

      /* "pyogrio/_io.pyx":3123
 * 
 *             # Set field values
 *             for field_idx in range(num_fields):             # <<<<<<<<<<<<<<
 *                 field_value = field_data[field_idx][i]
 *                 field_type = field_ogr_types[field_idx]
*/
      __pyx_t_28 = __pyx_v_num_fields;
      __pyx_t_24 = __pyx_t_28;
      for (__pyx_t_37 = 0; __pyx_t_37 < __pyx_t_24; __pyx_t_37+=1) {
        __pyx_v_field_idx = __pyx_t_37;

        /* "pyogrio/_io.pyx":3124
 *             # Set field values
 *             for field_idx in range(num_fields):
 *                 field_value = field_data[field_idx][i]             # <<<<<<<<<<<<<<
 *                 field_type = field_ogr_types[field_idx]
 *                 field_index = field_indexes[field_idx]
*/
        if (unlikely(__pyx_v_field_data == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 3124, __pyx_L23_error)
        }
        __pyx_t_14 = __Pyx_GetItemInt_List(__pyx_v_field_data, __pyx_v_field_idx, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3124, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_14, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3124, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF_SET(__pyx_v_field_value, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "pyogrio/_io.pyx":3125
 *             for field_idx in range(num_fields):
 *                 field_value = field_data[field_idx][i]
 *                 field_type = field_ogr_types[field_idx]             # <<<<<<<<<<<<<<
 *                 field_index = field_indexes[field_idx]
 * 
*/
        __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_field_ogr_types, __pyx_v_field_idx, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3125, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_38 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_38 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3125, __pyx_L23_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_v_field_type = __pyx_t_38;

        /* "pyogrio/_io.pyx":3126
 *                 field_value = field_data[field_idx][i]
 *                 field_type = field_ogr_types[field_idx]
 *                 field_index = field_indexes[field_idx]             # <<<<<<<<<<<<<<
 * 
 *                 mask = field_mask[field_idx]
*/
        __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_field_indexes, __pyx_v_field_idx, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3126, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_38 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_38 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3126, __pyx_L23_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_v_field_index = __pyx_t_38;

        /* "pyogrio/_io.pyx":3128
 *                 field_index = field_indexes[field_idx]
 * 
 *                 mask = field_mask[field_idx]             # <<<<<<<<<<<<<<
 *                 if mask is not None and mask[i]:
 *                     OGR_F_SetFieldNull(ogr_feature, field_index)
*/
        if (unlikely(__pyx_v_field_mask == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 3128, __pyx_L23_error)
        }
        __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_field_mask, __pyx_v_field_idx, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3128, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_XDECREF_SET(__pyx_v_mask, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "pyogrio/_io.pyx":3129
 * 
 *                 mask = field_mask[field_idx]
 *                 if mask is not None and mask[i]:             # <<<<<<<<<<<<<<
 *                     OGR_F_SetFieldNull(ogr_feature, field_index)
 * 
*/
        __pyx_t_7 = (__pyx_v_mask != Py_None);
        if (__pyx_t_7) {
        } else {
          __pyx_t_2 = __pyx_t_7;
          goto __pyx_L82_bool_binop_done;
        }
        __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_mask, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3129, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 3129, __pyx_L23_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_2 = __pyx_t_7;
        __pyx_L82_bool_binop_done:;
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":3130
 *                 mask = field_mask[field_idx]
 *                 if mask is not None and mask[i]:
 *                     OGR_F_SetFieldNull(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 * 
 *                 elif field_type == OFTString:
*/
          OGR_F_SetFieldNull(__pyx_v_ogr_feature, __pyx_v_field_index);

          /* "pyogrio/_io.pyx":3129
 * 
 *                 mask = field_mask[field_idx]
 *                 if mask is not None and mask[i]:             # <<<<<<<<<<<<<<
 *                     OGR_F_SetFieldNull(ogr_feature, field_index)
 * 
*/
          goto __pyx_L81;
        }

        /* "pyogrio/_io.pyx":3132
 *                     OGR_F_SetFieldNull(ogr_feature, field_index)
 * 
 *                 elif field_type == OFTString:             # <<<<<<<<<<<<<<
 *                     if (
 *                         field_value is None
*/
        __pyx_t_2 = (__pyx_v_field_type == OFTString);
        if (__pyx_t_2) {

          /* "pyogrio/_io.pyx":3134
 *                 elif field_type == OFTString:
 *                     if (
 *                         field_value is None             # <<<<<<<<<<<<<<
 *                         or (isinstance(field_value, float) and isnan(field_value))
 *                     ):
*/
          __pyx_t_7 = (__pyx_v_field_value == Py_None);
          if (!__pyx_t_7) {
          } else {
            __pyx_t_2 = __pyx_t_7;
            goto __pyx_L85_bool_binop_done;
          }

          /* "pyogrio/_io.pyx":3135
 *                     if (
 *                         field_value is None
 *                         or (isinstance(field_value, float) and isnan(field_value))             # <<<<<<<<<<<<<<
 *                     ):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
*/
          __pyx_t_7 = PyFloat_Check(__pyx_v_field_value); 
          if (__pyx_t_7) {
          } else {
            __pyx_t_2 = __pyx_t_7;
            goto __pyx_L85_bool_binop_done;
          }
          __pyx_t_39 = __Pyx_PyFloat_AsDouble(__pyx_v_field_value); if (unlikely((__pyx_t_39 == (long double)-1) && PyErr_Occurred())) __PYX_ERR(0, 3135, __pyx_L23_error)
          __pyx_t_7 = isnan(__pyx_t_39);
          __pyx_t_2 = __pyx_t_7;
          __pyx_L85_bool_binop_done:;

          /* "pyogrio/_io.pyx":3133
 * 
 *                 elif field_type == OFTString:
 *                     if (             # <<<<<<<<<<<<<<
 *                         field_value is None
 *                         or (isinstance(field_value, float) and isnan(field_value))
*/
          if (__pyx_t_2) {

            /* "pyogrio/_io.pyx":3137
 *                         or (isinstance(field_value, float) and isnan(field_value))
 *                     ):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 * 
 *                     else:
*/
            OGR_F_SetFieldNull(__pyx_v_ogr_feature, __pyx_v_field_index);

            /* "pyogrio/_io.pyx":3133
 * 
 *                 elif field_type == OFTString:
 *                     if (             # <<<<<<<<<<<<<<
 *                         field_value is None
 *                         or (isinstance(field_value, float) and isnan(field_value))
*/
            goto __pyx_L84;
          }

          /* "pyogrio/_io.pyx":3140
 * 
 *                     else:
 *                         if not isinstance(field_value, str):             # <<<<<<<<<<<<<<
 *                             field_value = str(field_value)
 * 
*/
          /*else*/ {
            __pyx_t_2 = PyUnicode_Check(__pyx_v_field_value); 
            __pyx_t_7 = (!__pyx_t_2);
            if (__pyx_t_7) {

              /* "pyogrio/_io.pyx":3141
 *                     else:
 *                         if not isinstance(field_value, str):
 *                             field_value = str(field_value)             # <<<<<<<<<<<<<<
 * 
 *                         try:
*/
              __pyx_t_4 = __Pyx_PyObject_Unicode(__pyx_v_field_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3141, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF_SET(__pyx_v_field_value, __pyx_t_4);
              __pyx_t_4 = 0;

              /* "pyogrio/_io.pyx":3140
 * 
 *                     else:
 *                         if not isinstance(field_value, str):             # <<<<<<<<<<<<<<
 *                             field_value = str(field_value)
 * 
*/
            }

            /* "pyogrio/_io.pyx":3143
 *                             field_value = str(field_value)
 * 
 *                         try:             # <<<<<<<<<<<<<<
 *                             value_b = field_value.encode(encoding)
 *                             OGR_F_SetFieldString(ogr_feature, field_index, value_b)
*/
            {
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __Pyx_ExceptionSave(&__pyx_t_32, &__pyx_t_31, &__pyx_t_30);
              __Pyx_XGOTREF(__pyx_t_32);
              __Pyx_XGOTREF(__pyx_t_31);
              __Pyx_XGOTREF(__pyx_t_30);
              /*try:*/ {

                /* "pyogrio/_io.pyx":3144
 * 
 *                         try:
 *                             value_b = field_value.encode(encoding)             # <<<<<<<<<<<<<<
 *                             OGR_F_SetFieldString(ogr_feature, field_index, value_b)
 * 
*/
                __pyx_t_14 = __pyx_v_field_value;
                __Pyx_INCREF(__pyx_t_14);
                __pyx_t_6 = 0;
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_encoding};
                  __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
                  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3144, __pyx_L89_error)
                  __Pyx_GOTREF(__pyx_t_4);
                }
                __Pyx_XDECREF_SET(__pyx_v_value_b, __pyx_t_4);
                __pyx_t_4 = 0;

                /* "pyogrio/_io.pyx":3145
 *                         try:
 *                             value_b = field_value.encode(encoding)
 *                             OGR_F_SetFieldString(ogr_feature, field_index, value_b)             # <<<<<<<<<<<<<<
 * 
 *                         except AttributeError:
*/
                __pyx_t_40 = __Pyx_PyObject_AsWritableString(__pyx_v_value_b); if (unlikely((!__pyx_t_40) && PyErr_Occurred())) __PYX_ERR(0, 3145, __pyx_L89_error)
                OGR_F_SetFieldString(__pyx_v_ogr_feature, __pyx_v_field_index, __pyx_t_40);

                /* "pyogrio/_io.pyx":3143
 *                             field_value = str(field_value)
 * 
 *                         try:             # <<<<<<<<<<<<<<
 *                             value_b = field_value.encode(encoding)
 *                             OGR_F_SetFieldString(ogr_feature, field_index, value_b)
*/
              }
              __Pyx_XDECREF(__pyx_t_32); __pyx_t_32 = 0;
              __Pyx_XDECREF(__pyx_t_31); __pyx_t_31 = 0;
              __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
              goto __pyx_L96_try_end;
              __pyx_L89_error:;
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
              __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

              /* "pyogrio/_io.pyx":3147
 *                             OGR_F_SetFieldString(ogr_feature, field_index, value_b)
 * 
 *                         except AttributeError:             # <<<<<<<<<<<<<<
 *                             raise ValueError(
 *                                 f"Could not encode value '{field_value}' in field "
*/
              __pyx_t_38 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_AttributeError))));
              if (__pyx_t_38) {
                __Pyx_AddTraceback("pyogrio._io.ogr_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_14, &__pyx_t_5) < 0) __PYX_ERR(0, 3147, __pyx_L91_except_error)
                __Pyx_XGOTREF(__pyx_t_4);
                __Pyx_XGOTREF(__pyx_t_14);
                __Pyx_XGOTREF(__pyx_t_5);

                /* "pyogrio/_io.pyx":3148
 * 
 *                         except AttributeError:
 *                             raise ValueError(             # <<<<<<<<<<<<<<
 *                                 f"Could not encode value '{field_value}' in field "
 *                                 f"'{fields[field_idx]}' to string"
*/
                __pyx_t_15 = NULL;

                /* "pyogrio/_io.pyx":3149
 *                         except AttributeError:
 *                             raise ValueError(
 *                                 f"Could not encode value '{field_value}' in field "             # <<<<<<<<<<<<<<
 *                                 f"'{fields[field_idx]}' to string"
 *                             )
*/
                __pyx_t_13 = __Pyx_PyObject_FormatSimple(__pyx_v_field_value, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3149, __pyx_L91_except_error)
                __Pyx_GOTREF(__pyx_t_13);

                /* "pyogrio/_io.pyx":3150
 *                             raise ValueError(
 *                                 f"Could not encode value '{field_value}' in field "
 *                                 f"'{fields[field_idx]}' to string"             # <<<<<<<<<<<<<<
 *                             )
 * 
*/
                if (unlikely(__pyx_v_fields == Py_None)) {
                  PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                  __PYX_ERR(0, 3150, __pyx_L91_except_error)
                }
                __pyx_t_10 = __Pyx_GetItemInt_List(__pyx_v_fields, __pyx_v_field_idx, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3150, __pyx_L91_except_error)
                __Pyx_GOTREF(__pyx_t_10);
                __pyx_t_26 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 3150, __pyx_L91_except_error)
                __Pyx_GOTREF(__pyx_t_26);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __pyx_t_41[0] = __pyx_mstate_global->__pyx_kp_u_Could_not_encode_value;
                __pyx_t_41[1] = __pyx_t_13;
                __pyx_t_41[2] = __pyx_mstate_global->__pyx_kp_u_in_field;
                __pyx_t_41[3] = __pyx_t_26;
                __pyx_t_41[4] = __pyx_mstate_global->__pyx_kp_u_to_string;

                /* "pyogrio/_io.pyx":3149
 *                         except AttributeError:
 *                             raise ValueError(
 *                                 f"Could not encode value '{field_value}' in field "             # <<<<<<<<<<<<<<
 *                                 f"'{fields[field_idx]}' to string"
 *                             )
*/
                __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_41, 5, 24 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13) + 12 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_26) + 11, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_26));
                if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3149, __pyx_L91_except_error)
                __Pyx_GOTREF(__pyx_t_10);
                __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
                __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
                __pyx_t_6 = 1;
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_t_10};
                  __pyx_t_25 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                  if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3148, __pyx_L91_except_error)
                  __Pyx_GOTREF(__pyx_t_25);
                }
                __Pyx_Raise(__pyx_t_25, 0, 0, 0);
                __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
                __PYX_ERR(0, 3148, __pyx_L91_except_error)
              }

              /* "pyogrio/_io.pyx":3153
 *                             )
 * 
 *                         except Exception:             # <<<<<<<<<<<<<<
 *                             raise
 * 
*/
              __pyx_t_38 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
              if (__pyx_t_38) {
                __Pyx_AddTraceback("pyogrio._io.ogr_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_14, &__pyx_t_4) < 0) __PYX_ERR(0, 3153, __pyx_L91_except_error)
                __Pyx_XGOTREF(__pyx_t_5);
                __Pyx_XGOTREF(__pyx_t_14);
                __Pyx_XGOTREF(__pyx_t_4);

                /* "pyogrio/_io.pyx":3154
 * 
 *                         except Exception:
 *                             raise             # <<<<<<<<<<<<<<
 * 
 *                 elif field_type == OFTInteger:
*/
                __Pyx_GIVEREF(__pyx_t_5);
                __Pyx_GIVEREF(__pyx_t_14);
                __Pyx_XGIVEREF(__pyx_t_4);
                __Pyx_ErrRestoreWithState(__pyx_t_5, __pyx_t_14, __pyx_t_4);
                __pyx_t_5 = 0;  __pyx_t_14 = 0;  __pyx_t_4 = 0; 
                __PYX_ERR(0, 3154, __pyx_L91_except_error)
              }
              goto __pyx_L91_except_error;

              /* "pyogrio/_io.pyx":3143
 *                             field_value = str(field_value)
 * 
 *                         try:             # <<<<<<<<<<<<<<
 *                             value_b = field_value.encode(encoding)
 *                             OGR_F_SetFieldString(ogr_feature, field_index, value_b)
*/
              __pyx_L91_except_error:;
              __Pyx_XGIVEREF(__pyx_t_32);
              __Pyx_XGIVEREF(__pyx_t_31);
              __Pyx_XGIVEREF(__pyx_t_30);
              __Pyx_ExceptionReset(__pyx_t_32, __pyx_t_31, __pyx_t_30);
              goto __pyx_L23_error;
              __pyx_L96_try_end:;
            }
          }
          __pyx_L84:;

          /* "pyogrio/_io.pyx":3132
 *                     OGR_F_SetFieldNull(ogr_feature, field_index)
 * 
 *                 elif field_type == OFTString:             # <<<<<<<<<<<<<<
 *                     if (
 *                         field_value is None
*/
          goto __pyx_L81;
        }

        /* "pyogrio/_io.pyx":3156
 *                             raise
 * 
 *                 elif field_type == OFTInteger:             # <<<<<<<<<<<<<<
 *                     OGR_F_SetFieldInteger(ogr_feature, field_index, field_value)
 * 
*/
        __pyx_t_7 = (__pyx_v_field_type == OFTInteger);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3157
 * 
 *                 elif field_type == OFTInteger:
 *                     OGR_F_SetFieldInteger(ogr_feature, field_index, field_value)             # <<<<<<<<<<<<<<
 * 
 *                 elif field_type == OFTInteger64:
*/
          __pyx_t_38 = __Pyx_PyLong_As_int(__pyx_v_field_value); if (unlikely((__pyx_t_38 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3157, __pyx_L23_error)
          OGR_F_SetFieldInteger(__pyx_v_ogr_feature, __pyx_v_field_index, __pyx_t_38);

          /* "pyogrio/_io.pyx":3156
 *                             raise
 * 
 *                 elif field_type == OFTInteger:             # <<<<<<<<<<<<<<
 *                     OGR_F_SetFieldInteger(ogr_feature, field_index, field_value)
 * 
*/
          goto __pyx_L81;
        }

        /* "pyogrio/_io.pyx":3159
 *                     OGR_F_SetFieldInteger(ogr_feature, field_index, field_value)
 * 
 *                 elif field_type == OFTInteger64:             # <<<<<<<<<<<<<<
 *                     OGR_F_SetFieldInteger64(ogr_feature, field_index, field_value)
 * 
*/
        __pyx_t_7 = (__pyx_v_field_type == OFTInteger64);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3160
 * 
 *                 elif field_type == OFTInteger64:
 *                     OGR_F_SetFieldInteger64(ogr_feature, field_index, field_value)             # <<<<<<<<<<<<<<
 * 
 *                 elif field_type == OFTReal:
*/
          __pyx_t_42 = __Pyx_PyLong_As_int64_t(__pyx_v_field_value); if (unlikely((__pyx_t_42 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 3160, __pyx_L23_error)
          OGR_F_SetFieldInteger64(__pyx_v_ogr_feature, __pyx_v_field_index, __pyx_t_42);

          /* "pyogrio/_io.pyx":3159
 *                     OGR_F_SetFieldInteger(ogr_feature, field_index, field_value)
 * 
 *                 elif field_type == OFTInteger64:             # <<<<<<<<<<<<<<
 *                     OGR_F_SetFieldInteger64(ogr_feature, field_index, field_value)
 * 
*/
          goto __pyx_L81;
        }

        /* "pyogrio/_io.pyx":3162
 *                     OGR_F_SetFieldInteger64(ogr_feature, field_index, field_value)
 * 
 *                 elif field_type == OFTReal:             # <<<<<<<<<<<<<<
 *                     if nan_as_null and isnan(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
*/
        __pyx_t_7 = (__pyx_v_field_type == OFTReal);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3163
 * 
 *                 elif field_type == OFTReal:
 *                     if nan_as_null and isnan(field_value):             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
*/
          if (__pyx_v_nan_as_null) {
          } else {
            __pyx_t_7 = __pyx_v_nan_as_null;
            goto __pyx_L102_bool_binop_done;
          }
          __pyx_t_39 = __Pyx_PyFloat_AsDouble(__pyx_v_field_value); if (unlikely((__pyx_t_39 == (long double)-1) && PyErr_Occurred())) __PYX_ERR(0, 3163, __pyx_L23_error)
          __pyx_t_2 = isnan(__pyx_t_39);
          __pyx_t_7 = __pyx_t_2;
          __pyx_L102_bool_binop_done:;
          if (__pyx_t_7) {

            /* "pyogrio/_io.pyx":3164
 *                 elif field_type == OFTReal:
 *                     if nan_as_null and isnan(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 *                     else:
 *                         OGR_F_SetFieldDouble(ogr_feature, field_index, field_value)
*/
            OGR_F_SetFieldNull(__pyx_v_ogr_feature, __pyx_v_field_index);

            /* "pyogrio/_io.pyx":3163
 * 
 *                 elif field_type == OFTReal:
 *                     if nan_as_null and isnan(field_value):             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
*/
            goto __pyx_L101;
          }

          /* "pyogrio/_io.pyx":3166
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
 *                         OGR_F_SetFieldDouble(ogr_feature, field_index, field_value)             # <<<<<<<<<<<<<<
 * 
 *                 elif field_type == OFTDate:
*/
          /*else*/ {
            __pyx_t_43 = __Pyx_PyFloat_AsDouble(__pyx_v_field_value); if (unlikely((__pyx_t_43 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 3166, __pyx_L23_error)
            OGR_F_SetFieldDouble(__pyx_v_ogr_feature, __pyx_v_field_index, __pyx_t_43);
          }
          __pyx_L101:;

          /* "pyogrio/_io.pyx":3162
 *                     OGR_F_SetFieldInteger64(ogr_feature, field_index, field_value)
 * 
 *                 elif field_type == OFTReal:             # <<<<<<<<<<<<<<
 *                     if nan_as_null and isnan(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
*/
          goto __pyx_L81;
        }

        /* "pyogrio/_io.pyx":3168
 *                         OGR_F_SetFieldDouble(ogr_feature, field_index, field_value)
 * 
 *                 elif field_type == OFTDate:             # <<<<<<<<<<<<<<
 *                     if np.isnat(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
*/
        __pyx_t_7 = (__pyx_v_field_type == OFTDate);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3169
 * 
 *                 elif field_type == OFTDate:
 *                     if np.isnat(field_value):             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
*/
          __pyx_t_14 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3169, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_25 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_isnat); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3169, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_25);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_25))) {
            __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_25);
            assert(__pyx_t_14);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_25);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_25, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_v_field_value};
            __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_25, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3169, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
          }
          __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 3169, __pyx_L23_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (__pyx_t_7) {

            /* "pyogrio/_io.pyx":3170
 *                 elif field_type == OFTDate:
 *                     if np.isnat(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 *                     else:
 *                         datetime_value = field_value.item()
*/
            OGR_F_SetFieldNull(__pyx_v_ogr_feature, __pyx_v_field_index);

            /* "pyogrio/_io.pyx":3169
 * 
 *                 elif field_type == OFTDate:
 *                     if np.isnat(field_value):             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
*/
            goto __pyx_L104;
          }

          /* "pyogrio/_io.pyx":3172
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
 *                         datetime_value = field_value.item()             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldDateTimeEx(
 *                             ogr_feature,
*/
          /*else*/ {
            __pyx_t_25 = __pyx_v_field_value;
            __Pyx_INCREF(__pyx_t_25);
            __pyx_t_6 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_25, NULL};
              __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_item, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3172, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_4);
            }
            __Pyx_XDECREF_SET(__pyx_v_datetime_value, __pyx_t_4);
            __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3176
 *                             ogr_feature,
 *                             field_index,
 *                             datetime_value.year,             # <<<<<<<<<<<<<<
 *                             datetime_value.month,
 *                             datetime_value.day,
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_year); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3176, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_38 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_38 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3176, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3177
 *                             field_index,
 *                             datetime_value.year,
 *                             datetime_value.month,             # <<<<<<<<<<<<<<
 *                             datetime_value.day,
 *                             0,
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_month); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3177, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_44 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_44 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3177, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3178
 *                             datetime_value.year,
 *                             datetime_value.month,
 *                             datetime_value.day,             # <<<<<<<<<<<<<<
 *                             0,
 *                             0,
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_day); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3178, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_45 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_45 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3178, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3173
 *                     else:
 *                         datetime_value = field_value.item()
 *                         OGR_F_SetFieldDateTimeEx(             # <<<<<<<<<<<<<<
 *                             ogr_feature,
 *                             field_index,
*/
            OGR_F_SetFieldDateTimeEx(__pyx_v_ogr_feature, __pyx_v_field_index, __pyx_t_38, __pyx_t_44, __pyx_t_45, 0, 0, 0.0, 0);
          }
          __pyx_L104:;

          /* "pyogrio/_io.pyx":3168
 *                         OGR_F_SetFieldDouble(ogr_feature, field_index, field_value)
 * 
 *                 elif field_type == OFTDate:             # <<<<<<<<<<<<<<
 *                     if np.isnat(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
*/
          goto __pyx_L81;
        }

        /* "pyogrio/_io.pyx":3185
 *                         )
 * 
 *                 elif field_type == OFTDateTime:             # <<<<<<<<<<<<<<
 *                     if np.isnat(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
*/
        __pyx_t_7 = (__pyx_v_field_type == OFTDateTime);
        if (likely(__pyx_t_7)) {

          /* "pyogrio/_io.pyx":3186
 * 
 *                 elif field_type == OFTDateTime:
 *                     if np.isnat(field_value):             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
*/
          __pyx_t_25 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3186, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_isnat); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3186, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_25 = PyMethod_GET_SELF(__pyx_t_5);
            assert(__pyx_t_25);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_25);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_25, __pyx_v_field_value};
            __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3186, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
          }
          __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 3186, __pyx_L23_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (__pyx_t_7) {

            /* "pyogrio/_io.pyx":3187
 *                 elif field_type == OFTDateTime:
 *                     if np.isnat(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)             # <<<<<<<<<<<<<<
 *                     else:
 *                         datetime_value = field_value.astype("datetime64[ms]").item()
*/
            OGR_F_SetFieldNull(__pyx_v_ogr_feature, __pyx_v_field_index);

            /* "pyogrio/_io.pyx":3186
 * 
 *                 elif field_type == OFTDateTime:
 *                     if np.isnat(field_value):             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
*/
            goto __pyx_L105;
          }

          /* "pyogrio/_io.pyx":3189
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
 *                     else:
 *                         datetime_value = field_value.astype("datetime64[ms]").item()             # <<<<<<<<<<<<<<
 *                         tz_array = gdal_tz_offsets.get(fields[field_idx], None)
 *                         if tz_array is None:
*/
          /*else*/ {
            __pyx_t_14 = __pyx_v_field_value;
            __Pyx_INCREF(__pyx_t_14);
            __pyx_t_6 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_mstate_global->__pyx_kp_u_datetime64_ms};
              __pyx_t_25 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_astype, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3189, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_25);
            }
            __pyx_t_5 = __pyx_t_25;
            __Pyx_INCREF(__pyx_t_5);
            __pyx_t_6 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_5, NULL};
              __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_item, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3189, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_4);
            }
            __Pyx_XDECREF_SET(__pyx_v_datetime_value, __pyx_t_4);
            __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3190
 *                     else:
 *                         datetime_value = field_value.astype("datetime64[ms]").item()
 *                         tz_array = gdal_tz_offsets.get(fields[field_idx], None)             # <<<<<<<<<<<<<<
 *                         if tz_array is None:
 *                             gdal_tz = 0
*/
            __pyx_t_25 = __pyx_v_gdal_tz_offsets;
            __Pyx_INCREF(__pyx_t_25);
            if (unlikely(__pyx_v_fields == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
              __PYX_ERR(0, 3190, __pyx_L23_error)
            }
            __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_fields, __pyx_v_field_idx, int, 1, __Pyx_PyLong_From_int, 1, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3190, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_5);
            __pyx_t_6 = 0;
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_25, __pyx_t_5, Py_None};
              __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3190, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_4);
            }
            __Pyx_XDECREF_SET(__pyx_v_tz_array, __pyx_t_4);
            __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3191
 *                         datetime_value = field_value.astype("datetime64[ms]").item()
 *                         tz_array = gdal_tz_offsets.get(fields[field_idx], None)
 *                         if tz_array is None:             # <<<<<<<<<<<<<<
 *                             gdal_tz = 0
 *                         elif np.isnan(tz_array[i]):
*/
            __pyx_t_7 = (__pyx_v_tz_array == Py_None);
            if (__pyx_t_7) {

              /* "pyogrio/_io.pyx":3192
 *                         tz_array = gdal_tz_offsets.get(fields[field_idx], None)
 *                         if tz_array is None:
 *                             gdal_tz = 0             # <<<<<<<<<<<<<<
 *                         elif np.isnan(tz_array[i]):
 *                             gdal_tz = 0
*/
              __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
              __Pyx_XDECREF_SET(__pyx_v_gdal_tz, __pyx_mstate_global->__pyx_int_0);

              /* "pyogrio/_io.pyx":3191
 *                         datetime_value = field_value.astype("datetime64[ms]").item()
 *                         tz_array = gdal_tz_offsets.get(fields[field_idx], None)
 *                         if tz_array is None:             # <<<<<<<<<<<<<<
 *                             gdal_tz = 0
 *                         elif np.isnan(tz_array[i]):
*/
              goto __pyx_L106;
            }

            /* "pyogrio/_io.pyx":3193
 *                         if tz_array is None:
 *                             gdal_tz = 0
 *                         elif np.isnan(tz_array[i]):             # <<<<<<<<<<<<<<
 *                             gdal_tz = 0
 *                         else:
*/
            __pyx_t_5 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_25, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3193, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_25);
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_25, __pyx_mstate_global->__pyx_n_u_isnan); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3193, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
            __pyx_t_25 = __Pyx_GetItemInt(__pyx_v_tz_array, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3193, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_25);
            __pyx_t_6 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_14);
              assert(__pyx_t_5);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
              __Pyx_INCREF(__pyx_t_5);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
              __pyx_t_6 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_25};
              __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3193, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_4);
            }
            __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 3193, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (__pyx_t_7) {

              /* "pyogrio/_io.pyx":3194
 *                             gdal_tz = 0
 *                         elif np.isnan(tz_array[i]):
 *                             gdal_tz = 0             # <<<<<<<<<<<<<<
 *                         else:
 *                             gdal_tz = tz_array[i]
*/
              __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
              __Pyx_XDECREF_SET(__pyx_v_gdal_tz, __pyx_mstate_global->__pyx_int_0);

              /* "pyogrio/_io.pyx":3193
 *                         if tz_array is None:
 *                             gdal_tz = 0
 *                         elif np.isnan(tz_array[i]):             # <<<<<<<<<<<<<<
 *                             gdal_tz = 0
 *                         else:
*/
              goto __pyx_L106;
            }

            /* "pyogrio/_io.pyx":3196
 *                             gdal_tz = 0
 *                         else:
 *                             gdal_tz = tz_array[i]             # <<<<<<<<<<<<<<
 *                         OGR_F_SetFieldDateTimeEx(
 *                             ogr_feature,
*/
            /*else*/ {
              __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_tz_array, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3196, __pyx_L23_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_XDECREF_SET(__pyx_v_gdal_tz, __pyx_t_4);
              __pyx_t_4 = 0;
            }
            __pyx_L106:;

            /* "pyogrio/_io.pyx":3200
 *                             ogr_feature,
 *                             field_index,
 *                             datetime_value.year,             # <<<<<<<<<<<<<<
 *                             datetime_value.month,
 *                             datetime_value.day,
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_year); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3200, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_45 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_45 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3200, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3201
 *                             field_index,
 *                             datetime_value.year,
 *                             datetime_value.month,             # <<<<<<<<<<<<<<
 *                             datetime_value.day,
 *                             datetime_value.hour,
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_month); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3201, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_44 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_44 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3201, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3202
 *                             datetime_value.year,
 *                             datetime_value.month,
 *                             datetime_value.day,             # <<<<<<<<<<<<<<
 *                             datetime_value.hour,
 *                             datetime_value.minute,
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_day); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3202, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_38 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_38 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3202, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3203
 *                             datetime_value.month,
 *                             datetime_value.day,
 *                             datetime_value.hour,             # <<<<<<<<<<<<<<
 *                             datetime_value.minute,
 *                             datetime_value.second + datetime_value.microsecond / 10**6,
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_hour); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3203, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_46 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_46 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3203, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3204
 *                             datetime_value.day,
 *                             datetime_value.hour,
 *                             datetime_value.minute,             # <<<<<<<<<<<<<<
 *                             datetime_value.second + datetime_value.microsecond / 10**6,
 *                             gdal_tz
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_minute); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3204, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_47 = __Pyx_PyLong_As_int(__pyx_t_4); if (unlikely((__pyx_t_47 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3204, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "pyogrio/_io.pyx":3205
 *                             datetime_value.hour,
 *                             datetime_value.minute,
 *                             datetime_value.second + datetime_value.microsecond / 10**6,             # <<<<<<<<<<<<<<
 *                             gdal_tz
 *                         )
*/
            __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_second); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3205, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_datetime_value, __pyx_mstate_global->__pyx_n_u_microsecond); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3205, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_14);
            __pyx_t_25 = __Pyx_PyLong_TrueDivideObjC(__pyx_t_14, __pyx_mstate_global->__pyx_int_1000000, 0xf4240, 0, 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3205, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_25);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __pyx_t_14 = PyNumber_Add(__pyx_t_4, __pyx_t_25); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3205, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
            __pyx_t_48 = __Pyx_PyFloat_AsFloat(__pyx_t_14); if (unlikely((__pyx_t_48 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 3205, __pyx_L23_error)
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

            /* "pyogrio/_io.pyx":3206
 *                             datetime_value.minute,
 *                             datetime_value.second + datetime_value.microsecond / 10**6,
 *                             gdal_tz             # <<<<<<<<<<<<<<
 *                         )
 * 
*/
            __pyx_t_49 = __Pyx_PyLong_As_int(__pyx_v_gdal_tz); if (unlikely((__pyx_t_49 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3206, __pyx_L23_error)

            /* "pyogrio/_io.pyx":3197
 *                         else:
 *                             gdal_tz = tz_array[i]
 *                         OGR_F_SetFieldDateTimeEx(             # <<<<<<<<<<<<<<
 *                             ogr_feature,
 *                             field_index,
*/
            OGR_F_SetFieldDateTimeEx(__pyx_v_ogr_feature, __pyx_v_field_index, __pyx_t_45, __pyx_t_44, __pyx_t_38, __pyx_t_46, __pyx_t_47, __pyx_t_48, __pyx_t_49);
          }
          __pyx_L105:;

          /* "pyogrio/_io.pyx":3185
 *                         )
 * 
 *                 elif field_type == OFTDateTime:             # <<<<<<<<<<<<<<
 *                     if np.isnat(field_value):
 *                         OGR_F_SetFieldNull(ogr_feature, field_index)
*/
          goto __pyx_L81;
        }

        /* "pyogrio/_io.pyx":3210
 * 
 *                 else:
 *                     raise NotImplementedError(             # <<<<<<<<<<<<<<
 *                         f"OGR field type is not supported for writing: {field_type}"
 *                     )
*/
        /*else*/ {
          __pyx_t_25 = NULL;

          /* "pyogrio/_io.pyx":3211
 *                 else:
 *                     raise NotImplementedError(
 *                         f"OGR field type is not supported for writing: {field_type}"             # <<<<<<<<<<<<<<
 *                     )
 * 
*/
          __pyx_t_4 = __Pyx_PyUnicode_From_int(__pyx_v_field_type, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3211, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_5 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_OGR_field_type_is_not_supported, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3211, __pyx_L23_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_6 = 1;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_25, __pyx_t_5};
            __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_NotImplementedError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3210, __pyx_L23_error)
            __Pyx_GOTREF(__pyx_t_14);
          }
          __Pyx_Raise(__pyx_t_14, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __PYX_ERR(0, 3210, __pyx_L23_error)
        }
        __pyx_L81:;
      }

      /* "pyogrio/_io.pyx":3215
 * 
 *             # Add feature to the layer
 *             try:             # <<<<<<<<<<<<<<
 *                 check_int(OGR_L_CreateFeature(ogr_layer, ogr_feature))
 * 
*/
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_30, &__pyx_t_31, &__pyx_t_32);
        __Pyx_XGOTREF(__pyx_t_30);
        __Pyx_XGOTREF(__pyx_t_31);
        __Pyx_XGOTREF(__pyx_t_32);
        /*try:*/ {

          /* "pyogrio/_io.pyx":3216
 *             # Add feature to the layer
 *             try:
 *                 check_int(OGR_L_CreateFeature(ogr_layer, ogr_feature))             # <<<<<<<<<<<<<<
 * 
 *             except CPLE_BaseError as exc:
*/
          __pyx_t_28 = __pyx_f_7pyogrio_4_err_check_int(OGR_L_CreateFeature(__pyx_v_ogr_layer, __pyx_v_ogr_feature)); if (unlikely(__pyx_t_28 == ((int)-1))) __PYX_ERR(0, 3216, __pyx_L107_error)

          /* "pyogrio/_io.pyx":3215
 * 
 *             # Add feature to the layer
 *             try:             # <<<<<<<<<<<<<<
 *                 check_int(OGR_L_CreateFeature(ogr_layer, ogr_feature))
 * 
*/
        }
        __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
        __Pyx_XDECREF(__pyx_t_31); __pyx_t_31 = 0;
        __Pyx_XDECREF(__pyx_t_32); __pyx_t_32 = 0;
        goto __pyx_L114_try_end;
        __pyx_L107_error:;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
        __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "pyogrio/_io.pyx":3218
 *                 check_int(OGR_L_CreateFeature(ogr_layer, ogr_feature))
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(
 *                     f"Could not add feature to layer at index {i}: {exc}"
*/
        __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_5, &__pyx_t_25);
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_CPLE_BaseError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3218, __pyx_L109_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_28 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_14, __pyx_t_4);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_ErrRestore(__pyx_t_14, __pyx_t_5, __pyx_t_25);
        __pyx_t_14 = 0; __pyx_t_5 = 0; __pyx_t_25 = 0;
        if (__pyx_t_28) {
          __Pyx_AddTraceback("pyogrio._io.ogr_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_25, &__pyx_t_5, &__pyx_t_14) < 0) __PYX_ERR(0, 3218, __pyx_L109_except_error)
          __Pyx_XGOTREF(__pyx_t_25);
          __Pyx_XGOTREF(__pyx_t_5);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_5);
          __pyx_v_exc = __pyx_t_5;
          /*try:*/ {

            /* "pyogrio/_io.pyx":3219
 * 
 *             except CPLE_BaseError as exc:
 *                 raise FeatureError(             # <<<<<<<<<<<<<<
 *                     f"Could not add feature to layer at index {i}: {exc}"
 *                 ) from None
*/
            __pyx_t_10 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_FeatureError); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3219, __pyx_L120_error)
            __Pyx_GOTREF(__pyx_t_15);

            /* "pyogrio/_io.pyx":3220
 *             except CPLE_BaseError as exc:
 *                 raise FeatureError(
 *                     f"Could not add feature to layer at index {i}: {exc}"             # <<<<<<<<<<<<<<
 *                 ) from None
 * 
*/
            __pyx_t_26 = __Pyx_PyUnicode_From_int(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_26)) __PYX_ERR(0, 3220, __pyx_L120_error)
            __Pyx_GOTREF(__pyx_t_26);
            __pyx_t_13 = __Pyx_PyObject_FormatSimple(__pyx_v_exc, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3220, __pyx_L120_error)
            __Pyx_GOTREF(__pyx_t_13);
            __pyx_t_36[0] = __pyx_mstate_global->__pyx_kp_u_Could_not_add_feature_to_layer_a;
            __pyx_t_36[1] = __pyx_t_26;
            __pyx_t_36[2] = __pyx_mstate_global->__pyx_kp_u_;
            __pyx_t_36[3] = __pyx_t_13;
            __pyx_t_50 = __Pyx_PyUnicode_Join(__pyx_t_36, 4, 40 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_26) + 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13));
            if (unlikely(!__pyx_t_50)) __PYX_ERR(0, 3220, __pyx_L120_error)
            __Pyx_GOTREF(__pyx_t_50);
            __Pyx_DECREF(__pyx_t_26); __pyx_t_26 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            __pyx_t_6 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_15))) {
              __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_15);
              assert(__pyx_t_10);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
              __Pyx_INCREF(__pyx_t_10);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
              __pyx_t_6 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_50};
              __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_DECREF(__pyx_t_50); __pyx_t_50 = 0;
              __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
              if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3219, __pyx_L120_error)
              __Pyx_GOTREF(__pyx_t_4);
            }

            /* "pyogrio/_io.pyx":3221
 *                 raise FeatureError(
 *                     f"Could not add feature to layer at index {i}: {exc}"
 *                 ) from None             # <<<<<<<<<<<<<<
 * 
 *             OGR_F_Destroy(ogr_feature)
*/
            __Pyx_Raise(__pyx_t_4, 0, 0, Py_None);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __PYX_ERR(0, 3219, __pyx_L120_error)
          }

          /* "pyogrio/_io.pyx":3218
 *                 check_int(OGR_L_CreateFeature(ogr_layer, ogr_feature))
 * 
 *             except CPLE_BaseError as exc:             # <<<<<<<<<<<<<<
 *                 raise FeatureError(
 *                     f"Could not add feature to layer at index {i}: {exc}"
*/
          /*finally:*/ {
            __pyx_L120_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_52 = 0; __pyx_t_53 = 0; __pyx_t_54 = 0;
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_XDECREF(__pyx_t_50); __pyx_t_50 = 0;
               __Pyx_ExceptionSwap(&__pyx_t_52, &__pyx_t_53, &__pyx_t_54);
              if ( unlikely(__Pyx_GetException(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19) < 0)) __Pyx_ErrFetch(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
              __Pyx_XGOTREF(__pyx_t_17);
              __Pyx_XGOTREF(__pyx_t_18);
              __Pyx_XGOTREF(__pyx_t_19);
              __Pyx_XGOTREF(__pyx_t_52);
              __Pyx_XGOTREF(__pyx_t_53);
              __Pyx_XGOTREF(__pyx_t_54);
              __pyx_t_28 = __pyx_lineno; __pyx_t_24 = __pyx_clineno; __pyx_t_51 = __pyx_filename;
              {
                __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_52);
              __Pyx_XGIVEREF(__pyx_t_53);
              __Pyx_XGIVEREF(__pyx_t_54);
              __Pyx_ExceptionReset(__pyx_t_52, __pyx_t_53, __pyx_t_54);
              __Pyx_XGIVEREF(__pyx_t_17);
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_ErrRestore(__pyx_t_17, __pyx_t_18, __pyx_t_19);
              __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_52 = 0; __pyx_t_53 = 0; __pyx_t_54 = 0;
              __pyx_lineno = __pyx_t_28; __pyx_clineno = __pyx_t_24; __pyx_filename = __pyx_t_51;
              goto __pyx_L109_except_error;
            }
          }
        }
        goto __pyx_L109_except_error;

        /* "pyogrio/_io.pyx":3215
 * 
 *             # Add feature to the layer
 *             try:             # <<<<<<<<<<<<<<
 *                 check_int(OGR_L_CreateFeature(ogr_layer, ogr_feature))
 * 
*/
        __pyx_L109_except_error:;
        __Pyx_XGIVEREF(__pyx_t_30);
        __Pyx_XGIVEREF(__pyx_t_31);
        __Pyx_XGIVEREF(__pyx_t_32);
        __Pyx_ExceptionReset(__pyx_t_30, __pyx_t_31, __pyx_t_32);
        goto __pyx_L23_error;
        __pyx_L114_try_end:;
      }

      /* "pyogrio/_io.pyx":3223
 *                 ) from None
 * 
 *             OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *             ogr_feature = NULL
 * 
*/
      OGR_F_Destroy(__pyx_v_ogr_feature);

      /* "pyogrio/_io.pyx":3224
 * 
 *             OGR_F_Destroy(ogr_feature)
 *             ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *         if supports_transactions:
*/
      __pyx_v_ogr_feature = NULL;
    }

    /* "pyogrio/_io.pyx":3226
 *             ogr_feature = NULL
 * 
 *         if supports_transactions:             # <<<<<<<<<<<<<<
 *             commit_transaction(ogr_dataset)
 * 
*/
    __pyx_t_7 = (__pyx_v_supports_transactions != 0);
    if (__pyx_t_7) {

      /* "pyogrio/_io.pyx":3227
 * 
 *         if supports_transactions:
 *             commit_transaction(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *         log.info(f"Created {num_records:,} records")
*/
      __pyx_t_8 = __pyx_f_7pyogrio_3_io_commit_transaction(__pyx_v_ogr_dataset); if (unlikely(__pyx_t_8 == ((int)1))) __PYX_ERR(0, 3227, __pyx_L23_error)

      /* "pyogrio/_io.pyx":3226
 *             ogr_feature = NULL
 * 
 *         if supports_transactions:             # <<<<<<<<<<<<<<
 *             commit_transaction(ogr_dataset)
 * 
*/
    }

    /* "pyogrio/_io.pyx":3229
 *             commit_transaction(ogr_dataset)
 * 
 *         log.info(f"Created {num_records:,} records")             # <<<<<<<<<<<<<<
 * 
 *         # close dataset to force driver to flush data
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_25, __pyx_mstate_global->__pyx_n_u_log); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3229, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_25);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_25, __pyx_mstate_global->__pyx_n_u_info); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3229, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
    __pyx_t_25 = __Pyx_PyLong_From_int(__pyx_v_num_records); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3229, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_25);
    __pyx_t_15 = __Pyx_PyObject_Format(__pyx_t_25, __pyx_mstate_global->__pyx_kp_u__11); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3229, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
    __pyx_t_27[0] = __pyx_mstate_global->__pyx_kp_u_Created;
    __pyx_t_27[1] = __pyx_t_15;
    __pyx_t_27[2] = __pyx_mstate_global->__pyx_kp_u_records;
    __pyx_t_25 = __Pyx_PyUnicode_Join(__pyx_t_27, 3, 8 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15));
    if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3229, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_25);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_25};
      __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3229, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_14);
    }
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

    /* "pyogrio/_io.pyx":3232
 * 
 *         # close dataset to force driver to flush data
 *         exc = ogr_close(ogr_dataset)             # <<<<<<<<<<<<<<
 *         ogr_dataset = NULL
 *         if exc:
*/
    __pyx_t_14 = __pyx_f_7pyogrio_3_io_ogr_close(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3232, __pyx_L23_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_v_exc = __pyx_t_14;
    __pyx_t_14 = 0;

    /* "pyogrio/_io.pyx":3233
 *         # close dataset to force driver to flush data
 *         exc = ogr_close(ogr_dataset)
 *         ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *         if exc:
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")
*/
    __pyx_v_ogr_dataset = NULL;

    /* "pyogrio/_io.pyx":3234
 *         exc = ogr_close(ogr_dataset)
 *         ogr_dataset = NULL
 *         if exc:             # <<<<<<<<<<<<<<
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")
 * 
*/
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_exc); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 3234, __pyx_L23_error)
    if (unlikely(__pyx_t_7)) {

      /* "pyogrio/_io.pyx":3235
 *         ogr_dataset = NULL
 *         if exc:
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")             # <<<<<<<<<<<<<<
 * 
 *         # copy in-memory file back to path_or_fp object
*/
      __pyx_t_4 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_25, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3235, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_25);
      __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_v_path, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3235, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_exc, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 3235, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_15);
      __pyx_t_36[0] = __pyx_mstate_global->__pyx_kp_u_Failed_to_write_features_to_data;
      __pyx_t_36[1] = __pyx_t_5;
      __pyx_t_36[2] = __pyx_mstate_global->__pyx_kp_u__14;
      __pyx_t_36[3] = __pyx_t_15;
      __pyx_t_50 = __Pyx_PyUnicode_Join(__pyx_t_36, 4, 36 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5) + 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15));
      if (unlikely(!__pyx_t_50)) __PYX_ERR(0, 3235, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_50);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_25))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_25);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_25);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_25, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_50};
        __pyx_t_14 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_25, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_50); __pyx_t_50 = 0;
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3235, __pyx_L23_error)
        __Pyx_GOTREF(__pyx_t_14);
      }
      __Pyx_Raise(__pyx_t_14, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __PYX_ERR(0, 3235, __pyx_L23_error)

      /* "pyogrio/_io.pyx":3234
 *         exc = ogr_close(ogr_dataset)
 *         ogr_dataset = NULL
 *         if exc:             # <<<<<<<<<<<<<<
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")
 * 
*/
    }

    /* "pyogrio/_io.pyx":3238
 * 
 *         # copy in-memory file back to path_or_fp object
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             read_vsimem_to_buffer(path, path_or_fp)
 * 
*/
    if (__pyx_v_use_tmp_vsimem) {

      /* "pyogrio/_io.pyx":3239
 *         # copy in-memory file back to path_or_fp object
 *         if use_tmp_vsimem:
 *             read_vsimem_to_buffer(path, path_or_fp)             # <<<<<<<<<<<<<<
 * 
 *     finally:
*/
      __pyx_t_14 = __pyx_v_path;
      __Pyx_INCREF(__pyx_t_14);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_14))||((__pyx_t_14) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_14))) __PYX_ERR(0, 3239, __pyx_L23_error)
      __pyx_t_25 = __pyx_f_7pyogrio_4_vsi_read_vsimem_to_buffer(((PyObject*)__pyx_t_14), __pyx_v_path_or_fp); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3239, __pyx_L23_error)
      __Pyx_GOTREF(__pyx_t_25);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;

      /* "pyogrio/_io.pyx":3238
 * 
 *         # copy in-memory file back to path_or_fp object
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             read_vsimem_to_buffer(path, path_or_fp)
 * 
*/
    }
  }

  /* "pyogrio/_io.pyx":3245
 *         # make sure that all objects allocated above are released if exceptions
 *         # are raised, and the dataset is closed
 *         if ogr_fielddef != NULL:             # <<<<<<<<<<<<<<
 *             OGR_Fld_Destroy(ogr_fielddef)
 *             ogr_fielddef = NULL
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_7 = (__pyx_v_ogr_fielddef != NULL);
      if (__pyx_t_7) {

        /* "pyogrio/_io.pyx":3246
 *         # are raised, and the dataset is closed
 *         if ogr_fielddef != NULL:
 *             OGR_Fld_Destroy(ogr_fielddef)             # <<<<<<<<<<<<<<
 *             ogr_fielddef = NULL
 * 
*/
        OGR_Fld_Destroy(__pyx_v_ogr_fielddef);

        /* "pyogrio/_io.pyx":3247
 *         if ogr_fielddef != NULL:
 *             OGR_Fld_Destroy(ogr_fielddef)
 *             ogr_fielddef = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_feature != NULL:
*/
        __pyx_v_ogr_fielddef = NULL;

        /* "pyogrio/_io.pyx":3245
 *         # make sure that all objects allocated above are released if exceptions
 *         # are raised, and the dataset is closed
 *         if ogr_fielddef != NULL:             # <<<<<<<<<<<<<<
 *             OGR_Fld_Destroy(ogr_fielddef)
 *             ogr_fielddef = NULL
*/
      }

      /* "pyogrio/_io.pyx":3249
 *             ogr_fielddef = NULL
 * 
 *         if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *             OGR_F_Destroy(ogr_feature)
 *             ogr_feature = NULL
*/
      __pyx_t_7 = (__pyx_v_ogr_feature != NULL);
      if (__pyx_t_7) {

        /* "pyogrio/_io.pyx":3250
 * 
 *         if ogr_feature != NULL:
 *             OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *             ogr_feature = NULL
 * 
*/
        OGR_F_Destroy(__pyx_v_ogr_feature);

        /* "pyogrio/_io.pyx":3251
 *         if ogr_feature != NULL:
 *             OGR_F_Destroy(ogr_feature)
 *             ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_geometry != NULL:
*/
        __pyx_v_ogr_feature = NULL;

        /* "pyogrio/_io.pyx":3249
 *             ogr_fielddef = NULL
 * 
 *         if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *             OGR_F_Destroy(ogr_feature)
 *             ogr_feature = NULL
*/
      }

      /* "pyogrio/_io.pyx":3253
 *             ogr_feature = NULL
 * 
 *         if ogr_geometry != NULL:             # <<<<<<<<<<<<<<
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *             ogr_geometry = NULL
*/
      __pyx_t_7 = (__pyx_v_ogr_geometry != NULL);
      if (__pyx_t_7) {

        /* "pyogrio/_io.pyx":3254
 * 
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)             # <<<<<<<<<<<<<<
 *             ogr_geometry = NULL
 * 
*/
        OGR_G_DestroyGeometry(__pyx_v_ogr_geometry);

        /* "pyogrio/_io.pyx":3255
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *             ogr_geometry = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
        __pyx_v_ogr_geometry = NULL;

        /* "pyogrio/_io.pyx":3253
 *             ogr_feature = NULL
 * 
 *         if ogr_geometry != NULL:             # <<<<<<<<<<<<<<
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *             ogr_geometry = NULL
*/
      }

      /* "pyogrio/_io.pyx":3257
 *             ogr_geometry = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
      __pyx_t_7 = (__pyx_v_ogr_dataset != NULL);
      if (__pyx_t_7) {

        /* "pyogrio/_io.pyx":3258
 * 
 *         if ogr_dataset != NULL:
 *             ogr_close(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
        __pyx_t_25 = __pyx_f_7pyogrio_3_io_ogr_close(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3258, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_25);
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;

        /* "pyogrio/_io.pyx":3257
 *             ogr_geometry = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
      }

      /* "pyogrio/_io.pyx":3260
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      if (__pyx_v_use_tmp_vsimem) {

        /* "pyogrio/_io.pyx":3261
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 * 
*/
        __pyx_t_25 = __pyx_v_path;
        __Pyx_INCREF(__pyx_t_25);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_25))||((__pyx_t_25) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_25))) __PYX_ERR(0, 3261, __pyx_L1_error)
        __pyx_t_14 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_25), 0); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3261, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

        /* "pyogrio/_io.pyx":3260
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      }
      goto __pyx_L24;
    }
    __pyx_L23_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_32 = 0; __pyx_t_31 = 0; __pyx_t_30 = 0; __pyx_t_54 = 0; __pyx_t_53 = 0; __pyx_t_52 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_25); __pyx_t_25 = 0;
      __Pyx_XDECREF(__pyx_t_26); __pyx_t_26 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_50); __pyx_t_50 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_54, &__pyx_t_53, &__pyx_t_52);
      if ( unlikely(__Pyx_GetException(&__pyx_t_32, &__pyx_t_31, &__pyx_t_30) < 0)) __Pyx_ErrFetch(&__pyx_t_32, &__pyx_t_31, &__pyx_t_30);
      __Pyx_XGOTREF(__pyx_t_32);
      __Pyx_XGOTREF(__pyx_t_31);
      __Pyx_XGOTREF(__pyx_t_30);
      __Pyx_XGOTREF(__pyx_t_54);
      __Pyx_XGOTREF(__pyx_t_53);
      __Pyx_XGOTREF(__pyx_t_52);
      __pyx_t_8 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_55 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":3245
 *         # make sure that all objects allocated above are released if exceptions
 *         # are raised, and the dataset is closed
 *         if ogr_fielddef != NULL:             # <<<<<<<<<<<<<<
 *             OGR_Fld_Destroy(ogr_fielddef)
 *             ogr_fielddef = NULL
*/
        __pyx_t_7 = (__pyx_v_ogr_fielddef != NULL);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3246
 *         # are raised, and the dataset is closed
 *         if ogr_fielddef != NULL:
 *             OGR_Fld_Destroy(ogr_fielddef)             # <<<<<<<<<<<<<<
 *             ogr_fielddef = NULL
 * 
*/
          OGR_Fld_Destroy(__pyx_v_ogr_fielddef);

          /* "pyogrio/_io.pyx":3247
 *         if ogr_fielddef != NULL:
 *             OGR_Fld_Destroy(ogr_fielddef)
 *             ogr_fielddef = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_feature != NULL:
*/
          __pyx_v_ogr_fielddef = NULL;

          /* "pyogrio/_io.pyx":3245
 *         # make sure that all objects allocated above are released if exceptions
 *         # are raised, and the dataset is closed
 *         if ogr_fielddef != NULL:             # <<<<<<<<<<<<<<
 *             OGR_Fld_Destroy(ogr_fielddef)
 *             ogr_fielddef = NULL
*/
        }

        /* "pyogrio/_io.pyx":3249
 *             ogr_fielddef = NULL
 * 
 *         if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *             OGR_F_Destroy(ogr_feature)
 *             ogr_feature = NULL
*/
        __pyx_t_7 = (__pyx_v_ogr_feature != NULL);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3250
 * 
 *         if ogr_feature != NULL:
 *             OGR_F_Destroy(ogr_feature)             # <<<<<<<<<<<<<<
 *             ogr_feature = NULL
 * 
*/
          OGR_F_Destroy(__pyx_v_ogr_feature);

          /* "pyogrio/_io.pyx":3251
 *         if ogr_feature != NULL:
 *             OGR_F_Destroy(ogr_feature)
 *             ogr_feature = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_geometry != NULL:
*/
          __pyx_v_ogr_feature = NULL;

          /* "pyogrio/_io.pyx":3249
 *             ogr_fielddef = NULL
 * 
 *         if ogr_feature != NULL:             # <<<<<<<<<<<<<<
 *             OGR_F_Destroy(ogr_feature)
 *             ogr_feature = NULL
*/
        }

        /* "pyogrio/_io.pyx":3253
 *             ogr_feature = NULL
 * 
 *         if ogr_geometry != NULL:             # <<<<<<<<<<<<<<
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *             ogr_geometry = NULL
*/
        __pyx_t_7 = (__pyx_v_ogr_geometry != NULL);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3254
 * 
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)             # <<<<<<<<<<<<<<
 *             ogr_geometry = NULL
 * 
*/
          OGR_G_DestroyGeometry(__pyx_v_ogr_geometry);

          /* "pyogrio/_io.pyx":3255
 *         if ogr_geometry != NULL:
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *             ogr_geometry = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
          __pyx_v_ogr_geometry = NULL;

          /* "pyogrio/_io.pyx":3253
 *             ogr_feature = NULL
 * 
 *         if ogr_geometry != NULL:             # <<<<<<<<<<<<<<
 *             OGR_G_DestroyGeometry(ogr_geometry)
 *             ogr_geometry = NULL
*/
        }

        /* "pyogrio/_io.pyx":3257
 *             ogr_geometry = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
        __pyx_t_7 = (__pyx_v_ogr_dataset != NULL);
        if (__pyx_t_7) {

          /* "pyogrio/_io.pyx":3258
 * 
 *         if ogr_dataset != NULL:
 *             ogr_close(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
          __pyx_t_14 = __pyx_f_7pyogrio_3_io_ogr_close(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3258, __pyx_L135_error)
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

          /* "pyogrio/_io.pyx":3257
 *             ogr_geometry = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
        }

        /* "pyogrio/_io.pyx":3260
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        if (__pyx_v_use_tmp_vsimem) {

          /* "pyogrio/_io.pyx":3261
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 * 
*/
          if (unlikely(!__pyx_v_path)) { __Pyx_RaiseUnboundLocalError("path"); __PYX_ERR(0, 3261, __pyx_L135_error) }
          __pyx_t_14 = __pyx_v_path;
          __Pyx_INCREF(__pyx_t_14);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_14))||((__pyx_t_14) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_14))) __PYX_ERR(0, 3261, __pyx_L135_error)
          __pyx_t_25 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_14), 0); if (unlikely(!__pyx_t_25)) __PYX_ERR(0, 3261, __pyx_L135_error)
          __Pyx_GOTREF(__pyx_t_25);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_25); __pyx_t_25 = 0;

          /* "pyogrio/_io.pyx":3260
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_54);
      __Pyx_XGIVEREF(__pyx_t_53);
      __Pyx_XGIVEREF(__pyx_t_52);
      __Pyx_ExceptionReset(__pyx_t_54, __pyx_t_53, __pyx_t_52);
      __Pyx_XGIVEREF(__pyx_t_32);
      __Pyx_XGIVEREF(__pyx_t_31);
      __Pyx_XGIVEREF(__pyx_t_30);
      __Pyx_ErrRestore(__pyx_t_32, __pyx_t_31, __pyx_t_30);
      __pyx_t_32 = 0; __pyx_t_31 = 0; __pyx_t_30 = 0; __pyx_t_54 = 0; __pyx_t_53 = 0; __pyx_t_52 = 0;
      __pyx_lineno = __pyx_t_8; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_55;
      goto __pyx_L1_error;
      __pyx_L135_error:;
      __Pyx_XGIVEREF(__pyx_t_54);
      __Pyx_XGIVEREF(__pyx_t_53);
      __Pyx_XGIVEREF(__pyx_t_52);
      __Pyx_ExceptionReset(__pyx_t_54, __pyx_t_53, __pyx_t_52);
      __Pyx_XDECREF(__pyx_t_32); __pyx_t_32 = 0;
      __Pyx_XDECREF(__pyx_t_31); __pyx_t_31 = 0;
      __Pyx_XDECREF(__pyx_t_30); __pyx_t_30 = 0;
      __pyx_t_54 = 0; __pyx_t_53 = 0; __pyx_t_52 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L24:;
  }

  /* "pyogrio/_io.pyx":2901
 * 
 * # TODO: set geometry and field data as memory views?
 * def ogr_write(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_25);
  __Pyx_XDECREF(__pyx_t_26);
  __Pyx_XDECREF(__pyx_t_50);
  __Pyx_AddTraceback("pyogrio._io.ogr_write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_field_indexes);
  __Pyx_XDECREF(__pyx_v_field_ogr_types);
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_layer_created);
  __Pyx_XDECREF(__pyx_v_field_types);
  __Pyx_XDECREF(__pyx_v_field_ogr_type);
  __Pyx_XDECREF(__pyx_v_field_subtype);
  __Pyx_XDECREF(__pyx_v_width);
  __Pyx_XDECREF(__pyx_v__precision);
  __Pyx_XDECREF(__pyx_v_name_b);
  __Pyx_XDECREF(__pyx_v_wkb);
  __Pyx_XDECREF(__pyx_v_field_value);
  __Pyx_XDECREF(__pyx_v_mask);
  __Pyx_XDECREF(__pyx_v_value_b);
  __Pyx_XDECREF(__pyx_v_datetime_value);
  __Pyx_XDECREF(__pyx_v_tz_array);
  __Pyx_XDECREF(__pyx_v_gdal_tz);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_9genexpr13__pyx_v_field);
  __Pyx_XDECREF(__pyx_v_field_mask);
  __Pyx_XDECREF(__pyx_v_geometry_type);
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XDECREF(__pyx_v_gdal_tz_offsets);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":3264
 * 
 * 
 * def ogr_write_arrow(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/

/* Python wrapper */
static PyObject *__pyx_pw_7pyogrio_3_io_14ogr_write_arrow(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_7pyogrio_3_io_13ogr_write_arrow, "Write the data to a file, using the GDAL Arrow Interface.");
static PyMethodDef __pyx_mdef_7pyogrio_3_io_14ogr_write_arrow = {"ogr_write_arrow", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_7pyogrio_3_io_14ogr_write_arrow, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_7pyogrio_3_io_13ogr_write_arrow};
static PyObject *__pyx_pw_7pyogrio_3_io_14ogr_write_arrow(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path_or_fp = 0;
  PyObject *__pyx_v_layer = 0;
  PyObject *__pyx_v_driver = 0;
  PyObject *__pyx_v_arrow_obj = 0;
  PyObject *__pyx_v_crs = 0;
  PyObject *__pyx_v_geometry_type = 0;
  PyObject *__pyx_v_geometry_name = 0;
  PyObject *__pyx_v_encoding = 0;
  PyObject *__pyx_v_dataset_kwargs = 0;
  PyObject *__pyx_v_layer_kwargs = 0;
  int __pyx_v_append;
  PyObject *__pyx_v_dataset_metadata = 0;
  PyObject *__pyx_v_layer_metadata = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[13] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ogr_write_arrow (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path_or_fp,&__pyx_mstate_global->__pyx_n_u_layer,&__pyx_mstate_global->__pyx_n_u_driver,&__pyx_mstate_global->__pyx_n_u_arrow_obj,&__pyx_mstate_global->__pyx_n_u_crs,&__pyx_mstate_global->__pyx_n_u_geometry_type,&__pyx_mstate_global->__pyx_n_u_geometry_name,&__pyx_mstate_global->__pyx_n_u_encoding,&__pyx_mstate_global->__pyx_n_u_dataset_kwargs,&__pyx_mstate_global->__pyx_n_u_layer_kwargs,&__pyx_mstate_global->__pyx_n_u_append,&__pyx_mstate_global->__pyx_n_u_dataset_metadata,&__pyx_mstate_global->__pyx_n_u_layer_metadata,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 3264, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "ogr_write_arrow", 0) < (0)) __PYX_ERR(0, 3264, __pyx_L3_error)

      /* "pyogrio/_io.pyx":3276
 *     object layer_kwargs,
 *     bint append=False,
 *     dataset_metadata=None,             # <<<<<<<<<<<<<<
 *     layer_metadata=None,
 * ):
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":3277
 *     bint append=False,
 *     dataset_metadata=None,
 *     layer_metadata=None,             # <<<<<<<<<<<<<<
 * ):
 *     """Write the data to a file, using the GDAL Arrow Interface."""
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 10; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("ogr_write_arrow", 0, 10, 13, i); __PYX_ERR(0, 3264, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case 13:
        values[12] = __Pyx_ArgRef_FASTCALL(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_FASTCALL(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_FASTCALL(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 3264, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_FASTCALL(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[8] = __Pyx_ArgRef_FASTCALL(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[7] = __Pyx_ArgRef_FASTCALL(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 3264, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 3264, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "pyogrio/_io.pyx":3276
 *     object layer_kwargs,
 *     bint append=False,
 *     dataset_metadata=None,             # <<<<<<<<<<<<<<
 *     layer_metadata=None,
 * ):
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "pyogrio/_io.pyx":3277
 *     bint append=False,
 *     dataset_metadata=None,
 *     layer_metadata=None,             # <<<<<<<<<<<<<<
 * ):
 *     """Write the data to a file, using the GDAL Arrow Interface."""
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_path_or_fp = values[0];
    __pyx_v_layer = ((PyObject*)values[1]);
    __pyx_v_driver = ((PyObject*)values[2]);
    __pyx_v_arrow_obj = values[3];
    __pyx_v_crs = ((PyObject*)values[4]);
    __pyx_v_geometry_type = ((PyObject*)values[5]);
    __pyx_v_geometry_name = ((PyObject*)values[6]);
    __pyx_v_encoding = ((PyObject*)values[7]);
    __pyx_v_dataset_kwargs = values[8];
    __pyx_v_layer_kwargs = values[9];
    if (values[10]) {
      __pyx_v_append = __Pyx_PyObject_IsTrue(values[10]); if (unlikely((__pyx_v_append == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3275, __pyx_L3_error)
    } else {

      /* "pyogrio/_io.pyx":3275
 *     object dataset_kwargs,
 *     object layer_kwargs,
 *     bint append=False,             # <<<<<<<<<<<<<<
 *     dataset_metadata=None,
 *     layer_metadata=None,
*/
      __pyx_v_append = ((int)((int)0));
    }
    __pyx_v_dataset_metadata = values[11];
    __pyx_v_layer_metadata = values[12];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ogr_write_arrow", 0, 10, 13, __pyx_nargs); __PYX_ERR(0, 3264, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("pyogrio._io.ogr_write_arrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_layer), (&PyUnicode_Type), 1, "layer", 1))) __PYX_ERR(0, 3266, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_driver), (&PyUnicode_Type), 1, "driver", 1))) __PYX_ERR(0, 3267, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_crs), (&PyUnicode_Type), 1, "crs", 1))) __PYX_ERR(0, 3269, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_geometry_type), (&PyUnicode_Type), 1, "geometry_type", 1))) __PYX_ERR(0, 3270, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_geometry_name), (&PyUnicode_Type), 1, "geometry_name", 1))) __PYX_ERR(0, 3271, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_encoding), (&PyUnicode_Type), 1, "encoding", 1))) __PYX_ERR(0, 3272, __pyx_L1_error)
  __pyx_r = __pyx_pf_7pyogrio_3_io_13ogr_write_arrow(__pyx_self, __pyx_v_path_or_fp, __pyx_v_layer, __pyx_v_driver, __pyx_v_arrow_obj, __pyx_v_crs, __pyx_v_geometry_type, __pyx_v_geometry_name, __pyx_v_encoding, __pyx_v_dataset_kwargs, __pyx_v_layer_kwargs, __pyx_v_append, __pyx_v_dataset_metadata, __pyx_v_layer_metadata);

  /* "pyogrio/_io.pyx":3264
 * 
 * 
 * def ogr_write_arrow(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7pyogrio_3_io_13ogr_write_arrow(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path_or_fp, PyObject *__pyx_v_layer, PyObject *__pyx_v_driver, PyObject *__pyx_v_arrow_obj, PyObject *__pyx_v_crs, PyObject *__pyx_v_geometry_type, PyObject *__pyx_v_geometry_name, PyObject *__pyx_v_encoding, PyObject *__pyx_v_dataset_kwargs, PyObject *__pyx_v_layer_kwargs, int __pyx_v_append, PyObject *__pyx_v_dataset_metadata, PyObject *__pyx_v_layer_metadata) {
  OGRDataSourceH __pyx_v_ogr_dataset;
  OGRLayerH __pyx_v_ogr_layer;
  char **__pyx_v_options;
  int __pyx_v_use_tmp_vsimem;
  struct ArrowArrayStream *__pyx_v_stream;
  struct ArrowSchema __pyx_v_schema;
  struct ArrowArray __pyx_v_array;
  PyObject *__pyx_v_path = NULL;
  PyObject *__pyx_v_layer_created = NULL;
  PyObject *__pyx_v_opts = NULL;
  PyObject *__pyx_v_stream_capsule = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_gdal_msg = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  size_t __pyx_t_6;
  char **__pyx_t_7;
  void *__pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11[4];
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ogr_write_arrow", 0);

  /* "pyogrio/_io.pyx":3283
 *         raise RuntimeError("Need GDAL>=3.8 for Arrow write support")
 * 
 *     cdef OGRDataSourceH ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef char **options = NULL
*/
  __pyx_v_ogr_dataset = NULL;

  /* "pyogrio/_io.pyx":3284
 * 
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL             # <<<<<<<<<<<<<<
 *     cdef char **options = NULL
 *     cdef bint use_tmp_vsimem = False
*/
  __pyx_v_ogr_layer = NULL;

  /* "pyogrio/_io.pyx":3285
 *     cdef OGRDataSourceH ogr_dataset = NULL
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef char **options = NULL             # <<<<<<<<<<<<<<
 *     cdef bint use_tmp_vsimem = False
 *     cdef ArrowArrayStream* stream = NULL
*/
  __pyx_v_options = NULL;

  /* "pyogrio/_io.pyx":3286
 *     cdef OGRLayerH ogr_layer = NULL
 *     cdef char **options = NULL
 *     cdef bint use_tmp_vsimem = False             # <<<<<<<<<<<<<<
 *     cdef ArrowArrayStream* stream = NULL
 *     cdef ArrowSchema schema
*/
  __pyx_v_use_tmp_vsimem = 0;

  /* "pyogrio/_io.pyx":3287
 *     cdef char **options = NULL
 *     cdef bint use_tmp_vsimem = False
 *     cdef ArrowArrayStream* stream = NULL             # <<<<<<<<<<<<<<
 *     cdef ArrowSchema schema
 *     cdef ArrowArray array
*/
  __pyx_v_stream = NULL;

  /* "pyogrio/_io.pyx":3291
 *     cdef ArrowArray array
 * 
 *     schema.release = NULL             # <<<<<<<<<<<<<<
 *     array.release = NULL
 * 
*/
  __pyx_v_schema.release = NULL;

  /* "pyogrio/_io.pyx":3292
 * 
 *     schema.release = NULL
 *     array.release = NULL             # <<<<<<<<<<<<<<
 * 
 *     try:
*/
  __pyx_v_array.release = NULL;

  /* "pyogrio/_io.pyx":3294
 *     array.release = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         # Setup in-memory handler if needed
 *         path, use_tmp_vsimem = get_ogr_vsimem_write_path(path_or_fp, driver)
*/
  /*try:*/ {

    /* "pyogrio/_io.pyx":3296
 *     try:
 *         # Setup in-memory handler if needed
 *         path, use_tmp_vsimem = get_ogr_vsimem_write_path(path_or_fp, driver)             # <<<<<<<<<<<<<<
 * 
 *         layer_created = create_ogr_dataset_layer(
*/
    __pyx_t_1 = __pyx_f_7pyogrio_4_vsi_get_ogr_vsimem_write_path(__pyx_v_path_or_fp, __pyx_v_driver); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3296, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(__pyx_t_1 != Py_None)) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 3296, __pyx_L4_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_3);
      #else
      __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3296, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3296, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_3);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 3296, __pyx_L4_error)
    }
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 3296, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_path = __pyx_t_2;
    __pyx_t_2 = 0;
    __pyx_v_use_tmp_vsimem = __pyx_t_4;

    /* "pyogrio/_io.pyx":3299
 * 
 *         layer_created = create_ogr_dataset_layer(
 *             path, use_tmp_vsimem, layer, driver, crs, geometry_type, encoding,             # <<<<<<<<<<<<<<
 *             dataset_kwargs, layer_kwargs, append,
 *             dataset_metadata, layer_metadata,
*/
    __pyx_t_1 = __pyx_v_path;
    __Pyx_INCREF(__pyx_t_1);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 3299, __pyx_L4_error)

    /* "pyogrio/_io.pyx":3298
 *         path, use_tmp_vsimem = get_ogr_vsimem_write_path(path_or_fp, driver)
 * 
 *         layer_created = create_ogr_dataset_layer(             # <<<<<<<<<<<<<<
 *             path, use_tmp_vsimem, layer, driver, crs, geometry_type, encoding,
 *             dataset_kwargs, layer_kwargs, append,
*/
    __pyx_t_3 = __pyx_f_7pyogrio_3_io_create_ogr_dataset_layer(((PyObject*)__pyx_t_1), __pyx_v_use_tmp_vsimem, __pyx_v_layer, __pyx_v_driver, __pyx_v_crs, __pyx_v_geometry_type, __pyx_v_encoding, __pyx_v_dataset_kwargs, __pyx_v_layer_kwargs, __pyx_v_append, __pyx_v_dataset_metadata, __pyx_v_layer_metadata, (&__pyx_v_ogr_dataset), (&__pyx_v_ogr_layer)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3298, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_layer_created = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3309
 *         # encoding
 *         if (
 *             encoding and encoding.replace("-", "").upper() != "UTF8"             # <<<<<<<<<<<<<<
 *             and driver != "ESRI Shapefile"
 *         ):
*/
    if (__pyx_v_encoding == Py_None) __pyx_t_5 = 0;
    else
    {
      Py_ssize_t __pyx_temp = __Pyx_PyUnicode_IS_TRUE(__pyx_v_encoding);
      if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 3309, __pyx_L4_error)
      __pyx_t_5 = (__pyx_temp != 0);
    }

    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L7_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":3310
 *         if (
 *             encoding and encoding.replace("-", "").upper() != "UTF8"
 *             and driver != "ESRI Shapefile"             # <<<<<<<<<<<<<<
 *         ):
 *             raise ValueError(
*/
    if (unlikely(__pyx_v_encoding == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "replace");
      __PYX_ERR(0, 3309, __pyx_L4_error)
    }

    /* "pyogrio/_io.pyx":3309
 *         # encoding
 *         if (
 *             encoding and encoding.replace("-", "").upper() != "UTF8"             # <<<<<<<<<<<<<<
 *             and driver != "ESRI Shapefile"
 *         ):
*/
    __pyx_t_2 = PyUnicode_Replace(__pyx_v_encoding, __pyx_mstate_global->__pyx_kp_u__10, __pyx_mstate_global->__pyx_kp_u__9, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3309, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_upper, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3309, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_UTF8, Py_NE)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 3309, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L7_bool_binop_done;
    }

    /* "pyogrio/_io.pyx":3310
 *         if (
 *             encoding and encoding.replace("-", "").upper() != "UTF8"
 *             and driver != "ESRI Shapefile"             # <<<<<<<<<<<<<<
 *         ):
 *             raise ValueError(
*/
    __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_driver, __pyx_mstate_global->__pyx_kp_u_ESRI_Shapefile, Py_NE)); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 3310, __pyx_L4_error)
    __pyx_t_4 = __pyx_t_5;
    __pyx_L7_bool_binop_done:;

    /* "pyogrio/_io.pyx":3308
 *         # during dataset creation and GDAL auto-translates from UTF-8 values to that
 *         # encoding
 *         if (             # <<<<<<<<<<<<<<
 *             encoding and encoding.replace("-", "").upper() != "UTF8"
 *             and driver != "ESRI Shapefile"
*/
    if (unlikely(__pyx_t_4)) {

      /* "pyogrio/_io.pyx":3312
 *             and driver != "ESRI Shapefile"
 *         ):
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "non-UTF-8 encoding is not supported for Arrow; use the non-Arrow "
 *                 "interface instead"
*/
      __pyx_t_2 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_non_UTF_8_encoding_is_not_suppor};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3312, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 3312, __pyx_L4_error)

      /* "pyogrio/_io.pyx":3308
 *         # during dataset creation and GDAL auto-translates from UTF-8 values to that
 *         # encoding
 *         if (             # <<<<<<<<<<<<<<
 *             encoding and encoding.replace("-", "").upper() != "UTF8"
 *             and driver != "ESRI Shapefile"
*/
    }

    /* "pyogrio/_io.pyx":3317
 *             )
 * 
 *         if geometry_name:             # <<<<<<<<<<<<<<
 *             opts = {"GEOMETRY_NAME": geometry_name}
 *         else:
*/
    if (__pyx_v_geometry_name == Py_None) __pyx_t_4 = 0;
    else
    {
      Py_ssize_t __pyx_temp = __Pyx_PyUnicode_IS_TRUE(__pyx_v_geometry_name);
      if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 3317, __pyx_L4_error)
      __pyx_t_4 = (__pyx_temp != 0);
    }

    if (__pyx_t_4) {

      /* "pyogrio/_io.pyx":3318
 * 
 *         if geometry_name:
 *             opts = {"GEOMETRY_NAME": geometry_name}             # <<<<<<<<<<<<<<
 *         else:
 *             opts = {}
*/
      __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3318, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (PyDict_SetItem(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_GEOMETRY_NAME, __pyx_v_geometry_name) < (0)) __PYX_ERR(0, 3318, __pyx_L4_error)
      __pyx_v_opts = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;

      /* "pyogrio/_io.pyx":3317
 *             )
 * 
 *         if geometry_name:             # <<<<<<<<<<<<<<
 *             opts = {"GEOMETRY_NAME": geometry_name}
 *         else:
*/
      goto __pyx_L10;
    }

    /* "pyogrio/_io.pyx":3320
 *             opts = {"GEOMETRY_NAME": geometry_name}
 *         else:
 *             opts = {}             # <<<<<<<<<<<<<<
 * 
 *         options = dict_to_options(opts)
*/
    /*else*/ {
      __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3320, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_v_opts = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;
    }
    __pyx_L10:;

    /* "pyogrio/_io.pyx":3322
 *             opts = {}
 * 
 *         options = dict_to_options(opts)             # <<<<<<<<<<<<<<
 * 
 *         stream_capsule = arrow_obj.__arrow_c_stream__()
*/
    __pyx_t_7 = __pyx_f_7pyogrio_3_io_dict_to_options(__pyx_v_opts); if (unlikely(__pyx_t_7 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 3322, __pyx_L4_error)
    __pyx_v_options = __pyx_t_7;

    /* "pyogrio/_io.pyx":3324
 *         options = dict_to_options(opts)
 * 
 *         stream_capsule = arrow_obj.__arrow_c_stream__()             # <<<<<<<<<<<<<<
 *         stream = <ArrowArrayStream*>PyCapsule_GetPointer(
 *             stream_capsule, "arrow_array_stream"
*/
    __pyx_t_2 = __pyx_v_arrow_obj;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_arrow_c_stream, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3324, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_v_stream_capsule = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3325
 * 
 *         stream_capsule = arrow_obj.__arrow_c_stream__()
 *         stream = <ArrowArrayStream*>PyCapsule_GetPointer(             # <<<<<<<<<<<<<<
 *             stream_capsule, "arrow_array_stream"
 *         )
*/
    __pyx_t_8 = PyCapsule_GetPointer(__pyx_v_stream_capsule, __pyx_k_arrow_array_stream); if (unlikely(__pyx_t_8 == ((void *)NULL) && PyErr_Occurred())) __PYX_ERR(0, 3325, __pyx_L4_error)
    __pyx_v_stream = ((struct ArrowArrayStream *)__pyx_t_8);

    /* "pyogrio/_io.pyx":3329
 *         )
 * 
 *         if stream == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Could not extract valid Arrow array stream.")
 * 
*/
    __pyx_t_4 = (__pyx_v_stream == NULL);
    if (unlikely(__pyx_t_4)) {

      /* "pyogrio/_io.pyx":3330
 * 
 *         if stream == NULL:
 *             raise RuntimeError("Could not extract valid Arrow array stream.")             # <<<<<<<<<<<<<<
 * 
 *         if stream.release == NULL:
*/
      __pyx_t_2 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Could_not_extract_valid_Arrow_ar};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3330, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 3330, __pyx_L4_error)

      /* "pyogrio/_io.pyx":3329
 *         )
 * 
 *         if stream == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Could not extract valid Arrow array stream.")
 * 
*/
    }

    /* "pyogrio/_io.pyx":3332
 *             raise RuntimeError("Could not extract valid Arrow array stream.")
 * 
 *         if stream.release == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Arrow array stream was already released.")
 * 
*/
    __pyx_t_4 = (__pyx_v_stream->release == NULL);
    if (unlikely(__pyx_t_4)) {

      /* "pyogrio/_io.pyx":3333
 * 
 *         if stream.release == NULL:
 *             raise RuntimeError("Arrow array stream was already released.")             # <<<<<<<<<<<<<<
 * 
 *         if stream.get_schema(stream, &schema) != 0:
*/
      __pyx_t_2 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Arrow_array_stream_was_already_r};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3333, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 3333, __pyx_L4_error)

      /* "pyogrio/_io.pyx":3332
 *             raise RuntimeError("Could not extract valid Arrow array stream.")
 * 
 *         if stream.release == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Arrow array stream was already released.")
 * 
*/
    }

    /* "pyogrio/_io.pyx":3335
 *             raise RuntimeError("Arrow array stream was already released.")
 * 
 *         if stream.get_schema(stream, &schema) != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Could not get Arrow schema from stream.")
 * 
*/
    __pyx_t_4 = (__pyx_v_stream->get_schema(__pyx_v_stream, (&__pyx_v_schema)) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "pyogrio/_io.pyx":3336
 * 
 *         if stream.get_schema(stream, &schema) != 0:
 *             raise RuntimeError("Could not get Arrow schema from stream.")             # <<<<<<<<<<<<<<
 * 
 *         if layer_created:
*/
      __pyx_t_2 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Could_not_get_Arrow_schema_from};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3336, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 3336, __pyx_L4_error)

      /* "pyogrio/_io.pyx":3335
 *             raise RuntimeError("Arrow array stream was already released.")
 * 
 *         if stream.get_schema(stream, &schema) != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Could not get Arrow schema from stream.")
 * 
*/
    }

    /* "pyogrio/_io.pyx":3338
 *             raise RuntimeError("Could not get Arrow schema from stream.")
 * 
 *         if layer_created:             # <<<<<<<<<<<<<<
 *             create_fields_from_arrow_schema(ogr_layer, &schema, options, geometry_name)
 * 
*/
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_layer_created); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 3338, __pyx_L4_error)
    if (__pyx_t_4) {

      /* "pyogrio/_io.pyx":3339
 * 
 *         if layer_created:
 *             create_fields_from_arrow_schema(ogr_layer, &schema, options, geometry_name)             # <<<<<<<<<<<<<<
 * 
 *         while True:
*/
      __pyx_t_3 = __pyx_f_7pyogrio_3_io_create_fields_from_arrow_schema(__pyx_v_ogr_layer, (&__pyx_v_schema), __pyx_v_options, __pyx_v_geometry_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3339, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "pyogrio/_io.pyx":3338
 *             raise RuntimeError("Could not get Arrow schema from stream.")
 * 
 *         if layer_created:             # <<<<<<<<<<<<<<
 *             create_fields_from_arrow_schema(ogr_layer, &schema, options, geometry_name)
 * 
*/
    }

    /* "pyogrio/_io.pyx":3341
 *             create_fields_from_arrow_schema(ogr_layer, &schema, options, geometry_name)
 * 
 *         while True:             # <<<<<<<<<<<<<<
 *             if stream.get_next(stream, &array) != 0:
 *                 raise RuntimeError("Error while accessing batch from stream.")
*/
    while (1) {

      /* "pyogrio/_io.pyx":3342
 * 
 *         while True:
 *             if stream.get_next(stream, &array) != 0:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("Error while accessing batch from stream.")
 * 
*/
      __pyx_t_4 = (__pyx_v_stream->get_next(__pyx_v_stream, (&__pyx_v_array)) != 0);
      if (unlikely(__pyx_t_4)) {

        /* "pyogrio/_io.pyx":3343
 *         while True:
 *             if stream.get_next(stream, &array) != 0:
 *                 raise RuntimeError("Error while accessing batch from stream.")             # <<<<<<<<<<<<<<
 * 
 *             # We've reached the end of the stream
*/
        __pyx_t_2 = NULL;
        __pyx_t_6 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_Error_while_accessing_batch_from};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3343, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 3343, __pyx_L4_error)

        /* "pyogrio/_io.pyx":3342
 * 
 *         while True:
 *             if stream.get_next(stream, &array) != 0:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("Error while accessing batch from stream.")
 * 
*/
      }

      /* "pyogrio/_io.pyx":3346
 * 
 *             # We've reached the end of the stream
 *             if array.release == NULL:             # <<<<<<<<<<<<<<
 *                 break
 * 
*/
      __pyx_t_4 = (__pyx_v_array.release == NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":3347
 *             # We've reached the end of the stream
 *             if array.release == NULL:
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             if not OGR_L_WriteArrowBatch(ogr_layer, &schema, &array, options):
*/
        goto __pyx_L16_break;

        /* "pyogrio/_io.pyx":3346
 * 
 *             # We've reached the end of the stream
 *             if array.release == NULL:             # <<<<<<<<<<<<<<
 *                 break
 * 
*/
      }

      /* "pyogrio/_io.pyx":3349
 *                 break
 * 
 *             if not OGR_L_WriteArrowBatch(ogr_layer, &schema, &array, options):             # <<<<<<<<<<<<<<
 *                 exc = check_last_error()
 *                 gdal_msg = f": {str(exc)}" if exc else "."
*/
      __pyx_t_4 = (!OGR_L_WriteArrowBatch(__pyx_v_ogr_layer, (&__pyx_v_schema), (&__pyx_v_array), __pyx_v_options));
      if (unlikely(__pyx_t_4)) {

        /* "pyogrio/_io.pyx":3350
 * 
 *             if not OGR_L_WriteArrowBatch(ogr_layer, &schema, &array, options):
 *                 exc = check_last_error()             # <<<<<<<<<<<<<<
 *                 gdal_msg = f": {str(exc)}" if exc else "."
 *                 raise DataLayerError(
*/
        __pyx_t_3 = __pyx_f_7pyogrio_4_err_check_last_error(); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3350, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_v_exc = __pyx_t_3;
        __pyx_t_3 = 0;

        /* "pyogrio/_io.pyx":3351
 *             if not OGR_L_WriteArrowBatch(ogr_layer, &schema, &array, options):
 *                 exc = check_last_error()
 *                 gdal_msg = f": {str(exc)}" if exc else "."             # <<<<<<<<<<<<<<
 *                 raise DataLayerError(
 *                     f"Error while writing batch to OGR layer{gdal_msg}"
*/
        __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_exc); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 3351, __pyx_L4_error)
        if (__pyx_t_4) {
          __pyx_t_2 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3351, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3351, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_3 = __pyx_t_1;
          __pyx_t_1 = 0;
        } else {
          __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u__2);
          __pyx_t_3 = __pyx_mstate_global->__pyx_kp_u__2;
        }
        __pyx_v_gdal_msg = ((PyObject*)__pyx_t_3);
        __pyx_t_3 = 0;

        /* "pyogrio/_io.pyx":3352
 *                 exc = check_last_error()
 *                 gdal_msg = f": {str(exc)}" if exc else "."
 *                 raise DataLayerError(             # <<<<<<<<<<<<<<
 *                     f"Error while writing batch to OGR layer{gdal_msg}"
 *                 )
*/
        __pyx_t_1 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_DataLayerError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3352, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);

        /* "pyogrio/_io.pyx":3353
 *                 gdal_msg = f": {str(exc)}" if exc else "."
 *                 raise DataLayerError(
 *                     f"Error while writing batch to OGR layer{gdal_msg}"             # <<<<<<<<<<<<<<
 *                 )
 * 
*/
        __pyx_t_9 = __Pyx_PyUnicode_Unicode(__pyx_v_gdal_msg); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 3353, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Error_while_writing_batch_to_OGR, __pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3353, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
          assert(__pyx_t_1);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_10};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3352, __pyx_L4_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 3352, __pyx_L4_error)

        /* "pyogrio/_io.pyx":3349
 *                 break
 * 
 *             if not OGR_L_WriteArrowBatch(ogr_layer, &schema, &array, options):             # <<<<<<<<<<<<<<
 *                 exc = check_last_error()
 *                 gdal_msg = f": {str(exc)}" if exc else "."
*/
      }

      /* "pyogrio/_io.pyx":3356
 *                 )
 * 
 *             if array.release != NULL:             # <<<<<<<<<<<<<<
 *                 array.release(&array)
 * 
*/
      __pyx_t_4 = (__pyx_v_array.release != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":3357
 * 
 *             if array.release != NULL:
 *                 array.release(&array)             # <<<<<<<<<<<<<<
 * 
 *         # close dataset to force driver to flush data
*/
        __pyx_v_array.release((&__pyx_v_array));

        /* "pyogrio/_io.pyx":3356
 *                 )
 * 
 *             if array.release != NULL:             # <<<<<<<<<<<<<<
 *                 array.release(&array)
 * 
*/
      }
    }
    __pyx_L16_break:;

    /* "pyogrio/_io.pyx":3360
 * 
 *         # close dataset to force driver to flush data
 *         exc = ogr_close(ogr_dataset)             # <<<<<<<<<<<<<<
 *         ogr_dataset = NULL
 *         if exc:
*/
    __pyx_t_3 = __pyx_f_7pyogrio_3_io_ogr_close(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3360, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_exc = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3361
 *         # close dataset to force driver to flush data
 *         exc = ogr_close(ogr_dataset)
 *         ogr_dataset = NULL             # <<<<<<<<<<<<<<
 *         if exc:
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")
*/
    __pyx_v_ogr_dataset = NULL;

    /* "pyogrio/_io.pyx":3362
 *         exc = ogr_close(ogr_dataset)
 *         ogr_dataset = NULL
 *         if exc:             # <<<<<<<<<<<<<<
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")
 * 
*/
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_exc); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 3362, __pyx_L4_error)
    if (unlikely(__pyx_t_4)) {

      /* "pyogrio/_io.pyx":3363
 *         ogr_dataset = NULL
 *         if exc:
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")             # <<<<<<<<<<<<<<
 * 
 *         # copy in-memory file back to path_or_fp object
*/
      __pyx_t_2 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_DataSourceError); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3363, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_path, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3363, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = __Pyx_PyObject_FormatSimple(__pyx_v_exc, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 3363, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_11[0] = __pyx_mstate_global->__pyx_kp_u_Failed_to_write_features_to_data;
      __pyx_t_11[1] = __pyx_t_1;
      __pyx_t_11[2] = __pyx_mstate_global->__pyx_kp_u__14;
      __pyx_t_11[3] = __pyx_t_9;
      __pyx_t_12 = __Pyx_PyUnicode_Join(__pyx_t_11, 4, 36 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
      if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 3363, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_10);
        assert(__pyx_t_2);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_12};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3363, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 3363, __pyx_L4_error)

      /* "pyogrio/_io.pyx":3362
 *         exc = ogr_close(ogr_dataset)
 *         ogr_dataset = NULL
 *         if exc:             # <<<<<<<<<<<<<<
 *             raise DataSourceError(f"Failed to write features to dataset {path}; {exc}")
 * 
*/
    }

    /* "pyogrio/_io.pyx":3366
 * 
 *         # copy in-memory file back to path_or_fp object
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             read_vsimem_to_buffer(path, path_or_fp)
 * 
*/
    if (__pyx_v_use_tmp_vsimem) {

      /* "pyogrio/_io.pyx":3367
 *         # copy in-memory file back to path_or_fp object
 *         if use_tmp_vsimem:
 *             read_vsimem_to_buffer(path, path_or_fp)             # <<<<<<<<<<<<<<
 * 
 *     finally:
*/
      __pyx_t_3 = __pyx_v_path;
      __Pyx_INCREF(__pyx_t_3);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 3367, __pyx_L4_error)
      __pyx_t_10 = __pyx_f_7pyogrio_4_vsi_read_vsimem_to_buffer(((PyObject*)__pyx_t_3), __pyx_v_path_or_fp); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3367, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "pyogrio/_io.pyx":3366
 * 
 *         # copy in-memory file back to path_or_fp object
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             read_vsimem_to_buffer(path, path_or_fp)
 * 
*/
    }
  }

  /* "pyogrio/_io.pyx":3370
 * 
 *     finally:
 *         if stream != NULL and stream.release != NULL:             # <<<<<<<<<<<<<<
 *             stream.release(stream)
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_5 = (__pyx_v_stream != NULL);
      if (__pyx_t_5) {
      } else {
        __pyx_t_4 = __pyx_t_5;
        goto __pyx_L24_bool_binop_done;
      }
      __pyx_t_5 = (__pyx_v_stream->release != NULL);
      __pyx_t_4 = __pyx_t_5;
      __pyx_L24_bool_binop_done:;
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":3371
 *     finally:
 *         if stream != NULL and stream.release != NULL:
 *             stream.release(stream)             # <<<<<<<<<<<<<<
 * 
 *         if schema.release != NULL:
*/
        __pyx_v_stream->release(__pyx_v_stream);

        /* "pyogrio/_io.pyx":3370
 * 
 *     finally:
 *         if stream != NULL and stream.release != NULL:             # <<<<<<<<<<<<<<
 *             stream.release(stream)
 * 
*/
      }

      /* "pyogrio/_io.pyx":3373
 *             stream.release(stream)
 * 
 *         if schema.release != NULL:             # <<<<<<<<<<<<<<
 *             schema.release(&schema)
 * 
*/
      __pyx_t_4 = (__pyx_v_schema.release != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":3374
 * 
 *         if schema.release != NULL:
 *             schema.release(&schema)             # <<<<<<<<<<<<<<
 * 
 *         if array.release != NULL:
*/
        __pyx_v_schema.release((&__pyx_v_schema));

        /* "pyogrio/_io.pyx":3373
 *             stream.release(stream)
 * 
 *         if schema.release != NULL:             # <<<<<<<<<<<<<<
 *             schema.release(&schema)
 * 
*/
      }

      /* "pyogrio/_io.pyx":3376
 *             schema.release(&schema)
 * 
 *         if array.release != NULL:             # <<<<<<<<<<<<<<
 *             array.release(&array)
 * 
*/
      __pyx_t_4 = (__pyx_v_array.release != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":3377
 * 
 *         if array.release != NULL:
 *             array.release(&array)             # <<<<<<<<<<<<<<
 * 
 *         if options != NULL:
*/
        __pyx_v_array.release((&__pyx_v_array));

        /* "pyogrio/_io.pyx":3376
 *             schema.release(&schema)
 * 
 *         if array.release != NULL:             # <<<<<<<<<<<<<<
 *             array.release(&array)
 * 
*/
      }

      /* "pyogrio/_io.pyx":3379
 *             array.release(&array)
 * 
 *         if options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(options)
 *             options = NULL
*/
      __pyx_t_4 = (__pyx_v_options != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":3380
 * 
 *         if options != NULL:
 *             CSLDestroy(options)             # <<<<<<<<<<<<<<
 *             options = NULL
 * 
*/
        CSLDestroy(__pyx_v_options);

        /* "pyogrio/_io.pyx":3381
 *         if options != NULL:
 *             CSLDestroy(options)
 *             options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
        __pyx_v_options = NULL;

        /* "pyogrio/_io.pyx":3379
 *             array.release(&array)
 * 
 *         if options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(options)
 *             options = NULL
*/
      }

      /* "pyogrio/_io.pyx":3383
 *             options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
      __pyx_t_4 = (__pyx_v_ogr_dataset != NULL);
      if (__pyx_t_4) {

        /* "pyogrio/_io.pyx":3384
 * 
 *         if ogr_dataset != NULL:
 *             ogr_close(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
        __pyx_t_10 = __pyx_f_7pyogrio_3_io_ogr_close(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3384, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "pyogrio/_io.pyx":3383
 *             options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
      }

      /* "pyogrio/_io.pyx":3386
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      if (__pyx_v_use_tmp_vsimem) {

        /* "pyogrio/_io.pyx":3387
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 * 
*/
        __pyx_t_10 = __pyx_v_path;
        __Pyx_INCREF(__pyx_t_10);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_10))||((__pyx_t_10) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_10))) __PYX_ERR(0, 3387, __pyx_L1_error)
        __pyx_t_3 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_10), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3387, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "pyogrio/_io.pyx":3386
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
      }
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
      if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_21);
      __pyx_t_13 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
      {

        /* "pyogrio/_io.pyx":3370
 * 
 *     finally:
 *         if stream != NULL and stream.release != NULL:             # <<<<<<<<<<<<<<
 *             stream.release(stream)
 * 
*/
        __pyx_t_5 = (__pyx_v_stream != NULL);
        if (__pyx_t_5) {
        } else {
          __pyx_t_4 = __pyx_t_5;
          goto __pyx_L34_bool_binop_done;
        }
        __pyx_t_5 = (__pyx_v_stream->release != NULL);
        __pyx_t_4 = __pyx_t_5;
        __pyx_L34_bool_binop_done:;
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":3371
 *     finally:
 *         if stream != NULL and stream.release != NULL:
 *             stream.release(stream)             # <<<<<<<<<<<<<<
 * 
 *         if schema.release != NULL:
*/
          __pyx_v_stream->release(__pyx_v_stream);

          /* "pyogrio/_io.pyx":3370
 * 
 *     finally:
 *         if stream != NULL and stream.release != NULL:             # <<<<<<<<<<<<<<
 *             stream.release(stream)
 * 
*/
        }

        /* "pyogrio/_io.pyx":3373
 *             stream.release(stream)
 * 
 *         if schema.release != NULL:             # <<<<<<<<<<<<<<
 *             schema.release(&schema)
 * 
*/
        __pyx_t_4 = (__pyx_v_schema.release != NULL);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":3374
 * 
 *         if schema.release != NULL:
 *             schema.release(&schema)             # <<<<<<<<<<<<<<
 * 
 *         if array.release != NULL:
*/
          __pyx_v_schema.release((&__pyx_v_schema));

          /* "pyogrio/_io.pyx":3373
 *             stream.release(stream)
 * 
 *         if schema.release != NULL:             # <<<<<<<<<<<<<<
 *             schema.release(&schema)
 * 
*/
        }

        /* "pyogrio/_io.pyx":3376
 *             schema.release(&schema)
 * 
 *         if array.release != NULL:             # <<<<<<<<<<<<<<
 *             array.release(&array)
 * 
*/
        __pyx_t_4 = (__pyx_v_array.release != NULL);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":3377
 * 
 *         if array.release != NULL:
 *             array.release(&array)             # <<<<<<<<<<<<<<
 * 
 *         if options != NULL:
*/
          __pyx_v_array.release((&__pyx_v_array));

          /* "pyogrio/_io.pyx":3376
 *             schema.release(&schema)
 * 
 *         if array.release != NULL:             # <<<<<<<<<<<<<<
 *             array.release(&array)
 * 
*/
        }

        /* "pyogrio/_io.pyx":3379
 *             array.release(&array)
 * 
 *         if options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(options)
 *             options = NULL
*/
        __pyx_t_4 = (__pyx_v_options != NULL);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":3380
 * 
 *         if options != NULL:
 *             CSLDestroy(options)             # <<<<<<<<<<<<<<
 *             options = NULL
 * 
*/
          CSLDestroy(__pyx_v_options);

          /* "pyogrio/_io.pyx":3381
 *         if options != NULL:
 *             CSLDestroy(options)
 *             options = NULL             # <<<<<<<<<<<<<<
 * 
 *         if ogr_dataset != NULL:
*/
          __pyx_v_options = NULL;

          /* "pyogrio/_io.pyx":3379
 *             array.release(&array)
 * 
 *         if options != NULL:             # <<<<<<<<<<<<<<
 *             CSLDestroy(options)
 *             options = NULL
*/
        }

        /* "pyogrio/_io.pyx":3383
 *             options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
        __pyx_t_4 = (__pyx_v_ogr_dataset != NULL);
        if (__pyx_t_4) {

          /* "pyogrio/_io.pyx":3384
 * 
 *         if ogr_dataset != NULL:
 *             ogr_close(ogr_dataset)             # <<<<<<<<<<<<<<
 * 
 *         if use_tmp_vsimem:
*/
          __pyx_t_3 = __pyx_f_7pyogrio_3_io_ogr_close(__pyx_v_ogr_dataset); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3384, __pyx_L32_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "pyogrio/_io.pyx":3383
 *             options = NULL
 * 
 *         if ogr_dataset != NULL:             # <<<<<<<<<<<<<<
 *             ogr_close(ogr_dataset)
 * 
*/
        }

        /* "pyogrio/_io.pyx":3386
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        if (__pyx_v_use_tmp_vsimem) {

          /* "pyogrio/_io.pyx":3387
 * 
 *         if use_tmp_vsimem:
 *             vsimem_rmtree_toplevel(path)             # <<<<<<<<<<<<<<
 * 
 * 
*/
          if (unlikely(!__pyx_v_path)) { __Pyx_RaiseUnboundLocalError("path"); __PYX_ERR(0, 3387, __pyx_L32_error) }
          __pyx_t_3 = __pyx_v_path;
          __Pyx_INCREF(__pyx_t_3);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_3))) __PYX_ERR(0, 3387, __pyx_L32_error)
          __pyx_t_10 = __pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel(((PyObject*)__pyx_t_3), 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3387, __pyx_L32_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

          /* "pyogrio/_io.pyx":3386
 *             ogr_close(ogr_dataset)
 * 
 *         if use_tmp_vsimem:             # <<<<<<<<<<<<<<
 *             vsimem_rmtree_toplevel(path)
 * 
*/
        }
      }
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
      __pyx_lineno = __pyx_t_13; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
      goto __pyx_L1_error;
      __pyx_L32_error:;
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "pyogrio/_io.pyx":3264
 * 
 * 
 * def ogr_write_arrow(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("pyogrio._io.ogr_write_arrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XDECREF(__pyx_v_layer_created);
  __Pyx_XDECREF(__pyx_v_opts);
  __Pyx_XDECREF(__pyx_v_stream_capsule);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_gdal_msg);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":3390
 * 
 * 
 * cdef get_arrow_extension_metadata(const ArrowSchema* schema):             # <<<<<<<<<<<<<<
 *     """Parse the metadata of the ArrowSchema and extract extension type metadata.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_get_arrow_extension_metadata(struct ArrowSchema const *__pyx_v_schema) {
  char const *__pyx_v_metadata;
  PyObject *__pyx_v_extension_name = NULL;
  PyObject *__pyx_v_extension_metadata = NULL;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_pos = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_key_length = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value_length = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char const *__pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_arrow_extension_metadata", 0);

  /* "pyogrio/_io.pyx":3407
 * 
 *     """
 *     cdef const char *metadata = schema.metadata             # <<<<<<<<<<<<<<
 * 
 *     extension_name = None
*/
  __pyx_t_1 = __pyx_v_schema->metadata;
  __pyx_v_metadata = __pyx_t_1;

  /* "pyogrio/_io.pyx":3409
 *     cdef const char *metadata = schema.metadata
 * 
 *     extension_name = None             # <<<<<<<<<<<<<<
 *     extension_metadata = None
 * 
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_extension_name = ((PyObject*)Py_None);

  /* "pyogrio/_io.pyx":3410
 * 
 *     extension_name = None
 *     extension_metadata = None             # <<<<<<<<<<<<<<
 * 
 *     if metadata == NULL:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_extension_metadata = ((PyObject*)Py_None);

  /* "pyogrio/_io.pyx":3412
 *     extension_metadata = None
 * 
 *     if metadata == NULL:             # <<<<<<<<<<<<<<
 *         return extension_name, extension_metadata
 * 
*/
  __pyx_t_2 = (__pyx_v_metadata == NULL);
  if (__pyx_t_2) {

    /* "pyogrio/_io.pyx":3413
 * 
 *     if metadata == NULL:
 *         return extension_name, extension_metadata             # <<<<<<<<<<<<<<
 * 
 *     # the number of metadata key/value pairs is stored
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_extension_name);
    __Pyx_GIVEREF(__pyx_v_extension_name);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_extension_name) != (0)) __PYX_ERR(0, 3413, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_extension_metadata);
    __Pyx_GIVEREF(__pyx_v_extension_metadata);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_extension_metadata) != (0)) __PYX_ERR(0, 3413, __pyx_L1_error);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "pyogrio/_io.pyx":3412
 *     extension_metadata = None
 * 
 *     if metadata == NULL:             # <<<<<<<<<<<<<<
 *         return extension_name, extension_metadata
 * 
*/
  }

  /* "pyogrio/_io.pyx":3417
 *     # the number of metadata key/value pairs is stored
 *     # as an int32 value in the first 4 bytes
 *     n = int.from_bytes(metadata[:4], byteorder=sys.byteorder)             # <<<<<<<<<<<<<<
 *     pos = 4
 * 
*/
  __pyx_t_4 = ((PyObject *)(&PyLong_Type));
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_metadata + 0, 4 - 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_sys); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 3417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 3417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_8 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, __pyx_t_5};
    __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 3417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_byteorder, __pyx_t_7, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 3417, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_from_bytes, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3417, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_n = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyogrio/_io.pyx":3418
 *     # as an int32 value in the first 4 bytes
 *     n = int.from_bytes(metadata[:4], byteorder=sys.byteorder)
 *     pos = 4             # <<<<<<<<<<<<<<
 * 
 *     for i in range(n):
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_4);
  __pyx_v_pos = __pyx_mstate_global->__pyx_int_4;

  /* "pyogrio/_io.pyx":3420
 *     pos = 4
 * 
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         # for each metadata key/value pair, the first 4 bytes is the byte length
 *         # of the key as an int32, then follows the key (not null-terminated),
*/
  __pyx_t_6 = NULL;
  __pyx_t_8 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_n};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(&PyRange_Type), __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3420, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_6 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 3420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_9 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 3420, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    {
      __pyx_t_3 = __pyx_t_9(__pyx_t_6);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 3420, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3424
 *         # of the key as an int32, then follows the key (not null-terminated),
 *         # and then the same for the value length and bytes
 *         key_length = int.from_bytes(             # <<<<<<<<<<<<<<
 *             metadata[pos:pos+4], byteorder=sys.byteorder, signed=True
 *         )
*/
    __pyx_t_7 = ((PyObject *)(&PyLong_Type));
    __Pyx_INCREF(__pyx_t_7);

    /* "pyogrio/_io.pyx":3425
 *         # and then the same for the value length and bytes
 *         key_length = int.from_bytes(
 *             metadata[pos:pos+4], byteorder=sys.byteorder, signed=True             # <<<<<<<<<<<<<<
 *         )
 *         pos += 4
*/
    __Pyx_INCREF(__pyx_v_pos);
    __pyx_t_5 = __pyx_v_pos;
    __pyx_t_2 = (__pyx_t_5 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_10 = 0;
    } else {
      __pyx_t_11 = __Pyx_PyIndex_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_11 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3425, __pyx_L1_error)
      __pyx_t_10 = __pyx_t_11;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyLong_AddObjC(__pyx_v_pos, __pyx_mstate_global->__pyx_int_4, 4, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = (__pyx_t_5 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = PY_SSIZE_T_MAX;
    } else {
      __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_5); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3425, __pyx_L1_error)
      __pyx_t_11 = __pyx_t_12;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_metadata + __pyx_t_10, __pyx_t_11 - __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_sys); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_7, __pyx_t_5};
      __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3424, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_byteorder, __pyx_t_13, __pyx_t_4, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 3424, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_signed, Py_True, __pyx_t_4, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 3424, __pyx_L1_error)
      __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_from_bytes, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3424, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_key_length, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3427
 *             metadata[pos:pos+4], byteorder=sys.byteorder, signed=True
 *         )
 *         pos += 4             # <<<<<<<<<<<<<<
 *         key = metadata[pos:pos+key_length]
 *         pos += key_length
*/
    __pyx_t_3 = __Pyx_PyLong_AddObjC(__pyx_v_pos, __pyx_mstate_global->__pyx_int_4, 4, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_pos, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3428
 *         )
 *         pos += 4
 *         key = metadata[pos:pos+key_length]             # <<<<<<<<<<<<<<
 *         pos += key_length
 *         value_length = int.from_bytes(
*/
    __Pyx_INCREF(__pyx_v_pos);
    __pyx_t_3 = __pyx_v_pos;
    __pyx_t_2 = (__pyx_t_3 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = 0;
    } else {
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3428, __pyx_L1_error)
      __pyx_t_11 = __pyx_t_10;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Add(__pyx_v_pos, __pyx_v_key_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3428, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__pyx_t_3 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_10 = PY_SSIZE_T_MAX;
    } else {
      __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3428, __pyx_L1_error)
      __pyx_t_10 = __pyx_t_12;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_metadata + __pyx_t_11, __pyx_t_10 - __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3428, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3429
 *         pos += 4
 *         key = metadata[pos:pos+key_length]
 *         pos += key_length             # <<<<<<<<<<<<<<
 *         value_length = int.from_bytes(
 *             metadata[pos:pos+4], byteorder=sys.byteorder, signed=True
*/
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_pos, __pyx_v_key_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_pos, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3430
 *         key = metadata[pos:pos+key_length]
 *         pos += key_length
 *         value_length = int.from_bytes(             # <<<<<<<<<<<<<<
 *             metadata[pos:pos+4], byteorder=sys.byteorder, signed=True
 *         )
*/
    __pyx_t_4 = ((PyObject *)(&PyLong_Type));
    __Pyx_INCREF(__pyx_t_4);

    /* "pyogrio/_io.pyx":3431
 *         pos += key_length
 *         value_length = int.from_bytes(
 *             metadata[pos:pos+4], byteorder=sys.byteorder, signed=True             # <<<<<<<<<<<<<<
 *         )
 *         pos += 4
*/
    __Pyx_INCREF(__pyx_v_pos);
    __pyx_t_13 = __pyx_v_pos;
    __pyx_t_2 = (__pyx_t_13 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_10 = 0;
    } else {
      __pyx_t_11 = __Pyx_PyIndex_AsSsize_t(__pyx_t_13); if (unlikely((__pyx_t_11 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3431, __pyx_L1_error)
      __pyx_t_10 = __pyx_t_11;
    }
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_t_13 = __Pyx_PyLong_AddObjC(__pyx_v_pos, __pyx_mstate_global->__pyx_int_4, 4, 0, 0); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_2 = (__pyx_t_13 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = PY_SSIZE_T_MAX;
    } else {
      __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_13); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3431, __pyx_L1_error)
      __pyx_t_11 = __pyx_t_12;
    }
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_t_13 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_metadata + __pyx_t_10, __pyx_t_11 - __pyx_t_10); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_sys); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_byteorder); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 3431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_4, __pyx_t_13};
      __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_byteorder, __pyx_t_7, __pyx_t_5, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 3430, __pyx_L1_error)
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_signed, Py_True, __pyx_t_5, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 3430, __pyx_L1_error)
      __pyx_t_3 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_from_bytes, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3430, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_value_length, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3433
 *             metadata[pos:pos+4], byteorder=sys.byteorder, signed=True
 *         )
 *         pos += 4             # <<<<<<<<<<<<<<
 *         value = metadata[pos:pos+value_length]
 *         pos += value_length
*/
    __pyx_t_3 = __Pyx_PyLong_AddObjC(__pyx_v_pos, __pyx_mstate_global->__pyx_int_4, 4, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3433, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_pos, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3434
 *         )
 *         pos += 4
 *         value = metadata[pos:pos+value_length]             # <<<<<<<<<<<<<<
 *         pos += value_length
 * 
*/
    __Pyx_INCREF(__pyx_v_pos);
    __pyx_t_3 = __pyx_v_pos;
    __pyx_t_2 = (__pyx_t_3 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_11 = 0;
    } else {
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3434, __pyx_L1_error)
      __pyx_t_11 = __pyx_t_10;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Add(__pyx_v_pos, __pyx_v_value_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = (__pyx_t_3 == Py_None);
    if (__pyx_t_2) {
      __pyx_t_10 = PY_SSIZE_T_MAX;
    } else {
      __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 3434, __pyx_L1_error)
      __pyx_t_10 = __pyx_t_12;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_metadata + __pyx_t_11, __pyx_t_10 - __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_value, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3435
 *         pos += 4
 *         value = metadata[pos:pos+value_length]
 *         pos += value_length             # <<<<<<<<<<<<<<
 * 
 *         if key == b"ARROW:extension:name":
*/
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_pos, __pyx_v_value_length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_pos, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyogrio/_io.pyx":3437
 *         pos += value_length
 * 
 *         if key == b"ARROW:extension:name":             # <<<<<<<<<<<<<<
 *             extension_name = value
 *         elif key == b"ARROW:extension:metadata":
*/
    __pyx_t_2 = (__Pyx_PyBytes_Equals(__pyx_v_key, __pyx_mstate_global->__pyx_kp_b_ARROW_extension_name, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 3437, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3438
 * 
 *         if key == b"ARROW:extension:name":
 *             extension_name = value             # <<<<<<<<<<<<<<
 *         elif key == b"ARROW:extension:metadata":
 *             extension_metadata = value
*/
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_DECREF_SET(__pyx_v_extension_name, __pyx_v_value);

      /* "pyogrio/_io.pyx":3437
 *         pos += value_length
 * 
 *         if key == b"ARROW:extension:name":             # <<<<<<<<<<<<<<
 *             extension_name = value
 *         elif key == b"ARROW:extension:metadata":
*/
      goto __pyx_L6;
    }

    /* "pyogrio/_io.pyx":3439
 *         if key == b"ARROW:extension:name":
 *             extension_name = value
 *         elif key == b"ARROW:extension:metadata":             # <<<<<<<<<<<<<<
 *             extension_metadata = value
 * 
*/
    __pyx_t_2 = (__Pyx_PyBytes_Equals(__pyx_v_key, __pyx_mstate_global->__pyx_kp_b_ARROW_extension_metadata, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 3439, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3440
 *             extension_name = value
 *         elif key == b"ARROW:extension:metadata":
 *             extension_metadata = value             # <<<<<<<<<<<<<<
 * 
 *         if extension_name is not None and extension_metadata is not None:
*/
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_DECREF_SET(__pyx_v_extension_metadata, __pyx_v_value);

      /* "pyogrio/_io.pyx":3439
 *         if key == b"ARROW:extension:name":
 *             extension_name = value
 *         elif key == b"ARROW:extension:metadata":             # <<<<<<<<<<<<<<
 *             extension_metadata = value
 * 
*/
    }
    __pyx_L6:;

    /* "pyogrio/_io.pyx":3442
 *             extension_metadata = value
 * 
 *         if extension_name is not None and extension_metadata is not None:             # <<<<<<<<<<<<<<
 *             break
 * 
*/
    __pyx_t_14 = (__pyx_v_extension_name != ((PyObject*)Py_None));
    if (__pyx_t_14) {
    } else {
      __pyx_t_2 = __pyx_t_14;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_14 = (__pyx_v_extension_metadata != ((PyObject*)Py_None));
    __pyx_t_2 = __pyx_t_14;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_2) {

      /* "pyogrio/_io.pyx":3443
 * 
 *         if extension_name is not None and extension_metadata is not None:
 *             break             # <<<<<<<<<<<<<<
 * 
 *     return extension_name, extension_metadata
*/
      goto __pyx_L5_break;

      /* "pyogrio/_io.pyx":3442
 *             extension_metadata = value
 * 
 *         if extension_name is not None and extension_metadata is not None:             # <<<<<<<<<<<<<<
 *             break
 * 
*/
    }

    /* "pyogrio/_io.pyx":3420
 *     pos = 4
 * 
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         # for each metadata key/value pair, the first 4 bytes is the byte length
 *         # of the key as an int32, then follows the key (not null-terminated),
*/
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  goto __pyx_L10_for_end;
  __pyx_L5_break:;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  goto __pyx_L10_for_end;
  __pyx_L10_for_end:;

  /* "pyogrio/_io.pyx":3445
 *             break
 * 
 *     return extension_name, extension_metadata             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 3445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_extension_name);
  __Pyx_GIVEREF(__pyx_v_extension_name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_extension_name) != (0)) __PYX_ERR(0, 3445, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_extension_metadata);
  __Pyx_GIVEREF(__pyx_v_extension_metadata);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_extension_metadata) != (0)) __PYX_ERR(0, 3445, __pyx_L1_error);
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":3390
 * 
 * 
 * cdef get_arrow_extension_metadata(const ArrowSchema* schema):             # <<<<<<<<<<<<<<
 *     """Parse the metadata of the ArrowSchema and extract extension type metadata.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pyogrio._io.get_arrow_extension_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_extension_name);
  __Pyx_XDECREF(__pyx_v_extension_metadata);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_pos);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_key_length);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value_length);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":3448
 * 
 * 
 * cdef is_arrow_geometry_field(const ArrowSchema* schema):             # <<<<<<<<<<<<<<
 *     """Check if the field is a geometry field or not.
 * 
*/

static PyObject *__pyx_f_7pyogrio_3_io_is_arrow_geometry_field(struct ArrowSchema const *__pyx_v_schema) {
  PyObject *__pyx_v_name = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  int __pyx_t_6;
  int __pyx_t_7;
  size_t __pyx_t_8;
  PyObject *__pyx_t_9[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_arrow_geometry_field", 0);

  /* "pyogrio/_io.pyx":3467
 * 
 *     """
 *     name, _ = get_arrow_extension_metadata(schema)             # <<<<<<<<<<<<<<
 *     if name is not None:
 *         if name == b"geoarrow.wkb" or name == b"ogc.wkb":
*/
  __pyx_t_1 = __pyx_f_7pyogrio_3_io_get_arrow_extension_metadata(__pyx_v_schema); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 3467, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1);
      __Pyx_INCREF(__pyx_t_3);
    } else {
      __pyx_t_2 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3467, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3467, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_3);
    }
    #else
    __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 3467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3467, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4);
    index = 0; __pyx_t_2 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 2) < (0)) __PYX_ERR(0, 3467, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 3467, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_name = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v__ = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pyogrio/_io.pyx":3468
 *     """
 *     name, _ = get_arrow_extension_metadata(schema)
 *     if name is not None:             # <<<<<<<<<<<<<<
 *         if name == b"geoarrow.wkb" or name == b"ogc.wkb":
 *             return True
*/
  __pyx_t_6 = (__pyx_v_name != Py_None);
  if (__pyx_t_6) {

    /* "pyogrio/_io.pyx":3469
 *     name, _ = get_arrow_extension_metadata(schema)
 *     if name is not None:
 *         if name == b"geoarrow.wkb" or name == b"ogc.wkb":             # <<<<<<<<<<<<<<
 *             return True
 * 
*/
    __pyx_t_7 = (__Pyx_PyBytes_Equals(__pyx_v_name, __pyx_mstate_global->__pyx_kp_b_geoarrow_wkb, Py_EQ)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 3469, __pyx_L1_error)
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = (__Pyx_PyBytes_Equals(__pyx_v_name, __pyx_mstate_global->__pyx_kp_b_ogc_wkb, Py_EQ)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 3469, __pyx_L1_error)
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_6) {

      /* "pyogrio/_io.pyx":3470
 *     if name is not None:
 *         if name == b"geoarrow.wkb" or name == b"ogc.wkb":
 *             return True             # <<<<<<<<<<<<<<
 * 
 *         # raise an error for other geoarrow types
*/
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(Py_True);
      __pyx_r = Py_True;
      goto __pyx_L0;

      /* "pyogrio/_io.pyx":3469
 *     name, _ = get_arrow_extension_metadata(schema)
 *     if name is not None:
 *         if name == b"geoarrow.wkb" or name == b"ogc.wkb":             # <<<<<<<<<<<<<<
 *             return True
 * 
*/
    }

    /* "pyogrio/_io.pyx":3473
 * 
 *         # raise an error for other geoarrow types
 *         if name.startswith(b"geoarrow."):             # <<<<<<<<<<<<<<
 *             raise NotImplementedError(
 *                 f"Writing a geometry column of type {name.decode()} is not yet "
*/
    __pyx_t_3 = __pyx_v_name;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_b_geoarrow};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_startswith, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3473, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 3473, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__pyx_t_6)) {

      /* "pyogrio/_io.pyx":3474
 *         # raise an error for other geoarrow types
 *         if name.startswith(b"geoarrow."):
 *             raise NotImplementedError(             # <<<<<<<<<<<<<<
 *                 f"Writing a geometry column of type {name.decode()} is not yet "
 *                 "supported. Only WKB is currently supported ('geoarrow.wkb' or "
*/
      __pyx_t_3 = NULL;

      /* "pyogrio/_io.pyx":3475
 *         if name.startswith(b"geoarrow."):
 *             raise NotImplementedError(
 *                 f"Writing a geometry column of type {name.decode()} is not yet "             # <<<<<<<<<<<<<<
 *                 "supported. Only WKB is currently supported ('geoarrow.wkb' or "
 *                 "'ogc.wkb' types)."
*/
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_mstate_global->__pyx_n_u_decode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3475, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3475, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_FormatSimple(__pyx_t_4, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3475, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9[0] = __pyx_mstate_global->__pyx_kp_u_Writing_a_geometry_column_of_typ;
      __pyx_t_9[1] = __pyx_t_2;
      __pyx_t_9[2] = __pyx_mstate_global->__pyx_kp_u_is_not_yet_supported_Only_WKB_i;
      __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_9, 3, 34 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 91, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 3475, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_NotImplementedError)), __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3474, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 3474, __pyx_L1_error)

      /* "pyogrio/_io.pyx":3473
 * 
 *         # raise an error for other geoarrow types
 *         if name.startswith(b"geoarrow."):             # <<<<<<<<<<<<<<
 *             raise NotImplementedError(
 *                 f"Writing a geometry column of type {name.decode()} is not yet "
*/
    }

    /* "pyogrio/_io.pyx":3468
 *     """
 *     name, _ = get_arrow_extension_metadata(schema)
 *     if name is not None:             # <<<<<<<<<<<<<<
 *         if name == b"geoarrow.wkb" or name == b"ogc.wkb":
 *             return True
*/
  }

  /* "pyogrio/_io.pyx":3480
 *             )
 * 
 *     return False             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "pyogrio/_io.pyx":3448
 * 
 * 
 * cdef is_arrow_geometry_field(const ArrowSchema* schema):             # <<<<<<<<<<<<<<
 *     """Check if the field is a geometry field or not.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyogrio._io.is_arrow_geometry_field", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyogrio/_io.pyx":3483
 * 
 * 
 * cdef create_fields_from_arrow_schema(             # <<<<<<<<<<<<<<
 *     OGRLayerH destLayer, const ArrowSchema* schema, char** options, str geometry_name
 * ):
*/

static PyObject *__pyx_f_7pyogrio_3_io_create_fields_from_arrow_schema(OGRLayerH __pyx_v_destLayer, struct ArrowSchema const *__pyx_v_schema, char **__pyx_v_options, PyObject *__pyx_v_geometry_name) {
  PyObject *__pyx_v_fid_column = NULL;
  struct ArrowSchema *__pyx_v_child;
  int64_t __pyx_v_i;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_gdal_msg = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int64_t __pyx_t_2;
  int64_t __pyx_t_3;
  int64_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9[3];
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15[8];
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_fields_from_arrow_schema", 0);

  /* "pyogrio/_io.pyx":3513
 *     # Doing so triggers a bug in GDAL < 3.10.1:
 *     # https://github.com/OSGeo/gdal/issues/11527#issuecomment-2556092722
 *     fid_column = get_string(OGR_L_GetFIDColumn(destLayer))             # <<<<<<<<<<<<<<
 * 
 *     # The schema object is a struct type where each child is a column.
*/
  __pyx_t_1 = __pyx_f_7pyogrio_4_ogr_get_string(OGR_L_GetFIDColumn(__pyx_v_destLayer), NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_fid_column = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyogrio/_io.pyx":3517
 *     # The schema object is a struct type where each child is a column.
 *     cdef ArrowSchema* child
 *     for i in range(schema.n_children):             # <<<<<<<<<<<<<<
 *         child = schema.children[i]
 * 
*/
  __pyx_t_2 = __pyx_v_schema->n_children;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "pyogrio/_io.pyx":3518
 *     cdef ArrowSchema* child
 *     for i in range(schema.n_children):
 *         child = schema.children[i]             # <<<<<<<<<<<<<<
 * 
 *         if child == NULL:
*/
    __pyx_v_child = (__pyx_v_schema->children[__pyx_v_i]);

    /* "pyogrio/_io.pyx":3520
 *         child = schema.children[i]
 * 
 *         if child == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Received invalid Arrow schema (null child)")
 * 
*/
    __pyx_t_5 = (__pyx_v_child == NULL);
    if (unlikely(__pyx_t_5)) {

      /* "pyogrio/_io.pyx":3521
 * 
 *         if child == NULL:
 *             raise RuntimeError("Received invalid Arrow schema (null child)")             # <<<<<<<<<<<<<<
 * 
 *         # Don't create property for geometry column
*/
      __pyx_t_6 = NULL;
      __pyx_t_7 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_Received_invalid_Arrow_schema_nu};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3521, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 3521, __pyx_L1_error)

      /* "pyogrio/_io.pyx":3520
 *         child = schema.children[i]
 * 
 *         if child == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Received invalid Arrow schema (null child)")
 * 
*/
    }

    /* "pyogrio/_io.pyx":3524
 * 
 *         # Don't create property for geometry column
 *         if get_string(child.name) == geometry_name or is_arrow_geometry_field(child):             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
    __pyx_t_1 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_v_child->name, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_v_geometry_name, Py_EQ)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 3524, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!__pyx_t_8) {
    } else {
      __pyx_t_5 = __pyx_t_8;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_1 = __pyx_f_7pyogrio_3_io_is_arrow_geometry_field(__pyx_v_child); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 3524, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __pyx_t_8;
    __pyx_L7_bool_binop_done:;
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":3525
 *         # Don't create property for geometry column
 *         if get_string(child.name) == geometry_name or is_arrow_geometry_field(child):
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         # Don't create property for column that will already be used as FID
*/
      goto __pyx_L3_continue;

      /* "pyogrio/_io.pyx":3524
 * 
 *         # Don't create property for geometry column
 *         if get_string(child.name) == geometry_name or is_arrow_geometry_field(child):             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
    }

    /* "pyogrio/_io.pyx":3535
 *         # Note2: in the non-arrow path, the FID column is also treated
 *         # case-insensitive, so this is consistent with that.
 *         if fid_column != "" and get_string(child.name) == fid_column:             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
    __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_v_fid_column, __pyx_mstate_global->__pyx_kp_u__9, Py_NE)); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 3535, __pyx_L1_error)
    if (__pyx_t_8) {
    } else {
      __pyx_t_5 = __pyx_t_8;
      goto __pyx_L10_bool_binop_done;
    }
    __pyx_t_1 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_v_child->name, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = PyObject_RichCompare(__pyx_t_1, __pyx_v_fid_column, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 3535, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 3535, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_5 = __pyx_t_8;
    __pyx_L10_bool_binop_done:;
    if (__pyx_t_5) {

      /* "pyogrio/_io.pyx":3536
 *         # case-insensitive, so this is consistent with that.
 *         if fid_column != "" and get_string(child.name) == fid_column:
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         if not OGR_L_CreateFieldFromArrowSchema(destLayer, child, options):
*/
      goto __pyx_L3_continue;

      /* "pyogrio/_io.pyx":3535
 *         # Note2: in the non-arrow path, the FID column is also treated
 *         # case-insensitive, so this is consistent with that.
 *         if fid_column != "" and get_string(child.name) == fid_column:             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
    }

    /* "pyogrio/_io.pyx":3538
 *             continue
 * 
 *         if not OGR_L_CreateFieldFromArrowSchema(destLayer, child, options):             # <<<<<<<<<<<<<<
 *             exc = check_last_error()
 *             gdal_msg = f" ({str(exc)})" if exc else ""
*/
    __pyx_t_5 = (!OGR_L_CreateFieldFromArrowSchema(__pyx_v_destLayer, __pyx_v_child, __pyx_v_options));
    if (unlikely(__pyx_t_5)) {

      /* "pyogrio/_io.pyx":3539
 * 
 *         if not OGR_L_CreateFieldFromArrowSchema(destLayer, child, options):
 *             exc = check_last_error()             # <<<<<<<<<<<<<<
 *             gdal_msg = f" ({str(exc)})" if exc else ""
 *             raise FieldError(
*/
      __pyx_t_6 = __pyx_f_7pyogrio_4_err_check_last_error(); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 3539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_v_exc = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":3540
 *         if not OGR_L_CreateFieldFromArrowSchema(destLayer, child, options):
 *             exc = check_last_error()
 *             gdal_msg = f" ({str(exc)})" if exc else ""             # <<<<<<<<<<<<<<
 *             raise FieldError(
 *                 f"Error while creating field from Arrow for field {i} with name "
*/
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_exc); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 3540, __pyx_L1_error)
      if (__pyx_t_5) {
        __pyx_t_1 = __Pyx_PyObject_Unicode(__pyx_v_exc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3540, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9[0] = __pyx_mstate_global->__pyx_kp_u__15;
        __pyx_t_9[1] = __pyx_t_1;
        __pyx_t_9[2] = __pyx_mstate_global->__pyx_kp_u__5;
        __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_9, 3, 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 3540, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_6 = __pyx_t_10;
        __pyx_t_10 = 0;
      } else {
        __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u__9);
        __pyx_t_6 = __pyx_mstate_global->__pyx_kp_u__9;
      }
      __pyx_v_gdal_msg = ((PyObject*)__pyx_t_6);
      __pyx_t_6 = 0;

      /* "pyogrio/_io.pyx":3541
 *             exc = check_last_error()
 *             gdal_msg = f" ({str(exc)})" if exc else ""
 *             raise FieldError(             # <<<<<<<<<<<<<<
 *                 f"Error while creating field from Arrow for field {i} with name "
 *                 f"'{get_string(child.name)}' and type {get_string(child.format)}"
*/
      __pyx_t_10 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_FieldError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "pyogrio/_io.pyx":3542
 *             gdal_msg = f" ({str(exc)})" if exc else ""
 *             raise FieldError(
 *                 f"Error while creating field from Arrow for field {i} with name "             # <<<<<<<<<<<<<<
 *                 f"'{get_string(child.name)}' and type {get_string(child.format)}"
 *                 f"{gdal_msg}."
*/
      __pyx_t_11 = __Pyx_PyUnicode_From_int64_t(__pyx_v_i, 0, ' ', 'd'); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 3542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);

      /* "pyogrio/_io.pyx":3543
 *             raise FieldError(
 *                 f"Error while creating field from Arrow for field {i} with name "
 *                 f"'{get_string(child.name)}' and type {get_string(child.format)}"             # <<<<<<<<<<<<<<
 *                 f"{gdal_msg}."
 *             )
*/
      __pyx_t_12 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_v_child->name, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 3543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_13 = __Pyx_PyObject_FormatSimple(__pyx_t_12, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 3543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = __pyx_f_7pyogrio_4_ogr_get_string(__pyx_v_child->format, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 3543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_14 = __Pyx_PyObject_FormatSimple(__pyx_t_12, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "pyogrio/_io.pyx":3544
 *                 f"Error while creating field from Arrow for field {i} with name "
 *                 f"'{get_string(child.name)}' and type {get_string(child.format)}"
 *                 f"{gdal_msg}."             # <<<<<<<<<<<<<<
 *             )
*/
      __pyx_t_12 = __Pyx_PyUnicode_Unicode(__pyx_v_gdal_msg); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 3544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_15[0] = __pyx_mstate_global->__pyx_kp_u_Error_while_creating_field_from;
      __pyx_t_15[1] = __pyx_t_11;
      __pyx_t_15[2] = __pyx_mstate_global->__pyx_kp_u_with_name;
      __pyx_t_15[3] = __pyx_t_13;
      __pyx_t_15[4] = __pyx_mstate_global->__pyx_kp_u_and_type;
      __pyx_t_15[5] = __pyx_t_14;
      __pyx_t_15[6] = __pyx_t_12;
      __pyx_t_15[7] = __pyx_mstate_global->__pyx_kp_u__2;

      /* "pyogrio/_io.pyx":3542
 *             gdal_msg = f" ({str(exc)})" if exc else ""
 *             raise FieldError(
 *                 f"Error while creating field from Arrow for field {i} with name "             # <<<<<<<<<<<<<<
 *                 f"'{get_string(child.name)}' and type {get_string(child.format)}"
 *                 f"{gdal_msg}."
*/
      __pyx_t_16 = __Pyx_PyUnicode_Join(__pyx_t_15, 8, 48 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11) + 12 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14) + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_12) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_12));
      if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 3542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_7 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
        assert(__pyx_t_10);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
        __pyx_t_7 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_16};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 3541, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 3541, __pyx_L1_error)

      /* "pyogrio/_io.pyx":3538
 *             continue
 * 
 *         if not OGR_L_CreateFieldFromArrowSchema(destLayer, child, options):             # <<<<<<<<<<<<<<
 *             exc = check_last_error()
 *             gdal_msg = f" ({str(exc)})" if exc else ""
*/
    }
    __pyx_L3_continue:;
  }

  /* "pyogrio/_io.pyx":3483
 * 
 * 
 * cdef create_fields_from_arrow_schema(             # <<<<<<<<<<<<<<
 *     OGRLayerH destLayer, const ArrowSchema* schema, char** options, str geometry_name
 * ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("pyogrio._io.create_fields_from_arrow_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fid_column);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_gdal_msg);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */

static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata[--__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata))))
  {
    __pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata[__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata++] = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata},
  {Py_tp_new, (void *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata},
  {0, 0},
};
static PyType_Spec __pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata_spec = {
  "pyogrio._io.__pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata",
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER,
  __pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata_slots,
};
#else

static PyTypeObject __pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""__pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata", /*tp_name*/
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_1_genexpr > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_1_genexpr[--__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_1_genexpr];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_1_genexpr(PyObject *o) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_1_genexpr) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_1_genexpr < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr))))
  {
    __pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_1_genexpr[__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject *)p->__pyx_outer_scope), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_1_genexpr},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_1_genexpr},
  {Py_tp_new, (void *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_1_genexpr},
  {0, 0},
};
static PyType_Spec __pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr_spec = {
  "pyogrio._io.__pyx_scope_struct_1_genexpr",
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr_slots,
};
#else

static PyTypeObject __pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""__pyx_scope_struct_1_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_1_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_1_genexpr, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_1_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_1_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *p;
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features[--__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *)o);
  p->__pyx_v_fields.data = NULL;
  p->__pyx_v_fields.memview = NULL;
  return o;
}

static void __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features(PyObject *o) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *p = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_fid_chunks);
  Py_CLEAR(p->__pyx_v_field_data_chunks);
  Py_CLEAR(p->__pyx_v_geom_chunks);
  __PYX_XCLEAR_MEMVIEW(&p->__pyx_v_fields, 1);
  p->__pyx_v_fields.memview = NULL; p->__pyx_v_fields.data = NULL;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features))))
  {
    __pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features[__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features++] = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *p = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_fid_chunks) {
    e = (*v)(p->__pyx_v_fid_chunks, a); if (e) return e;
  }
  if (p->__pyx_v_field_data_chunks) {
    e = (*v)(p->__pyx_v_field_data_chunks, a); if (e) return e;
  }
  if (p->__pyx_v_geom_chunks) {
    e = (*v)(p->__pyx_v_geom_chunks, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *p = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features *)o;
  tmp = ((PyObject*)p->__pyx_v_fid_chunks);
  p->__pyx_v_fid_chunks = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_field_data_chunks);
  p->__pyx_v_field_data_chunks = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_geom_chunks);
  p->__pyx_v_geom_chunks = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features},
  {Py_tp_clear, (void *)__pyx_tp_clear_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features},
  {Py_tp_new, (void *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features},
  {0, 0},
};
static PyType_Spec __pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features_spec = {
  "pyogrio._io.__pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features",
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features_slots,
};
#else

static PyTypeObject __pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""__pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features", /*tp_name*/
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features, /*tp_traverse*/
  __pyx_tp_clear_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow[--__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow(PyObject *o) {
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *p = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_bbox);
  Py_CLEAR(p->__pyx_v_capsule);
  Py_CLEAR(p->__pyx_v_columns);
  Py_CLEAR(p->__pyx_v_crs);
  Py_CLEAR(p->__pyx_v_dataset_kwargs);
  Py_CLEAR(p->__pyx_v_datetime_as_string);
  Py_CLEAR(p->__pyx_v_encoding);
  Py_CLEAR(p->__pyx_v_ex);
  Py_CLEAR(p->__pyx_v_feature_count);
  Py_CLEAR(p->__pyx_8genexpr7__pyx_v_fid);
  Py_CLEAR(p->__pyx_v_fid_column);
  Py_CLEAR(p->__pyx_v_fid_column_where);
  Py_CLEAR(p->__pyx_v_fids);
  Py_CLEAR(p->__pyx_v_fids_str);
  Py_CLEAR(p->__pyx_8genexpr8__pyx_v_field);
  Py_CLEAR(p->__pyx_8genexpr9__pyx_v_field);
  Py_CLEAR(p->__pyx_v_field);
  Py_CLEAR(p->__pyx_v_field_b);
  Py_CLEAR(p->__pyx_v_fields);
  Py_CLEAR(p->__pyx_v_geometry_name);
  Py_CLEAR(p->__pyx_v_geometry_type);
  Py_CLEAR(p->__pyx_v_idx);
  Py_CLEAR(p->__pyx_v_ignored_fields);
  Py_CLEAR(p->__pyx_v_layer);
  Py_CLEAR(p->__pyx_v_mask);
  Py_CLEAR(p->__pyx_v_meta);
  Py_CLEAR(p->__pyx_v_ogr_subtypes);
  Py_CLEAR(p->__pyx_v_ogr_types);
  Py_CLEAR(p->__pyx_v_pa);
  Py_CLEAR(p->__pyx_v_path);
  Py_CLEAR(p->__pyx_v_path_or_buffer);
  Py_CLEAR(p->__pyx_v_reader);
  Py_CLEAR(p->__pyx_v_sql);
  Py_CLEAR(p->__pyx_v_sql_dialect);
  Py_CLEAR(p->__pyx_v_use_pyarrow);
  Py_CLEAR(p->__pyx_v_where);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow, sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow))))
  {
    __pyx_mstate_global->__pyx_freelist_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow[__pyx_mstate_global->__pyx_freecount_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow++] = ((struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *p = (struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_bbox) {
    e = (*v)(p->__pyx_v_bbox, a); if (e) return e;
  }
  if (p->__pyx_v_capsule) {
    e = (*v)(p->__pyx_v_capsule, a); if (e) return e;
  }
  if (p->__pyx_v_columns) {
    e = (*v)(p->__pyx_v_columns, a); if (e) return e;
  }
  if (p->__pyx_v_dataset_kwargs) {
    e = (*v)(p->__pyx_v_dataset_kwargs, a); if (e) return e;
  }
  if (p->__pyx_v_datetime_as_string) {
    e = (*v)(p->__pyx_v_datetime_as_string, a); if (e) return e;
  }
  if (p->__pyx_v_encoding) {
    e = (*v)(p->__pyx_v_encoding, a); if (e) return e;
  }
  if (p->__pyx_v_ex) {
    e = (*v)(p->__pyx_v_ex, a); if (e) return e;
  }
  if (p->__pyx_v_feature_count) {
    e = (*v)(p->__pyx_v_feature_count, a); if (e) return e;
  }
  if (p->__pyx_8genexpr7__pyx_v_fid) {
    e = (*v)(p->__pyx_8genexpr7__pyx_v_fid, a); if (e) return e;
  }
  if (p->__pyx_v_fid_column) {
    e = (*v)(p->__pyx_v_fid_column, a); if (e) return e;
  }
  if (p->__pyx_v_fid_column_where) {
    e = (*v)(p->__pyx_v_fid_column_where, a); if (e) return e;
  }
  if (p->__pyx_v_fids) {
    e = (*v)(p->__pyx_v_fids, a); if (e) return e;
  }
  if (p->__pyx_8genexpr8__pyx_v_field) {
    e = (*v)(p->__pyx_8genexpr8__pyx_v_field, a); if (e) return e;
  }
  if (p->__pyx_8genexpr9__pyx_v_field) {
    e = (*v)(p->__pyx_8genexpr9__pyx_v_field, a); if (e) return e;
  }
  if (p->__pyx_v_field) {
    e = (*v)(p->__pyx_v_field, a); if (e) return e;
  }
  if (p->__pyx_v_field_b) {
    e = (*v)(p->__pyx_v_field_b, a); if (e) return e;
  }
  if (p->__pyx_v_fields) {
    e = (*v)(p->__pyx_v_fields, a); if (e) return e;
  }
  if (p->__pyx_v_geometry_name) {
    e = (*v)(p->__pyx_v_geometry_name, a); if (e) return e;
  }
  if (p->__pyx_v_idx) {
    e = (*v)(p->__pyx_v_idx, a); if (e) return e;
  }
  if (p->__pyx_v_ignored_fields) {
    e = (*v)(p->__pyx_v_ignored_fields, a); if (e) return e;
  }
  if (p->__pyx_v_layer) {
    e = (*v)(p->__pyx_v_layer, a); if (e) return e;
  }
  if (p->__pyx_v_mask) {
    e = (*v)(p->__pyx_v_mask, a); if (e) return e;
  }
  if (p->__pyx_v_meta) {
    e = (*v)(p->__pyx_v_meta, a); if (e) return e;
  }
  if (p->__pyx_v_ogr_subtypes) {
    e = (*v)(p->__pyx_v_ogr_subtypes, a); if (e) return e;
  }
  if (p->__pyx_v_ogr_types) {
    e = (*v)(p->__pyx_v_ogr_types, a); if (e) return e;
  }
  if (p->__pyx_v_pa) {
    e = (*v)(p->__pyx_v_pa, a); if (e) return e;
  }
  if (p->__pyx_v_path) {
    e = (*v)(p->__pyx_v_path, a); if (e) return e;
  }
  if (p->__pyx_v_path_or_buffer) {
    e = (*v)(p->__pyx_v_path_or_buffer, a); if (e) return e;
  }
  if (p->__pyx_v_reader) {
    e = (*v)(p->__pyx_v_reader, a); if (e) return e;
  }
  if (p->__pyx_v_use_pyarrow) {
    e = (*v)(p->__pyx_v_use_pyarrow, a); if (e) return e;
  }
  if (p->__pyx_v_where) {
    e = (*v)(p->__pyx_v_where, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow},
  {Py_tp_new, (void *)__pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow},
  {0, 0},
};
static PyType_Spec __pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow_spec = {
  "pyogrio._io.__pyx_scope_struct_3_ogr_open_arrow",
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow_slots,
};
#else

static PyTypeObject __pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""__pyx_scope_struct_3_ogr_open_arrow", /*tp_name*/
  sizeof(struct __pyx_obj_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_array __pyx_vtable_array;

static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_array_obj *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_array_obj *)o);
  p->__pyx_vtab = __pyx_vtabptr_array;
  p->mode = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_format = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_array___cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_array(PyObject *o) {
  struct __pyx_array_obj *p = (struct __pyx_array_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_array) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_array___dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->mode);
  Py_CLEAR(p->_format);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static PyObject *__pyx_sq_item_array(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyLong_FromSsize_t(i); if(!x) return 0;
  #if CYTHON_USE_TYPE_SLOTS || (!CYTHON_USE_TYPE_SPECS && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  #else
  r = ((binaryfunc)PyType_GetSlot(Py_TYPE(o), Py_mp_subscript))(o, x);
  #endif
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_array(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_array___setitem__(o, i, v);
  }
  else {
    __Pyx_TypeName o_type_name;
    o_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(o));
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by " __Pyx_FMT_TYPENAME, o_type_name);
    __Pyx_DECREF_TypeName(o_type_name);
    return -1;
  }
}

static PyObject *__pyx_tp_getattro_array(PyObject *o, PyObject *n) {
  PyObject *v = PyObject_GenericGetAttr(o, n);
  if (!v && PyErr_ExceptionMatches(PyExc_AttributeError)) {
    PyErr_Clear();
    v = __pyx_array___getattr__(o, n);
  }
  return v;
}

static PyObject *__pyx_getprop___pyx_array_memview(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_5array_7memview_1__get__(o);
}

static PyMethodDef __pyx_methods_array[] = {
  {"__getattr__", (PyCFunction)__pyx_array___getattr__, METH_O|METH_COEXIST, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_array_1__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_array_3__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_array[] = {
  {"memview", __pyx_getprop___pyx_array_memview, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
#if !CYTHON_COMPILING_IN_LIMITED_API

static PyBufferProcs __pyx_tp_as_buffer_array = {
  __pyx_array_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};
#endif
static PyType_Slot __pyx_type___pyx_array_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_array},
  {Py_sq_length, (void *)__pyx_array___len__},
  {Py_sq_item, (void *)__pyx_sq_item_array},
  {Py_mp_length, (void *)__pyx_array___len__},
  {Py_mp_subscript, (void *)__pyx_array___getitem__},
  {Py_mp_ass_subscript, (void *)__pyx_mp_ass_subscript_array},
  {Py_tp_getattro, (void *)__pyx_tp_getattro_array},
  #if defined(Py_bf_getbuffer)
  {Py_bf_getbuffer, (void *)__pyx_array_getbuffer},
  #endif
  {Py_tp_methods, (void *)__pyx_methods_array},
  {Py_tp_getset, (void *)__pyx_getsets_array},
  {Py_tp_new, (void *)__pyx_tp_new_array},
  {0, 0},
};
static PyType_Spec __pyx_type___pyx_array_spec = {
  "pyogrio._io.array",
  sizeof(struct __pyx_array_obj),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_SEQUENCE,
  __pyx_type___pyx_array_slots,
};
#else

static PySequenceMethods __pyx_tp_as_sequence_array = {
  __pyx_array___len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_array, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_array = {
  __pyx_array___len__, /*mp_length*/
  __pyx_array___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_array, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_array = {
  __pyx_array_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_array = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""array", /*tp_name*/
  sizeof(struct __pyx_array_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_array, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_array, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_array, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  __pyx_tp_getattro_array, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_array, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_SEQUENCE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_array, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_array, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_array, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_MemviewEnum_obj *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_MemviewEnum_obj *)o);
  p->name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_Enum(PyObject *o) {
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_Enum) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_Enum(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_Enum(PyObject *o) {
  PyObject* tmp;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_Enum[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_MemviewEnum_1__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_MemviewEnum_3__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type___pyx_MemviewEnum_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_Enum},
  {Py_tp_repr, (void *)__pyx_MemviewEnum___repr__},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_Enum},
  {Py_tp_clear, (void *)__pyx_tp_clear_Enum},
  {Py_tp_methods, (void *)__pyx_methods_Enum},
  {Py_tp_init, (void *)__pyx_MemviewEnum___init__},
  {Py_tp_new, (void *)__pyx_tp_new_Enum},
  {0, 0},
};
static PyType_Spec __pyx_type___pyx_MemviewEnum_spec = {
  "pyogrio._io.Enum",
  sizeof(struct __pyx_MemviewEnum_obj),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type___pyx_MemviewEnum_slots,
};
#else

static PyTypeObject __pyx_type___pyx_MemviewEnum = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""Enum", /*tp_name*/
  sizeof(struct __pyx_MemviewEnum_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_Enum, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_MemviewEnum___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_Enum, /*tp_traverse*/
  __pyx_tp_clear_Enum, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_Enum, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_MemviewEnum___init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_Enum, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_memoryview __pyx_vtable_memoryview;

static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryview_obj *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryview_obj *)o);
  p->__pyx_vtab = __pyx_vtabptr_memoryview;
  p->obj = Py_None; Py_INCREF(Py_None);
  p->_size = Py_None; Py_INCREF(Py_None);
  p->view.obj = NULL;
  if (unlikely(__pyx_memoryview___cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_memoryview(PyObject *o) {
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_memoryview) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_memoryview___dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->obj);
  Py_CLEAR(p->_size);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_memoryview(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->obj) {
    e = (*v)(p->obj, a); if (e) return e;
  }
  if (p->_size) {
    e = (*v)(p->_size, a); if (e) return e;
  }
  if (p->view.obj) {
    e = (*v)(p->view.obj, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_memoryview(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  tmp = ((PyObject*)p->obj);
  p->obj = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_size);
  p->_size = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  Py_CLEAR(p->view.obj);
  return 0;
}

static PyObject *__pyx_sq_item_memoryview(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyLong_FromSsize_t(i); if(!x) return 0;
  #if CYTHON_USE_TYPE_SLOTS || (!CYTHON_USE_TYPE_SPECS && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  #else
  r = ((binaryfunc)PyType_GetSlot(Py_TYPE(o), Py_mp_subscript))(o, x);
  #endif
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_memoryview(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_memoryview___setitem__(o, i, v);
  }
  else {
    __Pyx_TypeName o_type_name;
    o_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(o));
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by " __Pyx_FMT_TYPENAME, o_type_name);
    __Pyx_DECREF_TypeName(o_type_name);
    return -1;
  }
}

static PyObject *__pyx_getprop___pyx_memoryview_T(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_1T_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_base(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4base_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_shape(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_5shape_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_strides(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_7strides_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_suboffsets(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_10suboffsets_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_ndim(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4ndim_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_itemsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_8itemsize_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_nbytes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_6nbytes_1__get__(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_15View_dot_MemoryView_10memoryview_4size_1__get__(o);
}

static PyMethodDef __pyx_methods_memoryview[] = {
  {"is_c_contig", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_memoryview_is_c_contig, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"is_f_contig", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_memoryview_is_f_contig, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"copy", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_memoryview_copy, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"copy_fortran", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_memoryview_copy_fortran, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_memoryview_1__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_memoryview_3__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_memoryview[] = {
  {"T", __pyx_getprop___pyx_memoryview_T, 0, 0, 0},
  {"base", __pyx_getprop___pyx_memoryview_base, 0, 0, 0},
  {"shape", __pyx_getprop___pyx_memoryview_shape, 0, 0, 0},
  {"strides", __pyx_getprop___pyx_memoryview_strides, 0, 0, 0},
  {"suboffsets", __pyx_getprop___pyx_memoryview_suboffsets, 0, 0, 0},
  {"ndim", __pyx_getprop___pyx_memoryview_ndim, 0, 0, 0},
  {"itemsize", __pyx_getprop___pyx_memoryview_itemsize, 0, 0, 0},
  {"nbytes", __pyx_getprop___pyx_memoryview_nbytes, 0, 0, 0},
  {"size", __pyx_getprop___pyx_memoryview_size, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
#if !CYTHON_COMPILING_IN_LIMITED_API

static PyBufferProcs __pyx_tp_as_buffer_memoryview = {
  __pyx_memoryview_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};
#endif
static PyType_Slot __pyx_type___pyx_memoryview_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_memoryview},
  {Py_tp_repr, (void *)__pyx_memoryview___repr__},
  {Py_sq_length, (void *)__pyx_memoryview___len__},
  {Py_sq_item, (void *)__pyx_sq_item_memoryview},
  {Py_mp_length, (void *)__pyx_memoryview___len__},
  {Py_mp_subscript, (void *)__pyx_memoryview___getitem__},
  {Py_mp_ass_subscript, (void *)__pyx_mp_ass_subscript_memoryview},
  {Py_tp_str, (void *)__pyx_memoryview___str__},
  #if defined(Py_bf_getbuffer)
  {Py_bf_getbuffer, (void *)__pyx_memoryview_getbuffer},
  #endif
  {Py_tp_traverse, (void *)__pyx_tp_traverse_memoryview},
  {Py_tp_clear, (void *)__pyx_tp_clear_memoryview},
  {Py_tp_methods, (void *)__pyx_methods_memoryview},
  {Py_tp_getset, (void *)__pyx_getsets_memoryview},
  {Py_tp_new, (void *)__pyx_tp_new_memoryview},
  {0, 0},
};
static PyType_Spec __pyx_type___pyx_memoryview_spec = {
  "pyogrio._io.memoryview",
  sizeof(struct __pyx_memoryview_obj),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type___pyx_memoryview_slots,
};
#else

static PySequenceMethods __pyx_tp_as_sequence_memoryview = {
  __pyx_memoryview___len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_memoryview, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_memoryview = {
  __pyx_memoryview___len__, /*mp_length*/
  __pyx_memoryview___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_memoryview, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_memoryview = {
  __pyx_memoryview_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_memoryview = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""memoryview", /*tp_name*/
  sizeof(struct __pyx_memoryview_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_memoryview, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_memoryview___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_memoryview, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_memoryview, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_memoryview___str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_memoryview, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_memoryview, /*tp_traverse*/
  __pyx_tp_clear_memoryview, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_memoryview, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_memoryview, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_memoryview, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct__memoryviewslice __pyx_vtable__memoryviewslice;

static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryviewslice_obj *p;
  PyObject *o = __pyx_tp_new_memoryview(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryviewslice_obj *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_memoryview*)__pyx_vtabptr__memoryviewslice;
  p->from_object = Py_None; Py_INCREF(Py_None);
  p->from_slice.memview = NULL;
  return o;
}

static void __pyx_tp_dealloc__memoryviewslice(PyObject *o) {
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc__memoryviewslice) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_memoryviewslice___dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->from_object);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_memoryview(o);
}

static int __pyx_tp_traverse__memoryviewslice(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  e = __pyx_tp_traverse_memoryview(o, v, a); if (e) return e;
  if (p->from_object) {
    e = (*v)(p->from_object, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear__memoryviewslice(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  __pyx_tp_clear_memoryview(o);
  tmp = ((PyObject*)p->from_object);
  p->from_object = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  __PYX_XCLEAR_MEMVIEW(&p->from_slice, 1);
  return 0;
}

static PyMethodDef __pyx_methods__memoryviewslice[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_memoryviewslice_1__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw___pyx_memoryviewslice_3__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type___pyx_memoryviewslice_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc__memoryviewslice},
  {Py_tp_doc, (void *)PyDoc_STR("Internal class for passing memoryview slices to Python")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse__memoryviewslice},
  {Py_tp_clear, (void *)__pyx_tp_clear__memoryviewslice},
  {Py_tp_methods, (void *)__pyx_methods__memoryviewslice},
  {Py_tp_new, (void *)__pyx_tp_new__memoryviewslice},
  {0, 0},
};
static PyType_Spec __pyx_type___pyx_memoryviewslice_spec = {
  "pyogrio._io._memoryviewslice",
  sizeof(struct __pyx_memoryviewslice_obj),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_SEQUENCE,
  __pyx_type___pyx_memoryviewslice_slots,
};
#else

static PyTypeObject __pyx_type___pyx_memoryviewslice = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyogrio._io.""_memoryviewslice", /*tp_name*/
  sizeof(struct __pyx_memoryviewslice_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc__memoryviewslice, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  #if CYTHON_COMPILING_IN_PYPY || 0
  __pyx_memoryview___repr__, /*tp_repr*/
  #else
  0, /*tp_repr*/
  #endif
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY || 0
  __pyx_memoryview___str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC|Py_TPFLAGS_SEQUENCE, /*tp_flags*/
  PyDoc_STR("Internal class for passing memoryview slices to Python"), /*tp_doc*/
  __pyx_tp_traverse__memoryviewslice, /*tp_traverse*/
  __pyx_tp_clear__memoryviewslice, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods__memoryviewslice, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new__memoryviewslice, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_collections_abc_Sequence = Py_None; Py_INCREF(Py_None);
  generic = Py_None; Py_INCREF(Py_None);
  strided = Py_None; Py_INCREF(Py_None);
  indirect = Py_None; Py_INCREF(Py_None);
  contiguous = Py_None; Py_INCREF(Py_None);
  indirect_contiguous = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata)) __PYX_ERR(0, 638, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata_spec, __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata) < (0)) __PYX_ERR(0, 638, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata = &__pyx_type_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata) < (0)) __PYX_ERR(0, 638, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata->tp_dictoffset && __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct____pyx_f_7pyogrio_3_io_get_metadata->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr)) __PYX_ERR(0, 657, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr_spec, __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr) < (0)) __PYX_ERR(0, 657, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr = &__pyx_type_7pyogrio_3_io___pyx_scope_struct_1_genexpr;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr) < (0)) __PYX_ERR(0, 657, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr->tp_dictoffset && __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_1_genexpr->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features)) __PYX_ERR(0, 1225, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features_spec, __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features) < (0)) __PYX_ERR(0, 1225, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features = &__pyx_type_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features) < (0)) __PYX_ERR(0, 1225, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features->tp_dictoffset && __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_2___pyx_f_7pyogrio_3_io_get_features->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow)) __PYX_ERR(0, 1908, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow_spec, __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow) < (0)) __PYX_ERR(0, 1908, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow = &__pyx_type_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow) < (0)) __PYX_ERR(0, 1908, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow->tp_dictoffset && __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_7pyogrio_3_io___pyx_scope_struct_3_ogr_open_arrow->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  __pyx_vtabptr_array = &__pyx_vtable_array;
  __pyx_vtable_array.get_memview = (PyObject *(*)(struct __pyx_array_obj *))__pyx_array_get_memview;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_array_type = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type___pyx_array_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_array_type)) __PYX_ERR(1, 118, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate->__pyx_array_type->tp_as_buffer = &__pyx_tp_as_buffer_array;
  if (!__pyx_mstate->__pyx_array_type->tp_as_buffer->bf_releasebuffer && __pyx_mstate->__pyx_array_type->tp_base->tp_as_buffer && __pyx_mstate->__pyx_array_type->tp_base->tp_as_buffer->bf_releasebuffer) {
    __pyx_mstate->__pyx_array_type->tp_as_buffer->bf_releasebuffer = __pyx_mstate->__pyx_array_type->tp_base->tp_as_buffer->bf_releasebuffer;
  }
  #elif defined(Py_bf_getbuffer) && defined(Py_bf_releasebuffer)
  /* PY_VERSION_HEX >= 0x03090000 || Py_LIMITED_API >= 0x030B0000 */
  #elif defined(_MSC_VER)
  #pragma message ("The buffer protocol is not supported in the Limited C-API < 3.11.")
  #else
  #warning "The buffer protocol is not supported in the Limited C-API < 3.11."
  #endif
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type___pyx_array_spec, __pyx_mstate->__pyx_array_type) < (0)) __PYX_ERR(1, 118, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_array_type = &__pyx_type___pyx_array;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_array_type) < (0)) __PYX_ERR(1, 118, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_array_type);
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_array_type, __pyx_vtabptr_array) < (0)) __PYX_ERR(1, 118, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_array_type) < (0)) __PYX_ERR(1, 118, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_array_type) < (0)) __PYX_ERR(1, 118, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_MemviewEnum_type = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type___pyx_MemviewEnum_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_MemviewEnum_type)) __PYX_ERR(1, 307, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type___pyx_MemviewEnum_spec, __pyx_mstate->__pyx_MemviewEnum_type) < (0)) __PYX_ERR(1, 307, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_MemviewEnum_type = &__pyx_type___pyx_MemviewEnum;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_MemviewEnum_type) < (0)) __PYX_ERR(1, 307, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_MemviewEnum_type);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_MemviewEnum_type->tp_dictoffset && __pyx_mstate->__pyx_MemviewEnum_type->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_MemviewEnum_type->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_MemviewEnum_type) < (0)) __PYX_ERR(1, 307, __pyx_L1_error)
  __pyx_vtabptr_memoryview = &__pyx_vtable_memoryview;
  __pyx_vtable_memoryview.get_item_pointer = (char *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_get_item_pointer;
  __pyx_vtable_memoryview.is_slice = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_is_slice;
  __pyx_vtable_memoryview.setitem_slice_assignment = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_slice_assignment;
  __pyx_vtable_memoryview.setitem_slice_assign_scalar = (PyObject *(*)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_setitem_slice_assign_scalar;
  __pyx_vtable_memoryview.setitem_indexed = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_indexed;
  __pyx_vtable_memoryview.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryview_convert_item_to_object;
  __pyx_vtable_memoryview.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryview_assign_item_from_object;
  __pyx_vtable_memoryview._get_base = (PyObject *(*)(struct __pyx_memoryview_obj *))__pyx_memoryview__get_base;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_memoryview_type = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type___pyx_memoryview_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_memoryview_type)) __PYX_ERR(1, 342, __pyx_L1_error)
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate->__pyx_memoryview_type->tp_as_buffer = &__pyx_tp_as_buffer_memoryview;
  if (!__pyx_mstate->__pyx_memoryview_type->tp_as_buffer->bf_releasebuffer && __pyx_mstate->__pyx_memoryview_type->tp_base->tp_as_buffer && __pyx_mstate->__pyx_memoryview_type->tp_base->tp_as_buffer->bf_releasebuffer) {
    __pyx_mstate->__pyx_memoryview_type->tp_as_buffer->bf_releasebuffer = __pyx_mstate->__pyx_memoryview_type->tp_base->tp_as_buffer->bf_releasebuffer;
  }
  #elif defined(Py_bf_getbuffer) && defined(Py_bf_releasebuffer)
  /* PY_VERSION_HEX >= 0x03090000 || Py_LIMITED_API >= 0x030B0000 */
  #elif defined(_MSC_VER)
  #pragma message ("The buffer protocol is not supported in the Limited C-API < 3.11.")
  #else
  #warning "The buffer protocol is not supported in the Limited C-API < 3.11."
  #endif
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type___pyx_memoryview_spec, __pyx_mstate->__pyx_memoryview_type) < (0)) __PYX_ERR(1, 342, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_memoryview_type = &__pyx_type___pyx_memoryview;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_memoryview_type) < (0)) __PYX_ERR(1, 342, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_memoryview_type);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_memoryview_type->tp_dictoffset && __pyx_mstate->__pyx_memoryview_type->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_memoryview_type->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_memoryview_type, __pyx_vtabptr_memoryview) < (0)) __PYX_ERR(1, 342, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_memoryview_type) < (0)) __PYX_ERR(1, 342, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_memoryview_type) < (0)) __PYX_ERR(1, 342, __pyx_L1_error)
  __pyx_vtabptr__memoryviewslice = &__pyx_vtable__memoryviewslice;
  __pyx_vtable__memoryviewslice.__pyx_base = *__pyx_vtabptr_memoryview;
  __pyx_vtable__memoryviewslice.__pyx_base.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryviewslice_convert_item_to_object;
  __pyx_vtable__memoryviewslice.__pyx_base.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryviewslice_assign_item_from_object;
  __pyx_vtable__memoryviewslice.__pyx_base._get_base = (PyObject *(*)(struct __pyx_memoryview_obj *))__pyx_memoryviewslice__get_base;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_memoryview_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_memoryviewslice_type = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type___pyx_memoryviewslice_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_memoryviewslice_type)) __PYX_ERR(1, 856, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type___pyx_memoryviewslice_spec, __pyx_mstate->__pyx_memoryviewslice_type) < (0)) __PYX_ERR(1, 856, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_memoryviewslice_type = &__pyx_type___pyx_memoryviewslice;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_memoryviewslice_type->tp_base = __pyx_mstate_global->__pyx_memoryview_type;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_memoryviewslice_type) < (0)) __PYX_ERR(1, 856, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_memoryviewslice_type);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_memoryviewslice_type->tp_dictoffset && __pyx_mstate->__pyx_memoryviewslice_type->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_memoryviewslice_type->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_memoryviewslice_type, __pyx_vtabptr__memoryviewslice) < (0)) __PYX_ERR(1, 856, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_memoryviewslice_type) < (0)) __PYX_ERR(1, 856, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_memoryviewslice_type) < (0)) __PYX_ERR(1, 856, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("pyogrio._err"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7pyogrio_4_err_ErrorHandler = __Pyx_ImportType_3_2_8(__pyx_t_1, "pyogrio._err", "ErrorHandler",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_7pyogrio_4_err_ErrorHandler), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(struct __pyx_obj_7pyogrio_4_err_ErrorHandler),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_7pyogrio_4_err_ErrorHandler), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(struct __pyx_obj_7pyogrio_4_err_ErrorHandler),
  #else
  sizeof(struct __pyx_obj_7pyogrio_4_err_ErrorHandler), __PYX_GET_STRUCT_ALIGNMENT_3_2_8(struct __pyx_obj_7pyogrio_4_err_ErrorHandler),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_8); if (!__pyx_mstate->__pyx_ptype_7pyogrio_4_err_ErrorHandler) __PYX_ERR(2, 5, __pyx_L1_error)
  __pyx_vtabptr_7pyogrio_4_err_ErrorHandler = (struct __pyx_vtabstruct_7pyogrio_4_err_ErrorHandler*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_7pyogrio_4_err_ErrorHandler); if (unlikely(!__pyx_vtabptr_7pyogrio_4_err_ErrorHandler)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("pyogrio._err"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_void_int_int_void_void);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 42;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_7pyogrio_4_err_check_last_error, (void (**)(void))&__pyx_f_7pyogrio_4_err_check_int, (void (**)(void))&__pyx_f_7pyogrio_4_err_check_pointer, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_8(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("pyogrio._ogr"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_char_const_struct___pyx);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 95;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_7pyogrio_4_ogr_get_string, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_8(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("pyogrio._geometry"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_OGRwkbGeometryType_PyObject_PyOb);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 51;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_7pyogrio_9_geometry_get_geometry_type_code, (void (**)(void))&__pyx_f_7pyogrio_9_geometry_get_geometry_type, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_8(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  {
    __pyx_t_1 = PyImport_ImportModule("pyogrio._vsi"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_PyObject_PyObject_PyObj);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 107;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_7pyogrio_4_vsi_read_buffer_to_vsimem, (void (**)(void))&__pyx_f_7pyogrio_4_vsi_get_ogr_vsimem_write_path, (void (**)(void))&__pyx_f_7pyogrio_4_vsi_read_vsimem_to_buffer, (void (**)(void))&__pyx_f_7pyogrio_4_vsi_vsimem_rmtree_toplevel, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_8(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__io(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__io},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "_io",
      __pyx_k_IO_support_for_OGR_vector_data_s, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit__io(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__io(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__io(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  static PyThread_type_lock __pyx_t_7[8];
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_io' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "_io" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_NOT_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit__io", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_pyogrio___io) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyogrio._io")) {
      if (unlikely((PyDict_SetItemString(modules, "pyogrio._io", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  (void)__Pyx_modinit_function_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/

  /* "View.MemoryView":108
 * 
 * cdef object __pyx_collections_abc_Sequence "__pyx_collections_abc_Sequence"
 * try:             # <<<<<<<<<<<<<<
 *     __pyx_collections_abc_Sequence = __import__("collections.abc").abc.Sequence
 * except:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "View.MemoryView":109
 * cdef object __pyx_collections_abc_Sequence "__pyx_collections_abc_Sequence"
 * try:
 *     __pyx_collections_abc_Sequence = __import__("collections.abc").abc.Sequence             # <<<<<<<<<<<<<<
 * except:
 * 
*/
      __pyx_t_5 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_collections_abc};
        __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin___import__, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 109, __pyx_L2_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_abc); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 109, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_Sequence); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 109, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XGOTREF(__pyx_collections_abc_Sequence);
      __Pyx_DECREF_SET(__pyx_collections_abc_Sequence, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;

      /* "View.MemoryView":108
 * 
 * cdef object __pyx_collections_abc_Sequence "__pyx_collections_abc_Sequence"
 * try:             # <<<<<<<<<<<<<<
 *     __pyx_collections_abc_Sequence = __import__("collections.abc").abc.Sequence
 * except:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L7_try_end;
    __pyx_L2_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "View.MemoryView":110
 * try:
 *     __pyx_collections_abc_Sequence = __import__("collections.abc").abc.Sequence
 * except:             # <<<<<<<<<<<<<<
 * 
 *     __pyx_collections_abc_Sequence = None
*/
    /*except:*/ {
      __Pyx_ErrRestore(0,0,0);

      /* "View.MemoryView":112
 * except:
 * 
 *     __pyx_collections_abc_Sequence = None             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __Pyx_INCREF(Py_None);
      __Pyx_XGOTREF(__pyx_collections_abc_Sequence);
      __Pyx_DECREF_SET(__pyx_collections_abc_Sequence, Py_None);
      __Pyx_GIVEREF(Py_None);
      goto __pyx_L3_exception_handled;
    }
    __pyx_L3_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L7_try_end:;
  }

  /* "View.MemoryView":247
 * 
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_1);
    /*try:*/ {

      /* "View.MemoryView":248
 * 
 *     try:
 *         count = __pyx_collections_abc_Sequence.count             # <<<<<<<<<<<<<<
 *         index = __pyx_collections_abc_Sequence.index
 *     except:
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_collections_abc_Sequence, __pyx_mstate_global->__pyx_n_u_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 248, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_array_type, __pyx_mstate_global->__pyx_n_u_count, __pyx_t_4) < (0)) __PYX_ERR(1, 248, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "View.MemoryView":249
 *     try:
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index             # <<<<<<<<<<<<<<
 *     except:
 *         pass
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_collections_abc_Sequence, __pyx_mstate_global->__pyx_n_u_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 249, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_array_type, __pyx_mstate_global->__pyx_n_u_index, __pyx_t_4) < (0)) __PYX_ERR(1, 249, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "View.MemoryView":247
 * 
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index
*/
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L15_try_end;
    __pyx_L10_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "View.MemoryView":250
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index
 *     except:             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
    /*except:*/ {
      __Pyx_ErrRestore(0,0,0);
      goto __pyx_L11_exception_handled;
    }
    __pyx_L11_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_2, __pyx_t_1);
    __pyx_L15_try_end:;
  }

  /* "View.MemoryView":315
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
*/
  __pyx_t_5 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_strided_and_direct_or_indirect};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_MemviewEnum_type, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 315, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_4);
  }
  __Pyx_XGOTREF(generic);
  __Pyx_DECREF_SET(generic, ((PyObject *)__pyx_t_4));
  __Pyx_GIVEREF((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "View.MemoryView":316
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
*/
  __pyx_t_5 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_strided_and_direct};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_MemviewEnum_type, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 316, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_4);
  }
  __Pyx_XGOTREF(strided);
  __Pyx_DECREF_SET(strided, ((PyObject *)__pyx_t_4));
  __Pyx_GIVEREF((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "View.MemoryView":317
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_5 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_strided_and_indirect};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_MemviewEnum_type, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 317, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_4);
  }
  __Pyx_XGOTREF(indirect);
  __Pyx_DECREF_SET(indirect, ((PyObject *)__pyx_t_4));
  __Pyx_GIVEREF((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "View.MemoryView":320
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
*/
  __pyx_t_5 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_contiguous_and_direct};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_MemviewEnum_type, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 320, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_4);
  }
  __Pyx_XGOTREF(contiguous);
  __Pyx_DECREF_SET(contiguous, ((PyObject *)__pyx_t_4));
  __Pyx_GIVEREF((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "View.MemoryView":321
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_5 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_contiguous_and_indirect};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_MemviewEnum_type, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 321, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_4);
  }
  __Pyx_XGOTREF(indirect_contiguous);
  __Pyx_DECREF_SET(indirect_contiguous, ((PyObject *)__pyx_t_4));
  __Pyx_GIVEREF((PyObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "View.MemoryView":329
 * 
 * 
 * cdef int __pyx_memoryview_thread_locks_used = 0             # <<<<<<<<<<<<<<
 * cdef PyThread_type_lock[8] __pyx_memoryview_thread_locks = [
 *     PyThread_allocate_lock(),
*/
  __pyx_memoryview_thread_locks_used = 0;

  /* "View.MemoryView":330
 * 
 * cdef int __pyx_memoryview_thread_locks_used = 0
 * cdef PyThread_type_lock[8] __pyx_memoryview_thread_locks = [             # <<<<<<<<<<<<<<
 *     PyThread_allocate_lock(),
 *     PyThread_allocate_lock(),
*/
  __pyx_t_7[0] = PyThread_allocate_lock();
  __pyx_t_7[1] = PyThread_allocate_lock();
  __pyx_t_7[2] = PyThread_allocate_lock();
  __pyx_t_7[3] = PyThread_allocate_lock();
  __pyx_t_7[4] = PyThread_allocate_lock();
  __pyx_t_7[5] = PyThread_allocate_lock();
  __pyx_t_7[6] = PyThread_allocate_lock();
  __pyx_t_7[7] = PyThread_allocate_lock();
  memcpy(&(__pyx_memoryview_thread_locks[0]), __pyx_t_7, sizeof(__pyx_memoryview_thread_locks[0]) * (8));

  /* "View.MemoryView":888
 * 
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "View.MemoryView":889
 * 
 *     try:
 *         count = __pyx_collections_abc_Sequence.count             # <<<<<<<<<<<<<<
 *         index = __pyx_collections_abc_Sequence.index
 *     except:
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_collections_abc_Sequence, __pyx_mstate_global->__pyx_n_u_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 889, __pyx_L18_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_memoryviewslice_type, __pyx_mstate_global->__pyx_n_u_count, __pyx_t_4) < (0)) __PYX_ERR(1, 889, __pyx_L18_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "View.MemoryView":890
 *     try:
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index             # <<<<<<<<<<<<<<
 *     except:
 *         pass
*/
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_collections_abc_Sequence, __pyx_mstate_global->__pyx_n_u_index); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 890, __pyx_L18_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_memoryviewslice_type, __pyx_mstate_global->__pyx_n_u_index, __pyx_t_4) < (0)) __PYX_ERR(1, 890, __pyx_L18_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "View.MemoryView":888
 * 
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L23_try_end;
    __pyx_L18_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "View.MemoryView":891
 *         count = __pyx_collections_abc_Sequence.count
 *         index = __pyx_collections_abc_Sequence.index
 *     except:             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
    /*except:*/ {
      __Pyx_ErrRestore(0,0,0);
      goto __pyx_L19_exception_handled;
    }
    __pyx_L19_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L23_try_end:;
  }

  /* "View.MemoryView":894
 *         pass
 * 
 * try:             # <<<<<<<<<<<<<<
 *     if __pyx_collections_abc_Sequence:
 * 
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_1);
    /*try:*/ {

      /* "View.MemoryView":895
 * 
 * try:
 *     if __pyx_collections_abc_Sequence:             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_collections_abc_Sequence); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(1, 895, __pyx_L26_error)
      if (__pyx_t_8) {

        /* "View.MemoryView":899
 * 
 * 
 *         __pyx_collections_abc_Sequence.register(_memoryviewslice)             # <<<<<<<<<<<<<<
 *         __pyx_collections_abc_Sequence.register(array)
 * except:
*/
        __pyx_t_5 = __pyx_collections_abc_Sequence;
        __Pyx_INCREF(__pyx_t_5);
        __pyx_t_6 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, ((PyObject *)__pyx_mstate_global->__pyx_memoryviewslice_type)};
          __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_register, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 899, __pyx_L26_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "View.MemoryView":900
 * 
 *         __pyx_collections_abc_Sequence.register(_memoryviewslice)
 *         __pyx_collections_abc_Sequence.register(array)             # <<<<<<<<<<<<<<
 * except:
 *     pass  # ignore failure, it's a minor issue
*/
        __pyx_t_5 = __pyx_collections_abc_Sequence;
        __Pyx_INCREF(__pyx_t_5);
        __pyx_t_6 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, ((PyObject *)__pyx_mstate_global->__pyx_array_type)};
          __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_register, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 900, __pyx_L26_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "View.MemoryView":895
 * 
 * try:
 *     if __pyx_collections_abc_Sequence:             # <<<<<<<<<<<<<<
 * 
 * 
*/
      }

      /* "View.MemoryView":894
 *         pass
 * 
 * try:             # <<<<<<<<<<<<<<
 *     if __pyx_collections_abc_Sequence:
 * 
*/
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L31_try_end;
    __pyx_L26_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "View.MemoryView":901
 *         __pyx_collections_abc_Sequence.register(_memoryviewslice)
 *         __pyx_collections_abc_Sequence.register(array)
 * except:             # <<<<<<<<<<<<<<
 *     pass  # ignore failure, it's a minor issue
 * 
*/
    /*except:*/ {
      __Pyx_ErrRestore(0,0,0);
      goto __pyx_L27_exception_handled;
    }
    __pyx_L27_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_2, __pyx_t_1);
    __pyx_L31_try_end:;
  }

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_Enum(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x82a3537, 0x6ae9995, 0xb068931, b'name')
*/
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_15View_dot_MemoryView_1__pyx_unpickle_Enum, NULL, __pyx_mstate_global->__pyx_n_u_View_MemoryView); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_Enum, __pyx_t_4) < (0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":5
 * """IO support for OGR vector data sources."""
 * 
 * import contextlib             # <<<<<<<<<<<<<<
 * import datetime
 * import locale
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_contextlib, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_contextlib, __pyx_t_4) < (0)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":6
 * 
 * import contextlib
 * import datetime             # <<<<<<<<<<<<<<
 * import locale
 * import logging
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_datetime, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_datetime, __pyx_t_4) < (0)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":7
 * import contextlib
 * import datetime
 * import locale             # <<<<<<<<<<<<<<
 * import logging
 * import os
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_locale, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_locale, __pyx_t_4) < (0)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":8
 * import datetime
 * import locale
 * import logging             # <<<<<<<<<<<<<<
 * import os
 * import shutil
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_logging, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_logging, __pyx_t_4) < (0)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":9
 * import locale
 * import logging
 * import os             # <<<<<<<<<<<<<<
 * import shutil
 * import sys
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_os, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_os, __pyx_t_4) < (0)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":10
 * import logging
 * import os
 * import shutil             # <<<<<<<<<<<<<<
 * import sys
 * import warnings
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_shutil, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_shutil, __pyx_t_4) < (0)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":11
 * import os
 * import shutil
 * import sys             # <<<<<<<<<<<<<<
 * import warnings
 * IF CTE_GDAL_VERSION < (3, 8, 0):
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_sys, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_sys, __pyx_t_4) < (0)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":12
 * import shutil
 * import sys
 * import warnings             # <<<<<<<<<<<<<<
 * IF CTE_GDAL_VERSION < (3, 8, 0):
 *     from pathlib import Path
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_warnings, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 12, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_warnings, __pyx_t_4) < (0)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":24
 * from cpython.pycapsule cimport PyCapsule_New, PyCapsule_GetPointer
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * 
 * from pyogrio._err cimport (
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_numpy, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_np, __pyx_t_4) < (0)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":29
 *     check_last_error, check_int, check_pointer, ErrorHandler
 * )
 * from pyogrio._err import (             # <<<<<<<<<<<<<<
 *     CPLE_BaseError,
 *     CPLE_NotSupportedError,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CPLE_BaseError,__pyx_mstate_global->__pyx_n_u_CPLE_NotSupportedError,__pyx_mstate_global->__pyx_n_u_NullPointerError,__pyx_mstate_global->__pyx_n_u_capture_errors};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pyogrio__err, __pyx_imported_names, 4, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  }
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CPLE_BaseError,__pyx_mstate_global->__pyx_n_u_CPLE_NotSupportedError,__pyx_mstate_global->__pyx_n_u_NullPointerError,__pyx_mstate_global->__pyx_n_u_capture_errors};
    for (__pyx_t_9=0; __pyx_t_9 < 4; __pyx_t_9++) {
      __pyx_t_5 = __Pyx_ImportFrom(__pyx_t_4, __pyx_imported_names[__pyx_t_9]); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 29, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_9], __pyx_t_5) < (0)) __PYX_ERR(0, 29, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":37
 * from pyogrio._geometry cimport get_geometry_type, get_geometry_type_code
 * from pyogrio._ogr cimport *
 * from pyogrio._ogr import MULTI_EXTENSIONS             # <<<<<<<<<<<<<<
 * from pyogrio._vsi cimport *
 * from pyogrio.errors import (
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_MULTI_EXTENSIONS};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pyogrio__ogr, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  }
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_MULTI_EXTENSIONS};
    __pyx_t_9 = 0; {
      __pyx_t_5 = __Pyx_ImportFrom(__pyx_t_4, __pyx_imported_names[__pyx_t_9]); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 37, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_9], __pyx_t_5) < (0)) __PYX_ERR(0, 37, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":39
 * from pyogrio._ogr import MULTI_EXTENSIONS
 * from pyogrio._vsi cimport *
 * from pyogrio.errors import (             # <<<<<<<<<<<<<<
 *     CRSError, DataSourceError, DataLayerError, GeometryError, FieldError, FeatureError
 * )
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CRSError,__pyx_mstate_global->__pyx_n_u_DataSourceError,__pyx_mstate_global->__pyx_n_u_DataLayerError,__pyx_mstate_global->__pyx_n_u_GeometryError,__pyx_mstate_global->__pyx_n_u_FieldError,__pyx_mstate_global->__pyx_n_u_FeatureError};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pyogrio_errors, __pyx_imported_names, 6, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  }
  __pyx_t_4 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_4);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CRSError,__pyx_mstate_global->__pyx_n_u_DataSourceError,__pyx_mstate_global->__pyx_n_u_DataLayerError,__pyx_mstate_global->__pyx_n_u_GeometryError,__pyx_mstate_global->__pyx_n_u_FieldError,__pyx_mstate_global->__pyx_n_u_FeatureError};
    for (__pyx_t_9=0; __pyx_t_9 < 6; __pyx_t_9++) {
      __pyx_t_5 = __Pyx_ImportFrom(__pyx_t_4, __pyx_imported_names[__pyx_t_9]); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 39, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_9], __pyx_t_5) < (0)) __PYX_ERR(0, 39, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":43
 * )
 * 
 * log = logging.getLogger(__name__)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_logging); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_getLogger); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_10};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_log, __pyx_t_4) < (0)) __PYX_ERR(0, 43, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":48
 * # Mapping of OGR integer field types to Python field type names
 * # (index in array is the integer field type)
 * FIELD_TYPES = [             # <<<<<<<<<<<<<<
 *     "int32",           # OFTInteger, Simple 32bit integer
 *     "list(int32)",     # OFTIntegerList, List of 32bit integers
*/
  __pyx_t_4 = __Pyx_PyList_Pack(14, __pyx_mstate_global->__pyx_n_u_int32, __pyx_mstate_global->__pyx_kp_u_list_int32, __pyx_mstate_global->__pyx_n_u_float64, __pyx_mstate_global->__pyx_kp_u_list_float64, __pyx_mstate_global->__pyx_n_u_object, __pyx_mstate_global->__pyx_kp_u_list_str, Py_None, Py_None, __pyx_mstate_global->__pyx_n_u_object, __pyx_mstate_global->__pyx_kp_u_datetime64_D, __pyx_mstate_global->__pyx_n_u_object, __pyx_mstate_global->__pyx_kp_u_datetime64_ms, __pyx_mstate_global->__pyx_n_u_int64, __pyx_mstate_global->__pyx_kp_u_list_int64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_FIELD_TYPES, __pyx_t_4) < (0)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":68
 * # (index in array is the integer field type)
 * FIELD_TYPE_NAMES = {
 *     OFTInteger: "OFTInteger",                # Simple 32bit integer             # <<<<<<<<<<<<<<
 *     OFTIntegerList: "OFTIntegerList",        # List of 32bit integers
 *     OFTReal: "OFTReal",                      # Double Precision floating point
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(14); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTInteger) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":69
 * FIELD_TYPE_NAMES = {
 *     OFTInteger: "OFTInteger",                # Simple 32bit integer
 *     OFTIntegerList: "OFTIntegerList",        # List of 32bit integers             # <<<<<<<<<<<<<<
 *     OFTReal: "OFTReal",                      # Double Precision floating point
 *     OFTRealList: "OFTRealList",              # List of doubles
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTIntegerList); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTIntegerList) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":70
 *     OFTInteger: "OFTInteger",                # Simple 32bit integer
 *     OFTIntegerList: "OFTIntegerList",        # List of 32bit integers
 *     OFTReal: "OFTReal",                      # Double Precision floating point             # <<<<<<<<<<<<<<
 *     OFTRealList: "OFTRealList",              # List of doubles
 *     OFTString: "OFTString",                  # String of UTF-8 chars
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTReal); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTReal) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":71
 *     OFTIntegerList: "OFTIntegerList",        # List of 32bit integers
 *     OFTReal: "OFTReal",                      # Double Precision floating point
 *     OFTRealList: "OFTRealList",              # List of doubles             # <<<<<<<<<<<<<<
 *     OFTString: "OFTString",                  # String of UTF-8 chars
 *     OFTStringList: "OFTStringList",          # Array of strings
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTRealList); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTRealList) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":72
 *     OFTReal: "OFTReal",                      # Double Precision floating point
 *     OFTRealList: "OFTRealList",              # List of doubles
 *     OFTString: "OFTString",                  # String of UTF-8 chars             # <<<<<<<<<<<<<<
 *     OFTStringList: "OFTStringList",          # Array of strings
 *     OFTWideString: "OFTWideString",          # deprecated, not supported
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTString); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 72, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTString) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":73
 *     OFTRealList: "OFTRealList",              # List of doubles
 *     OFTString: "OFTString",                  # String of UTF-8 chars
 *     OFTStringList: "OFTStringList",          # Array of strings             # <<<<<<<<<<<<<<
 *     OFTWideString: "OFTWideString",          # deprecated, not supported
 *     OFTWideStringList: "OFTWideStringList",  # deprecated, not supported
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTStringList); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTStringList) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":74
 *     OFTString: "OFTString",                  # String of UTF-8 chars
 *     OFTStringList: "OFTStringList",          # Array of strings
 *     OFTWideString: "OFTWideString",          # deprecated, not supported             # <<<<<<<<<<<<<<
 *     OFTWideStringList: "OFTWideStringList",  # deprecated, not supported
 *     OFTBinary: "OFTBinary",                  # Raw Binary data
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTWideString); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTWideString) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":75
 *     OFTStringList: "OFTStringList",          # Array of strings
 *     OFTWideString: "OFTWideString",          # deprecated, not supported
 *     OFTWideStringList: "OFTWideStringList",  # deprecated, not supported             # <<<<<<<<<<<<<<
 *     OFTBinary: "OFTBinary",                  # Raw Binary data
 *     OFTDate: "OFTDate",                      # Date
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTWideStringList); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTWideStringList) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":76
 *     OFTWideString: "OFTWideString",          # deprecated, not supported
 *     OFTWideStringList: "OFTWideStringList",  # deprecated, not supported
 *     OFTBinary: "OFTBinary",                  # Raw Binary data             # <<<<<<<<<<<<<<
 *     OFTDate: "OFTDate",                      # Date
 *     OFTTime: "OFTTime",                      # Time: not directly supported in numpy
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTBinary); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTBinary) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":77
 *     OFTWideStringList: "OFTWideStringList",  # deprecated, not supported
 *     OFTBinary: "OFTBinary",                  # Raw Binary data
 *     OFTDate: "OFTDate",                      # Date             # <<<<<<<<<<<<<<
 *     OFTTime: "OFTTime",                      # Time: not directly supported in numpy
 *     OFTDateTime: "OFTDateTime",              # Date and Time
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTDate); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTDate) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":78
 *     OFTBinary: "OFTBinary",                  # Raw Binary data
 *     OFTDate: "OFTDate",                      # Date
 *     OFTTime: "OFTTime",                      # Time: not directly supported in numpy             # <<<<<<<<<<<<<<
 *     OFTDateTime: "OFTDateTime",              # Date and Time
 *     OFTInteger64: "OFTInteger64",            # Single 64bit integer
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTTime); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTTime) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":79
 *     OFTDate: "OFTDate",                      # Date
 *     OFTTime: "OFTTime",                      # Time: not directly supported in numpy
 *     OFTDateTime: "OFTDateTime",              # Date and Time             # <<<<<<<<<<<<<<
 *     OFTInteger64: "OFTInteger64",            # Single 64bit integer
 *     OFTInteger64List: "OFTInteger64List",    # List of 64bit integers
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTDateTime); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTDateTime) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":80
 *     OFTTime: "OFTTime",                      # Time: not directly supported in numpy
 *     OFTDateTime: "OFTDateTime",              # Date and Time
 *     OFTInteger64: "OFTInteger64",            # Single 64bit integer             # <<<<<<<<<<<<<<
 *     OFTInteger64List: "OFTInteger64List",    # List of 64bit integers
 * }
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger64); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTInteger64) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":81
 *     OFTDateTime: "OFTDateTime",              # Date and Time
 *     OFTInteger64: "OFTInteger64",            # Single 64bit integer
 *     OFTInteger64List: "OFTInteger64List",    # List of 64bit integers             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger64List); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFTInteger64List) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_FIELD_TYPE_NAMES, __pyx_t_4) < (0)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":85
 * 
 * FIELD_SUBTYPES = {
 *     OFSTNone: None,           # No subtype             # <<<<<<<<<<<<<<
 *     OFSTBoolean: "bool",      # Boolean integer
 *     OFSTInt16: "int16",       # Signed 16-bit integer
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, Py_None) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":86
 * FIELD_SUBTYPES = {
 *     OFSTNone: None,           # No subtype
 *     OFSTBoolean: "bool",      # Boolean integer             # <<<<<<<<<<<<<<
 *     OFSTInt16: "int16",       # Signed 16-bit integer
 *     OFSTFloat32: "float32",   # Single precision (32 bit) floating point
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTBoolean); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_bool) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":87
 *     OFSTNone: None,           # No subtype
 *     OFSTBoolean: "bool",      # Boolean integer
 *     OFSTInt16: "int16",       # Signed 16-bit integer             # <<<<<<<<<<<<<<
 *     OFSTFloat32: "float32",   # Single precision (32 bit) floating point
 * }
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTInt16); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_int16) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":88
 *     OFSTBoolean: "bool",      # Boolean integer
 *     OFSTInt16: "int16",       # Signed 16-bit integer
 *     OFSTFloat32: "float32",   # Single precision (32 bit) floating point             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTFloat32); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_float32) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_FIELD_SUBTYPES, __pyx_t_4) < (0)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":92
 * 
 * FIELD_SUBTYPE_NAMES = {
 *     OFSTNone: "OFSTNone",             # No subtype             # <<<<<<<<<<<<<<
 *     OFSTBoolean: "OFSTBoolean",       # Boolean integer
 *     OFSTInt16: "OFSTInt16",           # Signed 16-bit integer
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFSTNone) < (0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":93
 * FIELD_SUBTYPE_NAMES = {
 *     OFSTNone: "OFSTNone",             # No subtype
 *     OFSTBoolean: "OFSTBoolean",       # Boolean integer             # <<<<<<<<<<<<<<
 *     OFSTInt16: "OFSTInt16",           # Signed 16-bit integer
 *     OFSTFloat32: "OFSTFloat32",       # Single precision (32 bit) floating point
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTBoolean); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFSTBoolean) < (0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":94
 *     OFSTNone: "OFSTNone",             # No subtype
 *     OFSTBoolean: "OFSTBoolean",       # Boolean integer
 *     OFSTInt16: "OFSTInt16",           # Signed 16-bit integer             # <<<<<<<<<<<<<<
 *     OFSTFloat32: "OFSTFloat32",       # Single precision (32 bit) floating point
 *     OFSTJSON: "OFSTJSON",             # String with JSON content
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTInt16); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFSTInt16) < (0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":95
 *     OFSTBoolean: "OFSTBoolean",       # Boolean integer
 *     OFSTInt16: "OFSTInt16",           # Signed 16-bit integer
 *     OFSTFloat32: "OFSTFloat32",       # Single precision (32 bit) floating point             # <<<<<<<<<<<<<<
 *     OFSTJSON: "OFSTJSON",             # String with JSON content
 *     OFSTUUID: "OFSTUUID",             # String with a UUID
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTFloat32); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFSTFloat32) < (0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":96
 *     OFSTInt16: "OFSTInt16",           # Signed 16-bit integer
 *     OFSTFloat32: "OFSTFloat32",       # Single precision (32 bit) floating point
 *     OFSTJSON: "OFSTJSON",             # String with JSON content             # <<<<<<<<<<<<<<
 *     OFSTUUID: "OFSTUUID",             # String with a UUID
 *     OFSTMaxSubType: "OFSTMaxSubType",
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTJSON); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFSTJSON) < (0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":97
 *     OFSTFloat32: "OFSTFloat32",       # Single precision (32 bit) floating point
 *     OFSTJSON: "OFSTJSON",             # String with JSON content
 *     OFSTUUID: "OFSTUUID",             # String with a UUID             # <<<<<<<<<<<<<<
 *     OFSTMaxSubType: "OFSTMaxSubType",
 * }
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTUUID); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFSTUUID) < (0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":98
 *     OFSTJSON: "OFSTJSON",             # String with JSON content
 *     OFSTUUID: "OFSTUUID",             # String with a UUID
 *     OFSTMaxSubType: "OFSTMaxSubType",             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldSubType(OFSTMaxSubType); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_4, __pyx_t_11, __pyx_mstate_global->__pyx_n_u_OFSTMaxSubType) < (0)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_FIELD_SUBTYPE_NAMES, __pyx_t_4) < (0)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":103
 * # Mapping of numpy ndarray dtypes to (field type, subtype)
 * DTYPE_OGR_FIELD_TYPES = {
 *     "int8": (OFTInteger, OFSTInt16),             # <<<<<<<<<<<<<<
 *     "int16": (OFTInteger, OFSTInt16),
 *     "int32": (OFTInteger, OFSTNone),
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTInt16); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int8, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":104
 * DTYPE_OGR_FIELD_TYPES = {
 *     "int8": (OFTInteger, OFSTInt16),
 *     "int16": (OFTInteger, OFSTInt16),             # <<<<<<<<<<<<<<
 *     "int32": (OFTInteger, OFSTNone),
 *     "int": (OFTInteger64, OFSTNone),
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTInteger); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTInt16); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int16, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":105
 *     "int8": (OFTInteger, OFSTInt16),
 *     "int16": (OFTInteger, OFSTInt16),
 *     "int32": (OFTInteger, OFSTNone),             # <<<<<<<<<<<<<<
 *     "int": (OFTInteger64, OFSTNone),
 *     "int64": (OFTInteger64, OFSTNone),
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int32, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":106
 *     "int16": (OFTInteger, OFSTInt16),
 *     "int32": (OFTInteger, OFSTNone),
 *     "int": (OFTInteger64, OFSTNone),             # <<<<<<<<<<<<<<
 *     "int64": (OFTInteger64, OFSTNone),
 *     # unsigned ints have to be converted to ints; these are converted
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTInteger64); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":107
 *     "int32": (OFTInteger, OFSTNone),
 *     "int": (OFTInteger64, OFSTNone),
 *     "int64": (OFTInteger64, OFSTNone),             # <<<<<<<<<<<<<<
 *     # unsigned ints have to be converted to ints; these are converted
 *     # to the next largest integer size
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger64); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int64, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":110
 *     # unsigned ints have to be converted to ints; these are converted
 *     # to the next largest integer size
 *     "uint8": (OFTInteger, OFSTInt16),             # <<<<<<<<<<<<<<
 *     "uint16": (OFTInteger, OFSTNone),
 *     "uint32": (OFTInteger64, OFSTNone),
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTInteger); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTInt16); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint8, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":111
 *     # to the next largest integer size
 *     "uint8": (OFTInteger, OFSTInt16),
 *     "uint16": (OFTInteger, OFSTNone),             # <<<<<<<<<<<<<<
 *     "uint32": (OFTInteger64, OFSTNone),
 *     # TODO: these might get truncated, check maximum value and raise error
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint16, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":112
 *     "uint8": (OFTInteger, OFSTInt16),
 *     "uint16": (OFTInteger, OFSTNone),
 *     "uint32": (OFTInteger64, OFSTNone),             # <<<<<<<<<<<<<<
 *     # TODO: these might get truncated, check maximum value and raise error
 *     "uint": (OFTInteger64, OFSTNone),
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTInteger64); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint32, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":114
 *     "uint32": (OFTInteger64, OFSTNone),
 *     # TODO: these might get truncated, check maximum value and raise error
 *     "uint": (OFTInteger64, OFSTNone),             # <<<<<<<<<<<<<<
 *     "uint64": (OFTInteger64, OFSTNone),
 * 
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger64); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":115
 *     # TODO: these might get truncated, check maximum value and raise error
 *     "uint": (OFTInteger64, OFSTNone),
 *     "uint64": (OFTInteger64, OFSTNone),             # <<<<<<<<<<<<<<
 * 
 *     # bool is handled as integer with boolean subtype
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTInteger64); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint64, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":118
 * 
 *     # bool is handled as integer with boolean subtype
 *     "bool": (OFTInteger, OFSTBoolean),             # <<<<<<<<<<<<<<
 * 
 *     "float32": (OFTReal, OFSTFloat32),
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTInteger); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTBoolean); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_bool, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":120
 *     "bool": (OFTInteger, OFSTBoolean),
 * 
 *     "float32": (OFTReal, OFSTFloat32),             # <<<<<<<<<<<<<<
 *     "float": (OFTReal, OFSTNone),
 *     "float64": (OFTReal, OFSTNone),
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTReal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTFloat32); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_float32, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":121
 * 
 *     "float32": (OFTReal, OFSTFloat32),
 *     "float": (OFTReal, OFSTNone),             # <<<<<<<<<<<<<<
 *     "float64": (OFTReal, OFSTNone),
 * 
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTReal); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_float, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":122
 *     "float32": (OFTReal, OFSTFloat32),
 *     "float": (OFTReal, OFSTNone),
 *     "float64": (OFTReal, OFSTNone),             # <<<<<<<<<<<<<<
 * 
 *     "datetime64[D]": (OFTDate, OFSTNone),
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTReal); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_float64, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

  /* "pyogrio/_io.pyx":124
 *     "float64": (OFTReal, OFSTNone),
 * 
 *     "datetime64[D]": (OFTDate, OFSTNone),             # <<<<<<<<<<<<<<
 *     "datetime64": (OFTDateTime, OFSTNone),
 * }
*/
  __pyx_t_11 = __Pyx_PyLong_From_OGRFieldType(OFTDate); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_Pack(2, __pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_datetime64_D, __pyx_t_5) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pyogrio/_io.pyx":125
 * 
 *     "datetime64[D]": (OFTDate, OFSTNone),
 *     "datetime64": (OFTDateTime, OFSTNone),             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __pyx_t_5 = __Pyx_PyLong_From_OGRFieldType(OFTDateTime); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyLong_From_OGRFieldSubType(OFSTNone); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_Pack(2, __pyx_t_5, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_datetime64, __pyx_t_11) < (0)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DTYPE_OGR_FIELD_TYPES, __pyx_t_4) < (0)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "pyogrio/_io.pyx":1626
 *     object layer=None,
 *     object encoding=None,
 *     int read_geometry=True,             # <<<<<<<<<<<<<<
 *     int force_2d=False,
 *     object columns=None,
*/
  __pyx_t_4 = __Pyx_PyBool_FromLong(((int)1)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "pyogrio/_io.pyx":1627
 *     object encoding=None,
 *     int read_geometry=True,
 *     int force_2d=False,             # <<<<<<<<<<<<<<
 *     object columns=None,
 *     int skip_features=0,
*/
  __pyx_t_11 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);

  /* "pyogrio/_io.pyx":1629
 *     int force_2d=False,
 *     object columns=None,
 *     int skip_features=0,             # <<<<<<<<<<<<<<
 *     int max_features=0,
 *     object where=None,
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "pyogrio/_io.pyx":1630
 *     object columns=None,
 *     int skip_features=0,
 *     int max_features=0,             # <<<<<<<<<<<<<<
 *     object where=None,
 *     tuple bbox=None,
*/
  __pyx_t_5 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1630, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "pyogrio/_io.pyx":1637
 *     str sql=None,
 *     str sql_dialect=None,
 *     int return_fids=False,             # <<<<<<<<<<<<<<
 *     bint datetime_as_string=False,
 * ):
*/
  __pyx_t_12 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "pyogrio/_io.pyx":1638
 *     str sql_dialect=None,
 *     int return_fids=False,
 *     bint datetime_as_string=False,             # <<<<<<<<<<<<<<
 * ):
 *     """Read features from a dataset.
*/
  __pyx_t_13 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "pyogrio/_io.pyx":1621
 * 
 * 
 * def ogr_read(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object dataset_kwargs,
*/
  __pyx_t_14 = PyTuple_Pack(15, Py_None, Py_None, __pyx_t_4, __pyx_t_11, Py_None, __pyx_t_10, __pyx_t_5, Py_None, Py_None, Py_None, Py_None, Py_None, Py_None, __pyx_t_12, __pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_1ogr_read, 0, __pyx_mstate_global->__pyx_n_u_ogr_read, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_13, __pyx_t_14);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ogr_read, __pyx_t_13) < (0)) __PYX_ERR(0, 1621, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "pyogrio/_io.pyx":1898
 * 
 * 
 * class _ArrowStream:             # <<<<<<<<<<<<<<
 *     def __init__(self, capsule):
 *         self._capsule = capsule
*/
  __pyx_t_13 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_mstate_global->__pyx_empty_tuple, __pyx_mstate_global->__pyx_n_u_ArrowStream, __pyx_mstate_global->__pyx_n_u_ArrowStream, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, (PyObject *) NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "pyogrio/_io.pyx":1899
 * 
 * class _ArrowStream:
 *     def __init__(self, capsule):             # <<<<<<<<<<<<<<
 *         self._capsule = capsule
 * 
*/
  __pyx_t_14 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_12_ArrowStream_1__init__, 0, __pyx_mstate_global->__pyx_n_u_ArrowStream___init, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_14);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_init, __pyx_t_14) < (0)) __PYX_ERR(0, 1899, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "pyogrio/_io.pyx":1902
 *         self._capsule = capsule
 * 
 *     def __arrow_c_stream__(self, requested_schema=None):             # <<<<<<<<<<<<<<
 *         if requested_schema is not None:
 *             raise NotImplementedError("requested_schema is not supported")
*/
  __pyx_t_14 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_12_ArrowStream_3__arrow_c_stream__, 0, __pyx_mstate_global->__pyx_n_u_ArrowStream___arrow_c_stream, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_14);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_14, __pyx_mstate_global->__pyx_tuple[9]);
  if (__Pyx_SetNameInClass(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_arrow_c_stream, __pyx_t_14) < (0)) __PYX_ERR(0, 1902, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "pyogrio/_io.pyx":1898
 * 
 * 
 * class _ArrowStream:             # <<<<<<<<<<<<<<
 *     def __init__(self, capsule):
 *         self._capsule = capsule
*/
  __pyx_t_14 = __Pyx_Py3ClassCreate(((PyObject*)&PyType_Type), __pyx_mstate_global->__pyx_n_u_ArrowStream, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_13, NULL, 0, 0); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_14);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ArrowStream, __pyx_t_14) < (0)) __PYX_ERR(0, 1898, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "pyogrio/_io.pyx":1908
 * 
 * 
 * @contextlib.contextmanager             # <<<<<<<<<<<<<<
 * def ogr_open_arrow(
 *     object path_or_buffer,
*/
  __pyx_t_14 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_contextlib); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_contextmanager); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "pyogrio/_io.pyx":1914
 *     object layer=None,
 *     object encoding=None,
 *     int read_geometry=True,             # <<<<<<<<<<<<<<
 *     int force_2d=False,
 *     object columns=None,
*/
  __pyx_t_12 = __Pyx_PyBool_FromLong(((int)1)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1914, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "pyogrio/_io.pyx":1915
 *     object encoding=None,
 *     int read_geometry=True,
 *     int force_2d=False,             # <<<<<<<<<<<<<<
 *     object columns=None,
 *     int skip_features=0,
*/
  __pyx_t_10 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "pyogrio/_io.pyx":1917
 *     int force_2d=False,
 *     object columns=None,
 *     int skip_features=0,             # <<<<<<<<<<<<<<
 *     int max_features=0,
 *     object where=None,
*/
  __pyx_t_11 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);

  /* "pyogrio/_io.pyx":1918
 *     object columns=None,
 *     int skip_features=0,
 *     int max_features=0,             # <<<<<<<<<<<<<<
 *     object where=None,
 *     tuple bbox=None,
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1918, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "pyogrio/_io.pyx":1925
 *     str sql=None,
 *     str sql_dialect=None,
 *     int return_fids=False,             # <<<<<<<<<<<<<<
 *     int batch_size=0,
 *     use_pyarrow=False,
*/
  __pyx_t_15 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1925, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);

  /* "pyogrio/_io.pyx":1926
 *     str sql_dialect=None,
 *     int return_fids=False,
 *     int batch_size=0,             # <<<<<<<<<<<<<<
 *     use_pyarrow=False,
 *     datetime_as_string=False,
*/
  __pyx_t_16 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1926, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);

  /* "pyogrio/_io.pyx":1908
 * 
 * 
 * @contextlib.contextmanager             # <<<<<<<<<<<<<<
 * def ogr_open_arrow(
 *     object path_or_buffer,
*/
  __pyx_t_17 = PyTuple_Pack(17, Py_None, Py_None, __pyx_t_12, __pyx_t_10, Py_None, __pyx_t_11, __pyx_t_4, Py_None, Py_None, Py_None, Py_None, Py_None, Py_None, __pyx_t_15, __pyx_t_16, ((PyObject*)Py_False), ((PyObject*)Py_False)); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_16 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_3ogr_open_arrow, 0, __pyx_mstate_global->__pyx_n_u_ogr_open_arrow, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_16);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_16, __pyx_t_17);
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_16};
    __pyx_t_13 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ogr_open_arrow, __pyx_t_13) < (0)) __PYX_ERR(0, 1908, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "pyogrio/_io.pyx":2245
 *     object layer=None,
 *     object encoding=None,
 *     int read_geometry=True,             # <<<<<<<<<<<<<<
 *     int force_2d=False,
 *     object columns=None,
*/
  __pyx_t_13 = __Pyx_PyBool_FromLong(((int)1)); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 2245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);

  /* "pyogrio/_io.pyx":2246
 *     object encoding=None,
 *     int read_geometry=True,
 *     int force_2d=False,             # <<<<<<<<<<<<<<
 *     object columns=None,
 *     int skip_features=0,
*/
  __pyx_t_5 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "pyogrio/_io.pyx":2248
 *     int force_2d=False,
 *     object columns=None,
 *     int skip_features=0,             # <<<<<<<<<<<<<<
 *     int max_features=0,
 *     object where=None,
*/
  __pyx_t_16 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);

  /* "pyogrio/_io.pyx":2249
 *     object columns=None,
 *     int skip_features=0,
 *     int max_features=0,             # <<<<<<<<<<<<<<
 *     object where=None,
 *     tuple bbox=None,
*/
  __pyx_t_14 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);

  /* "pyogrio/_io.pyx":2241
 * 
 * 
 * def ogr_read_bounds(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     object layer=None,
*/
  __pyx_t_17 = PyTuple_Pack(10, Py_None, Py_None, __pyx_t_13, __pyx_t_5, Py_None, __pyx_t_16, __pyx_t_14, Py_None, Py_None, Py_None); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_6ogr_read_bounds, 0, __pyx_mstate_global->__pyx_n_u_ogr_read_bounds, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_14);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_14, __pyx_t_17);
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ogr_read_bounds, __pyx_t_14) < (0)) __PYX_ERR(0, 2241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "pyogrio/_io.pyx":2313
 *     object layer=None,
 *     object encoding=None,
 *     int force_feature_count=False,             # <<<<<<<<<<<<<<
 *     int force_total_bounds=False
 * ):
*/
  __pyx_t_14 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);

  /* "pyogrio/_io.pyx":2314
 *     object encoding=None,
 *     int force_feature_count=False,
 *     int force_total_bounds=False             # <<<<<<<<<<<<<<
 * ):
 *     """Read metadata information about the dataset."""
*/
  __pyx_t_17 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);

  /* "pyogrio/_io.pyx":2308
 * 
 * 
 * def ogr_read_info(             # <<<<<<<<<<<<<<
 *     object path_or_buffer,
 *     dataset_kwargs,
*/
  __pyx_t_16 = PyTuple_Pack(4, Py_None, Py_None, __pyx_t_14, __pyx_t_17); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_8ogr_read_info, 0, __pyx_mstate_global->__pyx_n_u_ogr_read_info, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_17, __pyx_t_16);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ogr_read_info, __pyx_t_17) < (0)) __PYX_ERR(0, 2308, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "pyogrio/_io.pyx":2408
 * 
 * 
 * def ogr_list_layers(object path_or_buffer):             # <<<<<<<<<<<<<<
 *     """List the layers in the dataset.
 * 
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_10ogr_list_layers, 0, __pyx_mstate_global->__pyx_n_u_ogr_list_layers, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ogr_list_layers, __pyx_t_17) < (0)) __PYX_ERR(0, 2408, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "pyogrio/_io.pyx":2914
 *     object dataset_kwargs,
 *     object layer_kwargs,
 *     bint promote_to_multi=False,             # <<<<<<<<<<<<<<
 *     bint nan_as_null=True,
 *     bint append=False,
*/
  __pyx_t_17 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 2914, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);

  /* "pyogrio/_io.pyx":2915
 *     object layer_kwargs,
 *     bint promote_to_multi=False,
 *     bint nan_as_null=True,             # <<<<<<<<<<<<<<
 *     bint append=False,
 *     dataset_metadata=None,
*/
  __pyx_t_16 = __Pyx_PyBool_FromLong(((int)1)); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 2915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);

  /* "pyogrio/_io.pyx":2916
 *     bint promote_to_multi=False,
 *     bint nan_as_null=True,
 *     bint append=False,             # <<<<<<<<<<<<<<
 *     dataset_metadata=None,
 *     layer_metadata=None,
*/
  __pyx_t_14 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2916, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);

  /* "pyogrio/_io.pyx":2901
 * 
 * # TODO: set geometry and field data as memory views?
 * def ogr_write(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/
  __pyx_t_5 = PyTuple_Pack(6, __pyx_t_17, __pyx_t_16, __pyx_t_14, Py_None, Py_None, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 2901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_12ogr_write, 0, __pyx_mstate_global->__pyx_n_u_ogr_write, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 2901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_14);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_14, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ogr_write, __pyx_t_14) < (0)) __PYX_ERR(0, 2901, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "pyogrio/_io.pyx":3275
 *     object dataset_kwargs,
 *     object layer_kwargs,
 *     bint append=False,             # <<<<<<<<<<<<<<
 *     dataset_metadata=None,
 *     layer_metadata=None,
*/
  __pyx_t_14 = __Pyx_PyBool_FromLong(((int)0)); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);

  /* "pyogrio/_io.pyx":3264
 * 
 * 
 * def ogr_write_arrow(             # <<<<<<<<<<<<<<
 *     object path_or_fp,
 *     str layer,
*/
  __pyx_t_5 = PyTuple_Pack(3, __pyx_t_14, Py_None, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 3264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_CyFunction_New(&__pyx_mdef_7pyogrio_3_io_14ogr_write_arrow, 0, __pyx_mstate_global->__pyx_n_u_ogr_write_arrow, NULL, __pyx_mstate_global->__pyx_n_u_pyogrio__io, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 3264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_14);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_14, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ogr_write_arrow, __pyx_t_14) < (0)) __PYX_ERR(0, 3264, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "pyogrio/_io.pyx":1
 * #define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION             # <<<<<<<<<<<<<<
 * 
 * """IO support for OGR vector data sources."""
*/
  __pyx_t_14 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_14) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init pyogrio._io", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyogrio._io");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_object); if (!__pyx_builtin_object) __PYX_ERR(0, 776, __pyx_L1_error)
  __pyx_builtin_round = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_round); if (!__pyx_builtin_round) __PYX_ERR(0, 1159, __pyx_L1_error)
  __pyx_builtin___import__ = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_import); if (!__pyx_builtin___import__) __PYX_ERR(1, 109, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(1, 165, __pyx_L1_error)
  __pyx_builtin_Ellipsis = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_Ellipsis); if (!__pyx_builtin_Ellipsis) __PYX_ERR(1, 418, __pyx_L1_error)
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_id); if (!__pyx_builtin_id) __PYX_ERR(1, 628, __pyx_L1_error)

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  __pyx_mstate->__pyx_umethod_PyUnicode_Type__upper.type = (PyObject*)(&PyUnicode_Type);
  __pyx_mstate->__pyx_umethod_PyUnicode_Type__upper.method_name = &__pyx_mstate->__pyx_n_u_upper;
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "View.MemoryView":592
 *     def suboffsets(self):
 *         if self.view.suboffsets == NULL:
 *             return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *         return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_New(1); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(1, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_neg_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_mstate_global->__pyx_tuple[0], 0, __pyx_mstate_global->__pyx_int_neg_1) != (0)) __PYX_ERR(1, 592, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "View.MemoryView":689
 *     tup = <tuple>index if isinstance(index, tuple) else (index,)
 * 
 *     result = [slice(None)] * ndim             # <<<<<<<<<<<<<<
 *     have_slices = False
 *     seen_ellipsis = False
*/
  __pyx_mstate_global->__pyx_slice[0] = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_slice[0])) __PYX_ERR(1, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_slice[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);

  /* "pyogrio/_io.pyx":269
 *         # WARNING: GDAL logs warnings about invalid open options to stderr
 *         # instead of raising an error
 *         with capture_errors() as errors:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 ogr_dataset = GDALOpenEx(
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);

  /* "pyogrio/_io.pyx":777
 * 
 *     fields = np.empty(shape=(field_count, 5), dtype=object)
 *     fields_view = fields[:, :]             # <<<<<<<<<<<<<<
 * 
 *     skipped_fields = False
*/
  __pyx_mstate_global->__pyx_tuple[2] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_slice[0], __pyx_mstate_global->__pyx_slice[0]); if (unlikely(!__pyx_mstate_global->__pyx_tuple[2])) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[2]);

  /* "pyogrio/_io.pyx":817
 *     if skipped_fields:
 *         # filter out skipped fields
 *         mask = np.array([idx is not None for idx in fields[:, 0]])             # <<<<<<<<<<<<<<
 *         fields = fields[mask]
 * 
*/
  __pyx_mstate_global->__pyx_tuple[3] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_slice[0], __pyx_mstate_global->__pyx_int_0); if (unlikely(!__pyx_mstate_global->__pyx_tuple[3])) __PYX_ERR(0, 817, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[3]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[3]);

  /* "pyogrio/_io.pyx":961
 * 
 *     if feature_count == 0:
 *         return 0, 0             # <<<<<<<<<<<<<<
 * 
 *     if skip_features >= feature_count:
*/
  __pyx_mstate_global->__pyx_tuple[4] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_int_0, __pyx_mstate_global->__pyx_int_0); if (unlikely(!__pyx_mstate_global->__pyx_tuple[4])) __PYX_ERR(0, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[4]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[4]);

  /* "pyogrio/_io.pyx":1744
 *         if columns is not None:
 *             # identify ignored fields first
 *             ignored_fields = list(set(fields[:, 2]) - set(columns))             # <<<<<<<<<<<<<<
 * 
 *             # Fields are matched exactly by name, duplicates are dropped.
*/
  __pyx_mstate_global->__pyx_tuple[5] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_slice[0], __pyx_mstate_global->__pyx_int_2); if (unlikely(!__pyx_mstate_global->__pyx_tuple[5])) __PYX_ERR(0, 1744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[5]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[5]);

  /* "pyogrio/_io.pyx":1841
 *             "encoding": encoding,
 *             "fields": fields[:, 2],
 *             "dtypes": fields[:, 3],             # <<<<<<<<<<<<<<
 *             "ogr_types": ogr_types,
 *             "ogr_subtypes": ogr_subtypes,
*/
  __pyx_mstate_global->__pyx_tuple[6] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_slice[0], __pyx_mstate_global->__pyx_int_3); if (unlikely(!__pyx_mstate_global->__pyx_tuple[6])) __PYX_ERR(0, 1841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[6]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[6]);

  /* "pyogrio/_io.pyx":2029
 *                 encoding = "UTF-8"
 * 
 *             elif encoding.replace("-", "").upper() != "UTF8":             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "non-UTF-8 encoding is not supported for Arrow; use the non-Arrow "
*/
  __pyx_mstate_global->__pyx_tuple[7] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_kp_u__10, __pyx_mstate_global->__pyx_kp_u__9); if (unlikely(!__pyx_mstate_global->__pyx_tuple[7])) __PYX_ERR(0, 2029, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[7]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[7]);

  /* "pyogrio/_io.pyx":2465
 *         dataset_name = os.path.basename(get_string(OGR_DS_GetName(ogr_dataset)))
 * 
 *         other_layer_names = ", ".join([f"'{lyr}'" for lyr in layers[1:, 0]])             # <<<<<<<<<<<<<<
 *         warnings.warn(
 *             f"More than one layer found in '{dataset_name}': '{first_layer_name}' "
*/
  __pyx_mstate_global->__pyx_slice[1] = PySlice_New(__pyx_mstate_global->__pyx_int_1, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_slice[1])) __PYX_ERR(0, 2465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_slice[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[1]);
  __pyx_mstate_global->__pyx_tuple[8] = PyTuple_Pack(2, __pyx_mstate_global->__pyx_slice[1], __pyx_mstate_global->__pyx_int_0); if (unlikely(!__pyx_mstate_global->__pyx_tuple[8])) __PYX_ERR(0, 2465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[8]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[8]);

  /* "pyogrio/_io.pyx":1902
 *         self._capsule = capsule
 * 
 *     def __arrow_c_stream__(self, requested_schema=None):             # <<<<<<<<<<<<<<
 *         if requested_schema is not None:
 *             raise NotImplementedError("requested_schema is not supported")
*/
  __pyx_mstate_global->__pyx_tuple[9] = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[9])) __PYX_ERR(0, 1902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[9]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[9]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<10; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_slice;
    for (Py_ssize_t i=0; i<2; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 12; } index[] = {{2},{23},{40},{35},{33},{54},{37},{60},{24},{52},{40},{34},{35},{44},{30},{34},{24},{43},{38},{39},{50},{30},{33},{25},{32},{21},{44},{22},{8},{26},{7},{5},{14},{34},{20},{21},{40},{48},{38},{28},{38},{42},{29},{27},{36},{247},{5},{10},{33},{19},{17},{29},{14},{45},{22},{118},{7},{15},{30},{4},{179},{45},{37},{78},{42},{22},{15},{48},{5},{32},{34},{54},{1},{1},{2},{1},{2},{2},{1},{1},{1},{1},{1},{3},{1},{4},{0},{25},{8},{5},{11},{6},{94},{128},{103},{33},{99},{100},{50},{15},{23},{25},{21},{13},{14},{13},{7},{37},{6},{26},{21},{72},{45},{56},{37},{55},{15},{55},{28},{14},{37},{2},{4},{6},{35},{12},{91},{9},{30},{44},{13},{11},{11},{9},{27},{50},{82},{8},{15},{16},{8},{33},{28},{52},{47},{20},{32},{22},{10},{11},{30},{37},{24},{5},{12},{5},{12},{31},{21},{14},{22},{8},{21},{14},{15},{8},{4},{8},{3},{14},{19},{11},{16},{12},{10},{10},{13},{7},{10},{13},{16},{3},{16},{1},{3},{11},{11},{9},{8},{14},{8},{8},{9},{7},{11},{10},{12},{16},{14},{7},{11},{9},{13},{7},{13},{17},{7},{12},{3},{11},{20},{23},{14},{6},{8},{4},{7},{15},{4},{3},{14},{15},{12},{6},{5},{18},{9},{7},{6},{18},{1},{4},{8},{10},{4},{4},{6},{9},{6},{1},{12},{7},{8},{14},{9},{17},{18},{5},{7},{11},{10},{14},{5},{3},{14},{16},{15},{4},{8},{10},{18},{14},{3},{6},{8},{7},{6},{5},{15},{6},{5},{6},{8},{9},{9},{3},{5},{2},{3},{6},{8},{18},{22},{19},{17},{13},{8},{3},{10},{16},{8},{4},{8},{5},{7},{7},{10},{16},{9},{11},{13},{10},{14},{15},{13},{10},{11},{11},{6},{8},{5},{5},{7},{7},{8},{19},{18},{6},{7},{10},{8},{8},{7},{15},{7},{10},{8},{13},{13},{3},{9},{36},{29},{20},{12},{4},{1},{2},{3},{14},{10},{14},{5},{4},{8},{3},{5},{5},{5},{4},{4},{11},{13},{5},{5},{5},{9},{4},{5},{8},{4},{5},{13},{12},{14},{10},{6},{4},{6},{3},{7},{8},{4},{12},{7},{4},{13},{11},{6},{4},{10},{5},{4},{8},{6},{3},{11},{7},{4},{7},{4},{2},{12},{14},{10},{11},{5},{3},{6},{11},{11},{14},{12},{12},{9},{15},{14},{8},{15},{13},{12},{9},{9},{15},{7},{4},{2},{23},{2},{4},{4},{14},{10},{3},{10},{11},{19},{16},{7},{12},{11},{12},{14},{14},{11},{10},{19},{14},{12},{11},{13},{6},{10},{17},{13},{8},{7},{16},{11},{14},{5},{6},{5},{6},{6},{4},{4},{12},{10},{12},{19},{5},{6},{6},{4},{13},{4},{5},{8},{3},{11},{10},{5},{10},{4},{4},{4},{6},{14},{6},{21},{3},{8},{5},{4},{12},{4},{8},{1},{4},{6},{6},{6},{5},{6},{6},{6},{5},{11},{14},{5},{7},{6},{4},{7},{8},{5},{5},{3},{10},{7},{1},{4},{5},{24},{20},{91},{199},{105},{82},{9},{12},{160},{9},{32},{2},{55},{2058},{619},{108},{596},{316},{1227},{7},{8},{1}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (6433 bytes) */
const char* const cstring = "(\265/\375`\0345\275\310\000Z\310<*H\360\354\272\001\314\300\010)$\025\2601b\253\200\025RH*`\23107\007\227\r|C\324\235\221^\331\r!n\226\223\247I\007\370\320u\017\t\360\216\275\211A57\001,:PiN6\310\345\362\220\016tCh\010\247\375,c\002h\002\263\002;\223\311\264\277\216w?\361\204\222\304$/\034X+F\203\343\"\311y\322\366m\225\221\034\367i\326z\316@\016\306\235S^\213\\c\023^\231b\360\n\2052\311\244(\2340NIQ\361 \367!}?E\r\342\031\222\237\346\261\211\343\250\343\242RS\323I\215\246\376\0019\351\243\267\250M\032N\2254\034\203{\366}s\\s\321P\340\207\303\360\034O\373\3333\232\032\277_\331&\177z\317Nx\364g\014\257xj\367\320\224Mc\327\345\036\361\255\207\374\264\365\335\273\016?[\377\200[\\ZTr\\9\246\215\207\033{\022\007\307\004\336\366H\336\354\334\244L\253\035Z\n\227\332+\231\231k\0076k\301\230r^\227@f\211w6\317?\036\232\346\356\306\313p\347\260\317\3418\014O\000\342\236\304\341\024\023\211M\237)\005\210\264\331Lx\2653\367\251\30362\332\010\003wy\302T*/\020\\\\Tb4\330\t\343\300pj\233d\211\352+$7q\344\256\\\351\032\034u\271\222\302\255\254\254z\351}\356\204m\347k\237\275\363\034\214S\235\373\216\323#\267\231'\271\323~\334\266=v\320$\277=n\023\333\260Z\257nj\337c6H\212'\033\032k\334%\371i.\212\344\007\352\232\331+\226\373\274\266\365\276\277O\363\334O\250\353\376\374\266\351J\215d\316\357\276\035\226\247\307\276\024];,u\216\206a3\232k\365\242\345\270Z\340\tn\321\350\021!\360\252\203#\327\336F\256Hs{\356\371\006\265\270s_\367&y\333\267\024\006\371\031j\037\274A\362\033\267\337]{\r\342\033\351\303M\277\301\236\256\305\231\235\351\267Hn\234\336\351\020\203\240\246\351#\311\373\270E\016b\377m\223\367\321\204w\271\3638\347\371\017\006\332\240H\232\267\035\332\264\347\346\217\n&=\036q\274w\235\332\231\244\270E<r\334O\362\304q?v\216\275\316y\033\326x\373\017\357\302\221\304\243\246\251a\340\301\340\366\2376\375\266\201\341I~\2306\243\351\033\344\273\223%\320\306\236\034q\337$\210\1776\367\301\346\276\335\316\231\024\270\273\336\3416\361\016\0145\356\372\007\213z\233\276\343\2356M\215""\177H\323d\305\364m\236^\357[.yO\307\270\221\357\302\262d\341U\341x\272\234\347\351G~\2369\277\353=w\377\235&\351{\031n\237\314\303\262wp\207\271\236\3011\334\335\334\333\364\242\271\367\247\303\022gn\221\004C}\352|FW\317\307\313\361\024\317^\222<\265+\271\017O1{\322'g^\346\362f\222\271\324e\316\235\354|W\226c\376\331\243-\347\307\360<G.\202\243\371q1\213[\314#\327\"\016G\206\273\347\236Nv\356G=vnr\260sP\325?\027\356\311\333\310\277,\362\234\311\021\307A\316s\035\364\270\224L\017aR8\322\306\347\235Yn\\\277\335\310\014\302\343\272\010\342r\316\031|\220\367\255\343t\330mg\317}\003\201\374\263\345\234=\351s\250\271\327e\336\331\362\230o@\373\003\332@[<w\316=w\360#3\010\202\273\334u\336\270\236\334\315O\217Z\364\343\356d\370\365|\331o\\\316=\347\035\236\275o\356y\310m\272\324\337\227\273In\277\263\331\262\315\3267\371\215\273\256\273\356\032,w\316\271g<s_\006u\006\232M\206\324\370C\216=\373\033i,\0134s\317\314n{\206\223\375\307[@\273\234\234gZ\257\035\362\232\330\r-\226i^<\264\320wn\323\022\301\334\270-\362\332\336\231z\3443\265\232X\013\246\243S\263\2354 }\023A\345GX\315\250wH\22235ZM\214\326\212\305\310\034\234\232\306\201\321p\\\265\014\256\014\253\331\350\334\274t\233\206\023\243]\3200\253\271nb\256\027\271_^\20757\254\031\035Z\013V\323y/y\030\272h-Zn\331\270`5\331\225\243\241\326\242\315h\273\353\231\354\262\311\241\311=\177Y\347\234\3120\030\r\215e\032o~<\361*\325S\344\t\302\344\276O\3356w\347\332\304@\2732E\362,\263?}\313\371\363'\210s\336\337\tn3gr\224A=`A\265D\360\232\014o'\337y\250M&\0078\006\006o\020\274\203\235\247P\244\255\347'\245M\247\361\214\341\324\336\231\260\0377\007A\\\352\303\265\036|\253S\3579\217\033{\226\360\362\322b2a\335m\3429\342\024\236\260\367\336\371\361\354)(\037\236\332C\027=\2648\264D\330\"\367\033\334&\031\230\317\246S\235\374P\344.\014\265\357\371N8\005(\305\264\315\235\3731\005\352\311\303\345\010\261\264\265d\025\255 R\223\207\273\243\323\352\310\332\257\307\322+\365\014\360hl\t\333W\232U\265~\250_)%!\271\033\243\227\225j\244\337""\227`\261~HYR\324?\241\032\264Yk\340U\322\226\322P\247\374\217W\245\212\"!\036\207\225\265\266\340\266TZAuBZJA\030\221\214\227\203\027\342UP\360#\240\032\031\256\356\311z8\336\014/F\245e\251)\244\226\364\3526Z\2525\300\352\000\267\257G\373Q d\"\215\010\001q\365*[}T\255\246H\356\307\310%u\274\242\236\014{`\325\251+#\255\354\217l\350\326\0215\244\236H\356\350\025\2612*\311\r\251\264\224\004J\332\350Y\360t\354HLZ2rE\031\"\000\251\356s|\217\214W\327\323\275\t<q\375Q\023\244X\007\025\200fh\236\006v\304\232V\235\217\366\303\325hG\210\027C!\r\220\356.N\350P\022\006\205\201\023\022\202.~\351{\010\302\030Qq\\\031\217\\7V\222jJIB\253\003R+\343\205\260!\326eA\273\000A\213\300\342\025P\361\226\376\207\205[\352 \320\316\374\033*C9T\274\000\274\026\253b?\326\320JSG\252+\345\021}\020\237\365Q\320\022\313\027\021\244\203XD\337\004\202\2611ri\241oC'\204\276G\001\025\032\000\335}@92JRp\205\226\320\036\244;\252\321\r\324\225\262R\224/\372\037\312\201\222\350R\232\364\345{t&\372\241\377!\010\001Z\202\2060A\240\342\325}\254E<\020+\201\265C\355)\371\037\275\025\357\305\202\254+\251(\375\376H\205\313\372!\025\256\010\025\241\032\241\024\253\r\030T\270%\025\245\302\255(\251pu\177A\205I\000\305\200\023\037\002*\334\001\002(,N\204L\310T\025\352\203\022X\270:\225\245'\nZ\005\007ZQZ\252hC\337\206e\242\343{X\274*\336\016\013[yfRU\372\372!\027/K\0351\340U\255\2455D:K1\276\016\355\020\271\241O\312pC=\035\017\003K\302zPY\352-5II\022\264\t.\\Q\032J!\374X\320\205\205+SU,^\315\016`w*\001U\227r\244\346\327\377\007\355pu\354\214\365Y\271\372\223\352t@j\373\262\217C9.\\U\320\001\213\227e\242\364\235\002u\242@\241\340\226,!\347\017\233:\367\225\024\024\250\262s\020\343\224|\242@\241\340\330.\347\204W+\236\330l\306\352 L\275w@9\017\362\237\336m\231w\237\\\212>\357~\232\273<\307\263g\344\0011\236\330l\306\212\271^\257hcD\304]0\242bE\200(\270\301\031\031\030\245\332\022\272?\033\301\372\"\242\265G\364fh\255\262\250;\351(u}\036\263EgR\342~x H\031a]N\332\324\032B\201\350\325\2450\245\237\177\362?2\n\230N66H""\354\324\332\263\227\200\267\363*\207\315'qi|\353LK\224\326^\227\037\"\242@p\302c\202\n\356\3257\007^\325*b\224>\255\305d\321\233)\361:K\303\236\026T\r\265\003\216\332\221F8\361\266t@\312\300\014\312\320\275\022\250\320\227^K(\320i\t\006m\t\224$S\033(\300xE:\010j\210\337k_\300\357x5:\203\216\230\241$Bh+\350\324\261\244%/\n\344@{\221\342\0137|E\354\313I{\302\240e\240b'\240\033\nF.J\213\222\341\316Jp\010i\327\006\"-\205S\253K7\204$\234\2647\017i\217j\342\362\177AG\357\005\303ZQKO6Z\226\350\2151\360[\350\213\252r\203\216X\250\306\312\007\341\265HZ\014w@\305\273\000]IH}\016B\353\262+\2246D\243,\250\213\350\223P\221\306/p\202\245\267o\362u~\006tC\306\262X\230\321\250\273\220B\370 \031\034-\017\227\001D\243\205\241gPF\tH\003\352S\300\242\000\317\202\325\255\030D\353\227\001\304\003\215\220\3022iMD\255h\205@\322z\240\243v\364\022\320,\013\033\200e\260v\365\204\224\222~\315\000\211\205\202\004\025@\033\004\035`\245&\034$\033%\026\310\017\360cB*\203t&ke9mG\217\244\025\245+:\304\203\022\350R\014_\365?\320\020\nA\367\377\305K\352\360#W\243\325\270\350QE.\002\014\225\217\026\244Fk}O\205\207\263\222\254\2477\347\363\027\210\204\200\220\334R/de\254\317\252W\223\022\222\253\323\273\201iU%\016\327\n\242\242B\246\216\247\362\236iK*B\363D\330\033\336\010\224H\210[\3025\330U\261%v\205]\226\212\222\222\204\270x\301\253\352\211\254\310\032\322\261\366\244\014B\\\017\322\253U\031J\210\253[U@\374\226\037\342\301\306\2742\036\213w\262-\354\353\302\017\331\300\033/k\035\221\216*\324Vz~\002\037\002\010\264#C\224e\326\207?\242\3631P\021\006\213!\212\245&,\032\330K\373\342\241Y\031S+\3435X\245\376\003\215x\232WC\303\273`S(\216\266C\204#\323\312\330\030\225\230P\317BK\253ZY2\336\022+bA\2503\265D\355\036.\317N\2733k{\217\246\247\352}\360>\326\244\305@\206\360\250\3037\213\242&\2352#\"#\222$I:C0@\000A\251T0\240\224\253\273\001\023t\324\271$\2454BDF\204\002\t$\220$RP\222\024$\215\001\r\246\013\3126\205\314\304\240\034\324\223\354\307g\317\356+\217\242\364\374T4\223i\331\251\301M\325\352P\001`\353""\334\222\021\356V\tc\032F\027]w\374h[`\343\003\331\310C\366\177?\346\376;\254T\300\317\314\022\276\306b`'u\224\240\315d\005\304\343\3035\307\254\200$D\0039b$\323\007v\235\302o\357\021\312\267\357\357\030\217v\013\277)\024\250\201\315*\331\266\245;a\317\027\375x>T\037\202m\030}$\245CRO2\201\204\344\013h\242w\370,(@o\215\323\311W\215\377\224\302\330'v\003-\013)6\216a\001\361\323\314(lAM\035k\306\300LT\245\004Ys\316\373\224\376\365/\261g\304RcY\020\332F\377\255\023,h\023z\203\372\345c@+\326\330\210\336\277\336\243\364\273\340\005\333U\037i\022mN\337\026i\026\024L\335\307J\010\300\377\034\325\003~\337\314L\322\334*_\356\334$$\274\270%Y)\351\014\270<;\275\216\255\334#\315\245\336\r\321\222@\0069\343\201\222\256b\242Dl\3368\273\347\027\232\203BP\303\244\245`\367D\370d~)D\n'\255d}\323\324\322\257\265\243-e\n\351\265\350\310\311\353\234<7\343T\207%\220\275\014\203\234t2\252\315\226\377\2645g\203 \3056\335\354bx\250m\231\235e\223\333QLQP\313\224\022\304\3023\207 \206\026C\212g\036\253\025\021#\271\342\030\377\265z&\271t\356\336\026\254`VF\331\234\301\307\216\220\225\006R\345\300\004\352c3\376^\243\200xg\254\227\246\276y\305v\337\2420\377\236i@M3\202\233\351\267\201Q\224H\267\305\314\3601\323\371\305\3463\177\240\337\245\3144\364\025\035\3651\242\322\243\320M\235\203\022\231\031\271\0211\026\207\344\343\301=\204\005\347\221\207\334\030#'\313\216\362w\033W\260*d\233\317\367\303\002\004\302\265m\221e\323\323\324C\313 \347\214&*\210\2001\205C2\266[\031z\210\333\016\366\265H.\235=\0130z\024F!\340>\311\035\263o\223\347+S\273\352\364\234\3303E\210Q\030\333zq\327\013\204-4=;\364D.\335{\303\265\253[\372,)A\n\217\220\243p\177\314J\177\302 \332\247\016+o3\316E\304(\302\022Z\256\364\330\363\325\317J\273\271\252\262-)K\273O\037\244>*3TN\366\362\r\242\320\274pnrB\312\002\240:\024[_X\np\366z\206&6\014%\212H\232j.\236\250\266q \222~W\377\326\006)\022\025N[\034W\330 \336\240\237D\362\225\237\311\224x\010r(\341\343}\217\352\345Z~e\0171\352\352\312L\331\016\035\330\233\323\252\262\007\342\331N""\362\347\320\207\2265$\203+\032Z\200\326\td\034\260\0172-\333\246\203\261\214%\264\353\010\354\264\301(i\370\351\006\331\177U\303\260\232%\347\t\240n\217\013\263\001\321\372\022\246\243\273)\204\034SA\323Za`^\\G\004\221<\240+\3176$\267\255\342\222\236]\271\351`u\035r\206\204\223\261#4\220\235\"8\3706b6\377\264\315\313\257\341\316;\242\335\236$\255]\332\373\027K\241\274\206\r\014\362&\314M\301=\334z\317M-+F\234C\013h\252|\201\211\301\265\207\222&\016\006\376\344\340\177\356\3100\362\t\207\356\330b\311\013\034\264\337s\207\0329I\224%R\250\370Q\275\254\302\016\212\310d~d\354\346\340F\341\035\355z\261\233\325\366\205b\375\214\004\355Tj\272\207\007\371%g\233\254d\347E\260p\340x\377F\n\ts\274\024\n\217\326$#pz\326\017\306\322\227]\203q4\032\221X$\\\202\000~\034\246o\242)\356b;\251\375'p\347\232\262]\314\302\326\207\331\243\271\341@\311x\020#\222%\300\356\312\324:\241\313\206Uk !V\016!\351\027\275\352\r(\370\220`*\367J\332Q`DFF\032s\034\200L\302-\201e\301\034|\201\241\334\n\365[\020\304\366\372\332\033\263\346?@\330)\322|\004\202\211)\341\324\0034\360K\021\335\017\206\021%|\313\311\222k\033a\242\350y.\205\226\354\016\230\300n\214\362\017L\350%[\3501\004t\326a\030[\321\274\027\357C\330\222\361g\001\017#i\317\345\212~hd\360$)\206\240X;\203d~`\330ID\340 95\n(\246\325\224C\275cC\354i\"\336\264\264BZ\246vm\016\277\314d\025V\341\346\003_\034>\312\013\036\260\332\220\262\261i)\250\310\313\200Z\242\306;\373\007\371\177y\260\037\024\254\024u\007\022+\340@W\000:\301$K\374R=0\\>\207\267\210\033\251\372\343\006\247\216\240\324\306\305Z\332\214\206~W\210\216\355N\006\356\002\266\303\301!?\326\304\320F\023\030\354\200\2335\241'\262\0260\210$\202\232\000B\343\237\2150\347\311\027x\314w\232sv\375\010\277:\364\ns<\233l\345\036\035\037\271\277gxb\315\027)-\224?\315`R\356\n79\342\026U/#\232\235+j\366)\372\037>\371\313\002A\320\231I\225SJC\tpD\241\n{|\234\001cn\003\336\n\224\325\333\301\253\245y<\320\361\005\"\331\320\350\312\307\225\025~7B\322!\300\241\334\025\361L\235\272.\357aN\315\257""\310\270\325\346~\326:\271\305f\014-\212\024\232\216DF\355\367!\016\261\342J\234/=\224\305Rh\377\314JXO\255\030\360\261\231XQ7\r\265\274\360(*\322\307k\260\215\010R\307Z\356Pk\317jE\025)\373Q`]\033Z\212\030\010\0224/\n<5\324\240\234\246f\322|Js\331\010`F\302\373\332\327\000\014\322\2730\027\326\r\221\313q\320\020\034\246y\000\032\345\313\301~\257j!\370\221\340\010tR\n2J\344*\204\350\374vI\"$P\204L\020R;\317!\037\030h\337\240?\004\334\027\302IO\276\304\207\022\326X4Vd\232\026\272<\237#JoV\367-\271G\254iZ\340\246\223d\316\"\361X\352^\361e[\223\010\221$\206\225\\\315;\2358\270!\267%\360\244n\211sD1o4\254\037_\3542\215\303\212\376\360z\313\207\022_\240;\003\255\377\240\3126U\351\n\202-\331;P\005K\036b\261\020\212\356\333B; f>z\337\221\"\205c\321s\031S\3019~\255\227\262M.\017\222\363\223\016\200\376\367\370\333Tz\025@\214\217b\360\343\317F\345L\262qg:\254\004X\027d\211\013\016Z/\326\240\007)~6/\254\027\241x0`\"\316\256\232>\227)\243\310p\246\260\301,33\205\243D\2760\343a\214\222\223\2634\276\032F-\242\361\267rPa@u\302)\342\204=,j4\353cL\201\230\217\337\302\206\342\225\353\361\355\021}\001\305t6\263\311\234\027T\251\307\231Zs\236+,-\314\323\366o\036\n%:\204\311\266\310\224D\354\004\250\363\226|\375\225\316ni\203\307\027>`\215\2062\363\301\001\255\224\325BX\362\0167\035\207\210\211D\376\366\002l\331\001~\347\177\201\357\014\370\263\332\276\332\276\rr!`\235\376\350F\010R\251\2536#\321JX\020\3554w\345\022#x\314'N\346\315rM\330\343\200\360\004\310\262v\272\336\2541\224\202\303 _\002\256}\273\231\r\253\322n\327\367\254`\245\216\254\321\215\325:\315\n\217\th\221\002\362\233u5\373\304l\270f\222\\\310\032}I\304L\213\374\206\340@\n\243\244C_'\025\334]t\177DM\204\330\362\214\257+\332\344\223\001a\275.\277\363\324X}\241\353T=c\356\302\002v\021\272\0368\343\r\026Rg\035\345\330k$\236\030\326+\302\260\022j)^\343\225\324\315N\rD(\363\337n\340\031\014\247'\2659O\027\220\031WV\323\333d,\333\210\364\305b\001N\355\037\355\324\232\230\321\251#\305\345F\264\362\026\213\261\224o\264\\_\252""\230\036\361\223\000\350\336\371\276\300\232em\262\270\230+&\363\272-M\204;\203\026\220\227Y9\017\2541\017\010\032\016j\241b\201\276\021DD ^\253L#\341\332V\337/|\265\027\323\256w,\2179\205\264``\300/\316d\362O\212\266\303\367\354Dg'\023\262\212u+o\263%\334\220\237\322\345\323\306`\007(1K\251\276\256\264\021D47\314\\\027\277\300\022}_\211u\021\014\305\327\245w%\007qtV+\375\370\313\246|\022\307\355\302{D\026\334b\267\265\274\3221q\"\274\340\306\212X}\274\357Yk\023\305\n+Yc\204O\363/\362Z\000\313r\272\367\"\350\342\216\227\351\002\215\006\223\365\200!c\336\244\271\271\323\320\276\212Bh\360\004\206|\317\224\371R\271*3\213\351\326\202b\2649\235\362\027>\3306[sQ8\037\222$\177\316\324\316\260\031\336t\204\220C\252\236\203\020\307\240\020\247_D3H0\3220]\212\252?\320 \252.\270\206\217\035.\261\201\201\020G}\025\335q\301\351\373\005\361\235\202l}-\350L|\342\322\347\332/+\211\320\224w5\325\0304\037\313\211\313\242fh\032{\212`8x\177\034\370\026\246\005<\004\327m\302S>m!\353\253\226M\200\315\232\033\352r\340\220\326\224`\021\237\221\354[\224OX\302\020\2422\003\214'5\206 \3560\344\240\006G\264 7E\276\2231\323\320\0276\344g>\006\375T\325\017\257\3700\257\226\364\004I.2`\271\360\221\205\236\247XS\024\005]\374\027Tm\3307e\261\220\232\031\274\\+\247g\334'BI\2207\3077\234\306\016\025P\222\347X\3445|3B\005W\021`1\3136Tu\341+wR\302\222<\310\211\351\340tk\273+\237\227;\216G\201\256I\024\264\264\027/\006\025\033\225\232a\337!ih0\220}Qp\335\244_\017\200[\341N\247\275\177\242T(?\030xx\372\315O\0103b+\367\273\262\331\017\273\261\013G\366\340\364\005\261)\251\3170\265\262$\t\306\001%\204\320\261\321\251\2104\000#\342w\365\317\245\2609\353\277I\351\2103#\213\315\207\347Pf\272\235\205\306%\370\\6\204\314\360\000\000[\342Bf\303\325\017S}q\007\247\325\356\232\354\271J?}\210>\300\327uOv\330l\222\341pN\000\002\242He\270\344!\225ey\346\370\0351i\314&\261\353\356\273B*\213B}Hrs\247\260m\223`2z\336\204\340\022\316=\005DF~\331\351\2144\303\301\360\227\212\226\030\273\002\325\343\232\344\215\242\233]L""\030tf5\364Q\356\270\224\033U&\017\353\234`[\233{\013\022\306m3\355:[\326\313\303\330c\333\356\204\026nJ\267\322\005\267\224?CD\212\230\037\320\234/v\265\177\306\336\322k\036\245\376\225dC\037A\267';8&zV\222Z\037q\033%\263\222p!\205e>\220\017\271\301Bb\345V}\323\317\224\222\375\r\374\023\n1\257Dy\027\0264rc\033\230a f\212\363\014\336e0<=\033\361L\361(e\312z9V[\213t\340\340\374e\250\301\246\005f'5\034\254\372\272\247\366S\354\177\250\2424\000s\234\373\242%\nj\305\216)\345y}\007z\0014\352\245@\256m-\022\025\024a\263\343P\252\271\351VH\307b\257\001$\364\030x\034\224\261v\250Y\001\006\334\031\215\n\324\205t?T\312\310\3607\264U\204\247J\306\275keG\010g5\320n\207\260e\254\2009m\277\212v\263\032\352\207\3215\220\355\264\220\262\311\335\266\300\3116\306\320\2178x\250g\\F\340\202\033\262\002\002\346[\005\334\3357:{A\3502`\227C\341@(8E\016[3\327\036\223\253%=\252\257yE<^\351!\315\326y6\023;\362\270\215\2117\243\205\370\005\245<\220\315;3\203\327\323\004\260y\177\0251W!\034'c\245\275T\325e2\260:\266\3573\017R\252\230\337&\005C\210ud\010\r\234\036e\265\0034L\2638\016rPA?&\267R\255\242\224\256\3101\002^\353@\227\261\221\345\234\230\005F\312E;\016P\305\231R|\346>\005\301\265\3745X>\3419\232y\020\014F\024\272>\305H\247r\022\267\261\211d!\177\273\0026p\301\265\367O\346\377J\002\311\261\t\261\221$\006\353r\310\303\342\353\376\211\317\365\320\330:\036\024'\004P\032QF\203\r\000,\000\307%\241\310\207u:\004\227H5\244\010\220u\231Za\353\307G\334\001\244!w\016\276\007E\234\252\334>\276LH-\314J\3008\202\240&\371\266\201\032\336!\n\210\24009q\t\326t\276\232\351\327\353\3731\252\325\003\373\334*\031{\021\253P\337d\031\351\025\316\016\335\035vF!\303\265\025ryA\356\314\022\324\021A\354\014\213\256\276\372\335%t\252\032h\307\026{F}\333\362\371\266h\245)\237\351,\216\213\232\252o>\312\310?\313kx<\273\3142\362\235U\203A'iW\240\212\005\312\305\275\244\214\207\301.aR\215\301\270\202'\336d\330\205\030+\033\004\227\220\244?F\2071\271\250\240\026\020L\321X\314\217\353\377\237\nI\250\275\331\246\0243""\250\007e\260)\023B\2259\304\227B8b\336\222d\354x\360\013\236P\244\241K\353Y\035\252AC A\325\266\217\250\333\3376\302l\270\250<w\001\2668+\272*9\366\030w}iB\263\016@jQ\335\266\253Y\017\216\267\327\273\256\207\200\276(7\251'\302\270R\364\271\230\225\306e9\356\340S>3oE`\354lx\202\\\256\262!\256\263\007r\277Y:q#\206n";
    PyObject *data = __Pyx_DecompressString(cstring, 6433, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (6406 bytes) */
const char* const cstring = "BZh91AY&SY1\361#\330\000\006W\377\377\377\377\377\377\377\377\377\377\377\377\377\376\277\377\377\377\300@@@@@@@@@@@@\000@\000`\032\235\367\2305\017\023\326\364\005v\356\345\033\246\255\332Ji\203'Pe\247\205\3356\330\267sU\275\306\271\266\315\353\271h6\367\271\345O\014n\235\235\336m]\275\315\352W\2404|G\274\001\360J\020\210 \321\204\304\030\232L\214\324\324\360i0F\022~\233SDM4OSf\232\232zMM\250\363T~\224f\232h\323\320\247\250z\214\203M\020\002i4\004\310h\214F\224\374\215I\232Fjhi\246\324i\221\246\200\0004\000\000d\006\200h\320\006\230\202DI\223&\205<\247\250\006\310\324\0314\000\320\365\000\006\200\0004\000\003 4\320\320\006\200\t5\"\023\"1\n\236\315S14P=O(\032\0004\000\321\241\240\000\000\000\003G\250\032h\000\000j\236\251\351@\000\000\000\000\001\241\351\344\032\246\200\r\000\320m@h\r\251\2404\0004\000\320\002DHh\023&\246\204\321\242\247\344\3155M\351OQ\341S\3127\212\203\322~\244\363T\365\030\214\320\004\036\246L\002i\241\221\246F\324\365=@<\031\352\355\275\026*\223\247u\013x\310U\024Af\332(\025\242\326-P\243g\003\032\255\357\362?\313\375'\333\237\304\311\177z\024[\372\376\004\037\356/\204?\340> \377\223\375\013\267x.\200\310\"\303!!e\340 \310C\0106?\312\213\212\300q8\241\2275T\224\270\255`B\006\205\353\376\001\003\376\220\004\030\203\000\210\213\030\007\375`\032c9I:\365\232\350\t?\364\222\025\260\331\331\rx\302e1\232\032_2\006\260\330\212(\220Y\005\026E\020R\010\202\"\213\026\014\027<4\342\342 \035\031\354/bi6l/\273\2776\207\026\2239 j\342\243Hm$\332\300b\0020\025`\262E\026,P\212\002\221`(C9%\341\273u\215\310-iZ\237`\337\211\213\224c\270\311\"\301\235\004\213\022\320\243\005\236\370V\233\010\007\022,QAH\253\t\253&\030\005\370\207\n\231\347\216\227^\263\002\304-\204\263\324\262\313\025\202\327J\024V\304]\013\245\014\232L\020A*X\270\240sB\243\222\224-\273@\334\267\347\243\003s3\302\027\306\270\264\326\231\r\215\232\225U5\263l\323`\030&\214L\260\030\325k\275^\266F\010\300\322\202\365\301\227\356X\030l\274\001\306e\212\307\005\270o\030F\222\021\"o$\260\266\227u\345""\242\256\025\250\306\304w\225K\325s\336\367\317A\260^\350\200\203\222(T$\252\220Q6\014\201\313\001\212\024\270\275\356\037\224\025{\342\243IA\205b#\025\n\242\241\325\360\rrM\262\224\3023*jBR\200\\j\331\204;\006 \177\355/\017\261\026\360\014\336\363\315frm\230\227+H\342|\333\367nF\355\320s\217\306\025/\332\206\3666\364\272z\222.WkC&\r\325\275\367\3717\325\032TWr\204E\345\324\236\ru\032\1776\357f\327\246\302\224=\237;\231s\355G)G\236\307\305\243\010\346\221\261\022\270s\275j8=c\373\037r\277\257\333>/\320f~\206\027\2641\254\343\353\251\355\322\370\343\316r|p;\374\356\375\216\016>$\020\017VW\027\177W\030p\007\007n\236\272\312zg:_f8Ri\370t\321\327\266i\273u\021g\245\276\314\246\257\252\017'4\213\310\232=\204\276\225S\027\306s\320\017O\372\234\356G'N\373\355\367b\303\034[03\0008\242<pB\313\t\323:\205\021\232\342\254\350{\031\334\347\331\003JE\037h=\365\314\352\311\363\374\346d=\rv\351\263\204+OF/\036\273\304CF\322\213+\264\03016\n\026XE\242\010\212\307\330\323\306\3265\206y96}XB\243W.\234Zq\020^'C\034\361\304\026\027^\025M$\232:\033\013\2028\234&\243\007\306M\354\333\010\326\274Vu\037\227\343l\322|U-\303\363\2326v\327\007 '\231^\322*\030a\201\250\316,F\223\355\3523\244/0\004I(\200\204\035\265kDiXF\363r\253\244\3162y95\365\220\256\204\326\2470\200r\211\271\353\004\343M\252\333x\242gy\301\247\004\031\204\302`\322\225\266P)Lq\006\256\271z\243qz@\016I\030\027K\233\343|MS\341=$mr\312Y\315\334\004\375H\377\207D\325$\264\321\302\303tk\311\302\206\224\266Y\263=\311{=\321\327\313\265\270\036\262\333\ro\370P\3245\320\267\245\336\324.\201\334\202AH\366\367\205/\002m\242\033hn\311\303$C\222\2028Hq\324\225\001\230\226\300:\230\227\225\343\230\212\000\231\"\255\t\2411\024\374\207\234.\031y]\005\355\357\3363\351|\343\334\323:\237\274e\2222\003\217vy\036<rL\342\367\265d\223\220\224\204\245\351\375!D\265\355\236\343E\307\321n!t\274*&\260\220\0241i:\014\230d\267f{\260\262\3347\236\333k\275\224\367\255\337\2276R\271\263\211\204\r\2036\334\275*\233\353\200_\260\310*\210J\211\223\354\325a6""\224\325SUC)+\351\0329\245\252\311Y\325\225%\231\037\\\220IO\tB\010*\226N\3649\305z`\203V}\264n\017\0236\270sz_x\360b\235Y\271\303\212(\211I\021i$b=\n9\302\025j}<uo\237\201)#*\351vj\001\215M\263W\025\354\276J\tB}\003\270cy\377/\277s\362l|\364\214Y\241\215\306\264\322\231Ov\227[32\372\316\243\213\336\210h\335'\255m+\027\3577\322n\274\352f\304\t[1L\313\206\023\240/\325f\245\275\231S\274\315\205\323K\006f\225\214\200\367:\264\t\346^\007\345W\030$\322\330Yl\272:\027\2673K-\272\t\202\250\245\260\022\344\266)\240\"\365S\204\030\302\212\024\201 02n\353WM\013]\177\016O\336k$R\266\205B\272\231\343\356\3763]\335\022\0320\314\310\312\335Lp\276\340\335S\371MO\001\335P\277\022\270W]\345\326\201Dh\037\203c\321\244\036L\212\234\207\340\257\343\277\231><\022,T\3173m\331\30444\364W\275\213\307\243+\273\354-\236\312\311[\246\031}p\347\244\342l\262\335S\014\016\265k\007c\331\337\300tw\201\317\355i\251\371m17\362\336k\251\304\036.\237\n\3516\300\357\271/\212-.\204\374<\376\326\n,0y\016T\365`\3663#\235\221\335b\222\241\244\213\220`\244\364\nf\037\246\355\n\253\374\223\355\005D\2061\010gZ\206L\035^\254;\310C\036S\375\321\347\215xz?lr\334LN\265\307\253p\025\354\335\245x:\361\307htb\007gm\320%\335E?\341\351\345](P1\245\321\243\363\341\247\335\222\nZj^\377\245\014WT10\261:\246FB\024l\245\035\263\033\365\236\203\213\362\343\251=l\313\341a2}=*%\251\350Ly\035\257\214f\263\372+\213\006\213lM\211\263briNl{\230)\034h\020W\2372\3653o\203\334in\370\351\020\244\306\237\217\207\255\030\304\000\276\321\277\371\225*e*IH\244Y\235\313\203\322\31275\256h\250\237\256\016@K\353G\235\217\024\214\205\234\247\345\252R,R\271s\2671j\305\274\014\024\345&/\265\016\205\\\2443\317v6N~\275\n\366\274r\027\371\2759\363\222\321f]Mlo\306\006\355\241\375\275)\300\002\201`\244Y\323\344\263v.Xd8\274t\337\207H\027w\273\361z\276N%\256\243W\305\307uv\256\242\271\262*\225j\315tS\314#\025o\275\366\222r!+]q\354Lti\222\251T\272@\205\034)\336\201\313J\032\001\026\032$\260\351Y\002\310\204\340\034Y\3166\340\326\210\362\210\254\234h""\370\003U\016\2657\016H\"\211m\223T\347\310\216\216=\316\334vc\021\"\234\235A\223\275E\306\236\316TX\t:~\026\036zN'O\3559\202\231g\247ir\351\373\322\222\022\360\346k\"T\254b\356_t\246\032k\233p\345Z\361*\374\325\253VJ\325\216\222\025\361\nT\262\244c_]\037\210\257\000\246\270\206@\333\224n\374\304\235\372\265\356\335\350l\330\233\006\t\227\223\352yv\266C\346\212\245x{!\325\207ryx\2152\3367\350\035#X\007\001\332\226C/\215\317\277<!\315\205\002\033>\351\0174ZM\351o\363F\210'*'\027\237\037\013\270t)Xu\347\035\323\342\200\332GT\363 !&_\335P\226\301\317\034\205|\205\350\355G\206J\377\225\346\311\363.s\332\264\325\030[\2562\002\275\250\301\317\303\344\230\270\022\267\317\346:s\232\342<\345\367\031\037\002b`\024\346f\247cO\014C-\242p\314\216\301!\001\005J\020\304\005\306\2779\240G\005\320Xj\307\010\2702\226\353\224*\335n\363aH4xHdn\272N\306G\341?[\2519\276\370P9PC\rG\016u'\205&D-\221\321\320\341\014%\2347\274\030\343\243\000\305\000\212h\n\206\201dC\375\305\235\036w\t\342\340f\321\023\241c\003\2529\350a5U/\322\037\027\3510\305\361j\236\342\330\r\241Hq\003\027\355\260\341\373XkL\023e\307\356\343\257=\265\035\n\255\342$U\341\221\024\023\202\326\236`\026!\370\362[\375PN\010\332QTa\264\000\367`7~\265|\006\n\022\200%w\241\240\311)\354U\215\374jR\315mIp\260\241n,h|\376\233\003Z\226\327\024\256\245\002h\203\013\016\330\255G)\303I!\253;)\243\264d\265\337n\r:\334\177\325\235\034\373\234\274\020H\032s\243\033H\347\r\323M\230\230\373q]\341]\020w\306\200\314\313tc\306\321\311\021q\250K\341I\276-&R\307,R\266\347X\244\244-\341\254\372\032#\323\221,\014~>\364\r\266\202R\361yE\337B\361\240\361\201\343\373\001\357\307\220*\005\234\200\320`T\240@,JPr\262D\214\300\330R\352\255\241)\321\343\336\360\301 \236\267o\241\020}O\205\r\370\303\007~`G]xT\316\327\312Hh\220)\336\270\\\322\031\242\334\r\321KF\r\314\021\271\363Sc\227\206~\264\265y(\031\222\332{\230Lsc1\t&\272\273JM\216p\010c\025\204\306\231\016\332\016\331\363\024\346\215a`\017\027vt\305\206P\310v;\367\223\310\356\265\244zq(:x!g$:\377\021$""\242\333d[\257\242\224\2333\335p-\230\264h(2\232\243+S\013\343I\221\300\340\034\016\256\276\316\361\236\311\325*\2369fU5\020\260\331\262U\247f\332/\013\322%}\034j\341\014+\013\255\001W(\265\027,r6L*\343\200\206KTar\243\346\324\301.!~\311\224\336\034\227R\242\213P\350-j\325\3713\366\031\327\205\255AZ\353\275\307\030\210\3725Fgs\261\005s\210\251\265JoB\214}\332m`\354\231j\374\213\330\024(\306\0229\316p\022=D\214e\001G\206\365\304i\304\203 \024\232\226e\031|\210\010U\342|#p\276\216\351\337\330\276\367\2311-\212\344\254\\&\217\210\203\316\226s\240Y\361.\027x\216\2160&\266\312\342\367N\357z\200\355\244=\215D\346\374^R\017:8\306M]\3627\306\316\332\206\264\016\013$\240\311t2m\204A\336\246\243)\211\222\263K\357q\302\334+F\007\241\266\230\320\260\353\260\302R<A\354\215\010\235i\241\245I\201=\010/4\302\017\002>\024@\250\342\036P1\"A\354\030/\\ ^\241D\230\035nz\2209\314:\022cb9OEH\271\311\343P\020\335U\353;\002\302\345{\222Ye\301JWH\327;\271\204\"\233\260\317}\331U\215J \203\3105\363+E&\345k)\310\354\327\226\214\\Y\366I\272\247p\321\031\030\214\273Y\217l\265:RDH\300\211\007\355\020u\367\034Qa-\222s\314\271\210\243\032\214\022\024\220\307-J\220\033\341\224\3579\343\261(\304\351l;\035\022%k\225t\013\267|\3161*\027D\365'hk\241 \006x\311\220\314n\334\367\361(c\223\202\215(\2719\234\035|\314\025w'2p\270R\305N\211+uIm\010\336\324\244\020C\3002\262\01417\001\255#\004\0347\362M\215o\262\201q\347~\3619\023\004g\306#\332\2358gv6n\245\201`Y\275\250\030\r\332\327(\026:\213y\025\216\031F\223\360\2107C\231\000\351\332\242[\375\276@\371pO\313\324(,\327R\013\365=Y\321$8d+\270\364\352\273kGrK\301\265d\267\207\nS&*\307?Z)\326\017*<\036RI\021\007A\0106y\327\024\350\346\204r<\031\325qD\221\341Q\256*>\013\001z\3055O\260\347w\272\305\036\032\234x[\225(\261;\017yD\205\202\334\345+S\363r\236\302\363l%\326\213\021\0247C\304\332\304\030q0\256\000\225R\240r\263\227}\020H\332hUKN\254V\024^\236v\323\351SH\214\225\021\206Ii\333 #\252*$\371\363d\t\345\020\021/\031\217\031$\023\r\000c\240t\311%\336J\002k""\275\023\252\016N\2732x\301J\240\326\344l\207\022P\206V\363\353X \016L\225H\035\315\260\230\317\016\356\211\277\022\327\341h2!Y=\374\356d\020\034\252\241\312h)\210\303,\301\375\325\355\033koUW\300\276\007\314\357\352\036=v\303\252\267\314\034\031\316qD`\261V[\204\233\355\315!\253\234F\362J\000e \376\255]xa\301h\327\256\340z\010\017\t\315\236\023b(\221!\324\020\246k,\277\241\007\261\224\330\2617I\216\373\000\354\342\037 \363\036\260\005\256D|p\362xnY4\326z\201\021.\370\027\310\241\024\201\276\271R\246K\336\354m\n\010\306\002Xz]\372CO\0179<\225}\027\r\372VF\307\275D;\301\004\365i$N\033j\226\024\346W\210z\261\300:\272\t\232$\203=\003\351Iq\026\200\003?\005g\360\323\034\245s<\006\261\356L\006\034|\201\257\244\242\355\355\263\2523\346\234BA\320Y\213\213\026\331lhu\207O\261\303)\251\225D\244+M\013\335\250\267'6\224Wi\356D\356KX8VZ\214\311\244\030*\267\242\202\240\364!\207wC\332\343FRc\252@go\347\273\350\021\375\367D0TG\346\230\343p\227\320\223%\263\206\240@\341\244sx\365\334a\005\272m \301h\224\271\230JW\243\252p\"WY\311\023<\2213\234\243\343`\213\305\365w\362\344\"$q\355\347\273\313aWyE\0026\010\004\n\224\014(\256DG\373\365E\324\271C\006\211\336\353\305\251&\352\014\332K\332\020Q\014\271L\"\365\311U\345\353\230B~\275z\323c\310$X\n\"1\220T\267^gN\335&\222\253i\033'\024\305\210\233\231\n\240\254\031f\345D*\312\365a\332\365\2710zV*b,8\\z\332$oi+\200\274}\371R\272\310\3231\372y\245\374\320\t\334\032\301%\020\026\017\010%\237\243\331\362&0\251\270\226\032)c\333\325\000\306k\024S\350G\207\232\343\211,D\331\214\030$\016x\316!\367Y\034\241$\244\333j\305C\345\247\374\005\005E\326a\305\321\350\323;\326\317\220\210\031\311\311\226\373\003\020p8\010h\201\270mC\021i\001\225\3543=;]j\261\020f\200\221\343rT\223\255\024\357U!\260\033\033B8\265P\242\273#\371:\376\356\341\2407Xd\302h+=\214\372\335\365S\216\220$nh\301\036\023nQs\357\357\025\236<R\353\017.\025\342\335\267\301\216\376P\342TX\"[\225=\231`\273\307<Oqm\013\241\275\2275!)\210\010\346X\234\340\204G\302\223\320\277/s\221\022\234z""\202\211\rl\216\236\030\325\025\021p\021\362\332\265\030\227\350\331\223\257(\t\263\304\336l\024S=\rT\325\r\240\311:\305\324!DCV\017\272(a\201Y4\304!4\242\360y.u\221\t\215h^U\3429c\313\244dQQG\267\321kE\024X\306+\301\241\202\014\252\240`\202*\240\276@RR\"\026i%M\266%\0334X\271P\014\255Sp\304\355\222S$d\260\271]\264|\231\236\344\0358\241z\351u5\335\274\265\366\315\006\003\3238\260\310\300\256\216\227\226\335B'\257\215\260\000\253\356\302\261\253\350\231\267\\B\332\222jp\0317\311\263\254\237\005>\0070\255\273%\247\026!\230Y\336\003M\020\211\342w\224T\022:\235\036\372\245\337h2\230h\207\264U\213\001\230I\204`\305\n\014\r*Sj\010'\031\345\334\242)r\250oV\031\031]\235\263\247\253\207\003s\225;N\325\341iW/y\373\322\027\004#\210WFj\304\222\351\253\002\023AA\031\256\247\256\301\222\302}3\255m\0329\3309\322~\n\313\026\242q\240t`Xn\014\372Ue\232\364*1e\005\201\213\324J\254\333\256\027\002B\024\030\002K\227|\244J\033\234[)\244Q\251\217\226\201r\002\344h\230\221\256(\215\346%\254\353\344\325\006AYXV\330\317<\254\303\316x\214\346\267\323\023\023rL\300\251<\0207\022\202\224\034B\266\227\326V.\227\035\252\233W\321\020\0258\231\037\276\231\t\225\273-\324,6/^:,\332\r\330q\306\025\354\256\312G:\351:WWV}\327\234$a\214\215\363\363Yl[A\241\246\026(J\301\242\364\321eC\310\324Ju8[\013\024\327\225n\3469\346\334\003\024k\206w\301\324\312\240/\201{\250\321@\202I\001\010d\263P<l\241\261\331F\327\331\312\215X\262\204\210\216r\024\314\273\267\271/=~\271WX\300i\215Ca\206\313]\232\206\024\224\301F\261\266\031\342\331\242\250\205\312\231\003\024\300\242\325\027\236\023K\232\0211\005P\342\036\253\001\3054F\021\242\206&E\326C\332\202\020 0;q\021 \305\355\002v\270\3565Q\276\342;\374X\241iYc\240.P\332\004\033z/\233\233\036\247\211\3522\000\207\023\245\204!\334p\360s\227\022\374\266\003\006\035\301\245W \327\312d\2101\306o\246\231\316V\305\212#\206\255\233`f\2202\n3o'\034\345\330\327:\\\265\026oU\222}.\300\354:\210\226+\243\353\344\033\263\312x\001\202\024LW3\234\221\003\324c\0106U\014i$\222\276\000!6jc""u\327\034,=\310\230\002.W[@\230\005\024\222\325k\020D\224\217\241b\375\020\252\252\325@I\260\357B\252\354t\354#\321\265\232\334W\327\350\016\342;\336\037\017GG\014\263J!\363\235\375\376\002\342nArjdm\230\371>\260\356\272\354\206\371\277\301\274\326q\252\243m\241\2151\256s>\316\274\362Y\3432\245\025X\021\334^\357\037k\236\227`\025W\244\243\304o;|\231I\320\3439\332\253SPd\365\327\271=0\366\361(!\246\226\335\335\352\301#\272\245\302m\003\327\023[\372\252\0262\331\217x\224(\200\224\034@x\242n~( H\340\313+y\315J\330\322\264\270\366\032\033\004R\305n\013\315\322\024zO\002\000\356\347\034\035\253\ri\"\275\261\rh8e,\364\001\357\021\021\217\373\200s\241\004\223\314\014&\030z{[\315\237)\363\237\037k\370\237\215\220\363<\360\363\225\024\021\221\031\026,g5\022\206\013\354\212\212\2763\360y\377\264H>5O)\222\242\241\333q>\377\250\3714$c\330\366\217Rq\335\202\017\203\374\346z\227w\374\237\316\033\035\241-\003\021\270\327\231S\341u\222\010\315\376\2107\350\336g\036\336s\231\360+\240\250F%\317O\373\347\036\017\377G\364\347\302LKv\250L\370~\221\323\342\025\037\300A\r \363\226\257\343:\274C\034\013\367\262xQ\036\363\373E\010)\022\212\212\276T\017/ \332\317\027\215\275\313\351%4f\206\233\207)\021\243\316\356\006\177\361UQ]\327\324@\324iE\2623\211|\026\374\343kz\016\215\210\225\021\344\310\3211\004\311\344\232\224\367\357\244\273\342\020;W\367{4\t\363\023\346\031\036x\036*\376\214\307\354+\274\215\330\024\364\253\3726\333\205`8p\355\316\373(s2<7X\374v\243\301\372\007\026\177\227\230\303\243\357|\301\215\373\357\222\363I\037\351\271\036\017/\361\201\372\353\3201C\340\213?\t\253\306s]s/\265\230QdOp\027$d<m\300\203\225E\017+\235{\275\005\210\030ys\203}]\277\022\204\210\362\350\365\211\215\005\221N\361{\334\342\354\221\336\365p\335xz\310\232\002\221\216a\224W\216\274\350\257/\027j\331uC\314\247\237\3769\307\0022\260\247\014\034\343\342\310\301\005\234I\251p^\016V|\036\210\241y^/G\014\244\034=b\033\203}M4\340\347r3\244\344n\026w\"\222\227\211H\265\022o\037\023\256\251v\0032\273\201\216G\311s""\302\267t\277u\377:C\005\301\232AA\313\010\245C\202|q\250\365e\225r\357'\355\376>\022\014t\"\271\313)\243\204/>\034L\021\362\311\341;W\270]\"\3178\2409n\232\230\013p\354\233\316\221E\005?=\377\336?W\252\314\010\227p\376_\303\357\207\234^\225\357\372\210\220\037\335\244\271D0\373\207\372\261za\371\025\003\331\361\336\347\335\336\330U/\235q^\363}\301\275\240v\021g\014\355\277\033\203\306\275\034\242\355m}\233\320m\336\317\313\026\361p\3216\222y\342~\353\371\337\271\026\327A\342f\006]\255\223oq\207\033:\272\334\235q\265\212\ns[\360\374N\263rvu\340\350}q\363\031\206\277\324.\322\367\327\231y\273z\273\007a\2760\037\340\201\r\255XI`\004\262.\357\016\303h0\306\320\210\2646\240Q\223Q\346fC\245\034\341\250\316\230#(\220\t\276\030\354D\245}\310\016\222\3663f\n\205k\224\013\223A\351\201\005\367\334\310;*\303\333\001}#\272;\210\251q\341\216\253\374\365)F\tBe\236\260Q\037!\371\320L\004\027\360Y\334\343\262\031\306C\305\226\271\013[\301}7\006\324\234uz\274V\260\222\321y~\277\201\370^S\323\236\277i\370e\251\305C'\223\014\351\027{\246J\251\222\247\253OErN\377\335\271\221ZD\307\253\362\334\313\336\251o\273\261L\306\251\342\241\333\313.\033$\004\223\373\247\333\362n\246q\227\334<A\352Z\344\313\337&}O\305\014\334\251\222\270E\035\277\203{\370}\026;\250M\226'\363\225\014\337\005\246\302\3047X_\026\217\274\352\367W=\337\354\222|a\204PR)3\326\362/!\277\324\360Vb\321\201\341\226L\021\225C\333F\324dNl\031\301\261bi\220\203\027\nfX\255\206 \244(s5OUI-\252o-\311u\255k\030\261 \363\034\006\340>;F\370\005\2220\214\220d\032\004\203\2277/a3\264iT\205q#D03t\"A\003\265\030\2520\207\241\2459\263\273\014\277\021\351\346N\212\023D\335h\326\r^\204\326\322\035\n\036\035\241\344\353\204\201\244\010\033\367P\024w\235\316\322\035\020u\365~!\307\272\323a\034\231\010\211\204\274\256~\373\216<;\333\203\037)\257/\366\340\361(P 3\324\357\357\353X\350\346\324\362M\3337\247\261\230F\210\006\341+\265\024\204\243t\3451r\tV\031=[\267=\261s3\251\315V\256\360P\233\2213})\335\333\3255\226.\350\336\2709zDYzQ\2741\327""\033\306\363\214a7\202\276\353[\353\0234\2031\3530W\221\014\013\316\353N5\240\264l\257xu\035\035\262\306\321\202\350g\021\377\305\334\221N\024$\014|H\366\000";
    PyObject *data = __Pyx_DecompressString(cstring, 6406, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (6484 bytes) */
const char* const cstring = "x\332\255ZK[\333\326\272\016\247\264!\tIp\002\341\232D\220\213!\rNi\310\215\\z\314%lv\023\010\230$\355\331\247\217\036Y\026F\301\226\214%\003Nw\232\016\031z\250\241\206\036z\350\241\207\014=\364\320?\241?\341\274\337\267\226l\001Iv\273\317\356\343Hk-\255\353wy\277\367[tF\211\347r\231\242i\245\025g\313\314\251\033\206\346\026\362\206\3634\236\317\333\273\212\226\317kE\305q\363\206\226Uv5G\3212(\246\212J\336\310\030\232c\244b\263\205\215\r#\257\354\230\306\256\222\262\rG\261lW1\366r\266c\32083e8sk\tE\267\013\231\024\177K\032\370\354\3465\3355R\nf|\373\343\372\234f)\266\225)*:&w\rES\222bVwSs\025\323\301h\3135\323\005\273\340(\246\245d\215\254\235/\3060\212\346\323\034\307L[\212k+\264\263I\236G\364\240=\311Nr\342\335\274\351j\311\214!;\210]o\344\355\354\227\306\232V\312\330SvMwSq\2139C\211\312v\234\301r\370\234\355!\242\033F\230yCw\225\224\2315,\307\264-g\256u~-\225R\244\224i\323\031\255\210\203j\3012\355~r\313A\327\317\367H\033v\326pq\232?\321\205\317\372\366\307\331/\364\315j\316Vk@\373k\312p\371Dys\007\373\2657\224\224\346\302\000\334v\017\303\322\355\224\241\354h\231\002\t\251\335.\264M\037\314\224r\334\256b\355\256\033\330\023\036\006\252bw\033v^\326C\023\246\rWN\343\350\233FV\023\247:6\031u\023cS\006&6]\350!4\341'$@#\204>\332#\332_M\327\310\223\200l\022@\340'3\241\341v\322\325`\236BD\341\017d\\-E\262\205l@\022\355\357\020\243\002'\231Q~\375p\270\261\2557;\377%K\240\276\350\250\221N\346X\215)e>\260=\345F\212|\210\225\310f9/t\270\360*\2618\263\220X[R\022\233Z\016\007\316\030\013\331\234\013\265PUq\01398\n-\254\027\335M\333\212\261\316\026 \367<\3310!\206\324\214h3\366\014\275\3402\220lg\202\306\335M\314\252h\272n\300K\361)\251\271\372\346!}\205\373\261\005\266'\346nB\321m\265\205\373\223;\267g\2053\255,\256\t\005>\327\360=EM\272\235\315\232\322Ya\205\220G\350\233\260xi\311B3\201\366\332\275\354\234a\211m\304I\002\t\001\206\274\271\027\207\327\342\236\301l\343Yx\303\323\366G\307\325\362\237\331\007\235\243\345\351\016\265\004\223,\316\307_\300|\320n\241+P\306@M\267""\363)\207\260\321\"\2405\030H\223\004\306\000Q4\033\004\266\226K>\032\352\013\003\321\010\3340i\321.\344I.I\323\322\330)\320\023\332\207y\003\337\3230a\007\373\334\204\311\310]\304\024\345\025\243\2758 \2265\035\007>\216\221\213\020Y!\251\214o\272n\316\231\271s'\315\365\030&\277\003\333\315iVJs\356\344\212v:o\332wx\2243!\240:g\347\005d`\253F\236\264\310K\032\244\336\230\262\264\254\214/%V&\037>\274\367hrj\3518\236\n\263/\344h\032#\265d\tpI\254\276P\266\013F\2768\243\034k:\326\300F%\334=\032|L&\355\275\366X\322\340\355\266\214\243zT\301\220(\306\221\032\243\267\22546\021t\026^\003\377\327\366p\216\307\312\022$n\2467]e\323\310\344\330\0000\215\271Q\224\032\312\347Cp\212%2\246n\272\010?\311\242\222\313\303\031\367\204H\014\322\214\241\3444\230&\333g\364\311\374\332\322\233\205\265g3X\337\210\245c@\306\304\233\031\352\020\215\275\020\247y\362\222\003\322\033\nH\320\355K2\0336\027\3332\344\2117 w\002Y%\272\214\306e\3335\204\221\314\261\257\023^\244\214\214\231d\304\3130R\233\220A^L\363j\341\325\344\364\303iXB\n\232|\207\2038\320ER\317 \026\303~\261d\262\000{\302\354\0240\035\250s\203\214N\261\014a\3579\364\013\017\3001-\3060:o\224\243+\033\246\212\341\020CT\342\026DE\243\237k\031\307\210\221\253\013D\340\240l\036\261\010V\256\304\207\031e\245\300\356\220\244C;d\267\222_\010`R\306Yek@i\022:{Y\333\031\351\300\024\320uFU\250\007+\253/\324E\303}.\372\314\261\267\035\335\300\232\241\033\3300\3118\034\370d\304\032\267\n\231\214\242\003\305R\023\211\205ut\202p\r\307UY;\216\362TI\200\215\345\332X\030S\022\322|\204\376rZ^\003\342\223FlE\333\261\261\000\373\317\256\226\2670*\366z\375\371\344\303\327\026\223\035FA\013v\3061,\313H\225$\255\275\225\370\251\265\003\215ng\nYF\004\026\353\317\320Z\266\000<\310\345\355\035p9\n>\004\005\356\247\306@\236b\267\223\267o+\360\000e\374\207\330\263\350\304\304\214\3624:\243D\3615\244\036S\004\227\254\346\306\020MT\213,\220\354)\312O^\234\026\373n\257\265\"\031/c\232\226R[kcQ\001\217*B\252#\366\n\206\271\236'|\312\313\255\265\333\227y\222<TeM""\032\024\354t\301\345\202\343%m\270\030\023\031\222K[\310\264\247\261\205\345\271\225\371\245\345\3051\340 \231\347c\245\000L\374D\357\220\263\007\216\336\352EvI\270\2528@a\335\370\367\326\0276 \302%\341wh;\344B\307'\220\313\220\217\361\022Q\002:!\352(\021\276\350\261\016$N\321A\263\212$\367\350\356&L\024\240#\306\336\226\003\361F\274\347W8\177\020H\231\325\366\332-\377\361%n\037_C\031\207\177\351[2qp9e\241x\246e(\324Je\000\317\340o6\007\327\tXH\306\340x\354\304\264\244\376$\224j\320\316\004\362<;\332\0340\374g\321\303\231\r\005H#\005\375\032.B\325\375\351\177\314\377\022\252d\235_\260CpK\255\220q'ncr\207\035\264\2257I\377\010\010A\301\342\215\001>\334\242\301\276l\010\372\025\244l\"j\013v\324b\023\010\037\320\273\301\201\216D\374\030\221\177/\246X\205lR\366\025Q\307\221\204Gl\207\273\247L\215\204\241D\201q\250G\t\325\246\037=z\300^\255\262\331j\001CW9Q\010\034\313\201\235\301O\255\264\273\031\356\3144\237`\237Y\315\341~\224\374q_G\364;:\316\3713\223\177\262\023\315\334\316\212x\017\343\241\254b&|\200\2772\313\227\342\r\013\2262+\376\214\212N\n\372~\376\323\221\211\203AZ\217\245S O\304\"\322\314\316)(\3210\244L\206\345r\262\333\242<@FT%\345\016f-\032\241\231c\312\n%\260\264\t\312\231\0130v\213\214\275\275\364x\024\007\322h\355\330\356VR\370\250\215mp\205c\365D\314t\204\251\245(V8\346{Cy\362T\371\356X\"\020e\014\212\2060*\020\241Fl\201\246\206\223\t\212\2241\035w|#ck\356\375\351\t\256\340\323\335\357[\305\240\025\343&\216\370s0\3513\354\301\262[\306\252\252y#U\320\rUUR\005\216p\204\350 );\260`|\325){SUj\344H\330\206\304\317\253\243\215\2374L\204k\216\313\033\232n0\212P\032'\"\347\263\200\323\252\246\035\313\025\367\224\274f\022\001'J\316.*\231w\336\000\325\304\300\224*\003\377\321\345\217\242f\370\2742\206\320\256\217H\345\363\207`\244\014)E\202n\322\220\204\037}\005\215\024\352{\"\256hR\207\260\356x\233\320\346\247\276\207\2000\270\314\210\212l\207\014\271\320\" \000`[\347\273\035vGr\340\330'\276\n\022M\023\313\273\243\330\214R\260\332\347$\326Gv\212Vw\003j\345\263X""\344\263\321xbniIe!\210\024-\\\216\251*\233\275\252\253\"\357T\325#\237\205\301\314\275z\261\240\316\"\325\3414\223k\340\306\211`}\321\272\226\340\367\374\372\317\257\026Tb\203\317\227\026^\314\253TM\314\343`\314\302E\027\324\022\034\347\271\032\004p\312\274\0272\0313\347\230\316\363\245y1<\361z\226g8TS\227\343/\203\246\320\327\320'a\023<\375s\302\006Q\312h\356\242a\203\345..\254\274\\X_\373\231{\243\351\357\211\225e\371J\030\333\213\022\336x\320\313\327/\326\227\324\205\237\326\027\226\023K\370\274\254\255/\203\245\276\262\331\t\270\313\312\312|b\345yb}\326\266\301\311,*>'\267\276\373=\025\227,w\352>\025hvz\277\324\366\022\205\344:\364E5\342^\364~\375zi~\345\371\372,\262\317|\021\005\010\311\220\257u\200\035\212\230\310H\033\371v\351\376t\270\374\002X\321\256\313\032\330{F\276dK\202M\260U\220\255r\211\2670\256V\207v\205;-\316A\243\363\244\330@z+\211\227+\210\354\317\021b\027\347gU\365Uq\017\377\346\221\022\251\313\300\3525cc\215\375}\226.\"\204IQ6a\344\023\177\213CS\201\336\021R\001\252\t\302\004K7\000L`\350[\226\275kQ\246\026k'm?\275H\374\004.B\274\230\034E\3257\013\326V\340#\252\310]`\320\"O\002\006\202\017\030\310\264\311\263\216\333\271\250\003\2664\207{\200JC!\232S\264t`\027vm\323\255\215\341$\351\n\201\376\221;\361\215\212J\350OL,\tu\213\334)Yt\r\234\323\310S\301Qu]\313iI\023\374\3044\034\224\235B\306P\345\033/\262L\225\361\320\001$S\262\007\267\223\2054H\021\002\014U\261:\334O\245[B,\254o\351\031\3331$s\007\357\242C[\370G\024\014\302FV*KY\315\322`\000|\205\240\347\035y[\241n!\007J\267j\301\265XP\027\\\231>\033\0013k3\264\240\244j\216*\020\254\325\302\327\232)\255\2302\350\226SUS\244|\374\227\262u<\230P\245H\260\374PMG\025\201\202\253\016\247\032\342~4\010D\252j\220[\321\253\220\3454\033\202bY\031{\306\236n #u!.\274\261\314\006\224\026\300\277\312\347\345\026:\217\005A\250\311\242\312\314F\264\346\220\025h\031U\220Cnra(\031U\250\360\36042\244\200'\342\247\n\241\267K*\023r\252\223\370\210L\322?\022\r\363\020A\243\222\342\245\267\251Y\273$LW\324\315\324\236,""\360V\333EZ>\340c\242\204\330J\271\277q\270&\2739\205d\373\333\341\222\354\301\272\022\324R<\261\271\214\226v\230\202l\010\300\222t\204\210\232\241~\237\022\357\303\262\341\246C\242\343|U^\377\320\005\240*\334@%\246\256\2520\226\214\232u\322\374v\337\313\227jol@QN\032\231\301^./\031%\374%\340\226\301[%\307kU\3508\360\021\374^\330iX9\n-\002\020{BP\220q\236\305\016CD:d\357\355>\255\205]\272X\"+K\265\215\020\255\216\000\221M\304)\323LAKf\332\202\025\246T);\325\314R\364S[\005>\271\316\2523\255\r;\210\235\010\023&E\000\346v\314\352\360x\210\177:\007\020\270\264;\225\"\317ha\216\351\200<\230\216\245Y\364pM\307\026\022&`\020\354SR\320-,\306\334\202\037\252\270\267\025-\322\335E98\274\250\221<\305}\n\261K\026\207\221\261\323\370\245\371\354Y\r\250\303V\027fWY#K\177\311\241\251T\236P\002W\326\324\363\214KV*kZ\005\354\214q\000O\002=\274-w\223VT\325\3203\211\263\341GxB7>\022\244\301\231\000|\226\261\253\262\373Z9@@k}.\267\034\250U\343v\311*Q\004\377L\276\023\020C\356!\321\215\212r\236P\021\244\231k4\215,\007fFe\026\021\027\314\326\005\024U)\227\025\321\203jt\355\022\274\245;\264\252d\005T\221\236\351\264\034\226\n\234\313\266\nrB\001\302x9\266C\177O!\256\2472\371S\003\333\314!\264\350[t\257I\377T;/\003_P\333\000'\313\2510i\335\244\364HUe\336\313\205\235\243\223\301dm\227\274Y\315\"\1770sE\336\207d\3611\nR\255\262i\267\212x\005\345 \214\201\354\253|\307\340@\037\\\223\376CE\306~.\025\254\234\251o\3010\026\254\240\337\016\377\351\221b\306vA\313\010#\001\216\244\354,K\361\320\275V\236\331C;\313i\225D\n\026j0\366\250\230\206\352\310\263s\031\304\320\243IG\350\202L\026\371>\323@\rH\231\317\202(\300\000H\243\334]\330\270cd\000[VJ\025Q\206w\213\222L\300\270,\017\036*\007\273c\341;\233p\362\014\375q\026\351?\234\370P\246\343\000H\234\034\210\003?\340\004\310[\360S\345\r\004\246#\351\355\030\031\376\263\211x\020\331\307\261r\216k\343_^2\034\311s$\353@\255\200\341\202\260;j\350\317-\216S\204\372\\\272dU\335M(\237\302z\030\335Iy\200k\366\320B\001\230U`8+0\236\025\030\320\350\371\260`""\201\261lA\301\330b!G\004\001\253\031y$\217\2524+*\272\331\234\272\343\230\204&\024\215\370\241&\371\345\320%\255\274\250\225/\207\003\355\256\231r7\221\213\343'\215\035%\332\326^\321\320\362\357\241$'\276\266\266\362v\206\357\007\310\352g\002\304;\332N\n\003\203\305\370\200\343\023\rW\306_\025W\0304\224[\023'\332\345q\276@F\023E\220C!H%\316r\27494\36433\266\213\267\225p\227\317\365\201d\025\351Od')\223h\214\2769qB\000\216\340\274p_!S\336\021\201\212\250Jl!h\020\375e3\372\213\221'd\2030v\264\347\330\270B{\32175\272.\246\277}a7\302\214\344\206\000S*\305\030\365\201\004\003uZ\025 \352J\226\250\334j\365\344\373:\356.\244):\034\021\365\304\t:.]}L\234\020\242oi\200\261\005(\0143e\310\221\r\350*K9\221\217\265.r\3027:\277\307\233]g\366\247\032\335\227Jio\325K\372\337\370\232\277[6+;\325\265\352vC|\353/\355z\232\227\367/\371\333\345\223\345b\365\233\252\326\354\232,\2576\272\272\367W\274{~\207\037it\237\257\237\037\363\257\373\033\345x\371\227\352X\365\307\332\366AG\243g\300\353hD\372K\333\336Y\177\315\377\2652Z\231\342\266&\036\337`\316\017\345x\263{\304[\364\247\374Y\036\372\266\262T}S\233jv\rz\3754\357\357qZ\344\205\327Q\357\034\305*]g\352g\270{\243\373B\375\302\r\177\325\327\032]\347\367\335\322Ty\365\367\216?\272N|\375M\243\363\353\277\364\303\260\241\343\303\376\370\346\304\327QL\336y\325\333nt^\303\202\235X\255\321y\253<\332\350\274I\037\276-\243M\361;\344\207\213%T#\245H\243s\304\2137:\257x\272\037\361\357T\206\253\213\265\231\003|\037\362\256A\276[\345\376\312\311\312\373Z\204zh<\236VR\274\355f\347\311\337\177-]+A\250\347\366\337y\035^$hY+\271\336#\177\316\337>\362e\254\004\331\\*\351^\304\273\345\307}\326\306B\251\257\244\225\034^K\203N\366\235\322hi\006\225\224\177\r=z.\225\336\221Ti\202b\351\253\322\024\026?5\010u@\317\217KI\350\2525$\201\223\211\356\215\336>t;;,&\025kG=\322\305\376\335}m\377\237\336]>\353(TR\272\345\305\275Oo\344=\266y\315{\353/\224{\313\211JGe\250:U\235\257}U\233:\264\312\251K\245\325\322\2067\313\353\324O\366c\206\321F\327\2207\332\354\374""\272\331u\256~\356j\375\352\303jGu\244\266\312[\257\017|W\351`[\030\363\243\345\201J_\305\256\2556`Q\313\320S7\211\270\373\354\376/\336\250\307'\355\336\177@\022\376\343\364\211\263\003\245\355?\272\351U\364\277\362\177\200]>\255\221&N\365\227\264F[\030C\365!\250\031f\371\272\022\255\366V\177\256i\322[zJ_\227^\323\264\215\036\305?_\336\255dk\217\017\264\203\355f\317\240w\311\333\366\377\013\273\211`\013=\221F\3575\377)\316{\245\272]\353:\2405\372\006J\277\371\211rGc\000\306T~V\2154{\341 \215\201\361rG\371Je\033F\334w\305[\255_\271S\336\256\234\251>;\2104\373\007\032\275C\244\344\306@\254\254\225w*\253\025\2551<\"\346\372\300\2461\020\205ov\360\013\023\234\032\252\017\362\306\233\34400\341\272\362]\345+\370\036$3\017\003\350\203\223o7\303\247\030\250\017\334.\307\353\223O\311\373j\263\265\364\301\352\201\326\350\271X\272\357]\300\367`\371\377\256u\324.\221g7{.C\231\260:\022\310\023\302\007\022w\312\037\365\037\227S\225Q\022\345h]\231\256\304+\377\250uH\030\271\014[\310\226\341\333\201y`\013C\336m\332\013Y\010\031\222F\326\220\361G\353\327\026\261\312\351\372Z\202\224u\2614\356M\370w\375w\225.\350\236\260\006\036\377\000\226\201M\254\322&\311\302u\377\242\037o\366\3665\006\306\260\213\273\276^\036\252LU\346*N\365Z5Q;}0zp\367@\257\257\276\256\277~\323\030\214\372\357+\221\312\265J\242J\240\321w\25118^>\r+\230F\367\353U\2756t0u0w\340\324\327\336\324\337\274m\014\302\377!e\247|\255\274V~_\215TG\233=0\304\273\225\251\372\364\337\010\345.\226\236\302\214XL\223\244\214\247\325\325\372\263e2\010\241\246H\177\375\322\004\014\"V\355\256\315\t`$\274\010\272?\257i\265\240g/\265\367\227\366<\327\237\001\220~\013s\036\210\326\243\317\252\032\340x\2200\220\252\217\352\217\226`\030T\234\203\276\342\362\313u\177\205\014\231\333IU\021\034\216\326&S:[\205\250\010\315>\267\364\331H\351\254\367\032\262\2039\\\361\336\301\032I\200q\022\262\n\027X\245\326,\033UO\177\351={<\246\272\347\235\364\n\376|\271\223\273\364\002\032\375\017\225x\263O\021\002\351\243M\261\201\216x\177\347o\213\325""\351j\241\026\257\2556\331\376\237V\311'\006G\274i\357=$4Y\331n\214\334\202\240W\313Zs`\2601r\323\337\252\364\263\326G\340\352\225\313\325\354\301T\363\252\322\030\031\363o\225\343\re\246:Z\235j(\244\241\347\230\216\374\\\031m\214\\n\355`\202g\375Pk\357\211<\343e\3652\2349h\351/\375\323\237\366\013|\204`WMr\010h`\262\266\035\352\227g\367\356\010\365\232(\017W\026\260=F\242\221\313\374\243(\021z\\n\375\260\273K\243_\236k\261:U\177\362\362\3405l\2771p\r\330t\275\242!\032\257V3\007\243\215\201a\031\nF(\006]\276\352\355 \004n\002\023DK\023\017Q\373\302n\302\217\361\312Xe\2716y\340\326\023\353\264=\370\302p}\210B\373\0009 9\307\331s\r\200Z\204[\313[@\253\356\213\365\2137D0\201\2476z\3114]6\302\376\201f7\272\3021\273\t\262\205\327\003\204\021\335P\276W\352(\ryS\336\217\200(\300\303\371}\023\264\240\203B\230&\001\342B\351\216\037\251\217\301\245\016\372\353\257V\353\253k\002\265\361\t\346\355\337,w\000\263Q\177\352}\005L\352\036\306dq~\t\020\242Xqd}\331sT\304\247\303=\373\020j\002<'\310\230\002\023\270\324b\002X)D\0000\235,]\367\343D3\316\354\177\213P\335yj\177\270\024?\036\233\232]\003\177!2\341L\247i;\353\336\200?\000k\275Qv*7\253_U\357V5\2124'i\273M\021\\\021P\010\336Y$\275\365^\250\001\352(\341T,:\177\224x\323`}\354^\005A8R\217\\\257_\307\216h\325x(\376\221\240o\007Z\271_\032\244 }\244m\030q\374'6-\275\0229\364\r\013\223\377\374X\331\256v\325N\001\203\272N\021\036\357\224~F\217=&\210NeT\032\207O\356x\266g\277PZ\022\204\002\326\003\354\256_\244X7\214\340\360\023\354:]#[\252_\"\213\356\031\006\235\204{\370\177\007\005\335\246Y,\204\235\336'\325xu\365\310T}\245\237\210[x\377\017cj\211\344\265w\335\333\360\227\210\230\320\300qR\313\021\341p\233\027\027\206|\312\273H\226\027)\r\220\265\211\276\017Kw\2616x4\342\032\277\376\205\251\215\020\355\003\344\325o\001\302\230S\302\216\032$\315Q\332\367\275\372=Bx\034h\023\207t\375\007\300\210\031\212WPl\351*\2218\261\375\360\na\267\343e\232\235(\377\316\024\363\233C\3246\2742zJJ\313\006\316\274\026\372\341\335\200\324\303\340a\252""\215\263\330\225\234\232\372v\217\325\307\346\341\362=\030M\033\031\366\3560\3419\276_!\307\213 \253\322b\211\262\367\226^B\007\223e6\251G\245y\3574\266\373TP\224~\342\005\375ab\326\304\221%r\320\320x\275\357\206\277\316\216\262]\351\254\374\010\364yt\000.L,\252\373*\326\236\002\313\214\224\257\003\255\230\263\321\376N\223l\210&\377\3029\010\234\346\001\023\237nJP\240\255\r\342E\241\002`\204\335F\016\253\337x\000\203\357 \016\363?\376v}\374\007\2609\252\325/\307\030n\273\341\357\024h\276\255\220\217\327G(t\374\243\312\023\276\023\351\321\331\206\244\014\330\357G\"\"k\215\236[\365[\000\317F/\311\375\0024\330\350A\334~\214p\026j\271Y\277IZ\n\265\334\250\337x\204x\323ji\002M2\024\372\032\335\212\337M1\214!\242~f\230\tU`\221\200\234\243\026\311\366\322\362\212\t\262\010xZ\223R\271k`PpG\312\334N\266\034?lS\315/&UGm,0/\312\036\n\373\363\245\216C\271\314o\310q\016g7\037J\263\344\310\255\226c\246\370\347\014\255\33164q\314\373\245~\357ko\035\231\254#a\330\373\265,Ep\017H\273-X\352V9\322<{\201\263\034\006\213o\313\234n\014\224>\326oBBd\243\037}\266\253\013\3220?V\264\177\327\037\377\345\217\242\3221\207e_E@\373\264\353\222\224\027A\363\010N\3449\316\003:\027\311\342C\005\301\242Q`\002\276\035\n28K\317~\236\361\355\246\377\215\237\024\223\272@\313\033HG\357\373\375\345\303*,\354/\003)\377\267|\261\3747p\356Bu\256\352\324F\341\226\263\007\201\032\2419\n\234]\377\031\375\263T)\203\213\376e0\232\246\014W\304\254\204\327\301\251\317\004w\007\026\234\205\325d+SM\024\317\000F^\"l\303\210\316\355\247I\235\255\310+\262\331I\262!A\331\037S\006\334J\222\006\3778w\"2\302)'\022\314\217\222\021\2671\214\2024\247\317\017I\374MX\334:l\014\371\271\277Q\236\007\240\315Vt\3121\210{]D\226\277\200\265\306\274\027\210\230\227\312.2\224b\355\374A\241\276\316\211\014\354\354M\020\226\246\367\177\303D}H|\372\220\202\335@&\303<(J\274\240\265\365\036\304\256H\243g\310\273\341\355r\342\325C\324\006a\367\tB72\240f\344\0020\007_~\255\304\010x\003\341\210\270\307\016\002)\325o\020\255@r\333\372]\271\215""\255\237\343X>8\324\270v\2356\337\003\263\302\314\323\274R\036.\205\212$\004;\245\267\336\202\337\353\277)\337-k\2547\344\215Z\020\341\311:{\244\3375#|3\005\330\203|\266*D\177\316^&\\>\277\377\201\356\t8\247\021\253\375\346\255Qzs\325\373X\217\2219\000!?b\322\323La\371\242\244\321\323\013qR\354\016\027\226\305\335\010A\005RHQ\270\000g%\300\246\234c\2300\221\370\314d\371\237\000\347\317\234\371\312\325\306\345+\304\267qx\330`\337\277\033\246(;\373\327Q\t(\365'P\276\225'\213\224\007\354\353\257\341>\233\302a\000c\212\022\374\344\377\t\266\270\260\302\267\311+\377\007\006\212zy";
    PyObject *data = __Pyx_DecompressString(cstring, 6484, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (13852 bytes) */
const char* const bytes = ": Applying skip_features=Arrow array stream was already released.Buffer view does not expose stridesCRS could not be extracted as WKTCan only create a buffer that is contiguous in memory.Cannot assign to read-only memoryviewCannot create writable memory view from read-only memoryviewCannot index with type 'Cannot transpose memoryview with indirect dimensionsCould not add feature to layer at index Could not create feature at index Could not create geometry at index Could not create geometry from WKB at index Could not create mask geometryCould not detect driver of datasetCould not encode value 'Could not extract valid Arrow array stream.Could not find field index for field 'Could not get Arrow schema from stream.Could not get field definition for field at index Could not get layer definitionCould not iterate over features: Could not obtain driver: Could not read feature with fid Could not set CRS: {}Could not set geometry for feature at index Could not set metadataCreated Dimension %d is not directDriver EPSG:ESRI ShapefileEmpty shape tuple for cython.arrayError adding field 'Error executing sql 'Error while accessing batch from stream.Error while creating field from Arrow for field Error while writing batch to OGR layerFailed to commit transactionFailed to create dataset with driver: Failed to open ArrowArrayStream from LayerFailed to open dataset (mode=Failed to start transactionFailed to write features to dataset GDAL returned more records than expected based on the count of records that may meet your combination of filters against this dataset.  Please open an issue on Github (https://github.com/geopandas/pyogrio/issues) to report encountering this error. IN (ISO-8859-1Indirect dimensions not supportedInvalid SQL query: Invalid SQL queryInvalid SQL query for layer 'Invalid bbox: Invalid mode, expected 'c' or 'fortran', got Invalid shape in axis ; It might help to specify the correct driver explicitly by prefixing the file path with '<DRIVER>:', e.g. 'CSV:path'.La""yer '<MemoryView of More than one layer found in 'NoneNote that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.OGR field type is not supported for writing: Out of bounds on buffer access (axis Reading more features than indicated by OGR_L_GetFeatureCount is not supportedReceived invalid Arrow schema (null child)SET interest_layers = Skipping field . Specify layer parameter to avoid this warning.UTF-8Unable to convert item to objectWriting a geometry column of type You must provide at least a geometry column or a field-,,  ;  (?.>')): =': ' a supported file format.add_note and ' and type  at 0xat least one of read_geometry or return_fids must be True or columns must be None or non-emptycannot provide both encoding parameter and \"ENCODING\" option; use encoding parameter to specify correct encoding for data sourcecannot provide both encoding parameter and \"ENCODING\" layer creation option; use the encoding parametercannot set both 'bbox' and 'mask'cannot set both 'fids' and any of 'where', 'bbox', 'mask', 'sql', 'skip_features' or 'max_features'cannot set both 'fids' and any of 'where', 'bbox', 'mask', 'sql', 'skip_features', or 'max_features' (check that it was installed correctly into GDAL)collections.abc<contiguous and direct><contiguous and indirect>' could not be openeddatetime64[D]datetime64[ms]' (default), disable does not support write functionalityenableerror applying filter for failed to prepare SQL fids; max. number for drivers with default SQL dialect 'OGRSQL' is 4997field_data and field_mask must be same lengthfield_data array needs to be same length as fields arrayfield_data arrays must be same lengthfield_data arrays must be same length as geometry array (field index: field_mask arrays must be same length as geometry arrayfield type is not supported  for WKB type forcing 2D is not supported for Arrowgc.gdb (got got differing extent""s in dimension ' in field ' is not yet supported. Only WKB is currently supported ('geoarrow.wkb' or 'ogc.wkb' types).isenableditemsize <= 0 for cython.array'layer' parameter must be a str or int, got list(float64)list(int32)list(int64)list(str)'max_features' must be >= 0no default __reduce__ due to non-trivial __cinit__non-UTF-8 encoding is not supported for Arrow; use the non-Arrow interface instead object>pyogrio/_io.pyx raised an error recordsrequested_schema is not supported'skip_features' must be >= 0specifying 'max_features' is not supported for Arrow'sql' parameter cannot be combined with 'layer'<strided and direct><strided and direct or indirect><strided and indirect>' to layer' to stringunable to allocate array data.unable to allocate shape and strides.: unsupported OGR type: utf-8 with name 'ASCII_ArrowStream_ArrowStream.__arrow_c_stream___ArrowStream.__init__CPLE_BaseErrorCPLE_NotSupportedErrorCRSErrorDTYPE_OGR_FIELD_TYPESDataLayerErrorDataSourceErrorENCODINGEPSGEllipsisFIDFIELD_SUBTYPESFIELD_SUBTYPE_NAMESFIELD_TYPESFIELD_TYPE_NAMESFeatureErrorFieldErrorFlatGeobufGEOMETRY_NAMEGeoJSONGeoJSONSeqGeometryErrorMULTI_EXTENSIONSNaTNullPointerErrorOODSOFSTBooleanOFSTFloat32OFSTInt16OFSTJSONOFSTMaxSubTypeOFSTNoneOFSTUUIDOFTBinaryOFTDateOFTDateTimeOFTIntegerOFTInteger64OFTInteger64ListOFTIntegerListOFTRealOFTRealListOFTStringOFTStringListOFTTimeOFTWideStringOFTWideStringListOGC_FIDOGR_GEOMETRYOSMOpenFileGDB__Pyx_PyDict_NextRefRecordBatchStreamReaderSHAPE_ENCODINGSQLiteSequenceUTF8UnknownView.MemoryViewXLSXabcadd_data_chunkallocate_buffer__annotate__appendarray__arrow_c_stream__arrow_objasarrayastypeasyncio.coroutinesbbasebasenamebatch_sizebboxboolboundsbyteorderbytes_ccapabilitiescapsule_capsulecapture_errors__class____class_getitem__cline_in_tracebackclosecolumnsconcatenatecontextlibcontextmanagercountcrsdataset_kwargsdataset_metadatadataset_optionsdatedatetimedatetime64datetime_as_stringdatetime_valuedaydecode__dict____doc__driverdtypedtype_is_objectdtypesemptye""ncodeencoding__enter__enumerateerrerrorexexcexists__exit__fast_feature_countfast_set_next_by_indexfast_spatial_filterfast_total_boundsfeature_countfeaturesfidfid_columnfid_column_wherefid_datafidsfids_strfieldfield_bfield_cfield_datafield_data_chunkfield_idxfield_indexfield_indexesfield_maskfield_ogr_typefield_ogr_typesfield_subtypefield_typefield_typesfield_valuefieldsfields_cflagsfloatfloat32float64force_2dforce_feature_countforce_total_boundsformatfortranfrom_bytes__func__gdal_msggdal_tzgdal_tz_offsetsgenexprgeometriesgeometrygeometry_namegeometry_typegetgetLoggerget_features.<locals>.add_data_chunkget_metadata.<locals>.genexprgetpreferredencoding__getstate__houriididxignored_fields__import___import_from_cindexinfo__init__intint16int32int64int8intcintersect1d_is_coroutineisdirisnanisnatisoformatitemitemsitemsizekindlayerlayer_createdlayer_kwargslayer_metadatalayer_namelayerslistlocaleloglogging__main__maskmax_featuresmemviewmeta__metaclass__microsecondminutemode__module__monthname__name__name_bnannan_as_nullndarrayndim__new__nextnpnum_featuresnum_field_datanum_fieldsnum_recordsnumpyobjobjectogr_datasetogr_featureogr_featuredefogr_fielddefogr_geometryogr_layerogr_list_layersogr_open_arrowogr_readogr_read_boundsogr_read_infoogr_subtypesogr_typesogr_writeogr_write_arrowoptionsoptsosoverride_shape_encodingpapackpathpath_or_bufferpath_or_fppop_precision__prepare__prev_shape_encodingpromote_to_multipyarrowpyogrio._errpyogrio._iopyogrio._ogrpyogrio.errors__pyx_checksum__pyx_state__pyx_type__pyx_unpickle_Enum__pyx_vtable____qualname__random_readread_geometryreader__reduce____reduce_cython____reduce_ex__registerreplacerequested_schemareturn_fidsreturn_indicesrevalrmtreeroundschemasecondselfsend__set_name__setdefault__setstate____setstate_cython__shapeshutilsignedsizeskip_featuressortsplitsplitextsqlsql_dialectstacklevelstartstartswithstepstopstr_streamstream_capsulestructsupports_transactionssys__test__throwtimetotal_boundstypetz_arrayuuintuint16uint32uint64uint8unlinkunp""ackupdateupperuse_pyarrowuse_tmp_vsimemvaluevalue_bvalueswarnwarningwarningswherewidthwkbwkb_bufferwkbtypexyearzerosARROW:extension:metadataARROW:extension:nameOGRwkbGeometryType (PyObject *)\000PyObject *(void *)\000get_geometry_type_code\000get_geometry_typePyObject *(PyObject *)\000PyObject *(PyObject *, PyObject *)\000\000PyObject *(PyObject *, int __pyx_skip_dispatch)\000read_buffer_to_vsimem\000get_ogr_vsimem_write_path\000read_vsimem_to_buffer\000vsimem_rmtree_toplevelPyObject *(char const *, struct __pyx_opt_args_7pyogrio_4_ogr_get_string *__pyx_optional_args)\000get_stringPyObject *(void)\000int (int)\000void *(void *)\000check_last_error\000check_int\000check_pointergeoarrow.geoarrow.wkb\200A\340\010\013\2101\330\014\026\220g\230Q\230b\240\006\240a\240w\250i\260v\270R\270q\330\010\013\2101\330\014\027\220w\230a\230r\240\026\240q\250\007\250y\270\006\270a\340\010-\250Q\330\010\014\210O\2305\240\001\240\021\330\014\017\320\017\"\240$\240f\250A\250]\270\"\270K\300q\310\001\330\020\030\230\001\330\021\027\220q\230\r\240R\240{\260!\2601\330\020\030\230\001\340\020\030\230\006\230a\230}\250A\340\014\034\230G\2401\240B\240f\250A\250W\260I\270V\3001\340\010\031\230\027\240\001\240\021\200A\330\010\014\210L\230\001\320\004!\240\021\330\010\013\320\013\034\230G\2401\330\014\022\320\022%\240Q\240a\330\010\017\210t\2201\250Q\200\001\360\010\000\005\006\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\200\001\360\032\000\005\006\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\360\006\000\005'\240a\330\004\037\230q\330\004#\2401\330\004%\240Q\330\004*\250!\330\004&\240a\330\004+\2501\330\004 \240\001\330\004%\240Q\330\004\023\2201\330\004\021\220\021\330\004\034\230A\330\004\036\230c\240\021\240/\260\033\270G\300:\310Q\330\004\032\230#\230Q\230k\250\027\260\007\260z\300\021\330\004\036\230a\330\004 \240\001\360\006""\000\005 \230q\340\004\007\200{\220#\220Q\330\010\016\210j\230\001\230\021\340\004\007\200{\220#\220R\220t\2309\240C\240q\330\010\016\210j\230\001\230\021\340\004\007\200{\220\"\220A\330\010\026\220c\230\021\230*\240A\240Q\330\010\014\210E\220\025\220a\220s\230#\230Q\230a\330\014\017\210s\220!\220:\230Q\230d\240#\240Q\330\020\026\220j\240\001\240\021\340\004\007\200y\220\003\2201\360\006\000\t\031\230\001\340\010\013\210;\220b\230\001\330\014\017\210s\220!\220:\230S\240\001\330\020\026\220j\240\001\330\024\025\360\006\000\r\033\230#\230Q\230a\340\004\007\200{\220'\230\021\330\010\013\2103\210a\210|\2303\230c\240\021\240!\330\014\022\220*\230A\230Q\330\010\014\210E\220\025\220a\220s\230#\230Q\230a\330\014\017\210z\230\021\230#\230W\240E\250\024\250S\260\001\260\032\2701\270D\300\003\3001\330\020\026\220j\240\001\330\024\025\360\006\000\t\026\220Q\220f\230B\230a\340\004\007\320\007\027\220s\230!\330\010\032\230!\340\004\005\340\010\016\320\016\037\320\0378\270\001\270\034\300Q\360\006\000\t\031\320\0300\260\001\330\014\022\320\022\"\240'\250\030\260\025\260o\300Q\330\014\034\230N\250!\330\014\036\230a\330\014\r\210]\230!\2301\360\006\000\t\014\2107\220#\220Q\360\n\000\r\030\220q\360\014\000\r\030\220y\240\003\240?\260!\260=\300\001\360\006\000\t\027\220a\330\010\013\210;\220b\230\001\330\014\032\320\032+\2501\250A\250U\260'\270\024\270Y\300a\340\010\013\2101\330\014\020\220\005\220U\230!\2301\330\020 \240\017\250w\260m\300;\310a\310q\340\020\031\230\026\230q\240\002\240'\250\021\250!\330\020\021\330\024#\240=\260\001\260\036\270q\300\010\310\001\360\006\000\025\030\220~\240S\250\001\330\030*\250!\250>\270\021\340\024\027\220q\330\030(\250\001\250\036\260q\360\010\000\025\036\230Q\320\036/\250q\260\013\270>\310\021\340\027\030\330\024\032\230*\240A\330\030.\250a\250v\260Q\260a\330\033\034\360\006\000\025\030\220}\240C\240q\330\030'\240q\250\001\330\030'\240q\360\010\000\t\032\320\031+\2501\250A\340\010\013\320\013\037\230q\320 0\260\003\2601\330\014\034\230D\240\001""\240\025\240a\240q\340\014\020\220\005\220U\230!\2301\330\020\030\320\030,\250A\320-=\270V\3001\300B\300g\310Q\310a\330\020\023\2206\230\022\2301\330\024\032\230*\240A\330\030@\300\001\300\026\300q\310\001\340\020\035\230W\240A\240Q\340\010\013\210<\220w\230a\330\014\036\230d\240!\240;\250d\260!\360\006\000\t!\320 4\260A\260[\300\001\330\010\013\2101\330\014\035\230Q\230m\2501\340\010\014\210E\220\025\220a\220q\340\014\032\230,\240a\240q\330\014\017\210|\2303\230a\330\020\026\220l\240!\320#G\300q\310\n\320RS\360\n\000\r\023\220(\230)\2403\240j\260\010\270\001\270\021\330\014\017\210t\2207\230!\330\020\035\230Q\330\020\023\220:\230Q\230c\240\023\240A\340\024\025\330\030\"\240!\2403\240c\250\032\2601\260C\260s\270#\270S\300\n\310!\3103\310c\320QU\320UV\330\031'\240z\260\021\260#\260S\270\001\360\010\000\025\026\330\031(\250\n\260!\2604\260s\270$\270c\300\032\3101\310C\310s\320RV\320VW\330\031#\2401\240C\240s\250#\250R\250z\270\021\270!\340\020\037\320\0373\2601\3204H\310\001\330\020\023\220=\240\003\2401\330\024\032\230-\240q\330\030=\270Q\320>N\310a\310q\330\033\034\360\006\000\021\027\320\026)\250\021\250.\270\014\300C\300q\310\001\330\020\023\2201\330\024\032\230-\240q\330\030F\300a\300q\330\033\034\360\006\000\021\024\2201\330\024\027\220x\230t\240:\250]\270+\300Q\330\030'\320'>\270a\270q\330\031!\240\021\330\030'\320'9\3209I\310\021\340\030'\320'C\3001\300A\330\031!\240\021\330\030$\240O\260=\300\001\340\030'\320'@\300\001\300\021\360\010\000\021\027\320\026/\250q\260\r\270Q\330\020\037\230q\330\020\023\2201\330\024\032\230-\240q\330\030F\300a\300q\330\033\034\360\006\000\r\021\220\r\230U\240!\2401\330\020\036\230j\250\001\250\032\2601\260A\330\020\035\230_\250A\250Q\330\020\036\230m\2501\250A\340\020\027\220z\240\021\240!\330\020\023\2205\230\007\230u\240D\250\004\250A\250Q\330\024&\240a\240}\260A\340\025 \240\003\2401\330\024\025\330\030$\240C\240q\330\030\034\230J\240a\240}\260G\2704\270u\300A\300Q\340\030*\250!\250=\270\001\360\006\000\031\034\2304\230z\250""\021\250-\260q\330\034*\250#\250Q\250a\340\030\031\330\034&\240k\260\027\270\001\270\021\330\0340\260\001\260\035\270m\3101\340\037 \330\034\"\240*\250A\330 :\270!\2701\330 #\2401\240F\250!\2501\360\006\000 !\330\034\035\340\025 \240\003\2401\330\024)\250\021\250-\260}\300A\340\025 \240\003\2401\330\024+\2501\250M\270\035\300a\340\025 \240\003\2401\330\024\027\220|\2404\240u\250A\250Q\330\030*\250!\250=\270\001\340\030,\250A\250]\270-\300q\340\025 \240\003\2401\330\024\027\220r\230\026\230q\240\001\330\030*\250!\250=\270\001\340\030)\250\033\260E\270\021\330\0300\260\001\330\034\035\330\034\035\330\034*\250!\330\034*\250!\330\034*\250!\330\034\035\330\034\035\330\034\035\330\034\035\360\006\000\026!\240\003\2401\330\024\027\220r\230\026\230q\240\001\330\030*\250!\250=\270\001\340\030)\250\033\260G\2701\320<M\310U\320RS\330\030#\240?\260$\260a\260v\270Q\270l\310!\330\030\033\2309\240C\240q\330\034&\240a\330\035\037\230v\240Q\240h\250a\250q\330\034&\240a\340\034&\240h\250a\250q\330\0300\260\001\330\034\035\330\034\035\330\034*\250!\330\034*\250!\330\034*\250!\330\034*\250!\330\034*\250!\330\034*\250(\260\"\260N\300-\310t\320ST\330\034\035\360\010\000\025\033\320\032-\250Q\330\030G\300q\310\001\360\010\000\r\016\330\020\031\230\021\320\032-\250Q\250k\270\021\340\014\023\320\023%\240Q\330\020\026\220l\240!\330\024>\270a\270t\3001\300A\330\027\030\340\014\031\230\021\230!\330\014\032\230!\340\010\013\2101\330\014\036\230a\230q\340\010\013\2105\220\001\220\032\2301\230K\240q\360\006\000\t\017\210i\220q\230\001\330\010\026\220a\330\010\013\2101\330\014\022\220/\240\021\320\"H\310\001\310\027\320PQ\320QR\360\006\000\t\014\2101\330\014!\240\021\240&\250\001\360\014\000\t\014\210=\230\003\2301\330\014\033\2301\230A\330\014\033\2301\340\010\013\210<\220s\230!\330\014\031\230\021\230!\330\014\032\230!\340\010\013\210=\230\003\2301\330\014!\240\021\240!\330\014\033\2301\340\010\013\210<\220s\230!\330\014\025\220Q\220a\340\010\013\2101\330\014\"\240!\2401\200\001\360\026\000\005""\006\330\004\005\330\004\005\360\014\000\005'\240a\330\004\037\230q\330\004\032\230!\330\004\037\230q\330\004$\240A\360\010\000\005\013\210+\220Q\330\004\t\210\033\220A\340\004\005\340\010\016\320\016\037\320\0378\270\001\270\034\300Q\340\010\030\320\0300\260\001\330\014\022\320\022\"\240'\250\030\260\025\260o\300Q\330\014\034\230N\250!\330\014\036\230a\330\014\r\210]\230!\2301\360\014\000\t\n\330\014\025\220T\230\030\240\030\250\021\250%\250s\260&\270\003\2703\270a\330\014\020\220\007\220s\230!\340\014\022\220*\230A\330\020\021\360\010\000\t\014\2101\330\014\024\320\024%\240Q\340\014\023\2201\340\010\022\220/\240\021\240!\340\010\031\230\031\320\"5\260Q\330\010\021\320\021$\320$8\270\001\330\014\034\230A\360\006\000\t\014\2107\220#\220Q\330\014\022\220,\230a\230q\340\010\013\2106\220\031\230#\230Q\330\014\022\220,\230a\230q\340\010\013\2106\220\033\230A\230X\240Q\240h\250c\260\021\330\014\022\220,\230a\230q\340\010\013\2101\330\014+\2501\250K\260q\270\010\300\t\310\021\340\010\t\330\014\017\210v\220Y\230a\230x\240q\250\007\250s\260!\330\020\026\220l\240!\2401\360\006\000\r\020\210u\220I\230S\240\001\330\020\021\340\014\017\210t\320\023(\250\001\250\033\260A\260X\270Q\270g\300Q\330\020\026\320\026&\240a\330\020\033\2305\240\003\2401\240J\250i\260q\330\020\026\220n\240A\330\024<\270A\270Q\360\006\000\r\020\210u\220I\230S\240\001\330\020\025\220X\230Q\230a\230q\360\006\000\t\017\210i\220q\230\001\330\010\026\220a\330\010\013\2101\330\014\022\220/\240\021\320\"H\310\001\310\027\320PQ\320QR\360\006\000\t\014\2101\330\014!\240\021\240&\250\001\360\006\000\t\014\2107\220#\220U\230$\230f\240I\250S\260\001\330\014\022\220(\230!\2301\340\010\013\2106\220\031\230#\230Q\330\014\022\220(\230!\2301\230A\340\010\013\2105\220\t\230\023\230A\330\014\021\220\030\230\021\230!\2301\340\010\013\2108\2203\220a\330\014\026\220a\220q\330\014\026\220a\340\010\013\210<\220s\230!\330\014\025\220Q\220a\340\010\013\2101\330\014\"\240!\2401\200\001\360\034\000\005 \230z\250\021\320*:\270!\330""\004&\240a\340\004\005\330\010\t\330\014!\240\021\320\"5\3205I\310\021\340\010\026\220h\230a\230t\2407\250!\250:\260S\270\001\330\010\021\220\037\240\001\240\021\360\006\000\t\014\210<\220s\230!\330\014\025\220Q\220a\330\014\032\230!\340\010\013\2101\330\014\"\240!\2401\340\004\013\2101\200\001\360\006\000\005\006\330\004\005\330\004\005\330\004\005\360\006\000\005 \230z\250\021\320*:\270!\330\004\"\240!\330\004&\240a\330\004\037\230q\330\004+\2501\330\004(\250\001\340\004\005\330\010\017\320\017$\240A\330\014\r\330\r!\240\021\340\010\013\2101\330\014&\240a\330\014\"\320\"D\300A\330\020\"\240!\360\006\000\t\033\230/\250\021\250!\330\010\026\220h\230a\230t\2407\250!\250:\260S\270\001\340\010\013\2106\220\023\220A\330\014\024\320\024%\240Q\240a\330\010\024\220M\240\021\240-\250q\340\010\013\2109\220D\230\n\240!\240=\260\003\2601\330\014\027\220q\340\014\027\220y\240\003\240?\260!\260=\300\001\340\010\021\220\032\2301\230K\240q\330\010\024\220A\320\025%\240T\250\021\250%\250q\260\004\260K\270t\3009\310A\330\010\027\220q\330\014\037\230t\2401\240E\250\021\250$\250k\270\024\270Y\300a\360\006\000\t\n\330\014\032\230*\240A\240]\260!\2601\330\014\023\2207\230!\2301\330\014\030\230\001\330\014\026\220f\230D\240\001\330\014\026\220f\230D\240\001\330\014\031\230\021\330\014\034\230A\330\014\032\230*\240A\320%7\260q\270\001\330\014\035\230Z\240q\320(?\270q\300\001\330\014\035\320\035.\250a\250q\330\014\030\320\030)\250\021\250+\260Q\330\014\034\320\034,\250A\250[\270\001\330\014\026\220j\240\001\240\021\330\014\r\330\020\037\320\0373\2601\260K\270\177\310c\320QR\330\020*\320*>\270a\330\024\037\230q\330\022\025\220Q\330\020'\320';\2701\330\024\037\230q\330\022\025\220Q\330\020&\320&:\270!\330\024\037\230q\330\022\025\220Q\330\020%\320%9\270\021\330\024\037\230q\330\022\025\220Q\340\014\036\230l\250!\2501\330\014 \240\014\250A\250Q\360\010\000\t\014\320\013\033\2303\230a\330\014\026\220a\220q\330\014\036\230a\340\010\013\210<\220s\230!\330\014\025\220Q\220a\330\014\032\230!\360""\006\000\t\014\2101\330\014)\250\021\320*<\270A\340\014\017\320\017#\2403\240a\330\020\027\220q\230\007\230q\340\010\013\2101\330\014\"\240!\2401\340\004\013\2101\200\001\340\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\360\006\000\005 \230z\250\021\320*:\270!\330\004&\240a\330\004\037\230q\340\004\007\200u\210D\220\001\330\010\016\210j\230\001\230\021\340\004\007\200~\220R\220q\330\010\016\210j\230\001\230\021\340\004\007\200}\220B\220a\330\010\016\210j\230\001\230\021\340\004\005\330\010\017\320\017$\240A\330\014\r\330\r!\240\021\330\010\026\220h\230a\230t\2407\250!\250:\260S\270\001\340\010\013\2106\220\023\220A\330\014\024\320\024%\240Q\240a\340\010\024\220M\240\021\240-\250q\360\006\000\t\014\2106\220\027\230\005\230T\240\026\240s\250!\330\014\036\230a\230{\250!\360\006\000\t\014\2105\220\007\220q\330\014\035\230Q\230k\250\021\340\r\022\220'\230\021\330\014!\240\021\240+\250Q\360\006\000\t\030\220\177\320&<\270A\330\014\027\220\177\240a\360\006\000\t\022\220\032\2301\230K\240\177\260a\360\006\000\t\014\210<\220s\230!\330\014\025\220Q\220a\330\014\032\230!\340\010\013\2101\330\014\"\240!\2401\340\004\013\2101\200\001\360\006\000\005\006\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\330\004\005\360\026\000\005 \230z\250\021\320*:\270!\330\004\"\240!\330\004\037\230q\330\004\033\2301\330\004&\240a\330\004\037\230q\330\004+\2501\330\004(\250\001\340\004\007\200u\210G\2201\330\010\t\330\014\022\220'\230\021\330\014\017\210u\220G\2301\330\014\017\210u\220G\2301\330\014\017\210t\2207\230!\330\014\017\210q\330\014\017\210q\340\014\022\220*\230A\330\020\021\360\006\000\t\020\210r\220\030\230\021\230&\240\006\240b\250\001\340\004\007\200t\2107\220%\220t\2306\240\027\250\001\330\010\016\210j\230\001\230\021\340\004\007\200u\210N\230#\230\\\250\023\250H\260C\260u\270C\270s\300!\3009\310B\310a""\330\010\016\210j\230\001\330\014\r\360\010\000\005\010\200u\210D\220\001\330\010\016\210j\230\001\230\021\340\004\007\200~\220R\220q\330\010\016\210j\230\001\230\021\340\004\007\200}\220B\220a\330\010\016\210j\230\001\230\021\340\004\005\330\010\017\320\017$\240A\330\014\r\330\r!\240\021\340\010\013\2101\360\014\000\r'\240a\330\014\"\320\"D\300A\330\020\"\240!\360\006\000\t\033\230/\250\021\250!\330\010\026\220h\230a\230t\2407\250!\250:\260S\270\001\340\010\013\2104\210s\220!\330\014\017\210v\220S\230\001\330\020\030\320\030)\250\021\250!\330\014\030\230\r\240Q\240m\2601\340\014\030\230\013\2401\240M\260\025\260a\340\010\016\210g\220Q\220a\360\010\000\t\014\2101\330\014\017\210z\230\021\230-\240s\250!\360\006\000\021\024\220;\230c\240\021\330\024\032\230*\240A\330\030\031\360\016\000\021\034\2301\360\006\000\r\030\220\177\240a\240}\260A\340\010\021\220\032\2301\230K\240q\340\010\031\230\021\330\010\013\2108\2207\230!\340\014\035\230T\240\021\240#\240Q\240f\250D\260\004\260B\260c\270\021\270!\360\010\000\r\023\220\"\220E\230\021\230\"\230L\250\001\250\026\250t\2604\260y\300\017\310u\320TU\320UV\330\014\025\220V\2301\230A\340\010\013\2104\210~\230T\240\025\240c\250\025\250d\260%\260s\270!\330\014\032\230'\240\021\240!\360\010\000\t\014\2101\330\014\020\220\t\230\021\330\020\032\230%\230w\240a\240q\330\020\032\230!\330\020\033\230<\240q\250\n\260!\340\021\022\330\020&\240a\240{\260.\300\001\340\010\030\320\030)\250\021\250!\340\010\013\2105\220\007\220q\330\014\030\230\r\320%8\270\001\330\020\021\330\020\021\330\020\021\330\020\021\330\036,\250D\260\016\270g\300Q\330\031\032\330#$\360\010\000\r\020\210q\330\020\033\2304\230w\240a\240r\250\021\340\020\033\2301\360\006\000\r\020\210v\220W\230E\240\024\240V\2503\250a\330\020\"\240!\240;\250a\360\006\000\r\020\210u\220G\2301\330\020!\240\021\240+\250Q\340\021\026\220g\230Q\330\020%\240Q\240k\260\021\360\010\000\r\035\230A\330\014\017\210}\230B\230a\330\020\037\230q\360\010\000\r\020\210~\230R\230q\330\020\037\230\177\320.""D\300A\330\024\037\230\177\250a\360\n\000\r\016\330\020\026\220c\230\021\330\020\024\220E\230\023\230A\330\020\024\220E\230\023\230A\330\020\024\220N\240#\240Q\330\020\024\220M\240\023\240A\330\020\024\220M\240\022\2401\340\020\037\320\0370\260\001\260\033\270A\340\014\026\220l\240-\250|\2701\330\020\021\330\020\021\330\020\021\330\036,\250D\260\016\270g\300Q\330\031\032\330\036\037\330\035\036\330\034\035\330#$\360\006\000\t\025\220A\320\025%\240T\250\021\250%\250q\260\004\260K\270t\3009\310A\330\010\027\220q\330\014\037\230t\2401\240E\250\021\250$\250k\270\024\270Y\300a\360\006\000\t\n\330\014\023\2201\330\014\030\230\001\330\014\026\220f\230D\240\001\330\014\026\220f\230D\240\001\330\014\031\230\021\330\014\034\230A\330\014\035\230Q\360\010\000\t\014\320\013\033\2303\230a\330\014\026\220a\220q\330\014\036\230a\340\010\013\210<\220s\230!\330\014\017\210t\2207\230!\330\020+\2501\250M\270\021\340\014\025\220Q\220a\330\014\032\230!\360\006\000\t\014\2101\330\014)\250\021\320*<\270A\340\014\017\320\017#\2403\240a\330\020\027\220q\230\007\230q\330\020&\240a\340\010\013\2101\330\014\"\240!\2401\340\004\005\330\010\t\330\010\t\330\010\t\330\010\togc.wkbGEOARROWO";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 514; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 155) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 514; i < 536; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 536; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 514;
      for (Py_ssize_t i=0; i<22; ++i) {
        #if PY_VERSION_HEX >= 0x030F0000
        PyUnstable_SetImmortal(table[i]);
        #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
        #if PY_VERSION_HEX < 0x030E0000
        if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
        #else
        if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
        #endif
        {
          Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
        }
        #else
        if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 0;
    int8_t const cint_constants_1[] = {0,-1,1,2,3,4,5};
    int16_t const cint_constants_2[] = {500,1000};
    int32_t const cint_constants_4[] = {1000000L,136983863L};
    for (int i = 0; i < 11; i++) {
      numbertab[i] = PyLong_FromLong((i < 7 ? cint_constants_1[i - 0] : (i < 9 ? cint_constants_2[i - 7] : cint_constants_4[i - 9])));
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<11; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
typedef struct {
    unsigned int argcount : 5;
    unsigned int num_posonly_args : 1;
    unsigned int num_kwonly_args : 1;
    unsigned int nlocals : 6;
    unsigned int flags : 10;
    unsigned int first_line : 12;
} __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {0, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 657};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_i};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_genexpr, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1293};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_field_data_chunk, __pyx_mstate->__pyx_n_u_field_index_2, __pyx_mstate->__pyx_n_u_dtype};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_add_data_chunk, __pyx_mstate->__pyx_kp_b_iso88591_A_1_gQb_awivRq_1_war_q_y_a_Q_O5, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {19, 0, 0, 53, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 1908};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path_or_buffer, __pyx_mstate->__pyx_n_u_dataset_kwargs, __pyx_mstate->__pyx_n_u_layer, __pyx_mstate->__pyx_n_u_encoding, __pyx_mstate->__pyx_n_u_read_geometry, __pyx_mstate->__pyx_n_u_force_2d, __pyx_mstate->__pyx_n_u_columns, __pyx_mstate->__pyx_n_u_skip_features, __pyx_mstate->__pyx_n_u_max_features, __pyx_mstate->__pyx_n_u_where, __pyx_mstate->__pyx_n_u_bbox, __pyx_mstate->__pyx_n_u_mask, __pyx_mstate->__pyx_n_u_fids, __pyx_mstate->__pyx_n_u_sql, __pyx_mstate->__pyx_n_u_sql_dialect, __pyx_mstate->__pyx_n_u_return_fids, __pyx_mstate->__pyx_n_u_batch_size, __pyx_mstate->__pyx_n_u_use_pyarrow, __pyx_mstate->__pyx_n_u_datetime_as_string, __pyx_mstate->__pyx_n_u_use_tmp_vsimem, __pyx_mstate->__pyx_n_u_dataset_options, __pyx_mstate->__pyx_n_u_ogr_dataset, __pyx_mstate->__pyx_n_u_ogr_layer, __pyx_mstate->__pyx_n_u_fields_c, __pyx_mstate->__pyx_n_u_field_c, __pyx_mstate->__pyx_n_u_options, __pyx_mstate->__pyx_n_u_prev_shape_encoding, __pyx_mstate->__pyx_n_u_override_shape_encoding, __pyx_mstate->__pyx_n_u_stream, __pyx_mstate->__pyx_n_u_reader, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_crs, __pyx_mstate->__pyx_n_u_fields, __pyx_mstate->__pyx_n_u_ignored_fields, __pyx_mstate->__pyx_n_u_idx, __pyx_mstate->__pyx_n_u_geometry_type, __pyx_mstate->__pyx_n_u_geometry_name, __pyx_mstate->__pyx_n_u_fid_column, __pyx_mstate->__pyx_n_u_fid_column_where, __pyx_mstate->__pyx_n_u_fids_str, __pyx_mstate->__pyx_n_u_ex, __pyx_mstate->__pyx_n_u_feature_count, __pyx_mstate->__pyx_n_u_field, __pyx_mstate->__pyx_n_u_field_b, __pyx_mstate->__pyx_n_u_capsule, __pyx_mstate->__pyx_n_u_reval, __pyx_mstate->__pyx_n_u_pa, __pyx_mstate->__pyx_n_u_ogr_types, __pyx_mstate->__pyx_n_u_ogr_subtypes, __pyx_mstate->__pyx_n_u_meta, __pyx_mstate->__pyx_n_u_fid, __pyx_mstate->__pyx_n_u_field, __pyx_mstate->__pyx_n_u_field};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_ogr_open_arrow, __pyx_mstate->__pyx_kp_b_iso88591__17, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {17, 0, 0, 42, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1621};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path_or_buffer, __pyx_mstate->__pyx_n_u_dataset_kwargs, __pyx_mstate->__pyx_n_u_layer, __pyx_mstate->__pyx_n_u_encoding, __pyx_mstate->__pyx_n_u_read_geometry, __pyx_mstate->__pyx_n_u_force_2d, __pyx_mstate->__pyx_n_u_columns, __pyx_mstate->__pyx_n_u_skip_features, __pyx_mstate->__pyx_n_u_max_features, __pyx_mstate->__pyx_n_u_where, __pyx_mstate->__pyx_n_u_bbox, __pyx_mstate->__pyx_n_u_mask, __pyx_mstate->__pyx_n_u_fids, __pyx_mstate->__pyx_n_u_sql, __pyx_mstate->__pyx_n_u_sql_dialect, __pyx_mstate->__pyx_n_u_return_fids, __pyx_mstate->__pyx_n_u_datetime_as_string, __pyx_mstate->__pyx_n_u_use_tmp_vsimem, __pyx_mstate->__pyx_n_u_dataset_options, __pyx_mstate->__pyx_n_u_field_c, __pyx_mstate->__pyx_n_u_fields_c, __pyx_mstate->__pyx_n_u_ogr_dataset, __pyx_mstate->__pyx_n_u_ogr_layer, __pyx_mstate->__pyx_n_u_prev_shape_encoding, __pyx_mstate->__pyx_n_u_override_shape_encoding, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_crs, __pyx_mstate->__pyx_n_u_fields, __pyx_mstate->__pyx_n_u_ignored_fields, __pyx_mstate->__pyx_n_u_idx, __pyx_mstate->__pyx_n_u_field, __pyx_mstate->__pyx_n_u_field_b, __pyx_mstate->__pyx_n_u_geometry_type, __pyx_mstate->__pyx_n_u_geometries, __pyx_mstate->__pyx_n_u_field_data, __pyx_mstate->__pyx_n_u_fid_data, __pyx_mstate->__pyx_n_u_num_features, __pyx_mstate->__pyx_n_u_ogr_types, __pyx_mstate->__pyx_n_u_ogr_subtypes, __pyx_mstate->__pyx_n_u_meta, __pyx_mstate->__pyx_n_u_field, __pyx_mstate->__pyx_n_u_field};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_ogr_read, __pyx_mstate->__pyx_kp_b_iso88591_z_q_1_a_q_1_uG1_uG1_uG1_t7_q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1899};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_capsule};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_init, __pyx_mstate->__pyx_kp_b_iso88591_A_L, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1902};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_requested_schema};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_arrow_c_stream, __pyx_mstate->__pyx_kp_b_iso88591_G1_Qa_t1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {11, 0, 0, 17, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2241};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path_or_buffer, __pyx_mstate->__pyx_n_u_layer, __pyx_mstate->__pyx_n_u_encoding, __pyx_mstate->__pyx_n_u_read_geometry, __pyx_mstate->__pyx_n_u_force_2d, __pyx_mstate->__pyx_n_u_columns, __pyx_mstate->__pyx_n_u_skip_features, __pyx_mstate->__pyx_n_u_max_features, __pyx_mstate->__pyx_n_u_where, __pyx_mstate->__pyx_n_u_bbox, __pyx_mstate->__pyx_n_u_mask, __pyx_mstate->__pyx_n_u_use_tmp_vsimem, __pyx_mstate->__pyx_n_u_ogr_dataset, __pyx_mstate->__pyx_n_u_ogr_layer, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_num_features, __pyx_mstate->__pyx_n_u_bounds};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_ogr_read_bounds, __pyx_mstate->__pyx_kp_b_iso88591_z_a_q_uD_j_Rq_j_Ba_j_A_hat7_S_6, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {6, 0, 0, 19, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2308};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path_or_buffer, __pyx_mstate->__pyx_n_u_dataset_kwargs, __pyx_mstate->__pyx_n_u_layer, __pyx_mstate->__pyx_n_u_encoding, __pyx_mstate->__pyx_n_u_force_feature_count, __pyx_mstate->__pyx_n_u_force_total_bounds, __pyx_mstate->__pyx_n_u_use_tmp_vsimem, __pyx_mstate->__pyx_n_u_dataset_options, __pyx_mstate->__pyx_n_u_ogr_dataset, __pyx_mstate->__pyx_n_u_ogr_layer, __pyx_mstate->__pyx_n_u_prev_shape_encoding, __pyx_mstate->__pyx_n_u_override_shape_encoding, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_fields, __pyx_mstate->__pyx_n_u_ogr_types, __pyx_mstate->__pyx_n_u_ogr_subtypes, __pyx_mstate->__pyx_n_u_meta, __pyx_mstate->__pyx_n_u_field, __pyx_mstate->__pyx_n_u_field};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_ogr_read_info, __pyx_mstate->__pyx_kp_b_iso88591_z_a_q_1_A_1_a_DA_hat7_S_6_A_Qa, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2408};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path_or_buffer, __pyx_mstate->__pyx_n_u_use_tmp_vsimem, __pyx_mstate->__pyx_n_u_ogr_dataset, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_layers};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_ogr_list_layers, __pyx_mstate->__pyx_kp_b_iso88591_z_a_55I_hat7_S_s_Qa_1_1_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {18, 0, 0, 56, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2901};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path_or_fp, __pyx_mstate->__pyx_n_u_layer, __pyx_mstate->__pyx_n_u_driver, __pyx_mstate->__pyx_n_u_geometry, __pyx_mstate->__pyx_n_u_fields, __pyx_mstate->__pyx_n_u_field_data, __pyx_mstate->__pyx_n_u_field_mask, __pyx_mstate->__pyx_n_u_crs, __pyx_mstate->__pyx_n_u_geometry_type, __pyx_mstate->__pyx_n_u_encoding, __pyx_mstate->__pyx_n_u_dataset_kwargs, __pyx_mstate->__pyx_n_u_layer_kwargs, __pyx_mstate->__pyx_n_u_promote_to_multi, __pyx_mstate->__pyx_n_u_nan_as_null, __pyx_mstate->__pyx_n_u_append, __pyx_mstate->__pyx_n_u_dataset_metadata, __pyx_mstate->__pyx_n_u_layer_metadata, __pyx_mstate->__pyx_n_u_gdal_tz_offsets, __pyx_mstate->__pyx_n_u_ogr_dataset, __pyx_mstate->__pyx_n_u_ogr_layer, __pyx_mstate->__pyx_n_u_ogr_feature, __pyx_mstate->__pyx_n_u_ogr_geometry, __pyx_mstate->__pyx_n_u_ogr_featuredef, __pyx_mstate->__pyx_n_u_ogr_fielddef, __pyx_mstate->__pyx_n_u_wkb_buffer, __pyx_mstate->__pyx_n_u_wkbtype, __pyx_mstate->__pyx_n_u_supports_transactions, __pyx_mstate->__pyx_n_u_err, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_num_records, __pyx_mstate->__pyx_n_u_num_field_data, __pyx_mstate->__pyx_n_u_num_fields, __pyx_mstate->__pyx_n_u_field_indexes, __pyx_mstate->__pyx_n_u_field_ogr_types, __pyx_mstate->__pyx_n_u_field_index_2, __pyx_mstate->__pyx_n_u_field_type, __pyx_mstate->__pyx_n_u_use_tmp_vsimem, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_layer_created, __pyx_mstate->__pyx_n_u_field_types, __pyx_mstate->__pyx_n_u_field_ogr_type, __pyx_mstate->__pyx_n_u_field_subtype, __pyx_mstate->__pyx_n_u_width, __pyx_mstate->__pyx_n_u_precision, __pyx_mstate->__pyx_n_u_name_b, __pyx_mstate->__pyx_n_u_index, __pyx_mstate->__pyx_n_u_wkb, __pyx_mstate->__pyx_n_u_field_idx, __pyx_mstate->__pyx_n_u_field_value, __pyx_mstate->__pyx_n_u_mask, __pyx_mstate->__pyx_n_u_value_b, __pyx_mstate->__pyx_n_u_datetime_value, __pyx_mstate->__pyx_n_u_tz_array, __pyx_mstate->__pyx_n_u_gdal_tz, __pyx_mstate->__pyx_n_u_exc, __pyx_mstate->__pyx_n_u_field};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_ogr_write, __pyx_mstate->__pyx_kp_b_iso88591_a_q_1_Q_a_1_Q_1_A_c_G_Q_Qk_z_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {13, 0, 0, 26, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3264};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path_or_fp, __pyx_mstate->__pyx_n_u_layer, __pyx_mstate->__pyx_n_u_driver, __pyx_mstate->__pyx_n_u_arrow_obj, __pyx_mstate->__pyx_n_u_crs, __pyx_mstate->__pyx_n_u_geometry_type, __pyx_mstate->__pyx_n_u_geometry_name, __pyx_mstate->__pyx_n_u_encoding, __pyx_mstate->__pyx_n_u_dataset_kwargs, __pyx_mstate->__pyx_n_u_layer_kwargs, __pyx_mstate->__pyx_n_u_append, __pyx_mstate->__pyx_n_u_dataset_metadata, __pyx_mstate->__pyx_n_u_layer_metadata, __pyx_mstate->__pyx_n_u_ogr_dataset, __pyx_mstate->__pyx_n_u_ogr_layer, __pyx_mstate->__pyx_n_u_options, __pyx_mstate->__pyx_n_u_use_tmp_vsimem, __pyx_mstate->__pyx_n_u_stream, __pyx_mstate->__pyx_n_u_schema, __pyx_mstate->__pyx_n_u_array, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_layer_created, __pyx_mstate->__pyx_n_u_opts, __pyx_mstate->__pyx_n_u_stream_capsule, __pyx_mstate->__pyx_n_u_exc, __pyx_mstate->__pyx_n_u_gdal_msg};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_pyogrio__io_pyx, __pyx_mstate->__pyx_n_u_ogr_write_arrow, __pyx_mstate->__pyx_kp_b_iso88591_a_q_q_A_Q_A_8_Q_0_oQ_N_a_1_T_s, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CommonTypesMetaclass.init */
  if (likely(__pyx_CommonTypesMetaclass_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CythonFunctionShared.init */
  if (likely(__pyx_CyFunction_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* Generator.init */
  if (likely(__pyx_Generator_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by GetBuiltinName) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        Py_INCREF(src[i]);
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall (used by PyObjectCallOneArg) */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallOneArg (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTestFunc (used by ArgTypeTest) */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* CIntToDigits (used by CIntToPyUnicode) */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp = PySequence_Repeat(padding, uoffset - prepend_sign);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyUnicode_Concat(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyUnicode_Concat(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* COrdinalToPyUnicode (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value) {
    return value <= 1114111;
}
static PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t ulength, char padding_char) {
    Py_ssize_t padding_length = ulength - 1;
    if (likely((padding_length <= 250) && (value < 0xD800 || value > 0xDFFF))) {
        char chars[256];
        if (value <= 255) {
            memset(chars, padding_char, (size_t) padding_length);
            chars[ulength-1] = (char) value;
            return PyUnicode_DecodeLatin1(chars, ulength, NULL);
        }
        char *cpos = chars + sizeof(chars);
        if (value < 0x800) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xc0 | (value & 0x1f));
        } else if (value < 0x10000) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xe0 | (value & 0x0f));
        } else {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xf0 | (value & 0x07));
        }
        cpos -= padding_length;
        memset(cpos, padding_char, (size_t) padding_length);
        return PyUnicode_DecodeUTF8(cpos, chars + sizeof(chars) - cpos, NULL);
    }
    if (value <= 127 && CYTHON_USE_UNICODE_INTERNALS) {
        const char chars[1] = {(char) value};
        return __Pyx_PyUnicode_BuildFromAscii(ulength, chars, 1, 0, padding_char);
    }
    {
        PyObject *uchar, *padding_uchar, *padding, *result;
        padding_uchar = PyUnicode_FromOrdinal(padding_char);
        if (unlikely(!padding_uchar)) return NULL;
        padding = PySequence_Repeat(padding_uchar, padding_length);
        Py_DECREF(padding_uchar);
        if (unlikely(!padding)) return NULL;
        uchar = PyUnicode_FromOrdinal(value);
        if (unlikely(!uchar)) {
            Py_DECREF(padding);
            return NULL;
        }
        result = PyUnicode_Concat(padding, uchar);
        Py_DECREF(padding);
        Py_DECREF(uchar);
        return result;
    }
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (int) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_ssize_t neg_one = (Py_ssize_t) -1, const_zero = (Py_ssize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (Py_ssize_t) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_Py_ssize_t(Py_ssize_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(Py_ssize_t)*3+2];
    char *dpos, *end = digits + sizeof(Py_ssize_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    Py_ssize_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const Py_ssize_t neg_one = (Py_ssize_t) -1, const_zero = (Py_ssize_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (Py_ssize_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (Py_ssize_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (Py_ssize_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        Py_INCREF(values[i]);
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n)))
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS)) {
        return __Pyx_PyList_GetItemRefFast(o, wrapped_i, unsafe_shared);
    } else
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyList_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)) {
            return __Pyx_PyList_GetItemRefFast(o, n, unsafe_shared);
        } else if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_NewRef(PyList_GET_ITEM(o, n));
        }
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *pos = NULL;
#else
        Py_ssize_t pos = 0;
#endif
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL);
#if CYTHON_AVOID_BORROWED_REFS
        Py_XDECREF(pos);
#endif
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* DivInt[Py_ssize_t] */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b, int b_is_constant) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    Py_ssize_t adapt_python = (b_is_constant ?
        ((r != 0) & ((r < 0) ^ (b < 0))) :
        ((r != 0) & ((r ^ b) < 0))
    );
    return q - adapt_python;
}

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* GetTopmostException (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX > 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX > 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX > 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX <= 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* HasAttr (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* ImportImpl (used by Import) */
static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
    PyObject *imported_module = PyImport_GetModule(qualname);
    if (unlikely(!imported_module)) {
        *module = NULL;
        if (PyErr_Occurred()) {
            return -1;
        }
        return 0;
    }
    *module = imported_module;
    return 1;
}
static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
    PyObject *imported_module;
    PyObject *top_level_package_name;
    Py_ssize_t i;
    int status, module_found;
    Py_ssize_t dot_index;
    module_found = __Pyx__Import_GetModule(qualname, &imported_module);
    if (unlikely(!module_found || module_found == -1)) {
        *module = NULL;
        return module_found;
    }
    if (imported_names) {
        for (i = 0; i < len_imported_names; i++) {
            PyObject *imported_name = imported_names[i];
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
            int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
#else
            int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
            if (unlikely(has_imported_attribute == -1)) goto error;
#endif
            if (!has_imported_attribute) {
                goto not_found;
            }
        }
        *module = imported_module;
        return 1;
    }
    dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
    if (dot_index == -1) {
        *module = imported_module;
        return 1;
    }
    if (unlikely(dot_index == -2)) goto error;
    top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
    if (unlikely(!top_level_package_name)) goto error;
    Py_DECREF(imported_module);
    status = __Pyx__Import_GetModule(top_level_package_name, module);
    Py_DECREF(top_level_package_name);
    return status;
error:
    Py_DECREF(imported_module);
    *module = NULL;
    return -1;
not_found:
    Py_DECREF(imported_module);
    *module = NULL;
    return 0;
}
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *from_list = 0;
    int module_found;
    if (!qualname) {
        qualname = name;
    }
    module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
    if (likely(module_found == 1)) {
        return module;
    } else if (unlikely(module_found == -1)) {
        return NULL;
    }
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (imported_names) {
#if CYTHON_COMPILING_IN_CPYTHON
        from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
        if (unlikely(!from_list))
            goto bad;
#else
        from_list = PyList_New(len_imported_names);
        if (unlikely(!from_list)) goto bad;
        for (Py_ssize_t i=0; i<len_imported_names; ++i) {
            if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
        }
#endif
    }
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, moddict, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, moddict, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(from_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
    return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* PySequenceMultiply */
#if CYTHON_USE_TYPE_SLOTS
static PyObject* __Pyx_PySequence_Multiply_Generic(PyObject *seq, Py_ssize_t mul) {
    PyObject *result, *pymul = PyLong_FromSsize_t(mul);
    if (unlikely(!pymul))
        return NULL;
    result = PyNumber_Multiply(seq, pymul);
    Py_DECREF(pymul);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PySequence_Multiply(PyObject *seq, Py_ssize_t mul) {
    PyTypeObject *type = Py_TYPE(seq);
    if (likely(type->tp_as_sequence && type->tp_as_sequence->sq_repeat)) {
        return type->tp_as_sequence->sq_repeat(seq, mul);
    } else {
        return __Pyx_PySequence_Multiply_Generic(seq, mul);
    }
}
#endif

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* PyObjectFormat */
#if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* obj, PyObject* format_spec) {
    int ret;
    _PyUnicodeWriter writer;
    if (likely(PyFloat_CheckExact(obj))) {
        _PyUnicodeWriter_Init(&writer);
        ret = _PyFloat_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else if (likely(PyLong_CheckExact(obj))) {
        _PyUnicodeWriter_Init(&writer);
        ret = _PyLong_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else {
        return PyObject_Format(obj, format_spec);
    }
    if (unlikely(ret == -1)) {
        _PyUnicodeWriter_Dealloc(&writer);
        return NULL;
    }
    return _PyUnicodeWriter_Finish(&writer);
}
#endif

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (unlikely(!j)) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE && !CYTHON_AVOID_BORROWED_REFS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS && !__Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared))) {
            Py_INCREF(v);
            return PyList_SetItem(o, n, v);
        } else if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old;
            Py_INCREF(v);
            old = PyList_GET_ITEM(o, n);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 0;
        }
    } else
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_ass_subscript) {
            int r;
            PyObject *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return -1;
            r = mm->mp_ass_subscript(o, key, v);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return sm->sq_ass_item(o, i, v);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_SetItem(o, i, v);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_SetItemInt_Generic(o, PyLong_FromSsize_t(i), v);
}

/* RaiseUnboundLocalError */
static void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* DivInt[long] */
static CYTHON_INLINE long __Pyx_div_long(long a, long b, int b_is_constant) {
    long q = a / b;
    long r = a - q*b;
    long adapt_python = (b_is_constant ?
        ((r != 0) & ((r < 0) ^ (b < 0))) :
        ((r != 0) & ((r ^ b) < 0))
    );
    return q - adapt_python;
}

/* IterFinish (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* PyObjectCallNoArg (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}
#endif

/* PyObjectCallMethod0 (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[1] = {obj};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_CallNoArg;
    return PyObject_VectorcallMethod(method_name, args, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
#endif
}

/* UnpackItemEndCheck (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* UnpackTupleError (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else {
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(t);
 #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(size < 0)) return;
 #endif
      if (size < index) {
        __Pyx_RaiseNeedMoreValuesError(size);
      } else {
        __Pyx_RaiseTooManyValuesError(index);
      }
    }
}

/* UnpackTuple2 (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
        PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple) {
    if (likely(is_tuple || PyTuple_Check(tuple))) {
        Py_ssize_t size;
        if (has_known_size) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        size = __Pyx_PyTuple_GET_SIZE(tuple);
        if (likely(size == 2)) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        if (size >= 0) {
            __Pyx_UnpackTupleError(tuple, 2);
        }
        return -1;
    } else {
        return __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple);
    }
}
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
    value1 = __Pyx_PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = __Pyx_PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = __Pyx_PyObject_GetIterNextFunc(iter);
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
#if CYTHON_AVOID_BORROWED_REFS
#include <string.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_AVOID_BORROWED_REFS
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#else
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_AVOID_BORROWED_REFS
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
#if !CYTHON_AVOID_BORROWED_REFS
static CYTHON_INLINE int __Pyx_dict_iter_next_source_is_dict(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem) {
    PyObject *key, *value;
    if (unlikely(orig_length != PyDict_Size(iter_obj))) {
        PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
        return -1;
    }
    if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
        return 0;
    }
    if (pitem) {
        PyObject* tuple = PyTuple_New(2);
        if (unlikely(!tuple)) {
            return -1;
        }
        Py_INCREF(key);
        Py_INCREF(value);
        #if CYTHON_ASSUME_SAFE_MACROS
        PyTuple_SET_ITEM(tuple, 0, key);
        PyTuple_SET_ITEM(tuple, 1, value);
        #else
        if (unlikely(PyTuple_SetItem(tuple, 0, key) < 0)) {
            Py_DECREF(value);
            Py_DECREF(tuple);
            return -1;
        }
        if (unlikely(PyTuple_SetItem(tuple, 1, value) < 0)) {
            Py_DECREF(tuple);
            return -1;
        }
        #endif
        *pitem = tuple;
    } else {
        if (pkey) {
            Py_INCREF(key);
            *pkey = key;
        }
        if (pvalue) {
            Py_INCREF(value);
            *pvalue = value;
        }
    }
    return 1;
}
#endif
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_AVOID_BORROWED_REFS
    if (source_is_dict) {
        int result;
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_BEGIN_CRITICAL_SECTION(iter_obj);
#endif
        result = __Pyx_dict_iter_next_source_is_dict(iter_obj, orig_length, ppos, pkey, pvalue, pitem);
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_END_CRITICAL_SECTION();
#endif
        return result;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(tuple_size < 0)) return -1;
        #endif
        if (unlikely(pos >= tuple_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_ASSUME_SAFE_MACROS
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        #else
        next_item = PyTuple_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #endif
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t list_size = __Pyx_PyList_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(list_size < 0)) return -1;
        #endif
        if (unlikely(pos >= list_size)) return 0;
        *ppos = pos + 1;
        next_item = __Pyx_PyList_GetItemRef(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* PyObjectLookupSpecial */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* decode_c_string */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* UnicodeConcatInPlace */
# if CYTHON_COMPILING_IN_CPYTHON
static int
__Pyx_unicode_modifiable(PyObject *unicode, int unsafe_shared)
{
    if (!__Pyx_IS_UNIQUELY_REFERENCED(unicode, unsafe_shared))
        return 0;
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030F0000
    if (PyUnstable_Unicode_GET_CACHED_HASH(unicode) != -1)
        return 0;
#endif
    if (!PyUnicode_CheckExact(unicode))
        return 0;
    if (PyUnicode_CHECK_INTERNED(unicode))
        return 0;
    return 1;
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    ) {
    PyObject *left = *p_left;
    Py_ssize_t left_len, right_len, new_len;
    if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
        return NULL;
    if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
        return NULL;
    left_len = PyUnicode_GET_LENGTH(left);
    if (left_len == 0) {
        Py_INCREF(right);
        return right;
    }
    right_len = PyUnicode_GET_LENGTH(right);
    if (right_len == 0) {
        Py_INCREF(left);
        return left;
    }
    if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
        PyErr_SetString(PyExc_OverflowError,
                        "strings are too large to concat");
        return NULL;
    }
    new_len = left_len + right_len;
    if (__Pyx_unicode_modifiable(left, unsafe_shared)
            && PyUnicode_CheckExact(right)
            && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
            && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
        int ret;
        __Pyx_GIVEREF(*p_left);
        ret = PyUnicode_Resize(p_left, new_len);
        __Pyx_GOTREF(*p_left);
        if (unlikely(ret != 0))
            return NULL;
        #if PY_VERSION_HEX >= 0x030d0000
        if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
        #else
        _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
        #endif
        __Pyx_INCREF(*p_left);
        __Pyx_GIVEREF(*p_left);
        return *p_left;
    } else {
        return __Pyx_PyUnicode_Concat(left, right);
    }
  }
#endif

/* PyUnicode_Unicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj) {
    if (unlikely(obj == Py_None))
        obj = __pyx_mstate_global->__pyx_kp_u_None;
    return __Pyx_NewRef(obj);
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc, *new_exc;
    __Pyx_PyThreadState_declare
    int is_async_stopiteration = 0;
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    __Pyx_PyThreadState_assign
    cur_exc = __Pyx_PyErr_CurrentExceptionType();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else {
            return;
        }
    }
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(tb);
    new_exc = PyObject_CallFunction(PyExc_RuntimeError, "s",
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        "generator raised StopIteration");
    if (!new_exc) {
        Py_XDECREF(val);
        return;
    }
    PyException_SetCause(new_exc, val); // steals ref to val
    PyErr_SetObject(PyExc_RuntimeError, new_exc);
    Py_DECREF(new_exc);
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    Py_INCREF(key);
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 1;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) == 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 0;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 0;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal == 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a == (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyLong_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyLong_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_MultiplyObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceMultiply : PyNumber_Multiply)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_MultiplyObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a;
    const PY_LONG_LONG llb = intval;
    PY_LONG_LONG lla;
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        return __Pyx_NewRef(op1);
    }
    const int is_positive = __Pyx_PyLong_IsPos(op1);
    const digit* digits = __Pyx_PyLong_Digits(op1);
    const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
    if (likely(size == 1)) {
        a = (long) digits[0];
        if (!is_positive) a *= -1;
    } else {
        switch (size) {
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT+30) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT+30) {
                    lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT+30) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT+30) {
                    lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT+30) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT+30) {
                    lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
        }
        return PyLong_Type.tp_as_number->nb_multiply(op1, op2);
    }
    calculate_long:
        CYTHON_UNUSED_VAR(a);
        CYTHON_UNUSED_VAR(b);
        lla = a;
        goto calculate_long_long;
    calculate_long_long:
        {
            PY_LONG_LONG llx;
            llx = lla * llb;
            return PyLong_FromLongLong(llx);
        }
    
}
#endif
static PyObject* __Pyx_Float___Pyx_PyLong_MultiplyObjC(PyObject *float_val, long intval, int zerodivision_check) {
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    double a = __Pyx_PyFloat_AS_DOUBLE(float_val);
        double result;
        
        result = ((double)a) * (double)b;
        return PyFloat_FromDouble(result);
}
static CYTHON_INLINE PyObject* __Pyx_PyLong_MultiplyObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_MultiplyObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        return __Pyx_Float___Pyx_PyLong_MultiplyObjC(op1, intval, zerodivision_check);
    }
    return __Pyx_Fallback___Pyx_PyLong_MultiplyObjC(op1, op2, inplace);
}
#endif

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a;
    const PY_LONG_LONG llb = intval;
    PY_LONG_LONG lla;
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        return __Pyx_NewRef(op2);
    }
    const int is_positive = __Pyx_PyLong_IsPos(op1);
    const digit* digits = __Pyx_PyLong_Digits(op1);
    const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
    if (likely(size == 1)) {
        a = (long) digits[0];
        if (!is_positive) a *= -1;
    } else {
        switch (size) {
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
        }
        return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    calculate_long:
        {
            long x;
            x = a + b;
            return PyLong_FromLong(x);
        }
    calculate_long_long:
        {
            PY_LONG_LONG llx;
            llx = lla + llb;
            return PyLong_FromLongLong(llx);
        }
    
}
#endif
static PyObject* __Pyx_Float___Pyx_PyLong_AddObjC(PyObject *float_val, long intval, int zerodivision_check) {
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    double a = __Pyx_PyFloat_AS_DOUBLE(float_val);
        double result;
        
        result = ((double)a) + (double)b;
        return PyFloat_FromDouble(result);
}
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        return __Pyx_Float___Pyx_PyLong_AddObjC(op1, intval, zerodivision_check);
    }
    return __Pyx_Fallback___Pyx_PyLong_AddObjC(op1, op2, inplace);
}
#endif

/* RaiseClosureNameError */
static void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* dict_setdefault (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if __PYX_LIMITED_VERSION_HEX >= 0x030F0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4)
    PyDict_SetDefaultRef(d, key, default_value, &value);
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* LimitedApiGetTypeDict (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType (used by FetchCommonType) */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_vectorcall_offset = memb->offset;
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
#if !CYTHON_COMPILING_IN_PYPY
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
#endif  // !CYTHON_COMPILING_IN_PYPY
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
#if !CYTHON_COMPILING_IN_PYPY
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
#else
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#endif  // !CYTHON_COMPILING_IN_PYPY
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* AddModuleRef (used by FetchSharedCythonModule) */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* FetchSharedCythonModule (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* FetchCommonType (used by CommonTypesMetaclass) */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    cached_type = __Pyx_PyType_FromMetaclass(
        metaclass,
        CYTHON_USE_MODULE_STATE ? module : abi_module,
        spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass (used by CythonFunctionShared) */
static PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
#if __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject* __pyx_CommonTypesMetaclass_call(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *args, CYTHON_UNUSED PyObject *kwds) {
    PyErr_SetString(PyExc_TypeError, "Cannot instantiate Cython internal types");
    return NULL;
}
static int __pyx_CommonTypesMetaclass_setattr(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *attr, CYTHON_UNUSED PyObject *value) {
    PyErr_SetString(PyExc_TypeError, "Cython internal types are immutable");
    return -1;
}
#endif
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {Py_tp_call, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_new, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_setattro, (void*)__pyx_CommonTypesMetaclass_setattr},
    #endif
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* CallTypeTraverse (used by CythonFunctionShared) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* PyMethodNew (used by CythonFunctionShared) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i;
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos;
    #else
    Py_ssize_t pos;
    #endif
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = 0;
    i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (__Pyx_PyDict_NextRef(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(pos);
    #endif
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared (used by CythonFunction) */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
#endif
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int __Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value);
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    if (unlikely(__Pyx_CyFunction_set_annotate_in_dict_if_exists((PyObject*) op, Py_None) < 0)) return -1;
    return 0;
}
static int
__Pyx_CyFunction_get_dict_if_exists(PyObject *op_in, PyObject **dict) {
    /* Return 1 if the function dict exists, 0 otherwise.  This cannot fail:
     * _PyObject_GetDictPtr() may clear errors internally, but never reports them. */
#if CYTHON_COMPILING_IN_PYPY
    *dict = PyObject_GenericGetDict(op_in, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030C0000
    *dict = ((__pyx_CyFunctionObject*) op_in)->func_dict;
#else
    PyObject **dictptr = _PyObject_GetDictPtr(op_in);
    *dict = likely(dictptr) ? *dictptr : NULL;
#endif
    return *dict ? 1 : 0;
}
static int
__Pyx_CyFunction_get_annotate_from_dict_if_exists(PyObject *op_in, PyObject **annotate) {
    PyObject *dict;
    int dict_found;
    *annotate = NULL;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return __Pyx_PyDict_GetItemRef(dict, __pyx_mstate_global->__pyx_n_u_annotate, annotate);
}
static int
__Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int dict_found;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
}
static int
__Pyx_CyFunction_set_annotate_in_dict(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int result;
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = __Pyx_CyFunction_get_dict((__pyx_CyFunctionObject*) op_in, NULL);
#else
    dict = PyObject_GenericGetDict(op_in, NULL);
#endif
    if (unlikely(!dict)) return -1;
    result = PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
    Py_DECREF(dict);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    PyObject *result = NULL;
    __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    result = __Pyx_XNewRef(op->func_annotations);
    __Pyx_END_CRITICAL_SECTION();
    if (result) return result;
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (!annotate || annotate == Py_None) {
        Py_XDECREF(annotate);
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        result = __Pyx_CyFunction_get_annotations_locked(op);
        __Pyx_END_CRITICAL_SECTION();
        return result;
    }
    PyObject *format = PyLong_FromLong(1L);  // annotationlib.Format.VALUE
    if (likely(format)) {
        result = __Pyx_PyObject_CallOneArg(annotate, format);
        Py_DECREF(format);
    }
    Py_DECREF(annotate);
    if (unlikely(!result)) return NULL;
    if (unlikely(!PyDict_Check(result))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must return a dict");
        Py_DECREF(result);
        return NULL;
    }
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    __Pyx_Py_XDECREF_SET(op->func_annotations, __Pyx_NewRef(result));
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *__Pyx_CyFunction_annotate_impl(PyObject *self, PyObject *args) {
    CYTHON_UNUSED_VAR(args);
    if (unlikely(!self)) {
        PyErr_SetString(PyExc_SystemError, "cython __annotate__ called without 'self' argument");
    }
    Py_XINCREF(self);
    return self;
}
static PyMethodDef __Pyx_CyFunction_annotate_method = {
    "__annotate__",
    (PyCFunction)(void (*)(void))__Pyx_CyFunction_annotate_impl,
    METH_VARARGS,
    "Placeholder __annotate__ function to allow 'functools.wraps' to work "
    "on Cython functions."
};
static PyObject *
__Pyx_CyFunction_get_annotate(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (annotate) return annotate;
    PyObject *annotations = __Pyx_CyFunction_get_annotations(op_in, NULL);
    if (unlikely(!annotations)) return NULL;
    PyObject *method = PyCFunction_New(
        &__Pyx_CyFunction_annotate_method,
        annotations);
    Py_DECREF(annotations);
    return method;
}
static int
__Pyx_CyFunction_set_annotate(PyObject *op_in, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ cannot be deleted");
        return -1;
    }
    if (unlikely(value != Py_None && !PyCallable_Check(value))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must be callable or None");
        return -1;
    }
    if (value != Py_None) {
        __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        Py_CLEAR(op->func_annotations);
        __Pyx_END_CRITICAL_SECTION();
    }
    return __Pyx_CyFunction_set_annotate_in_dict(op_in, value);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
#else
    {"func_dict", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
#endif
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"__annotate__", (getter)__Pyx_CyFunction_get_annotate, (setter)__Pyx_CyFunction_set_annotate, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#endif
#if CYTHON_METH_FASTCALL
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_dict = NULL;
#endif
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func_dict);
#elif PY_VERSION_HEX < 0x030d0000
    _PyObject_ClearManagedDict((PyObject*)m);
#else
    PyObject_ClearManagedDict((PyObject*)m);
#endif
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func_dict);
#else
    {
        int e =
#if PY_VERSION_HEX < 0x030d0000
            _PyObject_VisitManagedDict
#else
            PyObject_VisitManagedDict
#endif
                ((PyObject*)m, visit, arg);
        if (e != 0) return e;
    }
#endif
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_annotations);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    #if PY_VERSION_HEX < 0x030e00A6
    size_t nargs_value = (size_t) nargs;
    #else
    Py_ssize_t nargs_value = nargs;
    #endif
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, nargs_value, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_DICT |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
    else state = (PyGILState_STATE)0;
    CYTHON_UNUSED_VAR(clineno);
    CYTHON_UNUSED_VAR(lineno);
    CYTHON_UNUSED_VAR(filename);
    CYTHON_MAYBE_UNUSED_VAR(nogil);
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(0);
    }
    ctx = PyUnicode_FromString(name);
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
    if (nogil)
        PyGILState_Release(state);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* unicode_tailmatch */
static int __Pyx_PyUnicode_TailmatchTuple(PyObject* s, PyObject* substrings,
                                          Py_ssize_t start, Py_ssize_t end, int direction) {
    Py_ssize_t i, count = __Pyx_PyTuple_GET_SIZE(substrings);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(count < 0)) return -1;
    #endif
    for (i = 0; i < count; i++) {
        Py_ssize_t result;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        result = PyUnicode_Tailmatch(s, PyTuple_GET_ITEM(substrings, i),
                                     start, end, direction);
#else
        PyObject* sub = __Pyx_PySequence_ITEM(substrings, i);
        if (unlikely(!sub)) return -1;
        result = PyUnicode_Tailmatch(s, sub, start, end, direction);
        Py_DECREF(sub);
#endif
        if (result) {
            return (int) result;
        }
    }
    return 0;
}
static int __Pyx_PyUnicode_Tailmatch(PyObject* s, PyObject* substr,
                                     Py_ssize_t start, Py_ssize_t end, int direction) {
    if (unlikely(PyTuple_Check(substr))) {
        return __Pyx_PyUnicode_TailmatchTuple(s, substr, start, end, direction);
    }
    return (int) PyUnicode_Tailmatch(s, substr, start, end, direction);
}

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceFloorDivide : PyNumber_FloorDivide)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a;
    const PY_LONG_LONG llb = intval;
    PY_LONG_LONG lla;
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        return __Pyx_NewRef(op1);
    }
    const int is_positive = __Pyx_PyLong_IsPos(op1);
    const digit* digits = __Pyx_PyLong_Digits(op1);
    const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
    if (likely(size == 1)) {
        a = (long) digits[0];
        if (!is_positive) a *= -1;
    } else {
        switch (size) {
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
        }
        return PyLong_Type.tp_as_number->nb_floor_divide(op1, op2);
    }
    calculate_long:
        {
            long q, r;
            q = a / b;
            r = a - q*b;
            q -= ((r != 0) & ((r ^ b) < 0));
            return PyLong_FromLong(q);
        }
    calculate_long_long:
        {
            PY_LONG_LONG q, r;
            q = lla / llb;
            r = lla - q*llb;
            q -= ((r != 0) & ((r ^ llb) < 0));
            return PyLong_FromLongLong(q);
        }
    
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_FloorDivideObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    return __Pyx_Fallback___Pyx_PyLong_FloorDivideObjC(op1, op2, inplace);
}
#endif

/* DictGetItem */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    if (unlikely(__Pyx_PyDict_GetItemRef(d, key, &value) == 0)) { // no value, no error
        if (unlikely(PyTuple_Check(key))) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args)) {
                PyErr_SetObject(PyExc_KeyError, args);
                Py_DECREF(args);
            }
        } else {
            PyErr_SetObject(PyExc_KeyError, key);
        }
    }
    return value;
}
#endif

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (unsigned int) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_unsigned_int(unsigned int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(unsigned int)*3+2];
    char *dpos, *end = digits + sizeof(unsigned int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    unsigned int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (unsigned int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (unsigned int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (unsigned int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_TrueDivideObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_TrueDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a;
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
    }
    const int is_positive = __Pyx_PyLong_IsPos(op1);
    const digit* digits = __Pyx_PyLong_Digits(op1);
    const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
    if (likely(size == 1)) {
        a = (long) digits[0];
        if (!is_positive) a *= -1;
    } else {
        switch (size) {
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT && 1 * PyLong_SHIFT < 53) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                }
                break;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT && 2 * PyLong_SHIFT < 53) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                }
                break;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT && 3 * PyLong_SHIFT < 53) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                }
                break;
        }
        return PyLong_Type.tp_as_number->nb_true_divide(op1, op2);
    }
    calculate_long:
        if ((8 * sizeof(long) <= 53 || likely(labs(a) <= ((PY_LONG_LONG)1 << 53)))
                || __Pyx_PyLong_DigitCount(op1) <= 52 / PyLong_SHIFT) {
            return PyFloat_FromDouble((double)a / (double)b);
        }
        return PyLong_Type.tp_as_number->nb_true_divide(op1, op2);
}
#endif
static PyObject* __Pyx_Float___Pyx_PyLong_TrueDivideObjC(PyObject *float_val, long intval, int zerodivision_check) {
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    double a = __Pyx_PyFloat_AS_DOUBLE(float_val);
        double result;
        
        result = ((double)a) / (double)b;
        return PyFloat_FromDouble(result);
}
static CYTHON_INLINE PyObject* __Pyx_PyLong_TrueDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_TrueDivideObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        return __Pyx_Float___Pyx_PyLong_TrueDivideObjC(op1, intval, zerodivision_check);
    }
    return __Pyx_Fallback___Pyx_PyLong_TrueDivideObjC(op1, op2, inplace);
}
#endif

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int64_t(int64_t value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (int64_t) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int64_t(int64_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int64_t)*3+2];
    char *dpos, *end = digits + sizeof(int64_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int64_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int64_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int64_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int64_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* AllocateExtensionType */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final) {
    if (is_final || likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
        allocfunc alloc_func = __Pyx_PyType_GetSlot(t, tp_alloc, allocfunc);
        return alloc_func(t, 0);
    } else {
        newfunc tp_new = __Pyx_PyType_TryGetSlot(&PyBaseObject_Type, tp_new, newfunc);
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (!tp_new) {
            PyObject *new_str = PyUnicode_FromString("__new__");
            if (likely(new_str)) {
                PyObject *o = PyObject_CallMethodObjArgs((PyObject *)&PyBaseObject_Type, new_str, t, NULL);
                Py_DECREF(new_str);
                return o;
            } else
                return NULL;
        } else
    #endif
        return tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
}

/* ValidateBasesTuple (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* DelItemOnTypeDict (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_2_8
#define __PYX_HAVE_RT_ImportType_3_2_8
static PyTypeObject *__Pyx_ImportType_3_2_8(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_8 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_8 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_8 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* PxdImportShared (used by FunctionImport) */
#ifndef __PYX_HAVE_RT_ImportFromPxd_3_2_8
#define __PYX_HAVE_RT_ImportFromPxd_3_2_8
static int __Pyx_ImportFromPxd_3_2_8(PyObject *module, const char *name, void **p, const char *sig, const char *what) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, "__pyx_capi__");
    if (!d)
        goto bad;
#if (defined(Py_LIMITED_API) && Py_LIMITED_API >= 0x030d0000) || (!defined(Py_LIMITED_API) && PY_VERSION_HEX >= 0x030d0000)
    PyDict_GetItemStringRef(d, name, &cobj);
#else
    cobj = PyDict_GetItemString(d, name);
    Py_XINCREF(cobj);
#endif
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C %.8s %.200s",
                PyModule_GetName(module), what, name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C %.8s %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             what, PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(d);
    Py_XDECREF(cobj);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_2_8
#define __PYX_HAVE_RT_ImportFunction_3_2_8
static int __Pyx_ImportFunction_3_2_8(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    int result = __Pyx_ImportFromPxd_3_2_8(module, funcname, &tmp.p, sig, "function");
    if (result == 0) {
        *f = tmp.fp;
    }
    return result;
}
#endif

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__2);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* ListPack */
static PyObject *__Pyx_PyList_Pack(Py_ssize_t n, ...) {
    va_list va;
    PyObject *l = PyList_New(n);
    va_start(va, n);
    if (unlikely(!l)) goto end;
    for (Py_ssize_t i=0; i<n; ++i) {
        PyObject *arg = va_arg(va, PyObject*);
        Py_INCREF(arg);
        if (__Pyx_PyList_SET_ITEM(l, i, arg) != (0)) {
            Py_CLEAR(l);
            goto end;
        }
    }
    end:
    va_end(va);
    return l;
}

/* CalculateMetaclass (used by Py3ClassCreate) */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases;
#if CYTHON_ASSUME_SAFE_SIZE
    nbases = PyTuple_GET_SIZE(bases);
#else
    nbases = PyTuple_Size(bases);
    if (nbases < 0) return NULL;
#endif
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
#if CYTHON_ASSUME_SAFE_MACROS
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *tmp = PyTuple_GetItem(bases, i);
        if (!tmp) return NULL;
#endif
        tmptype = Py_TYPE(tmp);
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
        metaclass = &PyType_Type;
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* PyObjectCall2Args (used by Py3ClassCreate) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_mstate_global->__pyx_n_u_prepare);
        if (prep) {
            PyObject *pargs[3] = {NULL, name, bases};
            ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
            Py_DECREF(prep);
        } else {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result;
    PyObject *owned_metaclass = NULL;
    PyObject *margs[4] = {NULL, name, bases, dict};
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_mstate_global->__pyx_n_u_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
#define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
#elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
#else
PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
    PyObject **dict_ptr = _PyObject_GetDictPtr(o);
    return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
}
#endif
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline = NULL;
    PyObject *ptype, *pvalue, *ptraceback;
    PyObject *cython_runtime_dict;
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
    cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, cython_runtime_dict,
            __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
    }
    if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    Py_XDECREF(cython_runtime_dict);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache (used by AddTraceback) */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        if (c_line) {
            py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        } else {
            py_funcname = PyUnicode_FromString(funcname);
        }
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* MemviewRefcount */
#include <stdio.h>
#ifndef _Py_NO_RETURN
#define _Py_NO_RETURN
#endif
_Py_NO_RETURN
static void __pyx_fatalerror(const char *fmt, ...) {
    va_list vargs;
    char msg[200];
#if PY_VERSION_HEX >= 0x030A0000 || defined(HAVE_STDARG_PROTOTYPES)
    va_start(vargs, fmt);
#else
    va_start(vargs);
#endif
    vsnprintf(msg, 200, fmt, vargs);
    va_end(vargs);
    Py_FatalError(msg);
}
static CYTHON_INLINE int
__pyx_add_acquisition_count_locked(__pyx_atomic_int_type *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)++;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE int
__pyx_sub_acquisition_count_locked(__pyx_atomic_int_type *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)--;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE void
__Pyx_INC_MEMVIEW(__Pyx_memviewslice *memslice, int have_gil, int lineno)
{
    __pyx_nonatomic_int_type old_acquisition_count;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None)) {
        return;
    }
    old_acquisition_count = __pyx_add_acquisition_count(memview);
    if (unlikely(old_acquisition_count <= 0)) {
        if (likely(old_acquisition_count == 0)) {
            if (have_gil) {
                Py_INCREF((PyObject *) memview);
            } else {
                PyGILState_STATE _gilstate = PyGILState_Ensure();
                Py_INCREF((PyObject *) memview);
                PyGILState_Release(_gilstate);
            }
        } else {
            __pyx_fatalerror("Acquisition count is %d (line %d)",
                             old_acquisition_count+1, lineno);
        }
    }
}
static CYTHON_INLINE void __Pyx_XCLEAR_MEMVIEW(__Pyx_memviewslice *memslice,
                                             int have_gil, int lineno) {
    __pyx_nonatomic_int_type old_acquisition_count;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (unlikely(!memview || (PyObject *) memview == Py_None)) {
        memslice->memview = NULL;
        return;
    }
    old_acquisition_count = __pyx_sub_acquisition_count(memview);
    memslice->data = NULL;
    if (likely(old_acquisition_count > 1)) {
        memslice->memview = NULL;
    } else if (likely(old_acquisition_count == 1)) {
        if (have_gil) {
            Py_CLEAR(memslice->memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_CLEAR(memslice->memview);
            PyGILState_Release(_gilstate);
        }
    } else {
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         old_acquisition_count-1, lineno);
    }
}

/* MemviewSliceIsContig */
static int
__pyx_memviewslice_is_contig(const __Pyx_memviewslice mvs, char order, int ndim)
{
    int i, index, step, start;
    Py_ssize_t itemsize = mvs.memview->view.itemsize;
    if (order == 'F') {
        step = 1;
        start = 0;
    } else {
        step = -1;
        start = ndim - 1;
    }
    for (i = 0; i < ndim; i++) {
        index = start + step * i;
        if (mvs.suboffsets[index] >= 0 || mvs.strides[index] != itemsize)
            return 0;
        itemsize *= mvs.shape[index];
    }
    return 1;
}

/* OverlappingSlices */
static void
__pyx_get_array_memory_extents(__Pyx_memviewslice *slice,
                               void **out_start, void **out_end,
                               int ndim, size_t itemsize)
{
    char *start, *end;
    int i;
    start = end = slice->data;
    for (i = 0; i < ndim; i++) {
        Py_ssize_t stride = slice->strides[i];
        Py_ssize_t extent = slice->shape[i];
        if (extent == 0) {
            *out_start = *out_end = start;
            return;
        } else {
            if (stride > 0)
                end += stride * (extent - 1);
            else
                start += stride * (extent - 1);
        }
    }
    *out_start = start;
    *out_end = end + itemsize;
}
static int
__pyx_slices_overlap(__Pyx_memviewslice *slice1,
                     __Pyx_memviewslice *slice2,
                     int ndim, size_t itemsize)
{
    void *start1, *end1, *start2, *end2;
    __pyx_get_array_memory_extents(slice1, &start1, &end1, ndim, itemsize);
    __pyx_get_array_memory_extents(slice2, &start2, &end2, ndim, itemsize);
    return (start1 < end2) && (start2 < end1);
}

/* MemviewSliceInit */
static int
__Pyx_init_memviewslice(struct __pyx_memoryview_obj *memview,
                        int ndim,
                        __Pyx_memviewslice *memviewslice,
                        int memview_is_new_reference)
{
    __Pyx_RefNannyDeclarations
    int i, retval=-1;
    Py_buffer *buf = &memview->view;
    __Pyx_RefNannySetupContext("init_memviewslice", 0);
    if (unlikely(memviewslice->memview || memviewslice->data)) {
        PyErr_SetString(PyExc_ValueError,
            "memviewslice is already initialized!");
        goto fail;
    }
    if (buf->strides) {
        for (i = 0; i < ndim; i++) {
            memviewslice->strides[i] = buf->strides[i];
        }
    } else {
        Py_ssize_t stride = buf->itemsize;
        for (i = ndim - 1; i >= 0; i--) {
            memviewslice->strides[i] = stride;
            stride *= buf->shape[i];
        }
    }
    for (i = 0; i < ndim; i++) {
        memviewslice->shape[i]   = buf->shape[i];
        if (buf->suboffsets) {
            memviewslice->suboffsets[i] = buf->suboffsets[i];
        } else {
            memviewslice->suboffsets[i] = -1;
        }
    }
    memviewslice->memview = memview;
    memviewslice->data = (char *)buf->buf;
    if (__pyx_add_acquisition_count(memview) == 0 && !memview_is_new_reference) {
        Py_INCREF((PyObject*)memview);
    }
    retval = 0;
    goto no_fail;
fail:
    memviewslice->memview = 0;
    memviewslice->data = 0;
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}

/* SliceMemoryviewSlice */
static void __pyx_memoryview_slice_memviewslice_err_dim(PyObject *error, const char* msg, int dim) {
    PyGILState_STATE gilstate = PyGILState_Ensure();
    PyErr_Format(error, msg, dim);
    PyGILState_Release(gilstate);
}
static CYTHON_INLINE int __pyx_memoryview_slice_memviewslice(
        __Pyx_memviewslice *dst,
        Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
        int dim, int new_ndim, int *suboffset_dim,
        Py_ssize_t start, Py_ssize_t stop, Py_ssize_t step,
        int have_start, int have_stop, int have_step,
        int is_slice) {
    if (!is_slice) {
        if (start < 0) {
            start += shape;
        }
        if (unlikely(!(0 <= start && start < shape))) {
            __pyx_memoryview_slice_memviewslice_err_dim(PyExc_IndexError, "Index out of bounds (axis %d)", dim);
            return -1;
        }
    } else {
        int negative_step;
        if (have_step) {
            negative_step = step < 0;
            if (unlikely(step == 0)) {
                __pyx_memoryview_slice_memviewslice_err_dim(PyExc_ValueError, "Step may not be zero (axis %d)", dim);
                return -1;
            }
        } else {
            negative_step = 0;
            step = 1;
        }
        if (have_start) {
            if (start < 0) {
                start += shape;
                if (start < 0) {
                    start = 0;
                }
            } else if (start >= shape) {
                start = negative_step ? (shape - 1) : shape;
            }
        } else {
            start = negative_step ? (shape - 1) : 0;
        }
        if (have_stop) {
            if (stop < 0) {
                stop += shape;
                if (stop < 0) {
                    stop = 0;
                }
            } else if (stop > shape) {
                stop = shape;
            }
        } else {
            stop = negative_step ? -1 : shape;
        }
        Py_ssize_t new_shape = (stop - start) / step;
        if ((stop - start) - step * new_shape) {
            ++new_shape;
        }
        if (new_shape < 0) {
            new_shape = 0;
        }
        dst->strides[new_ndim] = stride * step;
        dst->shape[new_ndim] = new_shape;
        dst->suboffsets[new_ndim] = suboffset;
    }
    if (suboffset_dim[0] < 0) {
        dst->data += start * stride;
    } else {
        dst->suboffsets[suboffset_dim[0]] += start * stride;
    }
    if (suboffset >= 0) {
        if (!is_slice) {
            if (likely(new_ndim == 0)) {
                dst->data = ((char **)(dst->data))[0] + suboffset;
            } else {
                __pyx_memoryview_slice_memviewslice_err_dim(
                    PyExc_IndexError,
                    "All dimensions preceding dimension %d must be indexed and not sliced",
                    dim);
                return -1;
            }
        } else {
            suboffset_dim[0] = new_ndim;
        }
    }
    return 0;
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* MemviewObjectToObject */
static PyObject *__pyx_memview_get_object(const char *itemp) {
    PyObject *result = *(PyObject **) itemp;
    Py_INCREF(result);
    return result;
}
static int __pyx_memview_set_object(const char *itemp, PyObject *obj) {
    Py_INCREF(obj);
    Py_DECREF(*(PyObject **) itemp);
    *(PyObject **) itemp = obj;
    return 1;
}

/* MemviewDtypeToObject */
static CYTHON_INLINE PyObject *__pyx_memview_get_int(const char *itemp) {
    return (PyObject *) __Pyx_PyLong_From_int(*(int const *) itemp);
}
static CYTHON_INLINE int __pyx_memview_set_int(char *itemp, PyObject *obj) {
    int value = __Pyx_PyLong_As_int(obj);
    if (unlikely((value == (int)-1) && PyErr_Occurred()))
        return 0;
    *(int *) itemp = value;
    return 1;
}

/* IsLittleEndian (used by BufferFormatCheck) */
static CYTHON_INLINE int __Pyx_Is_Little_Endian(void)
{
  union {
    uint32_t u32;
    uint8_t u8[4];
  } S;
  S.u32 = 0x01020304;
  return S.u8[0] == 4;
}

/* BufferFormatCheck (used by MemviewSliceValidateAndInit) */
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              const __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t <= '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case '?': return "'bool'";
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparsable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, int is_complex) {
  CYTHON_UNUSED_VAR(is_complex);
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, int is_complex) {
  CYTHON_UNUSED_VAR(is_complex);
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case '?': case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    const __Pyx_StructField* field = ctx->head->field;
    const __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    const __Pyx_StructField* field = ctx->head->field;
    const __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static int
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number, ndim;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return -1;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return -1;
    ndim = ctx->head->field->type->ndim;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return -1;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i]) {
            PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
            return -1;
        }
        if (*ts != ',' && *ts != ')') {
            PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
            return -1;
        }
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim) {
        PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
        return -1;
    }
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return -1;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return 0;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_Is_Little_Endian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
        CYTHON_FALLTHROUGH;
      case '?': case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if ((ctx->enc_type == *ts) && (got_Z == ctx->is_complex) &&
            (ctx->enc_packmode == ctx->new_packmode) && (!ctx->is_valid_array)) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
        CYTHON_FALLTHROUGH;
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (__pyx_buffmt_parse_array(ctx, &ts) < 0) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}

/* TypeInfoCompare (used by MemviewSliceValidateAndInit) */
  static int
  __pyx_typeinfo_cmp(const __Pyx_TypeInfo *a, const __Pyx_TypeInfo *b)
  {
      int i;
      if (!a || !b)
          return 0;
      if (a == b)
          return 1;
      if (a->size != b->size || a->typegroup != b->typegroup ||
              a->is_unsigned != b->is_unsigned || a->ndim != b->ndim) {
          if (a->typegroup == 'H' || b->typegroup == 'H') {
              return a->size == b->size;
          } else {
              return 0;
          }
      }
      if (a->ndim) {
          for (i = 0; i < a->ndim; i++)
              if (a->arraysize[i] != b->arraysize[i])
                  return 0;
      }
      if (a->typegroup == 'S') {
          if (a->flags != b->flags)
              return 0;
          if (a->fields || b->fields) {
              if (!(a->fields && b->fields))
                  return 0;
              for (i = 0; a->fields[i].type && b->fields[i].type; i++) {
                  const __Pyx_StructField *field_a = a->fields + i;
                  const __Pyx_StructField *field_b = b->fields + i;
                  if (field_a->offset != field_b->offset ||
                      !__pyx_typeinfo_cmp(field_a->type, field_b->type))
                      return 0;
              }
              return !a->fields[i].type && !b->fields[i].type;
          }
      }
      return 1;
  }
  
/* MemviewSliceValidateAndInit (used by ObjectToMemviewSlice) */
  static int
  __pyx_check_strides(Py_buffer *buf, int dim, int ndim, int spec)
  {
      if (buf->shape[dim] <= 1)
          return 1;
      if (buf->strides) {
          if (spec & __Pyx_MEMVIEW_CONTIG) {
              if (spec & (__Pyx_MEMVIEW_PTR|__Pyx_MEMVIEW_FULL)) {
                  if (unlikely(buf->strides[dim] != sizeof(void *))) {
                      PyErr_Format(PyExc_ValueError,
                                   "Buffer is not indirectly contiguous "
                                   "in dimension %d.", dim);
                      goto fail;
                  }
              } else if (unlikely(buf->strides[dim] != buf->itemsize)) {
                  PyErr_SetString(PyExc_ValueError,
                                  "Buffer and memoryview are not contiguous "
                                  "in the same dimension.");
                  goto fail;
              }
          }
          if (spec & __Pyx_MEMVIEW_FOLLOW) {
              Py_ssize_t stride = buf->strides[dim];
              if (stride < 0)
                  stride = -stride;
              if (unlikely(stride < buf->itemsize)) {
                  PyErr_SetString(PyExc_ValueError,
                                  "Buffer and memoryview are not contiguous "
                                  "in the same dimension.");
                  goto fail;
              }
          }
      } else {
          if (unlikely(spec & __Pyx_MEMVIEW_CONTIG && dim != ndim - 1)) {
              PyErr_Format(PyExc_ValueError,
                           "C-contiguous buffer is not contiguous in "
                           "dimension %d", dim);
              goto fail;
          } else if (unlikely(spec & (__Pyx_MEMVIEW_PTR))) {
              PyErr_Format(PyExc_ValueError,
                           "C-contiguous buffer is not indirect in "
                           "dimension %d", dim);
              goto fail;
          } else if (unlikely(buf->suboffsets)) {
              PyErr_SetString(PyExc_ValueError,
                              "Buffer exposes suboffsets but no strides");
              goto fail;
          }
      }
      return 1;
  fail:
      return 0;
  }
  static int
  __pyx_check_suboffsets(Py_buffer *buf, int dim, int ndim, int spec)
  {
      CYTHON_UNUSED_VAR(ndim);
      if (spec & __Pyx_MEMVIEW_DIRECT) {
          if (unlikely(buf->suboffsets && buf->suboffsets[dim] >= 0)) {
              PyErr_Format(PyExc_ValueError,
                           "Buffer not compatible with direct access "
                           "in dimension %d.", dim);
              goto fail;
          }
      }
      if (spec & __Pyx_MEMVIEW_PTR) {
          if (unlikely(!buf->suboffsets || (buf->suboffsets[dim] < 0))) {
              PyErr_Format(PyExc_ValueError,
                           "Buffer is not indirectly accessible "
                           "in dimension %d.", dim);
              goto fail;
          }
      }
      return 1;
  fail:
      return 0;
  }
  static int
  __pyx_verify_contig(Py_buffer *buf, int ndim, int c_or_f_flag)
  {
      int i;
      if (c_or_f_flag & __Pyx_IS_F_CONTIG) {
          Py_ssize_t stride = 1;
          for (i = 0; i < ndim; i++) {
              if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                  PyErr_SetString(PyExc_ValueError,
                      "Buffer not fortran contiguous.");
                  goto fail;
              }
              stride = stride * buf->shape[i];
          }
      } else if (c_or_f_flag & __Pyx_IS_C_CONTIG) {
          Py_ssize_t stride = 1;
          for (i = ndim - 1; i >- 1; i--) {
              if (unlikely(stride * buf->itemsize != buf->strides[i]  &&  buf->shape[i] > 1)) {
                  PyErr_SetString(PyExc_ValueError,
                      "Buffer not C contiguous.");
                  goto fail;
              }
              stride = stride * buf->shape[i];
          }
      }
      return 1;
  fail:
      return 0;
  }
  static int __Pyx_ValidateAndInit_memviewslice(
                  int *axes_specs,
                  int c_or_f_flag,
                  int buf_flags,
                  int ndim,
                  const __Pyx_TypeInfo *dtype,
                  __Pyx_BufFmt_StackElem stack[],
                  __Pyx_memviewslice *memviewslice,
                  PyObject *original_obj)
  {
      struct __pyx_memoryview_obj *memview, *new_memview;
      __Pyx_RefNannyDeclarations
      Py_buffer *buf;
      int i, spec = 0, retval = -1;
      __Pyx_BufFmt_Context ctx;
      int from_memoryview = __pyx_memoryview_check(original_obj);
      __Pyx_RefNannySetupContext("ValidateAndInit_memviewslice", 0);
      if (from_memoryview && __pyx_typeinfo_cmp(dtype, ((struct __pyx_memoryview_obj *)
                                                              original_obj)->typeinfo)) {
          memview = (struct __pyx_memoryview_obj *) original_obj;
          new_memview = NULL;
      } else {
          memview = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                              original_obj, buf_flags, 0, dtype);
          new_memview = memview;
          if (unlikely(!memview))
              goto fail;
      }
      buf = &memview->view;
      if (unlikely(buf->ndim != ndim)) {
          PyErr_Format(PyExc_ValueError,
                  "Buffer has wrong number of dimensions (expected %d, got %d)",
                  ndim, buf->ndim);
          goto fail;
      }
      if (new_memview) {
          __Pyx_BufFmt_Init(&ctx, stack, dtype);
          if (unlikely(!__Pyx_BufFmt_CheckString(&ctx, buf->format))) goto fail;
      }
      if (unlikely((unsigned) buf->itemsize != dtype->size)) {
          PyErr_Format(PyExc_ValueError,
                       "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "u byte%s) "
                       "does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "u byte%s)",
                       buf->itemsize,
                       (buf->itemsize > 1) ? "s" : "",
                       dtype->name,
                       dtype->size,
                       (dtype->size > 1) ? "s" : "");
          goto fail;
      }
      if (buf->len > 0) {
          for (i = 0; i < ndim; i++) {
              spec = axes_specs[i];
              if (unlikely(!__pyx_check_strides(buf, i, ndim, spec)))
                  goto fail;
              if (unlikely(!__pyx_check_suboffsets(buf, i, ndim, spec)))
                  goto fail;
          }
          if (unlikely(buf->strides && !__pyx_verify_contig(buf, ndim, c_or_f_flag)))
              goto fail;
      }
      if (unlikely(__Pyx_init_memviewslice(memview, ndim, memviewslice,
                                           new_memview != NULL) == -1)) {
          goto fail;
      }
      retval = 0;
      goto no_fail;
  fail:
      Py_XDECREF((PyObject*)new_memview);
      retval = -1;
  no_fail:
      __Pyx_RefNannyFinishContext();
      return retval;
  }
  
/* ObjectToMemviewSlice */
  static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_int(PyObject *obj, int writable_flag) {
      __Pyx_memviewslice result = __Pyx_MEMSLICE_INIT;
      __Pyx_BufFmt_StackElem stack[1];
      int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
      int retcode;
      if (obj == Py_None) {
          result.memview = (struct __pyx_memoryview_obj *) Py_None;
          return result;
      }
      retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                   PyBUF_RECORDS_RO | writable_flag, 1,
                                                   &__Pyx_TypeInfo_int, stack,
                                                   &result, obj);
      if (unlikely(retcode == -1))
          goto __pyx_fail;
      return result;
  __pyx_fail:
      result.memview = NULL;
      result.data = NULL;
      return result;
  }
  
/* ObjectToMemviewSlice */
  static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_object(PyObject *obj, int writable_flag) {
      __Pyx_memviewslice result = __Pyx_MEMSLICE_INIT;
      __Pyx_BufFmt_StackElem stack[1];
      int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
      int retcode;
      if (obj == Py_None) {
          result.memview = (struct __pyx_memoryview_obj *) Py_None;
          return result;
      }
      retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                   PyBUF_RECORDS_RO | writable_flag, 2,
                                                   &__Pyx_TypeInfo_object, stack,
                                                   &result, obj);
      if (unlikely(retcode == -1))
          goto __pyx_fail;
      return result;
  __pyx_fail:
      result.memview = NULL;
      result.data = NULL;
      return result;
  }
  
/* MemviewSliceCopy */
  static __Pyx_memviewslice
  __pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                   const char *mode, int ndim,
                                   size_t sizeof_dtype, int contig_flag,
                                   int dtype_is_object)
  {
      __Pyx_RefNannyDeclarations
      int i;
      __Pyx_memviewslice new_mvs = __Pyx_MEMSLICE_INIT;
      struct __pyx_memoryview_obj *from_memview = from_mvs->memview;
      Py_buffer *buf = &from_memview->view;
      PyObject *shape_tuple = NULL;
      struct __pyx_array_obj *array_obj = NULL;
      struct __pyx_memoryview_obj *memview_obj = NULL;
      __Pyx_RefNannySetupContext("__pyx_memoryview_copy_new_contig", 0);
      for (i = 0; i < ndim; i++) {
          if (unlikely(from_mvs->suboffsets[i] >= 0)) {
              PyErr_Format(PyExc_ValueError, "Cannot copy memoryview slice with "
                                             "indirect dimensions (axis %d)", i);
              goto fail;
          }
      }
      shape_tuple = PyTuple_New(ndim);
      if (unlikely(!shape_tuple)) {
          goto fail;
      }
      __Pyx_GOTREF(shape_tuple);
      for(i = 0; i < ndim; i++) {
          PyObject *temp_int = PyLong_FromSsize_t(from_mvs->shape[i]);
          if(unlikely(!temp_int)) {
              goto fail;
          } else {
  #if CYTHON_ASSUME_SAFE_MACROS
              PyTuple_SET_ITEM(shape_tuple, i, temp_int);
  #else
              if (PyTuple_SetItem(shape_tuple, i, temp_int) < 0) {
                  goto fail;
              }
  #endif
          }
      }
      array_obj = __pyx_array_new(shape_tuple, sizeof_dtype, buf->format, mode, NULL);
      if (unlikely(!array_obj)) {
          goto fail;
      }
      __Pyx_GOTREF(array_obj);
      memview_obj = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                      (PyObject *) array_obj, contig_flag,
                                      dtype_is_object,
                                      from_mvs->memview->typeinfo);
      if (unlikely(!memview_obj))
          goto fail;
      if (unlikely(__Pyx_init_memviewslice(memview_obj, ndim, &new_mvs, 1) < 0))
          goto fail;
      if (unlikely(__pyx_memoryview_copy_contents(*from_mvs, new_mvs, ndim, ndim,
                                                  dtype_is_object) < 0))
          goto fail;
      goto no_fail;
  fail:
      __Pyx_XDECREF((PyObject *) new_mvs.memview);
      new_mvs.memview = NULL;
      new_mvs.data = NULL;
  no_fail:
      __Pyx_XDECREF(shape_tuple);
      __Pyx_XDECREF((PyObject *) array_obj);
      __Pyx_RefNannyFinishContext();
      return new_mvs;
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_OGRFieldType(OGRFieldType value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const OGRFieldType neg_one = (OGRFieldType) -1, const_zero = (OGRFieldType) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(OGRFieldType) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(OGRFieldType) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(OGRFieldType) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(OGRFieldType) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(OGRFieldType) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(OGRFieldType),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(OGRFieldType));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_OGRFieldSubType(OGRFieldSubType value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const OGRFieldSubType neg_one = (OGRFieldSubType) -1, const_zero = (OGRFieldSubType) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(OGRFieldSubType) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(OGRFieldSubType) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(OGRFieldSubType) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(OGRFieldSubType) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(OGRFieldSubType) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(OGRFieldSubType),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(OGRFieldSubType));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const int neg_one = (int) -1, const_zero = (int) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          int val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (int) -1;
          val = __Pyx_PyLong_As_int(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                              return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                              return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                              return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (int) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(int) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                              return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                              return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                              return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                              return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                              return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                              return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(int) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
          } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          int val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyObject *v;
          PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
          int bits, remaining_bits, is_negative = 0;
          int chunk_size = (sizeof(long) < 8) ? 30 : 62;
          if (likely(PyLong_CheckExact(x))) {
              v = __Pyx_NewRef(x);
          } else {
              v = PyNumber_Long(x);
              if (unlikely(!v)) return (int) -1;
              assert(PyLong_CheckExact(v));
          }
          {
              int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
              if (unlikely(result < 0)) {
                  Py_DECREF(v);
                  return (int) -1;
              }
              is_negative = result == 1;
          }
          if (is_unsigned && unlikely(is_negative)) {
              Py_DECREF(v);
              goto raise_neg_overflow;
          } else if (is_negative) {
              stepval = PyNumber_Invert(v);
              Py_DECREF(v);
              if (unlikely(!stepval))
                  return (int) -1;
          } else {
              stepval = v;
          }
          v = NULL;
          val = (int) 0;
          mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
          shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
          for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
              PyObject *tmp, *digit;
              long idigit;
              digit = PyNumber_And(stepval, mask);
              if (unlikely(!digit)) goto done;
              idigit = PyLong_AsLong(digit);
              Py_DECREF(digit);
              if (unlikely(idigit < 0)) goto done;
              val |= ((int) idigit) << bits;
              tmp = PyNumber_Rshift(stepval, shift);
              if (unlikely(!tmp)) goto done;
              Py_DECREF(stepval); stepval = tmp;
          }
          Py_DECREF(shift); shift = NULL;
          Py_DECREF(mask); mask = NULL;
          {
              long idigit = PyLong_AsLong(stepval);
              if (unlikely(idigit < 0)) goto done;
              remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
              if (unlikely(idigit >= (1L << remaining_bits)))
                  goto raise_overflow;
              val |= ((int) idigit) << bits;
          }
          if (!is_unsigned) {
              if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                  goto raise_overflow;
              if (is_negative)
                  val = ~val;
          }
          ret = 0;
      done:
          Py_XDECREF(shift);
          Py_XDECREF(mask);
          Py_XDECREF(stepval);
  #endif
          if (unlikely(ret))
              return (int) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to int");
      return (int) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to int");
      return (int) -1;
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const int neg_one = (int) -1, const_zero = (int) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(int) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(int) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(int) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(int),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int64_t(int64_t value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(int64_t) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(int64_t) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(int64_t) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(int64_t) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(int64_t),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int64_t));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_GIntBig(GIntBig value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const GIntBig neg_one = (GIntBig) -1, const_zero = (GIntBig) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(GIntBig) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(GIntBig) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(GIntBig) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(GIntBig) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(GIntBig) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(GIntBig),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(GIntBig));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const long neg_one = (long) -1, const_zero = (long) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(long) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(long) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(long) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(long),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* CIntFromPy */
  static CYTHON_INLINE OGRFieldType __Pyx_PyLong_As_OGRFieldType(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const OGRFieldType neg_one = (OGRFieldType) -1, const_zero = (OGRFieldType) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          OGRFieldType val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (OGRFieldType) -1;
          val = __Pyx_PyLong_As_OGRFieldType(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(OGRFieldType, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(OGRFieldType) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) >= 2 * PyLong_SHIFT)) {
                              return (OGRFieldType) (((((OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(OGRFieldType) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) >= 3 * PyLong_SHIFT)) {
                              return (OGRFieldType) (((((((OGRFieldType)digits[2]) << PyLong_SHIFT) | (OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(OGRFieldType) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) >= 4 * PyLong_SHIFT)) {
                              return (OGRFieldType) (((((((((OGRFieldType)digits[3]) << PyLong_SHIFT) | (OGRFieldType)digits[2]) << PyLong_SHIFT) | (OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (OGRFieldType) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(OGRFieldType) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldType, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(OGRFieldType) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldType, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(OGRFieldType, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(OGRFieldType) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) - 1 > 2 * PyLong_SHIFT)) {
                              return (OGRFieldType) (((OGRFieldType)-1)*(((((OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(OGRFieldType) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) - 1 > 2 * PyLong_SHIFT)) {
                              return (OGRFieldType) ((((((OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(OGRFieldType) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) - 1 > 3 * PyLong_SHIFT)) {
                              return (OGRFieldType) (((OGRFieldType)-1)*(((((((OGRFieldType)digits[2]) << PyLong_SHIFT) | (OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(OGRFieldType) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) - 1 > 3 * PyLong_SHIFT)) {
                              return (OGRFieldType) ((((((((OGRFieldType)digits[2]) << PyLong_SHIFT) | (OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(OGRFieldType) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) - 1 > 4 * PyLong_SHIFT)) {
                              return (OGRFieldType) (((OGRFieldType)-1)*(((((((((OGRFieldType)digits[3]) << PyLong_SHIFT) | (OGRFieldType)digits[2]) << PyLong_SHIFT) | (OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(OGRFieldType) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldType) - 1 > 4 * PyLong_SHIFT)) {
                              return (OGRFieldType) ((((((((((OGRFieldType)digits[3]) << PyLong_SHIFT) | (OGRFieldType)digits[2]) << PyLong_SHIFT) | (OGRFieldType)digits[1]) << PyLong_SHIFT) | (OGRFieldType)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(OGRFieldType) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldType, long, PyLong_AsLong(x))
          } else if ((sizeof(OGRFieldType) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldType, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          OGRFieldType val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyErr_SetString(PyExc_RuntimeError,
                          "_PyLong_AsByteArray() or PyLong_AsNativeBytes() not available, cannot convert large enums");
          val = (OGRFieldType) -1;
  #endif
          if (unlikely(ret))
              return (OGRFieldType) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to OGRFieldType");
      return (OGRFieldType) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to OGRFieldType");
      return (OGRFieldType) -1;
  }
  
/* CIntFromPy */
  static CYTHON_INLINE OGRFieldSubType __Pyx_PyLong_As_OGRFieldSubType(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const OGRFieldSubType neg_one = (OGRFieldSubType) -1, const_zero = (OGRFieldSubType) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          OGRFieldSubType val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (OGRFieldSubType) -1;
          val = __Pyx_PyLong_As_OGRFieldSubType(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(OGRFieldSubType) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) >= 2 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) (((((OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(OGRFieldSubType) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) >= 3 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) (((((((OGRFieldSubType)digits[2]) << PyLong_SHIFT) | (OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(OGRFieldSubType) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) >= 4 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) (((((((((OGRFieldSubType)digits[3]) << PyLong_SHIFT) | (OGRFieldSubType)digits[2]) << PyLong_SHIFT) | (OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (OGRFieldSubType) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(OGRFieldSubType) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldSubType, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(OGRFieldSubType) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldSubType, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(OGRFieldSubType) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) - 1 > 2 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) (((OGRFieldSubType)-1)*(((((OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(OGRFieldSubType) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) - 1 > 2 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) ((((((OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(OGRFieldSubType) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) - 1 > 3 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) (((OGRFieldSubType)-1)*(((((((OGRFieldSubType)digits[2]) << PyLong_SHIFT) | (OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(OGRFieldSubType) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) - 1 > 3 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) ((((((((OGRFieldSubType)digits[2]) << PyLong_SHIFT) | (OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(OGRFieldSubType) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) - 1 > 4 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) (((OGRFieldSubType)-1)*(((((((((OGRFieldSubType)digits[3]) << PyLong_SHIFT) | (OGRFieldSubType)digits[2]) << PyLong_SHIFT) | (OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(OGRFieldSubType) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(OGRFieldSubType, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(OGRFieldSubType) - 1 > 4 * PyLong_SHIFT)) {
                              return (OGRFieldSubType) ((((((((((OGRFieldSubType)digits[3]) << PyLong_SHIFT) | (OGRFieldSubType)digits[2]) << PyLong_SHIFT) | (OGRFieldSubType)digits[1]) << PyLong_SHIFT) | (OGRFieldSubType)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(OGRFieldSubType) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldSubType, long, PyLong_AsLong(x))
          } else if ((sizeof(OGRFieldSubType) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(OGRFieldSubType, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          OGRFieldSubType val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyErr_SetString(PyExc_RuntimeError,
                          "_PyLong_AsByteArray() or PyLong_AsNativeBytes() not available, cannot convert large enums");
          val = (OGRFieldSubType) -1;
  #endif
          if (unlikely(ret))
              return (OGRFieldSubType) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to OGRFieldSubType");
      return (OGRFieldSubType) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to OGRFieldSubType");
      return (OGRFieldSubType) -1;
  }
  
/* CIntFromPy */
  static CYTHON_INLINE int64_t __Pyx_PyLong_As_int64_t(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          int64_t val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (int64_t) -1;
          val = __Pyx_PyLong_As_int64_t(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(int64_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(int64_t) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) >= 2 * PyLong_SHIFT)) {
                              return (int64_t) (((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(int64_t) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) >= 3 * PyLong_SHIFT)) {
                              return (int64_t) (((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(int64_t) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) >= 4 * PyLong_SHIFT)) {
                              return (int64_t) (((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (int64_t) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(int64_t) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(int64_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(int64_t) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                              return (int64_t) (((int64_t)-1)*(((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(int64_t) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                              return (int64_t) ((((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                              return (int64_t) (((int64_t)-1)*(((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(int64_t) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                              return (int64_t) ((((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT)) {
                              return (int64_t) (((int64_t)-1)*(((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(int64_t) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT)) {
                              return (int64_t) ((((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(int64_t) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(int64_t, long, PyLong_AsLong(x))
          } else if ((sizeof(int64_t) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(int64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          int64_t val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyObject *v;
          PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
          int bits, remaining_bits, is_negative = 0;
          int chunk_size = (sizeof(long) < 8) ? 30 : 62;
          if (likely(PyLong_CheckExact(x))) {
              v = __Pyx_NewRef(x);
          } else {
              v = PyNumber_Long(x);
              if (unlikely(!v)) return (int64_t) -1;
              assert(PyLong_CheckExact(v));
          }
          {
              int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
              if (unlikely(result < 0)) {
                  Py_DECREF(v);
                  return (int64_t) -1;
              }
              is_negative = result == 1;
          }
          if (is_unsigned && unlikely(is_negative)) {
              Py_DECREF(v);
              goto raise_neg_overflow;
          } else if (is_negative) {
              stepval = PyNumber_Invert(v);
              Py_DECREF(v);
              if (unlikely(!stepval))
                  return (int64_t) -1;
          } else {
              stepval = v;
          }
          v = NULL;
          val = (int64_t) 0;
          mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
          shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
          for (bits = 0; bits < (int) sizeof(int64_t) * 8 - chunk_size; bits += chunk_size) {
              PyObject *tmp, *digit;
              long idigit;
              digit = PyNumber_And(stepval, mask);
              if (unlikely(!digit)) goto done;
              idigit = PyLong_AsLong(digit);
              Py_DECREF(digit);
              if (unlikely(idigit < 0)) goto done;
              val |= ((int64_t) idigit) << bits;
              tmp = PyNumber_Rshift(stepval, shift);
              if (unlikely(!tmp)) goto done;
              Py_DECREF(stepval); stepval = tmp;
          }
          Py_DECREF(shift); shift = NULL;
          Py_DECREF(mask); mask = NULL;
          {
              long idigit = PyLong_AsLong(stepval);
              if (unlikely(idigit < 0)) goto done;
              remaining_bits = ((int) sizeof(int64_t) * 8) - bits - (is_unsigned ? 0 : 1);
              if (unlikely(idigit >= (1L << remaining_bits)))
                  goto raise_overflow;
              val |= ((int64_t) idigit) << bits;
          }
          if (!is_unsigned) {
              if (unlikely(val & (((int64_t) 1) << (sizeof(int64_t) * 8 - 1))))
                  goto raise_overflow;
              if (is_negative)
                  val = ~val;
          }
          ret = 0;
      done:
          Py_XDECREF(shift);
          Py_XDECREF(mask);
          Py_XDECREF(stepval);
  #endif
          if (unlikely(ret))
              return (int64_t) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to int64_t");
      return (int64_t) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to int64_t");
      return (int64_t) -1;
  }
  
/* PyObjectCallMethod1 (used by UpdateUnpickledDict) */
  #if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
  static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
      PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
      Py_DECREF(method);
      return result;
  }
  #endif
  static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
  #if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
      PyObject *args[2] = {obj, arg};
      (void) __Pyx_PyObject_CallOneArg;
      (void) __Pyx_PyObject_Call2Args;
      return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
  #else
      PyObject *method = NULL, *result;
      int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
      if (likely(is_method)) {
          result = __Pyx_PyObject_Call2Args(method, obj, arg);
          Py_DECREF(method);
          return result;
      }
      if (unlikely(!method)) return NULL;
      return __Pyx__PyObject_CallMethod1(method, arg);
  #endif
  }
  
/* UpdateUnpickledDict */
  static int __Pyx__UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
      PyObject *state_dict = __Pyx_PySequence_ITEM(state, index);
      if (unlikely(!state_dict)) {
          return -1;
      }
      int non_empty = PyObject_IsTrue(state_dict);
      if (non_empty == 0) {
          Py_DECREF(state_dict);
          return 0;
      } else if (unlikely(non_empty == -1)) {
          return -1;
      }
      PyObject *dict;
      #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
      dict = PyObject_GetAttrString(obj, "__dict__");
      #else
      dict = PyObject_GenericGetDict(obj, NULL);
      #endif
      if (unlikely(!dict)) {
          Py_DECREF(state_dict);
          return -1;
      }
      int result;
      if (likely(PyDict_CheckExact(dict))) {
          result = PyDict_Update(dict, state_dict);
      } else {
          PyObject *obj_result = __Pyx_PyObject_CallMethod1(dict, __pyx_mstate_global->__pyx_n_u_update, state_dict);
          if (likely(obj_result)) {
              Py_DECREF(obj_result);
              result = 0;
          } else {
              result = -1;
          }
      }
      Py_DECREF(state_dict);
      Py_DECREF(dict);
      return result;
  }
  static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
      Py_ssize_t state_size = __Pyx_PyTuple_GET_SIZE(state);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(state_size == -1)) return -1;
      #endif
      if (state_size <= index) {
          return 0;
      }
      return __Pyx__UpdateUnpickledDict(obj, state, index);
  }
  
/* CheckUnpickleChecksum */
  static void __Pyx_RaiseUnpickleChecksumError(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
      PyObject *pickle_module = PyImport_ImportModule("pickle");
      if (unlikely(!pickle_module)) return;
      PyObject *pickle_error = PyObject_GetAttrString(pickle_module, "PickleError");
      Py_DECREF(pickle_module);
      if (unlikely(!pickle_error)) return;
      if (checksum2 == checksum1) {
          PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x) = (%s))",
              checksum, checksum1, members);
      } else if (checksum3 == checksum2) {
          PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x) = (%s))",
              checksum, checksum1, checksum2, members);
      } else {
          PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x, 0x%x) = (%s))",
              checksum, checksum1, checksum2, checksum3, members);
      }
      Py_DECREF(pickle_error);
  }
  static int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
      int found = 0;
      found |= checksum1 == checksum;
      found |= checksum2 == checksum;
      found |= checksum3 == checksum;
      if (likely(found))
          return 0;
      __Pyx_RaiseUnpickleChecksumError(checksum, checksum1, checksum2, checksum3, members);
      return -1;
  }
  
/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const long neg_one = (long) -1, const_zero = (long) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          long val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (long) -1;
          val = __Pyx_PyLong_As_long(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                              return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                              return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                              return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (long) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(long) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                              return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                              return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                              return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                              return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                              return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                              return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(long) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
          } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          long val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyObject *v;
          PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
          int bits, remaining_bits, is_negative = 0;
          int chunk_size = (sizeof(long) < 8) ? 30 : 62;
          if (likely(PyLong_CheckExact(x))) {
              v = __Pyx_NewRef(x);
          } else {
              v = PyNumber_Long(x);
              if (unlikely(!v)) return (long) -1;
              assert(PyLong_CheckExact(v));
          }
          {
              int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
              if (unlikely(result < 0)) {
                  Py_DECREF(v);
                  return (long) -1;
              }
              is_negative = result == 1;
          }
          if (is_unsigned && unlikely(is_negative)) {
              Py_DECREF(v);
              goto raise_neg_overflow;
          } else if (is_negative) {
              stepval = PyNumber_Invert(v);
              Py_DECREF(v);
              if (unlikely(!stepval))
                  return (long) -1;
          } else {
              stepval = v;
          }
          v = NULL;
          val = (long) 0;
          mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
          shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
          for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
              PyObject *tmp, *digit;
              long idigit;
              digit = PyNumber_And(stepval, mask);
              if (unlikely(!digit)) goto done;
              idigit = PyLong_AsLong(digit);
              Py_DECREF(digit);
              if (unlikely(idigit < 0)) goto done;
              val |= ((long) idigit) << bits;
              tmp = PyNumber_Rshift(stepval, shift);
              if (unlikely(!tmp)) goto done;
              Py_DECREF(stepval); stepval = tmp;
          }
          Py_DECREF(shift); shift = NULL;
          Py_DECREF(mask); mask = NULL;
          {
              long idigit = PyLong_AsLong(stepval);
              if (unlikely(idigit < 0)) goto done;
              remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
              if (unlikely(idigit >= (1L << remaining_bits)))
                  goto raise_overflow;
              val |= ((long) idigit) << bits;
          }
          if (!is_unsigned) {
              if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                  goto raise_overflow;
              if (is_negative)
                  val = ~val;
          }
          ret = 0;
      done:
          Py_XDECREF(shift);
          Py_XDECREF(mask);
          Py_XDECREF(stepval);
  #endif
          if (unlikely(ret))
              return (long) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to long");
      return (long) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to long");
      return (long) -1;
  }
  
/* CIntFromPy */
  static CYTHON_INLINE char __Pyx_PyLong_As_char(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const char neg_one = (char) -1, const_zero = (char) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          char val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (char) -1;
          val = __Pyx_PyLong_As_char(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(char, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(char) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) >= 2 * PyLong_SHIFT)) {
                              return (char) (((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(char) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) >= 3 * PyLong_SHIFT)) {
                              return (char) (((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(char) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) >= 4 * PyLong_SHIFT)) {
                              return (char) (((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (char) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(char) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(char, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(char) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(char, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(char) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) - 1 > 2 * PyLong_SHIFT)) {
                              return (char) (((char)-1)*(((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(char) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) - 1 > 2 * PyLong_SHIFT)) {
                              return (char) ((((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(char) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) - 1 > 3 * PyLong_SHIFT)) {
                              return (char) (((char)-1)*(((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(char) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) - 1 > 3 * PyLong_SHIFT)) {
                              return (char) ((((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(char) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) - 1 > 4 * PyLong_SHIFT)) {
                              return (char) (((char)-1)*(((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(char) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(char) - 1 > 4 * PyLong_SHIFT)) {
                              return (char) ((((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(char) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(char, long, PyLong_AsLong(x))
          } else if ((sizeof(char) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(char, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          char val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyObject *v;
          PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
          int bits, remaining_bits, is_negative = 0;
          int chunk_size = (sizeof(long) < 8) ? 30 : 62;
          if (likely(PyLong_CheckExact(x))) {
              v = __Pyx_NewRef(x);
          } else {
              v = PyNumber_Long(x);
              if (unlikely(!v)) return (char) -1;
              assert(PyLong_CheckExact(v));
          }
          {
              int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
              if (unlikely(result < 0)) {
                  Py_DECREF(v);
                  return (char) -1;
              }
              is_negative = result == 1;
          }
          if (is_unsigned && unlikely(is_negative)) {
              Py_DECREF(v);
              goto raise_neg_overflow;
          } else if (is_negative) {
              stepval = PyNumber_Invert(v);
              Py_DECREF(v);
              if (unlikely(!stepval))
                  return (char) -1;
          } else {
              stepval = v;
          }
          v = NULL;
          val = (char) 0;
          mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
          shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
          for (bits = 0; bits < (int) sizeof(char) * 8 - chunk_size; bits += chunk_size) {
              PyObject *tmp, *digit;
              long idigit;
              digit = PyNumber_And(stepval, mask);
              if (unlikely(!digit)) goto done;
              idigit = PyLong_AsLong(digit);
              Py_DECREF(digit);
              if (unlikely(idigit < 0)) goto done;
              val |= ((char) idigit) << bits;
              tmp = PyNumber_Rshift(stepval, shift);
              if (unlikely(!tmp)) goto done;
              Py_DECREF(stepval); stepval = tmp;
          }
          Py_DECREF(shift); shift = NULL;
          Py_DECREF(mask); mask = NULL;
          {
              long idigit = PyLong_AsLong(stepval);
              if (unlikely(idigit < 0)) goto done;
              remaining_bits = ((int) sizeof(char) * 8) - bits - (is_unsigned ? 0 : 1);
              if (unlikely(idigit >= (1L << remaining_bits)))
                  goto raise_overflow;
              val |= ((char) idigit) << bits;
          }
          if (!is_unsigned) {
              if (unlikely(val & (((char) 1) << (sizeof(char) * 8 - 1))))
                  goto raise_overflow;
              if (is_negative)
                  val = ~val;
          }
          ret = 0;
      done:
          Py_XDECREF(shift);
          Py_XDECREF(mask);
          Py_XDECREF(stepval);
  #endif
          if (unlikely(ret))
              return (char) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to char");
      return (char) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to char");
      return (char) -1;
  }
  
/* FormatTypeName */
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
  static __Pyx_TypeName
  __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
  {
      PyObject *module = NULL, *name = NULL, *result = NULL;
      #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
      name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                                 __pyx_mstate_global->__pyx_n_u_qualname);
      #else
      name = PyType_GetQualName(tp);
      #endif
      if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
      module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                                 __pyx_mstate_global->__pyx_n_u_module);
      if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
      if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
          result = name;
          name = NULL;
          goto done;
      }
      result = PyUnicode_FromFormat("%U.%U", module, name);
      if (unlikely(result == NULL)) goto bad;
    done:
      Py_XDECREF(name);
      Py_XDECREF(module);
      return result;
    bad:
      PyErr_Clear();
      if (name) {
          result = name;
          name = NULL;
      } else {
          result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__16);
      }
      goto done;
  }
  #endif
  
/* GetRuntimeVersion */
  #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
  void __Pyx_init_runtime_version(void) {
      if (__Pyx_cached_runtime_version == 0) {
          const char* rt_version = Py_GetVersion();
          unsigned long version = 0;
          unsigned long factor = 0x01000000UL;
          unsigned int digit = 0;
          int i = 0;
          while (factor) {
              while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                  digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                  ++i;
              }
              version += factor * digit;
              if (rt_version[i] != '.')
                  break;
              digit = 0;
              factor >>= 8;
              ++i;
          }
          __Pyx_cached_runtime_version = version;
      }
  }
  #endif
  static unsigned long __Pyx_get_runtime_version(void) {
  #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
      return Py_Version & ~0xFFUL;
  #else
      return __Pyx_cached_runtime_version;
  #endif
  }
  
/* IterNextPlain (used by CoroutineBase) */
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
  static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void) {
      if (unlikely(!__pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache))
          __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache = __Pyx_GetBuiltinName(__pyx_mstate_global->__pyx_n_u_next);
      return __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache;
  }
  #endif
  static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator) {
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
      PyObject *result;
      PyObject *next = __Pyx_GetBuiltinNext_LimitedAPI();
      if (unlikely(!next)) return NULL;
      result = PyObject_CallFunctionObjArgs(next, iterator, NULL);
      return result;
  #else
      (void)__Pyx_GetBuiltinName; // only for early limited API
      iternextfunc iternext = __Pyx_PyObject_GetIterNextFunc(iterator);
      assert(iternext);
      return iternext(iterator);
  #endif
  }
  
/* ReturnWithStopIteration (used by CoroutineBase) */
  static void __Pyx__ReturnWithStopIteration(PyObject* value, int async);
  static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext) {
      if (value == Py_None) {
          if (async || !iternext)
              PyErr_SetNone(async ? PyExc_StopAsyncIteration : PyExc_StopIteration);
          return;
      }
      __Pyx__ReturnWithStopIteration(value, async);
  }
  static void __Pyx__ReturnWithStopIteration(PyObject* value, int async) {
  #if CYTHON_COMPILING_IN_CPYTHON
      __Pyx_PyThreadState_declare
  #endif
      PyObject *exc;
      PyObject *exc_type = async ? PyExc_StopAsyncIteration : PyExc_StopIteration;
  #if CYTHON_COMPILING_IN_CPYTHON
      if ((PY_VERSION_HEX >= (0x030C00A6)) || unlikely(PyTuple_Check(value) || PyExceptionInstance_Check(value))) {
          if (PY_VERSION_HEX >= (0x030e00A1)) {
              exc = __Pyx_PyObject_CallOneArg(exc_type, value);
          } else {
              PyObject *args_tuple = PyTuple_New(1);
              if (unlikely(!args_tuple)) return;
              Py_INCREF(value);
              PyTuple_SET_ITEM(args_tuple, 0, value);
              exc = PyObject_Call(exc_type, args_tuple, NULL);
              Py_DECREF(args_tuple);
          }
          if (unlikely(!exc)) return;
      } else {
          Py_INCREF(value);
          exc = value;
      }
      #if CYTHON_FAST_THREAD_STATE
      __Pyx_PyThreadState_assign
      #if CYTHON_USE_EXC_INFO_STACK
      if (!__pyx_tstate->exc_info->exc_value)
      #else
      if (!__pyx_tstate->exc_type)
      #endif
      {
          Py_INCREF(exc_type);
          __Pyx_ErrRestore(exc_type, exc, NULL);
          return;
      }
      #endif
  #else
      exc = __Pyx_PyObject_CallOneArg(exc_type, value);
      if (unlikely(!exc)) return;
  #endif
      PyErr_SetObject(exc_type, exc);
      Py_DECREF(exc);
  }
  
/* CoroutineBase (used by Generator) */
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #include <frameobject.h>
  #if PY_VERSION_HEX >= 0x030b00a6 && !defined(PYPY_VERSION)
    #ifndef Py_BUILD_CORE
      #define Py_BUILD_CORE 1
    #endif
    #include "internal/pycore_frame.h"
  #endif
  #endif // CYTHON_COMPILING_IN_LIMITED_API
  static CYTHON_INLINE void
  __Pyx_Coroutine_Undelegate(__pyx_CoroutineObject *gen) {
  #if CYTHON_USE_AM_SEND
      gen->yieldfrom_am_send = NULL;
  #endif
      Py_CLEAR(gen->yieldfrom);
  }
  static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
      PyObject *et, *ev, *tb;
      PyObject *value = NULL;
      CYTHON_UNUSED_VAR(__pyx_tstate);
      __Pyx_ErrFetch(&et, &ev, &tb);
      if (!et) {
          Py_XDECREF(tb);
          Py_XDECREF(ev);
          Py_INCREF(Py_None);
          *pvalue = Py_None;
          return 0;
      }
      if (likely(et == PyExc_StopIteration)) {
          if (!ev) {
              Py_INCREF(Py_None);
              value = Py_None;
          }
          else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
              #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
              value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
              if (unlikely(!value)) goto limited_api_failure;
              #else
              value = ((PyStopIterationObject *)ev)->value;
              Py_INCREF(value);
              #endif
              Py_DECREF(ev);
          }
          else if (unlikely(PyTuple_Check(ev))) {
              Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(ev);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely(tuple_size < 0)) {
                  Py_XDECREF(tb);
                  Py_DECREF(ev);
                  Py_DECREF(et);
                  return -1;
              }
              #endif
              if (tuple_size >= 1) {
  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  value = PyTuple_GET_ITEM(ev, 0);
                  Py_INCREF(value);
  #elif CYTHON_ASSUME_SAFE_MACROS
                  value = PySequence_ITEM(ev, 0);
  #else
                  value = PySequence_GetItem(ev, 0);
                  if (!value) goto limited_api_failure;
  #endif
              } else {
                  Py_INCREF(Py_None);
                  value = Py_None;
              }
              Py_DECREF(ev);
          }
          else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
              value = ev;
          }
          if (likely(value)) {
              Py_XDECREF(tb);
              Py_DECREF(et);
              *pvalue = value;
              return 0;
          }
      } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
          __Pyx_ErrRestore(et, ev, tb);
          return -1;
      }
      PyErr_NormalizeException(&et, &ev, &tb);
      if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
          __Pyx_ErrRestore(et, ev, tb);
          return -1;
      }
      Py_XDECREF(tb);
      Py_DECREF(et);
  #if CYTHON_COMPILING_IN_LIMITED_API
      value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
  #else
      value = ((PyStopIterationObject *)ev)->value;
      Py_INCREF(value);
  #endif
      Py_DECREF(ev);
  #if CYTHON_COMPILING_IN_LIMITED_API
      if (unlikely(!value)) return -1;
  #endif
      *pvalue = value;
      return 0;
  #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL || !CYTHON_ASSUME_SAFE_MACROS
    limited_api_failure:
      Py_XDECREF(et);
      Py_XDECREF(tb);
      Py_XDECREF(ev);
      return -1;
  #endif
  }
  static CYTHON_INLINE
  __Pyx_PySendResult __Pyx_Coroutine_status_from_result(PyObject **retval) {
      if (*retval) {
          return PYGEN_NEXT;
      } else if (likely(__Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, retval) == 0)) {
          return PYGEN_RETURN;
      } else {
          return PYGEN_ERROR;
      }
  }
  static CYTHON_INLINE
  void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
  #if PY_VERSION_HEX >= 0x030B00a4
      Py_CLEAR(exc_state->exc_value);
  #else
      PyObject *t, *v, *tb;
      t = exc_state->exc_type;
      v = exc_state->exc_value;
      tb = exc_state->exc_traceback;
      exc_state->exc_type = NULL;
      exc_state->exc_value = NULL;
      exc_state->exc_traceback = NULL;
      Py_XDECREF(t);
      Py_XDECREF(v);
      Py_XDECREF(tb);
  #endif
  }
  #define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
  static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
      const char *msg;
      CYTHON_MAYBE_UNUSED_VAR(gen);
      if ((0)) {
      #ifdef __Pyx_Coroutine_USED
      } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
          msg = "coroutine already executing";
      #endif
      #ifdef __Pyx_AsyncGen_USED
      } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
          msg = "async generator already executing";
      #endif
      } else {
          msg = "generator already executing";
      }
      PyErr_SetString(PyExc_ValueError, msg);
  }
  static void __Pyx_Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
      CYTHON_MAYBE_UNUSED_VAR(gen);
      CYTHON_MAYBE_UNUSED_VAR(closing);
      #ifdef __Pyx_Coroutine_USED
      if (!closing && __Pyx_Coroutine_Check(gen)) {
          PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
      } else
      #endif
      if (value) {
          #ifdef __Pyx_AsyncGen_USED
          if (__Pyx_AsyncGen_CheckExact(gen))
              PyErr_SetNone(PyExc_StopAsyncIteration);
          else
          #endif
          PyErr_SetNone(PyExc_StopIteration);
      }
  }
  static
  __Pyx_PySendResult __Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, PyObject **result, int closing) {
      __Pyx_PyThreadState_declare
      PyThreadState *tstate;
      __Pyx_ExcInfoStruct *exc_state;
      PyObject *retval;
      assert(__Pyx_Coroutine_get_is_running(self));  // Callers should ensure is_running
      if (unlikely(self->resume_label == -1)) {
          __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
          return PYGEN_ERROR;
      }
  #if CYTHON_FAST_THREAD_STATE
      __Pyx_PyThreadState_assign
      tstate = __pyx_tstate;
  #else
      tstate = __Pyx_PyThreadState_Current;
  #endif
      exc_state = &self->gi_exc_state;
      if (exc_state->exc_value) {
          #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
          #else
          PyObject *exc_tb;
          #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
          exc_tb = PyException_GetTraceback(exc_state->exc_value);
          #elif PY_VERSION_HEX >= 0x030B00a4
          exc_tb = ((PyBaseExceptionObject*) exc_state->exc_value)->traceback;
          #else
          exc_tb = exc_state->exc_traceback;
          #endif
          if (exc_tb) {
              PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
              PyFrameObject *f = tb->tb_frame;
              assert(f->f_back == NULL);
              #if PY_VERSION_HEX >= 0x030B00A1
              f->f_back = PyThreadState_GetFrame(tstate);
              #else
              Py_XINCREF(tstate->frame);
              f->f_back = tstate->frame;
              #endif
              #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
              Py_DECREF(exc_tb);
              #endif
          }
          #endif
      }
  #if CYTHON_USE_EXC_INFO_STACK
      exc_state->previous_item = tstate->exc_info;
      tstate->exc_info = exc_state;
  #else
      if (exc_state->exc_type) {
          __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
      } else {
          __Pyx_Coroutine_ExceptionClear(exc_state);
          __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
      }
  #endif
      retval = self->body(self, tstate, value);
  #if CYTHON_USE_EXC_INFO_STACK
      exc_state = &self->gi_exc_state;
      tstate->exc_info = exc_state->previous_item;
      exc_state->previous_item = NULL;
      __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
  #endif
      *result = retval;
      if (self->resume_label == -1) {
          return likely(retval) ? PYGEN_RETURN : PYGEN_ERROR;
      }
      return PYGEN_NEXT;
  }
  static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
  #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
      CYTHON_UNUSED_VAR(exc_state);
  #else
      PyObject *exc_tb;
      #if PY_VERSION_HEX >= 0x030B00a4
      if (!exc_state->exc_value) return;
      exc_tb = PyException_GetTraceback(exc_state->exc_value);
      #else
      exc_tb = exc_state->exc_traceback;
      #endif
      if (likely(exc_tb)) {
          PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
          PyFrameObject *f = tb->tb_frame;
          Py_CLEAR(f->f_back);
          #if PY_VERSION_HEX >= 0x030B00a4
          Py_DECREF(exc_tb);
          #endif
      }
  #endif
  }
  #define __Pyx_Coroutine_MethodReturnFromResult(gen, result, retval, iternext)\
      ((result) == PYGEN_NEXT ? (retval) : __Pyx__Coroutine_MethodReturnFromResult(gen, result, retval, iternext))
  static PyObject *
  __Pyx__Coroutine_MethodReturnFromResult(PyObject* gen, __Pyx_PySendResult result, PyObject *retval, int iternext) {
      CYTHON_MAYBE_UNUSED_VAR(gen);
      if (likely(result == PYGEN_RETURN)) {
          int is_async = 0;
          #ifdef __Pyx_AsyncGen_USED
          is_async = __Pyx_AsyncGen_CheckExact(gen);
          #endif
          __Pyx_ReturnWithStopIteration(retval, is_async, iternext);
          Py_XDECREF(retval);
      }
      return NULL;
  }
  #if CYTHON_COMPILING_IN_CPYTHON
  static CYTHON_INLINE
  PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
  #if PY_VERSION_HEX <= 0x030A00A1
      return _PyGen_Send(gen, arg);
  #else
      PyObject *result;
      if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
          if (PyAsyncGen_CheckExact(gen)) {
              assert(result == Py_None);
              PyErr_SetNone(PyExc_StopAsyncIteration);
          }
          else if (result == Py_None) {
              PyErr_SetNone(PyExc_StopIteration);
          }
          else {
  #if PY_VERSION_HEX < 0x030d00A1
              _PyGen_SetStopIterationValue(result);
  #else
              if (!PyTuple_Check(result) && !PyExceptionInstance_Check(result)) {
                  PyErr_SetObject(PyExc_StopIteration, result);
              } else {
                  PyObject *exc = __Pyx_PyObject_CallOneArg(PyExc_StopIteration, result);
                  if (likely(exc != NULL)) {
                      PyErr_SetObject(PyExc_StopIteration, exc);
                      Py_DECREF(exc);
                  }
              }
  #endif
          }
          Py_DECREF(result);
          result = NULL;
      }
      return result;
  #endif
  }
  #endif
  static CYTHON_INLINE __Pyx_PySendResult
  __Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen, PyObject** retval) {
      __Pyx_PySendResult result;
      PyObject *val = NULL;
      assert(__Pyx_Coroutine_get_is_running(gen));
      __Pyx_Coroutine_Undelegate(gen);
      __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
      result = __Pyx_Coroutine_SendEx(gen, val, retval, 0);
      Py_XDECREF(val);
      return result;
  }
  #if CYTHON_USE_AM_SEND
  static __Pyx_PySendResult
  __Pyx_Coroutine_SendToDelegate(__pyx_CoroutineObject *gen, __Pyx_pyiter_sendfunc gen_am_send, PyObject *value, PyObject **retval) {
      PyObject *ret = NULL;
      __Pyx_PySendResult delegate_result, result;
      assert(__Pyx_Coroutine_get_is_running(gen));
      delegate_result = gen_am_send(gen->yieldfrom, value, &ret);
      if (delegate_result == PYGEN_NEXT) {
          assert (ret != NULL);
          *retval = ret;
          return PYGEN_NEXT;
      }
      assert (delegate_result != PYGEN_ERROR || ret == NULL);
      __Pyx_Coroutine_Undelegate(gen);
      result = __Pyx_Coroutine_SendEx(gen, ret, retval, 0);
      Py_XDECREF(ret);
      return result;
  }
  #endif
  static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
      PyObject *retval = NULL;
      __Pyx_PySendResult result = __Pyx_Coroutine_AmSend(self, value, &retval);
      return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
  }
  static __Pyx_PySendResult
  __Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval) {
      __Pyx_PySendResult result;
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
      if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
          *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
          return PYGEN_ERROR;
      }
      #if CYTHON_USE_AM_SEND
      if (gen->yieldfrom_am_send) {
          result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, value, retval);
      } else
      #endif
      if (gen->yieldfrom) {
          PyObject *yf = gen->yieldfrom;
          PyObject *ret;
        #if !CYTHON_USE_AM_SEND
          #ifdef __Pyx_Generator_USED
          if (__Pyx_Generator_CheckExact(yf)) {
              ret = __Pyx_Coroutine_Send(yf, value);
          } else
          #endif
          #ifdef __Pyx_Coroutine_USED
          if (__Pyx_Coroutine_Check(yf)) {
              ret = __Pyx_Coroutine_Send(yf, value);
          } else
          #endif
          #ifdef __Pyx_AsyncGen_USED
          if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
              ret = __Pyx_async_gen_asend_send(yf, value);
          } else
          #endif
          #if CYTHON_COMPILING_IN_CPYTHON
          if (PyGen_CheckExact(yf)) {
              ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
          } else
          if (PyCoro_CheckExact(yf)) {
              ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
          } else
          #endif
        #endif
          {
              #if !CYTHON_COMPILING_IN_LIMITED_API || __PYX_LIMITED_VERSION_HEX >= 0x03080000
              if (value == Py_None && PyIter_Check(yf))
                  ret = __Pyx_PyIter_Next_Plain(yf);
              else
              #endif
                  ret = __Pyx_PyObject_CallMethod1(yf, __pyx_mstate_global->__pyx_n_u_send, value);
          }
          if (likely(ret)) {
              __Pyx_Coroutine_unset_is_running(gen);
              *retval = ret;
              return PYGEN_NEXT;
          }
          result = __Pyx_Coroutine_FinishDelegation(gen, retval);
      } else {
          result = __Pyx_Coroutine_SendEx(gen, value, retval, 0);
      }
      __Pyx_Coroutine_unset_is_running(gen);
      return result;
  }
  static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
      __Pyx_PySendResult result;
      PyObject *retval = NULL;
      CYTHON_UNUSED_VAR(gen);
      assert(__Pyx_Coroutine_get_is_running(gen));
      #ifdef __Pyx_Generator_USED
      if (__Pyx_Generator_CheckExact(yf)) {
          result = __Pyx_Coroutine_Close(yf, &retval);
      } else
      #endif
      #ifdef __Pyx_Coroutine_USED
      if (__Pyx_Coroutine_Check(yf)) {
          result = __Pyx_Coroutine_Close(yf, &retval);
      } else
      if (__Pyx_CoroutineAwait_CheckExact(yf)) {
          result = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf);
      } else
      #endif
      #ifdef __Pyx_AsyncGen_USED
      if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
          retval = __Pyx_async_gen_asend_close(yf, NULL);
          result = PYGEN_RETURN;
      } else
      if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
          retval = __Pyx_async_gen_athrow_close(yf, NULL);
          result = PYGEN_RETURN;
      } else
      #endif
      {
          PyObject *meth;
          result = PYGEN_RETURN;
          meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_close);
          if (unlikely(!meth)) {
              if (unlikely(PyErr_Occurred())) {
                  PyErr_WriteUnraisable(yf);
              }
          } else {
              retval = __Pyx_PyObject_CallNoArg(meth);
              Py_DECREF(meth);
              if (unlikely(!retval)) {
                  result = PYGEN_ERROR;
              }
          }
      }
      Py_XDECREF(retval);
      return result == PYGEN_ERROR ? -1 : 0;
  }
  static PyObject *__Pyx_Generator_Next(PyObject *self) {
      __Pyx_PySendResult result;
      PyObject *retval = NULL;
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
      if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
          return __Pyx_Coroutine_AlreadyRunningError(gen);
      }
      #if CYTHON_USE_AM_SEND
      if (gen->yieldfrom_am_send) {
          result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, Py_None, &retval);
      } else
      #endif
      if (gen->yieldfrom) {
          PyObject *yf = gen->yieldfrom;
          PyObject *ret;
          #ifdef __Pyx_Generator_USED
          if (__Pyx_Generator_CheckExact(yf)) {
              ret = __Pyx_Generator_Next(yf);
          } else
          #endif
          #ifdef __Pyx_Coroutine_USED
          if (__Pyx_Coroutine_CheckExact(yf)) {
              ret = __Pyx_Coroutine_Send(yf, Py_None);
          } else
          #endif
          #if CYTHON_COMPILING_IN_CPYTHON && (PY_VERSION_HEX < 0x030A00A3 || !CYTHON_USE_AM_SEND)
          if (PyGen_CheckExact(yf)) {
              ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
          } else
          #endif
              ret = __Pyx_PyIter_Next_Plain(yf);
          if (likely(ret)) {
              __Pyx_Coroutine_unset_is_running(gen);
              return ret;
          }
          result = __Pyx_Coroutine_FinishDelegation(gen, &retval);
      } else {
          result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
      }
      __Pyx_Coroutine_unset_is_running(gen);
      return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 1);
  }
  static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
      PyObject *retval = NULL;
      __Pyx_PySendResult result;
      CYTHON_UNUSED_VAR(arg);
      result = __Pyx_Coroutine_Close(self, &retval);
      if (unlikely(result == PYGEN_ERROR))
          return NULL;
      Py_XDECREF(retval);
      Py_RETURN_NONE;
  }
  static __Pyx_PySendResult
  __Pyx_Coroutine_Close(PyObject *self, PyObject **retval) {
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
      __Pyx_PySendResult result;
      PyObject *yf;
      int err = 0;
      if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
          *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
          return PYGEN_ERROR;
      }
      yf = gen->yieldfrom;
      if (yf) {
          Py_INCREF(yf);
          err = __Pyx_Coroutine_CloseIter(gen, yf);
          __Pyx_Coroutine_Undelegate(gen);
          Py_DECREF(yf);
      }
      if (err == 0)
          PyErr_SetNone(PyExc_GeneratorExit);
      result = __Pyx_Coroutine_SendEx(gen, NULL, retval, 1);
      if (result == PYGEN_ERROR) {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_Coroutine_unset_is_running(gen);
          if (!__Pyx_PyErr_Occurred()) {
              return PYGEN_RETURN;
          } else if (likely(__Pyx_PyErr_ExceptionMatches2(PyExc_GeneratorExit, PyExc_StopIteration))) {
              __Pyx_PyErr_Clear();
              return PYGEN_RETURN;
          }
          return PYGEN_ERROR;
      } else if (likely(result == PYGEN_RETURN && *retval == Py_None)) {
          __Pyx_Coroutine_unset_is_running(gen);
          return PYGEN_RETURN;
      } else {
          const char *msg;
          Py_DECREF(*retval);
          *retval = NULL;
          if ((0)) {
          #ifdef __Pyx_Coroutine_USED
          } else if (__Pyx_Coroutine_Check(self)) {
              msg = "coroutine ignored GeneratorExit";
          #endif
          #ifdef __Pyx_AsyncGen_USED
          } else if (__Pyx_AsyncGen_CheckExact(self)) {
              msg = "async generator ignored GeneratorExit";
          #endif
          } else {
              msg = "generator ignored GeneratorExit";
          }
          PyErr_SetString(PyExc_RuntimeError, msg);
          __Pyx_Coroutine_unset_is_running(gen);
          return PYGEN_ERROR;
      }
  }
  static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                          PyObject *args, int close_on_genexit) {
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
      PyObject *yf;
      if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen)))
          return __Pyx_Coroutine_AlreadyRunningError(gen);
      yf = gen->yieldfrom;
      if (yf) {
          __Pyx_PySendResult result;
          PyObject *ret;
          Py_INCREF(yf);
          if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
              int err = __Pyx_Coroutine_CloseIter(gen, yf);
              Py_DECREF(yf);
              __Pyx_Coroutine_Undelegate(gen);
              if (err < 0)
                  goto propagate_exception;
              goto throw_here;
          }
          if (0
          #ifdef __Pyx_Generator_USED
              || __Pyx_Generator_CheckExact(yf)
          #endif
          #ifdef __Pyx_Coroutine_USED
              || __Pyx_Coroutine_Check(yf)
          #endif
              ) {
              ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
          #ifdef __Pyx_Coroutine_USED
          } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
              ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
          #endif
          } else {
              PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_throw);
              if (unlikely(!meth)) {
                  Py_DECREF(yf);
                  if (unlikely(PyErr_Occurred())) {
                      __Pyx_Coroutine_unset_is_running(gen);
                      return NULL;
                  }
                  __Pyx_Coroutine_Undelegate(gen);
                  goto throw_here;
              }
              if (likely(args)) {
                  ret = __Pyx_PyObject_Call(meth, args, NULL);
              } else {
                  PyObject *cargs[4] = {NULL, typ, val, tb};
                  size_t nargs = 1U + (val != NULL) + (tb != NULL);
                  ret = __Pyx_PyObject_FastCall(meth, cargs+1, nargs | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
              }
              Py_DECREF(meth);
          }
          Py_DECREF(yf);
          if (ret) {
              __Pyx_Coroutine_unset_is_running(gen);
              return ret;
          }
          result = __Pyx_Coroutine_FinishDelegation(gen, &ret);
          __Pyx_Coroutine_unset_is_running(gen);
          return __Pyx_Coroutine_MethodReturnFromResult(self, result, ret, 0);
      }
  throw_here:
      __Pyx_Raise(typ, val, tb, NULL);
  propagate_exception:
      {
          PyObject *retval = NULL;
          __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, NULL, &retval, 0);
          __Pyx_Coroutine_unset_is_running(gen);
          return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
      }
  }
  static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
      PyObject *typ;
      PyObject *val = NULL;
      PyObject *tb = NULL;
      if (unlikely(!PyArg_UnpackTuple(args, "throw", 1, 3, &typ, &val, &tb)))
          return NULL;
      return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
  }
  static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
  #if PY_VERSION_HEX >= 0x030B00a4
      Py_VISIT(exc_state->exc_value);
  #else
      Py_VISIT(exc_state->exc_type);
      Py_VISIT(exc_state->exc_value);
      Py_VISIT(exc_state->exc_traceback);
  #endif
      return 0;
  }
  static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
      {
          int e = __Pyx_call_type_traverse((PyObject*)gen, 1, visit, arg);
          if (e) return e;
      }
      Py_VISIT(gen->closure);
      Py_VISIT(gen->classobj);
      Py_VISIT(gen->yieldfrom);
      return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
  }
  static int __Pyx_Coroutine_clear(PyObject *self) {
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
      Py_CLEAR(gen->closure);
      Py_CLEAR(gen->classobj);
      __Pyx_Coroutine_Undelegate(gen);
      __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
  #ifdef __Pyx_AsyncGen_USED
      if (__Pyx_AsyncGen_CheckExact(self)) {
          Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
      }
  #endif
      Py_CLEAR(gen->gi_code);
      Py_CLEAR(gen->gi_frame);
      Py_CLEAR(gen->gi_name);
      Py_CLEAR(gen->gi_qualname);
      Py_CLEAR(gen->gi_modulename);
      return 0;
  }
  static void __Pyx_Coroutine_dealloc(PyObject *self) {
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
      PyObject_GC_UnTrack(gen);
      #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      if (gen->gi_weakreflist != NULL)
      #endif
          PyObject_ClearWeakRefs(self);
      if (gen->resume_label >= 0) {
          PyObject_GC_Track(self);
  #if CYTHON_USE_TP_FINALIZE
          if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
  #else
          {
              destructor del = __Pyx_PyObject_GetSlot(gen, tp_del, destructor);
              if (del) del(self);
          }
          if (unlikely(Py_REFCNT(self) > 0))
  #endif
          {
              return;
          }
          PyObject_GC_UnTrack(self);
      }
  #ifdef __Pyx_AsyncGen_USED
      if (__Pyx_AsyncGen_CheckExact(self)) {
          /* We have to handle this case for asynchronous generators
             right here, because this code has to be between UNTRACK
             and GC_Del. */
          Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
      }
  #endif
      __Pyx_Coroutine_clear(self);
      __Pyx_PyHeapTypeObject_GC_Del(gen);
  }
  #if CYTHON_USE_TP_FINALIZE
  static void __Pyx_Coroutine_del(PyObject *self) {
      PyObject *error_type, *error_value, *error_traceback;
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
      __Pyx_PyThreadState_declare
      if (gen->resume_label < 0) {
          return;
      }
      __Pyx_PyThreadState_assign
      __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
  #ifdef __Pyx_AsyncGen_USED
      if (__Pyx_AsyncGen_CheckExact(self)) {
          __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
          PyObject *finalizer = agen->ag_finalizer;
          if (finalizer && !agen->ag_closed) {
              PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
              if (unlikely(!res)) {
                  PyErr_WriteUnraisable(self);
              } else {
                  Py_DECREF(res);
              }
              __Pyx_ErrRestore(error_type, error_value, error_traceback);
              return;
          }
      }
  #endif
      if (unlikely(gen->resume_label == 0 && !error_value)) {
  #ifdef __Pyx_Coroutine_USED
  #ifdef __Pyx_Generator_USED
      if (!__Pyx_Generator_CheckExact(self))
  #endif
          {
          PyObject_GC_UnTrack(self);
          if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
              PyErr_WriteUnraisable(self);
          PyObject_GC_Track(self);
          }
  #endif
      } else {
          PyObject *retval = NULL;
          __Pyx_PySendResult result = __Pyx_Coroutine_Close(self, &retval);
          if (result == PYGEN_ERROR) {
              PyErr_WriteUnraisable(self);
          } else {
              Py_XDECREF(retval);
          }
      }
      __Pyx_ErrRestore(error_type, error_value, error_traceback);
  }
  #endif
  static PyObject *
  __Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
  {
      PyObject *name = self->gi_name;
      CYTHON_UNUSED_VAR(context);
      if (unlikely(!name)) name = Py_None;
      Py_INCREF(name);
      return name;
  }
  static int
  __Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      if (unlikely(value == NULL || !PyUnicode_Check(value))) {
          PyErr_SetString(PyExc_TypeError,
                          "__name__ must be set to a string object");
          return -1;
      }
      Py_INCREF(value);
      __Pyx_Py_XDECREF_SET(self->gi_name, value);
      return 0;
  }
  static PyObject *
  __Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
  {
      PyObject *name = self->gi_qualname;
      CYTHON_UNUSED_VAR(context);
      if (unlikely(!name)) name = Py_None;
      Py_INCREF(name);
      return name;
  }
  static int
  __Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      if (unlikely(value == NULL || !PyUnicode_Check(value))) {
          PyErr_SetString(PyExc_TypeError,
                          "__qualname__ must be set to a string object");
          return -1;
      }
      Py_INCREF(value);
      __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
      return 0;
  }
  #if !CYTHON_COMPILING_IN_LIMITED_API
  static PyObject *
  __Pyx__Coroutine_get_frame_locked(__pyx_CoroutineObject *self) {
      PyObject *frame;
      frame = self->gi_frame;
      if (!frame) {
          if (unlikely(!self->gi_code)) {
              Py_RETURN_NONE;
          }
          PyObject *globals = PyDict_New();
          if (unlikely(!globals)) return NULL;
          frame = (PyObject *) PyFrame_New(
              PyThreadState_Get(),            /*PyThreadState *tstate,*/
              (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
              globals,                        /*PyObject *globals,*/
              0                               /*PyObject *locals*/
          );
          Py_DECREF(globals);
          if (unlikely(!frame))
              return NULL;
          if (unlikely(self->gi_frame)) {
              Py_DECREF(frame);
              frame = self->gi_frame;
          } else {
              self->gi_frame = frame;
          }
      }
      Py_INCREF(frame);
      return frame;
  }
  #endif
  static PyObject *
  __Pyx__Coroutine_get_frame(__pyx_CoroutineObject *self)
  {
  #if !CYTHON_COMPILING_IN_LIMITED_API
      PyObject *frame;
      __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)self);
      frame = __Pyx__Coroutine_get_frame_locked(self);
      __Pyx_END_CRITICAL_SECTION();
      return frame;
  #else
      CYTHON_UNUSED_VAR(self);
      Py_RETURN_NONE;
  #endif
  }
  static PyObject *
  __Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context) {
      CYTHON_UNUSED_VAR(context);
      PyObject *frame = self->gi_frame;
      if (frame)
          return __Pyx_NewRef(frame);
      return __Pyx__Coroutine_get_frame(self);
  }
  static __pyx_CoroutineObject *__Pyx__Coroutine_New(
              PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
              PyObject *name, PyObject *qualname, PyObject *module_name) {
      __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
      if (unlikely(!gen))
          return NULL;
      return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
  }
  static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
              __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
              PyObject *name, PyObject *qualname, PyObject *module_name) {
      gen->body = body;
      gen->closure = closure;
      Py_XINCREF(closure);
      gen->is_running = 0;
      gen->resume_label = 0;
      gen->classobj = NULL;
      gen->yieldfrom = NULL;
      gen->yieldfrom_am_send = NULL;
      #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_LIMITED_API
      gen->gi_exc_state.exc_value = NULL;
      #else
      gen->gi_exc_state.exc_type = NULL;
      gen->gi_exc_state.exc_value = NULL;
      gen->gi_exc_state.exc_traceback = NULL;
      #endif
  #if CYTHON_USE_EXC_INFO_STACK
      gen->gi_exc_state.previous_item = NULL;
  #endif
  #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      gen->gi_weakreflist = NULL;
  #endif
      Py_XINCREF(qualname);
      gen->gi_qualname = qualname;
      Py_XINCREF(name);
      gen->gi_name = name;
      Py_XINCREF(module_name);
      gen->gi_modulename = module_name;
      Py_XINCREF(code);
      gen->gi_code = code;
      gen->gi_frame = NULL;
  #if CYTHON_USE_SYS_MONITORING && (CYTHON_PROFILE || CYTHON_TRACE)
      memset(gen->__pyx_pymonitoring_state, 0, sizeof(gen->__pyx_pymonitoring_state));
      gen->__pyx_pymonitoring_version = 0;
  #endif
      PyObject_GC_Track(gen);
      return gen;
  }
  static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen) {
      char result;
      __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)gen);
      result = gen->is_running;
      gen->is_running = 1;
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen) {
      __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)gen);
      assert(gen->is_running);
      gen->is_running = 0;
      __Pyx_END_CRITICAL_SECTION();
  }
  static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen) {
      char result;
      __Pyx_BEGIN_CRITICAL_SECTION((PyObject*)gen);
      result = gen->is_running;
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure) {
      CYTHON_UNUSED_VAR(closure);
      char result = __Pyx_Coroutine_get_is_running((__pyx_CoroutineObject*)gen);
      if (result) Py_RETURN_TRUE;
      else Py_RETURN_FALSE;
  }
  #if __PYX_HAS_PY_AM_SEND == 2
  static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send) {
      Py_ssize_t ptr_offset = (char*)(type->tp_as_async) - (char*)type;
      if (ptr_offset < 0 || ptr_offset > type->tp_basicsize) {
          return;
      }
      memcpy((void*)static_amsend_methods, (void*)(type->tp_as_async), sizeof(*type->tp_as_async));
      static_amsend_methods->am_send = am_send;
      type->tp_as_async = __Pyx_SlotTpAsAsync(static_amsend_methods);
  }
  #endif
  static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg) {
      CYTHON_UNUSED_VAR(arg);
      __Pyx_TypeName self_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE((PyObject*)self));
      PyErr_Format(PyExc_TypeError, "cannot pickle '" __Pyx_FMT_TYPENAME "' object",
                           self_type_name);
      __Pyx_DECREF_TypeName(self_type_name);
      return NULL;
  }
  
/* Generator */
  static PyMethodDef __pyx_Generator_methods[] = {
      {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
       PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
      {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
       PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
      {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
       PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
      {"__reduce_ex__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_O, 0},
      {"__reduce__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_NOARGS, 0},
      {0, 0, 0, 0}
  };
  static PyMemberDef __pyx_Generator_memberlist[] = {
      {"gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
       PyDoc_STR("object being iterated by 'yield from', or None")},
      {"gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
      {"__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
  #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
  #endif
      {0, 0, 0, 0, 0}
  };
  static PyGetSetDef __pyx_Generator_getsets[] = {
      {"__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
       PyDoc_STR("name of the generator"), 0},
      {"__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
       PyDoc_STR("qualified name of the generator"), 0},
      {"gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
       PyDoc_STR("Frame of the generator"), 0},
      {"gi_running", __Pyx_Coroutine_get_is_running_getter, NULL, NULL, NULL},
      {0, 0, 0, 0, 0}
  };
  static PyType_Slot __pyx_GeneratorType_slots[] = {
      {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
      {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
      {Py_tp_iter, (void *)PyObject_SelfIter},
      {Py_tp_iternext, (void *)__Pyx_Generator_Next},
      {Py_tp_methods, (void *)__pyx_Generator_methods},
      {Py_tp_members, (void *)__pyx_Generator_memberlist},
      {Py_tp_getset, (void *)__pyx_Generator_getsets},
      {Py_tp_getattro, (void *) PyObject_GenericGetAttr},
  #if CYTHON_USE_TP_FINALIZE
      {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
  #endif
  #if __PYX_HAS_PY_AM_SEND == 1
      {Py_am_send, (void *)__Pyx_Coroutine_AmSend},
  #endif
      {0, 0},
  };
  static PyType_Spec __pyx_GeneratorType_spec = {
      __PYX_TYPE_MODULE_PREFIX "generator",
      sizeof(__pyx_CoroutineObject),
      0,
  #if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
      Py_TPFLAGS_MANAGED_WEAKREF |
  #endif
      Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
      Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | __Pyx_TPFLAGS_HAVE_AM_SEND,
      __pyx_GeneratorType_slots
  };
  #if __PYX_HAS_PY_AM_SEND == 2
  static __Pyx_PyAsyncMethodsStruct __pyx_Generator_as_async;
  #endif
  static int __pyx_Generator_init(PyObject *module) {
      __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
      mstate->__pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(
          mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_GeneratorType_spec, NULL);
      if (unlikely(!mstate->__pyx_GeneratorType)) {
          return -1;
      }
  #if __PYX_HAS_PY_AM_SEND == 2
      __Pyx_SetBackportTypeAmSend(mstate->__pyx_GeneratorType, &__pyx_Generator_as_async, &__Pyx_Coroutine_AmSend);
  #endif
      return 0;
  }
  static PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self) {
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
      PyObject *retval = NULL;
      if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
          return __Pyx_Coroutine_AlreadyRunningError(gen);
      }
      __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
      __Pyx_Coroutine_unset_is_running(gen);
      (void) result;
      assert (result == PYGEN_RETURN || result == PYGEN_ERROR);
      assert ((result == PYGEN_RETURN && retval != NULL) || (result == PYGEN_ERROR && retval == NULL));
      return retval;
  }
  
/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
      const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
      if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
          return 0;
      if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
          return 1;
      {
          char message[200];
          PyOS_snprintf(message, sizeof(message),
                        "compile time Python version %d.%d "
                        "of module '%.100s' "
                        "%s "
                        "runtime version %d.%d",
                         (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                         __Pyx_MODULE_NAME,
                         (allow_newer) ? "was newer than" : "does not match",
                         (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
         );
          return PyErr_WarnEx(NULL, message, 1);
      }
  }
  
/* NewCodeObj */
  #if CYTHON_COMPILING_IN_LIMITED_API
      static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
          PyObject *exception_table = NULL;
          PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
          #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
          PyObject *version_info;
          PyObject *py_minor_version = NULL;
          #endif
          long minor_version = 0;
          PyObject *type, *value, *traceback;
          PyErr_Fetch(&type, &value, &traceback);
          #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
          minor_version = 11;
          #else
          if (!(version_info = PySys_GetObject("version_info"))) goto end;
          if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
          minor_version = PyLong_AsLong(py_minor_version);
          Py_DECREF(py_minor_version);
          if (minor_version == -1 && PyErr_Occurred()) goto end;
          #endif
          if (!(types_module = PyImport_ImportModule("types"))) goto end;
          if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
          if (minor_version <= 7) {
              (void)p;
              result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                            c, n, v, fn, name, fline, lnos, fv, cell);
          } else if (minor_version <= 10) {
              result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                            c, n, v, fn, name, fline, lnos, fv, cell);
          } else {
              if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
              result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                            c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
          }
      end:
          Py_XDECREF(code_type);
          Py_XDECREF(exception_table);
          Py_XDECREF(types_module);
          if (type) {
              PyErr_Restore(type, value, traceback);
          }
          return result;
      }
  #elif PY_VERSION_HEX >= 0x030B0000
    static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                           PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                           PyObject *fv, PyObject *cell, PyObject* fn,
                                           PyObject *name, int fline, PyObject *lnos) {
      PyCodeObject *result;
      result =
        #if PY_VERSION_HEX >= 0x030C0000
          PyUnstable_Code_NewWithPosOnlyArgs
        #else
          PyCode_NewWithPosOnlyArgs
        #endif
          (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
      #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030c00A1
      if (likely(result))
          result->_co_firsttraceable = 0;
      #endif
      return result;
    }
  #elif !CYTHON_COMPILING_IN_PYPY
    #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
            PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #else
    #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
            PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #endif
  static PyObject* __Pyx_PyCode_New(
          const __Pyx_PyCode_New_function_description descr,
          PyObject * const *varnames,
          PyObject *filename,
          PyObject *funcname,
          PyObject *line_table,
          PyObject *tuple_dedup_map
  ) {
      PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL;
      Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
      PyObject *varnames_tuple = PyTuple_New(var_count);
      if (unlikely(!varnames_tuple)) return NULL;
      for (Py_ssize_t i=0; i < var_count; i++) {
          Py_INCREF(varnames[i]);
          if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
      }
      #if CYTHON_COMPILING_IN_LIMITED_API
      varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
      if (!varnames_tuple_dedup) {
          if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
          varnames_tuple_dedup = varnames_tuple;
      }
      #else
      varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
      if (unlikely(!varnames_tuple_dedup)) goto done;
      #endif
      #if CYTHON_AVOID_BORROWED_REFS
      Py_INCREF(varnames_tuple_dedup);
      #endif
      if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL && !CYTHON_COMPILING_IN_GRAAL) {
          Py_ssize_t line_table_length = __Pyx_PyBytes_GET_SIZE(line_table);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely(line_table_length == -1)) goto done;
          #endif
          Py_ssize_t code_len = (line_table_length * 2 + 4) & ~3LL;
          code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
          if (unlikely(!code_bytes)) goto done;
          char* c_code_bytes = PyBytes_AsString(code_bytes);
          if (unlikely(!c_code_bytes)) goto done;
          memset(c_code_bytes, 0, (size_t) code_len);
      }
      code_obj = (PyObject*) __Pyx__PyCode_New(
          (int) descr.argcount,
          (int) descr.num_posonly_args,
          (int) descr.num_kwonly_args,
          (int) descr.nlocals,
          0,
          (int) descr.flags,
          code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
          __pyx_mstate_global->__pyx_empty_tuple,
          __pyx_mstate_global->__pyx_empty_tuple,
          varnames_tuple_dedup,
          __pyx_mstate_global->__pyx_empty_tuple,
          __pyx_mstate_global->__pyx_empty_tuple,
          filename,
          funcname,
          (int) descr.first_line,
          (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table) ? line_table : __pyx_mstate_global->__pyx_empty_bytes
      );
  done:
      Py_XDECREF(code_bytes);
      #if CYTHON_AVOID_BORROWED_REFS
      Py_XDECREF(varnames_tuple_dedup);
      #endif
      Py_DECREF(varnames_tuple);
      return code_obj;
  }
  
/* DecompressString */
  static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
      PyObject *module = NULL, *decompress, *compressed_bytes, *decompressed;
      const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
      PyObject *methodname = PyUnicode_FromString("decompress");
      if (unlikely(!methodname)) return NULL;
      #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
      if (algo == 3) {
          PyObject *fromlist = Py_BuildValue("[O]", methodname);
          if (unlikely(!fromlist)) goto bad;
          module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
          Py_DECREF(fromlist);
      } else
      #endif
          module = PyImport_ImportModule(module_name);
      if (unlikely(!module)) goto import_failed;
      decompress = PyObject_GetAttr(module, methodname);
      if (unlikely(!decompress)) goto import_failed;
      {
          #ifdef __cplusplus
              char *memview_bytes = const_cast<char*>(s);
          #else
              #if defined(__clang__)
                #pragma clang diagnostic push
                #pragma clang diagnostic ignored "-Wcast-qual"
              #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
                #pragma GCC diagnostic push
                #pragma GCC diagnostic ignored "-Wcast-qual"
              #endif
              char *memview_bytes = (char*) s;
              #if defined(__clang__)
                #pragma clang diagnostic pop
              #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
                #pragma GCC diagnostic pop
              #endif
          #endif
          #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
          int memview_flags = 0x100;
          #else
          int memview_flags = PyBUF_READ;
          #endif
          compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
      }
      if (unlikely(!compressed_bytes)) {
          Py_DECREF(decompress);
          goto bad;
      }
      decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
      Py_DECREF(compressed_bytes);
      Py_DECREF(decompress);
      Py_DECREF(module);
      Py_DECREF(methodname);
      return decompressed;
  import_failed:
      PyErr_Format(PyExc_ImportError,
          "Failed to import '%.20s.decompress' - cannot initialise module strings. "
          "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
          module_name, algo);
  bad:
      Py_XDECREF(module);
      Py_DECREF(methodname);
      return NULL;
  }
  
#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


  /* MultiPhaseInitModuleState */
  #if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
  #ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  #if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
    #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
  #else
    #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
  #endif
  #endif
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
  #error "Module state with PEP489 requires atomics. Currently that's one of\
   C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
  #endif
  #if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  #define __Pyx_ModuleStateLookup_Lock()
  #define __Pyx_ModuleStateLookup_Unlock()
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
  static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
  #define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
  #elif defined(__cplusplus) && __cplusplus >= 201103L
  #include <mutex>
  static std::mutex __Pyx_ModuleStateLookup_mutex;
  #define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
  #define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
  #elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
  #include <threads.h>
  static mtx_t __Pyx_ModuleStateLookup_mutex;
  static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
  static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
      mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
  }
  #define __Pyx_ModuleStateLookup_Lock()\
    call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
    mtx_lock(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
  #elif defined(HAVE_PTHREAD_H)
  #include <pthread.h>
  static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
  #define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
  #elif defined(_WIN32)
  #include <Windows.h>  // synchapi.h on its own doesn't work
  static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
  #define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
  #else
  #error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
   Requires C standard >= C11, or C++ standard >= C++11,\
   or pthreads, or the Windows 32 API, or Python >= 3.13."
  #endif
  typedef struct {
      int64_t id;
      PyObject *module;
  } __Pyx_InterpreterIdAndModule;
  typedef struct {
      char interpreter_id_as_index;
      Py_ssize_t count;
      Py_ssize_t allocated;
      __Pyx_InterpreterIdAndModule table[1];
  } __Pyx_ModuleStateLookupData;
  #define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
  #endif
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
  #else
  static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
  #endif
  static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
          __Pyx_InterpreterIdAndModule* table,
          Py_ssize_t count,
          int64_t interpreterId) {
      __Pyx_InterpreterIdAndModule* begin = table;
      __Pyx_InterpreterIdAndModule* end = begin + count;
      if (begin->id == interpreterId) {
          return begin;
      }
      while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
          __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
          if (halfway->id == interpreterId) {
              return halfway;
          }
          if (halfway->id < interpreterId) {
              begin = halfway;
          } else {
              end = halfway;
          }
      }
      for (; begin < end; ++begin) {
          if (begin->id >= interpreterId) return begin;
      }
      return begin;
  }
  static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
      int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
      if (interpreter_id == -1) return NULL;
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
      {
          __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
          if (likely(data)) {
              __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
              if (likely(data == new_data)) {
                  goto read_finished;
              }
          }
          __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
          __Pyx_ModuleStateLookup_Lock();
          __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
          data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
          __Pyx_ModuleStateLookup_Unlock();
      }
    read_finished:;
  #else
      __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
  #endif
      __Pyx_InterpreterIdAndModule* found = NULL;
      if (unlikely(!data)) goto end;
      if (data->interpreter_id_as_index) {
          if (interpreter_id < data->count) {
              found = data->table+interpreter_id;
          }
      } else {
          found = __Pyx_State_FindModuleStateLookupTableLowerBound(
              data->table, data->count, interpreter_id);
      }
    end:
      {
          PyObject *result=NULL;
          if (found && found->id == interpreter_id) {
              result = found->module;
          }
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
          __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
  #endif
          return result;
      }
  }
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
      while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
  }
  #else
  #define __Pyx_ModuleStateLookup_wait_until_no_readers()
  #endif
  static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
      Py_ssize_t to_allocate = (*old_data)->allocated;
      while (to_allocate <= interpreter_id) {
          if (to_allocate == 0) to_allocate = 1;
          else to_allocate *= 2;
      }
      __Pyx_ModuleStateLookupData *new_data = *old_data;
      if (to_allocate != (*old_data)->allocated) {
           new_data = (__Pyx_ModuleStateLookupData *)realloc(
              *old_data,
              sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
          if (!new_data) {
              PyErr_NoMemory();
              return -1;
          }
          for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
              new_data->table[i].id = i;
              new_data->table[i].module = NULL;
          }
          new_data->allocated = to_allocate;
      }
      new_data->table[interpreter_id].module = module;
      if (new_data->count < interpreter_id+1) {
          new_data->count = interpreter_id+1;
      }
      *old_data = new_data;
      return 0;
  }
  static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
      __Pyx_InterpreterIdAndModule *read = data->table;
      __Pyx_InterpreterIdAndModule *write = data->table;
      __Pyx_InterpreterIdAndModule *end = read + data->count;
      for (; read<end; ++read) {
          if (read->module) {
              write->id = read->id;
              write->module = read->module;
              ++write;
          }
      }
      data->count = write - data->table;
      for (; write<end; ++write) {
          write->id = 0;
          write->module = NULL;
      }
      data->interpreter_id_as_index = 0;
  }
  static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
      int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
      if (interpreter_id == -1) return -1;
      int result = 0;
      __Pyx_ModuleStateLookup_Lock();
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
              __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
  #else
      __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
  #endif
      __Pyx_ModuleStateLookupData *new_data = old_data;
      if (!new_data) {
          new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
          if (!new_data) {
              result = -1;
              PyErr_NoMemory();
              goto end;
          }
          new_data->allocated = 1;
          new_data->interpreter_id_as_index = 1;
      }
      __Pyx_ModuleStateLookup_wait_until_no_readers();
      if (new_data->interpreter_id_as_index) {
          if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
              result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
              goto end;
          }
          __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
      }
      {
          Py_ssize_t insert_at = 0;
          {
              __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                  new_data->table, new_data->count, interpreter_id);
              assert(lower_bound);
              insert_at = lower_bound - new_data->table;
              if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                  lower_bound->module = module;
                  goto end;  // already in table, nothing more to do
              }
          }
          if (new_data->count+1 >= new_data->allocated) {
              Py_ssize_t to_allocate = (new_data->count+1)*2;
              new_data =
                  (__Pyx_ModuleStateLookupData*)realloc(
                      new_data,
                      sizeof(__Pyx_ModuleStateLookupData) +
                      (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
              if (!new_data) {
                  result = -1;
                  new_data = old_data;
                  PyErr_NoMemory();
                  goto end;
              }
              new_data->allocated = to_allocate;
          }
          ++new_data->count;
          int64_t last_id = interpreter_id;
          PyObject *last_module = module;
          for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
              int64_t current_id = new_data->table[i].id;
              new_data->table[i].id = last_id;
              last_id = current_id;
              PyObject *current_module = new_data->table[i].module;
              new_data->table[i].module = last_module;
              last_module = current_module;
          }
      }
    end:
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
  #else
      __Pyx_ModuleStateLookup_data = new_data;
  #endif
      __Pyx_ModuleStateLookup_Unlock();
      return result;
  }
  static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
      int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
      if (interpreter_id == -1) return -1;
      __Pyx_ModuleStateLookup_Lock();
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
              __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
  #else
      __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
  #endif
      if (data->interpreter_id_as_index) {
          if (interpreter_id < data->count) {
              data->table[interpreter_id].module = NULL;
          }
          goto done;
      }
      {
          __Pyx_ModuleStateLookup_wait_until_no_readers();
          __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
              data->table, data->count, interpreter_id);
          if (!lower_bound) goto done;
          if (lower_bound->id != interpreter_id) goto done;
          __Pyx_InterpreterIdAndModule *end = data->table+data->count;
          for (;lower_bound<end-1; ++lower_bound) {
              lower_bound->id = (lower_bound+1)->id;
              lower_bound->module = (lower_bound+1)->module;
          }
      }
      --data->count;
      if (data->count == 0) {
          free(data);
          data = NULL;
      }
    done:
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
  #else
      __Pyx_ModuleStateLookup_data = data;
  #endif
      __Pyx_ModuleStateLookup_Unlock();
      return 0;
  }
  #endif
  
/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
